#pragma once

#include <array>
#include <span>
#include <string_view>
#include <unordered_map>
#include <vector>

#include "faker-cxx/person.h"
#include "faker-cxx/types/locale.h"

namespace faker::person
{
struct NameFormat
{
    std::string_view format;
    unsigned weight;
};

using NameFormats = std::vector<NameFormat>;

struct PersonNames
{
    std::span<const std::string_view> firstNames;
    std::span<const std::string_view> lastNames;
    std::span<const std::string_view> prefixes;
    std::span<const std::string_view> suffixes;
};

struct PeopleNames
{
    PersonNames malesNames;
    PersonNames femalesNames;
    NameFormats nameFormats;
};

const auto bioParts = std::to_array<std::string_view>({
    "Activist",   "Artist",    "Author",      "Blogger",      "Business owner", "Coach",        "Creator",
    "Designer",   "Developer", "Dreamer",     "Educator",     "Engineer",       "Entrepreneur", "Environmentalist",
    "Film lover", "Filmmaker", "Foodie",      "Founder",      "Friend",         "Gamer",        "Geek",
    "Grad",       "Inventor",  "Leader",      "Model",        "Musician",       "Nerd",         "Parent",
    "Patriot",    "Person",    "Philosopher", "Photographer", "Public speaker", "Scientist",    "Singer",
    "Streamer",   "Student",   "Teacher",     "Traveler",     "Veteran",        "Writer",
});

const auto bioSupporters =
    std::to_array<std::string_view>({"Advocate", "Devotee", "Enthusiast", "Fan", "Junkie", "Lover", "Supporter"});

const auto bioFormats = std::to_array<std::string_view>(
    {"{bio_part}", "{bio_part}, {bio_part}", "{bio_part}, {bio_part}, {bio_part}",
     "{bio_part}, {bio_part}, {bio_part}, {emoji}", "{noun} {bio_supporter}", "{noun} {bio_supporter} {emoji}",
     "{noun} {bio_supporter}, {bio_part}", "{noun} {bio_supporter}, {bio_part} {emoji}"});

const auto genders = std::to_array<std::string_view>({
    "Male",
    "Female",
    "Transexual",
});

const auto hobbies = std::to_array<std::string_view>({
    "Playing guitar",
    "Reading books",
    "Hiking",
    "Cooking",
    "Painting",
    "Photography",
    "Swimming",
    "Gaming",
    "Dancing",
    "Traveling",
    "Bird watching",
    "Coding",
    "Singing",
    "Collecting stamps",
    "Fishing",
    "Yoga",
    "Sculpting",
    "Mountain biking",
    "Chess",
    "Archery",
    "Rock climbing",
    "Skiing",
    "Writing poetry",
    "Meditation",
    "Solving puzzles",
    "Gardening",
    "Playing basketball",
    "Volunteering",
    "Playing board games",
    "Astrophotography",
    "Camping",
    "Model building",
    "Astronomy",
    "Watching movies",
    "Stargazing",
    "Playing tennis",
    "Cooking ethnic cuisines",
    "Woodworking",
    "Baking",
    "Calligraphy",
    "Pottery",
    "Karaoke",
    "Scuba diving",
    "Metal detecting",
});

const auto jobDescriptors = std::to_array<std::string_view>({
    "Lead",          "Senior",   "Direct",   "Corporate", "Dynamic", "Future",   "Product",
    "National",      "Regional", "District", "Central",   "Global",  "Customer", "Investor",
    "International", "Legacy",   "Forward",  "Internal",  "Human",   "Chief",    "Principal",
});

const auto jobAreas = std::to_array<std::string_view>(
    {"Solutions",    "Program",        "Brand",          "Security",      "Research",       "Marketing",
     "Directives",   "Implementation", "Integration",    "Functionality", "Response",       "Paradigm",
     "Tactics",      "Identity",       "Markets",        "Group",         "Division",       "Applications",
     "Optimization", "Operations",     "Infrastructure", "Intranet",      "Communications", "Web",
     "Branding",     "Quality",        "Assurance",      "Mobility",      "Accounts",       "Data",
     "Creative",     "Configuration",  "Accountability", "Interactions",  "Factors",        "Usability",
     "Metrics",      "Software"});

const auto jobTypes = std::to_array<std::string_view>({
    "Supervisor",  "Associate",    "Executive",      "Liaison",       "Officer",   "Manager",    "Engineer",
    "Specialist",  "Director",     "Coordinator",    "Administrator", "Architect", "Analyst",    "Designer",
    "Planner",     "Orchestrator", "Technician",     "Developer",     "Producer",  "Consultant", "Assistant",
    "Facilitator", "Agent",        "Representative", "Strategist",
});

const auto languages = std::to_array<std::string_view>(
    {"English", "Polish", "Italian", "French", "German", "Russian", "Romanian", "Hindi", "Finnish", "Nepali", "Spanish",
     "Turkish", "Czech", "Slovak", "Ukrainian", "Danish", "Swedish", "Persian(Farsi)"});

const auto nationalities = std::to_array<std::string_view>({
    "American",     "British",       "Canadian",      "French",     "German",      "Japanese",      "Australian",
    "Chinese",      "Indian",        "Italian",       "Mexican",    "Russian",     "Brazilian",     "Swedish",
    "South Korean", "Spanish",       "Dutch",         "Norwegian",  "Turkish",     "Greek",         "Egyptian",
    "Israeli",      "Saudi Arabian", "South African", "Swiss",      "Argentinian", "New Zealander", "Thai",
    "Vietnamese",   "Irish",         "Polish",        "Portuguese", "Danish",      "Scottish",      "Welsh",
    "Belgian",      "Austrian",      "Finnish",       "Hungarian",  "Czech",       "Romanian",      "Ukrainian",
    "Swedish",      "Norwegian",     "Singaporean",   "Malaysian",  "Filipino",    "Indonesian",    "Kuwaiti",
    "Qatari",       "Emirati",       "Jordanian",     "Chilean",    "Peruvian",    "Colombian",     "Venezuelan",
    "Ecuadorian",   "Bolivian",      "Costa Rican",   "Panamanian", "Honduran",    "Guatemalan",
});

const std::unordered_map<Locale, std::string_view> passportFormats{
    {Locale::es_US, "AA0000000"}, {Locale::en_US, "AA0000000"}, {Locale::pl_PL, "AA0000000"},
    {Locale::fr_FR, "00AA00000"}, {Locale::ro_RO, "00000000"},
};

const std::unordered_map<Locale, std::string_view> ssnFormats{
    {Locale::pl_PL, "##[0-1][0-2][0-2]######"},
    {Locale::es_US, "###-##-####"},
    {Locale::en_US, "###-##-####"},
    {Locale::en_GB, "LL ## ## ## L"},
    {Locale::de_DE, "####[0-2]#[0-1][0-2][1-2][5-9]##"},
    {Locale::fr_FR, "## [0-1][0-2] [0-2]# ### ### ##"},
    {Locale::it_IT, "FFFF FFFF FFFF FFFF"},
    {Locale::es_ES, "X########L"},
    {Locale::ca_ES, "X########L"},
    {Locale::as_IN, "LLLLL####L"},
    {Locale::bn_IN, "LLLLL####L"},
    {Locale::en_IN, "LLLLL####L"},
    {Locale::gu_IN, "LLLLL####L"},
    {Locale::hi_IN, "LLLLL####L"},
    {Locale::kn_IN, "LLLLL####L"},
    {Locale::ks_IN, "LLLLL####L"},
    {Locale::ml_IN, "LLLLL####L"},
    {Locale::mr_IN, "LLLLL####L"},
    {Locale::or_IN, "LLLLL####L"},
    {Locale::pa_IN, "LLLLL####L"},
    {Locale::sa_IN, "LLLLL####L"},
    {Locale::ta_IN, "LLLLL####L"},
    {Locale::te_IN, "LLLLL####L"},
};

const auto westernZodiacs = std::to_array<std::string_view>({
    "Aries",
    "Taurus",
    "Gemini",
    "Cancer",
    "Leo",
    "Virgo",
    "Libra",
    "Scorpio",
    "Sagittarius",
    "Capricorn",
    "Aquarius",
    "Pisces",
});

const auto chineseZodiacs = std::to_array<std::string_view>({
    "Rat",
    "Ox",
    "Tiger",
    "Rabbit",
    "Dragon",
    "Snake"
    "Horse",
    "Sheep",
    "Monkey"
    "Rooster",
    "Dog",
    "Pig",
});

const std::unordered_map<Locale, std::unordered_map<Sex, std::string_view>> sexTranslations = {
    {Locale::en_US, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_AU, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_BW, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_CA, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_GB, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_GH, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_HK, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_IE, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_IN, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_MT, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_NZ, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_PH, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_SG, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::en_ZW, {{Sex::Male, "Male"}, {Sex::Female, "Female"}}},
    {Locale::pl_PL, {{Sex::Male, "Mężczyzna"}, {Sex::Female, "Kobieta"}}},
    {Locale::it_IT, {{Sex::Male, "Maschio"}, {Sex::Female, "Femmina"}}},
    {Locale::fr_FR, {{Sex::Male, "Homme"}, {Sex::Female, "Femme"}}},
    {Locale::de_DE, {{Sex::Male, "Mann"}, {Sex::Female, "Frau"}}},
    {Locale::ru_RU, {{Sex::Male, "Мужчина"}, {Sex::Female, "Женщина"}}},
    {Locale::ro_RO, {{Sex::Male, "Bărbat"}, {Sex::Female, "Femeie"}}},
    {Locale::hi_IN, {{Sex::Male, "पुरुष"}, {Sex::Female, "महिला"}}},
    {Locale::fi_FI, {{Sex::Male, "Mies"}, {Sex::Female, "Nainen"}}},
    {Locale::ne_NP, {{Sex::Male, "पुरुष"}, {Sex::Female, "महिला"}}},
    {Locale::es_ES, {{Sex::Male, "Hombre"}, {Sex::Female, "Mujer"}}},
    {Locale::tr_TR, {{Sex::Male, "Erkek"}, {Sex::Female, "Kadın"}}},
    {Locale::cs_CZ, {{Sex::Male, "Muž"}, {Sex::Female, "Žena"}}},
    {Locale::sk_SK, {{Sex::Male, "Muž"}, {Sex::Female, "Žena"}}},
    {Locale::uk_UA, {{Sex::Male, "Чоловік"}, {Sex::Female, "Жінка"}}},
    {Locale::da_DK, {{Sex::Male, "Mand"}, {Sex::Female, "Kvinde"}}},
    {Locale::sv_SE, {{Sex::Male, "Man"}, {Sex::Female, "Kvinna"}}},
    {Locale::pt_PT, {{Sex::Male, "Homem"}, {Sex::Female, "Mulher"}}},
    {Locale::nn_NO, {{Sex::Male, "Mann"}, {Sex::Female, "Kvinne"}}},
    {Locale::ja_JP, {{Sex::Male, "男性"}, {Sex::Female, "女性"}}},
    {Locale::hu_HU, {{Sex::Male, "Férfi"}, {Sex::Female, "Nő"}}},
    {Locale::hr_HR, {{Sex::Male, "Muškarac"}, {Sex::Female, "Žena"}}},
    {Locale::el_GR, {{Sex::Male, "Άνδρας"}, {Sex::Female, "Γυναίκα"}}},
    {Locale::sl_SI, {{Sex::Male, "Moški"}, {Sex::Female, "Ženska"}}},
    {Locale::nl_NL, {{Sex::Male, "Man"}, {Sex::Female, "Vrouw"}}},
    {Locale::zh_CN, {{Sex::Male, "男"}, {Sex::Female, "女"}}},
    {Locale::ko_KR, {{Sex::Male, "남자"}, {Sex::Female, "여자"}}},
    {Locale::sr_RS, {{Sex::Male, "Мушкарац"}, {Sex::Female, "Жена"}}},
    {Locale::mk_MK, {{Sex::Male, "Маж"}, {Sex::Female, "Жена"}}},
    {Locale::sq_AL, {{Sex::Male, "Mashkull"}, {Sex::Female, "Femër"}}},
    {Locale::lv_LV, {{Sex::Male, "Vīrietis"}, {Sex::Female, "Sieviete"}}},
    {Locale::be_BY, {{Sex::Male, "Мужчына"}, {Sex::Female, "Жанчына"}}},
    {Locale::et_EE, {{Sex::Male, "Mees"}, {Sex::Female, "Naine"}}},
    {Locale::ga_IE, {{Sex::Male, "fireannach"}, {Sex::Female, "baineann"}}}};

// Albania

const auto albanianMaleFirstNames = std::to_array<std::string_view>(
    {"Adriatik", "Afrim",    "Agim",    "Agron",    "Alban",    "Altin",   "Andi",    "Artion",   "Asim",    "Arbnor",
     "Arber",    "Ardian",   "Ardit",   "Armend",   "Arlind",   "Astrit",  "Avni",    "Bamir",    "Bardh",   "Bardhyl",
     "Bashkim",  "Besart",   "Besim",   "Besmir",   "Besnik",   "Niko",    "Bekim",   "Betim",    "Bledar",  "Bledian",
     "Blendi",   "Blerim",   "Bujar",   "Bukurosh", "Burim",    "Clirim",  "Halit",   "Hysen",    "Ilir",    "Ismail",
     "Jonuz",    "Julian",   "Kadri",   "Dashamir", "Dardan",   "Dash",    "Dëfrim",  "Dëshir",   "Diell",   "Drilon",
     "Dritan",   "Durim",    "Endrit",  "Endri",    "Enkel",    "Erin",    "Eris",    "Erion",    "Erjon",   "Ergys",
     "Erkand",   "Ermir",    "Elton",   "Ervin",    "Ermal",    "Erzen",   "Fatlum",  "Fatos",    "Festim",  "Fidan",
     "Fisnik",   "Flamur",   "FLorent", "Florenc",  "Lorenc",   "Lorik",   "Mërgim",  "Gash",     "Gaz",     "Gazmend",
     "Gent",     "Gentian",  "Gëzim",   "Gramoz",   "Gjergj",   "Gjergji", "Gjin",    "Gjon",     "Hekuran", "Jetmir",
     "Jon",      "Kastriot", "Kristaq", "Klajd",    "KLajdi",   "Klevis",  "Klodian", "Korab",    "Kujtim",  "Lavdim",
     "Ledian",   "Lek",      "Liridon", "Lirim",    "Lisian",   "Luan",    "Lul",     "Lulëzim",  "Marash",  "Medon",
     "Mentor",   "Migjen",   "Mirlind", "Mikel",    "Ndriçim",  "Nart",    "Oltian",  "Oltion",   "Orges",   "Orgest",
     "Pandi",    "Qemal",    "Pajtim",  "Pëllumb",  "Përparim", "Pirro",   "Petrit",  "Redon",    "Rexhep",  "Serxhio",
     "Rinor",    "Saimir",   "Samir",   "Skënder",  "Shqiprim", "Sokol",   "Spartak", "Shkëlqim", "Shkëmb",  "Shkumbin",
     "Shpend",   "Shkumbin", "Taulant", "Tomor",    "Trim",     "Urim",    "Valdet",  "Valmir",   "Vedat",   "Xhevdet",
     "Xhevahir", "Zef",      "Vath",    "Vidan",    "Vullnet",  "Ylber",   "Zamir"});

const auto albanianFemaleFirstNames = std::to_array<std::string_view>({
    "Adelina",  "Aferdita",  "Agnesa",  "Albana",    "Aldona",    "Aleksandra", "Alketa",  "Alma",      "Altina",
    "Amanda",   "Anastasia", "Anda",    "Andela",    "Ardiana",   "Ardita",     "Arlinda", "Arta",      "Artemisa",
    "Artiola",  "Arxhela",   "Asja",    "Astritana", "Aulona",    "Aurela",     "Aurora",  "Balina",    "Besa",
    "Besiana",  "Besjana",   "Blenda",  "Blerina",   "Bora",      "Bukurije",   "Dafina",  "Dajana",    "Dalina",
    "Dardana",  "Davina",    "Dea",     "Dhurata",   "Diana",     "Diona",      "Dita",    "Dorentina", "Drita",
    "Duljeta",  "Edlira",    "Ela",     "Elda",      "Elena",     "Elira",      "Elona",   "Elsa",      "Elvana",
    "Elvira",   "Ema",       "Embla",   "Eneida",    "Era",       "Erëblina",   "Erilda",  "Eriona",    "Erjona",
    "Ersa",     "Esmeralda", "Estela",  "Eva",       "Evelina",   "Flaka",      "Fjolla",  "Florina",   "Frida",
    "Gentiana", "Gresa",     "Hana",    "Ilda",      "Ira",       "Irena",      "Jeta",    "Joana",     "Jonida",
    "Klea",     "Klina",     "Krenare", "Kriselda",  "Larisa",    "Laura",      "Lavdi",   "Lea",       "Ledia",
    "Lena",     "Leonora",   "Lila",    "Linda",     "Liridona",  "Lirona",     "Lisjana", "Liza",      "Lorenta",
    "Luljeta",  "Majlinda",  "Manjola", "Margarita", "Mariola",   "Merita",     "Migena",  "Mimoza",    "Miranda",
    "Monika",   "Nada",      "Naima",   "Nertila",   "Njomza",    "Oltita",     "Olsa",    "Oltita",    "Oriana",
    "Rina",     "Rita",      "Romina",  "Rovena",    "Sara",      "Shpresa",    "Sidita",  "Silva",     "Simona",
    "Sonila",   "Tea",       "Teuta",   "Valbona",   "Valentina", "Vanessa",    "Vera",    "Verona",    "Vesa",
    "Veselina", "Viona",     "Vjollca", "Xhensila",  "Zamira",    "Zana",
});

const auto albanianLastNames = std::to_array<std::string_view>(
    {"Agolli",    "Ahmeti",      "Aliaj",    "Aliu",     "Arifi",    "Avdullahu", "Bajrami",   "Balaj",    "Basha",
     "Begolli",   "Berisha",     "Bilalli",  "Brahimi",  "Bytyqi",   "Cakuli",    "Cela",      "Cenaj",    "Dedvukaj",
     "Dervishi",  "Domgjonaj",   "Dreshaj",  "Driza",    "Duka",     "Duraku",    "Dushku",    "Elezi",    "Fazliu",
     "Ferizi",    "Gashi",       "Gega",     "Gjinaj",   "Gjokaj",   "Gjoka",     "Gjonaj",    "Gjoni",    "Gjuraj",
     "Gruda",     "Hadergjonaj", "Hajdari",  "Hajzeri",  "Haliti",   "Hamiti",    "Hasani",    "Hyseni",   "Ibrahimi",
     "Ismaili",   "Jashari",     "Jusufi",   "Kadriu",   "Kajtazi",  "Kapllani",  "Karaj",     "Kastrati", "Kavaja",
     "Kazazi",    "Kelmendi",    "Kicaj",    "Kola",     "Krasniqi", "Kukaj",     "Kulla",     "Kurti",    "Kusari",
     "Leka",      "Leka",        "Lushi",    "Majko",    "Marku",    "Maxhuni",   "Mazreku",   "Meta",     "Miftari",
     "Muharremi", "Murati",      "Murtishi", "Mustafa",  "Ndoj",     "Nimani",    "Nikçi",     "Nuhiu",    "Osmani",
     "Pacuku",    "Pajaziti",    "Paloka",   "Peci",     "Përmeti",  "Prenkaj",   "Qerimi",    "Rama",     "Rrahmani",
     "Rrudhani",  "Rugova",      "Rustemi",  "Sahiti",   "Salihu",   "Shala",     "Shala",     "Shkreli",  "Shoshi",
     "Shuaipi",   "Sopi",        "Spahiu",   "Statovci", "Sula",     "Sylaj",     "Thaçi",     "Thaqi",    "Topalli",
     "Topi",      "Tushi",       "Ujkani",   "Ulqini",   "Useni",    "Vehapi",    "Vejseli",   "Veliaj",   "Vocaj",
     "Vokshi",    "Vula",        "Vukaj",    "Xhaferi",  "Xhemajli", "Xhemalaj",  "Xhemshiti", "Xhokli",   "Xhufi",
     "Ylli",      "Zeka",        "Zeqiri",   "Zhubi",    "Zogaj",    "Zogu",      "Zeka",      "Zefi",     "Zeka",
     "Zekaj",     "Zhubi",       "Zhuta",    "Ziu",      "Zogiani"});

const NameFormats albanianPersonNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames albanianPeopleNames{{albanianMaleFirstNames, albanianLastNames, {}, {}},
                                      {albanianFemaleFirstNames, albanianLastNames, {}, {}},
                                      albanianPersonNameFormats};

// Argentina

const auto argentinianMaleFirstNames = std::to_array<std::string_view>({
    "Jose",      "Carlos",      "Jorge",      "Luis",         "Miguel",    "Hector",    "Ramon",      "Roberto",
    "Oscar",     "Daniel",      "Mario",      "Pedro",        "Ricardo",   "Raul",      "Pablo",      "Eduardo",
    "Sergio",    "Marcelo",     "Julio",      "Hugo",         "Alberto",   "Ruben",     "Gustavo",    "Antonio",
    "Victor",    "Francisco",   "Diego",      "Angel",        "Alejandro", "Claudio",   "Walter",     "Fernando",
    "Nestor",    "Guillermo",   "Manuel",     "Cristian",     "Martin",    "Alfredo",   "Enrique",    "Osvaldo",
    "Javier",    "Gabriel",     "Horacio",    "Cesar",        "Rodolfo",   "Omar",      "Andres",     "Ernesto",
    "Leonardo",  "Domingo",     "Marcos",     "Nicolas",      "Sebastian", "Adrian",    "Norberto",   "Mariano",
    "Esteban",   "Dario",       "Fabian",     "Edgardo",      "Rafael",    "Vicente",   "Orlando",    "Gerardo",
    "Hernan",    "Federico",    "Felix",      "Aldo",         "Armando",   "Adolfo",    "David",      "Emilio",
    "German",    "Anibal",      "Lucas",      "Mauricio",     "Humberto",  "Ignacio",   "Agustin",    "Felipe",
    "Leandro",   "Tomas",       "Abel",       "Gregorio",     "Luciano",   "Segundo",   "Rolando",    "Gaston",
    "Nelson",    "Maximiliano", "Salvador",   "Lorenzo",      "Christian", "Silvio",    "Arturo",     "Rodrigo",
    "Mauro",     "Dante",       "Bernardo",   "Gonzalo",      "Rogelio",   "Julian",    "Eugenio",    "Ismael",
    "Marcelino", "Santiago",    "Ezequiel",   "Justo",        "Benito",    "Maximo",    "Fabio",      "Atilio",
    "Patricio",  "Americo",     "Arnaldo",    "Lucio",        "Florencio", "Facundo",   "Emiliano",   "Ceferino",
    "Joaquin",   "Jacinto",     "Hipolito",   "Elias",        "Reinaldo",  "Geronimo",  "Eusebio",    "Elvio",
    "Ivan",      "Sixto",       "Valentin",   "Benjamin",     "Alfonso",   "Dardo",     "Fermin",     "Bruno",
    "Augusto",   "Matias",      "Isidro",     "Leopoldo",     "Elio",      "Teodoro",   "Simon",      "Edgar",
    "Faustino",  "Aurelio",     "Jaime",      "Carmelo",      "Edmundo",   "Ramiro",    "Samuel",     "Paulo",
    "Dionisio",  "Hilario",     "Sandro",     "Enzo",         "Cirilo",    "Marco",     "Demetrio",   "Guido",
    "Osmar",     "Fidel",       "Florentino", "Bernardino",   "Evaristo",  "Roque",     "Flavio",     "Anselmo",
    "Victorio",  "Isidoro",     "Argentino",  "Damian",       "Rufino",    "Paulino",   "Genaro",     "Celestino",
    "Saul",      "Victoriano",  "Amado",      "Celso",        "Teofilo",   "Eliseo",    "Eleuterio",  "Eulogio",
    "Ubaldo",    "Adalberto",   "Lisandro",   "Saturnino",    "Cristobal", "Isaac",     "Feliciano",  "Pascual",
    "Gilberto",  "Emanuel",     "Adan",       "Reynaldo",     "Modesto",   "Inocencio", "Cecilio",    "Raimundo",
    "Fortunato", "Catalino",    "Albino",     "Moises",       "Camilo",    "Dionicio",  "Marcial",    "Fabricio",
    "Virgilio",  "Lino",        "Candido",    "Avelino",      "Romulo",    "Amadeo",    "Rosendo",    "Rito",
    "Benigno",   "Elbio",       "Tito",       "Abelardo",     "Silverio",  "Remigio",   "Octavio",    "Estanislao",
    "Heriberto", "Abraham",     "Arnoldo",    "Bonifacio",    "Eladio",    "Silvano",   "Cipriano",   "Donato",
    "Leonel",    "Gabino",      "Clemente",   "Ireneo",       "Natalio",   "Marciano",  "Nicasio",    "Ovidio",
    "Alvaro",    "Epifanio",    "Anastacio",  "Fausto",       "Nicanor",   "Italo",     "Silvestre",  "Delfin",
    "Cornelio",  "Timoteo",     "Eustaquio",  "Desiderio",    "Zenon",     "Leoncio",   "Amilcar",    "Eloy",
    "Ulises",    "Ciriaco",     "Isabelino",  "Favio",        "Wenceslao", "Cosme",     "Basilio",    "Justino",
    "Conrado",   "Romualdo",    "Nolberto",   "Placido",      "Exequiel",  "Serafin",   "Victorino",  "Higinio",
    "Severo",    "Mateo",       "Calixto",    "Herminio",     "Ruperto",   "Salomon",   "Honorio",    "Arsenio",
    "Milton",    "Roman",       "Danilo",     "Cayetano",     "Franco",    "Venancio",  "Froilan",    "Aniceto",
    "Heraldo",   "Oreste",      "Sabino",     "Alejo",        "Bernabe",   "Edelmiro",  "Braulio",    "Martiniano",
    "Olegario",  "Secundino",   "Jonatan",    "Nemesio",      "Jeronimo",  "Leon",      "Anacleto",   "Simeon",
    "Elpidio",   "Narciso",     "Waldo",      "Hermenegildo", "Marino",    "Bartolome", "Wilfredo",   "Adriano",
    "Obdulio",   "Nelso",       "Efrain",     "Nahuel",       "Benedicto", "Lionel",    "Eleodoro",   "Laureano",
    "Renato",    "Abdon",       "Gumercindo", "Emmanuel",     "Casimiro",  "Jacobo",    "Israel",     "Roger",
    "Toribio",   "Telmo",       "Lazaro",     "Joel",         "Ambrosio",  "Regino",    "Waldemar",   "Ciro",
    "Hernando",  "Pio",         "Zoilo",      "Isaias",       "Eulalio",   "Brigido",   "Clementino", "Erasmo",
    "Nazario",   "Primo",       "Eric",       "Salustiano",   "Tiburcio",  "Ladislao",
});

const auto argentinianFemaleFirstNames = std::to_array<std::string_view>({
    "Maria",       "Juan",      "Ana",       "Silvia",      "Rosa",       "Norma",     "Marta",       "Claudia",
    "Graciela",    "Monica",    "Susana",    "Mirta",       "Alicia",     "Patricia",  "Nelida",      "Juana",
    "Liliana",     "Sandra",    "Laura",     "Olga",        "Andrea",     "Elsa",      "Ramona",      "Adriana",
    "Marcela",     "Carmen",    "Lidia",     "Teresa",      "Gabriela",   "Gladys",    "Veronica",    "Blanca",
    "Natalia",     "Margarita", "Irma",      "Beatriz",     "Mercedes",   "Alejandra", "Dora",        "Elena",
    "Angela",      "Cristina",  "Lorena",    "Delia",       "Isabel",     "Stella",    "Ariel",       "Mariana",
    "Mariela",     "Nilda",     "Estela",    "Nancy",       "Sara",       "Nora",      "Viviana",     "Lucia",
    "Analia",      "Julia",     "Luisa",     "Miriam",      "Karina",     "Antonia",   "Paola",       "Carolina",
    "Cecilia",     "Sonia",     "Hilda",     "Mabel",       "Elba",       "Valeria",   "Rosana",      "Silvana",
    "Silvina",     "Paula",     "Carina",    "Yolanda",     "Francisca",  "Noemi",     "Elvira",      "Romina",
    "Celia",       "Jesus",     "Marisa",    "Gloria",      "Clara",      "Angelica",  "Ines",        "Daniela",
    "Josefa",      "Catalina",  "Eva",       "Raquel",      "Martha",     "Marina",    "Elida",       "Rita",
    "Roxana",      "Griselda",  "Haydee",    "Vanesa",      "Esther",     "Amalia",    "Elisa",       "Amelia",
    "Victoria",    "Zulema",    "Nelly",     "Ester",       "Dominga",    "Zulma",     "Virginia",    "Fabiana",
    "Gladis",      "Aida",      "Irene",     "Adela",       "Elizabeth",  "Rene",      "Emilia",      "Rosario",
    "Josefina",    "Edith",     "Matilde",   "Noelia",      "Petrona",    "Mirtha",    "Elva",        "Aurora",
    "Vilma",       "Leonor",    "Luciana",   "Dolores",     "Gisela",     "Emma",      "Magdalena",   "Cintia",
    "Carla",       "Flavia",    "Ivana",     "Felisa",      "Sofia",      "Mirian",    "Erica",       "Florencia",
    "Yanina",      "Berta",     "Elda",      "Ilda",        "Diana",      "Jorgelina", "Myriam",      "Amanda",
    "Ofelia",      "Manuela",   "Rosalia",   "Lilia",       "Leticia",    "Alcira",    "Teresita",    "Nidia",
    "Iris",        "Vanina",    "Paulina",   "Soledad",     "Azucena",    "Adelina",   "Eugenia",     "Sabrina",
    "Agustina",    "Julieta",   "Eliana",    "Fernanda",    "Vicenta",    "Martina",   "Celina",      "Alcides",
    "Evangelina",  "Debora",    "Gregoria",  "Marcelina",   "Selva",      "Lilian",    "Natividad",   "Concepcion",
    "Ada",         "Herminia",  "Argentina", "Aurelia",     "Tomasa",     "Nadia",     "Lucrecia",    "Ema",
    "Guillermina", "Jesica",    "Nicolasa",  "Zunilda",     "Felipa",     "Fanny",     "Reina",       "Ruth",
    "Barbara",     "Filomena",  "Clelia",    "Ida",         "Anita",      "Lorenza",   "Delfina",     "Mariel",
    "Emilce",      "Enriqueta", "Pamela",    "Clementina",  "Pilar",      "Fatima",    "Juliana",     "Florinda",
    "Cynthia",     "Myrian",    "Ernestina", "Ermelinda",   "Nanci",      "Dina",      "Regina",      "Etelvina",
    "Adelaida",    "Melina",    "Corina",    "Jacinta",     "Delicia",    "Mirna",     "Valentina",   "Lydia",
    "Bibiana",     "Asuncion",  "America",   "Micaela",     "Isolina",    "Elisabet",  "Elina",       "Deolinda",
    "Lia",         "Perla",     "Marisol",   "Ignacia",     "Pabla",      "Benita",    "Florentina",  "Santos",
    "Clotilde",    "Sabina",    "Celestina", "Justina",     "Lucila",     "Maxima",    "Yesica",      "Georgina",
    "Teodora",     "Guadalupe", "Flora",     "Maricel",     "Genoveva",   "Arminda",   "Violeta",     "Rafaela",
    "Lina",        "Mara",      "Luz",       "Gilda",       "Erika",      "Segunda",   "Rocio",       "Jimena",
    "Albina",      "Transito",  "Hortencia", "Yamila",      "Trinidad",   "Angelina",  "Otilia",      "Justa",
    "Eusebia",     "Malvina",   "Marianela", "Mary",        "Araceli",    "Melisa",    "Hebe",        "Brigida",
    "Faustina",    "Eulalia",   "Hortensia", "Judith",      "Telma",      "Ceferina",  "Candida",     "Ercilia",
    "Isidora",     "Clarisa",   "Erminda",   "Eloisa",      "Fermina",    "Rufina",    "Modesta",     "Edit",
    "Geronima",    "Carola",    "Anabella",  "Encarnacion", "Bernardina", "Edelmira",  "Alejandrina", "Hermelinda",
    "Dionisia",    "Olinda",    "Palmira",   "Esperanza",   "Pascuala",   "Rebeca",    "Alexis",      "Marcia",
    "Victorina",   "Agueda",    "Jessica",   "Anahi",       "Elisabeth",  "Lelia",     "Estefania",   "Sixta",
    "Inocencia",   "Damiana",   "Ethel",     "Eulogia",     "Simona",     "Renee",     "Carlota",     "Eleonora",
    "Elma",        "Bernarda",  "Lourdes",   "Jaquelina",   "Mafalda",    "Anabel",    "Cirila",      "Emilse",
    "Miryam",      "Betina",    "Herminda",  "Elia",        "Tamara",     "Feliciana",
});

const auto argentinianLastNames = std::to_array<std::string_view>({
    "Gonzalez",   "Rodriguez",  "Gomez",       "Fernandez",  "Lopez",      "Diaz",       "Martinez",    "Perez",
    "Garcia",     "Sanchez",    "Romero",      "Sosa",       "Torres",     "Alvarez",    "Ruiz",        "Ramirez",
    "Flores",     "Benitez",    "Acosta",      "Medina",     "Herrera",    "Suarez",     "Aguirre",     "Gimenez",
    "Gutierrez",  "Pereyra",    "Rojas",       "Molina",     "Castro",     "Ortiz",      "Silva",       "Nuñez",
    "Luna",       "Juarez",     "Cabrera",     "Rios",       "Morales",    "Godoy",      "Moreno",      "Ferreyra",
    "Dominguez",  "Carrizo",    "Peralta",     "Castillo",   "Ledesma",    "Quiroga",    "Vega",        "Vera",
    "Muñoz",      "Ojeda",      "Ponce",       "Villalba",   "Cardozo",    "Navarro",    "Coronel",     "Vazquez",
    "Ramos",      "Vargas",     "Caceres",     "Arias",      "Figueroa",   "Cordoba",    "Correa",      "Maldonado",
    "Paz",        "Rivero",     "Miranda",     "Mansilla",   "Farias",     "Roldan",     "Mendez",      "Guzman",
    "Aguero",     "Hernandez",  "Lucero",      "Cruz",       "Paez",       "Escobar",    "Mendoza",     "Barrios",
    "Bustos",     "Avila",      "Ayala",       "Blanco",     "Soria",      "Maidana",    "Acuña",       "Leiva",
    "Duarte",     "Moyano",     "Campos",      "Soto",       "Martin",     "Valdez",     "Bravo",       "Chavez",
    "Velazquez",  "Olivera",    "Toledo",      "Franco",     "Ibañez",     "Leguizamon", "Montenegro",  "Delgado",
    "Arce",       "Ibarra",     "Gallardo",    "Santillan",  "Acevedo",    "Aguilar",    "Vallejos",    "Contreras",
    "Alegre",     "Galvan",     "Oviedo",      "Aranda",     "Albornoz",   "Baez",       "Sandoval",    "Barrionuevo",
    "Veron",      "Gauna",      "Zarate",      "Heredia",    "Mercado",    "Monzon",     "Marquez",     "Zalazar",
    "Mamani",     "Coria",      "Segovia",     "Romano",     "Jimenez",    "Salinas",    "Quinteros",   "Barrera",
    "Ortega",     "Cabral",     "Palacios",    "Cejas",      "Quintana",   "Zapata",     "Rosales",     "Altamirano",
    "Nieva",      "Bazan",      "Alonso",      "Burgos",     "Bustamante", "Varela",     "Lescano",     "Aguilera",
    "Paredes",    "Avalos",     "Cuello",      "Aquino",     "Orellana",   "Caballero",  "Reynoso",     "Reyes",
    "Villarreal", "Alarcon",    "Pacheco",     "Tapia",      "Galarza",    "Ocampo",     "Meza",        "Guerrero",
    "Salas",      "Frias",      "Videla",      "Miño",       "Jara",       "Garay",      "Rossi",       "Lezcano",
    "Valenzuela", "Oliva",      "Fuentes",     "Robledo",    "Espindola",  "Nieto",      "Pereira",     "Brizuela",
    "Andrada",    "Maciel",     "Funes",       "Robles",     "Sotelo",     "Cortez",     "Almiron",     "Rivas",
    "Gil",        "Villegas",   "Calderon",    "Vergara",    "Carabajal",  "Ceballos",   "Gallo",       "Palavecino",
    "Barreto",    "Alderete",   "Escudero",    "Saavedra",   "Serrano",    "Almada",     "Galeano",     "Espinosa",
    "Villagra",   "Gerez",      "Solis",       "Ochoa",      "Escalante",  "Luque",      "Amaya",       "Arguello",
    "Salazar",    "Lazarte",    "Barrientos",  "Vidal",      "Machado",    "Ferreira",   "Argañaraz",   "Iglesias",
    "Guevara",    "Centurion",  "Esquivel",    "Lencina",    "Jaime",      "Cano",       "Lujan",       "Espinoza",
    "Palacio",    "Villanueva", "Salvatierra", "Guerra",     "Barraza",    "Bordon",     "Saucedo",     "Ferrari",
    "Costa",      "Rolon",      "Zabala",      "Albarracin", "Duran",      "Peña",       "Tello",       "Quiroz",
    "Montes",     "Alfonso",    "Brito",       "Marin",      "Moreira",    "Olmos",      "Montiel",     "Pintos",
    "Olmedo",     "Bruno",      "Villafañe",   "Arroyo",     "Reinoso",    "Araujo",     "Gorosito",    "Cisneros",
    "Quevedo",    "Montero",    "Barros",      "Moya",       "Basualdo",   "Carballo",   "Insaurralde", "Prieto",
    "Alcaraz",    "Santos",     "Corvalan",    "Chamorro",   "Casas",      "Carranza",   "Moreyra",     "Chaves",
    "Riquelme",   "Arevalo",    "Bogado",      "Sequeira",   "Amarilla",   "Parra",      "Corbalan",    "Veliz",
    "Falcon",     "Moran",      "Cantero",     "Otero",      "Rocha",      "Lobo",       "Cuevas",      "Roman",
    "Caro",       "Jofre",      "Nievas",      "Pinto",      "Arrieta",    "Portillo",   "Rivera",      "Gramajo",
    "Quintero",   "Russo",      "Carrasco",    "Castaño",    "Vivas",      "Gamarra",    "Ludueña",     "Britez",
    "Britos",     "Cortes",     "Becerra",     "Prado",      "Merlo",      "Barboza",    "Hidalgo",     "Olguin",
    "Taborda",    "Tolaba",     "Villa",       "Leon",       "Sarmiento",  "Cañete",     "Gaitan",      "Sanabria",
    "Ovejero",    "Da'Pavon",   "Rey",         "Mora",       "Vasquez",    "Cardenas",   "Cornejo",     "Orellano",
    "Dos'Alaniz", "Araya",      "Vallejo",     "Enriquez",   "Bianchi",    "Barroso",    "Calvo",       "Lemos",
    "Casco",      "Trejo",      "Andrade",     "Balmaceda",
});

const NameFormats argentinianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames argentinianPeopleNames{{argentinianMaleFirstNames, argentinianLastNames, {}, {}},
                                         {argentinianFemaleFirstNames, argentinianLastNames, {}, {}},
                                         argentinianNameFormats};

// Australia

const auto australianMaleFirstNames = std::to_array<std::string_view>({
    "William",   "Jack",      "Oliver",   "Joshua",   "Thomas",   "Lachlan",   "Cooper",   "Noah",        "Ethan",
    "Lucas",     "James",     "Samuel",   "Jacob",    "Liam",     "Alexander", "Benjamin", "Max",         "Isaac",
    "Daniel",    "Riley",     "Ryan",     "Charlie",  "Tyler",    "Jake",      "Matthew",  "Xavier",      "Harry",
    "Jayden",    "Nicholas",  "Harrison", "Levi",     "Luke",     "Adam",      "Henry",    "Aiden",       "Dylan",
    "Oscar",     "Michael",   "Jackson",  "Logan",    "Joseph",   "Blake",     "Nathan",   "Connor",      "Elijah",
    "Nate",      "Archie",    "Bailey",   "Marcus",   "Cameron",  "Jordan",    "Zachary",  "Caleb",       "Hunter",
    "Ashton",    "Toby",      "Aidan",    "Hayden",   "Mason",    "Hamish",    "Edward",   "Angus",       "Eli",
    "Sebastian", "Christian", "Patrick",  "Andrew",   "Anthony",  "Luca",      "Kai",      "Beau",        "Alex",
    "George",    "Callum",    "Finn",     "Zac",      "Mitchell", "Jett",      "Jesse",    "Gabriel",     "Leo",
    "Declan",    "Charles",   "Jasper",   "Jonathan", "Aaron",    "Hugo",      "David",    "Christopher", "Chase",
    "Owen",      "Justin",    "Ali",      "Darcy",    "Lincoln",  "Cody",      "Phoenix",  "Sam",         "John",
    "Joel",
});

const auto australianFemaleFirstNames = std::to_array<std::string_view>({
    "Isabella", "Ruby",      "Chloe",    "Olivia",    "Charlotte", "Mia",     "Lily",     "Emily",    "Ella",
    "Sienna",   "Sophie",    "Amelia",   "Grace",     "Ava",       "Zoe",     "Emma",     "Sophia",   "Matilda",
    "Hannah",   "Jessica",   "Lucy",     "Georgia",   "Sarah",     "Abigail", "Zara",     "Eva",      "Scarlett",
    "Jasmine",  "Chelsea",   "Lilly",    "Ivy",       "Isla",      "Evie",    "Isabelle", "Maddison", "Layla",
    "Summer",   "Annabelle", "Alexis",   "Elizabeth", "Bella",     "Holly",   "Lara",     "Madison",  "Alyssa",
    "Maya",     "Tahlia",    "Claire",   "Hayley",    "Imogen",    "Jade",    "Ellie",    "Sofia",    "Addison",
    "Molly",    "Phoebe",    "Alice",    "Savannah",  "Gabriella", "Kayla",   "Mikayla",  "Abbey",    "Eliza",
    "Willow",   "Alexandra", "Poppy",    "Samantha",  "Stella",    "Amy",     "Amelie",   "Anna",     "Piper",
    "Gemma",    "Isabel",    "Victoria", "Stephanie", "Caitlin",   "Heidi",   "Paige",    "Rose",     "Amber",
    "Audrey",   "Claudia",   "Taylor",   "Madeline",  "Angelina",  "Natalie", "Charli",   "Lauren",   "Ashley",
    "Violet",   "Mackenzie", "Abby",     "Skye",      "Lillian",   "Alana",   "Lola",     "Leah",     "Eve",
    "Kiara",
});

const auto australianLastNames = std::to_array<std::string_view>({
    "Smith",       "Jones",      "Williams",  "Brown",       "Wilson",     "Taylor",       "Johnson",    "White",
    "Martin",      "Anderson",   "Thompson",  "Nguyen",      "Thomas",     "Walker",       "Harris",     "Lee",
    "Ryan",        "Robinson",   "Kelly",     "King",        "Davis",      "Wright",       "Evans",      "Roberts",
    "Green",       "Hall",       "Wood",      "Jackson",     "Clarke",     "Patel",        "Khan",       "Lewis",
    "James",       "Phillips",   "Mason",     "Mitchell",    "Rose",       "Davies",       "Rodriguez",  "Cox",
    "Alexander",   "Garden",     "Campbell",  "Johnston",    "Moore",      "Smyth",        "O'Neill ",   "Doherty",
    "Stewart",     "Quinn",      "Murphy",    "Graham",      "Mclaughlin", "Hamilton",     "Murray",     "Hughes",
    "Robertson",   "Thomson",    "Scott",     "Macdonald",   "Reid",       "Clark",        "Ross",       "Young",
    "Watson",      "Paterson",   "Morrison",  "Morgan",      "Griffiths",  "Edwards",      "Rees",       "Jenkins",
    "Owen",        "Price",      "Moss",      "Richards",    "Abbott",     "Adams",        "Armstrong",  "Bahringer",
    "Bailey",      "Barrows",    "Bartell",   "Bartoletti",  "Barton",     "Bauch",        "Baumbach",   "Bayer",
    "Beahan",      "Beatty",     "Becker",    "Beier",       "Berge",      "Bergstrom",    "Bode",       "Bogan",
    "Borer",       "Bosco",      "Botsford",  "Boyer",       "Boyle",      "Braun",        "Bruen",      "Carroll",
    "Carter",      "Cartwright", "Casper",    "Cassin",      "Champlin",   "Christiansen", "Cole",       "Collier",
    "Collins",     "Connelly",   "Conroy",    "Corkery",     "Cormier",    "Corwin",       "Cronin",     "Crooks",
    "Cruickshank", "Cummings",   "D' Amore",  "Daniel",      "Dare",       "Daugherty",    "Dickens",    "Dickinson",
    "Dietrich",    "Donnelly",   "Dooley",    "Douglas",     "Doyle",      "Durgan",       "Ebert",      "Emard",
    "Emmerich",    "Erdman",     "Ernser",    "Fadel",       "Fahey",      "Farrell",      "Fay",        "Feeney",
    "Feil",        "Ferry",      "Fisher",    "Flatley",     "Gibson",     "Gleason",      "Glover",     "Goldner",
    "Goodwin",     "Grady",      "Grant",     "Greenfelder", "Greenholt",  "Grimes",       "Gutmann",    "Hackett",
    "Hahn",        "Haley",      "Hammes",    "Hand",        "Hane",       "Hansen",       "Harber",     "Hartmann",
    "Harvey",      "Hayes",      "Heaney",    "Heathcote",   "Heller",     "Hermann",      "Hermiston",  "Hessel",
    "Hettinger",   "Hickle",     "Hill",      "Hills",       "Hoppe",      "Howe",         "Howell",     "Hudson",
    "Huel",        "Hyatt",      "Jacobi",    "Jacobs",      "Jacobson",   "Jerde",        "Johns",      "Keeling",
    "Kemmer",      "Kessler",    "Kiehn",     "Kirlin",      "Klein",      "Koch",         "Koelpin",    "Kohler",
    "Koss",        "Kovacek",    "Kreiger",   "Kris",        "Kuhlman",    "Kuhn",         "Kulas",      "Kunde",
    "Kutch",       "Lakin",      "Lang",      "Langworth",   "Larkin",     "Larson",       "Leannon",    "Leffler",
    "Little",      "Lockman",    "Lowe",      "Lynch",       "Mann",       "Marks",        "Marvin",     "Mayer",
    "Mccullough",  "Mcdermott",  "Mckenzie",  "Miller",      "Mills",      "Monahan",      "Morissette", "Mueller",
    "Muller",      "Nader",      "Nicolas",   "Nolan",       "O' Connell", "O 'Conner",    "O' Hara",    "O 'Keefe",
    "Olson",       "O' Reilly",  "Parisian",  "Parker",      "Quigley",    "Reilly",       "Reynolds",   "Rice",
    "Ritchie",     "Rohan",      "Rolfson",   "Rowe",        "Russel",     "Rutherford",   "Sanford",    "Sauer",
    "Schmidt",     "Schmitt",    "Schneider", "Schroeder",   "Schultz",    "Shields",      "Smitham",    "Spencer",
    "Stanton",     "Stark",      "Stokes",    "Swift",       "Tillman",    "Towne",        "Tremblay",   "Tromp",
    "Turcotte",    "Turner",     "Walsh",     "Walter",      "Ward",       "Waters",       "Weber",      "Welch",
    "West",        "Wilderman",  "Wilkinson", "Williamson",  "Windler",    "Wolf",
});

const NameFormats australianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames australianPeopleNames{{australianMaleFirstNames, australianLastNames, {}, {}},
                                        {australianFemaleFirstNames, australianLastNames, {}, {}},
                                        australianNameFormats};

// Austria

const auto austrianMaleFirstNames = std::to_array<std::string_view>({
    "Aaron",      "Abdul",    "Abdullah",  "Adam",       "Adrian",     "Adriano",     "Ahmad",     "Ahmed",
    "Ahmet",      "Alan",     "Albert",    "Alessandro", "Alessio",    "Alex",        "Alexander", "Alfred",
    "Ali",        "Amar",     "Amir",      "Amon",       "Andre",      "Andreas",     "Andrew",    "Angelo",
    "Ansgar",     "Anthony",  "Anton",     "Antonio",    "Arda",       "Arian",       "Armin",     "Arne",
    "Arno",       "Arthur",   "Artur",     "Arved",      "Arvid",      "Ayman",       "Baran",     "Baris",
    "Bastian",    "Batuhan",  "Bela",      "Ben",        "Benedikt",   "Benjamin",    "Bennet",    "Bennett",
    "Benno",      "Bent",     "Berat",     "Berkay",     "Bernd",      "Bilal",       "Bjarne",    "Björn",
    "Bo",         "Boris",    "Brandon",   "Brian",      "Bruno",      "Bryan",       "Burak",     "Calvin",
    "Can",        "Carl",     "Carlo",     "Carlos",     "Caspar",     "Cedric",      "Cedrik",    "Cem",
    "Charlie",    "Chris",    "Christian", "Christiano", "Christoph",  "Christopher", "Claas",     "Clemens",
    "Colin",      "Collin",   "Conner",    "Connor",     "Constantin", "Corvin",      "Curt",      "Damian",
    "Damien",     "Daniel",   "Danilo",    "Danny",      "Darian",     "Dario",       "Darius",    "Darren",
    "David",      "Davide",   "Davin",     "Dean",       "Deniz",      "Dennis",      "Denny",     "Devin",
    "Diego",      "Dion",     "Domenic",   "Domenik",    "Dominic",    "Dominik",     "Dorian",    "Dustin",
    "Dylan",      "Ecrin",    "Eddi",      "Eddy",       "Edgar",      "Edwin",       "Efe",       "Ege",
    "Elia",       "Eliah",    "Elias",     "Elijah",     "Emanuel",    "Emil",        "Emilian",   "Emilio",
    "Emir",       "Emirhan",  "Emre",      "Enes",       "Enno",       "Enrico",      "Eren",      "Eric",
    "Erik",       "Etienne",  "Fabian",    "Fabien",     "Fabio",      "Fabrice",     "Falk",      "Felix",
    "Ferdinand",  "Fiete",    "Filip",     "Finlay",     "Finley",     "Finn",        "Finnley",   "Florian",
    "Francesco",  "Franz",    "Frederic",  "Frederick",  "Frederik",   "Friedrich",   "Fritz",     "Furkan",
    "Fynn",       "Gabriel",  "Georg",     "Gerrit",     "Gian",       "Gianluca",    "Gino",      "Giuliano",
    "Giuseppe",   "Gregor",   "Gustav",    "Hagen",      "Hamza",      "Hannes",      "Hanno",     "Hans",
    "Hasan",      "Hassan",   "Hauke",     "Hendrik",    "Hennes",     "Henning",     "Henri",     "Henrick",
    "Henrik",     "Henry",    "Hugo",      "Hussein",    "Ian",        "Ibrahim",     "Ilias",     "Ilja",
    "Ilyas",      "Immanuel", "Ismael",    "Ismail",     "Ivan",       "Iven",        "Jack",      "Jacob",
    "Jaden",      "Jakob",    "Jamal",     "James",      "Jamie",      "Jan",         "Janek",     "Janis",
    "Janne",      "Jannek",   "Jannes",    "Jannik",     "Jannis",     "Jano",        "Janosch",   "Jared",
    "Jari",       "Jarne",    "Jarno",     "Jaron",      "Jason",      "Jasper",      "Jay",       "Jayden",
    "Jayson",     "Jean",     "Jens",      "Jeremias",   "Jeremie",    "Jeremy",      "Jermaine",  "Jerome",
    "Jesper",     "Jesse",    "Jim",       "Jimmy",      "Joe",        "Joel",        "Joey",      "Johann",
    "Johannes",   "John",     "Johnny",    "Jon",        "Jona",       "Jonah",       "Jonas",     "Jonathan",
    "Jonte",      "Joost",    "Jordan",    "Joris",      "Joscha",     "Joschua",     "Josef",     "Joseph",
    "Josh",       "Joshua",   "Josua",     "Juan",       "Julian",     "Julien",      "Julius",    "Juri",
    "Justin",     "Justus",   "Kaan",      "Kai",        "Kalle",      "Karim",       "Karl",      "Karlo",
    "Kay",        "Keanu",    "Kenan",     "Kenny",      "Keno",       "Kerem",       "Kerim",     "Kevin",
    "Kian",       "Kilian",   "Kim",       "Kimi",       "Kjell",      "Klaas",       "Klemens",   "Konrad",
    "Konstantin", "Koray",    "Korbinian", "Kurt",       "Lars",       "Lasse",       "Laurence",  "Laurens",
    "Laurenz",    "Laurin",   "Lean",      "Leander",    "Leandro",    "Leif",        "Len",       "Lenn",
    "Lennard",    "Lennart",  "Lennert",   "Lennie",     "Lennox",     "Lenny",       "Leo",       "Leon",
    "Leonard",    "Leonardo", "Leonhard",  "Leonidas",   "Leopold",    "Leroy",       "Levent",    "Levi",
    "Levin",      "Lewin",    "Lewis",     "Liam",       "Lian",       "Lias",        "Lino",      "Linus",
    "Lio",        "Lion",     "Lionel",    "Logan",      "Lorenz",     "Lorenzo",     "Loris",     "Louis",
    "Luan",       "Luc",      "Luca",      "Lucas",      "Lucian",     "Lucien",      "Ludwig",    "Luis",
    "Luiz",       "Luk",      "Luka",      "Lukas",      "Luke",       "Lutz",        "Maddox",    "Mads",
    "Magnus",     "Maik",     "Maksim",    "Malik",      "Malte",      "Manuel",      "Marc",      "Marcel",
    "Marco",      "Marcus",   "Marek",     "Marian",     "Mario",      "Marius",      "Mark",      "Marko",
    "Markus",     "Marlo",    "Marlon",    "Marten",     "Martin",     "Marvin",      "Marwin",    "Mateo",
    "Mathis",     "Matis",    "Mats",      "Matteo",     "Mattes",     "Matthias",    "Matthis",   "Matti",
    "Mattis",     "Maurice",  "Max",       "Maxim",      "Maximilian", "Mehmet",      "Meik",      "Melvin",
    "Merlin",     "Mert",     "Michael",   "Michel",     "Mick",       "Miguel",      "Mika",      "Mikail",
    "Mike",       "Milan",    "Milo",      "Mio",        "Mirac",      "Mirco",       "Mirko",     "Mohamed",
    "Mohammad",   "Mohammed", "Moritz",    "Morten",     "Muhammed",   "Murat",       "Mustafa",   "Nathan",
    "Nathanael",  "Nelson",   "Neo",       "Nevio",      "Nick",       "Niclas",      "Nico",      "Nicolai",
    "Nicolas",    "Niels",    "Nikita",    "Niklas",     "Niko",       "Nikolai",     "Nikolas",   "Nils",
    "Nino",       "Noah",     "Noel",      "Norman",     "Odin",       "Oke",         "Ole",       "Oliver",
    "Omar",       "Onur",     "Oscar",     "Oskar",      "Pascal",     "Patrice",     "Patrick",   "Paul",
    "Peer",       "Pepe",     "Peter",     "Phil",       "Philip",     "Philipp",     "Pierre",    "Piet",
    "Pit",        "Pius",     "Quentin",   "Quirin",     "Rafael",     "Raik",        "Ramon",     "Raphael",
    "Rasmus",     "Raul",     "Rayan",     "René",       "Ricardo",    "Riccardo",    "Richard",   "Rick",
    "Rico",       "Robert",   "Robin",     "Rocco",      "Roman",      "Romeo",       "Ron",       "Ruben",
    "Ryan",       "Said",     "Salih",     "Sam",        "Sami",       "Sammy",       "Samuel",    "Sandro",
    "Santino",    "Sascha",   "Sean",      "Sebastian",  "Selim",      "Semih",       "Shawn",     "Silas",
    "Simeon",     "Simon",    "Sinan",     "Sky",        "Stefan",     "Steffen",     "Stephan",   "Steve",
    "Steven",     "Sven",     "Sönke",     "Sören",      "Taha",       "Tamino",      "Tammo",     "Tarik",
    "Tayler",     "Taylor",   "Teo",       "Theo",       "Theodor",    "Thies",       "Thilo",     "Thomas",
    "Thorben",    "Thore",    "Thorge",    "Tiago",      "Til",        "Till",        "Tillmann",  "Tim",
    "Timm",       "Timo",     "Timon",     "Timothy",    "Tino",       "Titus",       "Tizian",    "Tjark",
    "Tobias",     "Tom",      "Tommy",     "Toni",       "Tony",       "Torben",      "Tore",      "Tristan",
    "Tyler",      "Tyron",    "Umut",      "Valentin",   "Valentino",  "Veit",        "Victor",    "Viktor",
    "Vin",        "Vincent",  "Vito",      "Vitus",      "Wilhelm",    "Willi",       "William",   "Willy",
    "Xaver",      "Yannic",   "Yannick",   "Yannik",     "Yannis",     "Yasin",       "Youssef",   "Yunus",
    "Yusuf",      "Yven",     "Yves",      "Ömer",
});

const auto austrianFemaleFirstNames = std::to_array<std::string_view>({
    "Aaliyah",   "Abby",      "Abigail",    "Ada",       "Adelina",   "Adriana",   "Aileen",    "Aimee",
    "Alana",     "Alea",      "Alena",      "Alessa",    "Alessia",   "Alexa",     "Alexandra", "Alexia",
    "Alexis",    "Aleyna",    "Alia",       "Alica",     "Alice",     "Alicia",    "Alina",     "Alisa",
    "Alisha",    "Alissa",    "Aliya",      "Aliyah",    "Allegra",   "Alma",      "Alyssa",    "Amalia",
    "Amanda",    "Amelia",    "Amelie",     "Amina",     "Amira",     "Amy",       "Ana",       "Anabel",
    "Anastasia", "Andrea",    "Angela",     "Angelina",  "Angelique", "Anja",      "Ann",       "Anna",
    "Annabel",   "Annabell",  "Annabelle",  "Annalena",  "Anne",      "Anneke",    "Annelie",   "Annemarie",
    "Anni",      "Annie",     "Annika",     "Anny",      "Anouk",     "Antonia",   "Ariana",    "Ariane",
    "Arwen",     "Ashley",    "Asya",       "Aurelia",   "Aurora",    "Ava",       "Ayleen",    "Aylin",
    "Ayse",      "Azra",      "Betty",      "Bianca",    "Bianka",    "Caitlin",   "Cara",      "Carina",
    "Carla",     "Carlotta",  "Carmen",     "Carolin",   "Carolina",  "Caroline",  "Cassandra", "Catharina",
    "Catrin",    "Cecile",    "Cecilia",    "Celia",     "Celina",    "Celine",    "Ceyda",     "Ceylin",
    "Chantal",   "Charleen",  "Charlotta",  "Charlotte", "Chayenne",  "Cheyenne",  "Chiara",    "Christin",
    "Christina", "Cindy",     "Claire",     "Clara",     "Clarissa",  "Colleen",   "Collien",   "Cora",
    "Corinna",   "Cosima",    "Dana",       "Daniela",   "Daria",     "Darleen",   "Defne",     "Delia",
    "Denise",    "Diana",     "Dilara",     "Dina",      "Dorothea",  "Eda",       "Eileen",    "Ela",
    "Elaine",    "Elanur",    "Elea",       "Elena",     "Eleni",     "Eleonora",  "Eliana",    "Elif",
    "Elina",     "Elisa",     "Elisabeth",  "Ella",      "Ellen",     "Elli",      "Elly",      "Elsa",
    "Emelie",    "Emely",     "Emilia",     "Emilie",    "Emily",     "Emma",      "Emmely",    "Emmi",
    "Emmy",      "Enie",      "Enna",       "Enya",      "Esma",      "Estelle",   "Esther",    "Eva",
    "Evelin",    "Evelina",   "Eveline",    "Evelyn",    "Fabienne",  "Fatima",    "Fatma",     "Felicia",
    "Felicitas", "Felina",    "Femke",      "Fenja",     "Fine",      "Finia",     "Finja",     "Finnja",
    "Fiona",     "Flora",     "Florentine", "Francesca", "Franka",    "Franziska", "Frederike", "Freya",
    "Frida",     "Frieda",    "Friederike", "Giada",     "Gina",      "Giulia",    "Giuliana",  "Greta",
    "Hailey",    "Hana",      "Hanna",      "Hannah",    "Heidi",     "Helen",     "Helena",    "Helene",
    "Helin",     "Henriette", "Henrike",    "Hermine",   "Ida",       "Ilayda",    "Imke",      "Ina",
    "Ines",      "Inga",      "Inka",       "Irem",      "Isa",       "Isabel",    "Isabell",   "Isabella",
    "Isabelle",  "Ivonne",    "Jacqueline", "Jamila",    "Jana",      "Jane",      "Janin",     "Janina",
    "Janine",    "Janna",     "Jara",       "Jasmin",    "Jasmina",   "Jasmine",   "Jella",     "Jenna",
    "Jennifer",  "Jenny",     "Jessica",    "Jessy",     "Jette",     "Jil",       "Jill",      "Joana",
    "Joanna",    "Joelina",   "Joeline",    "Joelle",    "Johanna",   "Joleen",    "Jolie",     "Jolien",
    "Jolin",     "Jolina",    "Joline",     "Jonna",     "Josefin",   "Josefine",  "Josephin",  "Josephine",
    "Josie",     "Josy",      "Joy",        "Joyce",     "Judith",    "Judy",      "Jule",      "Julia",
    "Juliana",   "Juliane",   "Julie",      "Julienne",  "Julika",    "Julina",    "Juna",      "Justine",
    "Kaja",      "Karina",    "Karla",      "Karlotta",  "Karolina",  "Karoline",  "Kassandra", "Katarina",
    "Katharina", "Kathrin",   "Katja",      "Katrin",    "Kaya",      "Kayra",     "Kiana",     "Kiara",
    "Kimberley", "Kimberly",  "Kira",       "Klara",     "Korinna",   "Kristin",   "Kyra",      "Laila",
    "Lana",      "Lara",      "Larissa",    "Laura",     "Laureen",   "Lavinia",   "Lea",       "Leah",
    "Leana",     "Leandra",   "Leann",      "Lee",       "Leila",     "Lena",      "Lene",      "Leni",
    "Lenia",     "Lenja",     "Lenya",      "Leona",     "Leoni",     "Leonie",    "Leonora",   "Leticia",
    "Letizia",   "Levke",     "Leyla",      "Lia",       "Liah",      "Liana",     "Lili",      "Lilia",
    "Lilian",    "Liliana",   "Lilith",     "Lilli",     "Lillian",   "Lilly",     "Lily",      "Lina",
    "Linda",     "Lindsay",   "Line",       "Linn",      "Linnea",    "Lisa",      "Lisann",    "Lisanne",
    "Liv",       "Livia",     "Liz",        "Lola",      "Loreen",    "Lorena",    "Lotta",     "Lotte",
    "Louisa",    "Louise",    "Luana",      "Lucia",     "Lucie",     "Lucienne",  "Lucy",      "Luisa",
    "Luise",     "Luna",      "Luzie",      "Lya",       "Lydia",     "Lyn",       "Lynn",      "Madeleine",
    "Madita",    "Madleen",   "Madlen",     "Magdalena", "Maike",     "Mailin",    "Maira",     "Maja",
    "Malena",    "Malia",     "Malin",      "Malina",    "Mandy",     "Mara",      "Marah",     "Mareike",
    "Maren",     "Maria",     "Mariam",     "Marie",     "Marieke",   "Mariella",  "Marika",    "Marina",
    "Marisa",    "Marissa",   "Marit",      "Marla",     "Marleen",   "Marlen",    "Marlena",   "Marlene",
    "Marta",     "Martha",    "Mary",       "Maryam",    "Mathilda",  "Mathilde",  "Matilda",   "Maxi",
    "Maxima",    "Maxine",    "Maya",       "Mayra",     "Medina",    "Medine",    "Meike",     "Melanie",
    "Melek",     "Melike",    "Melina",     "Melinda",   "Melis",     "Melisa",    "Melissa",   "Merle",
    "Merve",     "Meryem",    "Mette",      "Mia",       "Michaela",  "Michelle",  "Mieke",     "Mila",
    "Milana",    "Milena",    "Milla",      "Mina",      "Mira",      "Miray",     "Miriam",    "Mirja",
    "Mona",      "Monique",   "Nadine",     "Nadja",     "Naemi",     "Nancy",     "Naomi",     "Natalia",
    "Natalie",   "Nathalie",  "Neele",      "Nela",      "Nele",      "Nelli",     "Nelly",     "Nia",
    "Nicole",    "Nika",      "Nike",       "Nila",      "Nina",      "Nisa",      "Noemi",     "Nora",
    "Olivia",    "Patricia",  "Patrizia",   "Paula",     "Paulina",   "Pauline",   "Penelope",  "Philine",
    "Phoebe",    "Pia",       "Rahel",      "Rania",     "Rebecca",   "Rebekka",   "Riana",     "Rieke",
    "Rike",      "Romina",    "Romy",       "Ronja",     "Rosa",      "Rosalie",   "Ruby",      "Sabrina",
    "Sahra",     "Sally",     "Salome",     "Samantha",  "Samia",     "Samira",    "Sandra",    "Sandy",
    "Sanja",     "Saphira",   "Sara",       "Sarah",     "Saskia",    "Selin",     "Selina",    "Selma",
    "Sena",      "Sidney",    "Sienna",     "Silja",     "Sina",      "Sinja",     "Smilla",    "Sofia",
    "Sofie",     "Sonja",     "Sophia",     "Sophie",    "Soraya",    "Stefanie",  "Stella",    "Stephanie",
    "Stina",     "Sude",      "Summer",     "Susanne",   "Svea",      "Svenja",    "Sydney",    "Tabea",
    "Talea",     "Talia",     "Tamara",     "Tamia",     "Tamina",    "Tanja",     "Tara",      "Tarja",
    "Teresa",    "Tessa",     "Thalea",     "Thalia",    "Thea",      "Theresa",   "Tia",       "Tina",
    "Tomke",     "Tuana",     "Valentina",  "Valeria",   "Valerie",   "Vanessa",   "Vera",      "Veronika",
    "Victoria",  "Viktoria",  "Viola",      "Vivian",    "Vivien",    "Vivienne",  "Wibke",     "Wiebke",
    "Xenia",     "Yara",      "Yaren",      "Yasmin",    "Ylvi",      "Ylvie",     "Yvonne",    "Zara",
    "Zehra",     "Zeynep",    "Zoe",        "Zoey",      "Zoé",
});

const auto austrianLastNames = std::to_array<std::string_view>({
    "Abel",
    "Abicht",
    "Abraham",
    "Abramovic",
    "Abt",
    "Achilles",
    "Achkinadze",
    "Ackermann",
    "Adam",
    "Adams",
    "Ade",
    "Agostini",
    "Ahlke",
    "Ahrenberg",
    "Ahrens",
    "Aigner",
    "Albert",
    "Albrecht",
    "Alexa",
    "Alexander",
    "Alizadeh",
    "Allgeyer",
    "Amann",
    "Amberg",
    "Anding",
    "Anggreny",
    "Apitz",
    "Arendt",
    "Arens",
    "Arndt",
    "Aryee",
    "Aschenbroich",
    "Assmus",
    "Astafei",
    "Auer",
    "Axmann",
    "Baarck",
    "Bachmann",
    "Badane",
    "Bader",
    "Baganz",
    "Bahl",
    "Bak",
    "Balcer",
    "Balck",
    "Balkow",
    "Balnuweit",
    "Balzer",
    "Banse",
    "Barr",
    "Bartels",
    "Barth",
    "Barylla",
    "Baseda",
    "Battke",
    "Bauer",
    "Bauermeister",
    "Baumann",
    "Baumeister",
    "Bauschinger",
    "Bauschke",
    "Bayer",
    "Beavogui",
    "Beck",
    "Beckel",
    "Becker",
    "Beckmann",
    "Bedewitz",
    "Beele",
    "Beer",
    "Beggerow",
    "Beh",
    "Behr",
    "Behrenbruch",
    "Belz",
    "Bender",
    "Benecke",
    "Benner",
    "Benninger",
    "Benzing",
    "Berends",
    "Berger",
    "Berner",
    "Berning",
    "Bertenbreiter",
    "Best",
    "Bethke",
    "Betz",
    "Beushausen",
    "Beutelspacher",
    "Beyer",
    "Biba",
    "Bichler",
    "Bickel",
    "Biedermann",
    "Bieler",
    "Bielert",
    "Bienasch",
    "Bienias",
    "Biesenbach",
    "Bigdeli",
    "Birkemeyer",
    "Bittner",
    "Blank",
    "Blaschek",
    "Blassneck",
    "Bloch",
    "Blochwitz",
    "Blockhaus",
    "Blum",
    "Blume",
    "Bock",
    "Bode",
    "Bogdashin",
    "Bogenrieder",
    "Bohge",
    "Bolm",
    "Borgschulze",
    "Bork",
    "Bormann",
    "Bornscheuer",
    "Borrmann",
    "Borsch",
    "Boruschewski",
    "Bos",
    "Bosler",
    "Bourrouag",
    "Bouschen",
    "Boxhammer",
    "Boyde",
    "Bozsik",
    "Brand",
    "Brandenburg",
    "Brandis",
    "Brandt",
    "Brauer",
    "Braun",
    "Brehmer",
    "Breitenstein",
    "Bremer",
    "Bremser",
    "Brenner",
    "Brettschneider",
    "Breu",
    "Breuer",
    "Briesenick",
    "Bringmann",
    "Brinkmann",
    "Brix",
    "Broening",
    "Brosch",
    "Bruckmann",
    "Bruder",
    "Bruhns",
    "Brunner",
    "Bruns",
    "Bräutigam",
    "Brömme",
    "Brüggmann",
    "Buchholz",
    "Buchrucker",
    "Buder",
    "Bultmann",
    "Bunjes",
    "Burger",
    "Burghagen",
    "Burkhard",
    "Burkhardt",
    "Burmeister",
    "Busch",
    "Buschbaum",
    "Busemann",
    "Buss",
    "Busse",
    "Bussmann",
    "Byrd",
    "Bäcker",
    "Böhm",
    "Bönisch",
    "Börgeling",
    "Börner",
    "Böttner",
    "Büchele",
    "Bühler",
    "Büker",
    "Büngener",
    "Bürger",
    "Bürklein",
    "Büscher",
    "Büttner",
    "Camara",
    "Carlowitz",
    "Carlsohn",
    "Caspari",
    "Caspers",
    "Chapron",
    "Christ",
    "Cierpinski",
    "Clarius",
    "Cleem",
    "Cleve",
    "Co",
    "Conrad",
    "Cordes",
    "Cornelsen",
    "Cors",
    "Cotthardt",
    "Crews",
    "Cronjäger",
    "Crosskofp",
    "Da",
    "Dahm",
    "Dahmen",
    "Daimer",
    "Damaske",
    "Danneberg",
    "Danner",
    "Daub",
    "Daubner",
    "Daudrich",
    "Dauer",
    "Daum",
    "Dauth",
    "Dautzenberg",
    "De",
    "Decker",
    "Deckert",
    "Deerberg",
    "Dehmel",
    "Deja",
    "Delonge",
    "Demut",
    "Dengler",
    "Denner",
    "Denzinger",
    "Derr",
    "Dertmann",
    "Dethloff",
    "Deuschle",
    "Dieckmann",
    "Diedrich",
    "Diekmann",
    "Dienel",
    "Dies",
    "Dietrich",
    "Dietz",
    "Dietzsch",
    "Diezel",
    "Dilla",
    "Dingelstedt",
    "Dippl",
    "Dittmann",
    "Dittmar",
    "Dittmer",
    "Dix",
    "Dobbrunz",
    "Dobler",
    "Dohring",
    "Dolch",
    "Dold",
    "Dombrowski",
    "Donie",
    "Doskoczynski",
    "Dragu",
    "Drechsler",
    "Drees",
    "Dreher",
    "Dreier",
    "Dreissigacker",
    "Dressler",
    "Drews",
    "Duma",
    "Dutkiewicz",
    "Dyett",
    "Dylus",
    "Dächert",
    "Döbel",
    "Döring",
    "Dörner",
    "Dörre",
    "Dück",
    "Eberhard",
    "Eberhardt",
    "Ecker",
    "Eckhardt",
    "Edorh",
    "Effler",
    "Eggenmueller",
    "Ehm",
    "Ehmann",
    "Ehrig",
    "Eich",
    "Eifert",
    "Einert",
    "Eisenlauer",
    "Ekpo",
    "Elbe",
    "Eleyth",
    "Elss",
    "Emert",
    "Emmelmann",
    "Ender",
    "Engel",
    "Engelen",
    "Engelmann",
    "Eplinius",
    "Erdmann",
    "Erhardt",
    "Erlei",
    "Erm",
    "Ernst",
    "Ertl",
    "Erwes",
    "Esenwein",
    "Esser",
    "Evers",
    "Everts",
    "Ewald",
    "Fahner",
    "Faller",
    "Falter",
    "Farber",
    "Fassbender",
    "Faulhaber",
    "Fehrig",
    "Feld",
    "Felke",
    "Feller",
    "Fenner",
    "Fenske",
    "Feuerbach",
    "Fietz",
    "Figl",
    "Figura",
    "Filipowski",
    "Filsinger",
    "Fincke",
    "Fink",
    "Finke",
    "Fischer",
    "Fitschen",
    "Fleischer",
    "Fleischmann",
    "Floder",
    "Florczak",
    "Flore",
    "Flottmann",
    "Forkel",
    "Forst",
    "Frahmeke",
    "Frank",
    "Franke",
    "Franta",
    "Frantz",
    "Franz",
    "Franzis",
    "Franzmann",
    "Frauen",
    "Frauendorf",
    "Freigang",
    "Freimann",
    "Freimuth",
    "Freisen",
    "Frenzel",
    "Frey",
    "Fricke",
    "Fried",
    "Friedek",
    "Friedenberg",
    "Friedmann",
    "Friedrich",
    "Friess",
    "Frisch",
    "Frohn",
    "Frosch",
    "Fuchs",
    "Fuhlbrügge",
    "Fusenig",
    "Fust",
    "Förster",
    "Gaba",
    "Gabius",
    "Gabler",
    "Gadschiew",
    "Gakstädter",
    "Galander",
    "Gamlin",
    "Gamper",
    "Gangnus",
    "Ganzmann",
    "Garatva",
    "Gast",
    "Gastel",
    "Gatzka",
    "Gauder",
    "Gebhardt",
    "Geese",
    "Gehre",
    "Gehrig",
    "Gehring",
    "Gehrke",
    "Geiger",
    "Geisler",
    "Geissler",
    "Gelling",
    "Gens",
    "Gerbennow",
    "Gerdel",
    "Gerhardt",
    "Gerschler",
    "Gerson",
    "Gesell",
    "Geyer",
    "Ghirmai",
    "Ghosh",
    "Giehl",
    "Gierisch",
    "Giesa",
    "Giesche",
    "Gilde",
    "Glatting",
    "Goebel",
    "Goedicke",
    "Goldbeck",
    "Goldfuss",
    "Goldkamp",
    "Goldkühle",
    "Goller",
    "Golling",
    "Gollnow",
    "Golomski",
    "Gombert",
    "Gotthardt",
    "Gottschalk",
    "Gotz",
    "Goy",
    "Gradzki",
    "Graf",
    "Grams",
    "Grasse",
    "Gratzky",
    "Grau",
    "Greb",
    "Green",
    "Greger",
    "Greithanner",
    "Greschner",
    "Griem",
    "Griese",
    "Grimm",
    "Gromisch",
    "Gross",
    "Grosser",
    "Grossheim",
    "Grosskopf",
    "Grothaus",
    "Grothkopp",
    "Grotke",
    "Grube",
    "Gruber",
    "Grundmann",
    "Gruning",
    "Gruszecki",
    "Gröss",
    "Grötzinger",
    "Grün",
    "Grüner",
    "Gummelt",
    "Gunkel",
    "Gunther",
    "Gutjahr",
    "Gutowicz",
    "Gutschank",
    "Göbel",
    "Göckeritz",
    "Göhler",
    "Görlich",
    "Görmer",
    "Götz",
    "Götzelmann",
    "Güldemeister",
    "Günther",
    "Günz",
    "Gürbig",
    "Haack",
    "Haaf",
    "Habel",
    "Hache",
    "Hackbusch",
    "Hackelbusch",
    "Hadfield",
    "Hadwich",
    "Haferkamp",
    "Hahn",
    "Hajek",
    "Hallmann",
    "Hamann",
    "Hanenberger",
    "Hannecker",
    "Hanniske",
    "Hansen",
    "Hardy",
    "Hargasser",
    "Harms",
    "Harnapp",
    "Harter",
    "Harting",
    "Hartlieb",
    "Hartmann",
    "Hartwig",
    "Hartz",
    "Haschke",
    "Hasler",
    "Hasse",
    "Hassfeld",
    "Haug",
    "Hauke",
    "Haupt",
    "Haverney",
    "Heberstreit",
    "Hechler",
    "Hecht",
    "Heck",
    "Hedermann",
    "Hehl",
    "Heidelmann",
    "Heidler",
    "Heinemann",
    "Heinig",
    "Heinke",
    "Heinrich",
    "Heinze",
    "Heiser",
    "Heist",
    "Hellmann",
    "Helm",
    "Helmke",
    "Helpling",
    "Hengmith",
    "Henkel",
    "Hennes",
    "Henry",
    "Hense",
    "Hensel",
    "Hentel",
    "Hentschel",
    "Hentschke",
    "Hepperle",
    "Herberger",
    "Herbrand",
    "Hering",
    "Hermann",
    "Hermecke",
    "Herms",
    "Herold",
    "Herrmann",
    "Herschmann",
    "Hertel",
    "Herweg",
    "Herwig",
    "Herzenberg",
    "Hess",
    "Hesse",
    "Hessek",
    "Hessler",
    "Hetzler",
    "Heuck",
    "Heydemüller",
    "Hiebl",
    "Hildebrand",
    "Hildenbrand",
    "Hilgendorf",
    "Hillard",
    "Hiller",
    "Hingsen",
    "Hingst",
    "Hinrichs",
    "Hirsch",
    "Hirschberg",
    "Hirt",
    "Hodea",
    "Hoffman",
    "Hoffmann",
    "Hofmann",
    "Hohenberger",
    "Hohl",
    "Hohn",
    "Hohnheiser",
    "Hold",
    "Holdt",
    "Holinski",
    "Holl",
    "Holtfreter",
    "Holz",
    "Holzdeppe",
    "Holzner",
    "Hommel",
    "Honz",
    "Hooss",
    "Hoppe",
    "Horak",
    "Horn",
    "Horna",
    "Hornung",
    "Hort",
    "Howard",
    "Huber",
    "Huckestein",
    "Hudak",
    "Huebel",
    "Hugo",
    "Huhn",
    "Hujo",
    "Huke",
    "Huls",
    "Humbert",
    "Huneke",
    "Huth",
    "Häber",
    "Häfner",
    "Höcke",
    "Höft",
    "Höhne",
    "Hönig",
    "Hördt",
    "Hübenbecker",
    "Hübl",
    "Hübner",
    "Hügel",
    "Hüttcher",
    "Hütter",
    "Ibe",
    "Ihly",
    "Illing",
    "Isak",
    "Isekenmeier",
    "Itt",
    "Jacob",
    "Jacobs",
    "Jagusch",
    "Jahn",
    "Jahnke",
    "Jakobs",
    "Jakubczyk",
    "Jambor",
    "Jamrozy",
    "Jander",
    "Janich",
    "Janke",
    "Jansen",
    "Jarets",
    "Jaros",
    "Jasinski",
    "Jasper",
    "Jegorov",
    "Jellinghaus",
    "Jeorga",
    "Jerschabek",
    "Jess",
    "John",
    "Jonas",
    "Jossa",
    "Jucken",
    "Jung",
    "Jungbluth",
    "Jungton",
    "Just",
    "Jürgens",
    "Kaczmarek",
    "Kaesmacher",
    "Kahl",
    "Kahlert",
    "Kahles",
    "Kahlmeyer",
    "Kaiser",
    "Kalinowski",
    "Kallabis",
    "Kallensee",
    "Kampf",
    "Kampschulte",
    "Kappe",
    "Kappler",
    "Karhoff",
    "Karrass",
    "Karst",
    "Karsten",
    "Karus",
    "Kass",
    "Kasten",
    "Kastner",
    "Katzinski",
    "Kaufmann",
    "Kaul",
    "Kausemann",
    "Kawohl",
    "Kazmarek",
    "Kedzierski",
    "Keil",
    "Keiner",
    "Keller",
    "Kelm",
    "Kempe",
    "Kemper",
    "Kempter",
    "Kerl",
    "Kern",
    "Kesselring",
    "Kesselschläger",
    "Kette",
    "Kettenis",
    "Keutel",
    "Kick",
    "Kiessling",
    "Kinadeter",
    "Kinzel",
    "Kinzy",
    "Kirch",
    "Kirst",
    "Kisabaka",
    "Klaas",
    "Klabuhn",
    "Klapper",
    "Klauder",
    "Klaus",
    "Kleeberg",
    "Kleiber",
    "Klein",
    "Kleinert",
    "Kleininger",
    "Kleinmann",
    "Kleinsteuber",
    "Kleiss",
    "Klemme",
    "Klimczak",
    "Klinger",
    "Klink",
    "Klopsch",
    "Klose",
    "Kloss",
    "Kluge",
    "Kluwe",
    "Knabe",
    "Kneifel",
    "Knetsch",
    "Knies",
    "Knippel",
    "Knobel",
    "Knoblich",
    "Knoll",
    "Knorr",
    "Knorscheidt",
    "Knut",
    "Kobs",
    "Koch",
    "Kochan",
    "Kock",
    "Koczulla",
    "Koderisch",
    "Koehl",
    "Koehler",
    "Koenig",
    "Koester",
    "Kofferschlager",
    "Koha",
    "Kohle",
    "Kohlmann",
    "Kohnle",
    "Kohrt",
    "Koj",
    "Kolb",
    "Koleiski",
    "Kolokas",
    "Komoll",
    "Konieczny",
    "Konig",
    "Konow",
    "Konya",
    "Koob",
    "Kopf",
    "Kosenkow",
    "Koster",
    "Koszewski",
    "Koubaa",
    "Kovacs",
    "Kowalick",
    "Kowalinski",
    "Kozakiewicz",
    "Krabbe",
    "Kraft",
    "Kral",
    "Kramer",
    "Krauel",
    "Kraus",
    "Krause",
    "Krauspe",
    "Kreb",
    "Krebs",
    "Kreissig",
    "Kresse",
    "Kreutz",
    "Krieger",
    "Krippner",
    "Krodinger",
    "Krohn",
    "Krol",
    "Kron",
    "Krueger",
    "Krug",
    "Kruger",
    "Krull",
    "Kruschinski",
    "Krämer",
    "Kröckert",
    "Kröger",
    "Krüger",
    "Kubera",
    "Kufahl",
    "Kuhlee",
    "Kuhnen",
    "Kulimann",
    "Kulma",
    "Kumbernuss",
    "Kummle",
    "Kunz",
    "Kupfer",
    "Kupprion",
    "Kuprion",
    "Kurnicki",
    "Kurrat",
    "Kurschilgen",
    "Kuschewitz",
    "Kuschmann",
    "Kuske",
    "Kustermann",
    "Kutscherauer",
    "Kutzner",
    "Kwadwo",
    "Kähler",
    "Käther",
    "Köhler",
    "Köhrbrück",
    "Köhre",
    "Kölotzei",
    "König",
    "Köpernick",
    "Köseoglu",
    "Kúhn",
    "Kúhnert",
    "Kühn",
    "Kühnel",
    "Kühnemund",
    "Kühnert",
    "Kühnke",
    "Küsters",
    "Küter",
    "Laack",
    "Lack",
    "Ladewig",
    "Lakomy",
    "Lammert",
    "Lamos",
    "Landmann",
    "Lang",
    "Lange",
    "Langfeld",
    "Langhirt",
    "Lanig",
    "Lauckner",
    "Lauinger",
    "Laurén",
    "Lausecker",
    "Laux",
    "Laws",
    "Lax",
    "Leberer",
    "Lehmann",
    "Lehner",
    "Leibold",
    "Leide",
    "Leimbach",
    "Leipold",
    "Leist",
    "Leiter",
    "Leiteritz",
    "Leitheim",
    "Leiwesmeier",
    "Lenfers",
    "Lenk",
    "Lenz",
    "Lenzen",
    "Leo",
    "Lepthin",
    "Lesch",
    "Leschnik",
    "Letzelter",
    "Lewin",
    "Lewke",
    "Leyckes",
    "Lg",
    "Lichtenfeld",
    "Lichtenhagen",
    "Lichtl",
    "Liebach",
    "Liebe",
    "Liebich",
    "Liebold",
    "Lieder",
    "Lienshöft",
    "Linden",
    "Lindenberg",
    "Lindenmayer",
    "Lindner",
    "Linke",
    "Linnenbaum",
    "Lippe",
    "Lipske",
    "Lipus",
    "Lischka",
    "Lobinger",
    "Logsch",
    "Lohmann",
    "Lohre",
    "Lohse",
    "Lokar",
    "Loogen",
    "Lorenz",
    "Losch",
    "Loska",
    "Lott",
    "Loy",
    "Lubina",
    "Ludolf",
    "Lufft",
    "Lukoschek",
    "Lutje",
    "Lutz",
    "Löser",
    "Löwa",
    "Lübke",
    "Maak",
    "Maczey",
    "Madetzky",
    "Madubuko",
    "Mai",
    "Maier",
    "Maisch",
    "Malek",
    "Malkus",
    "Mallmann",
    "Malucha",
    "Manns",
    "Manz",
    "Marahrens",
    "Marchewski",
    "Margis",
    "Markowski",
    "Marl",
    "Marner",
    "Marquart",
    "Marschek",
    "Martel",
    "Marten",
    "Martin",
    "Marx",
    "Marxen",
    "Mathes",
    "Mathies",
    "Mathiszik",
    "Matschke",
    "Mattern",
    "Matthes",
    "Matula",
    "Mau",
    "Maurer",
    "Mauroff",
    "May",
    "Maybach",
    "Mayer",
    "Mebold",
    "Mehl",
    "Mehlhorn",
    "Mehlorn",
    "Meier",
    "Meisch",
    "Meissner",
    "Meloni",
    "Melzer",
    "Menga",
    "Menne",
    "Mensah",
    "Mensing",
    "Merkel",
    "Merseburg",
    "Mertens",
    "Mesloh",
    "Metzger",
    "Metzner",
    "Mewes",
    "Meyer",
    "Michallek",
    "Michel",
    "Mielke",
    "Mikitenko",
    "Milde",
    "Minah",
    "Mintzlaff",
    "Mockenhaupt",
    "Moede",
    "Moedl",
    "Moeller",
    "Moguenara",
    "Mohr",
    "Mohrhard",
    "Molitor",
    "Moll",
    "Moller",
    "Molzan",
    "Montag",
    "Moormann",
    "Mordhorst",
    "Morgenstern",
    "Morhelfer",
    "Moritz",
    "Moser",
    "Motchebon",
    "Motzenbbäcker",
    "Mrugalla",
    "Muckenthaler",
    "Mues",
    "Muller",
    "Mulrain",
    "Mächtig",
    "Mäder",
    "Möcks",
    "Mögenburg",
    "Möhsner",
    "Möldner",
    "Möllenbeck",
    "Möller",
    "Möllinger",
    "Mörsch",
    "Mühleis",
    "Müller",
    "Münch",
    "Nabein",
    "Nabow",
    "Nagel",
    "Nannen",
    "Nastvogel",
    "Nau",
    "Naubert",
    "Naumann",
    "Ne",
    "Neimke",
    "Nerius",
    "Neubauer",
    "Neubert",
    "Neuendorf",
    "Neumair",
    "Neumann",
    "Neupert",
    "Neurohr",
    "Neuschwander",
    "Newton",
    "Ney",
    "Nicolay",
    "Niedermeier",
    "Nieklauson",
    "Niklaus",
    "Nitzsche",
    "Noack",
    "Nodler",
    "Nolte",
    "Normann",
    "Norris",
    "Northoff",
    "Nowak",
    "Nussbeck",
    "Nwachukwu",
    "Nytra",
    "Nöh",
    "Oberem",
    "Obergföll",
    "Obermaier",
    "Ochs",
    "Oeser",
    "Olbrich",
    "Onnen",
    "Ophey",
    "Oppong",
    "Orth",
    "Orthmann",
    "Oschkenat",
    "Osei",
    "Osenberg",
    "Ostendarp",
    "Ostwald",
    "Otte",
    "Otto",
    "Paesler",
    "Pajonk",
    "Pallentin",
    "Panzig",
    "Paschke",
    "Patzwahl",
    "Paukner",
    "Peselman",
    "Peter",
    "Peters",
    "Petzold",
    "Pfeiffer",
    "Pfennig",
    "Pfersich",
    "Pfingsten",
    "Pflieger",
    "Pflügner",
    "Philipp",
    "Pichlmaier",
    "Piesker",
    "Pietsch",
    "Pingpank",
    "Pinnock",
    "Pippig",
    "Pitschugin",
    "Plank",
    "Plass",
    "Platzer",
    "Plauk",
    "Plautz",
    "Pletsch",
    "Plotzitzka",
    "Poehn",
    "Poeschl",
    "Pogorzelski",
    "Pohl",
    "Pohland",
    "Pohle",
    "Polifka",
    "Polizzi",
    "Pollmächer",
    "Pomp",
    "Ponitzsch",
    "Porsche",
    "Porth",
    "Poschmann",
    "Poser",
    "Pottel",
    "Prah",
    "Prange",
    "Prediger",
    "Pressler",
    "Preuk",
    "Preuss",
    "Prey",
    "Priemer",
    "Proske",
    "Pusch",
    "Pöche",
    "Pöge",
    "Raabe",
    "Rabenstein",
    "Rach",
    "Radtke",
    "Rahn",
    "Ranftl",
    "Rangen",
    "Ranz",
    "Rapp",
    "Rath",
    "Rau",
    "Raubuch",
    "Raukuc",
    "Rautenkranz",
    "Rehwagen",
    "Reiber",
    "Reichardt",
    "Reichel",
    "Reichling",
    "Reif",
    "Reifenrath",
    "Reimann",
    "Reinberg",
    "Reinelt",
    "Reinhardt",
    "Reinke",
    "Reitze",
    "Renk",
    "Rentz",
    "Renz",
    "Reppin",
    "Restle",
    "Restorff",
    "Retzke",
    "Reuber",
    "Reumann",
    "Reus",
    "Reuss",
    "Reusse",
    "Rheder",
    "Rhoden",
    "Richards",
    "Richter",
    "Riedel",
    "Riediger",
    "Rieger",
    "Riekmann",
    "Riepl",
    "Riermeier",
    "Riester",
    "Riethmüller",
    "Rietmüller",
    "Rietscher",
    "Ringel",
    "Ringer",
    "Rink",
    "Ripken",
    "Ritosek",
    "Ritschel",
    "Ritter",
    "Rittweg",
    "Ritz",
    "Roba",
    "Rockmeier",
    "Rodehau",
    "Rodowski",
    "Roecker",
    "Roggatz",
    "Rohländer",
    "Rohrer",
    "Rokossa",
    "Roleder",
    "Roloff",
    "Roos",
    "Rosbach",
    "Roschinsky",
    "Rose",
    "Rosenauer",
    "Rosenbauer",
    "Rosenthal",
    "Rosksch",
    "Rossberg",
    "Rossler",
    "Roth",
    "Rother",
    "Ruch",
    "Ruckdeschel",
    "Rumpf",
    "Rupprecht",
    "Ruth",
    "Ryjikh",
    "Ryzih",
    "Rädler",
    "Räntsch",
    "Rödiger",
    "Röse",
    "Röttger",
    "Rücker",
    "Rüdiger",
    "Rüter",
    "Sachse",
    "Sack",
    "Saflanis",
    "Sagafe",
    "Sagonas",
    "Sahner",
    "Saile",
    "Sailer",
    "Salow",
    "Salzer",
    "Salzmann",
    "Sammert",
    "Sander",
    "Sarvari",
    "Sattelmaier",
    "Sauer",
    "Sauerland",
    "Saumweber",
    "Savoia",
    "Scc",
    "Schacht",
    "Schaefer",
    "Schaffarzik",
    "Schahbasian",
    "Scharf",
    "Schedler",
    "Scheer",
    "Schelk",
    "Schellenbeck",
    "Schembera",
    "Schenk",
    "Scherbarth",
    "Scherer",
    "Schersing",
    "Scherz",
    "Scheurer",
    "Scheuring",
    "Scheytt",
    "Schielke",
    "Schieskow",
    "Schildhauer",
    "Schilling",
    "Schima",
    "Schimmer",
    "Schindzielorz",
    "Schirmer",
    "Schirrmeister",
    "Schlachter",
    "Schlangen",
    "Schlawitz",
    "Schlechtweg",
    "Schley",
    "Schlicht",
    "Schlitzer",
    "Schmalzle",
    "Schmid",
    "Schmidt",
    "Schmidtchen",
    "Schmitt",
    "Schmitz",
    "Schmuhl",
    "Schneider",
    "Schnelting",
    "Schnieder",
    "Schniedermeier",
    "Schnürer",
    "Schoberg",
    "Scholz",
    "Schonberg",
    "Schondelmaier",
    "Schorr",
    "Schott",
    "Schottmann",
    "Schouren",
    "Schrader",
    "Schramm",
    "Schreck",
    "Schreiber",
    "Schreiner",
    "Schreiter",
    "Schroder",
    "Schröder",
    "Schuermann",
    "Schuff",
    "Schuhaj",
    "Schuldt",
    "Schult",
    "Schulte",
    "Schultz",
    "Schultze",
    "Schulz",
    "Schulze",
    "Schumacher",
    "Schumann",
    "Schupp",
    "Schuri",
    "Schuster",
    "Schwab",
    "Schwalm",
    "Schwanbeck",
    "Schwandke",
    "Schwanitz",
    "Schwarthoff",
    "Schwartz",
    "Schwarz",
    "Schwarzer",
    "Schwarzkopf",
    "Schwarzmeier",
    "Schwatlo",
    "Schweisfurth",
    "Schwennen",
    "Schwerdtner",
    "Schwidde",
    "Schwirkschlies",
    "Schwuchow",
    "Schäfer",
    "Schäffel",
    "Schäffer",
    "Schäning",
    "Schöckel",
    "Schönball",
    "Schönbeck",
    "Schönberg",
    "Schönebeck",
    "Schönenberger",
    "Schönfeld",
    "Schönherr",
    "Schönlebe",
    "Schötz",
    "Schüler",
    "Schüppel",
    "Schütz",
    "Schütze",
    "Seeger",
    "Seelig",
    "Sehls",
    "Seibold",
    "Seidel",
    "Seiders",
    "Seigel",
    "Seiler",
    "Seitz",
    "Semisch",
    "Senkel",
    "Sewald",
    "Siebel",
    "Siebert",
    "Siegling",
    "Sielemann",
    "Siemon",
    "Siener",
    "Sievers",
    "Siewert",
    "Sihler",
    "Sillah",
    "Simon",
    "Sinnhuber",
    "Sischka",
    "Skibicki",
    "Sladek",
    "Slotta",
    "Smieja",
    "Soboll",
    "Sokolowski",
    "Soller",
    "Sollner",
    "Sommer",
    "Somssich",
    "Sonn",
    "Sonnabend",
    "Spahn",
    "Spank",
    "Spelmeyer",
    "Spiegelburg",
    "Spielvogel",
    "Spinner",
    "Spitzmüller",
    "Splinter",
    "Sporrer",
    "Sprenger",
    "Spöttel",
    "Stahl",
    "Stang",
    "Stanger",
    "Stauss",
    "Steding",
    "Steffen",
    "Steffny",
    "Steidl",
    "Steigauf",
    "Stein",
    "Steinecke",
    "Steinert",
    "Steinkamp",
    "Steinmetz",
    "Stelkens",
    "Stengel",
    "Stengl",
    "Stenzel",
    "Stepanov",
    "Stephan",
    "Stern",
    "Steuk",
    "Stief",
    "Stifel",
    "Stoll",
    "Stolle",
    "Stolz",
    "Storl",
    "Storp",
    "Stoutjesdijk",
    "Stratmann",
    "Straub",
    "Strausa",
    "Streck",
    "Streese",
    "Strege",
    "Streit",
    "Streller",
    "Strieder",
    "Striezel",
    "Strogies",
    "Strohschank",
    "Strunz",
    "Strutz",
    "Stube",
    "Stöckert",
    "Stöppler",
    "Stöwer",
    "Stürmer",
    "Suffa",
    "Sujew",
    "Sussmann",
    "Suthe",
    "Sutschet",
    "Swillims",
    "Szendrei",
    "Sören",
    "Sürth",
    "Tafelmeier",
    "Tang",
    "Tasche",
    "Taufratshofer",
    "Tegethof",
    "Teichmann",
    "Tepper",
    "Terheiden",
    "Terlecki",
    "Teufel",
    "Theele",
    "Thieke",
    "Thimm",
    "Thiomas",
    "Thomas",
    "Thriene",
    "Thränhardt",
    "Thust",
    "Thyssen",
    "Thöne",
    "Tidow",
    "Tiedtke",
    "Tietze",
    "Tilgner",
    "Tillack",
    "Timmermann",
    "Tischler",
    "Tischmann",
    "Tittman",
    "Tivontschik",
    "Tonat",
    "Tonn",
    "Trampeli",
    "Trauth",
    "Trautmann",
    "Travan",
    "Treff",
    "Tremmel",
    "Tress",
    "Tsamonikian",
    "Tschiers",
    "Tschirch",
    "Tuch",
    "Tucholke",
    "Tudow",
    "Tuschmo",
    "Tächl",
    "Többen",
    "Töpfer",
    "Uhlemann",
    "Uhlig",
    "Uhrig",
    "Uibel",
    "Uliczka",
    "Ullmann",
    "Ullrich",
    "Umbach",
    "Umlauft",
    "Umminger",
    "Unger",
    "Unterpaintner",
    "Urban",
    "Urbaniak",
    "Urbansky",
    "Urhig",
    "Vahlensieck",
    "Van",
    "Vangermain",
    "Vater",
    "Venghaus",
    "Verniest",
    "Verzi",
    "Vey",
    "Viellehner",
    "Vieweg",
    "Voelkel",
    "Vogel",
    "Vogelgsang",
    "Vogt",
    "Voigt",
    "Vokuhl",
    "Volk",
    "Volker",
    "Volkmann",
    "Von",
    "Vona",
    "Vontein",
    "Wachenbrunner",
    "Wachtel",
    "Wagner",
    "Waibel",
    "Wakan",
    "Waldmann",
    "Wallner",
    "Wallstab",
    "Walter",
    "Walther",
    "Walton",
    "Walz",
    "Wanner",
    "Wartenberg",
    "Waschbüsch",
    "Wassilew",
    "Wassiluk",
    "Weber",
    "Wehrsen",
    "Weidlich",
    "Weidner",
    "Weigel",
    "Weight",
    "Weiler",
    "Weimer",
    "Weis",
    "Weiss",
    "Weller",
    "Welsch",
    "Welz",
    "Welzel",
    "Weniger",
    "Wenk",
    "Werle",
    "Werner",
    "Werrmann",
    "Wessel",
    "Wessinghage",
    "Weyel",
    "Wezel",
    "Wichmann",
    "Wickert",
    "Wiebe",
    "Wiechmann",
    "Wiegelmann",
    "Wierig",
    "Wiese",
    "Wieser",
    "Wilhelm",
    "Wilky",
    "Will",
    "Willwacher",
    "Wilts",
    "Wimmer",
    "Winkelmann",
    "Winkler",
    "Winter",
    "Wischek",
    "Wischer",
    "Wissing",
    "Wittich",
    "Wittl",
    "Wolf",
    "Wolfarth",
    "Wolff",
    "Wollenberg",
    "Wollmann",
    "Woytkowska",
    "Wujak",
    "Wurm",
    "Wyludda",
    "Wölpert",
    "Wöschler",
    "Wühn",
    "Wünsche",
    "Zach",
    "Zaczkiewicz",
    "Zahn",
    "Zaituc",
    "Zandt",
    "Zanner",
    "Zapletal",
    "Zauber",
    "Zeidler",
    "Zekl",
    "Zender",
    "Zeuch",
    "Zeyen",
    "Zeyhle",
    "Ziegler",
    "Zimanyi",
    "Zimmer",
    "Zimmermann",
    "Zinser",
    "Zintl",
    "Zipp",
    "Zipse",
    "Zschunke",
    "Zuber",
    "Zwiener",
    "Zümsande",
    "Östringer",
    "Überacker",
});

const auto austrianMalePrefixes = std::to_array<std::string_view>({"Herr", "Dr.", "Prof. Dr."});

const auto austrianFemalePrefixes = std::to_array<std::string_view>({"Frau", "Dr.", "Prof. Dr."});

const NameFormats austrianNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 5}}};

const PeopleNames austrianPeopleNames{{austrianMaleFirstNames, austrianLastNames, austrianMalePrefixes, {}},
                                      {austrianFemaleFirstNames, austrianLastNames, austrianFemalePrefixes, {}},
                                      austrianNameFormats};

// Azerbaijan

const auto azerbaijaniMaleFirstNames = std::to_array<std::string_view>({
    "Anar",   "Amid",    "Afəl",     "Abbas", "Abdulla",  "Adil",   "Akif",   "Aqil",  "Bəhram",
    "Nurlan", "Rafiq",   "Tərlan",   "Zaur",  "Emin",     "Emil",   "Kamran", "Elnur", "Natiq",
    "Rəşad",  "Rəşid",   "Tahir",    "Əhməd", "Zahir",    "İlham",  "İlqar",  "Nahid", "Nihad",
    "Faiq",   "İxtiyar", "Şəhriyar", "Şaiq",  "Bəxtiyar", "Bəhruz", "Tunar",  "Nadir",
});

const auto azerbaijaniFemaleFirstNames = std::to_array<std::string_view>({
    "Anna",    "Adeliya", "Afaq",   "Afət",     "Afərim", "Aidə",    "Aygün",    "Aynur",    "Alsu",   "Ayan",
    "Aytən",   "Aygül",   "Aydan",  "Aylin",    "Bahar",  "Banu",    "Bəyaz",    "Billurə",  "Cansu",  "Ceyla",
    "Damla",   "Dəniz",   "Diana",  "Dilarə",   "Ella",   "Elza",    "Elyanora", "Ellada",   "Elvira", "Elnarə",
    "Esmira",  "Estella", "Fatimə", "Fəxriyyə", "Fərəh",  "Fərqanə", "Fidan",    "Firuzə",   "Gövhər", "Günay",
    "Gülay",   "Gülçin",  "Gülər",  "Gülsüm",   "Humay",  "Hüriyə",  "Hülya",    "Jalə",     "Jasmin", "Kübra",
    "Ləman",   "Lamiyə",  "Lalə",   "Liliya",   "Laura",  "Leyla",   "Maya",     "Mehriban", "Mələk",  "Nuray",
    "Nurgün",  "Nərgiz",  "Nigar",  "Ofelya",   "Pəri",   "Röya",    "Səbinə",   "Selcan",   "Tansu",  "Tuba",
    "Ülviyyə", "Ulduz",   "Ülkər",
});

const auto azerbaijaniMaleLastNames = std::to_array<std::string_view>({
    "Əhmədov",
    "Ələkbərov",
    "Əliyev",
    "Vəliyev",
    "Soltanov",
    "Quliyev",
    "Məmmədov",
    "Xəlilov",
    "Nəzərov",
    "Rəhimov",
});

const auto azerbaijaniFemaleLastNames = std::to_array<std::string_view>({
    "Qasımova",
    "Əfəndiyeva",
    "Soltanova",
    "Abdullayeva",
    "Rəşidova",
    "Ələkbərova",
    "Əliyeva",
    "Tahirova",
    "Seyidova",
    "Vəsiyeva",
});

const NameFormats azerbaijaniNameFormats{{
    {"{lastName} {firstName}", 1},
    {"{firstName} {lastName}", 1},
}};

const PeopleNames azerbaijaniPeopleNames{{azerbaijaniMaleFirstNames, azerbaijaniMaleLastNames, {}, {}},
                                         {azerbaijaniFemaleFirstNames, azerbaijaniFemaleLastNames, {}, {}},
                                         azerbaijaniNameFormats};

// Belarus

const auto belarusianMaleFirstNames = std::to_array<std::string_view>({
    "Sergey",    "Andrey",     "Aleksandr",  "Dmitriy",    "Vladimir",   "Alexander",  "Pavel",      "Aleksey",
    "Evgeniy",   "Dmitry",     "Igor",       "Nikolay",    "Maksim",     "Ivan",       "Artyom",     "Mikhail",
    "Alexey",    "Ilya",       "Oleg",       "Viktor",     "Roman",      "Anton",      "Vadim",      "Kirill",
    "Yuriy",     "Maxim",      "Evgeny",     "Vitaliy",    "Egor",       "Vladislav",  "Konstantin", "Artem",
    "Vitaly",    "Stanislav",  "Ruslan",     "Artur",      "Valeriy",    "Yury",       "Vyacheslav", "Gennadiy",
    "Anatoliy",  "Petr",       "Leonid",     "Daniil",     "Vasily",     "Anatoly",    "Eduard",     "Valentin",
    "Gleb",      "Danil",      "Yaroslav",   "Boris",      "Timofey",    "Matvey",     "Stepan",     "Bogdan",
    "Arseniy",   "Mark",       "Georgiy",    "Timur",      "Svyatoslav", "German",     "Andrei",     "Grigory",
    "Fedor",     "Sergei",     "Kiril",      "Nazar",      "David",      "Ignat",      "Alex",       "Marat",
    "Georgy",    "Rodion",     "Arkadiy",    "Rustam",     "Aleks",      "Alexandr",   "Rostislav",  "Albert",
    "Sergej",    "Semen",      "Taras",      "Lev",        "Philip",     "Miroslav",   "Robert",     "Vsevolod",
    "Ales",      "Renat",      "Filipp",     "Edgar",      "Vital",      "Erik",       "Alik",       "Andrej",
    "Alexandar", "Aleksander", "Aleksandar", "Adam",       "Victor",     "Edvard",     "Ali",        "Daniel",
    "Arsen",     "Dinis",      "Martin",     "Aleksei",    "Iosif",      "Emil",       "Mihail",     "Nikolai",
    "Mikola",    "Eldar",      "Leon",       "Arnold",     "Ernest",     "Vasil",      "Murat",      "Vitali",
    "Nick",      "Alexei",     "Vladimer",   "Eugene",     "Aleksej",    "Richard",    "Kris",       "Miron",
    "Zhan",      "Dmitri",     "Stefan",     "Aliaksandr", "Samir",      "Serge",      "Dmitrij",    "Rinat",
    "Angel",     "Sasa",       "Dzmitry",    "Dominik",    "Andre",      "Marek",      "Petro",      "John",
    "Serafim",   "Milan",      "Paul",       "Viktar",     "Feliks",     "Damir",      "Arslan",     "Felix",
    "Tomas",     "Tata",       "Raman",      "Armen",      "Rafael",     "Valdemar",   "Ramil",      "Akim",
    "Adrian",    "Alan",       "Ashot",      "Andrian",    "Sergio",     "Karim",      "Tom",        "Arman",
    "Vitalii",   "Jack",       "Tigran",     "Aziz",       "Peter",      "Oskar",      "Dimitry",    "Uladzimir",
    "Nikolas",   "Yanik",      "Sem",        "Murad",      "Illya",      "Filip",      "Suleyman",   "Siarhei",
    "Evgeni",    "Artiom",     "Sam",        "Andry",      "Vasili",     "Dimitriy",   "Adel",       "Jan",
    "Aslan",     "Aliaksei",   "Patrik",     "Romeo",      "Elvin",      "Anatoli",    "Kristian",   "Zaur",
    "Ismail",    "Aram",       "Romuald",    "Arthur",     "Amir",       "Sancho",     "Andriy",     "Shamil",
    "George",    "Farid",      "Antony",     "Karl",       "Marsel",     "Ostap",      "Ilyas",      "Mustafa",
    "Sergiy",    "Alfred",     "Ahmed",      "Bob",        "Said",       "Simon",      "Magomed",    "Deni",
    "Iliya",     "Yurii",      "Alen",       "Nil",        "Jon",        "Rafik",      "Emin",       "Sashko",
    "Ruben",     "Kamil",      "Gevorg",     "Ihar",       "Aleh",       "Slavomir",   "Leonard",    "Pol",
    "Ben",       "Ahmet",      "Tony",       "Di",         "Rudolf",     "Abram",      "Omar",       "Dionis",
    "Markus",    "Chris",      "Edward",     "Kevin",      "Gor",        "Michail",    "Luka",       "Samuil",
    "Gabriel",   "Bladimir",   "Nikolaj",    "Ramin",      "Natan",      "Edvin",      "Adolf",      "Rasim",
    "Sascha",    "Antoni",     "Raul",       "Zurab",      "Georg",      "James",      "Frank",      "Andreu",
    "Eugen",     "Ramiz",      "Niko",       "Donat",      "Papa",       "Amin",       "Sultan",     "Yegor",
    "Mir",       "Oleksandr",  "Fred",       "Ahmad",      "Mansur",     "Alessandro", "Pan",        "Pavlo",
    "Samvel",    "Just",       "Anatolii",   "Eric",       "Makha",      "Sergy",      "Olexandr",   "Ilja",
    "Kemal",     "Mehmet",     "Emir",       "Rauf",       "Nail",       "Pablo",      "William",    "Musa",
    "Ibrahim",   "Mohammad",   "Ladislav",   "Wiktor",     "Rashid",     "Dani",       "Da",         "Levon",
    "Arif",      "Ronald",     "Salim",      "Aron",       "Mohamad",    "Vakhtang",   "Suren",      "Vazgen",
    "Anis",      "Luis",       "Kosta",      "Valerii",    "Patrick",    "Elvis",      "Rashad",     "Dzhamal",
    "Dzianis",   "Mikhaylo",   "Vardan",     "Narek",      "Pedro",      "Migel",      "Irakli",     "Alexy",
    "Mohammed",  "Edgard",     "Levan",      "Pawel",      "Parviz",     "Abdul",      "Andranik",   "Asel",
    "Svetoslav", "Dante",      "Yunus",      "Gospodin",   "Arseni",     "Wladimir",
});

const auto belarusianFemaleFirstNames = std::to_array<std::string_view>({
    "Olga",      "Tatyana",      "Ekaterina",  "Natalya",     "Elena",     "Anastasiya", "Irina",     "Svetlana",
    "Anna",      "Marina",       "Darya",      "Denis",       "Viktoriya", "Mariya",     "Yuliya",    "Kristina",
    "Anastasia", "Nikita",       "Alina",      "Veronika",    "Lyudmila",  "Yulia",      "Nadezhda",  "Viktoria",
    "Anya",      "Alyona",       "Aleksandra", "Elizaveta",   "Yana",      "Diana",      "Valentina", "Galina",
    "Kseniya",   "Inna",         "Valeriya",   "Polina",      "Karina",    "Maria",      "Oksana",    "Oxana",
    "Margarita", "Alexandra",    "Alla",       "Evgeniya",    "Angelina",  "Sasha",      "Alena",     "Lyubov",
    "Zhanna",    "Larisa",       "Olesya",     "Nina",        "Tamara",    "Vera",       "Zhenya",    "Ksenia",
    "Valeria",   "Evgenia",      "Natalia",    "Anzhelika",   "Nataliya",  "Nika",       "Arina",     "Ulyana",
    "Valery",    "Snezhana",     "Violetta",   "Danila",      "Anzhela",   "Ilona",      "Yan",       "Alisa",
    "Antonina",  "Milana",       "Inga",       "Vladislava",  "Marta",     "Lilia",      "Karolina",  "Sofiya",
    "Liliya",    "Sanya",        "Dina",       "Kira",        "Zoya",      "Lidiya",     "Inessa",    "Raisa",
    "Yanina",    "Lina",         "Varvara",    "Zinaida",     "Elvira",    "Sofya",      "Mayya",     "Yuliana",
    "Darina",    "Mila",         "Eva",        "Lidia",       "Dana",      "Vasilisa",   "Albina",    "Milena",
    "Lilya",     "Asya",         "Lika",       "Sofia",       "Nelli",     "Regina",     "Lana",      "Lolita",
    "Vitalina",  "Sashka",       "Zlata",      "Evelina",     "Sabina",    "Julia",      "Elina",     "Marianna",
    "Renata",    "Liana",        "Yulianna",   "Rimma",       "Inka",      "Vasilina",   "Ruslana",   "Kate",
    "Anita",     "Tina",         "Tatiana",    "Emma",        "Zarina",    "Daria",      "Miroslava", "Lada",
    "Lora",      "Alevtina",     "Ilonka",     "Irena",       "Eleonora",  "Taisa",      "Nadka",     "Sandra",
    "Yaroslava", "Roza",         "Maryana",    "Victoria",    "Taisia",    "Erika",      "Veta",      "Agata",
    "Dominika",  "Rina",         "Ann",        "Katrina",     "Faina",     "Viola",      "Mira",      "Anfisa",
    "Liya",      "Natallia",     "Vitaliya",   "Dinara",      "Kamilla",   "Vitalia",    "Mari",      "Tatsiana",
    "Verka",     "Leyla",        "Elmira",     "Danuta",      "Emiliya",   "Tereza",     "Lia",       "Emilia",
    "Stella",    "Helen",        "Sabrina",    "Dayana",      "Amina",     "Maya",       "Dariya",    "Tania",
    "Nonna",     "Dinka",        "Violeta",    "Stanislava",  "Lara",      "Dan",        "Luiza",     "Kamila",
    "Klavdiya",  "Nastia",       "Mary",       "Ludmila",     "Alesia",    "Katarina",   "Ninel",     "Nadin",
    "Malvina",   "Yulian",       "Katia",      "Adelina",     "Elka",      "Rada",       "Ariana",    "Gulnara",
    "Syuzanna",  "Aksana",       "Karen",      "Teresa",      "Liliana",   "Ivanna",     "Ada",       "Bella",
    "Elza",      "Alexa",        "Meri",       "Linda",       "Anuta",     "Iya",        "Tonka",     "Laura",
    "Kim",       "Nadia",        "Marika",     "Lisa",        "Christina", "Maryan",     "Izabella",  "Yuri",
    "Nikola",    "Ariadna",      "Daniela",    "Hanna",       "Glafira",   "Monika",     "Ina",       "Lili",
    "Valeri",    "Ilia",         "Vanda",      "Snezhanka",   "Ana",       "Iryna",      "Sara",      "Alisha",
    "Fatima",    "Elen",         "Danka",      "Ivona",       "Nataly",    "Anet",       "Sana",      "Anika",
    "Adriana",   "Madina",       "Nona",       "Edita",       "Helena",    "Aida",       "Mishel",    "Zena",
    "Bogdana",   "Aleksandrina", "Elizabet",   "Samira",      "Pavlina",   "Ganna",      "Irma",      "Slavka",
    "Elizabeth", "Eliza",        "Darinka",    "Ania",        "Stefaniya", "Nelya",      "Beata",     "Ani",
    "Jane",      "Venera",       "Veronica",   "Sania",       "Angelika",  "Katy",       "Sima",      "Mariyka",
    "Tanja",     "Stefania",     "Sava",       "Ioanna",      "Alice",     "Pavla",      "Nella",     "Vania",
    "Yanis",     "Li",           "Tatjana",    "Annet",       "Mama",      "Gala",       "Klara",     "Sacha",
    "Vanessa",   "Kati",         "Natalie",    "Mariana",     "Nana",      "Nadzeya",    "Ya",        "Islam",
    "Radmila",   "Ayna",         "Nila",       "Tamila",      "Roksana",   "Katja",      "Petra",     "Dennis",
    "Mika",      "Beronika",     "Arianna",    "Angela",      "Helga",     "Verona",     "Zara",      "Viktoryia",
    "Elis",      "Victoriya",    "Jana",       "Alexandrina", "Mariam",    "Liudmila",   "Tinka",     "Juli",
    "Lyubka",    "Dunya",        "Emili",      "Amira",       "Dara",      "Roxana",     "Tana",      "Anastacia",
    "Barbara",   "Maryna",       "Lady",       "Aliona",      "Halina",    "Nelly",
});

const auto belarusianMaleLastNames = std::to_array<std::string_view>({
    "Ivanov",       "Novik",         "Zhuk",         "Moroz",       "Petrov",      "Novikov",      "Kravchenko",
    "Kovalenko",    "Makarevich",    "Kovalchuk",    "Karpovich",   "Pashkevich",  "Kozlov",       "Volkov",
    "Shevchenko",   "Kotov",         "Klimovich",    "Morozov",     "Smirnov",     "Bondarenko",   "Stankevich",
    "Tarasevich",   "Marchenko",     "Zaytsev",      "Lukashevich", "Kot",         "Savchenko",    "Bogdanovich",
    "Pinchuk",      "Kovalyov",      "Boyko",        "Kuzmich",     "Melnik",      "Korol",        "Matskevich",
    "Sinkevich",    "Bondar",        "Pavlovich",    "Petrovich",   "Melnikov",    "Kazak",        "Markevich",
    "Romanov",      "Borisevich",    "Sidorenko",    "Yushkevich",  "Kovalevich",  "Kozlovsky",    "Zayats",
    "Kuznetsov",    "Abramovich",    "Romanenko",    "Dashkevich",  "Yurchenko",   "Sokolov",      "Borisenko",
    "Kovalev",      "Rusak",         "Makarenko",    "Adamovich",   "Poznyak",     "Drozd",        "Kozel",
    "Ignatovich",   "Khomich",       "Vorobey",      "Kravchuk",    "Romanyuk",    "Baranov",      "Shevchuk",
    "Dubovik",      "Vasilevsky",    "Akulich",      "Kisel",       "Frolov",      "Romanovich",   "Savchuk",
    "Sakovich",     "Goncharov",     "Popov",        "Mironov",     "Orlov",       "Kulik",        "Murashko",
    "Golub",        "Pavlov",        "Gorbach",      "Gavrilenko",  "Andreev",     "Vashkevich",   "Timoshenko",
    "Vasilenko",    "Solovey",       "Makarov",      "Vasilevich",  "Sidorovich",  "Prokopovich",  "Kravtsov",
    "Lis",          "Zhdanovich",    "Kholod",       "Fedorovich",  "Matusevich",  "Gerasimovich", "Nikitin",
    "Zhukov",       "Yaroshevich",   "Mikhaylov",    "Yurkevich",   "Zakharenko",  "Bely",         "Burak",
    "Demidovich",   "Volk",          "Shinkevich",   "Zhukovsky",   "Gurinovich",  "Belov",        "Naumenko",
    "Shpak",        "Sergeev",       "Koval",        "Golovach",    "Mitskevich",  "Kovalevsky",   "Nesterovich",
    "Prikhodko",    "Medvedev",      "Meleshko",     "Kasperovich", "Stepanov",    "Korotkevich",  "Alexandrovich",
    "Lebedev",      "Baranovsky",    "Titov",        "Radkevich",   "Prokopenko",  "Ignatenko",    "Kazakevich",
    "Savich",       "Yakimovich",    "Kachan",       "Sergeevich",  "Ermakovich",  "Nikitenko",    "Babich",
    "Verenich",     "Davidovich",    "Yaskevich",    "Komar",       "Nikolaev",    "Grinevich",    "Aleksandrovich",
    "Sokolovsky",   "Potapenko",     "Savitsky",     "Kononovich",  "Kuntsevich",  "Marchuk",      "Egorov",
    "Urbanovich",   "Belous",        "Gayduk",       "Semashko",    "Kruk",        "Chernyavsky",  "Savko",
    "Drozdov",      "Nikolaevich",   "Kulesh",       "Korzun",      "Antonovich",  "Sushko",       "Leshkevich",
    "Kolesnik",     "Tarasenko",     "Karpenko",     "Ermolenko",   "Sokol",       "Antonov",      "Radyuk",
    "Loyko",        "Romanovsky",    "Vasilyev",     "Vasilev",     "Grib",        "Belyy",        "Gonchar",
    "Leonovich",    "Kondratenko",   "Bondarev",     "Sobol",       "Nesterenko",  "Semyonov",     "Kostyukevich",
    "Malinovsky",   "Vladimirovich", "Bogdanov",     "Denisenko",   "Astapenko",   "Petkevich",    "Gromyko",
    "Viktorovich",  "Petrovsky",     "Yakovlev",     "Kostyuk",     "Polyakov",    "Pavlovsky",    "Shevtsov",
    "Stasevich",    "Lyakh",         "Sadovsky",     "Butko",       "Belko",       "Golubev",      "Martinovich",
    "Kondratovich", "Karpov",        "Yanushkevich", "German",      "Mikulich",    "Romanchuk",    "Zakharov",
    "Senko",        "Tarasyuk",      "Borovik",      "Velichko",    "Reut",        "Gulevich",     "Yankovskaya",
    "Chernyak",     "Gritsuk",       "Shilo",        "Filipovich",  "Osipenko",    "Senkevich",    "Antonenko",
    "Korsak",       "Tereshko",      "Levchuk",      "Borisov",     "Panasyuk",    "Gritskevich",  "Sidorov",
    "Voytovich",    "Fedorenko",     "Dudko",        "Ivashkevich", "Kolesnikova", "Nazarenko",    "Moiseenko",
    "Tarasov",      "Rudko",         "Solovyov",     "Lavrinovich", "Savitskiy",   "Parkhomenko",  "Mikhalevich",
    "Petrenko",     "Korolyov",
});

const auto belarusianFemaleLastNames = std::to_array<std::string_view>({
    "Ivanova",    "Novikova",     "Kotova",       "Volkova",      "Petrova",     "Morozova",      "Smirnova",
    "Kozlova",    "Romanova",     "Mironova",     "Zaytseva",     "Kovalyova",   "Savitskaya",    "Melnikova",
    "Kuznetsova", "Andreeva",     "Kovaleva",     "Vasilevskaya", "Sokolova",    "Sergeeva",      "Goncharova",
    "Orlova",     "Pavlova",      "Mikhaylova",   "Kozlovskaya",  "Frolova",     "Kravtsova",     "Nikitina",
    "Lebedeva",   "Malinovskaya", "Sokolovskaya", "Kovalevskaya", "Belaya",      "Vysotskaya",    "Novitskaya",
    "Vasileva",   "Titova",       "Vishnevskaya", "Makarova",     "Baranova",    "Medvedeva",     "Zhukovskaya",
    "Shulga",     "Zhukova",      "Filatova",     "Belova",       "Bogdanova",   "Popova",        "Nikolaeva",
    "Stepanova",  "Baranovskaya", "Romanovskaya", "Polyakova",    "Soroka",      "Chernyavskaya", "Yakovleva",
    "Golubeva",   "Aleksandrova", "Sadovskaya",   "Matveeva",     "Ermakova",    "Starovoytova",  "Pavlovskaya",
    "Drozdova",   "Semyonova",    "Antonova",     "Lyubimova",    "Petrovskaya", "Sergeevna",     "Vasilyeva",
    "Tkachenko",  "Solovyova",    "Shevtsova",    "Fyodorova",    "Starovoytov", "Korolyova",     "Alexandrova",
    "Zakharova",  "Tarasova",     "Koleda",       "Egorova",      "Rudenko",     "Belskaya",      "Borisova",
    "Bondareva",  "Emelyanova",   "Kazakova",     "Markova",      "Litvinova",
});

const NameFormats belarusianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames belarusianPeopleNames{{belarusianMaleFirstNames, belarusianMaleLastNames, {}, {}},
                                        {belarusianFemaleFirstNames, belarusianFemaleLastNames, {}, {}},
                                        belarusianNameFormats};

// Belgium

const auto belgianMaleFirstNames = std::to_array<std::string_view>({
    "Lucas",     "Liam",    "Louis",   "Noah",   "Lars",     "Arthur",  "Mathis", "Kobe",    "Wout",   "Milan",
    "Alexander", "Daan",    "Adam",    "Vince",  "Jules",    "Victor",  "Finn",   "Stan",    "Leon",   "Seppe",
    "Senne",     "Thomas",  "Mats",    "Robbe",  "Matteo",   "Ferre",   "Nathan", "Warre",   "Jasper", "Vic",
    "Elias",     "Tuur",    "Lowie",   "Mauro",  "Maxim",    "Ruben",   "Rune",   "Mohamed", "Simon",  "Rayan",
    "Lukas",     "Lander",  "Xander",  "Emiel",  "Siebe",    "Jonas",   "Sam",    "Luca",    "Arne",   "Cas",
    "Felix",     "Jack",    "Mathias", "Jayden", "Quinten",  "Tibo",    "Lewis",  "Sem",     "Maxime", "Viktor",
    "Emile",     "Sander",  "Aaron",   "Oscar",  "Bas",      "Niels",   "Jef",    "Nand",    "Lou",    "Tristan",
    "Gust",      "David",   "Brent",   "Jelle",  "Lenn",     "Miel",    "Wannes", "Gilles",  "Jarne",  "Alex",
    "Imran",     "Lennert", "Milo",    "Jens",   "Matthias", "Dries",   "Axel",   "Juul",    "Hamza",  "Ilias",
    "Thibo",     "Nicolas", "Bent",    "Jesse",  "Bram",     "Maurice", "Ilyas",  "Gabriel", "Robin",  "Kasper",
});

const auto belgianFemaleFirstNames = std::to_array<std::string_view>({
    "Emma",    "Louise", "Marie",   "Elise",    "Noor",     "Ella",    "Julie",   "Lotte", "Lore",      "Fien",
    "Lena",    "Mila",   "Olivia",  "Amber",    "Anna",     "Elena",   "Hanne",   "Laura", "Charlotte", "Lina",
    "Nina",    "Fleur",  "Lisa",    "Nora",     "Juliette", "Luna",    "Amelie",  "Kato",  "Sara",      "Febe",
    "Axelle",  "Janne",  "Eline",   "Helena",   "Noa",      "Camille", "Roos",    "Lily",  "Jade",      "Nore",
    "Jana",    "Manon",  "Sarah",   "Liv",      "Amélie",   "Lara",    "Mona",    "Alice", "Tess",      "Kaat",
    "Femke",   "Aya",    "Eva",     "Lien",     "Bo",       "Zoe",     "Oona",    "Sofia", "Leonie",    "Linde",
    "Jolien",  "Fran",   "Pauline", "Laure",    "Merel",    "Marthe",  "Yana",    "Renée", "Paulien",   "Margot",
    "Yasmine", "Ines",   "Elisa",   "Emily",    "Maya",     "Floor",   "Amy",     "Lize",  "Amina",     "Lucie",
    "Julia",   "Lise",   "Hailey",  "Hannah",   "Alicia",   "Norah",   "Margaux", "Lieze", "Sien",      "Zoë",
    "Inaya",   "Aline",  "Amira",   "Victoria", "Flore",    "Lana",    "Sterre",  "Maud",  "Chloe",
});

const auto belgianLastNames = std::to_array<std::string_view>({
    "Claes",    "Claeys",       "Declerck",     "Declercq",   "Decock",    "Decoster",   "Desmet",   "Devos",
    "Dewilde",  "Gielen",       "Goossens",     "Hermans",    "Jacobs",    "Janssen",    "Janssens", "Lemmens",
    "Maes",     "Martens",      "Mertens",      "Michiels",   "Peeters",   "Smet",       "Smets",    "Thijs",
    "Vandamme", "Vandenberghe", "Vandenbroeck", "Vandevelde", "Verhaeghe", "Verstraete", "Willems",  "Wouters",
});

const auto belgianSuffixes = std::to_array<std::string_view>({"MBA", "Phd."});

const auto belgianPrefixes = std::to_array<std::string_view>({"Dr.", "Ir.", "Ing.", "Prof."});

const NameFormats belgianNameFormats{{
    {"{firstName} {lastName}", 8},
    {"{prefix} {firstName} {lastName}", 1},
    {"{firstName} {lastName} {suffix}", 1},
}};

const PeopleNames belgianPeopleNames{{belgianMaleFirstNames, belgianLastNames, belgianPrefixes, belgianSuffixes},
                                     {belgianFemaleFirstNames, belgianLastNames, belgianPrefixes, belgianSuffixes},
                                     belgianNameFormats};

// Bosnia

const auto bosnianMaleFirstNames = std::to_array<std::string_view>(
    {"Adin",    "Alem",   "Amar",    "Benjamin", "Dino",     "Edin",   "Elvir",  "Emir",   "Faruk", "Haris",
     "Ibrahim", "Jasmin", "Kenan",   "Lejla",    "Mehmed",   "Nedim",  "Osman",  "Safet",  "Tarik", "Vedad",
     "Zlatan",  "Emin",   "Nikola",  "Eldar",    "Senad",    "D?enan", "Aldin",  "Emir",   "Davor", "Adnan",
     "Emir",    "Amar",   "Alen",    "Nihad",    "Damir",    "Amir",   "Amer",   "Armin",  "Mirza", "Muhamed",
     "Eldin",   "Denis",  "Elmedin", "Selmir",   "Miralem",  "Miran",  "Anel",   "Semir",  "Lejla", "Azra",
     "Sara",    "Amna",   "Hana",    "Emina",    "Nermina",  "Lejla",  "Belma",  "Adna",   "Amina", "D?enita",
     "Ajla",    "Aldina", "Erna",    "Merjem",   "Lamija",   "Aida",   "Sabina", "Edna",   "Dina",  "Maja",
     "Vesna",   "Amela",  "Mirela",  "Ela",      "Dajana",   "Irma",   "Ena",    "Melisa", "Nina",  "Lana",
     "Una",     "Tara",   "Ela",     "Ariana",   "Adrijana", "Emina",  "Maida",  "Enisa",  "Lina",  "Amina",
     "Adna",    "Lejla",  "Ajla",    "Zerina",   "Mirnesa",  "Ayla",   "Adna",   "Dijana", "Hana",  "Amila"});

const auto bosnianFemaleFirstNames = std::to_array<std::string_view>(
    {"Adin",    "Alem",   "Amar",    "Benjamin", "Dino",     "Edin",   "Elvir",  "Emir",   "Faruk", "Haris",
     "Ibrahim", "Jasmin", "Kenan",   "Lejla",    "Mehmed",   "Nedim",  "Osman",  "Safet",  "Tarik", "Vedad",
     "Zlatan",  "Emin",   "Nikola",  "Eldar",    "Senad",    "D?enan", "Aldin",  "Emir",   "Davor", "Adnan",
     "Emir",    "Amar",   "Alen",    "Nihad",    "Damir",    "Amir",   "Amer",   "Armin",  "Mirza", "Muhamed",
     "Eldin",   "Denis",  "Elmedin", "Selmir",   "Miralem",  "Miran",  "Anel",   "Semir",  "Lejla", "Azra",
     "Sara",    "Amna",   "Hana",    "Emina",    "Nermina",  "Lejla",  "Belma",  "Adna",   "Amina", "D?enita",
     "Ajla",    "Aldina", "Erna",    "Merjem",   "Lamija",   "Aida",   "Sabina", "Edna",   "Dina",  "Maja",
     "Vesna",   "Amela",  "Mirela",  "Ela",      "Dajana",   "Irma",   "Ena",    "Melisa", "Nina",  "Lana",
     "Una",     "Tara",   "Ela",     "Ariana",   "Adrijana", "Emina",  "Maida",  "Enisa",  "Lina",  "Amina",
     "Adna",    "Lejla",  "Ajla",    "Zerina",   "Mirnesa",  "Ayla",   "Adna",   "Dijana", "Hana",  "Amila"});

const auto bosnianLastNames = std::to_array<std::string_view>(
    {"Hodzi",      "Mujagic",        "Hadzi",       "Ahmetovic",   "Ali",         "Delic",         "Osmanagic",
     "Hrnjici",    "Omeragic",       "Suljic",      "Kovacevic",   "Brkic",       "Kurtic",        "Mehic",
     "Zukic",      "Hasic",          "Zimic",       "Salihovic",   "Dedic",       "Kolic",         "Dzafi",
     "Salkic",     "Omerhodzi",      "Alagic",      "Sijercic",    "Suljagic",    "Dautovic",      "Habibovic",
     "Cauzevic",   "Hadziahmetovic", "Muminovic",   "Brkan",       "Cici",        "Zvizdic",       "Maksumic",
     "Pilav",      "Cehic",          "Skopljak",    "Dizdarevic",  "Cengic",      "Karahasanovic", "Piric",
     "Cici",       "Mujic",          "Kujundzic",   "Puzic",       "Suljic",      "Joldic",        "Maksumic",
     "Hota",       "Kova",           "Selimovic",   "Terzic",      "Malko",       "Aganovic",      "Polic",
     "Catic",      "Dedic",          "Okanovic",    "Suljanovic",  "Bajramovic",  "Mehanovic",     "Karahodzi",
     "Campara",    "Nukic",          "Ramic",       "Ibricimovic", "Musi",        "Talic",         "Zimic",
     "Zulfic",     "Dizdar",         "Spahic",      "Zaimovic",    "Ibrahimovic", "Mehic",         "Kapic",
     "Beglija",    "Zukic",          "Cehic",       "Mahmic",      "Hodzi",       "Osman",         "Zukan",
     "Dedic",      "Maksumic",       "Tiro",        "Fazlic",      "Smajic",      "Hatic",         "Muhamedagic",
     "Dervisevic", "Mujagic",        "Mujezinovic", "Husagic",     "Zeki",        "Mehanovic",     "Malko",
     "Alagic",     "Kolic",          "Nadarevic",   "Suljic",      "Omanovic",    "Omanovic",      "Huski",
     "Aljic",      "Cici",           "Begic",       "Sijercic",    "Cengic",      "Karacic",       "Begirbajic",
     "Smajlovic",  "Mahmutovic",     "Kapetanovic", "Osman",       "Ibrahimagic", "Hadibegic",     "Becirovic",
     "Jusic",      "Brkic",          "Cozic",       "Musi",        "Cehic",       "Selimovic",     "Cehic",
     "Muminovic",  "Hodzi"});

const NameFormats bosnianNameFormats{{"{lastName} {firstName}", 1}};

const PeopleNames bosnianPeopleNames{{bosnianMaleFirstNames, bosnianLastNames, {}, {}},
                                     {bosnianFemaleFirstNames, bosnianLastNames, {}, {}},
                                     bosnianNameFormats};

// Brazil

const auto brazilianMaleFirstNames = std::to_array<std::string_view>(
    {"Alessandro",   "Alexandre", "Anthony",    "Antônio",        "Arthur",     "Benjamin",    "Benício",    "Bernardo",
     "Breno",        "Bryan",     "Caio",       "Calebe",         "Carlos",     "Cauã",        "César",      "Daniel",
     "Danilo",       "Davi",      "Davi Lucca", "Deneval",        "Eduardo",    "Elísio",      "Emanuel",    "Enzo",
     "Enzo Gabriel", "Fabiano",   "Fabrício",   "Feliciano",      "Felipe",     "Frederico",   "Fábio",      "Félix",
     "Gabriel",      "Gael",      "Guilherme",  "Gustavo",        "Gúbio",      "Heitor",      "Henrique",   "Hugo",
     "Hélio",        "Isaac",     "Joaquim",    "João",           "João Lucas", "João Miguel", "João Pedro", "Júlio",
     "Júlio César",  "Kléber",    "Ladislau",   "Leonardo",       "Lorenzo",    "Lucas",       "Lucca",      "Marcelo",
     "Marcos",       "Matheus",   "Miguel",     "Murilo",         "Nataniel",   "Nicolas",     "Noah",       "Norberto",
     "Pablo",        "Paulo",     "Pedro",      "Pedro Henrique", "Pietro",     "Rafael",      "Raul",       "Ricardo",
     "Roberto",      "Salvador",  "Samuel",     "Silas",          "Sirineu",    "Tertuliano",  "Théo",       "Vicente",
     "Vitor",        "Víctor",    "Warley",     "Washington",     "Yago",       "Yango",       "Yuri",       "Ígor"});

const auto brazilianFemaleFirstNames = std::to_array<std::string_view>(
    {"Alessandra",  "Alice",         "Aline",       "Alícia",        "Ana Clara",    "Ana Júlia",   "Ana Laura",
     "Ana Luiza",   "Antonella",     "Beatriz",     "Bruna",         "Carla",        "Cecília",     "Clara",
     "Célia",       "Dalila",        "Eduarda",     "Elisa",         "Eloá",         "Emanuelly",   "Esther",
     "Fabrícia",    "Felícia",       "Giovanna",    "Helena",        "Heloísa",      "Isabel",      "Isabela",
     "Isabella",    "Isabelly",      "Isis",        "Janaína",       "Joana",        "Júlia",       "Karla",
     "Lara",        "Larissa",       "Laura",       "Lavínia",       "Liz",          "Lorena",      "Lorraine",
     "Luiza",       "Lívia",         "Maitê",       "Manuela",       "Marcela",      "Margarida",   "Maria",
     "Maria Alice", "Maria Cecília", "Maria Clara", "Maria Eduarda", "Maria Helena", "Maria Júlia", "Maria Luiza",
     "Mariana",     "Marina",        "Marli",       "Meire",         "Melissa",      "Morgana",     "Márcia",
     "Mércia",      "Natália",       "Núbia",       "Ofélia",        "Paula",        "Rafaela",     "Rebeca",
     "Roberta",     "Sara",          "Sarah",       "Sophia",        "Suélen",       "Sílvia",      "Talita",
     "Valentina",   "Vitória",       "Yasmin"});

const auto brazilianLastNames =
    std::to_array<std::string_view>({"Silva", "Souza",   "Carvalho", "Santos",   "Reis",    "Xavier",  "Franco",
                                     "Braga", "Macedo",  "Batista",  "Barros",   "Moraes",  "Costa",   "Pereira",
                                     "Melo",  "Saraiva", "Nogueira", "Oliveira", "Martins", "Moreira", "Albuquerque"});

const auto brazilianSuffixes = std::to_array<std::string_view>({"Jr.", "Neto", "Filho"});

const auto brazilianMalePrefixes = std::to_array<std::string_view>({"Sr.", "Dr."});

const auto brazilianFemalePrefixes = std::to_array<std::string_view>({"Sra.", "Srta.", "Dra."});

const NameFormats brazilianNameFormats{
    {{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames brazilianPeopleNames{
    {brazilianMaleFirstNames, brazilianLastNames, brazilianMalePrefixes, brazilianSuffixes},
    {brazilianFemaleFirstNames, brazilianLastNames, brazilianFemalePrefixes, brazilianSuffixes},
    brazilianNameFormats};

// Bulgaria

const auto bulgarianMaleFirstNames = std::to_array<std::string_view>({
    "Aleksandar", "Angel",   "Anton",  "Boris",   "Boyan",    "Dimitar", "Emil",    "Georgi",   "Hristo", "Iliya",
    "Ivan",       "Kaloyan", "Kiril",  "Lubomir", "Lyubomir", "Mihail",  "Nikolay", "Ognian",   "Petar",  "Plamen",
    "Radoslav",   "Stefan",  "Stoyan", "Todor",   "Valentin", "Vasil",   "Viktor",  "Vladimir", "Yordan", "Zahari",
});

const auto bulgarianFemaleFirstNames = std::to_array<std::string_view>({
    "Aleksandra", "Anastasiya", "Anna",     "Borislava", "Daniela", "Diana",    "Ekaterina", "Elitsa",
    "Gabriela",   "Gergana",    "Iliana",   "Ivanka",    "Kalina",  "Katerina", "Lidiya",    "Maria",
    "Margarita",  "Nadezhda",   "Nataliya", "Nevena",    "Olya",    "Petya",    "Radostina", "Rumyana",
    "Silvia",     "Sofiya",     "Svetlana", "Tsvetanka", "Vanya",   "Vesela",   "Yana",      "Zoya",
});

const auto bulgarianLastNames = std::to_array<std::string_view>({
    "Angelov",  "Atanasov", "Bozhilov", "Dimitrov", "Draganov",   "Georgiev", "Hristov",  "Iliev", "Ivanov",
    "Kolev",    "Kovachev", "Lazarov",  "Mihaylov", "Nikolov",    "Petrov",   "Popov",    "Radev", "Simeonov",
    "Stefanov", "Stoyanov", "Todorov",  "Vasilev",  "Vladimirov", "Yankov",   "Zahariev",
});

const auto bulgarianSuffixes = std::to_array<std::string_view>({"мл.", "ст."});

const auto bulgarianMalePrefixes = std::to_array<std::string_view>({"Г-н", "Д-р", "Проф."});

const auto bulgarianFemalePrefixes = std::to_array<std::string_view>({"Г-жа", "Г-ца", "Д-р", "Проф."});

const NameFormats bulgarianNameFormats{{
    {"{firstName} {lastName}", 8},
    {"{prefix} {firstName} {lastName}", 1},
    {"{firstName} {lastName} {suffix}", 1},
}};

const PeopleNames bulgarianPeopleNames{
    {bulgarianMaleFirstNames, bulgarianLastNames, bulgarianMalePrefixes, bulgarianSuffixes},
    {bulgarianFemaleFirstNames, bulgarianLastNames, bulgarianFemalePrefixes, bulgarianSuffixes},
    bulgarianNameFormats};

// Canada

const auto canadianMaleFirstNames = std::to_array<std::string_view>({
    "John",     "David",   "Robert",    "Michael",   "Paul",        "Richard",  "James",     "Peter",     "William",
    "Brian",    "Daniel",  "Mark",      "Chris",     "Michel",      "Kevin",    "George",    "Ken",       "Steven",
    "Jim",      "Andrew",  "Eric",      "Pierre",    "Ron",         "Andre",    "Gary",      "Thomas",    "Frank",
    "Jason",    "Jeff",    "Scott",     "Donald",    "Roger",       "Patrick",  "Wayne",     "Martin",    "Joe",
    "Jacques",  "Terry",   "Bob",       "Bruce",     "Marc",        "Greg",     "Gordon",    "Joseph",    "Stephen",
    "Gilles",   "Rick",    "Mike",      "Raymond",   "Edward",      "Guy",      "Larry",     "Charles",   "Tim",
    "Tony",     "Alain",   "Ryan",      "Jack",      "Alex",        "Ian",      "Mario",     "Gerald",    "Ronald",
    "Marcel",   "Fred",    "Steve",     "Keith",     "Allan",       "Bill",     "Randy",     "Dave",      "Barry",
    "Douglas",  "Serge",   "Sylvain",   "Alan",      "Ray",         "Jan",      "Luc",       "Matthew",   "Yves",
    "Tom",      "Adam",    "Craig",     "Louis",     "Sean",        "Norman",   "Nick",      "Jonathan",  "Roy",
    "Anthony",  "Yvon",    "Walter",    "Neil",      "Bernard",     "Maurice",  "Henry",     "Kenneth",   "Albert",
    "Glen",     "Derek",   "Simon",     "Christian", "Christopher", "Harold",   "Carl",      "Sam",       "Trevor",
    "Gerard",   "Glenn",   "Matt",      "Harry",     "Jamie",       "Dean",     "Colin",     "Darren",    "Benoit",
    "Arthur",   "Jerry",   "Justin",    "Philip",    "Ralph",       "Ross",     "Gerry",     "Roland",    "Bryan",
    "Grant",    "Leo",     "Benjamin",  "Vincent",   "Gaetan",      "Lloyd",    "Lawrence",  "Leonard",   "Rob",
    "Aaron",    "Kyle",    "Jose",      "Philippe",  "Joel",        "Stan",     "Bruno",     "Jay",       "Howard",
    "Jeremy",   "Victor",  "Tyler",     "Shane",     "Ivan",        "Graham",   "Russell",   "Garry",     "Jordan",
    "Mathieu",  "Yvan",    "Ali",       "Gaston",    "Jeffrey",     "Ernest",   "Alexander", "Fernand",   "Warren",
    "Gilbert",  "Jon",     "Karl",      "Alfred",    "Eugene",      "Allen",    "Stuart",    "Alexandre", "Dominic",
    "Nathan",   "Gabriel", "Hugh",      "Sebastien", "Adrian",      "Pascal",   "Stanley",   "Antonio",   "Georges",
    "Harvey",   "Nicolas", "Nicholas",  "Gregory",   "Josh",        "Lucien",   "Samuel",    "Cameron",   "Marco",
    "Laurent",  "Ben",     "Carlos",    "Brandon",   "Jesse",       "Brett",    "Nelson",    "Patrice",   "Lionel",
    "Timothy",  "Andy",    "Jake",      "Clement",   "Calvin",      "Jacob",    "Arnold",    "Max",       "Leon",
    "Mohamed",  "Joshua",  "Jimmy",     "Mohammad",  "Rodney",      "Daryl",    "Travis",    "Julien",    "Manuel",
    "Shaun",    "Bert",    "Maxime",    "Edwin",     "Frederic",    "Clifford", "Malcolm",   "Fran",      "Germain",
    "Bertrand", "Kris",    "Hans",      "Muhammad",  "Angelo",      "Henri",    "Phillip",   "Marvin",    "Armand",
    "Perry",    "Evan",    "Guillaume", "Charlie",   "Clayton",     "Mohammed", "Ahmed",     "Stewart",   "Remi",
    "Herbert",  "Luke",    "Olivier",   "Conrad",    "Alvin",       "Jerome",   "Syed",      "Luis",      "Edgar",
    "Jules",    "Derrick", "Kurt",      "Lance",     "Johnny",      "Bradley",  "Wilfred",   "Melvin",    "Frederick",
    "Erik",     "Gino",    "Joey",      "Duncan",    "Reginald",    "Dustin",   "Emile",     "Antoine",   "Wesley",
    "Geoffrey", "Owen",    "Brendan",   "Taylor",    "Herman",      "Stefan",   "Rudy",      "Andrzej",   "Hugo",
    "Hubert",   "Abdul",   "Fernando",  "Etienne",   "Vernon",      "Roman",    "Emmanuel",  "Adrien",    "Jaime",
    "Byron",    "Bobby",   "Amir",      "Roberto",   "Tommy",       "Jorge",    "Stephan",   "Julian",    "Eddie",
    "Dick",     "Elmer",   "Dylan",     "Felix",     "Cyril",       "Omar",     "Romeo",     "Jared",     "Oliver",
    "Luigi",    "Ahmad",   "Billy",     "Sunny",     "Vladimir",    "Carlo",    "Micheal",   "Angel",     "Mitchell",
    "Raj",      "Winston", "Morris",    "Hector",    "Igor",        "Nigel",    "Giuseppe",  "Gavin",     "Kenny",
    "Ricky",    "Wally",   "Kelvin",    "Giovanni",  "Lewis",       "Anton",    "Wilson",    "Kai",       "Angus",
    "Graeme",   "Ricardo", "Miguel",    "Mathew",    "Karim",       "Isaac",    "Edmond",    "Marcus",    "Wallace",
    "Austin",   "Chi",     "Oscar",     "Dino",      "Devin",       "Lucas",    "Boris",     "Norbert",
});

const auto canadianFemaleFirstNames = std::to_array<std::string_view>({
    "Mary",       "Linda",     "Karen",     "Jennifer",  "Lisa",      "Susan",     "Jean",       "Michelle",
    "Nancy",      "Kim",       "Kelly",     "Diane",     "Donna",     "Julie",     "Claude",     "Sandra",
    "Heather",    "Carol",     "Denis",     "Dan",       "Nicole",    "Barbara",   "Christine",  "Sarah",
    "Pat",        "Sharon",    "Brenda",    "Elizabeth", "Laura",     "Anne",      "Margaret",   "Debbie",
    "Wendy",      "Joanne",    "Marie",     "Kathy",     "Amanda",    "Judy",      "Cindy",      "Angela",
    "Helen",      "Stephanie", "Melissa",   "Janet",     "Joan",      "Denise",    "Cheryl",     "Cathy",
    "Anna",       "Dennis",    "Shirley",   "Catherine", "Lynn",      "Louise",    "Maria",      "Robin",
    "Andrea",     "Tracy",     "Jane",      "Ann",       "Patricia",  "Melanie",   "Rene",       "Sue",
    "Laurie",     "Shannon",   "Dale",      "Suzanne",   "Ruth",      "Betty",     "Janice",     "Amy",
    "Stephane",   "Tina",      "Danielle",  "Irene",     "Gail",      "Marilyn",   "Elaine",     "Sylvie",
    "Bonnie",     "Rose",      "Shelley",   "Sheila",    "Joyce",     "Caroline",  "Valerie",    "Erin",
    "Ashley",     "Rita",      "Tanya",     "Darlene",   "Dawn",      "Christina", "Nathalie",   "Lorraine",
    "Annie",      "Rachel",    "Carole",    "Dorothy",   "Maureen",   "Diana",     "Kathleen",   "Carolyn",
    "Chantal",    "Tara",      "Isabelle",  "Paula",     "Rebecca",   "Teresa",    "Lise",       "Pauline",
    "Tracey",     "Leslie",    "Connie",    "Theresa",   "Anita",     "Carmen",    "Jacqueline", "Sara",
    "Lindsay",    "Francis",   "Danny",     "Grace",     "Monique",   "Audrey",    "Crystal",    "Cynthia",
    "Natalie",    "Marlene",   "Yvonne",    "Doris",     "Dana",      "Alice",     "Claire",     "Michele",
    "Sylvia",     "Sandy",     "Jill",      "Katherine", "Marion",    "Emily",     "Kerry",      "Lee",
    "Monica",     "Samantha",  "Stacey",    "Lynda",     "Victoria",  "Evelyn",    "Jocelyn",    "Debra",
    "Sonia",      "Jenny",     "Eva",       "Dianne",    "Carrie",    "June",      "Vanessa",    "Megan",
    "Alison",     "Norma",     "Francine",  "Doreen",    "Krista",    "Johanne",   "Jessica",    "Deborah",
    "Allison",    "Holly",     "Judith",    "Manon",     "Annette",   "Peggy",     "Ellen",      "Renee",
    "Helene",     "Kate",      "Gloria",    "Lucie",     "Beth",      "Frances",   "Wanda",      "Sophie",
    "Natasha",    "Lois",      "Eileen",    "Kimberly",  "Carla",     "Leanne",    "Katie",      "Lynne",
    "Charlene",   "Angie",     "Joy",       "Lucy",      "Leah",      "Dominique", "Therese",    "Ginette",
    "Gina",       "Nadine",    "Phyllis",   "Kathryn",   "Arlene",    "Amber",     "France",     "Nadia",
    "Lorna",      "Lesley",    "Heidi",     "Florence",  "Alexandra", "Pamela",    "Martha",     "Edith",
    "Lauren",     "Vicki",     "April",     "Beverly",   "Sally",     "Tamara",    "Karine",     "Olga",
    "Dany",       "Genevieve", "Jeanne",    "Marjorie",  "Robyn",     "Gwen",      "Esther",     "Emma",
    "Joanna",     "Sabrina",   "Claudette", "Ana",       "Vivian",    "Vicky",     "Veronica",   "Lillian",
    "Brigitte",   "Charlotte", "Maggie",    "Marina",    "Noel",      "Agnes",     "Kristen",    "Elena",
    "Jessie",     "Camille",   "Cecile",    "Gisele",    "Vera",      "Stella",    "Nina",       "Tiffany",
    "Clare",      "Roxanne",   "Mona",      "Marian",    "Julia",     "Bernice",   "Ruby",       "Marianne",
    "Eleanor",    "Edna",      "Rosemary",  "Mel",       "Micheline", "Jocelyne",  "Beverley",   "Celine",
    "Lana",       "Liz",       "Roberta",   "Mandy",     "Nicola",    "Madeleine", "Hong",       "Kay",
    "Jeanette",   "Yvette",    "Kimberley", "Lina",      "Gladys",    "Glenda",    "Jeannine",   "Colette",
    "Erika",      "Leona",     "Gillian",   "Rosa",      "Brittany",  "Hazel",     "Elsie",      "Ingrid",
    "Stacy",      "Lucille",   "May",       "Kristina",  "Simone",    "Corinne",   "Faye",       "Lydia",
    "Nora",       "Muriel",    "Kristin",   "Janine",    "Jenna",     "Loretta",   "Lily",       "Kayla",
    "Candice",    "Isabel",    "Marcia",    "Leigh",     "Aline",     "Jeannette", "Paulette",   "Josephine",
    "Chelsea",    "Veronique", "Fiona",     "Ida",       "Lena",      "Li",        "Juan",       "Bernadette",
    "Sheryl",     "Maxine",    "Hilda",     "Nikki",     "Lou",       "Kari",      "Yan",        "Josie",
    "Sonya",      "Helena",    "Iris",      "Ming",      "Cassandra", "Geraldine", "Line",       "Rachelle",
    "Marguerite", "Virginia",  "Clara",     "Pierrette", "Karin",     "Katrina",
});

const auto canadianLastNames = std::to_array<std::string_view>({
    "Smith",      "Brown",      "Tremblay",    "Martin",    "Roy",        "Gagnon",    "Lee",          "Wilson",
    "Johnson",    "Macdonald",  "Taylor",      "Campbell",  "Anderson",   "Jones",     "Leblanc",      "Cote",
    "Williams",   "Miller",     "Thompson",    "Gauthier",  "White",      "Morin",     "Wong",         "Young",
    "Bouchard",   "Scott",      "Stewart",     "Pelletier", "Lavoie",     "Robinson",  "Moore",        "Belanger",
    "Singh",      "Fortin",     "Levesque",    "Chan",      "Reid",       "Ross",      "Clark",        "Johnston",
    "Walker",     "Thomas",     "King",        "Gagne",     "Bergeron",   "Li",        "Boucher",      "Landry",
    "Poirier",    "Murray",     "Murphy",      "Mcdonald",  "Wright",     "Richard",   "Mitchell",     "Girard",
    "Clarke",     "Davis",      "Simard",      "Kelly",     "Lewis",      "Graham",    "Caron",        "Wang",
    "Fraser",     "Fournier",   "Jackson",     "Beaulieu",  "Wood",       "Hall",      "Baker",        "Chen",
    "Hill",       "Harris",     "Green",       "Roberts",   "Lapointe",   "Bell",      "Ouellet",      "Patel",
    "Watson",     "Kennedy",    "Cloutier",    "Robertson", "Allen",      "Lefebvre",  "Nguyen",       "Hamilton",
    "Desjardins", "Adams",      "Gill",        "Khan",      "Cameron",    "Morrison",  "Dube",         "Evans",
    "Grant",      "Nadeau",     "Zhang",       "Peters",    "Armstrong",  "Phillips",  "Cook",         "Hebert",
    "Poulin",     "Liu",        "Michaud",     "Kim",       "Martel",     "Edwards",   "Turner",       "Nelson",
    "Bennett",    "Cooper",     "Ferguson",    "Gray",      "Paquette",   "Marshall",  "Cormier",      "Simpson",
    "Harvey",     "Mclean",     "Collins",     "Leclerc",   "Bedard",     "Grenier",   "Russell",      "Couture",
    "Lessard",    "Cyr",        "Ward",        "Shaw",      "Boudreau",   "Bernier",   "Lambert",      "Lalonde",
    "Friesen",    "Blais",      "Proulx",      "Morris",    "Arsenault",  "Henderson", "Parker",       "Demers",
    "Gilbert",    "Hunter",     "Gallant",     "Davidson",  "Dupuis",     "Elliott",   "Walsh",        "Turcotte",
    "Lemieux",    "Harrison",   "Lachance",    "Carter",    "Richardson", "James",     "Beaudoin",     "Foster",
    "Gosselin",   "Mackenzie",  "Gordon",      "Fisher",    "Hughes",     "Parent",    "Theriault",    "Lam",
    "Rogers",     "Perron",     "Gibson",      "Ryan",      "Morgan",     "Langlois",  "Savard",       "Perreault",
    "Patterson",  "Thibault",   "Mcleod",      "Bailey",    "Mercier",    "Mckay",     "Villeneuve",   "Thomson",
    "Dion",       "Fortier",    "Charbonneau", "Bernard",   "Robert",     "Dubois",    "Giroux",       "Leung",
    "Dufour",     "Schmidt",    "Black",       "Paradis",   "Davies",     "Ouellette", "Houle",        "Macleod",
    "Menard",     "Rose",       "Champagne",   "Mills",     "Plante",     "Tran",      "Benoit",       "Maclean",
    "Leduc",      "Boisvert",   "Wu",          "Allard",    "Legault",    "Hamel",     "Wiebe",        "Stevens",
    "Berube",     "Lemay",      "Lacroix",     "Rousseau",  "Labelle",    "Renaud",    "Bolduc",       "Klassen",
    "Paul",       "Parsons",    "Bertrand",    "Perry",     "Bilodeau",   "Henry",     "Ellis",        "Ng",
    "Wallace",    "Burns",      "Mason",       "Hunt",      "Park",       "Ho",        "Fontaine",     "Seguin",
    "Therrien",   "Andrews",    "Crawford",    "Butler",    "Brooks",     "Gervais",   "Yu",           "Kerr",
    "Dyck",       "Yang",       "Alexander",   "Price",     "Burke",      "Saunders",  "Boivin",       "Mckenzie",
    "Richards",   "Lawrence",   "Holmes",      "Dionne",    "Goulet",     "Sullivan",  "Power",        "Cole",
    "Guay",       "Lepage",     "Lauzon",      "Mackay",    "Vincent",    "Ali",       "Huang",        "Vachon",
    "Robichaud",  "Jacques",    "Dunn",        "Doucet",    "Gravel",     "Picard",    "Noel",         "Doyle",
    "Matthews",   "Carrier",    "Paquet",      "Moreau",    "Larocque",   "Peterson",  "Chapman",      "Sinclair",
    "Palmer",     "Sutherland", "Duncan",      "Cox",       "Stevenson",  "Pilon",     "Vaillancourt", "Craig",
    "Porter",     "Savoie",     "Jean",        "Chartrand", "Godin",      "Page",      "Mann",         "Comeau",
    "Cheung",     "George",     "Boyd",        "Daigle",    "Desrosiers", "Sharma",    "Trudel",       "Hart",
    "Wells",      "Penner",     "Robitaille",  "Pearson",   "Rioux",      "Lapierre",  "Hansen",       "Francis",
    "Dumont",     "Charron",    "Ford",        "Douglas",   "Fox",        "Gingras",   "Woods",        "Warren",
    "Dixon",      "Barnes",     "Lau",         "Chow",      "Spencer",    "Gendron",   "Lin",          "Reynolds",
    "Audet",      "Marchand",   "Jensen",      "Lavigne",   "Cunningham", "Mcintyre",  "Bourque",      "Lavallee",
    "Bradley",    "Deschenes",
});

const NameFormats canadianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames canadianPeopleNames{{canadianMaleFirstNames, canadianLastNames, {}, {}},
                                      {canadianFemaleFirstNames, canadianLastNames, {}, {}},
                                      canadianNameFormats};

// China

const auto chineseLastNames = std::to_array<std::string_view>({
    "赵",   "钱",   "孙",   "李",   "周",   "吴",   "郑",   "王",   "冯",   "陈",   "褚",   "卫",   "蒋",   "沈",
    "韩",   "杨",   "朱",   "秦",   "尤",   "许",   "何",   "吕",   "施",   "张",   "孔",   "曹",   "严",   "华",
    "金",   "魏",   "陶",   "姜",   "戚",   "谢",   "邹",   "喻",   "柏",   "水",   "窦",   "章",   "云",   "苏",
    "潘",   "葛",   "奚",   "范",   "彭",   "郎",   "鲁",   "韦",   "昌",   "马",   "苗",   "凤",   "花",   "方",
    "俞",   "任",   "袁",   "柳",   "酆",   "鲍",   "史",   "唐",   "费",   "廉",   "岑",   "薛",   "雷",   "贺",
    "倪",   "汤",   "滕",   "殷",   "罗",   "毕",   "郝",   "邬",   "安",   "常",   "乐",   "于",   "时",   "傅",
    "皮",   "卞",   "齐",   "康",   "伍",   "余",   "元",   "卜",   "顾",   "孟",   "平",   "黄",   "和",   "穆",
    "萧",   "尹",   "姚",   "邵",   "湛",   "汪",   "祁",   "毛",   "禹",   "狄",   "米",   "贝",   "明",   "臧",
    "计",   "伏",   "成",   "戴",   "谈",   "宋",   "茅",   "庞",   "熊",   "纪",   "舒",   "屈",   "项",   "祝",
    "董",   "梁",   "杜",   "阮",   "蓝",   "闵",   "席",   "季",   "麻",   "强",   "贾",   "路",   "娄",   "危",
    "江",   "童",   "颜",   "郭",   "梅",   "盛",   "林",   "刁",   "锺",   "徐",   "邱",   "骆",   "高",   "夏",
    "蔡",   "田",   "樊",   "胡",   "凌",   "蹇",   "称",   "诺",   "来",   "多",   "繁",   "戊",   "朴",   "回",
    "毓",   "税",   "荤",   "靖",   "绪",   "愈",   "硕",   "牢",   "买",   "但",   "巧",   "枚",   "撒",   "泰",
    "秘",   "亥",   "绍",   "以",   "壬",   "森",   "斋",   "释",   "奕",   "姒",   "朋",   "求",   "羽",   "用",
    "占",   "真",   "穰",   "翦",   "闾",   "漆",   "贵",   "代",   "贯",   "旁",   "崇",   "栋",   "告",   "休",
    "褒",   "谏",   "锐",   "皋",   "闳",   "在",   "歧",   "禾",   "示",   "是",   "委",   "钊",   "频",   "嬴",
    "呼",   "大",   "威",   "昂",   "律",   "冒",   "保",   "系",   "抄",   "定",   "化",   "莱",   "校",   "么",
    "抗",   "祢",   "綦",   "悟",   "宏",   "功",   "庚",   "务",   "敏",   "捷",   "拱",   "兆",   "丑",   "丙",
    "畅",   "苟",   "随",   "类",   "卯",   "俟",   "友",   "答",   "乙",   "允",   "甲",   "留",   "尾",   "佼",
    "玄",   "乘",   "裔",   "延",   "植",   "环",   "矫",   "赛",   "昔",   "侍",   "度",   "旷",   "遇",   "偶",
    "前",   "由",   "咎",   "塞",   "敛",   "受",   "泷",   "袭",   "衅",   "叔",   "圣",   "御",   "夫",   "仆",
    "镇",   "藩",   "邸",   "府",   "掌",   "首",   "员",   "焉",   "戏",   "可",   "智",   "尔",   "凭",   "悉",
    "进",   "笃",   "厚",   "仁",   "业",   "肇",   "资",   "合",   "仍",   "九",   "衷",   "哀",   "刑",   "俎",
    "仵",   "圭",   "夷",   "徭",   "蛮",   "汗",   "孛",   "乾",   "帖",   "罕",   "洛",   "淦",   "洋",   "邶",
    "郸",   "郯",   "邗",   "邛",   "剑",   "虢",   "隋",   "蒿",   "茆",   "菅",   "苌",   "树",   "桐",   "锁",
    "钟",   "机",   "盘",   "铎",   "斛",   "玉",   "线",   "针",   "箕",   "庹",   "绳",   "磨",   "蒉",   "瓮",
    "弭",   "刀",   "疏",   "牵",   "浑",   "恽",   "势",   "世",   "仝",   "同",   "蚁",   "止",   "戢",   "睢",
    "冼",   "种",   "涂",   "肖",   "己",   "泣",   "潜",   "卷",   "脱",   "谬",   "蹉",   "赧",   "浮",   "顿",
    "说",   "次",   "错",   "念",   "夙",   "斯",   "完",   "丹",   "表",   "聊",   "源",   "姓",   "吾",   "寻",
    "展",   "出",   "不",   "户",   "闭",   "才",   "无",   "书",   "学",   "愚",   "本",   "性",   "雪",   "霜",
    "烟",   "寒",   "少",   "字",   "桥",   "板",   "斐",   "独",   "千",   "诗",   "嘉",   "扬",   "善",   "揭",
    "祈",   "析",   "赤",   "紫",   "青",   "柔",   "刚",   "奇",   "拜",   "佛",   "陀",   "弥",   "阿",   "素",
    "长",   "僧",   "隐",   "仙",   "隽",   "宇",   "祭",   "酒",   "淡",   "塔",   "琦",   "闪",   "始",   "星",
    "南",   "天",   "接",   "波",   "碧",   "速",   "禚",   "腾",   "潮",   "镜",   "似",   "澄",   "潭",   "謇",
    "纵",   "渠",   "奈",   "风",   "春",   "濯",   "沐",   "茂",   "英",   "兰",   "檀",   "藤",   "枝",   "检",
    "生",   "折",   "登",   "驹",   "骑",   "貊",   "虎",   "肥",   "鹿",   "雀",   "野",   "禽",   "飞",   "节",
    "宜",   "鲜",   "粟",   "栗",   "豆",   "帛",   "官",   "布",   "衣",   "藏",   "宝",   "钞",   "银",   "门",
    "盈",   "庆",   "喜",   "及",   "普",   "建",   "营",   "巨",   "望",   "希",   "道",   "载",   "声",   "漫",
    "犁",   "力",   "贸",   "勤",   "革",   "改",   "兴",   "亓",   "睦",   "修",   "信",   "闽",   "北",   "守",
    "坚",   "勇",   "汉",   "练",   "尉",   "士",   "旅",   "五",   "令",   "将",   "旗",   "军",   "行",   "奉",
    "敬",   "恭",   "仪",   "母",   "堂",   "丘",   "义",   "礼",   "慈",   "孝",   "理",   "伦",   "卿",   "问",
    "永",   "辉",   "位",   "让",   "尧",   "依",   "犹",   "介",   "承",   "市",   "所",   "苑",   "杞",   "剧",
    "第",   "零",   "谌",   "招",   "续",   "达",   "忻",   "六",   "鄞",   "战",   "迟",   "候",   "宛",   "励",
    "粘",   "萨",   "邝",   "覃",   "辜",   "初",   "楼",   "城",   "区",   "局",   "台",   "原",   "考",   "妫",
    "纳",   "泉",   "老",   "清",   "德",   "卑",   "过",   "麦",   "曲",   "竹",   "百",   "福",   "言",   "霍",
    "虞",   "万",   "支",   "柯",   "昝",   "管",   "卢",   "莫",   "经",   "房",   "裘",   "缪",   "干",   "解",
    "应",   "宗",   "丁",   "宣",   "贲",   "邓",   "单",   "杭",   "洪",   "包",   "诸",   "左",   "石",   "崔",
    "吉",   "钮",   "龚",   "程",   "嵇",   "邢",   "滑",   "裴",   "陆",   "荣",   "翁",   "荀",   "羊",   "於",
    "惠",   "甄",   "麴",   "家",   "封",   "芮",   "羿",   "储",   "靳",   "汲",   "邴",   "糜",   "松",   "井",
    "段",   "富",   "巫",   "乌",   "焦",   "巴",   "弓",   "牧",   "隗",   "山",   "谷",   "车",   "侯",   "宓",
    "蓬",   "全",   "郗",   "班",   "仰",   "秋",   "仲",   "伊",   "宫",   "宁",   "仇",   "栾",   "暴",   "甘",
    "钭",   "历",   "戎",   "祖",   "武",   "符",   "刘",   "景",   "詹",   "束",   "龙",   "叶",   "幸",   "司",
    "韶",   "郜",   "黎",   "蓟",   "溥",   "印",   "宿",   "白",   "怀",   "蒲",   "邰",   "召",   "有",   "舜",
    "拉",   "丛",   "岳",   "寸",   "贰",   "皇",   "侨",   "彤",   "竭",   "端",   "赫",   "实",   "甫",   "集",
    "象",   "翠",   "狂",   "辟",   "典",   "良",   "函",   "芒",   "苦",   "其",   "京",   "中",   "夕",   "之",
    "从",   "鄂",   "索",   "咸",   "籍",   "赖",   "卓",   "蔺",   "屠",   "蒙",   "池",   "乔",   "阳",   "郁",
    "胥",   "能",   "苍",   "双",   "闻",   "莘",   "党",   "翟",   "谭",   "贡",   "劳",   "逄",   "姬",   "申",
    "扶",   "堵",   "冉",   "宰",   "郦",   "雍",   "却",   "璩",   "桑",   "桂",   "濮",   "牛",   "寿",   "通",
    "边",   "扈",   "燕",   "冀",   "僪",   "浦",   "尚",   "农",   "温",   "别",   "庄",   "晏",   "柴",   "瞿",
    "阎",   "充",   "慕",   "连",   "茹",   "习",   "宦",   "艾",   "鱼",   "容",   "向",   "古",   "易",   "慎",
    "戈",   "廖",   "庾",   "终",   "暨",   "居",   "衡",   "步",   "都",   "耿",   "满",   "弘",   "匡",   "国",
    "文",   "琴",   "况",   "亢",   "缑",   "帅",   "寇",   "广",   "禄",   "阙",   "东",   "欧",   "殳",   "沃",
    "利",   "蔚",   "越",   "夔",   "隆",   "师",   "巩",   "厍",   "聂",   "晁",   "勾",   "敖",   "融",   "冷",
    "訾",   "辛",   "阚",   "那",   "简",   "饶",   "空",   "曾",   "毋",   "沙",   "乜",   "养",   "鞠",   "须",
    "丰",   "巢",   "关",   "蒯",   "相",   "查",   "后",   "荆",   "红",   "游",   "特",   "察",   "竺",   "冠",
    "宾",   "香",   "赏",   "伯",   "佴",   "佘",   "佟",   "爱",   "年",   "笪",   "谯",   "哈",   "墨",   "牟",
    "商",   "海",   "归",   "钦",   "鄢",   "汝",   "法",   "闫",   "楚",   "晋",   "督",   "仉",   "盖",   "逯",
    "库",   "郏",   "逢",   "阴",   "薄",   "厉",   "稽",   "开",   "光",   "操",   "瑞",   "眭",   "泥",   "运",
    "摩",   "伟",   "铁",   "迮",   "果",   "权",   "逮",   "盍",   "益",   "桓",   "公",   "万俟", "司马", "上官",
    "欧阳", "夏侯", "诸葛", "闻人", "东方", "赫连", "皇甫", "尉迟", "公羊", "澹台", "公冶", "宗政", "濮阳", "淳于",
    "单于", "太叔", "申屠", "公孙", "仲孙", "轩辕", "令狐", "钟离", "宇文", "长孙", "慕容", "司徒", "司空", "章佳",
    "那拉", "觉罗", "纳喇", "乌雅", "范姜", "碧鲁",
});

const auto chineseMaleFirstNames = std::to_array<std::string_view>({
    "建华", "建国", "建军", "国强", "国平", "国良", "国栋", "国辉", "志国", "志明", "勇",   "军",   "伟",
    "强",   "刚",   "涛",   "斌",   "波",   "辉",   "磊",   "超",   "鹏",   "杰",   "浩",   "鑫",   "帅",
    "宇",   "晨",   "诚",   "成",   "民",   "明",   "阳",   "瑜",   "熙成", "熙瑶", "家豪", "家明", "俊杰",
    "俊熙", "沐宸", "浩宇", "浩然", "浩轩", "浩晨", "沐辰", "茗泽", "奕辰", "奕泽", "宇泽", "宇轩", "宇航",
    "沐阳", "梓诚", "梓豪", "梓睿", "梓浩", "浩辰", "癸霖", "一全", "三锋", "义轩", "俊凯", "子豪", "振东",
    "智杰", "哲新", "文昊", "中海", "超栋", "治涛", "治文", "文韬", "敬彪", "敬阳", "政君", "立伟", "呈轩",
});

const auto chineseFemaleFirstNames = std::to_array<std::string_view>({
    "秀英", "秀兰", "秀珍", "桂英", "桂兰", "玉兰", "玉珍", "玉英", "玉梅", "凤英", "兰英", "婷婷", "国英",
    "国珍", "国华", "国芳", "国兰", "国秀", "国琴", "国荣", "国香", "英",   "萍",   "蒙",   "红",   "丽",
    "敏",   "芳",   "静",   "霞",   "燕",   "娟",   "艳",   "娜",   "丹",   "玲",   "婷",   "珈",   "雪",
    "倩",   "悦",   "颖",   "洁",   "慧",   "开慧", "丽芬", "丽芳", "丽萍", "若汐", "一诺", "艺涵", "依诺",
    "梓涵", "梓晨", "梓馨", "梓萱", "梓妍", "梓玥", "苡沫", "雨桐", "欣怡", "语桐", "语汐", "雨涵", "雨欣",
    "诗雨", "婷方", "美方", "雅婷", "紫林", "天娇", "万佳", "子欣", "海燕", "乙萍", "安琪", "馨羽", "馥君",
    "思佳", "雅鑫", "静怡", "晨阳", "佳琪", "雯静", "榕融",
});

const NameFormats chineseNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames chinesePeopleNames{{chineseMaleFirstNames, chineseLastNames, {}, {}},
                                     {chineseFemaleFirstNames, chineseLastNames, {}, {}},
                                     chineseNameFormats};

// Croatia

const auto croatianMaleFirstNames = std::to_array<std::string_view>(
    {"Adi",      "Ado",       "Andel",   "Andelin",  "Andelko",   "Andelo",    "Andi",       "Andras",   "Andrej",
     "Andrija",  "Andro",     "Anko",    "Ante",     "Antek",     "Antonin",   "Antonio",    "Antun",    "Bela",
     "Belimir",  "Berti",     "Boris",   "Borko",    "Borna",     "Božidar",   "Branimir",   "Bruno",    "Časlav",
     "Dado",     "Damil",     "Damir",   "Danijel",  "Dario",     "Darko",     "David",      "Davor",    "Davorin",
     "Domagoj",  "Donat",     "Dražen",  "Držimir",  "Držislav",  "Dubravko",  "Duje",       "Dujo",     "Emil",
     "Emilijan", "Erik",      "Fabijan", "Filip",    "Franjo",    "Gaj",       "Gašpar",     "Goran",    "Gordan",
     "Gradimir", "Grga",      "Grgo",    "Grgur",    "Gvozden",   "Hrvoj",     "Hrvoje",     "Ico",      "Igor",
     "Ilija",    "Ilko",      "Ino",     "Ivan",     "Ive",       "Ivek",      "Ivica",      "Ivo",      "Jadran",
     "Jadranko", "Jadro",     "Jakov",   "Jan",      "Janko",     "Jasen",     "Jasenko",    "Josip",    "Joško",
     "Joso",     "Joža",      "Jože",    "Krešimir", "Kristijan", "Kruno",     "Krunoslav",  "Ladislav", "Ljudevit",
     "Lucijan",  "Luka",      "Marin",   "Marko",    "Martin",    "Matan",     "Mate",       "Matej",    "Matija",
     "Matko",    "Mato",      "Mihael",  "Milan",    "Mile",      "Milivoj",   "Miljenko",   "Mimo",     "Miran",
     "Mirko",    "Miro",      "Mislav",  "Mladen",   "Nedjeljko", "Neven",     "Nikola",     "Nino",     "Ozren",
     "Pavel",    "Pepo",      "Pero",    "Petar",    "Pravdan",   "Rino",      "Robert",     "Roberto",  "Robi",
     "Roko",     "Sanjin",    "Saša",    "Slavek",   "Slaven",    "Slavko",    "Srećko",     "Stanko",   "Štef",
     "Stipan",   "Stipe",     "Stjepan", "Stribor",  "Svebor",    "Tihomir",   "Toma",       "Tome",     "Tomislav",
     "Tomo",     "Tonči",     "Tone",    "Toni",     "Tonko",     "Trpimir",   "Trpko",      "Tvrtko",   "Valentin",
     "Vanja",    "Vatroslav", "Vedran",  "Velimir",  "Vijeko",    "Vjekoslav", "Vjenceslav", "Vladek",   "Vladislav",
     "Vlado",    "Vlatko",    "Zdenko",  "Zdravko",  "Željko",    "Zoltan",    "Zrinislav",  "Zrinko",   "Zvonimir"});

const auto croatianFemaleFirstNames = std::to_array<std::string_view>(
    {"Ada",      "Adina",    "Adriana",   "Adrijana",  "Agata",     "Alemka",    "Alenka",     "Alisa",
     "Alka",     "Alma",     "Amalija",   "Ana",       "Ančica",    "Anamarija", "Anastazija", "Anci",
     "Anči",     "Ancica",   "Anda",      "Anđela",    "Anđelika",  "Anđelina",  "Andi",       "Andrea",
     "Andreja",  "Andrina",  "Anesa",     "Aneta",     "Ani",       "Anica",     "Anja",       "Anka",
     "Antea",    "Antonija", "Asija",     "Asja",      "Bara",      "Barbara",   "Barica",     "Beata",
     "Bela",     "Biserka",  "Borka",     "Borna",     "Božena",    "Božica",    "Branimira",  "Branka",
     "Brankica", "Brigita",  "Dada",      "Dagmar",    "Dalibora",  "Daliborka", "Danica",     "Danijela",
     "Danka",    "Dara",     "Darinka",   "Dea",       "Đina",      "Dona",      "Donka",      "Dora",
     "Dorana",   "Dorica",   "Draga",     "Ela",       "Eli",       "Elka",      "Evelin",     "Ina",
     "Inga",     "Inge",     "Inja",      "Inka",      "Ita",       "Iva",       "Ivančica",   "Ivanka",
     "Ivica",    "Ivna",     "Jagica",    "Jana",      "Janica",    "Janka",     "Jasna",      "Jela",
     "Jelica",   "Ješa",     "Kata",      "Kate",      "Katica",    "Katija",    "Katina",     "Kika",
     "Lara",     "Laura",    "Lea",       "Lela",      "Lena",      "Leona",     "Lola",       "Maca",
     "Magda",    "Maja",     "Majda",     "Manda",     "Mara",      "Mare",      "Marena",     "Marica",
     "Marta",    "Maša",     "Mica",      "Mika",      "Mila",      "Milica",    "Milka",      "Mima",
     "Mina",     "Mira",     "Mirka",     "Naja",      "Nana",      "Natalija",  "Nataša",     "Natka",
     "Neda",     "Nela",     "Neli",      "Nena",      "Nera",      "Nesa",      "Neva",       "Nevenka",
     "Nina",     "Nuša",     "Paulina",   "Pava",      "Pave",      "Pavica",    "Rajna",      "Rija",
     "Rina",     "Ružica",   "Staša",     "Tasa",      "Teja",      "Tija",      "Tijana",     "Tina",
     "Tončica",  "Tonija",   "Tonka",     "Vala",      "Valera",    "Vali",      "Vela",       "Velana",
     "Velina",   "Vera",     "Verica",    "Vinka",     "Zlatica",   "Zlatka",    "Darija",     "Darina",
     "Divna",    "Donata",   "Doroteja",  "Dragica",   "Dubravka",  "Dunja",     "Đurda",      "Edita",
     "Elena",    "Ema",      "Eva",       "Gabrijela", "Goranka",   "Gordana",   "Grozdana",   "Helena",
     "Hrvojka",  "Ira",      "Irena",     "Irma",      "Ivana",     "Jadranka",  "Jaga",       "Jasmina",
     "Josipa",   "Karmela",  "Katarina",  "Kristina",  "Leda",      "Leonida",   "Lidija",     "Ljiljana",
     "Ljubica",  "Lovorka",  "Lucija",    "Magdalena", "Marija",    "Marijana",  "Marina",     "Martina",
     "Miranda",  "Mirna",    "Monika",    "Nada",      "Nadalina",  "Nedjeljka", "Nevena",     "Nora",
     "Paula",    "Petra",    "Renata",    "Romana",    "Rosana",    "Ruža",      "Sanja",      "Silvija",
     "Slavica",  "Snježana", "Tereza",    "Tomislava", "Valentina", "Valerija",  "Vedrana",    "Velimira",
     "Veronika", "Vesna",    "Vladimira", "Zdenka",    "Zdeslava",  "Zlata"});

const auto croatianLastNames = std::to_array<std::string_view>({
    "Abadžić",
    "Abadžija",
    "Abazi",
    "Alija",
    "Alikavazović",
    "Alili",
    "Alilović",
    "Dardalić",
    "Dardić",
    "Darić",
    "Darojković",
    "Daskalov",
    "Daskalova",
    "Dasović",
    "Dašić",
    "Gulam",
    "Gulam Kramarić",
    "Gulaš",
    "Guldan",
    "Gulija",
    "Guljaš",
    "Guljašević",
    "Gumbarević",
    "Gumbas",
    "Guncinam",
    "Gunjača",
    "Gusak",
    "Gushani",
    "Gušani",
    "Gutal",
    "Gutrovac",
    "Guzina",
    "Hrdelja",
    "Hrdlitschka",
    "Hrđavec",
    "Hrenovac",
    "HrešićDubaić",
    "Hrga",
    "Hrgota",
    "Hribar",
    "Hrkač",
    "Hrkać",
    "Hrkalović",
    "Hrnjak",
    "Hrnjkaš",
    "Jahaj",
    "Jahić",
    "Jahn",
    "Jajalo",
    "Jajčan",
    "Jajčanin",
    "Jajčinović",
    "Jajetić",
    "Jajić",
    "Jajiš",
    "Jakab",
    "Jakabfi",
    "Jakarić",
    "Jakelić",
    "JakelićŠklebek",
    "Jakeljić",
    "Jakešević",
    "Jakić",
    "Kuzmanić",
    "KuzmanićŠarac",
    "Kuzmanoski",
    "Luketina",
    "LukićGaličić",
    "LukićKovanović",
    "Mušanović",
    "Mušija",
    "Mušnjak",
    "Muštran",
    "Rumštajn",
    "Runac",
    "Runjak",
    "Rupaner",
    "Rupar",
    "Sabolek Tegzeš",
    "Zlatar",
    "Zlatarek",
    "Zlatarević",
    "Zlatarić",
    "Zlatić",
    "ZlatićKavgić",
    "Zlatković",
    "Zlatović",
    "Zlatunić",
    "Zlokapa",
    "Zlokolica",
    "Zlosa",
    "Zlovarić",
    "Zmaić",
    "Zmajić",
    "Zmajšek",
    "Znaor",
    "Znika",
    "Zobenica",
    "Zobundžija",
    "Zolaj",
    "Zolaji",
    "Zolota",
    "Zoraj",
    "Zoraja",
    "Zorbas",
    "Zorbaz",
    "Zorica",
    "Zorman",
    "Zoroja",
    "Zovak",
    "Zovo Susak",
    "Zrakić",
    "Zrinščak",
    "Zrmanjac",
    "Zrna",
    "Zrno Dimnjašević",
    "Zuanović",
    "Zubac",
    "Zubac Ištuk",
    "Zubaj",
    "Zubak",
    "Zuban",
    "Zubonja",
    "Zukan",
    "Zulbari",
    "Zundanović",
    "Zurak",
    "Zurapović",
    "Zuščak",
    "Zušćak",
    "Zuzjak",
    "Zuzjak Novak",
    "Zvanić",
    "Zvardon",
    "Zvekan",
    "Zvekanov",
    "Zvekanović",
    "Zveržina",
    "Zvocak",
    "Zvonar",
    "Zvonarek",
    "Zvonarević",
    "Zvonarić",
    "Žabarović",
    "Žabčić",
    "Žabić",
    "Žaček",
    "Žada",
    "Žadanji",
    "Žagač",
    "Žagar",
    "Žagarac",
    "Žagarova",
    "Žago",
    "Žaja",
    "Žajdlik",
    "Žajlik",
    "Žaki",
    "Žaknić",
    "Žakula",
    "Žalac",
    "Žalako",
    "Žalik",
    "Žalonkin",
    "Žampak",
    "Žanić",
    "Žanija",
    "Žanko",
    "Žaper",
    "Žaper Radičević",
    "Žarak",
    "Žarić",
    "Žarko",
    "Žarković",
    "ŽarkovićMaršal",
    "Ždrale",
    "Žeđa",
    "Žegarac",
    "Žegura Schonberger",
    "Želka",
    "Željeznak",
    "Željeznjak",
    "Žemla",
    "Žerav",
    "Žeravica",
    "Žeravica Lang",
    "Žerjav",
    "Žganec",
    "Žganjar",
    "Žganjer",
    "Žgela",
    "Žgrabljić",
    "Žibrat",
    "Židar",
    "Žido Lazarević",
    "Žigman",
    "Žiha",
    "Žilavec",
    "Žiljak",
    "Žindarec",
    "Žipac",
    "Žiraj",
    "ŽirošTatalović",
    "Žiška",
    "Žitnjak",
    "Živadinović",
    "Živaković",
    "ŽivakovićKerže",
    "Živalj",
    "Živaljić",
});

const auto croatianSuffixes = std::to_array<std::string_view>({"dipl.oecc", "dipl.ing"});

const auto croatianMalePrefixes = std::to_array<std::string_view>({"g.", "dr."});

const auto croatianFemalePrefixes = std::to_array<std::string_view>({"gđa.", "gđa", "dr."});

const NameFormats croatianNameFormats{
    {{"{firstName} {lastName}", 8}, {"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames croatianPeopleNames{
    {croatianMaleFirstNames, croatianLastNames, croatianMalePrefixes, croatianSuffixes},
    {croatianFemaleFirstNames, croatianLastNames, croatianFemalePrefixes, croatianSuffixes},
    croatianNameFormats};

// Czechia

const auto czechMaleFirstNames = std::to_array<std::string_view>(
    {"Abadon",     "Abdon",      "Ábel",       "Abelard",    "Abraham",   "Abrahám",    "Absolon",   "Absolón",
     "Adalbert",   "Adam",       "Adin",       "Adolf",      "Adrian",    "Adrián",     "Agaton",    "Achil",
     "Achiles",    "Alan",       "Alban",      "Albert",     "Albín",     "Albrecht",   "Aldo",      "Alen",
     "Aleš",       "Alexandr",   "Alexej",     "Alfons",     "Alfréd",    "Alois",      "Alojz",     "Alva",
     "Alvar",      "Alvin",      "Amadeus",    "Amand",      "Amát",      "Ambrož",     "Amos",      "Ámos",
     "Anastáz",    "Anatol",     "Anděl",      "Andělín",    "Andrej",    "Anselm",     "Antal",     "Antonín",
     "Aram",       "Ariel",      "Aristid",    "Arkád",      "Armand",    "Armin",      "Arne",      "Arnold",
     "Arnošt",     "Áron",       "Árón",       "Arpád",      "Arsen",     "Artur",      "Artuš",     "Arzen",
     "Atanas",     "Atanáš",     "Atila",      "August",     "Augustin",  "Augustýn",   "Aurel",     "Aurelián",
     "Axel",       "Baltazar",   "Barnabáš",   "Bartoloměj", "Basil",     "Bazil",      "Beatus",    "Bedřich",
     "Benedikt",   "Benjamin",   "Benjamín",   "Bernard",    "Bertold",   "Bertram",    "Bivoj",     "Blahomil",
     "Blahomír",   "Blahoslav",  "Blažej",     "Bohdan",     "Bohuchval", "Bohumil",    "Bohumír",   "Bohun",
     "Bohuslav",   "Bohuš",      "Bojan",      "Bolemír",    "Boleslav",  "Bonifác",    "Borek",     "Boris",
     "Borislav",   "Bořek",      "Bořislav",   "Bořivoj",    "Božetěch",  "Božidar",    "Božislav",  "Branimír",
     "Branislav",  "Bratislav",  "Bret",       "Brian",      "Brit",      "Bronislav",  "Bruno",     "Břetislav",
     "Budimír",    "Budislav",   "Budivoj",    "Cecil",      "Cedrik",    "Celestin",   "Celestýn",  "César",
     "Cézar",      "Ctibor",     "Ctirad",     "Ctislav",    "Cyprián",   "Cyril",      "Čeněk",     "Čestmír",
     "Čistoslav",  "Dag",        "Dalibor",    "Dalimil",    "Dalimír",   "Damián",     "Dan",       "Daniel",
     "Darek",      "Darius",     "David",      "Denis",      "Děpold",    "Dětmar",     "Dětřich",   "Dezider",
     "Dimitrij",   "Dino",       "Dionýz",     "Dionýzos",   "Diviš",     "Dluhoš",     "Dobromil",  "Dobromír",
     "Dobroslav",  "Dominik",    "Donald",     "Donát",      "Dorian",    "Dorián",     "Drahomil",  "Drahomír",
     "Drahoň",     "Drahoslav",  "Drahoš",     "Drahotín",   "Drahutin",  "Dušan",      "Edgar",     "Edmond",
     "Edmund",     "Eduard",     "Edvard",     "Edvin",      "Edvín",     "Egmont",     "Egon",      "Eliáš",
     "Elizej",     "Elizeus",    "Elmar",      "Elvis",      "Emanuel",   "Emerich",    "Emil",      "Emilián",
     "Engelbert",  "Erazim",     "Erazmus",    "Erhard",     "Erich",     "Erik",       "Ernest",    "Ernst",
     "Ervín",      "Eugen",      "Eusebius",   "Evald",      "Evan",      "Evarist",    "Evžen",     "Ezechiel",
     "Ezra",       "Fabián",     "Faust",      "Faustin",    "Faustýn",   "Fedor",      "Felicián",  "Felix",
     "Ferdinand",  "Fidel",      "Fidelius",   "Filemon",    "Filibert",  "Filip",      "Filomen",   "Flavián",
     "Flavius",    "Florentin",  "Florentýn",  "Florián",    "Fortunát",  "Fráňa",      "Franc",     "František",
     "Fridolín",   "Gabin",      "Gabriel",    "Gál",        "Garik",     "Gaston",     "Gedeon",    "Gejza",
     "Genadij",    "Gerald",     "Gerard",     "Gerazim",    "Gerhard",   "Géza",       "Gilbert",   "Gleb",
     "Glen",       "Gorazd",     "Gordon",     "Gothard",    "Gracián",   "Grant",      "Gunter",    "Gűnter",
     "Gustav",     "Hanuš",      "Harald",     "Harold",     "Haštal",    "Havel",      "Helmut",    "Herbert",
     "Herman",     "Heřman",     "Hilar",      "Hilarius",   "Hjalmar",   "Homér",      "Honor",     "Honorius",
     "Horác",      "Horst",      "Horymír",    "Hostimil",   "Hostimír",  "Hostislav",  "Hostivít",  "Hovard",
     "Hubert",     "Hugo",       "Hvězdoslav", "Hyacint",    "Hynek",     "Hypolit",    "Chrabroš",  "Chraniboj",
     "Chranibor",  "Chranislav", "Chrudoš",    "Chval",      "Ignác",     "Ignát",      "Igor",      "Ilja",
     "Inocenc",    "Irenej",     "Ireneus",    "Irvin",      "Isidor",    "Ivan",       "Ivar",      "Ivo",
     "Ivor",       "Izaiáš",     "Izák",       "Izidor",     "Izmael",    "Jacek",      "Jáchym",    "Jakub",
     "Jan",        "Jarmil",     "Jarolím",    "Jaromil",    "Jaromír",   "Jaroslav",   "Jason",     "Jasoň",
     "Jeremiáš",   "Jeroným",    "Jiljí",      "Jimram",     "Jindřich",  "Jiří",       "Job",       "Joel",
     "Jonáš",      "Jonatan",    "Jonathan",   "Jordan",     "Josef",     "Jošt",       "Jozef",     "Jozue",
     "Juda",       "Julián",     "Julius",     "Justin",     "Justýn",    "Kajetán",    "Kamil",     "Karel",
     "Kasián",     "Kastor",     "Kašpar",     "Kazimír",    "Kilián",    "Kim",        "Klaudián",  "Klaudius",
     "Klement",    "Kliment",    "Knut",       "Koloman",    "Kolombín",  "Kolumbán",   "Kolumbín",  "Konrád",
     "Konstantin", "Konstantýn", "Kornel",     "Kornelius",  "Kosma",     "Kosmas",     "Krasomil",  "Krasoslav",
     "Kristián",   "Kryšpín",    "Kryštof",    "Křesomysl",  "Křišťan",   "Kurt",       "Květoň",    "Květoslav",
     "Květoš",     "Kvido",      "Ladislav",   "Lambert",    "Lars",      "Laurenc",    "Lazar",     "Leander",
     "Leandr",     "Leo",        "Leodegar",   "Leon",       "Leonard",   "Leonid",     "Leontýn",   "Leopold",
     "Leoš",       "Lešek",      "Lev",        "Libor",      "Liboslav",  "Lionel",     "Livius",    "Lorenc",
     "Lotar",      "Lothar",     "Lubomír",    "Lubor",      "Luboslav",  "Luboš",      "Lucián",    "Lucius",
     "Luděk",      "Ludivoj",    "Ludomír",    "Ludoslav",   "Ludvík",    "Lukáš",      "Lukrecius", "Lumír",
     "Lutibor",    "Lutobor",    "Magnus",     "Makar",      "Manfred",   "Manfréd",    "Mansvet",   "Manuel",
     "Marcel",     "Marek",      "Marian",     "Marián",     "Marin",     "Mario",      "Marius",    "Martin",
     "Matěj",      "Matouš",     "Matyáš",     "Max",        "Maxim",     "Maximilián", "Maxmilián", "Mečislav",
     "Medard",     "Melichar",   "Merlin",     "Mervin",     "Metod",     "Metoděj",    "Michael",   "Michal",
     "Mikoláš",    "Mikuláš",    "Milan",      "Milíč",      "Milík",     "Milivoj",    "Miloň",     "Milorad",
     "Miloslav",   "Miloš",      "Milota",     "Milouš",     "Milovan",   "Milovín",    "Milutín",   "Mirek",
     "Mirko",      "Miromil",    "Miron",      "Miroslav",   "Mirtil",    "Mlad",       "Mladen",    "Mnata",
     "Mnislav",    "Modest",     "Mojmír",     "Mojžíš",     "Morgan",    "Moric",      "Moris",     "Mořic",
     "Mstislav",   "Myron",      "Myrtil",     "Napoleon",   "Narcis",    "Natan",      "Natanael",  "Nathan",
     "Nathanael",  "Něhoslav",   "Neklan",     "Nepomuk",    "Nezamysl",  "Nikita",     "Nikodém",   "Nikola",
     "Nikolas",    "Norbert",    "Norman",     "Odolen",     "Odon",      "Oktavián",   "Oktavius",  "Olaf",
     "Olbram",     "Oldřich",    "Oleg",       "Oliver",     "Omar",      "Ondřej",     "Orest",     "Oskar",
     "Osvald",     "Ota",        "Otakar",     "Otmar",      "Oto",       "Otokar",     "Otomar",    "Ovidius",
     "Palmiro",    "Pankrác",    "Pantaleon",  "Paris",      "Parsival",  "Paskal",     "Patrik",    "Pavel",
     "Pavlín",     "Pelhřim",    "Perikles",   "Petr",       "Petronius", "Pius",       "Platon",    "Platón",
     "Polykarp",   "Pravdomil",  "Pravomil",   "Prokop",     "Prosper",   "Přemysl",    "Přibyslav", "Radan",
     "Radegast",   "Radek",      "Radhost",    "Radim",      "Radimír",   "Radislav",   "Radivoj",   "Radko",
     "Radmil",     "Radomil",    "Radomír",    "Radoslav",   "Radoš",     "Radovan",    "Radúz",     "Radvan",
     "Rafael",     "Raimund",    "Rainald",    "Rainer",     "Rainhard",  "Rainold",    "Rajko",     "Ralf",
     "Ramon",      "Randolf",    "Ranek",      "Ranko",      "Rastislav", "Ratibor",    "Ratmír",    "Redmond",
     "Reginald",   "Remig",      "Remus",      "Renát",      "René",      "Richard",    "Robert",    "Robin",
     "Robinson",   "Rodan",      "Roderik",    "Rodrigo",    "Roger",     "Roch",       "Roland",    "Rolf",
     "Roman",      "Romeo",      "Romuald",    "Romul",      "Romulus",   "Ronald",     "Rostislav", "Ruben",
     "Rudolf",     "Rufus",      "Rupert",     "Ruprecht",   "Ruslan",    "Řehoř",      "Sába",      "Sámo",
     "Samson",     "Samuel",     "Saturnin",   "Saul",       "Sáva",      "Sebastian",  "Sebastián", "Sebestian",
     "Sedrik",     "Serafín",    "Serenus",    "Sergej",     "Servác",    "Severín",    "Sidon",     "Sigfríd",
     "Silvan",     "Silván",     "Silvestr",   "Silvius",    "Simeon",    "Simon",      "Sinkler",   "Sixt",
     "Sixtus",     "Slávek",     "Slaviboj",   "Slavibor",   "Slavoboj",  "Slavoj",     "Slavomil",  "Slavomír",
     "Smil",       "Soběslav",   "Sokrat",     "Soter",      "Spytihněv", "Stanimír",   "Stanislav", "Stojan",
     "Stojmír",    "Svatoboj",   "Svatobor",   "Svatomír",   "Svatopluk", "Svatoslav",  "Sven",      "Svetozar",
     "Šalamoun",   "Šalomoun",   "Šavel",      "Šebastián",  "Šimon",     "Šťasta",     "Štefan",    "Štěpán",
     "Tadeáš",     "Tankred",    "Taras",      "Teobald",    "Teodor",    "Teodorik",   "Teodoz",    "Teofan",
     "Teofil",     "Terenc",     "Terencius",  "Theobald",   "Theodor",   "Theodorik",  "Theofan",   "Theofil",
     "Tiber",      "Tiberius",   "Tibor",      "Tiburcius",  "Tichomil",  "Tichomír",   "Tichon",    "Timon",
     "Timotej",    "Timoteus",   "Timur",      "Titus",      "Tobiáš",    "Tomáš",      "Tomislav",  "Tor",
     "Torkvát",    "Torsten",    "Tristan",    "Udo",        "Ulrich",    "Upton",      "Urban",     "Uve",
     "Václav",     "Vadim",      "Valdemar",   "Valentin",   "Valentýn",  "Valerián",   "Valter",    "Valtr",
     "Vasil",      "Vavřinec",   "Veleslav",   "Velimír",    "Velislav",  "Věnceslav",  "Vendelín",  "Věnek",
     "Verner",     "Věroslav",   "Vidor",      "Viktor",     "Viktorin",  "Viktorín",   "Vilém",     "Vilibald",
     "Vilmar",     "Vincenc",    "Virgil",     "Virgin",     "Vít",       "Vítězslav",  "Vitold",    "Vítoslav",
     "Vivian",     "Vladan",     "Vladimír",   "Vladislav",  "Vladivoj",  "Vlastimil",  "Vlastimír", "Vlastislav",
     "Vlk",        "Vojen",      "Vojmil",     "Vojmír",     "Vojslav",   "Vojtěch",    "Vok",       "Volfgang",
     "Vratislav",  "Vsevolod",   "Všeboj",     "Všebor",     "Všerad",    "Všeslav",    "Xaver",     "Xaverius",
     "Záboj",      "Zachar",     "Zachariáš",  "Záviš",      "Zbislav",   "Zbyhněv",    "Zbyněk",    "Zbyslav",
     "Zbyšek",     "Zdeněk",     "Zderad",     "Zdeslav",    "Zdík",      "Zdirad",     "Zdislav",   "Zeno",
     "Zenon",      "Zikmund",    "Zlatan",     "Zlatko",     "Zlatomír",  "Zoltán",     "Zoran",     "Zoroslav",
     "Zosim",      "Zvonimír",   "Žarko",      "Ždan",       "Želibor",   "Želimír",    "Želislav",  "Želmír",
     "Žitomír",    "Žitoslav",   "Živan"});

const auto czechFemaleFirstNames = std::to_array<std::string_view>(
    {"Abigail",    "Ada",         "Adalberta",  "Adéla",      "Adelaida",    "Adina",       "Adolfa",     "Adolfína",
     "Adriana",    "Adriána",     "Adriena",    "Afra",       "Agáta",       "Aglaja",      "Aida",       "Alana",
     "Albena",     "Alberta",     "Albertina",  "Albertýna",  "Albína",      "Alena",       "Aleška",     "Alexandra",
     "Alfréda",    "Alice",       "Alida",      "Alina",      "Alma",        "Aloisie",     "Alojzije",   "Alžběta",
     "Amálie",     "Amanda",      "Amáta",      "Amélie",     "Anabela",     "Anastázie",   "Anatázie",   "Anatolie",
     "Anatólie",   "Anděla",      "Andělína",   "Andrea",     "Aneta",       "Anežka",      "Angela",     "Angelika",
     "Anita",      "Anna",        "Anselma",    "Antonie",    "Apolena",     "Arabela",     "Aranka",     "Areta",
     "Ariadna",    "Ariana",      "Ariela",     "Arleta",     "Armida",      "Arna",        "Arnolda",    "Arnoštka",
     "Astrid",     "Astrida",     "Atanázie",   "Augusta",    "Augustina",   "Augustýna",   "Aura",       "Aurélie",
     "Aurora",     "Babeta",      "Barbara",    "Barbora",    "Beáta",       "Beatrice",    "Bedřiška",   "Bela",
     "Běla",       "Belinda",     "Benedikta",  "Berenika",   "Berit",       "Bernarda",    "Berta",      "Bertolda",
     "Bianka",     "Bibiana",     "Birgit",     "Birgita",    "Blahomila",   "Blahomíra",   "Blahoslava", "Blanka",
     "Blažena",    "Bohdana",     "Bohumila",   "Bohumíra",   "Bohuna",      "Bohuslava",   "Bohuše",     "Bojana",
     "Bojislava",  "Boleslava",   "Borislava",  "Bořislava",  "Božena",      "Božetěcha",   "Božidara",   "Branimíra",
     "Branislava", "Bratislava",  "Brenda",     "Brigita",    "Brita",       "Bronislava",  "Bruna",      "Brunhilda",
     "Břetislava", "Cecilie",     "Cecílie",    "Celestina",  "Celestýna",   "Celie",       "Celina",     "Ctibora",
     "Ctirada",    "Ctislava",    "Cyntie",     "Cyrila",     "Čeňka",       "Čestmíra",    "Čistoslava", "Dagmar",
     "Dagmara",    "Dalibora",    "Dalida",     "Dalie",      "Dalila",      "Dalimila",    "Dalimíra",   "Damaris",
     "Damiana",    "Damiána",     "Dana",       "Danica",     "Daniela",     "Danuše",      "Danuta",     "Daria",
     "Darie",      "Darina",      "Darja",      "Davida",     "Debora",      "Delie",       "Denisa",     "Diana",
     "Dina",       "Dita",        "Diviška",    "Dobrava",    "Dobromila",   "Dobromíra",   "Dobroslava", "Dominika",
     "Donalda",    "Donáta",      "Dora",       "Doris",      "Dorota",      "Doubrava",    "Doubravka",  "Drahomila",
     "Drahomíra",  "Drahoslava",  "Drahotína",  "Drahuše",    "Dulcinea",    "Dušana",      "Edita",      "Eduarda",
     "Edvarda",    "Egona",       "Ela",        "Elektra",    "Elena",       "Eleonora",    "Elfrída",    "Eliška",
     "Elsa",       "Elvíra",      "Elza",       "Ema",        "Emanuela",    "Emilie",      "Emílie",     "Erika",
     "Erna",       "Ervína",      "Estela",     "Ester",      "Estera",      "Etela",       "Eufrozina",  "Eufrozína",
     "Eugenie",    "Eulálie",     "Eunika",     "Eusebie",    "Eva",         "Evelina",     "Evelína",    "Evženie",
     "Fabiána",    "Fabie",       "Fatima",     "Faustina",   "Faustýna",    "Féba",        "Fedora",     "Felicie",
     "Felície",    "Felicita",    "Ferdinanda", "Fidelie",    "Filipa",      "Filoména",    "Flavie",     "Flora",
     "Flóra",      "Florentina",  "Florentýna", "Františka",  "Frída",       "Gabriela",    "Gaja",       "Gajana",
     "Galina",     "Garika",      "Gema",       "Geralda",    "Geraldina",   "Gerarda",     "Gerardina",  "Gerda",
     "Gerharda",   "Gertruda",    "Gilberta",   "Gina",       "Gisela",      "Gita",        "Gizela",     "Glorie",
     "Gordana",    "Graciána",    "Gracie",     "Grácie",     "Gražina",     "Gréta",       "Griselda",   "Grizelda",
     "Gudrun",     "Gustava",     "Gvendolina", "Gvendolína", "Halina",      "Hana",        "Háta",       "Havla",
     "Heda",       "Hedvika",     "Heidrun",    "Helena",     "Helga",       "Herberta",    "Hermína",    "Herta",
     "Hilda",      "Hortensie",   "Hortenzie",  "Horymíra",   "Hostimila",   "Hostimíra",   "Hostislava", "Hvězdoslava",
     "Hyacinta",   "Chranislava", "Iboja",      "Ida",        "Ignácie",     "Ignáta",      "Ildika",     "Iljana",
     "Ilona",      "Ilsa",        "Ilza",       "Ines",       "Inesa",       "Inéz",        "Ingeborg",   "Ingeborga",
     "Ingrid",     "Ingrida",     "Inka",       "Irena",      "Iris",        "Irma",        "Isabela",    "Isidora",
     "Isolda",     "Iva",         "Ivana",      "Iveta",      "Ivona",       "Izabela",     "Izidora",    "Izolda",
     "Jadrana",    "Jadranka",    "Jakuba",     "Jakubka",    "Jana",        "Jarmila",     "Jarolíma",   "Jaromíra",
     "Jaroslava",  "Jasmína",     "Jasna",      "Jasněna",    "Jelena",      "Jenovéfa",    "Jesika",     "Jindra",
     "Jindřiška",  "Jiřina",      "Jitka",      "Johana",     "Jolana",      "Jolanta",     "Jordana",    "Jorga",
     "Josefa",     "Josefína",    "Jovana",     "Jozefa",     "Jozefína",    "Judita",      "Juliana",    "Juliána",
     "Julie",      "Justina",     "Justýna",    "Juta",       "Kamila",      "Karin",       "Karina",     "Karla",
     "Karmela",    "Karmen",      "Karolina",   "Karolína",   "Kateřina",    "Katrin",      "Katrina",    "Kazi",
     "Kazimíra",   "Kira",        "Klára",      "Klaudie",    "Klementina",  "Klementýna",  "Kleopatra",  "Klotylda",
     "Koleta",     "Kolombína",   "Kolumbína",  "Konstance",  "Konstancie",  "Konsuela",    "Konzuela",   "Kora",
     "Kordula",    "Korina",      "Kornélie",   "Krasava",    "Krasomila",   "Kristina",    "Kristýna",   "Kunhuta",
     "Květa",      "Květoslava",  "Květuše",    "Lada",       "Ladislava",   "Larisa",      "Laura",      "Laurencie",
     "Lea",        "Léda",        "Leila",      "Lejla",      "Lena",        "Lenka",       "Leokádie",   "Leona",
     "Leonora",    "Leontina",    "Leontýna",   "Leopolda",   "Leopoldina",  "Leopoldýna",  "Leticie",    "Lia",
     "Liana",      "Liběna",      "Libora",     "Liboslava",  "Libuše",      "Lidmila",     "Liliana",    "Lina",
     "Linda",      "Livie",       "Ljuba",      "Lola",       "Loreta",      "Lorna",       "Lota",       "Lubomíra",
     "Luboslava",  "Luciána",     "Lucie",      "Ludiše",     "Luďka",       "Ludmila",     "Ludomíra",   "Ludoslava",
     "Ludvika",    "Ludvíka",     "Luisa",      "Lujza",      "Lukrécie",    "Lumíra",      "Lydie",      "Lýdie",
     "Mabel",      "Mabela",      "Magda",      "Magdalena",  "Magdaléna",   "Mahulena",    "Maja",       "Mája",
     "Malvína",    "Manon",       "Manona",     "Manuela",    "Marcela",     "Marcelína",   "Margit",     "Margita",
     "Mariana",    "Marie",       "Marieta",    "Marika",     "Marilyn",     "Marina",      "Mariola",    "Marion",
     "Marisa",     "Marita",      "Markéta",    "Marlena",    "Marta",       "Martina",     "Matylda",    "Maud",
     "Maxima",     "Mečislava",   "Medea",      "Médea",      "Melánie",     "Melinda",     "Melisa",     "Melita",
     "Mercedes",   "Michaela",    "Michala",    "Milada",     "Milana",      "Milena",      "Miloslava",  "Milred",
     "Miluše",     "Mína",        "Mira",       "Mirabela",   "Miranda",     "Mirela",      "Miriam",     "Mirjam",
     "Mirka",      "Miromila",    "Miroslava",  "Mnislava",   "Mona",        "Monika",      "Muriel",     "Muriela",
     "Myrna",      "Naďa",        "Naděžda",    "Naneta",     "Narcisa",     "Natalie",     "Natálie",    "Nataša",
     "Neda",       "Nela",        "Nevena",     "Nika",       "Niké",        "Nikodéma",    "Nikol",      "Nikola",
     "Nila",       "Nina",        "Noema",      "Noemi",      "Nona",        "Nora",        "Norberta",   "Norma",
     "Odeta",      "Ofélie",      "Oktavie",    "Oktávie",    "Oldřiška",    "Olga",        "Oliva",      "Olivie",
     "Olympie",    "Ondřejka",    "Otakara",    "Otilie",     "Otýlie",      "Oxana",       "Palmira",    "Pamela",
     "Paskala",    "Patricie",    "Pavla",      "Pavlína",    "Pelagie",     "Penelopa",    "Perla",      "Persida",
     "Perzida",    "Petra",       "Petrana",    "Petronela",  "Petronila",   "Petruše",     "Petula",     "Pilar",
     "Polyxena",   "Pravdomila",  "Pravomila",  "Pravoslav",  "Pravoslava",  "Priscila",    "Priska",     "Prokopa",
     "Přibyslava", "Radana",      "Radimíra",   "Radislava",  "Radka",       "Radmila",     "Radomila",   "Radomíra",
     "Radoslava",  "Radovana",    "Radslava",   "Rafaela",    "Ráchel",      "Raisa",       "Rajsa",      "Ramona",
     "Rastislava", "Rebeka",      "Regina",     "Regína",     "Renata",      "Renáta",      "René",       "Ria",
     "Riana",      "Richarda",    "Rina",       "Rita",       "Roberta",     "Robina",      "Romana",     "Rosa",
     "Rosalinda",  "Rosamunda",   "Rosana",     "Rostislava", "Rovena",      "Roxana",      "Róza",       "Rozálie",
     "Rozalinda",  "Rozamunda",   "Rozana",     "Rozina",     "Rozita",      "Rozvita",     "Rudolfa",    "Rudolfina",
     "Rudolfína",  "Rut",         "Rút",        "Růžena",     "Řehořka",     "Sabina",      "Sabrina",    "Salomea",
     "Salomena",   "Samuela",     "Sandra",     "Sára",       "Saskia",      "Saskie",      "Saxona",     "Selena",
     "Selma",      "Senta",       "Serafína",   "Serena",     "Scholastika", "Sibyla",      "Sidonie",    "Silvána",
     "Silvie",     "Simeona",     "Simona",     "Skarlet",    "Skarleta",    "Slavěna",     "Slávka",     "Slavomila",
     "Slavomíra",  "Soběslava",   "Sofie",      "Sofronie",   "Solveig",     "Solveiga",    "Soňa",       "Sotira",
     "Stanislava", "Stáza",       "Stela",      "Svatava",    "Svatoslava",  "Světla",      "Světlana",   "Světluše",
     "Sylva",      "Sylvie",      "Šárka",      "Šarlota",    "Šimona",      "Štěpána",     "Štěpánka",   "Tamara",
     "Táňa",       "Taťána",      "Tea",        "Tekla",      "Teodora",     "Teodozie",    "Teofila",    "Tereza",
     "Terezie",    "Thea",        "Theodora",   "Theodosie",  "Theofila",    "Tomáška",     "Toska",      "Ulrika",
     "Una",        "Uršula",      "Václava",    "Valburga",   "Valdemara",   "Valentina",   "Valentýna",  "Valerie",
     "Valérie",    "Vanda",       "Vanesa",     "Věduna",     "Veleslava",   "Velislava",   "Věnceslava", "Vendelína",
     "Vendula",    "Vendulka",    "Věnka",      "Venuše",     "Věra",        "Verona",      "Veronika",   "Věroslava",
     "Věslava",    "Vesna",       "Viktorie",   "Viléma",     "Vilemína",    "Vilma",       "Vincencie",  "Viola",
     "Violeta",    "Virginie",    "Virgínie",   "Víta",       "Vítězslava",  "Viviana",     "Vladana",    "Vladěna",
     "Vladimíra",  "Vladislava",  "Vlasta",     "Vlastimila", "Vlastimíra",  "Vlastislava", "Vojmíra",    "Vojslava",
     "Vojtěška",   "Voršila",     "Vratislava", "Xaverie",    "Xenie",       "Zaida",       "Zaira",      "Zbyhněva",
     "Zbyňka",     "Zbyslava",    "Zbyška",     "Zdena",      "Zdenka",      "Zdeňka",      "Zdeslava",   "Zdislava",
     "Zenobie",    "Zina",        "Zinaida",    "Zita",       "Zlata",       "Zlatomíra",   "Zlatuše",    "Zoe",
     "Zoja",       "Zora",        "Zoroslava",  "Zuzana",     "Zvonimíra",   "Žakelina",    "Žakelína",   "Žaneta",
     "Ždana",      "Želimíra",    "Želislava",  "Želmíra",    "Žitomíra",    "Žitoslava",   "Živa",       "Živana",
     "Žofie"});

const auto czechMaleLastNames = std::to_array<std::string_view>(
    {"Adam",      "Adamec",   "Adámek",    "Albrecht",  "Ambrož",    "Anděl",      "Andrle",     "Antoš",
     "Bajer",     "Baláž",    "Balcar",    "Balog",     "Baloun",    "Barák",      "Baran",      "Bareš",
     "Bárta",     "Barták",   "Bartoň",    "Bartoš",    "Bartošek",  "Bartůněk",   "Bašta",      "Bauer",
     "Bayer",     "Bažant",   "Bečka",     "Bečvář",    "Bednář",    "Bednařík",   "Bělohlávek", "Benda",
     "Beneš",     "Beran",    "Beránek",   "Berger",    "Berka",     "Berky",      "Bernard",    "Bezděk",
     "Bílek",     "Bílý",     "Bína",      "Bittner",   "Blaha",     "Bláha",      "Blažek",     "Blecha",
     "Bobek",     "Boček",    "Boháč",     "Boháček",   "Böhm",      "Borovička",  "Bouček",     "Bouda",
     "Bouška",    "Brabec",   "Brabenec",  "Brada",     "Bradáč",    "Braun",      "Brázda",     "Brázdil",
     "Brejcha",   "Březina",  "Bříza",     "Brož",      "Brožek",    "Brychta",    "Bubeník",    "Buček",
     "Buchta",    "Burda",    "Bureš",     "Burian",    "Buriánek",  "Byrtus",     "Čada",       "Caha",
     "Čáp",       "Čapek",    "Čech",      "Čejka",     "Čermák",    "Černík",     "Černoch",    "Černohorský",
     "Černý",     "Červeňák", "Červenka",  "Červený",   "Červinka",  "Chaloupka",  "Chalupa",    "Charvát",
     "Chládek",   "Chlup",    "Chmelař",   "Chmelík",   "Chovanec",  "Chromý",     "Chudoba",    "Chvátal",
     "Chvojka",   "Chytil",   "Cibulka",   "Čihák",     "Cihlář",    "Císař",      "Čížek",      "Čonka",
     "Coufal",    "Čurda",    "Daněk",     "Daniel",    "Daniš",     "David",      "Dědek",      "Demeter",
     "Dittrich",  "Diviš",    "Dlouhý",    "Dobeš",     "Dobiáš",    "Dobrovolný", "Dočekal",    "Dočkal",
     "Dohnal",    "Dokoupil", "Doleček",   "Dolejš",    "Dolejší",   "Doležal",    "Doležel",    "Doskočil",
     "Dostál",    "Doubek",   "Doubrava",  "Douša",     "Drábek",    "Drozd",      "Dubský",     "Duchoň",
     "Duda",      "Dudek",    "Dufek",     "Dunka",     "Dušek",     "Dvořáček",   "Dvořák",     "Dvorský",
     "Eliáš",     "Erben",    "Fabián",    "Fanta",     "Farkaš",    "Fejfar",     "Fencl",      "Ferenc",
     "Ferko",     "Fiala",    "Fiedler",   "Filip",     "Fischer",   "Fišer",      "Florián",    "Fojtík",
     "Foltýn",    "Forman",   "Formánek",  "Fořt",      "Fousek",    "Franc",      "Franěk",     "Frank",
     "Fridrich",  "Frydrych", "Fuchs",     "Fučík",     "Fuksa",     "Gábor",      "Gabriel",    "Gajdoš",
     "Gaži",      "Gottwald", "Gregor",    "Gruber",    "Grundza",   "Grygar",     "Hájek",      "Hajný",
     "Hála",      "Hampl",    "Hána",      "Hanáček",   "Hanák",     "Hanousek",   "Hanus",      "Hanuš",
     "Hanzal",    "Hanzl",    "Hanzlík",   "Hartman",   "Hašek",     "Havel",      "Havelka",    "Havlíček",
     "Havlík",    "Havránek", "Heczko",    "Heger",     "Hejda",     "Hejduk",     "Hejl",       "Hejna",
     "Hendrych",  "Herman",   "Heřman",    "Heřmánek",  "Hladík",    "Hladký",     "Hlaváč",     "Hlaváček",
     "Hlavatý",   "Hlávka",   "Hloušek",   "Hoffmann",  "Hofman",    "Holan",      "Holas",      "Holec",
     "Holeček",   "Holík",    "Holoubek",  "Holub",     "Holý",      "Homola",     "Homolka",    "Hora",
     "Horáček",   "Horák",    "Hořejší",   "Horký",     "Horňák",    "Horníček",   "Horník",     "Horský",
     "Horvát",    "Horváth",  "Hošek",     "Houdek",    "Houška",    "Hovorka",    "Hrabal",     "Hrabovský",
     "Hradecký",  "Hradil",   "Hrbáček",   "Hrbek",     "Hrdina",    "Hrdlička",   "Hrdý",       "Hrnčíř",
     "Hroch",     "Hromádka", "Hron",      "Hrubeš",    "Hrubý",     "Hruška",     "Hrůza",      "Hubáček",
     "Hudec",     "Hudeček",  "Hůlka",     "Huml",      "Husák",     "Hušek",      "Hýbl",       "Hynek",
     "Jahoda",    "Jakeš",    "Jakl",      "Jakoubek",  "Jakubec",   "Janáček",    "Janák",      "Janata",
     "Janča",     "Jančík",   "Janda",     "Janeček",   "Janečka",   "Janíček",    "Janík",      "Janků",
     "Janota",    "Janoušek", "Janovský",  "Jansa",     "Jánský",    "Janů",       "Jareš",      "Jaroš",
     "Jašek",     "Javůrek",  "Jech",      "Jedlička",  "Jelen",     "Jelínek",    "Jeníček",    "Jeřábek",
     "Jež",       "Ježek",    "Jílek",     "Jindra",    "Jíra",      "Jirák",      "Jiránek",    "Jirásek",
     "Jiřík",     "Jirka",    "Jirků",     "Jiroušek",  "Jirsa",     "John",       "Jonáš",      "Junek",
     "Jurčík",    "Jurečka",  "Juřica",    "Juřík",     "Kabát",     "Kačírek",    "Kadeřábek",  "Kadlec",
     "Kafka",     "Kaiser",   "Kala",      "Kaláb",     "Kalaš",     "Kalina",     "Kalivoda",   "Kalous",
     "Kalousek",  "Kameník",  "Kaňa",      "Káňa",      "Kaňka",     "Kantor",     "Kaplan",     "Karas",
     "Karásek",   "Karban",   "Karel",     "Karlík",    "Kasal",     "Kašík",      "Kašpar",     "Kašpárek",
     "Kavka",     "Kazda",    "Kindl",     "Klečka",    "Klein",     "Klement",    "Klíma",      "Kliment",
     "Klimeš",    "Klouček",  "Klouda",    "Knap",      "Knotek",    "Koch",       "Kočí",       "Kocián",
     "Kocman",    "Kocourek", "Kohout",    "Kohoutek",  "Koláček",   "Kolář",      "Kolařík",    "Kolek",
     "Kolman",    "Komárek",  "Komínek",   "Konečný",   "Koníček",   "Kopal",      "Kopeček",    "Kopecký",
     "Kopečný",   "Kopřiva",  "Korbel",    "Kořínek",   "Kos",       "Kosík",      "Kosina",     "Košťál",
     "Kostka",    "Kotas",    "Kotek",     "Kotlár",    "Kotrba",    "Kouba",      "Koubek",     "Koudela",
     "Koudelka",  "Koukal",   "Kouřil",    "Koutný",    "Kováč",     "Kovář",      "Kovařík",    "Kovářík",
     "Kozák",     "Kozel",    "Krajíček",  "Král",      "Králíček",  "Králík",     "Krátký",     "Kratochvíl",
     "Kraus",     "Krčmář",   "Křeček",    "Krejčí",    "Krejčík",   "Krejčíř",    "Křenek",     "Krištof",
     "Křivánek",  "Kříž",     "Křížek",    "Kropáček",  "Kroupa",    "Krupa",      "Krupička",   "Krupka",
     "Kuba",      "Kubánek",  "Kubát",     "Kubec",     "Kubelka",   "Kubeš",      "Kubica",     "Kubíček",
     "Kubík",     "Kubín",    "Kubiš",     "Kuča",      "Kučera",    "Kuchař",     "Kuchta",     "Kudláček",
     "Kudrna",    "Kukla",    "Kulhánek",  "Kulhavý",   "Kunc",      "Kuneš",      "Kupec",      "Kupka",
     "Kurka",     "Kužel",    "Kvapil",    "Kvasnička", "Kyncl",     "Kysela",     "Lacina",     "Lacko",
     "Lakatoš",   "Landa",    "Lang",      "Langer",    "Langr",     "Látal",      "Lavička",    "Le",
     "Lebeda",    "Levý",     "Líbal",     "Linhart",   "Liška",     "Lorenc",     "Louda",      "Ludvík",
     "Lukáč",     "Lukáš",    "Lukášek",   "Lukeš",     "Macák",     "Macek",      "Mach",       "Mácha",
     "Machač",    "Macháč",   "Macháček",  "Machala",   "Machálek",  "Macura",     "Majer",      "Maleček",
     "Málek",     "Malík",    "Malina",    "Malý",      "Maňák",     "Mareček",    "Marek",      "Mareš",
     "Mařík",     "Maršálek", "Maršík",    "Martinec",  "Martinek",  "Martínek",   "Mašek",      "Masopust",
     "Matějíček", "Matějka",  "Matouš",    "Matoušek",  "Matula",    "Matuška",    "Matyáš",     "Matys",
     "Maxa",      "Mayer",    "Mazánek",   "Medek",     "Melichar",  "Mencl",      "Menšík",     "Merta",
     "Michal",    "Michalec", "Michálek",  "Michalík",  "Michna",    "Mička",      "Mika",       "Míka",
     "Mikeš",     "Miko",     "Mikula",    "Mikulášek", "Minář",     "Minařík",    "Mirga",      "Mládek",
     "Mlčoch",    "Mlejnek",  "Mojžíš",    "Mokrý",     "Molnár",    "Moravec",    "Morávek",    "Motl",
     "Motyčka",   "Moučka",   "Moudrý",    "Mráček",    "Mráz",      "Mrázek",     "Mrkvička",   "Mucha",
     "Müller",    "Műller",   "Musil",     "Mužík",     "Myška",     "Nagy",       "Najman",     "Navrátil",
     "Nečas",     "Nedbal",   "Nedoma",    "Nedvěd",    "Nejedlý",   "Němec",      "Němeček",    "Nešpor",
     "Nesvadba",  "Neubauer", "Neuman",    "Neumann",   "Nguyen",    "Nguyen van", "Nosek",      "Nováček",
     "Novák",     "Novosad",  "Novotný",   "Nový",      "Odehnal",   "Oláh",       "Oliva",      "Ondra",
     "Ondráček",  "Orság",    "Otáhal",    "Paleček",   "Pánek",     "Papež",      "Pařízek",    "Pašek",
     "Pátek",     "Patočka",  "Paul",      "Pavel",     "Pavelek",   "Pavelka",    "Pavlas",     "Pavlica",
     "Pavlíček",  "Pavlík",   "Pavlů",     "Pazdera",   "Pech",      "Pecha",      "Pecháček",   "Pecka",
     "Pekař",     "Pekárek",  "Pelc",      "Pelikán",   "Peřina",    "Pernica",    "Peroutka",   "Pešek",
     "Peška",     "Pešta",    "Peterka",   "Petr",      "Petrák",    "Petráš",     "Petříček",   "Petřík",
     "Petrů",     "Pham",     "Pícha",     "Pilař",     "Pilát",     "Píša",       "Pivoňka",    "Plaček",
     "Plachý",    "Plšek",    "Pluhař",    "Podzimek",  "Pohl",      "Pokorný",    "Poláček",    "Polách",
     "Polák",     "Polanský", "Polášek",   "Polívka",   "Popelka",   "Pospíchal",  "Pospíšil",   "Potůček",
     "Pour",      "Prachař",  "Prášek",    "Pražák",    "Prchal",    "Přibyl",     "Příhoda",    "Přikryl",
     "Procházka", "Prokeš",   "Prokop",    "Prošek",    "Provazník", "Průcha",     "Průša",      "Pšenička",
     "Ptáček",    "Rác",      "Rada",      "Rak",       "Rambousek", "Raška",      "Rataj",      "Řeháček",
     "Řehák",     "Řehoř",    "Remeš",     "Řezáč",     "Rezek",     "Řezníček",   "Richter",    "Richtr",
     "Říha",      "Roubal",   "Rous",      "Rozsypal",  "Rudolf",    "Růžek",      "Růžička",    "Ryba",
     "Rybář",     "Rýdl",     "Ryšavý",    "Sadílek",   "Šafář",     "Šafařík",    "Šafránek",   "Šálek",
     "Samek",     "Šanda",    "Šašek",     "Schejbal",  "Schmidt",   "Schneider",  "Schwarz",    "Šebek",
     "Šebela",    "Šebesta",  "Šeda",      "Šedivý",    "Sedláček",  "Sedlák",     "Sedlář",     "Sehnal",
     "Seidl",     "Seifert",  "Sekanina",  "Semerád",   "Šenk",      "Šesták",     "Ševčík",     "Severa",
     "Sikora",    "Šilhavý",  "Šíma",      "Šimáček",   "Šimák",     "Šimánek",    "Šimčík",     "Šimeček",
     "Šimek",     "Šimon",    "Šimůnek",   "Šindelář",  "Šindler",   "Šíp",        "Šípek",      "Šír",
     "Široký",    "Šiška",    "Sivák",     "Skácel",    "Skala",     "Skála",      "Skalický",   "Sklenář",
     "Škoda",     "Skopal",   "Skořepa",   "Škrabal",   "Skřivánek", "Slabý",      "Sládek",     "Sladký",
     "Sláma",     "Slanina",  "Slavíček",  "Slavík",    "Šlechta",   "Slezák",     "Slováček",   "Slovák",
     "Sluka",     "Smejkal",  "Šmejkal",   "Smékal",    "Šmerda",    "Smetana",    "Šmíd",       "Smola",
     "Smolík",    "Smolka",   "Smrčka",    "Smrž",      "Smutný",    "Šnajdr",     "Sobek",      "Sobotka",
     "Sochor",    "Sojka",    "Sokol",     "Šolc",      "Sommer",    "Souček",     "Soukup",     "Sova",
     "Špaček",    "Spáčil",   "Špička",    "Šplíchal",  "Spurný",    "Šrámek",     "Srb",        "Staněk",
     "Stárek",    "Starý",    "Šťastný",   "Štefan",    "Štefek",    "Štefl",      "Stehlík",    "Steiner",
     "Stejskal",  "Štěpán",   "Štěpánek",  "Štěrba",    "Stibor",    "Stoklasa",   "Straka",     "Stránský",
     "Strejček",  "Strnad",   "Strouhal",  "Stuchlík",  "Studený",   "Studnička",  "Stupka",     "Šubrt",
     "Suchánek",  "Suchomel", "Suchý",     "Suk",       "Šulc",      "Šustr",      "Šváb",       "Svačina",
     "Švanda",    "Švarc",    "Svatoň",    "Svatoš",    "Švec",      "Švehla",     "Švejda",     "Švestka",
     "Světlík",   "Sviták",   "Svoboda",   "Svozil",    "Sýkora",    "Synek",      "Syrový",     "Táborský",
     "Tancoš",    "Teplý",    "Tesař",     "Tichý",     "Toman",     "Tománek",    "Tomáš",      "Tomášek",
     "Tomeček",   "Tomek",    "Tomeš",     "Tóth",      "Tran",      "Trávníček",  "Trčka",      "Tříska",
     "Trnka",     "Trojan",   "Truhlář",   "Tuček",     "Tůma",      "Tureček",    "Turek",      "Tvrdík",
     "Tvrdý",     "Uher",     "Uhlíř",     "Ulrich",    "Urban",     "Urbanec",    "Urbánek",    "Vacek",
     "Vácha",     "Václavek", "Václavík",  "Vaculík",   "Vágner",    "Vala",       "Valášek",    "Válek",
     "Valenta",   "Valeš",    "Váňa",      "Vančura",   "Vaněček",   "Vaněk",      "Vaníček",    "Varga",
     "Vašák",     "Vašek",    "Vašíček",   "Vávra",     "Vavřík",    "Večeřa",     "Vejvoda",    "Verner",
     "Veselý",    "Veverka",  "Vícha",     "Vilímek",   "Vinš",      "Víšek",      "Vít",        "Vitásek",
     "Vítek",     "Vlach",    "Vlasák",    "Vlček",     "Vlk",       "Vobořil",    "Vodák",      "Vodička",
     "Vodrážka",  "Vojáček",  "Vojta",     "Vojtěch",   "Vojtek",    "Vojtíšek",   "Vokoun",     "Volek",
     "Volf",      "Volný",    "Vondra",    "Vondráček", "Vondrák",   "Voráček",    "Vorel",      "Voříšek",
     "Vorlíček",  "Votava",   "Votruba",   "Vrabec",    "Vrána",     "Vrba",       "Vrzal",      "Vybíral",
     "Vydra",     "Vymazal",  "Vyskočil",  "Vysloužil", "Wagner",    "Walter",     "Weber",      "Weiss",
     "Winkler",   "Wolf",     "Zábranský", "Žáček",     "Zach",      "Zahrádka",   "Zahradník",  "Zajíc",
     "Zajíček",   "Žák",      "Zálešák",   "Zámečník",  "Zapletal",  "Záruba",     "Zatloukal",  "Zavadil",
     "Zavřel",    "Zbořil",   "Žďárský",   "Zdražil",   "Zedník",    "Zelenka",    "Zelený",     "Zelinka",
     "Zeman",     "Zemánek",  "Žemlička",  "Zezula",    "Žídek",     "Žiga",       "Zíka",       "Zikmund",
     "Zima",      "Žižka",    "Zlámal",    "Zoubek",    "Zouhar",    "Žůrek",      "Zvěřina"});

const auto czechFemaleLastNames = std::to_array<std::string_view>(
    {"Adamová",      "Adamcová",      "Adámková",     "Albrechtová",  "Ambrožová",     "Andělová",    "Andrlová",
     "Antošová",     "Bajerová",      "Balážová",     "Balcarová",    "Balogová",      "Balounová",   "Baráková",
     "Baranová",     "Barešová",      "Bártová",      "Bartáková",    "Bartoňová",     "Bartošová",   "Bartošková",
     "Bartůňková",   "Baštová",       "Bauerová",     "Bayerová",     "Bažantová",     "Bečková",     "Bečvářová",
     "Bednářová",    "Bednaříková",   "Bělohlávková", "Bendová",      "Benešová",      "Beranová",    "Beránková",
     "Bergrová",     "Berková",       "Berkyová",     "Bernardová",   "Bezděková",     "Bílková",     "Bílá",
     "Bínová",       "Bittnerová",    "Blahová",      "Bláhová",      "Blažková",      "Blechová",    "Bobková",
     "Bočková",      "Boháčová",      "Boháčková",    "Böhmová",      "Borovičková",   "Boučková",    "Boudová",
     "Boušková",     "Brabcová",      "Brabencová",   "Bradová",      "Bradáčová",     "Braunová",    "Brázdová",
     "Brázdilová",   "Brejchová",     "Březinová",    "Břízová",      "Brožová",       "Brožková",    "Brychtová",
     "Bubeníková",   "Bučková",       "Buchtová",     "Burdová",      "Burešová",      "Burianová",   "Buriánková",
     "Byrtusová",    "Čadová",        "Cahová",       "Čápová",       "Čapková",       "Čechová",     "Čejková",
     "Čermáková",    "Černíková",     "Černochová",   "Černohorská",  "Černá",         "Červeňáková", "Červenková",
     "Červená",      "Červinková",    "Chaloupková",  "Chalupová",    "Charvátová",    "Chládková",   "Chlupová",
     "Chmelařová",   "Chmelíková",    "Chovancová",   "Chromá",       "Chudobová",     "Chvátalová",  "Chvojková",
     "Chytilová",    "Cibulková",     "Čiháková",     "Cihlářová",    "Císařová",      "Čížková",     "Čonková",
     "Coufalová",    "Čurdová",       "Danková",      "Danilová",     "Danišová",      "Davidová",    "Dědková",
     "Demetrová",    "Dittrichová",   "Divišová",     "Dlouhá",       "Dobešová",      "Dobiášová",   "Dobrovolná",
     "Dočekalová",   "Dočkalová",     "Dohnalová",    "Dokoupilová",  "Dolečková",     "Dolejšová",   "Dolejší",
     "Doležalová",   "Doleželová",    "Doskočilová",  "Dostálová",    "Doubková",      "Doubravová",  "Doušová",
     "Drábková",     "Drozdová",      "Dubská",       "Duchoňová",    "Dudová",        "Dudková",     "Dufková",
     "Dunková",      "Dušková",       "Dvořáčková",   "Dvořáková",    "Dvorská",       "Eliášová",    "Erbenová",
     "Fabiánová",    "Fantová",       "Farkašová",    "Fejfarová",    "Fenclová",      "Ferencová",   "Ferková",
     "Fialová",      "Fiedlerová",    "Filipová",     "Fischerová",   "Fišerová",      "Floriánová",  "Fojtíková",
     "Foltýnová",    "Formanová",     "Formánková",   "Fořtová",      "Fousková",      "Francová",    "Franková",
     "Fridrichová",  "Frydrychová",   "Fuchsová",     "Fučíková",     "Fuksová",       "Gáborová",    "Gabrielová",
     "Gajdošová",    "Gažiová",       "Gottwaldová",  "Gregorová",    "Grubrová",      "Grundzová",   "Grygarová",
     "Hájková",      "Hajná",         "Hálová",       "Hamplová",     "Hánová",        "Hanáčková",   "Hanáková",
     "Hanousková",   "Hanusová",      "Hanušová",     "Hanzalová",    "Hanzlová",      "Hanzlíková",  "Hartmanová",
     "Hašková",      "Havlová",       "Havelková",    "Havlíčková",   "Havlíková",     "Havránková",  "Heczková",
     "Hegerová",     "Hejdová",       "Hejduková",    "Hejlová",      "Hejnová",       "Hendrychová", "Hermanová",
     "Heřmanová",    "Heřmánková",    "Hladíková",    "Hladká",       "Hlaváčová",     "Hlaváčková",  "Hlavatá",
     "Hlávková",     "Hloušková",     "Hoffmannová",  "Hofmanová",    "Holanová",      "Holasová",    "Holcová",
     "Holečková",    "Holíková",      "Holoubková",   "Holubová",     "Holá",          "Homolová",    "Homolková",
     "Horová",       "Horáčková",     "Horáková",     "Hořejší",      "Horký",         "Horňáková",   "Horníčková",
     "Horníková",    "Horská",        "Horvátová",    "Horváthová",   "Hošková",       "Houdková",    "Houšková",
     "Hovorková",    "Hrabalová",     "Hrabovská",    "Hradecká",     "Hradilová",     "Hrbáčková",   "Hrbková",
     "Hrdinová",     "Hrdličková",    "Hrdá",         "Hrnčířová",    "Hrochová",      "Hromádková",  "Hronová",
     "Hrubešová",    "Hrubá",         "Hrušková",     "Hrůzová",      "Hubáčková",     "Hudcová",     "Hudečková",
     "Hůlková",      "Humlová",       "Husáková",     "Hušková",      "Hýblová",       "Hynková",     "Jahodová",
     "Jakešová",     "Jaklová",       "Jakoubková",   "Jakubcová",    "Janáčková",     "Janáková",    "Janatová",
     "Jančová",      "Jančíková",     "Jandová",      "Janečková",    "Janíčková",     "Janíková",    "Janků",
     "Janotová",     "Janoušková",    "Janovská",     "Jansová",      "Jánská",        "Janů",        "Jarešová",
     "Jarošová",     "Jašková",       "Javůrková",    "Jechová",      "Jedličková",    "Jelenová",    "Jelínková",
     "Jeníčková",    "Jeřábková",     "Ježová",       "Ježková",      "Jílková",       "Jindrová",    "Jírová",
     "Jiráková",     "Jiránková",     "Jirásková",    "Jiříková",     "Jirková",       "Jirků",       "Jiroušková",
     "Jirsová",      "Johnová",       "Jonášová",     "Junková",      "Jurčíková",     "Jurečková",   "Juřicová",
     "Juříková",     "Kabátová",      "Kačírková",    "Kadeřábková",  "Kadlecová",     "Kafková",     "Kaisrová",
     "Kalová",       "Kalábová",      "Kalašová",     "Kalinová",     "Kalivodová",    "Kalousová",   "Kalousková",
     "Kameníková",   "Kaňová",        "Káňová",       "Kaňková",      "Kantorová",     "Kaplanová",   "Karasová",
     "Karásková",    "Karbanová",     "Karlová",      "Karlíková",    "Kasalová",      "Kašíková",    "Kašparová",
     "Kašpárková",   "Kavková",       "Kazdová",      "Kindlová",     "Klečková",      "Kleinová",    "Klementová",
     "Klímová",      "Klimentová",    "Klimešová",    "Kloučková",    "Kloudová",      "Knapová",     "Knotková",
     "Kochová",      "Kočí",          "Kociánová",    "Kocmanová",    "Kocourková",    "Kohoutová",   "Kohoutková",
     "Koláčková",    "Kolářová",      "Kolaříková",   "Kolková",      "Kolmanová",     "Komárková",   "Komínková",
     "Konečná",      "Koníčková",     "Kopalová",     "Kopečková",    "Kopecká",       "Kopečná",     "Kopřivová",
     "Korbelová",    "Kořínková",     "Kosová",       "Kosíková",     "Kosinová",      "Košťálová",   "Kostková",
     "Kotasová",     "Kotková",       "Kotlárová",    "Kotrbová",     "Koubová",       "Koubková",    "Koudelová",
     "Koudelková",   "Koukalová",     "Kouřilová",    "Koutná",       "Kováčová",      "Kovářová",    "Kovaříková",
     "Kováříková",   "Kozáková",      "Kozlová",      "Krajíčková",   "Králová",       "Králíčková",  "Králíková",
     "Krátká",       "Kratochvílová", "Krausová",     "Krčmářová",    "Křečková",      "Krejčová",    "Krejčíková",
     "Krejčířová",   "Křenková",      "Krištofová",   "Křivánková",   "Křížová",       "Křížková",    "Kropáčková",
     "Kroupová",     "Krupová",       "Krupičková",   "Krupková",     "Kubová",        "Kubánková",   "Kubátová",
     "Kubcová",      "Kubelková",     "Kubešová",     "Kubicová",     "Kubíčková",     "Kubíková",    "Kubínová",
     "Kubišová",     "Kučová",        "Kučerová",     "Kuchařová",    "Kuchtová",      "Kudláčková",  "Kudrnová",
     "Kuklová",      "Kulhánková",    "Kulhavá",      "Kuncová",      "Kunešová",      "Kupcová",     "Kupková",
     "Kurková",      "Kuželová",      "Kvapilová",    "Kvasničková",  "Kynclová",      "Kyselová",    "Lacinová",
     "Lacková",      "Lakatošová",    "Landová",      "Langová",      "Langrová",      "Látalová",    "Lavičková",
     "Le",           "Lebedová",      "Levá",         "Líbalová",     "Linhartová",    "Lišková",     "Lorencová",
     "Loudová",      "Ludvíková",     "Lukáčová",     "Lukášová",     "Lukášková",     "Lukešová",    "Macáková",
     "Macková",      "Machová",       "Máchová",      "Machačová",    "Macháčová",     "Macháčková",  "Machalová",
     "Machálková",   "Macurová",      "Majerová",     "Malečková",    "Málková",       "Malíková",    "Malinová",
     "Malá",         "Maňáková",      "Marečková",    "Marková",      "Marešová",      "Maříková",    "Maršálková",
     "Maršíková",    "Martincová",    "Martinková",   "Martínková",   "Mašková",       "Masopustová", "Matějíčková",
     "Matějková",    "Matoušová",     "Matoušková",   "Matulová",     "Matušková",     "Matyášová",   "Matysová",
     "Maxová",       "Mayerová",      "Mazánková",    "Medková",      "Melicharová",   "Menclová",    "Menšíková",
     "Mertová",      "Michalová",     "Michalcová",   "Michálková",   "Michalíková",   "Michnová",    "Mičková",
     "Miková",       "Míková",        "Mikešová",     "Mikulová",     "Mikulášková",   "Minářová",    "Minaříková",
     "Mirgová",      "Mládková",      "Mlčochová",    "Mlejnková",    "Mojžíšová",     "Mokrá",       "Molnárová",
     "Moravcová",    "Morávková",     "Motlová",      "Motyčková",    "Moučková",      "Moudrá",      "Mráčková",
     "Mrázová",      "Mrázková",      "Mrkvičková",   "Muchová",      "Müllerová",     "Műllerová",   "Musilová",
     "Mužíková",     "Myšková",       "Nagyová",      "Najmanová",    "Navrátilová",   "Nečasová",    "Nedbalová",
     "Nedomová",     "Nedvědová",     "Nejedlá",      "Němcová",      "Němečková",     "Nešporová",   "Nesvadbová",
     "Neubauerová",  "Neumanová",     "Neumannová",   "Nguyenová",    "Nguyen vanová", "Nosková",     "Nováčková",
     "Nováková",     "Novosadová",    "Novotná",      "Nová",         "Odehnalová",    "Oláhová",     "Olivová",
     "Ondrová",      "Ondráčková",    "Orságová",     "Otáhalová",    "Palečková",     "Pánková",     "Papežová",
     "Pařízková",    "Pašková",       "Pátková",      "Patočková",    "Paulová",       "Pavlová",     "Pavelková",
     "Pavlasová",    "Pavlicová",     "Pavlíčková",   "Pavlíková",    "Pavlů",         "Pazderová",   "Pechová",
     "Pecháčková",   "Pecková",       "Pekařová",     "Pekárková",    "Pelcová",       "Pelikánová",  "Peřinová",
     "Pernicová",    "Peroutková",    "Pešková",      "Peštová",      "Peterková",     "Petrová",     "Petráková",
     "Petrášová",    "Petříčková",    "Petříková",    "Petrů",        "Pham",          "Píchová",     "Pilařová",
     "Pilátová",     "Píšová",        "Pivoňková",    "Plačková",     "Plachá",        "Plšková",     "Pluhařová",
     "Podzimková",   "Pohlová",       "Pokorná",      "Poláčková",    "Poláchová",     "Poláková",    "Polanská",
     "Polášková",    "Polívková",     "Popelková",    "Pospíchalová", "Pospíšilová",   "Potůčková",   "Pourová",
     "Prachařová",   "Prášková",      "Pražáková",    "Prchalová",    "Přibylová",     "Příhodová",   "Přikrylová",
     "Procházková",  "Prokešová",     "Prokopová",    "Prošková",     "Provazníková",  "Průchová",    "Průšová",
     "Pšeničková",   "Ptáčková",      "Rácová",       "Radová",       "Raková",        "Rambousková", "Rašková",
     "Ratajová",     "Řeháčková",     "Řeháková",     "Řehořová",     "Remešová",      "Řezáčová",    "Rezková",
     "Řezníčková",   "Richtrová",     "Říhová",       "Roubalová",    "Rousová",       "Rozsypalová", "Rudolfová",
     "Růžková",      "Růžičková",     "Rybová",       "Rybářová",     "Rýdlová",       "Ryšavá",      "Sadílková",
     "Šafářová",     "Šafaříková",    "Šafránková",   "Šálková",      "Samková",       "Šandová",     "Šašková",
     "Schejbalová",  "Schmidtová",    "Schneidrová",  "Schwarzová",   "Šebková",       "Šebelová",    "Šebestová",
     "Šedová",       "Šedivá",        "Sedláčková",   "Sedláková",    "Sedlářová",     "Sehnalová",   "Seidlová",
     "Seifertová",   "Sekaninová",    "Semerádová",   "Šenková",      "Šestáková",     "Ševčíková",   "Severová",
     "Sikorová",     "Šilhavá",       "Šímová",       "Šimáčková",    "Šimáková",      "Šimánková",   "Šimčíková",
     "Šimečková",    "Šimková",       "Šimonová",     "Šimůnková",    "Šindelářová",   "Šindlerová",  "Šípová",
     "Šípková",      "Šírová",        "Široká",       "Šišková",      "Siváková",      "Skácelová",   "Skalová",
     "Skálová",      "Skalická",      "Sklenářová",   "Škodová",      "Skopalová",     "Skořepová",   "Škrabalová",
     "Skřivánková",  "Slabá",         "Sládková",     "Sladká",       "Slámová",       "Slaninová",   "Slavíčková",
     "Slavíková",    "Šlechtová",     "Slezáková",    "Slováčková",   "Slováková",     "Sluková",     "Smejkalová",
     "Šmejkalová",   "Smékalová",     "Šmerdová",     "Smetanová",    "Šmídová",       "Smolová",     "Smolíková",
     "Smolková",     "Smrčková",      "Smržová",      "Smutná",       "Šnajdrová",     "Sobková",     "Sobotková",
     "Sochorová",    "Sojková",       "Sokolová",     "Šolcová",      "Sommrová",      "Součková",    "Soukupová",
     "Sovová",       "Špačková",      "Spáčilová",    "Špičková",     "Šplíchalová",   "Spurná",      "Šrámková",
     "Srbová",       "Staňková",      "Stárková",     "Stará",        "Šťastná",       "Štefanová",   "Štefková",
     "Šteflová",     "Stehlíková",    "Steinerová",   "Stejskalová",  "Štěpánová",     "Štěpánková",  "Štěrbová",
     "Stiborová",    "Stoklasová",    "Straková",     "Stránská",     "Strejčková",    "Strnadová",   "Strouhalová",
     "Stuchlíková",  "Studená",       "Studničková",  "Stupková",     "Šubrtová",      "Suchánková",  "Suchomelová",
     "Suchá",        "Suková",        "Šulcová",      "Šustrová",     "Švábová",       "Svačinová",   "Švandová",
     "Švarcová",     "Svatoňová",     "Svatošová",    "Švecová",      "Švehlová",      "Švejdová",    "Švestková",
     "Světlíková",   "Svitáková",     "Svobodová",    "Svozilová",    "Sýkorová",      "Synková",     "Syrová",
     "Táborská",     "Tancošová",     "Teplá",        "Tesařová",     "Tichá",         "Tomanová",    "Tománková",
     "Tomášová",     "Tomášková",     "Tomečková",    "Tomková",      "Tomešová",      "Tóthová",     "Tranová",
     "Trávníčková",  "Trčková",       "Třísková",     "Trnková",      "Trojanová",     "Truhlářová",  "Tučková",
     "Tůmová",       "Turečková",     "Turková",      "Tvrdíková",    "Tvrdá",         "Uhrová",      "Uhlířová",
     "Ulrichová",    "Urbanová",      "Urbancová",    "Urbánková",    "Vacková",       "Váchová",     "Václavková",
     "Václavíková",  "Vaculíková",    "Vágnerová",    "Valová",       "Valášková",     "Válková",     "Valentová",
     "Valešová",     "Váňová",        "Vančurová",    "Vaněčková",    "Vaňková",       "Vaníčková",   "Vargová",
     "Vašáková",     "Vašková",       "Vašíčková",    "Vávrová",      "Vavříková",     "Večeřová",    "Vejvodová",
     "Vernerová",    "Veselá",        "Veverková",    "Víchová",      "Vilímková",     "Vinšová",     "Víšková",
     "Vítová",       "Vitásková",     "Vítková",      "Vlachová",     "Vlasáková",     "Vlčková",     "Vlková",
     "Vobořilová",   "Vodáková",      "Vodičková",    "Vodrážková",   "Vojáčková",     "Vojtová",     "Vojtěchová",
     "Vojtková",     "Vojtíšková",    "Vokounová",    "Volková",      "Volfová",       "Volná",       "Vondrová",
     "Vondráčková",  "Vondráková",    "Voráčková",    "Vorlová",      "Voříšková",     "Vorlíčková",  "Votavová",
     "Votrubová",    "Vrabcová",      "Vránová",      "Vrbová",       "Vrzalová",      "Vybíralová",  "Vydrová",
     "Vymazalová",   "Vyskočilová",   "Vysloužilová", "Wagnerová",    "Waltrová",      "Webrová",     "Weissová",
     "Winklerová",   "Wolfová",       "Zábranská",    "Žáčková",      "Zachová",       "Zahrádková",  "Zahradníková",
     "Zajícová",     "Zajíčková",     "Žáková",       "Zálešáková",   "Zámečníková",   "Zapletalová", "Zárubová",
     "Zatloukalová", "Zavadilová",    "Zavřelová",    "Zbořilová",    "Žďárská",       "Zdražilová",  "Zedníková",
     "Zelenková",    "Zelená",        "Zelinková",    "Zemanová",     "Zemánková",     "Žemličková",  "Zezulová",
     "Žídková",      "Žigová",        "Zíková",       "Zikmundová",   "Zimová",        "Žižková",     "Zlámalová",
     "Zoubková",     "Zouharová",     "Žůrková",      "Zvěřinová"});

const auto czechPrefixes = std::to_array<std::string_view>({"Ing.", "Mgr.", "JUDr.", "MUDr."});

const auto czechSuffixes = std::to_array<std::string_view>({"Phd."});

const NameFormats czechNameFormats{
    {{"{firstName} {lastName}", 8}, {"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames czechPeopleNames{{czechMaleFirstNames, czechMaleLastNames, czechPrefixes, czechSuffixes},
                                   {czechFemaleFirstNames, czechFemaleLastNames, czechPrefixes, czechSuffixes},
                                   czechNameFormats};

// Denmark

const auto danishMaleFirstNames = std::to_array<std::string_view>({"Adam",
                                                                   "Adrian",
                                                                   "Ahmad",
                                                                   "Aksel",
                                                                   "Albert",
                                                                   "Alexander",
                                                                   "Alfred",
                                                                   "Ali",
                                                                   "Anders",
                                                                   "Andreas",
                                                                   "Anton",
                                                                   "Arthur",
                                                                   "Arunthavanathan",
                                                                   "August",
                                                                   "Benjamin",
                                                                   "Bjørn",
                                                                   "Brian",
                                                                   "Carl",
                                                                   "Christian",
                                                                   "Christopher",
                                                                   "Claus",
                                                                   "Daniel",
                                                                   "David",
                                                                   "Elias",
                                                                   "Elliot",
                                                                   "Emil",
                                                                   "Erik",
                                                                   "Esben",
                                                                   "Felix",
                                                                   "Frank",
                                                                   "Frederik",
                                                                   "Georg",
                                                                   "Hakim",
                                                                   "Hans",
                                                                   "Harald",
                                                                   "Henrik",
                                                                   "Ib",
                                                                   "Ivan",
                                                                   "Jacob",
                                                                   "Jakob",
                                                                   "Jan",
                                                                   "Jens",
                                                                   "Jeppe",
                                                                   "Jesper",
                                                                   "Johan",
                                                                   "John",
                                                                   "Jonas",
                                                                   "Jonathan",
                                                                   "Jørgen",
                                                                   "Kaj",
                                                                   "Karl-Emil",
                                                                   "Karl-Johan",
                                                                   "Karl",
                                                                   "Kasper",
                                                                   "Kevin",
                                                                   "Kim",
                                                                   "Kurt",
                                                                   "Lars",
                                                                   "Lasse",
                                                                   "Leif",
                                                                   "Leo",
                                                                   "Linus",
                                                                   "Lucas",
                                                                   "Lukas",
                                                                   "Mads",
                                                                   "Magnus",
                                                                   "Malthe",
                                                                   "Mark",
                                                                   "Martin",
                                                                   "Mathias",
                                                                   "Mathis",
                                                                   "Max",
                                                                   "Michael",
                                                                   "Mikkel",
                                                                   "Mohammad",
                                                                   "Morten",
                                                                   "Nicklas",
                                                                   "Niels-Christian",
                                                                   "Niels",
                                                                   "Niklas",
                                                                   "Noah",
                                                                   "Ole",
                                                                   "Oliver",
                                                                   "Oscar",
                                                                   "Otto",
                                                                   "Patrick",
                                                                   "Per",
                                                                   "Peter",
                                                                   "Philip",
                                                                   "Poul",
                                                                   "Rasmus",
                                                                   "Robert",
                                                                   "Samuel",
                                                                   "Sebastian",
                                                                   "Sigurd",
                                                                   "Simon",
                                                                   "Steen",
                                                                   "Stefan",
                                                                   "Stig",
                                                                   "Sune",
                                                                   "Svend",
                                                                   "Søren",
                                                                   "Theo",
                                                                   "Thomas",
                                                                   "Tim",
                                                                   "Tobias",
                                                                   "Troels",
                                                                   "Uffe",
                                                                   "Ulrik",
                                                                   "Vagn",
                                                                   "Valdemar",
                                                                   "Victor",
                                                                   "Viggo",
                                                                   "Viktor",
                                                                   "Vilhelm",
                                                                   "Villy",
                                                                   "Vincent",
                                                                   "William"});

const auto danishFemaleFirstNames = std::to_array<std::string_view>({"Agnes",
                                                                     "Alberte",
                                                                     "Alexandrea",
                                                                     "Alice",
                                                                     "Alma",
                                                                     "Amanda",
                                                                     "Anita",
                                                                     "Anna",
                                                                     "Anne",
                                                                     "Annette",
                                                                     "Astrid",
                                                                     "Bente",
                                                                     "Betina",
                                                                     "Birgitte",
                                                                     "Britt",
                                                                     "Camilla",
                                                                     "Caroline",
                                                                     "Cecilie",
                                                                     "Charlotte",
                                                                     "Christina",
                                                                     "Clara",
                                                                     "Ditte-Marie",
                                                                     "Dorit",
                                                                     "Dorthe",
                                                                     "Ea",
                                                                     "Elisabeth",
                                                                     "Elise",
                                                                     "Ella",
                                                                     "Ellen-Margrethe",
                                                                     "Else",
                                                                     "Emilie",
                                                                     "Emma",
                                                                     "Eva",
                                                                     "Fatima",
                                                                     "Freja",
                                                                     "Frida",
                                                                     "Gitte",
                                                                     "Gry",
                                                                     "Hanne",
                                                                     "Hedvig",
                                                                     "Helene",
                                                                     "Helle",
                                                                     "Hilda",
                                                                     "Ida",
                                                                     "Ingrid",
                                                                     "Isabella",
                                                                     "Janni",
                                                                     "Jeanette",
                                                                     "Jette",
                                                                     "Johanne",
                                                                     "Josefine",
                                                                     "Julie",
                                                                     "Karen",
                                                                     "Karla",
                                                                     "Kathrine",
                                                                     "Katrine",
                                                                     "Kirsten",
                                                                     "Laila",
                                                                     "Lene",
                                                                     "Lillian",
                                                                     "Line",
                                                                     "Lis",
                                                                     "Lisa",
                                                                     "Lisbeth",
                                                                     "Liv",
                                                                     "Lone",
                                                                     "Louise",
                                                                     "Luna",
                                                                     "Lykke",
                                                                     "Maja",
                                                                     "Malene",
                                                                     "Maren",
                                                                     "Maria",
                                                                     "Marianne",
                                                                     "Mathilde",
                                                                     "Merete",
                                                                     "Mette",
                                                                     "Mia",
                                                                     "Mille",
                                                                     "Mona",
                                                                     "Nadia",
                                                                     "Nanna",
                                                                     "Nicoline",
                                                                     "Nina",
                                                                     "Pia",
                                                                     "Rebecca",
                                                                     "Regitze",
                                                                     "Rikke",
                                                                     "Rosa-Maria",
                                                                     "Ruth",
                                                                     "Sabrina",
                                                                     "Sandra",
                                                                     "Sanne",
                                                                     "Sara",
                                                                     "Sarah",
                                                                     "Signe",
                                                                     "Silje",
                                                                     "Simone",
                                                                     "Sofie",
                                                                     "Stine",
                                                                     "Susanne",
                                                                     "Therese",
                                                                     "Tina",
                                                                     "Tine",
                                                                     "Trine",
                                                                     "Vibeke",
                                                                     "Viktoria",
                                                                     "Yasmin",
                                                                     "Zara"});

const auto danishLastNames = std::to_array<std::string_view>(
    {"Abbas",       "Abdi",         "Andersen",  "Andresen",    "Bak",         "Bang",      "Bech",
     "Beckmann",    "Berg",         "Bertelsen", "Bisgaard",    "Bjerregaard", "Blom",      "Bonde",
     "Brandt",      "Brix",         "Bruun",     "Buch",        "Buhl",        "Bundgaard", "Carlsen",
     "Christensen", "Christiansen", "Clausen",   "Dahl",        "Dalgaard",    "Dam",       "Damm",
     "Davidsen",    "Dupont",       "Esbensen",  "Fischer",     "Foged",       "Frandsen",  "Frederiksen",
     "Gade",        "Garcia",       "Gregersen", "Hald",        "Hansen",      "Haugaard",  "Hedegaard",
     "Henriksen",   "Hermansen",    "Hjort",     "Hjorth",      "Holm",        "Iversen",   "Jakobsen",
     "Jensen",      "Jeppesen",     "Jespersen", "Johannessen", "Johannsen",   "Johansen",  "Juhl",
     "Justesen",    "Jørgensen",    "Karlsen",   "Khan",        "Kjeldsen",    "Kjær",      "Klausen",
     "Knudsen",     "Kristensen",   "Krogh",     "Larsen",      "Lauridsen",   "Laursen",   "Lorentzen",
     "Lund",        "Madsen",       "Magnussen", "Marcussen",   "Mortensen",   "Munch",     "Munk",
     "Møller",      "Nguyen",       "Nielsen",   "Nissen",      "Nygaard",     "Olsen",     "Pallesen",
     "Pedersen",    "Petersen",     "Pham",      "Poulsen",     "Rasmussen",   "Ravn",      "Richter",
     "Schmidt",     "Schrøder",     "Simonsen",  "Skov",        "Steffensen",  "Storm",     "Svendsen",
     "Svensson",    "Sørensen",     "Thomsen",   "Toft",        "Vestergaard", "Villadsen", "Vinther",
     "Winther"});

const auto danishMalePrefixes = std::to_array<std::string_view>({"hr."});

const auto danishFemalePrefixes = std::to_array<std::string_view>({"fr."});

const NameFormats danishNameFormats{{{"{firstName} {lastName}", 1}, {"{firstName} {lastName}", 1}}};

const PeopleNames danishPeopleNames{{danishMaleFirstNames, danishMalePrefixes, {}, {}},
                                    {danishFemaleFirstNames, danishLastNames, danishFemalePrefixes, {}},
                                    danishNameFormats};

// Dutch

const auto dutchMaleFirstNames = std::to_array<std::string_view>({
    "Tymon",       "Steven",     "Semih",      "Seff",       "Sabir",      "Nico",    "Naoufal",   "Nadir",
    "Mirza",       "Miguel",     "Manoah",     "Kenji",      "Kenan",      "Joseph",  "Jordy",     "Jeppe",
    "Diego",       "Coen",       "Can",        "Bentley",    "Yasin",      "Tomas",   "Sjoerd",    "Savi",
    "Safouane",    "Raphaël",    "Mas",        "Maes",       "Louie",      "Karam",   "John",      "Jelmer",
    "Jayvano",     "Hein",       "Giel",       "Emin",       "Deen",       "Badr",    "Arda",      "Umut",
    "Teunis",      "Silas",      "Rayen",      "Nikodem",    "Mex",        "Marius",  "Kayden",    "Jidde",
    "Glenn",       "Frederik",   "Foss",       "Finnley",    "Ensar",      "Duco",    "Dies",      "Brenn",
    "Yousef",      "Siep",       "Rohan",      "Osman",      "Nox",        "Natan",   "Mathijs",   "Marley",
    "Jaxon",       "Jairo",      "Jaimy",      "Eray",       "Don",        "Chase",   "Bruno",     "Brian",
    "Anton",       "Xander",     "Senne",      "Reza",       "Oliwier",    "Mitch",   "Krijn",     "Jorrit",
    "Jimi",        "Dominik",    "Devin",      "Tobin",      "Stefan",     "Jakob",   "Igor",      "Giovanni",
    "Dylano",      "Armin",      "Adrian",     "Youp",       "Sebas",      "Sander",  "Robert",    "Nils",
    "Mozes",       "Micah",      "Menno",      "Jaylano",    "Ilias",      "Freek",   "Flip",      "Dion",
    "Christopher", "Ahmet",      "Yassir",     "Yannick",    "Valentino",  "Samuël",  "Noam",      "Mylan",
    "Kenzo",       "Jona",       "Danilo",     "Bowie",      "Zain",       "Wolf",    "Thije",     "Theo",
    "Mylo",        "Matz",       "Lieuwe",     "Joes",       "Jacobus",    "Idris",   "Gideon",    "Bruce",
    "Abdullah",    "Zef",        "Viktor",     "Seb",        "Rover",      "Raphael", "Kacper",    "Josh",
    "Jeremiah",    "Deniz",      "Aleksander", "Zev",        "Yves",       "Viggo",   "Naoufel",   "Matthew",
    "Keano",       "Jaylen",     "Bo",         "Bjorn",      "Anouar",     "Abraham", "Storm",     "Martin",
    "Manu",        "Jefta",      "Djayden",    "Benyamin",   "Benja",      "Ahmad",   "Nino",      "Malik",
    "Junayd",      "Isa",        "Finley",     "Ayman",      "Zeyd",       "Yassine", "Sev",       "Rico",
    "Mels",        "Maximilian", "Leendert",   "Kick",       "Enzo",       "Sjors",   "Silvan",    "Peter",
    "Otto",        "Milano",     "Marijn",     "Javi",       "Yahya",      "Wes",     "Toon",      "Tommy",
    "Loet",        "Leonardo",   "Len",        "Kaan",       "Dax",        "Davy",    "Adriaan",   "Thiago",
    "Mustafa",     "Merijn",     "Kerem",      "Johan",      "Isaiah",     "Bryan",   "Bastiaan",  "Axel",
    "Toby",        "Thom",       "Noan",       "Marcus",     "Ismail",     "Davi",    "Bodi",      "Tieme",
    "Ole",         "Nout",       "Musa",       "Moussa",     "Luciano",    "Liyam",   "Kyano",     "Dorian",
    "Bradley",     "Wouter",     "Siebe",      "Ralph",      "Quin",       "Fender",  "Arie",      "Albert",
    "Zion",        "Wessel",     "Rick",       "Miles",      "Mahir",      "Mads",    "Klaas",     "Flynn",
    "Dexx",        "Anthony",    "Tristan",    "Mink",       "Mehmet",     "Maas",    "Lyam",      "Jorn",
    "Joost",       "Christian",  "Alparslan",  "Tyler",      "Maher",      "Jakub",   "Alan",      "William",
    "Rafaël",      "Jur",        "Jimmy",      "George",     "Christiaan", "Brandon", "Ace",       "Tibbe",
    "Sebastian",   "Lorenzo",    "Bilal",      "Teije",      "Joe",        "Michael", "Louay",     "Lou",
    "Levy",        "Kevin",      "Isaac",      "Ilay",       "Harvey",     "Davey",   "Younes",    "Xem",
    "Tycho",       "Seppe",      "Riv",        "Marinus",    "Maarten",    "Ayoub",   "Ahmed",     "Tuur",
    "Tijmen",      "Kjeld",      "Fynn",       "Frenkie",    "Dante",      "Ayaz",    "Abe",       "Jackson",
    "Bart",        "Xavier",     "Ted",        "Sebastiaan", "Safouan",    "Midas",   "Gabriël",   "Fos",
    "Douwe",       "Boet",       "Berend",     "Yassin",     "Rowan",      "Revi",    "Maurits",   "Marcel",
    "Ivo",         "Eli",        "Shane",      "Rein",       "Muhammad",   "Lennox",  "Ivar",      "Huub",
    "Tijs",        "Matthias",   "Juda",       "Dave",       "Antoni",     "Damin",   "Colin",     "Zakaria",
    "Vincent",     "Sverre",     "Mik",        "Lennon",     "Donny",      "Thijn",   "Novan",     "Mart",
    "Bob",         "Senna",      "Olaf",       "Mohammad",   "Julan",      "Riff",    "Jaivey",    "Collin",
    "Nouri",       "Noël",       "Ferre",      "Mert",       "Boas",       "Sten",    "Gabriel",   "Pelle",
    "Mark",        "Leo",        "Rafael",     "Oskar",      "Naud",       "Aras",    "Sami",      "Kasper",
    "Jop",         "Ethan",      "Dirk",       "Bodhi",      "Eden",       "Bobby",   "Jordan",    "Ivan",
    "Vinz",        "Twan",       "Odin",       "Manuel",     "Jesper",     "Scott",   "Raff",      "Luke",
    "Brent",       "Zayd",       "Sil",        "Miran",      "Charlie",    "Amin",    "Valentijn", "Eymen",
    "Jules",       "Kees",       "Jayce",      "Matteo",     "Chris",      "Quinten", "Mateo",     "Lux",
    "Justin",      "Joey",       "Jay",        "Ayden",      "Laurens",    "Luka",    "Joah",      "Gerrit",
    "Evan",        "Ben",        "Jari",       "Mika",       "Kian",       "Ian",     "Ilyas",     "Jort",
    "Mike",        "Maxim",      "Ibrahim",    "Vigo",       "Victor",     "Louis",   "Jayson",    "Florian",
    "Youssef",     "Vic",        "Raf",        "Filip",      "Yusuf",      "Timo",    "Vik",       "Micha",
    "Matthijs",    "Lasse",      "Faas",       "Riley",      "Mace",       "Daniel",  "Jamie",     "Fabian",
    "Nick",        "Muhammed",   "Luc",        "Imran",      "Joël",       "Niels",   "Philip",    "Milo",
    "Lewis",       "Wout",       "Leon",       "Joas",       "Jasper",     "Seth",    "Omar",      "Koen",
    "Julius",      "Beau",       "Rens",       "Mayson",     "Kay",        "Hamza",   "Daley",     "Robin",
    "Nolan",       "Cornelis",   "Bas",        "Lukas",      "Jonathan",   "Jonas",   "Ali",       "Ravi",
    "Hendrik",     "Sef",        "Lex",        "Thijmen",    "Damian",     "Arthur",  "Aron",      "Emir",
    "Jonah",       "Ãmer",    "Jelte",      "Melle",      "Jacob",      "Alex",    "Jim",       "Elijah",
    "Stef",        "Kaj",        "Johannes",   "Jelle",      "Joris",      "Jip",     "Aaron",     "Loek",
    "Logan",       "Lev",        "Dylan",      "Simon",      "Kyan",       "Sep",     "Moos",      "Felix",
    "Job",         "Elias",      "Ezra",       "Oscar",      "Casper",     "Duuk",    "Sepp",      "Pieter",
    "Rayan",       "Aiden",      "Joshua",     "Dean",       "Niek",       "Lenn",    "Tygo",      "Jake",
    "Stan",        "Kai",        "Roan",       "Mick",       "Mohammed",   "Dani",    "Nathan",    "Oliver",
    "Alexander",   "Mohamed",    "Hidde",      "Samuel",     "Zayn",       "Amir",    "Xavi",      "Jurre",
    "Boris",       "Senn",       "Jason",      "Pepijn",     "Quinn",      "Pim",     "Jace",      "Jax",
    "Tijn",        "Tim",        "Willem",     "Jaxx",       "Otis",       "Fedde",   "Tom",       "Owen",
    "Ryan",        "Tobias",     "Floris",     "Abel",       "Jayden",     "Sven",    "Morris",    "Cas",
    "Ties",        "Daniël",     "Joep",       "Jan",        "Jens",       "Vince",   "Dex",       "Ruben",
    "Olivier",     "Guus",       "Thijs",      "Stijn",      "Jack",       "David",   "Teun",      "Hugo",
    "Mats",        "Gijs",       "Lars",       "Luca",       "Siem",       "Mason",   "Julian",    "Boaz",
    "Benjamin",    "Thomas",     "Max",        "Adam",       "Jesse",      "Mees",    "Sam",       "Noud",
    "Bram",        "Luuk",       "Milan",      "James",      "Liam",       "Finn",    "Sem",       "Levi",
    "Lucas",       "Daan",       "Noah",
});

const auto dutchFemaleFirstNames = std::to_array<std::string_view>({
    "Yarah",    "Vere",      "Siënna",  "Sanna",     "Salomë",    "Roxy",       "Nela",      "Myra",      "Liene",
    "Lia",      "Kayleigh",  "Kaylee",  "Kato",      "Florence",  "Eefje",      "Dieke",     "Zeyneb",    "Wende",
    "Vienna",   "Melisa",    "Maaike",  "Lova",      "Lilian",    "Juno",       "Joanna",    "Jesslyn",   "Jaelynn",
    "Gioia",    "Claire",    "Alma",    "Aliya",     "Alisa",     "Zofia",      "Vivian",    "Nowi",      "Noami",
    "Milana",   "Marlie",    "Macy",    "Maan",      "Lorena",    "Liselotte",  "Lien",      "Laure",     "Laila",
    "Kim",      "Khadija",   "Joya",    "Jacoba",    "Gigi",      "Wilhelmina", "Vivienne",  "Rose",      "Rana",
    "Milly",    "Michelle",  "Marrit",  "Malou",     "Jaimy",     "Gaia",       "Fayen",     "Fabiënne",  "Eveline",
    "Daisy",    "Babette",   "Aaltje",  "Scottie",   "Nomi",      "Mathilde",   "Linne",     "Laurie",    "Kyara",
    "Jalou",    "Iva",       "Inara",   "Grace",     "Emmy",      "Elle",       "Diana",     "Céline",    "Selena",
    "Safiya",   "Quinty",    "Oliwia",  "Nadine",    "Meghan",    "Marwa",      "Jada",      "Ines",      "Fardau",
    "Eliana",   "Carmen",    "Ayana",   "Asya",      "Annabel",   "Amara",      "Alexandra", "Rachel",    "Melissa",
    "Melina",   "Luus",      "Jayda",   "Jacky",     "Eleonora",  "Asiya",      "Annemijn",  "Sare",      "Ninthe",
    "Nena",     "Melody",    "Kira",    "Jasmine",   "Imani",     "Hira",       "Elynn",     "Dominique", "Ashley",
    "Aleyna",   "Pola",      "Noë",     "Lisanne",   "Lilou",     "Kiara",      "Jette",     "Jess",      "Isabeau",
    "Floortje", "Fiep",      "Fieke",   "Femm",      "Famke",     "Chelsey",    "Amelie",    "Vanessa",   "Skye",
    "Sienna",   "Safa",      "Lydia",   "Esma",      "Daantje",   "Catharina",  "Carlijn",   "Asel",      "Amaya",
    "Amalia",   "Ada",       "Zahra",   "Tirza",     "Nikkie",    "Mirre",      "Mayra",     "Lenne",     "Jule",
    "Jana",     "Farah",     "Defne",   "Ariana",    "Yenthe",    "Renske",     "Medina",    "Liliana",   "Kenza",
    "Jazz",     "Esra",      "Emilie",  "Dua",       "Chloe",     "Abby",       "Selma",     "Maren",     "Jolijn",
    "Febe",     "Elodie",    "Djenna",  "Diede",     "Cataleya",  "Aliyah",     "Aisha",     "Teddy",     "Nika",
    "Myla",     "Izzy",      "Dewi",    "Mirte",     "Mikki",     "Kensi",      "Karlijn",   "Jazzlynn",  "Imke",
    "Eleanor",  "Wies",      "Thirza",  "Safae",     "Meyra",     "Lune",       "Jazzlyn",   "Faya",      "Arya",
    "Nadia",    "Puk",       "Oumayra", "Lois",      "Josefien",  "Vive",       "Fayenne",   "Celine",    "Nynke",
    "Loes",     "Danique",   "Aurora",  "Angelina",  "Alya",      "Abigail",    "Tara",      "Quinn",     "Meike",
    "Maartje",  "Jolien",    "Selina",  "Phileine",  "Miley",     "Lexie",      "Florine",   "Coco",      "Britt",
    "Alyssa",   "Yasmine",   "Sammie",  "Nila",      "Lucie",     "Eef",        "Bella",     "Aylin",     "Ziva",
    "Neeltje",  "Jesslynn",  "Jente",   "Jara",      "Bobbie",    "Yinthe",     "Megan",     "Freya",     "Donna",
    "Alice",    "Milena",    "Joëlle",  "Jenna",     "Evelien",   "Nisa",       "Lott",      "Loa",       "Daphne",
    "Dani",     "Maysa",     "Hidaya",  "Hayley",    "Gwen",      "Esther",     "Elize",     "Dana",      "Maxime",
    "Madelief", "Keet",      "Jip",     "Ize",       "Noëlla",    "Layla",      "Hendrika",  "Hafsa",     "Giulia",
    "Eliza",    "Bowie",     "Marly",   "Linn",      "Jane",      "Filou",      "Zuzanna",   "Leyla",     "Fatima",
    "Evie",     "Marley",    "Kyra",    "Hanne",     "Jailey",    "Ise",        "Adriana",   "Liza",      "Doris",
    "Dina",     "Amina",     "Mina",    "Aaliyah",   "Romee",     "Pippa",      "Juliette",  "Alicia",    "Merle",
    "Lea",      "Elina",     "Sena",    "Flore",     "Riley",     "Nour",       "Lizz",      "Jessie",    "Cornelia",
    "Beau",     "Valerie",   "Silke",   "Valentina", "Loua",      "Vieve",      "Vesper",    "Ruby",      "Nienke",
    "Jula",     "Azra",      "Myrthe",  "Soof",      "Juliëtte",  "Lexi",       "Davina",    "Liya",      "Yasmin",
    "Esmée",    "Amelia",    "Niene",   "Mette",     "Ela",       "Aimée",      "Joy",       "Ilse",      "Zeynep",
    "Yfke",     "Elizabeth", "Mare",    "Isra",      "Helena",    "Stella",     "Jinthe",    "Meryem",    "Janna",
    "Isabelle", "Maja",      "Lieve",   "Yuna",      "Pien",      "Faye",       "Pleun",     "Noortje",   "Janne",
    "Novi",     "Ava",       "Sterre",  "Maeve",     "Elisabeth", "Bibi",       "Ayla",      "Liyana",    "Mae",
    "Leah",     "Marie",     "Juna",    "Lucy",      "Josephine", "Victoria",   "Indy",      "Maryam",    "Jaylinn",
    "Zoey",     "Rosie",     "Mira",    "Nore",      "Jinte",     "Zara",       "Bente",     "Senna",     "Pip",
    "Alina",    "Féline",    "Aya",     "Anouk",     "Amélie",    "Lou",        "Nine",      "Jet",       "Robin",
    "Nikki",    "June",      "Fem",     "Demi",      "Mirthe",    "Lana",       "Noëlle",    "Lise",      "Lilly",
    "Jackie",   "Benthe",    "Bobbi",   "Fiene",     "Elisa",     "Maya",       "Jade",      "Emilia",    "Elif",
    "Romy",     "Lot",       "Jolie",   "Guusje",    "Mara",      "Kate",       "Marit",     "Nola",      "Louise",
    "Lola",     "Laura",     "Iris",    "Philou",    "Tessa",     "Inaya",      "Elise",     "Eline",     "Sam",
    "Evy",      "Johanna",   "Femke",   "Charlie",   "Livia",     "Rosalie",    "Suus",      "Sanne",     "Fay",
    "Fenne",    "Linde",     "Isabel",  "Puck",      "Lize",      "Rosa",       "Amira",     "Lily",      "Naomi",
    "Lizzy",    "Liva",      "Juul",    "Jill",      "Hanna",     "Floor",      "Amber",     "Suze",      "Veerle",
    "Merel",    "Anne",      "Cato",    "Charlotte", "Kiki",      "Lena",       "Ivy",       "Feline",    "Jasmijn",
    "Liz",      "Elena",     "Maria",   "Norah",     "Lara",      "Fien",       "Isabella",  "Luna",      "Amy",
    "Sophia",   "Hailey",    "Bo",      "Vera",      "Julie",     "Lisa",       "Esmee",     "Noa",       "Ella",
    "Lina",     "Mia",       "Sofia",   "Loïs",      "Hannah",    "Roos",       "Emily",     "Isa",       "Fenna",
    "Elin",     "Fleur",     "Lynn",    "Sarah",     "Nova",      "Nina",       "Maud",      "Sofie",     "Milou",
    "Lauren",   "Lotte",     "Eva",     "Noor",      "Liv",       "Yara",       "Olivia",    "Lieke",     "Nora",
    "Saar",     "Evi",       "Anna",    "Sara",      "Tess",      "Julia",      "Zoë",       "Sophie",    "Mila",
    "Emma",
});

const auto dutchLastNames = std::to_array<std::string_view>({
    "Aalbers",
    "Bakker",
    "Bijl",
    "Blom",
    "Boer",
    "Bos",
    "Bosch",
    "Bosman",
    "Brouwer",
    "Cornelissen",
    "Corsten",
    "de Boer",
    "de Bruijn",
    "de Bruyn",
    "de Corte",
    "de Graaf",
    "de Groot",
    "de Haan",
    "de Jong",
    "de Jonge",
    "de Koning",
    "de Lange",
    "de Leeuw",
    "de Nijs",
    "de Ruiter",
    "de Vos",
    "de Vries",
    "de Wit",
    "Dekker",
    "den Adel",
    "Dijkstra",
    "Driessen",
    "Evers",
    "Freriks",
    "Gerritsen",
    "Groen",
    "Groothuizen",
    "Hartman",
    "Hazes",
    "Hendriks",
    "Hermans",
    "Hofman",
    "Huisman",
    "Jacobs",
    "Jansen",
    "Janssen",
    "Jonker",
    "Klein",
    "Kok",
    "Koning",
    "Kramer",
    "Kuijpers",
    "Kuiper",
    "Kuypers",
    "Lubbers",
    "Maas",
    "Martens",
    "Meeuwis",
    "Meijer",
    "Meyer",
    "Mol",
    "Molenaar",
    "Moors",
    "Mulder",
    "Nijland",
    "Oosterhuis",
    "Peeters",
    "Poels",
    "Post",
    "Postma",
    "Prinsen",
    "Rietman",
    "Ritsma",
    "Sanders",
    "Schipper",
    "Scholten",
    "Smeets",
    "Terlouw",
    "Timmermans",
    "van Beek",
    "van Dam",
    "van de Berg",
    "van de Brink",
    "van de Meer",
    "van de Pol",
    "van de Veen",
    "van de Velden",
    "van de Ven",
    "van de Wal",
    "van den Berg",
    "van den Bosch",
    "van den Broek",
    "van den Heuvel",
    "van den Pol",
    "van den Velde",
    "van der Berg",
    "van der Heijden",
    "van der Heyden",
    "van der Horst",
    "van der Laan",
    "van der Linden",
    "van der Meer",
    "van der Meulen",
    "van der Pol",
    "van der Velde",
    "van der Velden",
    "van der Ven",
    "van der Wal",
    "van Dijk",
    "van Dongen",
    "van Doorn",
    "van Ginneken",
    "van Iersel",
    "van Leeuwen",
    "van Loon",
    "van Veen",
    "van Vliet",
    "van Wijk",
    "Veenstra",
    "Verbeek",
    "Verhoeven",
    "Vermeulen",
    "Vink",
    "Visser",
    "Vos",
    "Wagenaar",
    "Willems",
    "Willemsen",
    "Wolters",
    "Zijlstra",
    "Zuiderveld",
});

const auto dutchSuffixes = std::to_array<std::string_view>({"Jr.", "Sr."});

const auto dutchMalePrefixes = std::to_array<std::string_view>({"Dhr.", "Bsc", "Msc", "Prof.", "Ir.", "Drs.", "Dr."});

const auto dutchFemalePrefixes =
    std::to_array<std::string_view>({"Mevr.", "Bsc", "Msc", "Prof.", "Ir.", "Drs.", "Dr."});

const NameFormats dutchNameFormats{{
    {"{firstName} {lastName}", 8},
    {"{prefix} {firstName} {lastName}", 1},
    {"{firstName} {lastName} {suffix}", 1},
}};

const PeopleNames dutchPeopleNames{{dutchMaleFirstNames, dutchLastNames, dutchMalePrefixes, dutchSuffixes},
                                   {dutchFemaleFirstNames, dutchLastNames, dutchFemalePrefixes, dutchSuffixes},
                                   dutchNameFormats};

// England

const auto englishMaleFirstNames = std::to_array<std::string_view>({
    "James",     "John",      "Robert",      "Michael",   "William",   "David",      "Richard",  "Charles",
    "Joseph",    "Thomas",    "Christopher", "Daniel",    "Paul",      "Mark",       "Donald",   "George",
    "Kenneth",   "Steven",    "Edward",      "Brian",     "Ronald",    "Anthony",    "Kevin",    "Jason",
    "Matthew",   "Gary",      "Timothy",     "Jose",      "Larry",     "Jeffrey",    "Frank",    "Scott",
    "Eric",      "Stephen",   "Andrew",      "Raymond",   "Gregory",   "Joshua",     "Jerry",    "Dennis",
    "Walter",    "Patrick",   "Peter",       "Harold",    "Douglas",   "Henry",      "Carl",     "Arthur",
    "Ryan",      "Roger",     "Joe",         "Juan",      "Jack",      "Albert",     "Jonathan", "Justin",
    "Terry",     "Gerald",    "Keith",       "Samuel",    "Willie",    "Ralph",      "Lawrence", "Nicholas",
    "Roy",       "Benjamin",  "Bruce",       "Brandon",   "Adam",      "Harry",      "Fred",     "Wayne",
    "Billy",     "Steve",     "Louis",       "Jeremy",    "Aaron",     "Randy",      "Howard",   "Eugene",
    "Carlos",    "Russell",   "Bobby",       "Victor",    "Martin",    "Ernest",     "Phillip",  "Todd",
    "Jesse",     "Craig",     "Alan",        "Shawn",     "Clarence",  "Sean",       "Philip",   "Chris",
    "Johnny",    "Earl",      "Jimmy",       "Antonio",   "Danny",     "Bryan",      "Tony",     "Luis",
    "Mike",      "Stanley",   "Leonard",     "Nathan",    "Dale",      "Manuel",     "Rodney",   "Curtis",
    "Norman",    "Allen",     "Marvin",      "Vincent",   "Glenn",     "Jeffery",    "Travis",   "Jeff",
    "Chad",      "Jacob",     "Lee",         "Melvin",    "Alfred",    "Kyle",       "Francis",  "Bradley",
    "Jesus",     "Herbert",   "Frederick",   "Ray",       "Joel",      "Edwin",      "Don",      "Eddie",
    "Ricky",     "Troy",      "Randall",     "Barry",     "Alexander", "Bernard",    "Mario",    "Leroy",
    "Francisco", "Marcus",    "Micheal",     "Theodore",  "Clifford",  "Miguel",     "Oscar",    "Jay",
    "Jim",       "Tom",       "Calvin",      "Alex",      "Jon",       "Ronnie",     "Bill",     "Lloyd",
    "Tommy",     "Leon",      "Derek",       "Warren",    "Darrell",   "Jerome",     "Floyd",    "Leo",
    "Alvin",     "Tim",       "Wesley",      "Gordon",    "Dean",      "Greg",       "Jorge",    "Dustin",
    "Pedro",     "Derrick",   "Dan",         "Lewis",     "Zachary",   "Corey",      "Herman",   "Maurice",
    "Vernon",    "Roberto",   "Clyde",       "Glen",      "Hector",    "Shane",      "Ricardo",  "Sam",
    "Rick",      "Lester",    "Brent",       "Ramon",     "Charlie",   "Tyler",      "Gilbert",  "Gene",
    "Marc",      "Reginald",  "Ruben",       "Brett",     "Angel",     "Nathaniel",  "Rafael",   "Leslie",
    "Edgar",     "Milton",    "Raul",        "Ben",       "Chester",   "Cecil",      "Duane",    "Franklin",
    "Andre",     "Elmer",     "Brad",        "Gabriel",   "Ron",       "Mitchell",   "Roland",   "Arnold",
    "Harvey",    "Jared",     "Adrian",      "Karl",      "Cory",      "Claude",     "Erik",     "Darryl",
    "Jamie",     "Neil",      "Jessie",      "Christian", "Javier",    "Fernando",   "Clinton",  "Ted",
    "Mathew",    "Tyrone",    "Darren",      "Lonnie",    "Lance",     "Cody",       "Julio",    "Kelly",
    "Kurt",      "Allan",     "Nelson",      "Guy",       "Clayton",   "Hugh",       "Max",      "Dwayne",
    "Dwight",    "Armando",   "Felix",       "Jimmie",    "Everett",   "Jordan",     "Ian",      "Wallace",
    "Ken",       "Bob",       "Jaime",       "Casey",     "Alfredo",   "Alberto",    "Dave",     "Ivan",
    "Johnnie",   "Sidney",    "Byron",       "Julian",    "Isaac",     "Morris",     "Clifton",  "Willard",
    "Daryl",     "Ross",      "Virgil",      "Andy",      "Marshall",  "Salvador",   "Perry",    "Kirk",
    "Sergio",    "Marion",    "Tracy",       "Seth",      "Kent",      "Terrance",   "Rene",     "Eduardo",
    "Terrence",  "Enrique",   "Freddie",     "Wade",      "Austin",    "Stuart",     "Fredrick", "Arturo",
    "Alejandro", "Jackie",    "Joey",        "Nick",      "Luther",    "Wendell",    "Jeremiah", "Evan",
    "Julius",    "Dana",      "Donnie",      "Otis",      "Shannon",   "Trevor",     "Oliver",   "Luke",
    "Homer",     "Gerard",    "Doug",        "Kenny",     "Hubert",    "Angelo",     "Shaun",    "Lyle",
    "Matt",      "Lynn",      "Alfonso",     "Orlando",   "Rex",       "Carlton",    "Ernesto",  "Cameron",
    "Neal",      "Pablo",     "Lorenzo",     "Omar",      "Wilbur",    "Blake",      "Grant",    "Horace",
    "Roderick",  "Kerry",     "Abraham",     "Willis",    "Rickey",    "Jean",       "Ira",      "Andres",
    "Cesar",     "Johnathan", "Malcolm",     "Rudolph",   "Damon",     "Kelvin",     "Rudy",     "Preston",
    "Alton",     "Archie",    "Marco",       "Wm",        "Pete",      "Randolph",   "Garry",    "Geoffrey",
    "Jonathon",  "Felipe",    "Bennie",      "Gerardo",   "Ed",        "Dominic",    "Robin",    "Loren",
    "Delbert",   "Colin",     "Guillermo",   "Earnest",   "Lucas",     "Benny",      "Noel",     "Spencer",
    "Rodolfo",   "Myron",     "Edmund",      "Garrett",   "Salvatore", "Cedric",     "Lowell",   "Gregg",
    "Sherman",   "Wilson",    "Devin",       "Sylvester", "Kim",       "Roosevelt",  "Israel",   "Jermaine",
    "Forrest",   "Wilbert",   "Leland",      "Simon",     "Guadalupe", "Clark",      "Irving",   "Carroll",
    "Bryant",    "Owen",      "Rufus",       "Woodrow",   "Sammy",     "Kristopher", "Mack",     "Levi",
    "Marcos",    "Gustavo",   "Jake",        "Lionel",    "Marty",     "Taylor",     "Ellis",    "Dallas",
    "Gilberto",  "Clint",     "Nicolas",     "Laurence",  "Ismael",    "Orville",    "Drew",     "Jody",
    "Ervin",     "Dewey",     "Al",          "Wilfred",   "Josh",      "Hugo",       "Ignacio",  "Caleb",
    "Tomas",     "Sheldon",   "Erick",       "Frankie",   "Stewart",   "Doyle",      "Darrel",   "Rogelio",
    "Terence",   "Santiago",  "Alonzo",      "Elias",     "Bert",      "Elbert",     "Ramiro",   "Conrad",
    "Pat",       "Noah",      "Grady",       "Phil",      "Cornelius", "Lamar",      "Rolando",  "Clay",
    "Percy",     "Dexter",    "Bradford",    "Merle",     "Darin",     "Amos",       "Terrell",  "Moses",
    "Irvin",     "Saul",      "Roman",       "Darnell",   "Randal",    "Tommie",     "Timmy",    "Darrin",
    "Winston",   "Brendan",   "Toby",        "Van",       "Abel",      "Dominick",   "Boyd",     "Courtney",
    "Jan",       "Emilio",    "Elijah",      "Cary",      "Domingo",   "Santos",     "Aubrey",   "Emmett",
    "Marlon",    "Emanuel",   "Jerald",      "Edmond",
});

const auto englishFemaleFirstNames = std::to_array<std::string_view>({
    "Mary",       "Patricia",   "Linda",     "Barbara",   "Elizabeth", "Jennifer",   "Maria",     "Susan",
    "Margaret",   "Dorothy",    "Lisa",      "Nancy",     "Karen",     "Betty",      "Helen",     "Sandra",
    "Donna",      "Carol",      "Ruth",      "Sharon",    "Michelle",  "Laura",      "Sarah",     "Kimberly",
    "Deborah",    "Jessica",    "Shirley",   "Cynthia",   "Angela",    "Melissa",    "Brenda",    "Amy",
    "Anna",       "Rebecca",    "Virginia",  "Kathleen",  "Pamela",    "Martha",     "Debra",     "Amanda",
    "Stephanie",  "Carolyn",    "Christine", "Marie",     "Janet",     "Catherine",  "Frances",   "Ann",
    "Joyce",      "Diane",      "Alice",     "Julie",     "Heather",   "Teresa",     "Doris",     "Gloria",
    "Evelyn",     "Jean",       "Cheryl",    "Mildred",   "Katherine", "Joan",       "Ashley",    "Judith",
    "Rose",       "Janice",     "Kelly",     "Nicole",    "Judy",      "Christina",  "Kathy",     "Theresa",
    "Beverly",    "Denise",     "Tammy",     "Irene",     "Jane",      "Lori",       "Rachel",    "Marilyn",
    "Andrea",     "Kathryn",    "Louise",    "Sara",      "Anne",      "Jacqueline", "Wanda",     "Bonnie",
    "Julia",      "Ruby",       "Lois",      "Tina",      "Phyllis",   "Norma",      "Paula",     "Diana",
    "Annie",      "Lillian",    "Emily",     "Robin",     "Peggy",     "Crystal",    "Gladys",    "Rita",
    "Dawn",       "Connie",     "Florence",  "Tracy",     "Edna",      "Tiffany",    "Carmen",    "Rosa",
    "Cindy",      "Grace",      "Wendy",     "Victoria",  "Edith",     "Kim",        "Sherry",    "Sylvia",
    "Josephine",  "Thelma",     "Shannon",   "Sheila",    "Ethel",     "Ellen",      "Elaine",    "Marjorie",
    "Carrie",     "Charlotte",  "Monica",    "Esther",    "Pauline",   "Emma",       "Juanita",   "Anita",
    "Rhonda",     "Hazel",      "Amber",     "Eva",       "Debbie",    "April",      "Leslie",    "Clara",
    "Lucille",    "Jamie",      "Joanne",    "Eleanor",   "Valerie",   "Danielle",   "Megan",     "Alicia",
    "Suzanne",    "Michele",    "Gail",      "Bertha",    "Darlene",   "Veronica",   "Jill",      "Erin",
    "Geraldine",  "Lauren",     "Cathy",     "Joann",     "Lorraine",  "Lynn",       "Sally",     "Regina",
    "Erica",      "Beatrice",   "Dolores",   "Bernice",   "Audrey",    "Yvonne",     "Annette",   "June",
    "Samantha",   "Marion",     "Dana",      "Stacy",     "Ana",       "Renee",      "Ida",       "Vivian",
    "Roberta",    "Holly",      "Brittany",  "Melanie",   "Loretta",   "Yolanda",    "Jeanette",  "Laurie",
    "Katie",      "Kristen",    "Vanessa",   "Alma",      "Sue",       "Elsie",      "Beth",      "Jeanne",
    "Vicki",      "Carla",      "Tara",      "Rosemary",  "Eileen",    "Terri",      "Gertrude",  "Lucy",
    "Tonya",      "Ella",       "Stacey",    "Wilma",     "Gina",      "Kristin",    "Jessie",    "Natalie",
    "Agnes",      "Vera",       "Willie",    "Charlene",  "Bessie",    "Delores",    "Melinda",   "Pearl",
    "Arlene",     "Maureen",    "Colleen",   "Allison",   "Tamara",    "Joy",        "Georgia",   "Constance",
    "Lillie",     "Claudia",    "Jackie",    "Marcia",    "Tanya",     "Nellie",     "Minnie",    "Marlene",
    "Heidi",      "Glenda",     "Lydia",     "Viola",     "Courtney",  "Marian",     "Stella",    "Caroline",
    "Dora",       "Jo",         "Vickie",    "Mattie",    "Terry",     "Maxine",     "Irma",      "Mabel",
    "Marsha",     "Myrtle",     "Lena",      "Christy",   "Deanna",    "Patsy",      "Hilda",     "Gwendolyn",
    "Jennie",     "Nora",       "Margie",    "Nina",      "Cassandra", "Leah",       "Penny",     "Kay",
    "Priscilla",  "Naomi",      "Carole",    "Brandy",    "Olga",      "Billie",     "Dianne",    "Tracey",
    "Leona",      "Jenny",      "Felicia",   "Sonia",     "Miriam",    "Velma",      "Becky",     "Bobbie",
    "Violet",     "Kristina",   "Toni",      "Misty",     "Mae",       "Shelly",     "Daisy",     "Ramona",
    "Sherri",     "Erika",      "Katrina",   "Claire",    "Lindsey",   "Lindsay",    "Geneva",    "Guadalupe",
    "Belinda",    "Margarita",  "Sheryl",    "Cora",      "Faye",      "Ada",        "Natasha",   "Sabrina",
    "Isabel",     "Marguerite", "Hattie",    "Harriet",   "Molly",     "Cecilia",    "Kristi",    "Brandi",
    "Blanche",    "Sandy",      "Rosie",     "Joanna",    "Iris",      "Eunice",     "Angie",     "Inez",
    "Lynda",      "Madeline",   "Amelia",    "Alberta",   "Genevieve", "Monique",    "Jodi",      "Janie",
    "Maggie",     "Kayla",      "Sonya",     "Jan",       "Lee",       "Kristine",   "Candace",   "Fannie",
    "Maryann",    "Opal",       "Alison",    "Yvette",    "Melody",    "Luz",        "Susie",     "Olivia",
    "Flora",      "Shelley",    "Kristy",    "Mamie",     "Lula",      "Lola",       "Verna",     "Beulah",
    "Antoinette", "Candice",    "Juana",     "Jeannette", "Pam",       "Kelli",      "Hannah",    "Whitney",
    "Bridget",    "Karla",      "Celia",     "Latoya",    "Patty",     "Shelia",     "Gayle",     "Della",
    "Vicky",      "Lynne",      "Sheri",     "Marianne",  "Kara",      "Jacquelyn",  "Erma",      "Blanca",
    "Myra",       "Leticia",    "Pat",       "Krista",    "Roxanne",   "Angelica",   "Johnnie",   "Robyn",
    "Francis",    "Adrienne",   "Rosalie",   "Alexandra", "Brooke",    "Bethany",    "Sadie",     "Bernadette",
    "Traci",      "Jody",       "Kendra",    "Jasmine",   "Nichole",   "Rachael",    "Chelsea",   "Mable",
    "Ernestine",  "Muriel",     "Marcella",  "Elena",     "Krystal",   "Angelina",   "Nadine",    "Kari",
    "Estelle",    "Dianna",     "Paulette",  "Lora",      "Mona",      "Doreen",     "Rosemarie", "Angel",
    "Desiree",    "Antonia",    "Hope",      "Ginger",    "Janis",     "Betsy",      "Christie",  "Freda",
    "Mercedes",   "Meredith",   "Lynette",   "Teri",      "Cristina",  "Eula",       "Leigh",     "Meghan",
    "Sophia",     "Eloise",     "Rochelle",  "Gretchen",  "Cecelia",   "Raquel",     "Henrietta", "Alyssa",
    "Jana",       "Kelley",     "Gwen",      "Kerry",     "Jenna",     "Tricia",     "Laverne",   "Olive",
    "Alexis",     "Tasha",      "Silvia",    "Elvira",    "Casey",     "Delia",      "Sophie",    "Kate",
    "Patti",      "Lorena",     "Kellie",    "Sonja",     "Lila",      "Lana",       "Darla",     "May",
    "Mindy",      "Essie",      "Mandy",     "Lorene",    "Elsa",      "Josefina",   "Jeannie",   "Miranda",
    "Dixie",      "Lucia",      "Marta",     "Faith",     "Lela",      "Johanna",    "Shari",     "Camille",
    "Tami",       "Shawna",     "Elisa",     "Ebony",     "Melba",     "Ora",        "Nettie",    "Tabitha",
    "Ollie",      "Jaime",      "Winifred",  "Kristie",
});

const auto englishLastNames = std::to_array<std::string_view>({
    "Abbott",      "Abernathy",     "Abshire",     "Adams",       "Altenwerth", "Anderson",     "Ankunding",
    "Armstrong",   "Auer",          "Aufderhar",   "Bahringer",   "Bailey",     "Balistreri",   "Barrows",
    "Bartell",     "Bartoletti",    "Barton",      "Bashirian",   "Batz",       "Bauch",        "Baumbach",
    "Bayer",       "Beahan",        "Beatty",      "Bechtelar",   "Becker",     "Bednar",       "Beer",
    "Beier",       "Berge",         "Bergnaum",    "Bergstrom",   "Bernhard",   "Bernier",      "Bins",
    "Blanda",      "Blick",         "Block",       "Bode",        "Boehm",      "Bogan",        "Bogisich",
    "Borer",       "Bosco",         "Botsford",    "Boyer",       "Boyle",      "Bradtke",      "Brakus",
    "Braun",       "Breitenberg",   "Brekke",      "Brown",       "Bruen",      "Buckridge",    "Carroll",
    "Carter",      "Cartwright",    "Casper",      "Cassin",      "Champlin",   "Christiansen", "Cole",
    "Collier",     "Collins",       "Conn",        "Connelly",    "Conroy",     "Considine",    "Corkery",
    "Cormier",     "Corwin",        "Cremin",      "Crist",       "Crona",      "Cronin",       "Crooks",
    "Cruickshank", "Cummerata",     "Cummings",    "Dach",        "Daniel",     "Dare",         "Daugherty",
    "Davis",       "Deckow",        "Denesik",     "Dibbert",     "Dickens",    "Dicki",        "Dickinson",
    "Dietrich",    "Donnelly",      "Dooley",      "Douglas",     "Doyle",      "DuBuque",      "Durgan",
    "Ebert",       "Effertz",       "Emard",       "Emmerich",    "Erdman",     "Ernser",       "Fadel",
    "Fahey",       "Farrell",       "Fay",         "Feeney",      "Feest",      "Feil",         "Ferry",
    "Fisher",      "Flatley",       "Frami",       "Franecki",    "Franey",     "Friesen",      "Fritsch",
    "Funk",        "Gerhold",       "Gerlach",     "Gibson",      "Gislason",   "Gleason",      "Gleichner",
    "Glover",      "Goldner",       "Goodwin",     "Gorczany",    "Gottlieb",   "Goyette",      "Grady",
    "Graham",      "Grant",         "Green",       "Greenfelder", "Greenholt",  "Grimes",       "Gulgowski",
    "Gusikowski",  "Gutkowski",     "Gutmann",     "Haag",        "Hackett",    "Hagenes",      "Hahn",
    "Haley",       "Halvorson",     "Hamill",      "Hammes",      "Hand",       "Hane",         "Hansen",
    "Harber",      "Harris",        "Hartmann",    "Harvey",      "Hauck",      "Hayes",        "Heaney",
    "Heathcote",   "Hegmann",       "Heidenreich", "Heller",      "Herman",     "Hermann",      "Hermiston",
    "Herzog",      "Hessel",        "Hettinger",   "Hickle",      "Hilll",      "Hills",        "Hilpert",
    "Hintz",       "Hirthe",        "Hodkiewicz",  "Hoeger",      "Homenick",   "Hoppe",        "Howe",
    "Howell",      "Hudson",        "Huel",        "Huels",       "Hyatt",      "Jacobi",       "Jacobs",
    "Jacobson",    "Jakubowski",    "Jaskolski",   "Jast",        "Jenkins",    "Jerde",        "Johns",
    "Johnson",     "Johnston",      "Jones",       "Kassulke",    "Kautzer",    "Keebler",      "Keeling",
    "Kemmer",      "Kerluke",       "Kertzmann",   "Kessler",     "Kiehn",      "Kihn",         "Kilback",
    "King",        "Kirlin",        "Klein",       "Kling",       "Klocko",     "Koch",         "Koelpin",
    "Koepp",       "Kohler",        "Konopelski",  "Koss",        "Kovacek",    "Kozey",        "Krajcik",
    "Kreiger",     "Kris",          "Kshlerin",    "Kub",         "Kuhic",      "Kuhlman",      "Kuhn",
    "Kulas",       "Kunde",         "Kunze",       "Kuphal",      "Kutch",      "Kuvalis",      "Labadie",
    "Lakin",       "Lang",          "Langosh",     "Langworth",   "Larkin",     "Larson",       "Leannon",
    "Lebsack",     "Ledner",        "Leffler",     "Legros",      "Lehner",     "Lemke",        "Lesch",
    "Leuschke",    "Lind",          "Lindgren",    "Littel",      "Little",     "Lockman",      "Lowe",
    "Lubowitz",    "Lueilwitz",     "Luettgen",    "Lynch",       "Macejkovic", "MacGyver",     "Maggio",
    "Mann",        "Mante",         "Marks",       "Marquardt",   "Marvin",     "Mayer",        "Mayert",
    "McClure",     "McCullough",    "McDermott",   "McGlynn",     "McKenzie",   "McLaughlin",   "Medhurst",
    "Mertz",       "Metz",          "Miller",      "Mills",       "Mitchell",   "Moen",         "Mohr",
    "Monahan",     "Moore",         "Morar",       "Morissette",  "Mosciski",   "Mraz",         "Mueller",
    "Muller",      "Murazik",       "Murphy",      "Murray",      "Nader",      "Nicolas",      "Nienow",
    "Nikolaus",    "Nitzsche",      "Nolan",       "Olson",       "Ondricka",   "Orn",          "Ortiz",
    "Osinski",     "Pacocha",       "Padberg",     "Pagac",       "Parisian",   "Parker",       "Paucek",
    "Pfeffer",     "Pollich",       "Pouros",      "Powlowski",   "Predovic",   "Price",        "Prohaska",
    "Prosacco",    "Purdy",         "Quigley",     "Quitzon",     "Rath",       "Ratke",        "Rau",
    "Raynor",      "Reichel",       "Reichert",    "Reilly",      "Reinger",    "Rempel",       "Renner",
    "Reynolds",    "Rice",          "Rippin",      "Ritchie",     "Robel",      "Roberts",      "Rodriguez",
    "Rogahn",      "Rohan",         "Rolfson",     "Romaguera",   "Roob",       "Rosenbaum",    "Rowe",
    "Ruecker",     "Runolfsdottir", "Runolfsson",  "Runte",       "Russel",     "Rutherford",   "Ryan",
    "Sanford",     "Satterfield",   "Sauer",       "Sawayn",      "Schaden",    "Schaefer",     "Schamberger",
    "Schiller",    "Schimmel",      "Schinner",    "Schmeler",    "Schmidt",    "Schmitt",      "Schneider",
    "Schoen",      "Schowalter",    "Schroeder",   "Schulist",    "Schultz",    "Schumm",       "Schuppe",
    "Schuster",    "Senger",        "Shanahan",    "Shields",     "Simonis",    "Sipes",        "Skiles",
    "Smith",       "Smitham",       "Spencer",     "Spinka",      "Sporer",     "Stamm",        "Stanton",
    "Stark",       "Stehr",         "Steuber",     "Stiedemann",  "Stokes",     "Stoltenberg",  "Stracke",
    "Streich",     "Stroman",       "Strosin",     "Swaniawski",  "Swift",      "Terry",        "Thiel",
    "Thompson",    "Tillman",       "Torp",        "Torphy",      "Towne",      "Toy",          "Trantow",
    "Tremblay",    "Treutel",       "Tromp",       "Turcotte",    "Turner",     "Ullrich",      "Upton",
    "Vandervort",  "Veum",          "Volkman",     "Von",         "VonRueden",  "Waelchi",      "Walker",
    "Walsh",       "Walter",        "Ward",        "Waters",      "Watsica",    "Weber",        "Wehner",
    "Weimann",     "Weissnat",      "Welch",       "West",        "White",      "Wiegand",      "Wilderman",
    "Wilkinson",   "Will",          "Williamson",  "Willms",      "Windler",    "Wintheiser",   "Wisoky",
    "Wisozk",      "Witting",       "Wiza",        "Wolf",        "Wolff",      "Wuckert",      "Wunsch",
    "Wyman",       "Yost",          "Yundt",       "Zboncak",     "Zemlak",     "Ziemann",      "Zieme",
    "Zulauf",
});

const auto englishSuffixes =
    std::to_array<std::string_view>({"Jr.", "Sr.", "I", "II", "III", "IV", "V", "MD", "DDS", "PhD", "DVM"});

const auto englishMalePrefixes = std::to_array<std::string_view>({"Mr.", "Dr."});

const auto englishFemalePrefixes = std::to_array<std::string_view>({"Mrs.", "Ms.", "Miss", "Dr."});

const NameFormats englishNameFormats{{{"{firstName} {lastName}", 49},
                                      {"{prefix} {firstName} {lastName}", 7},
                                      {"{firstName} {lastName} {suffix}", 7},
                                      {"{prefix} {firstName} {lastName} {suffix}", 1}}};

const PeopleNames englishPeopleNames{
    {englishMaleFirstNames, englishLastNames, englishMalePrefixes, englishSuffixes},
    {englishFemaleFirstNames, englishLastNames, englishFemalePrefixes, englishSuffixes},
    englishNameFormats};

// Estonia

const auto estonianMaleFirstNames = std::to_array<std::string_view>({
    "Aleksandr",  "Andres",    "Sergei",    "Vladimir",  "Andrei",     "Igor",        "Martin",    "Marko",
    "Ene",        "Dmitri",    "Aleksei",   "Oleg",      "Viktor",     "Juri",        "Roman",     "Marek",
    "Raul",       "Malle",     "Nikolai",   "Heli",      "Aleksander", "Alexander",   "Sven",      "Terje",
    "Pavel",      "Eduard",    "Kai",       "Sergey",    "Anton",      "Artur",       "Vadim",     "Maksim",
    "Timo",       "Ivar",      "Allan",     "Erik",      "Karl",       "Anatoli",     "Vitali",    "Ivo",
    "Jan",        "Silver",    "Hannes",    "Robert",    "Andrey",     "Konstantin",  "Rainer",    "Harri",
    "Ivan",       "Sander",    "Stanislav", "Paul",      "Oliver",     "Mark",        "Hans",      "Peter",
    "Lars",       "Matti",     "Alexey",    "Gunnar",    "Dmitry",     "Deniss",      "Ilja",      "Leonid",
    "Vladislav",  "Kirill",    "Mikhail",   "Ruslan",    "Andre",      "Henri",       "Gert",      "Roland",
    "Alex",       "Ragnar",    "Imre",      "Vassili",   "Arne",       "Arno",        "Boris",     "Michael",
    "Victor",     "Daniel",    "Rasmus",    "David",     "Valentin",   "Mario",       "Yury",      "John",
    "Joel",       "Edgar",     "Markus",    "Harry",     "Thomas",     "Einar",       "Leo",       "Albert",
    "Johannes",   "Evgeny",    "Per",       "Hendrik",   "Nikolay",    "Ari",         "Georg",     "Anders",
    "Egon",       "Richard",   "Alvar",     "Georgi",    "Henry",      "Oleksandr",   "Guido",     "Arnold",
    "Ervin",      "Henrik",    "Merlin",    "Valter",    "Raimond",    "Karel",       "Elmar",     "Maxim",
    "Aleksandrs", "Carl",      "Kristian",  "Fred",      "Johan",      "Nils",        "Sergejs",   "Kevin",
    "Daniil",     "Christian", "Anatoly",   "Aleks",     "Tom",        "Magnus",      "Oskar",     "German",
    "Andris",     "Grigori",   "Olav",      "Tomas",     "Andrew",     "Marco",       "Ilya",      "Sami",
    "Alan",       "Andrejs",   "Rudolf",    "Stefan",    "Marti",      "Bruno",       "Rolf",      "Artem",
    "Erich",      "Vitaly",    "Elmo",      "Valeriy",   "Vyacheslav", "Udo",         "Volodymyr", "Steven",
    "Heiko",      "Gennady",   "Ken",       "Harald",    "Jens",       "Arkadi",      "Ulf",       "Mauri",
    "Ralf",       "Danel",     "August",    "Lennart",   "Hugo",       "James",       "Otto",      "Simo",
    "Andi",       "Aldo",      "Jonas",     "Ove",       "Mikael",     "Marius",      "Aksel",     "Mattias",
    "Helmut",     "Ali",       "Roberto",   "Timur",     "Feliks",     "Lasse",       "Holger",    "Stig",
    "Lev",        "Ronald",    "Leif",      "Vlad",      "Helge",      "Marin",       "Vasily",    "Dmytro",
    "Roger",      "Chris",     "Klaus",     "Kjell",     "Igors",      "Stepan",      "Gustav",    "Alexandre",
    "Rune",       "William",   "Mihail",    "Andrii",    "Armin",      "Juris",       "Max",       "Aleksey",
    "Tony",       "Vladimirs", "Alfred",    "Eric",      "Dmitrijs",   "Edward",      "Aleksejs",  "Kris",
    "Alexandr",   "Edvin",     "Gleb",      "Simon",     "Dmitriy",    "Kenneth",     "Jakob",     "Mykola",
    "Petr",       "Patrick",   "Artyom",    "Gabriel",   "Aliaksandr", "Olegs",       "Ander",     "Dimitri",
    "Elvis",      "Andro",     "Viktors",   "Siarhei",   "Pauli",      "Ole",         "Niklas",    "Stephen",
    "Keith",      "Arthur",    "Mindaugas", "Olaf",      "Bogdan",     "Christopher", "Frank",     "Yuriy",
    "Sergii",     "Mirko",     "Remo",      "Paolo",     "Andriy",     "Marcus",      "Jurijs",    "Matthias",
    "Heinrich",   "Ernst",     "Marat",     "Bert",      "Egor",       "Morten",      "Niko",      "Edvard",
    "Fredrik",    "Iurii",     "Darius",    "Georgy",    "Viacheslav", "Antonio",     "Evgeniy",   "Filipp",
    "Jon",        "Andrius",   "Eugen",     "Jaroslav",  "Edgars",     "George",      "Algis",     "Kurt",
    "Emil",       "Niels",     "Norman",    "Ian",       "Andry",      "Tommy",       "Rustam",    "Pavlo",
    "Valerii",    "Evgeni",    "Jurgen",    "Sebastian", "Herbert",    "Pierre",      "Oleksii",   "Marc",
    "Samuel",     "Adam",      "Patrik",    "Aron",      "Hakan",      "Serguei",     "Petro",     "Franz",
    "Vasyl",      "Sergej",    "Benjamin",  "Brian",     "Joakim",     "Vitalii",     "Benno",     "Carlos",
    "Sergiy",     "Vitaliy",   "Jack",      "Knut",      "Rodion",     "Enrico",      "Grigory",   "Herman",
    "Philip",     "Jose",      "Tim",       "Omar",      "Armand",     "Vilmar",      "Rafael",    "Adrian",
    "Renat",      "Anatolii",  "Dzmitry",   "Danil",     "Eldar",      "Gerhard",     "Fedor",     "Raoul",
    "Sergo",      "Anthony",   "Jesper",    "Matthew",   "Francesco",  "Gary",
});

const auto estonianFemaleFirstNames = std::to_array<std::string_view>({
    "Olga",     "Irina",     "Jelena",     "Svetlana",   "Tatjana",    "Katrin",    "Natalja",   "Anna",
    "Anne",     "Marina",    "Galina",     "Maria",      "Julia",      "Anneli",    "Valentina", "Marika",
    "Karin",    "Natalia",   "Kristina",   "Eve",        "Helen",      "Merle",     "Margit",    "Mati",
    "Annika",   "Jana",      "Kaja",       "Kristel",    "Jekaterina", "Elena",     "Mari",      "Valeri",
    "Diana",    "Viktoria",  "Krista",     "Ingrid",     "Lea",        "Monika",    "Larissa",   "Oksana",
    "Maris",    "Evelin",    "Maire",      "Helle",      "Inna",       "Alla",      "Inga",      "Tamara",
    "Signe",    "Eva",       "Rita",       "Linda",      "Aili",       "Ilona",     "Rene",      "Tatiana",
    "Laura",    "Veronika",  "Janne",      "Kati",       "Aleksandra", "Jane",      "Aime",      "Margarita",
    "Helena",   "Ruth",      "Erika",      "Eda",        "Denis",      "Elina",     "Alina",     "Heidi",
    "Inge",     "Kari",      "Sigrid",     "Anastassia", "Regina",     "Agnes",     "Maia",      "Ester",
    "Margo",    "Birgit",    "Mai",        "Sandra",     "Ellen",      "Karina",    "Ekaterina", "Leida",
    "Marit",    "Angela",    "Nele",       "Nadezda",    "Asta",       "Karmen",    "Liudmila",  "Marianne",
    "Nina",     "Andreas",   "Mirjam",     "Grete",      "Kristin",    "Silvia",    "Egle",      "Hanna",
    "Lidia",    "Mika",      "Raissa",     "Valeria",    "Ksenia",     "Astrid",    "Tea",       "Anastasia",
    "Lilian",   "Nikita",    "Antonina",   "Yulia",      "Anni",       "Zinaida",   "Evelyn",    "Nelli",
    "Lilia",    "Aimar",     "Silva",      "Ann",        "Darja",      "Vera",      "Larisa",    "Janis",
    "Marianna", "Ludmilla",  "Saima",      "Ella",       "Julija",     "Andra",     "Anita",     "Leena",
    "Irene",    "Elen",      "Petri",      "Johanna",    "Helga",      "Margot",    "Stella",    "Marta",
    "Kirsti",   "Mariann",   "Liana",      "Nadezhda",   "Vilma",      "Arina",     "Kaur",      "Angelika",
    "Marie",    "Polina",    "Elis",       "Lia",        "Monica",     "Angelina",  "Gerda",     "Bo",
    "Elvira",   "Valery",    "Yana",       "Liubov",     "Victoria",   "Alice",     "Dagmar",    "Natalija",
    "Rimma",    "Olena",     "Irena",      "Marion",     "Ulla",       "Thea",      "Maila",     "Carmen",
    "Irma",     "Sofia",     "Berit",      "Ursula",     "Kristine",   "Oxana",     "Kelly",     "Lena",
    "Tatyana",  "Alena",     "Viktorija",  "Erna",       "Dina",       "Alexandra", "Natalya",   "Leila",
    "Karen",    "Elsa",      "Yuliya",     "Katerina",   "Iris",       "Marja",     "Agne",      "Renee",
    "Alisa",    "Helene",    "Alevtina",   "Hilda",      "Nataliya",   "Kerstin",   "Katja",     "Marilin",
    "Ira",      "Doris",     "Kim",        "Viktoriya",  "Tanja",      "Lina",      "Paula",     "Edith",
    "Katrina",  "Zhanna",    "Iryna",      "Christina",  "Dan",        "Pia",       "Eleonora",  "Dima",
    "Gea",      "Elisabeth", "Marian",     "Selma",      "Andrea",     "Ilse",      "Ruta",      "Albina",
    "Sasha",    "Ines",      "Aina",       "Evelina",    "Marija",     "Marleen",   "Ada",       "Susanna",
    "Tetiana",  "Katarina",  "Liza",       "Aira",       "Natali",     "Elin",      "Inessa",    "Lyudmila",
    "Ethel",    "Vivian",    "Ludmila",    "Anette",     "Karolina",   "Evgenia",   "Karol",     "Marili",
    "Iuliia",   "Kristen",   "Juliana",    "Reena",      "Britta",     "Lilli",     "Mariana",   "Anastasiya",
    "Mariya",   "Iveta",     "Marita",     "Elise",      "Lydia",      "Emma",      "Alma",      "Marlen",
    "Violetta", "Lily",      "Iren",       "Raisa",      "Nataliia",   "Evgeniya",  "Elizaveta", "Mia",
    "Mary",     "Naima",     "Lee",        "Renata",     "Greta",      "Daisy",     "Varvara",   "Raja",
    "Yuri",     "Iraida",    "Ilze",       "Robin",      "Britt",      "Renate",    "Daria",     "Anzelika",
    "Kira",     "Karine",    "Tetyana",    "Anya",       "Dana",       "Alyona",    "Svitlana",  "Nadja",
    "Livia",    "Ina",       "Ana",        "Katya",      "Lisa",       "Ksenija",   "Janina",    "Lada",
    "Maryna",   "Merlyn",    "Lana",       "Veronica",   "Alissa",     "Elga",      "Dennis",    "Sanna",
    "Maren",    "Brit",      "Laila",      "Enna",       "Emilia",     "Natasha",   "Nika",      "Inger",
    "Olesya",   "Rena",      "Susanne",    "Dasha",      "Nora",       "Kateryna",  "Mae",       "Dea",
    "Heidy",    "Larysa",    "Anastasiia", "Joanna",     "Anett",      "Magda",     "Natalie",   "Nataly",
    "Ida",      "Faina",     "Roza",       "Milana",     "Lili",       "Amanda",
});

const auto estonianLastNames = std::to_array<std::string_view>({
    "Tamm",       "Saar",      "Sepp",        "Kask",         "Mägi",       "Kukk",       "Rebane",     "Koppel",
    "Karu",       "Ilves",     "Lepik",       "Oja",          "Ivanov",     "Kuusk",      "Kaasik",     "Cuca",
    "Luik",       "Raudsepp",  "Vaher",       "Kallas",       "Lepp",       "Laur",       "Pärn",       "Kuusik",
    "Kivi",       "Ots",       "Liiv",        "Peterson",     "Teder",      "Mets",       "Põder",      "Kull",
    "Kütt",       "Puusepp",   "Kangur",      "Jakobson",     "Rand",       "Toom",       "Saks",       "Smirnov",
    "Leppik",     "Raud",      "Mitt",        "Sild",         "Sarapuu",    "Ivanova",    "Mölder",     "Roos",
    "Tamme",      "Allik",     "Kõiv",        "Paju",         "Paas",       "Laas",       "Pukk",       "Miller",
    "Palm",       "Tomson",    "Uibo",        "Põld",         "Jürgenson",  "Kalda",      "Lill",       "Männik",
    "Nõmm",       "Johanson",  "Orav",        "Unt",          "Parts",      "Lind",       "Hunt",       "Petrov",
    "Arro",       "Adamson",   "Hein",        "Lember",       "Rätsep",     "Järv",       "Nurk",       "Tali",
    "Pihlak",     "Kase",      "Aavik",       "Kruus",        "Valk",       "Hansen",     "Järve",      "Väli",
    "Vares",      "Aru",       "Rohtla",      "Kikas",        "Sikk",       "Mõttus",     "Soosaar",    "Kuznetsov",
    "Vassiljev",  "Kolk",      "Must",        "Sarv",         "Martin",     "Mänd",       "Annus",      "Kala",
    "Tilk",       "Luts",      "Varik",       "Tomingas",     "Raid",       "Smirnova",   "Pavlov",     "Kirsipuu",
    "Laine",      "Aas",       "Anderson",    "Lang",         "Kaur",       "Anton",      "Post",       "Aus",
    "Abel",       "Ojala",     "Kokk",        "Palu",         "Schmidt",    "Kuus",       "Kink",       "Gross",
    "Popov",      "Ott",       "Martinson",   "Rosenberg",    "Roots",      "Reimann",    "Sokolov",    "Allas",
    "Toome",      "Volkov",    "Sander",      "Kirs",         "Lipp",       "Põldma",     "Taal",       "Hanson",
    "Kalmus",     "Paal",      "Petrova",     "Link",         "Berg",       "Aun",        "Simson",     "Müürsepp",
    "Eller",      "Liivak",    "Kurg",        "Stepanov",     "Laan",       "Kass",       "Trei",       "Leis",
    "Koit",       "Morozov",   "Susi",        "Raja",         "Lumi",       "Meier",      "Vain",       "Lass",
    "Kim",        "Aasa",      "Parve",       "Mark",         "Lauri",      "Kozlov",     "Sutt",       "Soo",
    "Erm",        "Lebedev",   "Saul",        "Holm",         "Kuznetsova", "Popova",     "Poom",       "Soots",
    "Andersson",  "Jürgens",   "Kelder",      "Kalm",         "Eriksson",   "Kont",       "Kroon",      "Kutsar",
    "Pajula",     "Kasak",     "Salu",        "Koger",        "Nikitin",    "Peets",      "Siim",       "Johansson",
    "Ader",       "Savi",      "Novikov",     "Alas",         "Padar",      "Muru",       "Silm",       "Suvi",
    "Rosin",      "Kuzmin",    "Aleksandrov", "Markus",       "Zaitsev",    "Pihl",       "Joost",      "Meister",
    "Pavlova",    "Viik",      "Karp",        "Remmel",       "Orlov",      "Piho",       "Braun",      "Raidma",
    "Maksimov",   "Bergmann",  "Kool",        "Laks",         "Romanov",    "Li",         "Filippov",   "Antonov",
    "Hendrikson", "Talvik",    "Laos",        "Kruuse",       "Reinhold",   "Virtanen",   "Borissov",   "Solovjov",
    "Oks",        "Malm",      "Moor",        "Trofimov",     "Kovalenko",  "Türk",       "Tomberg",    "Jensen",
    "Noor",       "Fjodorova", "Kapp",        "Paap",         "Treier",     "Savolainen", "Bondarenko", "Kalle",
    "Rist",       "Ross",      "Stepanova",   "Larsson",      "Smith",      "Paulus",     "Orlova",     "Sillaots",
    "Jaanus",     "Vene",      "Semjonova",   "Vahi",         "Pent",       "Tisler",     "Volkova",    "Nilsson",
    "Lee",        "Roose",     "Janson",      "Makarov",      "Lehtonen",   "Kaup",       "Lille",      "Vill",
    "Frolov",     "Sisask",    "Erik",        "Raun",         "Karlsson",   "Belov",      "Loo",        "Vinogradov",
    "Lehtinen",   "Tasa",      "Pajo",        "Aava",         "Klein",      "Merila",     "Alt",        "Evert",
    "Pool",       "Lukas",     "Lääts",       "Mere",         "Pill",       "Vare",       "Baranov",    "Matt",
    "Steinberg",  "Reiman",    "Bogdanov",    "Pulk",         "Grünberg",   "Press",      "Koch",       "Soon",
    "Mandel",     "Jakovleva", "Andersen",    "Aleksandrova", "Kaldma",     "Sommer",     "Vilu",       "Mustonen",
    "Melnikov",   "Jaakson",   "Korhonen",    "Nieminen",     "Borissova",  "Rinne",      "Makarova",   "Otsa",
    "Hurt",       "Sorokin",   "Valdmann",    "Kanter",       "Kipper",     "Tint",       "Hint",       "Sein",
    "Nael",       "Bachmann",  "Laar",        "Suviste",      "Raig",       "Grigorjeva", "Sergejev",   "Klaas",
    "Nikitina",   "Haas",      "Lebedeva",    "Margus",       "Saag",       "Kirt",
});

const NameFormats estonianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames estonianPeopleNames{{estonianMaleFirstNames, estonianLastNames, {}, {}},
                                      {estonianFemaleFirstNames, estonianLastNames, {}, {}},
                                      estonianNameFormats};

// Finland

const auto finnishMaleFirstNames = std::to_array<std::string_view>({
    "Leo",   "Elias", "Eino",   "Väinö",   "Onni",  "Leevi",  "Niilo",   "Oliver", "Eetu",
    "Noel",  "Eeli",  "Veeti",  "Mikael",  "Joel",  "Daniel", "Andreas", "Edvin",  "Elia",
    "Eliel", "Felix", "Niila",  "Nikolas", "Oskar", "Oula",   "Petja",   "Petrus", "Rene",
    "Teo",   "Venni", "Viktor", "Aarre",   "Aarni", "Ahti",   "Aarne",   "Kuura",  "Kuutti",
    "Oras",  "Otso",  "Pyry",   "Tammi",   "Terho", "Touko",  "Ukko",    "Vesa",   "Visa",
});

const auto finnishFemaleFirstNames = std::to_array<std::string_view>({
    "Aino",  "Eevi",   "Emma",   "Sofia",  "Aada",     "Ella",   "Lilja",  "Helmi",   "Elli",    "Enni",
    "Venla", "Olivia", "Pihla",  "Isla",   "Adele",    "Aida",   "Bertta", "Cecilia", "Daniela", "Elle",
    "Erin",  "Fiona",  "Iita",   "Inna",   "Jasmiina", "Lila",   "Luna",   "Malla",   "Meea",    "Minka",
    "Minni", "Neela",  "Selina", "Unna",   "Aamu",     "Aava",   "Aura",   "Heini",   "Hilla",   "Hilja",
    "Ilma",  "Ilta",   "Iiris",  "Jasmin", "Kaisla",   "Kaneli",
});

const auto finnishLastNames = std::to_array<std::string_view>({
    "Mäkinen",   "Virtanen",   "Nieminen",   "Korhonen",    "Laine",       "Koskinen", "Hämäläinen", "Lehtonen",
    "Heikkinen", "Järvinen",   "Lahtinen",   "Mäkelä",      "Huttunen",    "Aalto",    "Koivisto",   "Salo",
    "Kallio",    "Rinne",      "Lindholm",   "Karjalainen", "Koskela",     "Turunen",  "Rantanen",   "Laitinen",
    "Jokinen",   "Harju",      "Vartiainen", "Hakala",      "Kosonen",     "Lampinen", "Rautio",     "Peltola",
    "Miettinen", "Savolainen", "Nurmi",      "Niemelä",     "Hakkarainen", "Laakso",   "Kinnunen",   "Mattila",
    "Salmi",     "Pekkanen",   "Leinonen",   "Ojala",       "Kankaanpää",  "Heikkilä",
});

const NameFormats finnishNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames finnishPeopleNames{{finnishMaleFirstNames, finnishLastNames, {}, {}},
                                     {finnishFemaleFirstNames, finnishLastNames, {}, {}},
                                     finnishNameFormats};

// France

const auto frenchMaleFirstNames = std::to_array<std::string_view>({
    "Aaron",       "Abdon",      "Abel",        "Abélard",    "Abelin",    "Abondance",    "Abraham",    "Absalon",
    "Acace",       "Achaire",    "Achille",     "Adalard",    "Adalbald",  "Adalbéron",    "Adalbert",   "Adalric",
    "Adam",        "Adegrin",    "Adel",        "Adelin",     "Adelphe",   "Adenet",       "Adéodat",    "Adhémar",
    "Adjutor",     "Adolphe",    "Adonis",      "Adrien",     "Agapet",    "Agathange",    "Agathon",    "Agilbert",
    "Agnan",       "Agrippin",   "Aimable",     "Aimé",       "Alain",     "Alban",        "Albéric",    "Albert",
    "Alcibiade",   "Alcide",     "Alcime",      "Aldonce",    "Aldric",    "Aleaume",      "Alexandre",  "Alexis",
    "Alix",        "Alliaume",   "Almine",      "Almire",     "Aloïs",     "Alphée",       "Alphonse",   "Alpinien",
    "Alverède",    "Amalric",    "Amandin",     "Amant",      "Ambroise",  "Amédée",       "Amélien",    "Amiel",
    "Amour",       "Anaël",      "Anastase",    "Anatole",    "Ancelin",   "Andéol",       "Andoche",    "André",
    "Ange",        "Angilbe",    "Angilran",    "Angoustan",  "Anicet",    "Anne",         "Annibal",    "Ansbert",
    "Anselme",     "Anthelme",   "Antide",      "Antoine",    "Antonin",   "Apollinaire",  "Aquilin",    "Arcade",
    "Archambaud",  "Archange",   "Archibald",   "Arian",      "Ariel",     "Ariste",       "Aristide",   "Armand",
    "Armel",       "Armin",      "Arnould",     "Arnaud",     "Arolde",    "Arsène",       "Arsinoé",    "Arthaud",
    "Arthème",     "Arthur",     "Ascelin",     "Athanase",   "Aubry",     "Audebert",     "Audouin",    "Audran",
    "Auguste",     "Aurèle",     "Aurian",      "Auxence",    "Axel",      "Aymard",       "Aymeric",    "Aymon",
    "Balthazar",   "Baptiste",   "Barnabé",     "Barthélemy", "Bartimée",  "Basile",       "Bastien",    "Baudouin",
    "Bénigne",     "Benjamin",   "Benoît",      "Béranger",   "Bérard",    "Bernard",      "Bertrand",   "Blaise",
    "Bohémond",    "Bon",        "Boniface",    "Bouchard",   "Briac",     "Brice",        "Brieuc",     "Bruno",
    "Calixte",     "Camélien",   "Camille",     "Camillien",  "Candide",   "Caribert",     "Carloman",   "Cassandre",
    "Cassien",     "Cédric",     "Céleste",     "Célestin",   "Célien",    "Césaire",      "César",      "Charles",
    "Charlemagne", "Childebert", "Chilpéric",   "Chrétien",   "Christian", "Christodule",  "Christophe", "Chrysole",
    "Chrysostome", "Clarence",   "Claude",      "Claudien",   "Cléandre",  "Clément",      "Cléry",      "Clotaire",
    "Clovis",      "Colin",      "Côme",        "Constance",  "Constant",  "Constantin",   "Corentin",   "Cyprien",
    "Cyriaque",    "Cyrille",    "Damien",      "Daniel",     "David",     "Delphin",      "Denis",      "Désiré",
    "Didier",      "Dieudonné",  "Dimitri",     "Dominique",  "Dorian",    "Edgard",       "Edmond",     "Édouard",
    "Éleuthère",   "Élie",       "Élisée",      "Élzéar",     "Émeric",    "Émile",        "Emmanuel",   "Enguerrand",
    "Épiphane",    "Éric",       "Ernest",      "Étienne",    "Eubert",    "Eudes",        "Eudoxe",     "Eugène",
    "Eusèbe",      "Eustache",   "Évariste",    "Évrard",     "Fabien",    "Fabrice",      "Falba",      "Fantin",
    "Félicité",    "Félix",      "Ferdinand",   "Fiacre",     "Fidèle",    "Firmin",       "Flavien",    "Flodoard",
    "Florent",     "Florestan",  "Florian",     "Fortuné",    "Foulques",  "Francisque",   "François",   "Frédéric",
    "Fulbert",     "Fulcran",    "Fulgence",    "Gabin",      "Gabriel",   "Gaël",         "Garnier",    "Gaston",
    "Gaspar",      "Gaspard",    "Gatien",      "Gaud",       "Gautier",   "Gédéon",       "Geoffroy",   "Georges",
    "Géraud",      "Gérard",     "Gerbert",     "Germain",    "Gervais",   "Ghislain",     "Gilbert",    "Gilles",
    "Girart",      "Gislebert",  "Gondebaud",   "Gonthier",   "Gontran",   "Gonzague",     "Grégoire",   "Guérin",
    "Gui",         "Guillaume",  "Gustave",     "Guy",        "Guyot",     "Hardouin",     "Hector",     "Hédelin",
    "Hélier",      "Henri",      "Herbert",     "Herluin",    "Hervé",     "Hilaire",      "Hildebert",  "Hincmar",
    "Hippolyte",   "Honoré",     "Hubert",      "Hugues",     "Innocent",  "Isabeau",      "Isidore",    "Jacques",
    "Japhet",      "Jason",      "Jean",        "Jeannel",    "Jeannot",   "Jérémie",      "Jérôme",     "Joachim",
    "Joanny",      "Job",        "Jocelyn",     "Joël",       "Johan",     "Jonas",        "Jonathan",   "Joseph",
    "Josse",       "Josselin",   "Jourdain",    "Jude",       "Judicaël",  "Jules",        "Julien",     "Juste",
    "Justin",      "Lambert",    "Landry",      "Laurent",    "Lazare",    "Léandre",      "Léon",       "Léonard",
    "Léopold",     "Leu",        "Leufroy",     "Libère",     "Liétald",   "Lionel",       "Loïc",       "Longin",
    "Lorrain",     "Lothaire",   "Louis",       "Loup",       "Luc",       "Lucas",        "Lucien",     "Ludolphe",
    "Ludovic",     "Macaire",    "Malo",        "Mamert",     "Manassé",   "Marc",         "Marceau",    "Marcel",
    "Marcelin",    "Marius",     "Martial",     "Martin",     "Mathurin",  "Matthias",     "Matthieu",   "Maugis",
    "Maurice",     "Maxence",    "Maxime",      "Maximilien", "Mayeul",    "Médéric",      "Melchior",   "Mence",
    "Merlin",      "Mérovée",    "Michaël",     "Michel",     "Moïse",     "Morgan",       "Nathan",     "Nathanaël",
    "Narcisse",    "Naudet",     "Néhémie",     "Nestor",     "Nicéphore", "Nicolas",      "Noé",        "Noël",
    "Norbert",     "Normand",    "Octave",      "Odilon",     "Odon",      "Oger",         "Olivier",    "Oury",
    "Pacôme",      "Palémon",    "Parfait",     "Pascal",     "Paterne",   "Patrice",      "Paul",       "Pépin",
    "Perceval",    "Philémon",   "Philibert",   "Philippe",   "Philothée", "Pie",          "Pierre",     "Pierrick",
    "Prosper",     "Quentin",    "Raoul",       "Raphaël",    "Raymond",   "Régis",        "Réjean",     "Rémi",
    "Renaud",      "René",       "Reybaud",     "Richard",    "Robert",    "Roch",         "Rodolphe",   "Rodrigue",
    "Roger",       "Roland",     "Romain",      "Romuald",    "Roméo",     "Ronan",        "Roselin",    "Rachid",
    "Salomon",     "Samuel",     "Sauveur",     "Savin",      "Savinien",  "Scholastique", "Sébastien",  "Séraphin",
    "Serge",       "Séverin",    "Sidoine",     "Sigebert",   "Sigismond", "Silvère",      "Simon",      "Sixte",
    "Stanislas",   "Stéphane",   "Sylvain",     "Sylvestre",  "Tancrède",  "Tanguy",       "Taurin",     "Théodore",
    "Théodose",    "Théophile",  "Théophraste", "Thibault",   "Thibert",   "Thierry",      "Thomas",     "Timoléon",
    "Timothée",    "Titien",     "Tonnin",      "Toussaint",  "Trajan",    "Tristan",      "Turold",     "Tim",
    "Ulysse",      "Urbain",     "Valentin",    "Valère",     "Valéry",    "Venance",      "Venceslas",  "Vianney",
    "Victor",      "Victorien",  "Victorin",    "Vigile",     "Vincent",   "Vital",        "Vivien",     "Waleran",
    "Wandrille",   "Xavier",     "Xénophon",    "Yves",       "Yoann",     "Zacharie",     "Zaché",      "Zéphirin",
});

const auto frenchFemaleFirstNames = std::to_array<std::string_view>({
    "Abdonie",    "Abeline",    "Abigaelle",  "Abigaïl",     "Acacie",     "Acanthe",      "Adalbaude",    "Adalsinde",
    "Adegrine",   "Adélaïde",   "Adèle",      "Adélie",      "Adeline",    "Adeltrude",    "Adolphie",     "Adonise",
    "Adrastée",   "Adrehilde",  "Adrienne",   "Agathe",      "Agilberte",  "Aglaé",        "Agnane",       "Agneflète",
    "Agnès",      "Agrippine",  "Aimée",      "Alaine",      "Alaïs",      "Albane",       "Albérade",     "Alberte",
    "Alcidie",    "Alcine",     "Alcyone",    "Aldegonde",   "Aleth",      "Alexandrine",  "Alexanne",     "Alexine",
    "Alice",      "Aliénor",    "Aliette",    "Aline",       "Alix",       "Alizé",        "Aloïse",       "Aloyse",
    "Alphonsine", "Althée",     "Amaliane",   "Amalthée",    "Amande",     "Amandine",     "Amante",       "Amarande",
    "Amaranthe",  "Amaryllis",  "Ambre",      "Ambroisie",   "Améliane",   "Amélie",       "Ameline",      "Améthyste",
    "Aminte",     "Anaëlle",    "Anaïs",      "Anastasie",   "Anatolie",   "Anceline",     "Andrée",       "Anémone",
    "Angadrême",  "Angèle",     "Angeline",   "Angélina",    "Angélique",  "Angilberte",   "Anicée",       "Anicette",
    "Annabelle",  "Anne",       "Annette",    "Annonciade",  "Ansberte",   "Anstrudie",    "Anthelmette",  "Antigone",
    "Antoinette", "Antonine",   "Aphélie",    "Apolline",    "Aquiline",   "Arabelle",     "Arcadie",      "Archange",
    "Argine",     "Ariane",     "Aricie",     "Ariel",       "Arielle",    "Arlette",      "Armance",      "Armande",
    "Armandine",  "Armeline",   "Armide",     "Armelle",     "Armine",     "Arnaude",      "Arsènie",      "Arsinoé",
    "Artémis",    "Arthurine",  "Asceline",   "Ascension",   "Assomption", "Astarté",      "Astérie",      "Astrée",
    "Astride",    "Athalie",    "Athanasie",  "Athénaïs",    "Athina",     "Aube",         "Aubertine",    "Aude",
    "Audeline",   "Audrey",     "Augustine",  "Aure",        "Aurélie",    "Aurélienne",   "Aurelle",      "Auriane",
    "Aurore",     "Auxane",     "Aveline",    "Avigaëlle",   "Avoye",      "Axeline",      "Axelle",       "Aymardine",
    "Aymonde",    "Azalée",     "Azélie",     "Azeline",     "Barbe",      "Basilisse",    "Bathilde",     "Béatrice",
    "Bénédicte",  "Bérangère",  "Bernadette", "Berthe",      "Bertille",   "Betty",        "Beuve",        "Blanche",
    "Blandine",   "Brigitte",   "Brunehaut",  "Brunehilde",  "Camille",    "Capucine",     "Carine",       "Caroline",
    "Cassandre",  "Catherine",  "Cécile",     "Céleste",     "Célestine",  "Céline",       "Chantal",      "Charlaine",
    "Charline",   "Charlotte",  "Chloé",      "Christelle",  "Christiane", "Christine",    "Claire",       "Clara",
    "Claude",     "Claudine",   "Clarisse",   "Clélie",      "Clémence",   "Clémentine",   "Clio",         "Clotilde",
    "Coline",     "Conception", "Constance",  "Coralie",     "Coraline",   "Corentine",    "Corinne",      "Cyrielle",
    "Danielle",   "Daphné",     "Débora",     "Delphine",    "Denise",     "Diane",        "Dieudonnée",   "Dominique",
    "Doriane",    "Dorine",     "Dorothée",   "Douce",       "Édith",      "Edmée",        "Éléonore",     "Éliane",
    "Élia",       "Élisabeth",  "Élise",      "Ella",        "Élodie",     "Éloïse",       "Elsa",         "Émeline",
    "Émérance",   "Émérencie",  "Émilie",     "Emma",        "Emmanuelle", "Emmelie",      "Ernestine",    "Esther",
    "Estelle",    "Eudoxie",    "Eugénie",    "Eulalie",     "Euphrasie",  "Eusébie",      "Évangéline",   "Eva",
    "Ève",        "Évelyne",    "Fanny",      "Fantine",     "Faustine",   "Félicie",      "Frédérique",   "Flavie",
    "Fleur",      "Flore",      "Florence",   "Florie",      "Fortunée",   "France",       "Francette",    "Francia",
    "Françoise",  "Francine",   "Gabrielle",  "Gaëlle",      "Garance",    "Geneviève",    "Georgette",    "Gerberge",
    "Germaine",   "Gertrude",   "Gisèle",     "Guenièvre",   "Guilhemine", "Guillemette",  "Gustavine",    "Gwenaëlle",
    "Hélène",     "Héloïse",    "Henriette",  "Hermine",     "Hippolyte",  "Honorine",     "Hortense",     "Huguette",
    "Inès",       "Irène",      "Irina",      "Iris",        "Isabeau",    "Isabelle",     "Iseult",       "Ismérie",
    "Jacinthe",   "Jacqueline", "Jade",       "Janine",      "Jeanne",     "Jeanne d’Arc", "Jehanne",      "Jocelyne",
    "Joëlle",     "Joséphine",  "Judith",     "Julia",       "Julie",      "Juliette",     "Justine",      "Laura",
    "Laurane",    "Laure",      "Laureline",  "Laurence",    "Laurène",    "Lauriane",     "Laurine",      "Léa",
    "Léna",       "Léopoldine", "Léonie",     "Léonne",      "Lorraine",   "Lucie",        "Lucienne",     "Lucille",
    "Ludivine",   "Lydie",      "Mégane",     "Madeleine",   "Magali",     "Maguelone",    "Mahaut",       "Mallaury",
    "Manon",      "Marceline",  "Margot",     "Marguerite",  "Marianne",   "Marie",        "Marine",       "Marion",
    "Marlène",    "Marthe",     "Martine",    "Mathilde",    "Maud",       "Maureen",      "Mauricette",   "Maxellende",
    "Maxime",     "Mélanie",    "Mélissa",    "Mélissandre", "Mélisande",  "Mélodie",      "Michèle",      "Mireille",
    "Miriam",     "Moïsette",   "Monique",    "Morgane",     "Muriel",     "Mylène",       "Nadège",       "Nadine",
    "Nathalie",   "Nicole",     "Nine",       "Noëlle",      "Noémie",     "Océane",       "Odette",       "Odile",
    "Olive",      "Olympe",     "Ombline",    "Ophélie",     "Oriande",    "Oriane",       "Orlane",       "Ozanne",
    "Pascale",    "Paule",      "Paulette",   "Pauline",     "Priscille",  "Pécine",       "Pélagie",      "Pénélope",
    "Perrine",    "Pétronille", "Philippine", "Philomène",   "Philothée",  "Primerose",    "Prudence",     "Pulchérie",
    "Quentine",   "Quiéta",     "Quintia",    "Rachel",      "Raphaëlle",  "Raymonde",     "Rebecca",      "Régine",
    "Reine",      "Réjeanne",   "Renée",      "Rita",        "Rolande",    "Romane",       "Rosalie",      "Rose",
    "Roseline",   "Sabine",     "Salomé",     "Sandra",      "Sandrine",   "Sarah",        "Scholastique", "Ségolène",
    "Séverine",   "Sibylle",    "Simone",     "Sixtine",     "Solange",    "Soline",       "Sophie",       "Stéphanie",
    "Suzanne",    "Suzon",      "Sylviane",   "Sylvie",      "Swassane",   "Tatiana",      "Thaïs",        "Théodora",
    "Thérèse",    "Tiphaine",   "Ursule",     "Valentine",   "Valérie",    "Véronique",    "Victoire",     "Vinciane",
    "Violette",   "Virginie",   "Viviane",    "Xavière",     "Yolande",    "Ysaline",      "Yseult",       "Yvette",
    "Yvonne",     "Zoé",        "Zoéva",
});

const auto frenchLastNames = std::to_array<std::string_view>({
    "Martin",    "Bernard",   "Dubois",     "Thomas",    "Robert",   "Richard",  "Petit",    "Durand",      "Leroy",
    "Moreau",    "Simon",     "Laurent",    "Lefebvre",  "Michel",   "Garcia",   "David",    "Bertrand",    "Roux",
    "Vincent",   "Fournier",  "Morel",      "Girard",    "Andre",    "Lefevre",  "Mercier",  "Dupont",      "Lambert",
    "Bonnet",    "Francois",  "Martinez",   "Legrand",   "Garnier",  "Faure",    "Rousseau", "Blanc",       "Guerin",
    "Muller",    "Henry",     "Roussel",    "Nicolas",   "Perrin",   "Morin",    "Mathieu",  "Clement",     "Gauthier",
    "Dumont",    "Lopez",     "Fontaine",   "Chevalier", "Robin",    "Masson",   "Sanchez",  "Gerard",      "Nguyen",
    "Boyer",     "Denis",     "Lemaire",    "Duval",     "Joly",     "Gautier",  "Roger",    "Roche",       "Roy",
    "Noel",      "Meyer",     "Lucas",      "Meunier",   "Jean",     "Perez",    "Marchand", "Dufour",      "Blanchard",
    "Marie",     "Barbier",   "Brun",       "Dumas",     "Brunet",   "Schmitt",  "Leroux",   "Colin",       "Fernandez",
    "Pierre",    "Renard",    "Arnaud",     "Rolland",   "Caron",    "Aubert",   "Giraud",   "Leclerc",     "Vidal",
    "Bourgeois", "Renaud",    "Lemoine",    "Picard",    "Gaillard", "Philippe", "Leclercq", "Lacroix",     "Fabre",
    "Dupuis",    "Olivier",   "Rodriguez",  "Da silva",  "Hubert",   "Louis",    "Charles",  "Guillot",     "Riviere",
    "Le gall",   "Guillaume", "Adam",       "Rey",       "Moulin",   "Gonzalez", "Berger",   "Lecomte",     "Menard",
    "Fleury",    "Deschamps", "Carpentier", "Julien",    "Benoit",   "Paris",    "Maillard", "Marchal",     "Aubry",
    "Vasseur",   "Le roux",   "Renault",    "Jacquet",   "Collet",   "Prevost",  "Poirier",  "Charpentier", "Royer",
    "Huet",      "Baron",     "Dupuy",      "Pons",      "Paul",     "Laine",    "Carre",    "Breton",      "Remy",
    "Schneider", "Perrot",    "Guyot",      "Barre",     "Marty",    "Cousin",
});

const auto frenchMalePrefixes = std::to_array<std::string_view>({"M", "Dr", "Prof"});

const auto frenchFemalePrefixes = std::to_array<std::string_view>({"Mme", "Mlle", "Dr", "Prof"});

const NameFormats frenchNameFormats{{{"{prefix} {firstName} {lastName}", 2}, {"{firstName} {lastName}", 8}}};

const PeopleNames frenchPeopleNames{{frenchMaleFirstNames, frenchLastNames, frenchMalePrefixes, {}},
                                    {frenchFemaleFirstNames, frenchLastNames, frenchFemalePrefixes, {}},
                                    frenchNameFormats};

// Germany

const auto germanMaleFirstNames = std::to_array<std::string_view>({
    "Aaron",      "Abdul",    "Abdullah",  "Adam",       "Adrian",     "Adriano",     "Ahmad",     "Ahmed",
    "Ahmet",      "Alan",     "Albert",    "Alessandro", "Alessio",    "Alex",        "Alexander", "Alfred",
    "Ali",        "Amar",     "Amir",      "Amon",       "Andre",      "Andreas",     "Andrew",    "Angelo",
    "Ansgar",     "Anthony",  "Anton",     "Antonio",    "Arda",       "Arian",       "Armin",     "Arne",
    "Arno",       "Arthur",   "Artur",     "Arved",      "Arvid",      "Ayman",       "Baran",     "Baris",
    "Bastian",    "Batuhan",  "Bela",      "Ben",        "Benedikt",   "Benjamin",    "Bennet",    "Bennett",
    "Benno",      "Bent",     "Berat",     "Berkay",     "Bernd",      "Bilal",       "Bjarne",    "Björn",
    "Bo",         "Boris",    "Brandon",   "Brian",      "Bruno",      "Bryan",       "Burak",     "Calvin",
    "Can",        "Carl",     "Carlo",     "Carlos",     "Caspar",     "Cedric",      "Cedrik",    "Cem",
    "Charlie",    "Chris",    "Christian", "Christiano", "Christoph",  "Christopher", "Claas",     "Clemens",
    "Colin",      "Collin",   "Conner",    "Connor",     "Constantin", "Corvin",      "Curt",      "Damian",
    "Damien",     "Daniel",   "Danilo",    "Danny",      "Darian",     "Dario",       "Darius",    "Darren",
    "David",      "Davide",   "Davin",     "Dean",       "Deniz",      "Dennis",      "Denny",     "Devin",
    "Diego",      "Dion",     "Domenic",   "Domenik",    "Dominic",    "Dominik",     "Dorian",    "Dustin",
    "Dylan",      "Ecrin",    "Eddi",      "Eddy",       "Edgar",      "Edwin",       "Efe",       "Ege",
    "Elia",       "Eliah",    "Elias",     "Elijah",     "Emanuel",    "Emil",        "Emilian",   "Emilio",
    "Emir",       "Emirhan",  "Emre",      "Enes",       "Enno",       "Enrico",      "Eren",      "Eric",
    "Erik",       "Etienne",  "Fabian",    "Fabien",     "Fabio",      "Fabrice",     "Falk",      "Felix",
    "Ferdinand",  "Fiete",    "Filip",     "Finlay",     "Finley",     "Finn",        "Finnley",   "Florian",
    "Francesco",  "Franz",    "Frederic",  "Frederick",  "Frederik",   "Friedrich",   "Fritz",     "Furkan",
    "Fynn",       "Gabriel",  "Georg",     "Gerrit",     "Gian",       "Gianluca",    "Gino",      "Giuliano",
    "Giuseppe",   "Gregor",   "Gustav",    "Hagen",      "Hamza",      "Hannes",      "Hanno",     "Hans",
    "Hasan",      "Hassan",   "Hauke",     "Hendrik",    "Hennes",     "Henning",     "Henri",     "Henrick",
    "Henrik",     "Henry",    "Hugo",      "Hussein",    "Ian",        "Ibrahim",     "Ilias",     "Ilja",
    "Ilyas",      "Immanuel", "Ismael",    "Ismail",     "Ivan",       "Iven",        "Jack",      "Jacob",
    "Jaden",      "Jakob",    "Jamal",     "James",      "Jamie",      "Jan",         "Janek",     "Janis",
    "Janne",      "Jannek",   "Jannes",    "Jannik",     "Jannis",     "Jano",        "Janosch",   "Jared",
    "Jari",       "Jarne",    "Jarno",     "Jaron",      "Jason",      "Jasper",      "Jay",       "Jayden",
    "Jayson",     "Jean",     "Jens",      "Jeremias",   "Jeremie",    "Jeremy",      "Jermaine",  "Jerome",
    "Jesper",     "Jesse",    "Jim",       "Jimmy",      "Joe",        "Joel",        "Joey",      "Johann",
    "Johannes",   "John",     "Johnny",    "Jon",        "Jona",       "Jonah",       "Jonas",     "Jonathan",
    "Jonte",      "Joost",    "Jordan",    "Joris",      "Joscha",     "Joschua",     "Josef",     "Joseph",
    "Josh",       "Joshua",   "Josua",     "Juan",       "Julian",     "Julien",      "Julius",    "Juri",
    "Justin",     "Justus",   "Kaan",      "Kai",        "Kalle",      "Karim",       "Karl",      "Karlo",
    "Kay",        "Keanu",    "Kenan",     "Kenny",      "Keno",       "Kerem",       "Kerim",     "Kevin",
    "Kian",       "Kilian",   "Kim",       "Kimi",       "Kjell",      "Klaas",       "Klemens",   "Konrad",
    "Konstantin", "Koray",    "Korbinian", "Kurt",       "Lars",       "Lasse",       "Laurence",  "Laurens",
    "Laurenz",    "Laurin",   "Lean",      "Leander",    "Leandro",    "Leif",        "Len",       "Lenn",
    "Lennard",    "Lennart",  "Lennert",   "Lennie",     "Lennox",     "Lenny",       "Leo",       "Leon",
    "Leonard",    "Leonardo", "Leonhard",  "Leonidas",   "Leopold",    "Leroy",       "Levent",    "Levi",
    "Levin",      "Lewin",    "Lewis",     "Liam",       "Lian",       "Lias",        "Lino",      "Linus",
    "Lio",        "Lion",     "Lionel",    "Logan",      "Lorenz",     "Lorenzo",     "Loris",     "Louis",
    "Luan",       "Luc",      "Luca",      "Lucas",      "Lucian",     "Lucien",      "Ludwig",    "Luis",
    "Luiz",       "Luk",      "Luka",      "Lukas",      "Luke",       "Lutz",        "Maddox",    "Mads",
    "Magnus",     "Maik",     "Maksim",    "Malik",      "Malte",      "Manuel",      "Marc",      "Marcel",
    "Marco",      "Marcus",   "Marek",     "Marian",     "Mario",      "Marius",      "Mark",      "Marko",
    "Markus",     "Marlo",    "Marlon",    "Marten",     "Martin",     "Marvin",      "Marwin",    "Mateo",
    "Mathis",     "Matis",    "Mats",      "Matteo",     "Mattes",     "Matthias",    "Matthis",   "Matti",
    "Mattis",     "Maurice",  "Max",       "Maxim",      "Maximilian", "Mehmet",      "Meik",      "Melvin",
    "Merlin",     "Mert",     "Michael",   "Michel",     "Mick",       "Miguel",      "Mika",      "Mikail",
    "Mike",       "Milan",    "Milo",      "Mio",        "Mirac",      "Mirco",       "Mirko",     "Mohamed",
    "Mohammad",   "Mohammed", "Moritz",    "Morten",     "Muhammed",   "Murat",       "Mustafa",   "Nathan",
    "Nathanael",  "Nelson",   "Neo",       "Nevio",      "Nick",       "Niclas",      "Nico",      "Nicolai",
    "Nicolas",    "Niels",    "Nikita",    "Niklas",     "Niko",       "Nikolai",     "Nikolas",   "Nils",
    "Nino",       "Noah",     "Noel",      "Norman",     "Odin",       "Oke",         "Ole",       "Oliver",
    "Omar",       "Onur",     "Oscar",     "Oskar",      "Pascal",     "Patrice",     "Patrick",   "Paul",
    "Peer",       "Pepe",     "Peter",     "Phil",       "Philip",     "Philipp",     "Pierre",    "Piet",
    "Pit",        "Pius",     "Quentin",   "Quirin",     "Rafael",     "Raik",        "Ramon",     "Raphael",
    "Rasmus",     "Raul",     "Rayan",     "René",       "Ricardo",    "Riccardo",    "Richard",   "Rick",
    "Rico",       "Robert",   "Robin",     "Rocco",      "Roman",      "Romeo",       "Ron",       "Ruben",
    "Ryan",       "Said",     "Salih",     "Sam",        "Sami",       "Sammy",       "Samuel",    "Sandro",
    "Santino",    "Sascha",   "Sean",      "Sebastian",  "Selim",      "Semih",       "Shawn",     "Silas",
    "Simeon",     "Simon",    "Sinan",     "Sky",        "Stefan",     "Steffen",     "Stephan",   "Steve",
    "Steven",     "Sven",     "Sönke",     "Sören",      "Taha",       "Tamino",      "Tammo",     "Tarik",
    "Tayler",     "Taylor",   "Teo",       "Theo",       "Theodor",    "Thies",       "Thilo",     "Thomas",
    "Thorben",    "Thore",    "Thorge",    "Tiago",      "Til",        "Till",        "Tillmann",  "Tim",
    "Timm",       "Timo",     "Timon",     "Timothy",    "Tino",       "Titus",       "Tizian",    "Tjark",
    "Tobias",     "Tom",      "Tommy",     "Toni",       "Tony",       "Torben",      "Tore",      "Tristan",
    "Tyler",      "Tyron",    "Umut",      "Valentin",   "Valentino",  "Veit",        "Victor",    "Viktor",
    "Vin",        "Vincent",  "Vito",      "Vitus",      "Wilhelm",    "Willi",       "William",   "Willy",
    "Xaver",      "Yannic",   "Yannick",   "Yannik",     "Yannis",     "Yasin",       "Youssef",   "Yunus",
    "Yusuf",      "Yven",     "Yves",      "Ömer",
});

const auto germanFemaleFirstNames = std::to_array<std::string_view>({
    "Aaliyah",   "Abby",      "Abigail",   "Ada",       "Adelina",    "Adriana",   "Aileen",    "Aimee",
    "Alana",     "Alea",      "Alena",     "Alessa",    "Alessia",    "Alexa",     "Alexandra", "Alexia",
    "Alexis",    "Aleyna",    "Alia",      "Alica",     "Alice",      "Alicia",    "Alina",     "Alisa",
    "Alisha",    "Alissa",    "Aliya",     "Aliyah",    "Allegra",    "Alma",      "Alyssa",    "Amalia",
    "Amanda",    "Amelia",    "Amelie",    "Amina",     "Amira",      "Amy",       "Ana",       "Anabel",
    "Anastasia", "Andrea",    "Angela",    "Angelina",  "Angelique",  "Anja",      "Ann",       "Anna",
    "Annabel",   "Annabell",  "Annabelle", "Annalena",  "Anne",       "Anneke",    "Annelie",   "Annemarie",
    "Anni",      "Annie",     "Annika",    "Anny",      "Anouk",      "Antonia",   "Arda",      "Ariana",
    "Ariane",    "Arwen",     "Ashley",    "Asya",      "Aurelia",    "Aurora",    "Ava",       "Ayleen",
    "Aylin",     "Ayse",      "Azra",      "Betty",     "Bianca",     "Bianka",    "Caitlin",   "Cara",
    "Carina",    "Carla",     "Carlotta",  "Carmen",    "Carolin",    "Carolina",  "Caroline",  "Cassandra",
    "Catharina", "Catrin",    "Cecile",    "Cecilia",   "Celia",      "Celina",    "Celine",    "Ceyda",
    "Ceylin",    "Chantal",   "Charleen",  "Charlotta", "Charlotte",  "Chayenne",  "Cheyenne",  "Chiara",
    "Christin",  "Christina", "Cindy",     "Claire",    "Clara",      "Clarissa",  "Colleen",   "Collien",
    "Cora",      "Corinna",   "Cosima",    "Dana",      "Daniela",    "Daria",     "Darleen",   "Defne",
    "Delia",     "Denise",    "Diana",     "Dilara",    "Dina",       "Dorothea",  "Ecrin",     "Eda",
    "Eileen",    "Ela",       "Elaine",    "Elanur",    "Elea",       "Elena",     "Eleni",     "Eleonora",
    "Eliana",    "Elif",      "Elina",     "Elisa",     "Elisabeth",  "Ella",      "Ellen",     "Elli",
    "Elly",      "Elsa",      "Emelie",    "Emely",     "Emilia",     "Emilie",    "Emily",     "Emma",
    "Emmely",    "Emmi",      "Emmy",      "Enie",      "Enna",       "Enya",      "Esma",      "Estelle",
    "Esther",    "Eva",       "Evelin",    "Evelina",   "Eveline",    "Evelyn",    "Fabienne",  "Fatima",
    "Fatma",     "Felicia",   "Felicitas", "Felina",    "Femke",      "Fenja",     "Fine",      "Finia",
    "Finja",     "Finnja",    "Fiona",     "Flora",     "Florentine", "Francesca", "Franka",    "Franziska",
    "Frederike", "Freya",     "Frida",     "Frieda",    "Friederike", "Giada",     "Gina",      "Giulia",
    "Giuliana",  "Greta",     "Hailey",    "Hana",      "Hanna",      "Hannah",    "Heidi",     "Helen",
    "Helena",    "Helene",    "Helin",     "Henriette", "Henrike",    "Hermine",   "Ida",       "Ilayda",
    "Imke",      "Ina",       "Ines",      "Inga",      "Inka",       "Irem",      "Isa",       "Isabel",
    "Isabell",   "Isabella",  "Isabelle",  "Ivonne",    "Jacqueline", "Jamie",     "Jamila",    "Jana",
    "Jane",      "Janin",     "Janina",    "Janine",    "Janna",      "Janne",     "Jara",      "Jasmin",
    "Jasmina",   "Jasmine",   "Jella",     "Jenna",     "Jennifer",   "Jenny",     "Jessica",   "Jessy",
    "Jette",     "Jil",       "Jill",      "Joana",     "Joanna",     "Joelina",   "Joeline",   "Joelle",
    "Johanna",   "Joleen",    "Jolie",     "Jolien",    "Jolin",      "Jolina",    "Joline",    "Jona",
    "Jonah",     "Jonna",     "Josefin",   "Josefine",  "Josephin",   "Josephine", "Josie",     "Josy",
    "Joy",       "Joyce",     "Judith",    "Judy",      "Jule",       "Julia",     "Juliana",   "Juliane",
    "Julie",     "Julienne",  "Julika",    "Julina",    "Juna",       "Justine",   "Kaja",      "Karina",
    "Karla",     "Karlotta",  "Karolina",  "Karoline",  "Kassandra",  "Katarina",  "Katharina", "Kathrin",
    "Katja",     "Katrin",    "Kaya",      "Kayra",     "Kiana",      "Kiara",     "Kim",       "Kimberley",
    "Kimberly",  "Kira",      "Klara",     "Korinna",   "Kristin",    "Kyra",      "Laila",     "Lana",
    "Lara",      "Larissa",   "Laura",     "Laureen",   "Lavinia",    "Lea",       "Leah",      "Leana",
    "Leandra",   "Leann",     "Lee",       "Leila",     "Lena",       "Lene",      "Leni",      "Lenia",
    "Lenja",     "Lenya",     "Leona",     "Leoni",     "Leonie",     "Leonora",   "Leticia",   "Letizia",
    "Levke",     "Leyla",     "Lia",       "Liah",      "Liana",      "Lili",      "Lilia",     "Lilian",
    "Liliana",   "Lilith",    "Lilli",     "Lillian",   "Lilly",      "Lily",      "Lina",      "Linda",
    "Lindsay",   "Line",      "Linn",      "Linnea",    "Lisa",       "Lisann",    "Lisanne",   "Liv",
    "Livia",     "Liz",       "Lola",      "Loreen",    "Lorena",     "Lotta",     "Lotte",     "Louisa",
    "Louise",    "Luana",     "Luca",      "Lucia",     "Lucie",      "Lucienne",  "Lucy",      "Luisa",
    "Luise",     "Luka",      "Luna",      "Luzie",     "Lya",        "Lydia",     "Lyn",       "Lynn",
    "Madeleine", "Madita",    "Madleen",   "Madlen",    "Magdalena",  "Maike",     "Mailin",    "Maira",
    "Maja",      "Malena",    "Malia",     "Malin",     "Malina",     "Mandy",     "Mara",      "Marah",
    "Mareike",   "Maren",     "Maria",     "Mariam",    "Marie",      "Marieke",   "Mariella",  "Marika",
    "Marina",    "Marisa",    "Marissa",   "Marit",     "Marla",      "Marleen",   "Marlen",    "Marlena",
    "Marlene",   "Marta",     "Martha",    "Mary",      "Maryam",     "Mathilda",  "Mathilde",  "Matilda",
    "Maxi",      "Maxima",    "Maxine",    "Maya",      "Mayra",      "Medina",    "Medine",    "Meike",
    "Melanie",   "Melek",     "Melike",    "Melina",    "Melinda",    "Melis",     "Melisa",    "Melissa",
    "Merle",     "Merve",     "Meryem",    "Mette",     "Mia",        "Michaela",  "Michelle",  "Mieke",
    "Mila",      "Milana",    "Milena",    "Milla",     "Mina",       "Mira",      "Miray",     "Miriam",
    "Mirja",     "Mona",      "Monique",   "Nadine",    "Nadja",      "Naemi",     "Nancy",     "Naomi",
    "Natalia",   "Natalie",   "Nathalie",  "Neele",     "Nela",       "Nele",      "Nelli",     "Nelly",
    "Nia",       "Nicole",    "Nika",      "Nike",      "Nikita",     "Nila",      "Nina",      "Nisa",
    "Noemi",     "Nora",      "Olivia",    "Patricia",  "Patrizia",   "Paula",     "Paulina",   "Pauline",
    "Penelope",  "Philine",   "Phoebe",    "Pia",       "Rahel",      "Rania",     "Rebecca",   "Rebekka",
    "Riana",     "Rieke",     "Rike",      "Romina",    "Romy",       "Ronja",     "Rosa",      "Rosalie",
    "Ruby",      "Sabrina",   "Sahra",     "Sally",     "Salome",     "Samantha",  "Samia",     "Samira",
    "Sandra",    "Sandy",     "Sanja",     "Saphira",   "Sara",       "Sarah",     "Saskia",    "Selin",
    "Selina",    "Selma",     "Sena",      "Sidney",    "Sienna",     "Silja",     "Sina",      "Sinja",
    "Smilla",    "Sofia",     "Sofie",     "Sonja",     "Sophia",     "Sophie",    "Soraya",    "Stefanie",
    "Stella",    "Stephanie", "Stina",     "Sude",      "Summer",     "Susanne",   "Svea",      "Svenja",
    "Sydney",    "Tabea",     "Talea",     "Talia",     "Tamara",     "Tamia",     "Tamina",    "Tanja",
    "Tara",      "Tarja",     "Teresa",    "Tessa",     "Thalea",     "Thalia",    "Thea",      "Theresa",
    "Tia",       "Tina",      "Tomke",     "Tuana",     "Valentina",  "Valeria",   "Valerie",   "Vanessa",
    "Vera",      "Veronika",  "Victoria",  "Viktoria",  "Viola",      "Vivian",    "Vivien",    "Vivienne",
    "Wibke",     "Wiebke",    "Xenia",     "Yara",      "Yaren",      "Yasmin",    "Ylvi",      "Ylvie",
    "Yvonne",    "Zara",      "Zehra",     "Zeynep",    "Zoe",        "Zoey",      "Zoé",
});

const auto germanLastNames = std::to_array<std::string_view>({
    "Abel",
    "Abicht",
    "Abraham",
    "Abramovic",
    "Abt",
    "Achilles",
    "Achkinadze",
    "Ackermann",
    "Adam",
    "Adams",
    "Ade",
    "Agostini",
    "Ahlke",
    "Ahrenberg",
    "Ahrens",
    "Aigner",
    "Albert",
    "Albrecht",
    "Alexa",
    "Alexander",
    "Alizadeh",
    "Allgeyer",
    "Amann",
    "Amberg",
    "Anding",
    "Anggreny",
    "Apitz",
    "Arendt",
    "Arens",
    "Arndt",
    "Aryee",
    "Aschenbroich",
    "Assmus",
    "Astafei",
    "Auer",
    "Axmann",
    "Baarck",
    "Bachmann",
    "Badane",
    "Bader",
    "Baganz",
    "Bahl",
    "Bak",
    "Balcer",
    "Balck",
    "Balkow",
    "Balnuweit",
    "Balzer",
    "Banse",
    "Barr",
    "Bartels",
    "Barth",
    "Barylla",
    "Baseda",
    "Battke",
    "Bauer",
    "Bauermeister",
    "Baumann",
    "Baumeister",
    "Bauschinger",
    "Bauschke",
    "Bayer",
    "Beavogui",
    "Beck",
    "Beckel",
    "Becker",
    "Beckmann",
    "Bedewitz",
    "Beele",
    "Beer",
    "Beggerow",
    "Beh",
    "Behr",
    "Behrenbruch",
    "Belz",
    "Bender",
    "Benecke",
    "Benner",
    "Benninger",
    "Benzing",
    "Berends",
    "Berger",
    "Berner",
    "Berning",
    "Bertenbreiter",
    "Best",
    "Bethke",
    "Betz",
    "Beushausen",
    "Beutelspacher",
    "Beyer",
    "Biba",
    "Bichler",
    "Bickel",
    "Biedermann",
    "Bieler",
    "Bielert",
    "Bienasch",
    "Bienias",
    "Biesenbach",
    "Bigdeli",
    "Birkemeyer",
    "Bittner",
    "Blank",
    "Blaschek",
    "Blassneck",
    "Bloch",
    "Blochwitz",
    "Blockhaus",
    "Blum",
    "Blume",
    "Bock",
    "Bode",
    "Bogdashin",
    "Bogenrieder",
    "Bohge",
    "Bolm",
    "Borgschulze",
    "Bork",
    "Bormann",
    "Bornscheuer",
    "Borrmann",
    "Borsch",
    "Boruschewski",
    "Bos",
    "Bosler",
    "Bourrouag",
    "Bouschen",
    "Boxhammer",
    "Boyde",
    "Bozsik",
    "Brand",
    "Brandenburg",
    "Brandis",
    "Brandt",
    "Brauer",
    "Braun",
    "Brehmer",
    "Breitenstein",
    "Bremer",
    "Bremser",
    "Brenner",
    "Brettschneider",
    "Breu",
    "Breuer",
    "Briesenick",
    "Bringmann",
    "Brinkmann",
    "Brix",
    "Broening",
    "Brosch",
    "Bruckmann",
    "Bruder",
    "Bruhns",
    "Brunner",
    "Bruns",
    "Bräutigam",
    "Brömme",
    "Brüggmann",
    "Buchholz",
    "Buchrucker",
    "Buder",
    "Bultmann",
    "Bunjes",
    "Burger",
    "Burghagen",
    "Burkhard",
    "Burkhardt",
    "Burmeister",
    "Busch",
    "Buschbaum",
    "Busemann",
    "Buss",
    "Busse",
    "Bussmann",
    "Byrd",
    "Bäcker",
    "Böhm",
    "Bönisch",
    "Börgeling",
    "Börner",
    "Böttner",
    "Büchele",
    "Bühler",
    "Büker",
    "Büngener",
    "Bürger",
    "Bürklein",
    "Büscher",
    "Büttner",
    "Camara",
    "Carlowitz",
    "Carlsohn",
    "Caspari",
    "Caspers",
    "Chapron",
    "Christ",
    "Cierpinski",
    "Clarius",
    "Cleem",
    "Cleve",
    "Co",
    "Conrad",
    "Cordes",
    "Cornelsen",
    "Cors",
    "Cotthardt",
    "Crews",
    "Cronjäger",
    "Crosskofp",
    "Da",
    "Dahm",
    "Dahmen",
    "Daimer",
    "Damaske",
    "Danneberg",
    "Danner",
    "Daub",
    "Daubner",
    "Daudrich",
    "Dauer",
    "Daum",
    "Dauth",
    "Dautzenberg",
    "De",
    "Decker",
    "Deckert",
    "Deerberg",
    "Dehmel",
    "Deja",
    "Delonge",
    "Demut",
    "Dengler",
    "Denner",
    "Denzinger",
    "Derr",
    "Dertmann",
    "Dethloff",
    "Deuschle",
    "Dieckmann",
    "Diedrich",
    "Diekmann",
    "Dienel",
    "Dies",
    "Dietrich",
    "Dietz",
    "Dietzsch",
    "Diezel",
    "Dilla",
    "Dingelstedt",
    "Dippl",
    "Dittmann",
    "Dittmar",
    "Dittmer",
    "Dix",
    "Dobbrunz",
    "Dobler",
    "Dohring",
    "Dolch",
    "Dold",
    "Dombrowski",
    "Donie",
    "Doskoczynski",
    "Dragu",
    "Drechsler",
    "Drees",
    "Dreher",
    "Dreier",
    "Dreissigacker",
    "Dressler",
    "Drews",
    "Duma",
    "Dutkiewicz",
    "Dyett",
    "Dylus",
    "Dächert",
    "Döbel",
    "Döring",
    "Dörner",
    "Dörre",
    "Dück",
    "Eberhard",
    "Eberhardt",
    "Ecker",
    "Eckhardt",
    "Edorh",
    "Effler",
    "Eggenmueller",
    "Ehm",
    "Ehmann",
    "Ehrig",
    "Eich",
    "Eifert",
    "Einert",
    "Eisenlauer",
    "Ekpo",
    "Elbe",
    "Eleyth",
    "Elss",
    "Emert",
});

const auto germanMalePrefixes = std::to_array<std::string_view>({"Herr", "Dr.", "Prof. Dr."});

const auto germanFemalePrefixes = std::to_array<std::string_view>({"Frau", "Dr.", "Prof. Dr."});

const NameFormats germanNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 8}}};

const PeopleNames germanPeopleNames{{germanMaleFirstNames, germanLastNames, germanMalePrefixes, {}},
                                    {germanFemaleFirstNames, germanLastNames, germanFemalePrefixes, {}},
                                    germanNameFormats};

// Ghana

const auto ghanaianMaleFirstNames = std::to_array<std::string_view>({
    "Aaron",     "Abeiku",   "Adam",     "Adrian",  "Akwesi",  "Albert",    "Alex",
    "Alexander", "Andrew",   "Anthony",  "Antony",  "Arthur",  "Ben",       "Benjamin",
    "Bernard",   "Bless",    "Blessing", "Bruce",   "Carl",    "Charles",   "Christian",
    "Clifford",  "Colins",   "Daniel",   "Danny",   "David",   "Denis",     "Dennis",
    "Derrick",   "Dominic",  "Donald",   "Douglas", "Duncan",  "Edward",    "Ekow",
    "Elliot",    "Elliott",  "Eyram",    "Eric",    "Fiifi",   "Francis",   "Frank",
    "Frederick", "George",   "Gerald",   "Gordon",  "Graham",  "Gregory",   "Harry",
    "Henry",     "Howard",   "Isaac",    "Akwasi",  "Jack",    "Jacob",     "Jake",
    "James",     "Jason",    "Jeffrey",  "Jeremy",  "Joe",     "Joel",      "John",
    "Jonathan",  "Joojo",    "Joseph",   "Josh",    "Joshua",  "Josiah",    "Julian",
    "Justin",    "Karl",     "Kenneth",  "Kevin",   "Kofi",    "Kojo",      "Kujoe",
    "Kwabena",   "Kwadwo",   "Kwaku",    "Kwame",   "Kwamena", "Kwasi",     "Kweku",
    "Kwesi",     "Kyle",     "Lawrence", "Leslie",  "Louis",   "Luke",      "Malcolm",
    "Marcus",    "Mark",     "Martin",   "Mathew",  "Matthew", "Max",       "Michael",
    "Nathan",    "Nicholas", "Nigel",    "Oliver",  "Patrick", "Paul",      "Peter",
    "Philip",    "Phillip",  "Promise",  "Raymond", "Richard", "Robert",    "Roger",
    "Ronald",    "Russell",  "Sam",      "Samuel",  "Shaun",   "Simon",     "Stanley",
    "Stephen",   "Steven",   "Terence",  "Thomas",  "Timothy", "Thank God", "Thy Will Be Done",
    "Tom",       "Tony",     "Victor",   "Vincent", "William", "Yaw",
});

const auto ghanaianFemaleFirstNames = std::to_array<std::string_view>({
    "Aba",       "Abena",     "Abigail",   "Adwoa",     "Afia",     "Afua",     "Akos",      "Akofa",    "Akorfa",
    "Akosua",    "Akua",      "Akumaa",    "Alice",     "Ama",      "Amanda",   "Amber",     "Amelia",   "Angela",
    "Ann",       "Annette",   "Awesi",     "Baaba",     "Barbara",  "Beatrice", "COmfort",   "Caroline", "Catherine",
    "Charlotte", "Christina", "Comfort",   "Constance", "Danielle", "Deborah",  "Debra",     "Denise",   "Dora",
    "Dorcas",    "Dorothy",   "Eliabeth",  "Elizabeth", "Emily",    "Emma",     "Ernestina", "Esi",      "Eunice",
    "Eyram",     "Felicia",   "Francesca", "Gemma",     "Georgia",  "Georgina", "Gifty",     "Grace",    "Hannabel",
    "Hannah",    "Harriet",   "Helen",     "Irene",     "Janet",    "Janice",   "Jasmine",   "Jennifer", "Jessica",
    "Jill",      "Joanna",    "Josephine", "Joyce",     "Judith",   "Julia",    "Juliana",   "Julie",    "Karen",
    "Kate",      "Katherine", "Katy",      "Lawrencia", "Linda",    "Lisa",     "Lorraine",  "Lucy",     "Lydia",
    "Mandy",     "Margaret",  "Maria",     "Marian",    "Marilyn",  "Mary",     "Maureen",   "Mawuena",  "Mawusi",
    "Michelle",  "Millicent", "Nana Ama",  "Naomi",     "Natalie",  "Natasha",  "Nicola",    "Nimakoah", "Olivia",
    "Pamela",    "Patricia",  "Paula",     "Priscilla", "Rachael",  "Rachel",   "Rebecca",   "Regina",   "Rita",
    "Roselyn",   "Rosemary",  "Ruth",      "Salomey",   "Samantha", "Sandra",   "Sarah",     "Sena",     "Sharon",
    "Sheila",    "Shirley",   "Stephanie", "Susan",     "Sylvia",   "Teresa",   "Tina",      "Tracy",    "Vanessa",
    "Veronica",  "Victoria",  "Vida",      "Wendy",     "Yaa",      "Yvonne",
});

const auto ghanaianLastNames = std::to_array<std::string_view>({
    "Acheampong", "Adadevoh", "Adomah",   "Adomako", "Adongo",   "Adu",      "Adusei",  "Adutwum", "Afirifa",
    "Afoakwa",    "Agyapong", "Agyare",   "Agyei",   "Agyemang", "Ahortor",  "Akoto",   "Akowua",  "Akyeamfuɔ",
    "Akyeampong", "Akyena",   "Akyerεko", "Amo",     "Amoa",     "Amoako",   "Amoasi",  "Ampadu",  "Ampofo",
    "Amponsah",   "Andorful", "Ankra",    "Anokye",  "Ansa",     "Antwi",    "Appia",   "Appiah",  "Asamoa",
    "Asamoah",    "Asante",   "Asare",    "Asenso",  "Asiama",   "Asiedu",   "Ata",     "Awuah",   "Baa",
    "Baafi",      "Baah",     "Baawia",   "Badu",    "Boadi",    "Boadu",    "Boahen",  "Boakye",  "Boaten",
    "Boateng",    "Bona",     "Bonsra",   "Bonsu",   "Daako",    "Danso",    "Darko",   "Darteh",  "Dartey",
    "Djokoto",    "Donkor",   "Duah",     "Dwamena", "Dzokoto",  "Fofie",    "Fosu",    "Gyamfi",  "Gyasi",
    "Karikari",   "Koomson",  "Kumi",     "Kusi",    "Kwaakye",  "Kwarteng", "Kyei",    "Mensa",   "Mensah",
    "Nkansa",     "Nkansah",  "Nkrumah",  "Nsia",    "Nti",      "Ntiamoa",  "Ntim",    "Nyaako",  "Nyame",
    "Nyantakyi",  "Obeng",    "Ofori",    "Ofosu",   "Okyere",   "Okpattah", "Omani",   "Opoku",   "Oppong",
    "Opuku",      "Osei",     "Oti",      "Otiwa",   "Otuo",     "Owusu",    "Prempeh", "Quartey", "Safo",
    "Sarpong",    "Takyi",    "Tawia",    "Tutu",    "Tweneboa", "Twumasi",  "Wiafe",   "Yaamoa",  "Yawson",
    "Yeboa",      "Yeboah",   "Yirenkyi",
});

const NameFormats ghanaianNameFormats{{{"{firstName} {lastName}"}, 1}};

const PeopleNames ghanaianPeopleNames{{ghanaianMaleFirstNames, ghanaianLastNames, {}, {}},
                                      {ghanaianFemaleFirstNames, ghanaianLastNames, {}, {}},
                                      ghanaianNameFormats};

// Greece

const auto greekMaleFirstNames = std::to_array<std::string_view>({
    "Georgios",     "Dimitrios",  "Ioannis",     "Nikolaos",   "Konstantin",  "Christos",   "Panagiotis", "Vasileios",
    "Konstantinos", "Athanasios", "Michail",     "Evangelos",  "Antonios",    "Emmanouil",  "Theodoros",  "Spyridon",
    "Anastasios",   "Ilias",      "Charalampos", "Alexandros", "Stavros",     "Apostolos",  "Petros",     "Stylianos",
    "Sotirios",     "Georg",      "Thomas",      "Ioan",       "Eleftherios", "George",     "Dimitris",   "Nikos",
    "Kostas",       "Alex",       "Markos",      "Anton",      "John",        "Artemis",    "Vasilis",    "Iosif",
    "Michael",      "Ali",        "Nick",        "Chris",      "Leon",        "Yannis",     "Adam",       "David",
    "Ilir",         "Ivan",       "Peter",       "Alexander",  "Vasil",       "Luan",       "Elias",      "Viktor",
    "Arben",        "Theodore",   "Nikolas",     "Mohamed",    "Dion",        "Filip",      "Anatoli",    "Robert",
    "Vladimir",     "Napoleon",   "Isaak",       "Niko",       "Nestor",      "Georgi",     "William",    "Albert",
    "Anastasio",    "Constantin", "Paul",        "Muhammad",   "Nicholas",    "Stefan",     "Dritan",     "Artur",
    "Moustafa",     "Haris",      "Alexandro",   "Ahmed",      "Daniel",      "Eduart",     "Daniil",     "Alfred",
    "Gregory",      "Myron",      "Ion",         "Edmond",     "Kosta",       "Aleksander", "Emmanuel",   "Arjan",
    "Martin",       "Leonard",    "Christof",    "Dimitar",    "Ismail",      "Harry",      "Andrew",     "Nicolas",
    "Dimitri",      "Elton",      "Adrian",      "Angel",      "Vasile",      "Nikolay",    "Roland",     "Mohammad",
    "Victor",       "James",      "Valentin",    "Mario",      "Florin",      "Mark",       "Petro",      "Jim",
    "Simon",        "Andon",      "Anthony",     "Steven",     "Sami",        "Gheorghe",   "Yordan",     "Theo",
    "Hristo",       "Aleks",      "Ervin",       "Antonio",    "Philip",      "Emil",       "Todor",      "Gabriel",
    "Asen",         "Singh",      "Aleksandar",  "Marius",     "Ibrahim",     "Osman",      "Igor",       "Hans",
    "Ionut",        "Roman",      "Alban",       "Hasan",      "Antony",      "Petar",      "Christian",  "Mustafa",
    "Angelo",       "Richard",    "Christopher", "Hussain",    "Tom",         "Jan",        "Marin",      "Florian",
    "Stoyan",       "Ahmad",      "Aurel",       "Sergey",     "Atanas",      "Andrei",     "Maksim",     "Joseph",
    "Frank",        "Renato",     "Abdul",       "Eduard",     "Irfan",       "Mihail",     "Ismet",      "Catalin",
    "Jason",        "Julian",     "Giorgio",     "Pavel",      "Dumitru",     "Ramadan",    "Krasimir",   "Iqbal",
    "Tony",         "Boris",      "Naim",        "Alexandru",  "Arsen",       "Teo",        "Marko",      "Stephen",
    "Christophe",   "Kevin",      "Romeo",       "Nicolae",    "Miroslav",    "Armand",     "Mihai",      "Georges",
    "Cristian",     "Oleg",       "Patrick",     "Michel",     "Arif",        "Mitko",      "Amarildo",   "Andrey",
    "Armando",      "Ananias",    "Plamen",      "Socrates",   "Marco",       "Filimon",    "Adonis",     "Enver",
    "Edison",       "Andi",       "Imran",       "Ian",        "Fran",        "Gianni",     "Rumen",      "Alan",
    "Mikel",        "Giorgi",     "Anastas",     "Armen",      "Karl",        "Stanislav",  "Elvis",      "Mahmoud",
    "Roberto",      "Theodor",    "Solomon",     "Mohammed",   "Hassan",      "Veselin",    "Iliya",      "Emin",
    "Viorel",       "Ilie",       "Amir",        "Simeon",     "Greg",        "Ionel",      "Philippe",   "Pal",
    "Samir",        "Bilal",      "Zoran",       "Irakli",     "Klaus",       "Isaia",      "Zamir",      "Ivaylo",
    "Aleksandr",    "Dragan",     "Charles",     "Louis",      "Brian",       "Sorin",      "Apostol",    "Kiril",
    "Sam",          "Eric",       "Aleko",       "Costel",     "Bruno",       "Valter",     "Werner",     "Tim",
    "Basil",        "Andre",      "Tahir",       "Vilson",     "Adnan",       "Fredi",      "Ferdinand",  "Vlad",
    "Petru",        "Paolo",      "Marsel",      "Giuseppe",   "Ram",         "Oleksandr",  "Asif",       "Artour",
    "Abbas",        "Raymond",    "Nasir",       "Bogdan",     "Trifon",      "Alberto",    "Vasili",     "Matthew",
    "Edward",       "Milan",      "Sasa",        "Iulian",     "Goran",       "Jimmy",      "Roger",      "Mohamad",
    "Erik",         "Volodymyr",  "Alessandro",  "Khalid",     "Mehmet",      "Michal",     "Sinan",      "Alin",
    "Lucian",       "Spiridon",   "Sadik",       "Francesco",  "Stefano",     "Dionis",     "Omer",       "Marc",
    "Jonathan",     "Omar",       "Borislav",    "Sasho",      "Johannes",    "Bernard",    "Shaban",     "Viktoras",
    "Shahid",       "Marcel",     "Walter",      "Felix",      "Giovanni",    "Paulin",     "Kumar",      "Pierre",
    "Tariq",        "Ramazan",    "Ashraf",      "Alain",      "Nicky",       "Mikhail",    "Sampson",    "Luigi",
    "Lucas",        "Frederik",   "Petko",       "Adel",       "Alexey",      "Faik",
});

const auto greekFemaleFirstNames = std::to_array<std::string_view>({
    "Maria",     "Eleni",      "Aikaterini", "Vasiliki",    "Sofia",     "Anastasia", "Georgia",     "Evangelia",
    "Anna",      "Dimitra",    "Ioanna",     "Christina",   "Andreas",   "Alexandra", "Theodora",    "Athina",
    "Olga",      "Evgenia",    "Zoi",        "Konstantina", "Marina",    "Antonia",   "Nikol",       "Margarita",
    "Varvara",   "Leonidas",   "Fani",       "Amalia",      "Nikoletta", "Katerina",  "Stella",      "Martha",
    "Marianna",  "Elena",      "Olympia",    "Ioulia",      "Nikoleta",  "Viktoria",  "Aglaia",      "Andriana",
    "Pelagia",   "Natalia",    "Flora",      "Virginia",    "Marika",    "Alexia",    "Katina",      "Stefania",
    "Tatiana",   "Valentina",  "Violeta",    "Andrea",      "Iliana",    "Mariana",   "Angela",      "Dorothea",
    "Nina",      "Violetta",   "Vera",       "Eleonora",    "Lydia",     "Mary",      "Marilena",    "Pavlina",
    "Svetlana",  "Louiza",     "Aris",       "Sali",        "Marietta",  "Irina",     "Ilia",        "Irene",
    "Paris",     "Daniela",    "Kristina",   "Melina",      "Vicky",     "Roza",      "Mirela",      "Sonia",
    "Mariya",    "Alexis",     "Despina",    "Helen",       "Diana",     "Sophia",    "Ivanka",      "Dora",
    "Elina",     "Christiana", "Elvira",     "Eva",         "Alma",      "Mihaela",   "Angelina",    "Irena",
    "Lina",      "Elsa",       "Lena",       "Lida",        "Michaela",  "Mina",      "Monika",      "Lia",
    "Rena",      "Liana",      "Barbara",    "Isidora",     "Marie",     "Tamara",    "Natasa",      "Rita",
    "Larisa",    "Tina",       "Joanna",     "Sylvia",      "Marian",    "Christine", "Alina",       "Rania",
    "Xenia",     "Natasha",    "Miranda",    "Cristina",    "Fatme",     "Liliana",   "Galina",      "Victoria",
    "Olena",     "Nikolina",   "Adelina",    "Elizabeth",   "Nataliya",  "Emine",     "Silvana",     "Magda",
    "Gianna",    "Jean",       "Milena",     "Irma",        "Nana",      "Tania",     "Iryna",       "Marinela",
    "Eleana",    "Tereza",     "Sara",       "Tanya",       "Rozalia",   "Eugenia",   "Kali",        "Kelly",
    "Catherine", "Zoe",        "Adriana",    "Maia",        "Polina",    "Brunilda",  "Svitlana",    "Aneta",
    "Eliza",     "Yordanka",   "Albana",     "Laura",       "Iordana",   "Karolina",  "Lou",         "Gabriela",
    "Tetyana",   "Merita",     "Katia",      "Veronika",    "Anne",      "Esmeralda", "Iris",        "Anita",
    "Suzana",    "Stanka",     "Nadia",      "Nikola",      "Oksana",    "Jenny",     "Leonora",     "Marieta",
    "Julia",     "Susan",      "Pandora",    "Ekaterina",   "Karen",     "Liza",      "Alla",        "Aleksandra",
    "Sandra",    "Petra",      "Rafaela",    "Penka",       "Dorina",    "Nancy",     "Rositsa",     "Lela",
    "Nelli",     "Silvia",     "Vasilka",    "Elisabeth",   "Mariyana",  "Hristina",  "Donka",       "Malvina",
    "Radka",     "Stela",      "Mari",       "Kassandra",   "Patricia",  "Mariam",    "Dina",        "Gina",
    "Denis",     "Linda",      "Manana",     "Magdalena",   "Inna",      "Emilia",    "Renata",      "Oxana",
    "Tonia",     "Leila",      "Claudia",    "Aida",        "Mara",      "Nicole",    "Stefka",      "Luiza",
    "Thalia",    "Yuliya",     "Nino",       "Neli",        "Simona",    "Matilda",   "Antoneta",    "Vanessa",
    "Julie",     "Marine",     "Valia",      "Sarah",       "Lila",      "Arian",     "Minka",       "Zamira",
    "Gergana",   "Penelope",   "Nicoleta",   "Dennis",      "Vivian",    "Nora",      "Lali",        "Evelina",
    "Sophie",    "Lili",       "Katya",      "Sarra",       "Ioana",     "Rumyana",   "Silviya",     "Isa",
    "Martina",   "Desislava",  "Carol",      "Ileana",      "Zlatka",    "Galya",     "Athena",      "Ina",
    "Paola",     "Teodora",    "Nadezhda",   "Zina",        "Jane",      "Florina",   "Ilona",       "Ionela",
    "Lilia",     "Karin",      "Elisabeta",  "Ann",         "Iva",       "Margaret",  "Tamar",       "Mzia",
    "Pamela",    "Liudmila",   "Aurora",     "Lidia",       "Anka",      "Emiliya",   "Helena",      "Klara",
    "Angelika",  "Khatuna",    "Viorica",    "Todorka",     "Rodica",    "Ivana",     "Dimitrinka",  "Albena",
    "Maya",      "Veneta",     "Krasimira",  "Mariyka",     "Monica",    "Jennifer",  "Tatyana",     "Daphne",
    "Emma",      "Melania",    "Aneliya",    "Katherine",   "Carmen",    "Anisa",     "Constantina", "Asteria",
    "Stoyanka",  "Antoaneta",  "Valeria",    "Diamantina",  "Karine",    "Andreea",   "Manuela",     "Atanaska",
    "Annie",     "Ani",        "Petrina",    "Viktoriya",   "Lilian",    "Ornela",    "Svetla",      "Peggy",
    "Flori",     "Fay",        "Erika",      "Tatjana",     "Kate",      "Inga",      "Karina",      "Marta",
    "Raisa",     "Natalya",    "Ramona",     "Milka",       "Larysa",    "Rina",
});

const auto greekLastNames = std::to_array<std::string_view>({
    "Papadopoulos",
    "Papadopoulou",
    "Papageorgiou",
    "Oikonomou",
    "Papadimitriou",
    "Georgiou",
    "Papaioannou",
    "Pappas",
    "Vasileiou",
    "Nikolaou",
    "Karagiannis",
    "Vlachos",
    "Antoniou",
    "Makris",
    "Papanikolaou",
    "Dimitriou",
    "Ioannidis",
    "Georgiadis",
    "Triantafyllou",
    "Papadakis",
    "Athanasiou",
    "Konstantinidis",
    "Ioannou",
    "Alexiou",
    "Christodoulou",
    "Theodorou",
    "Giannopoulos",
    "Nikolaidis",
    "Konstantinou",
    "Michailidis",
    "Panagiotopoulos",
    "Papakonstantinou",
    "Papathanasiou",
    "Antonopoulos",
    "Dimopoulos",
    "Karagianni",
    "Anastasiou",
    "Dimitriadis",
    "Pappa",
    "Vlachou",
    "Vasileiadis",
    "Giannakopoulos",
    "Angelopoulos",
    "Dimou",
    "Ioannidou",
    "Nikolopoulos",
    "Mylonas",
    "Stergiou",
    "Apostolou",
    "Petropoulos",
    "Lamprou",
    "Papadaki",
    "Christou",
    "Panagiotou",
    "Anagnostou",
    "Makri",
    "Konstantinidou",
    "Samaras",
    "Raptis",
    "Athanasopoulos",
    "Alexopoulos",
    "Christopoulos",
    "Stavropoulos",
    "Anagnostopoulos",
    "Markou",
    "Georgiadou",
    "Spanos",
    "Sidiropoulos",
    "Antoniadis",
    "Panagopoulos",
    "Efthymiou",
    "Spyropoulos",
    "Theodoropoulos",
    "Pavlidis",
    "Athanasiadis",
    "Apostolopoulos",
    "Petrou",
    "Michalopoulos",
    "Arvanitis",
    "Lazaridis",
    "Kontos",
    "Georgopoulos",
    "Panagiotidis",
    "Theodoridis",
    "Chatzis",
    "Anastasiadis",
    "Papavasileiou",
    "Papazoglou",
    "Vasilopoulos",
    "Iliopoulos",
    "Kostopoulos",
    "Politis",
    "Galanis",
    "Stavrou",
    "Paraskevopoulos",
    "Apostolidis",
    "Giannopoulou",
    "Diamantis",
    "Pantazis",
    "Andreou",
    "Petridis",
    "Stathopoulos",
    "Nikolaidou",
    "Michailidou",
    "Dimopoulou",
    "Mavridis",
    "Kyriakou",
    "Grigoriadis",
    "Efstathiou",
    "Sideris",
    "Kyriakidis",
    "Diamantopoulos",
    "Panagiotopoulou",
    "Anastasopoulos",
    "Savvidis",
    "Georgakopoulos",
    "Dimitropoulos",
    "Vasilakis",
    "Rizos",
    "Argyropoulos",
    "Moraitis",
    "Sotiropoulos",
    "Charalampidis",
    "Iliadis",
    "Antonopoulou",
    "Sotiriou",
    "Lampropoulos",
    "Chatzigeorgiou",
    "Angelis",
    "Nikolopoulou",
    "Giannakopoulou",
    "Kollias",
    "Margaritis",
    "Vasileiadou",
    "Katsaros",
    "Chatzi",
    "Sarris",
    "Roussos",
    "Evangelou",
    "Fotiadis",
    "Stefanidis",
    "Mylona",
    "Gkikas",
    "Angelopoulou",
    "Eleftheriadis",
    "Kontogiannis",
    "Dimitriadou",
    "Samara",
    "Spyrou",
    "Stavropoulou",
    "Petropoulou",
    "Symeonidis",
    "Kalogeropoulos",
    "Konstantopoulos",
    "Filippou",
    "Kyriakopoulos",
    "Vogiatzis",
    "Athanasopoulou",
    "Eleftheriou",
    "Fotopoulos",
    "Voulgaris",
    "Christopoulou",
    "Chatzopoulos",
    "Spyropoulou",
    "Argyriou",
    "Georgopoulou",
    "Triantafyllidis",
    "Sakellariou",
    "Avramidis",
    "Kanellopoulos",
    "Athanasiadou",
    "Spanou",
    "Sidiropoulou",
    "Kyriazis",
    "Papachristou",
    "Petrakis",
    "Daskalakis",
    "Stefanou",
    "Anagnostopoulou",
    "Kritikos",
    "Pavlou",
    "Kechagias",
    "Rigas",
    "Arampatzis",
    "Kostopoulou",
    "Zervas",
    "Lazaridou",
    "Alexopoulou",
    "Theodoridou",
    "Stamatiou",
    "Asimakopoulos",
    "Tsakiris",
    "Apostolopoulou",
    "Theodosiou",
    "Mitropoulos",
    "Paraskevopoulou",
    "Panagopoulou",
    "Panou",
    "Karalis",
    "Pavlidou",
    "Karras",
    "Dimitrakopoulos",
    "Ali",
    "Papanastasiou",
    "Michail",
    "Adamopoulos",
    "Michalopoulou",
    "Rapti",
    "Deligiannis",
    "Papatheodorou",
    "Vasilopoulou",
    "Iliopoulou",
    "Panagiotidou",
    "Markopoulos",
    "Antoniadou",
    "Singh",
    "Kefalas",
    "Papagiannis",
    "Kontou",
    "Zisis",
    "Stamatopoulos",
    "Grigoriou",
    "Theocharis",
    "Stamou",
    "Nikou",
    "Kolovos",
    "Xanthopoulos",
    "Theodoropoulou",
    "Giannoulis",
    "Angelidis",
    "Kyritsis",
    "Diamanti",
    "Apostolidou",
    "Doukas",
    "Paschalidis",
    "Floros",
    "Fragkos",
    "Mavridou",
    "Paraskevas",
    "Sideri",
    "Roussou",
    "Roumeliotis",
    "Tsoukalas",
    "Galani",
    "Drosos",
    "Lampropoulou",
    "Loukas",
    "Kosmidis",
    "Alexandris",
    "Pantazi",
    "Charalampidou",
    "Kalyvas",
    "Sarri",
    "Papakostas",
    "Argyropoulou",
    "Kyriakidou",
    "Anastasiadou",
    "Kokkinos",
    "Stathopoulou",
    "Anastasopoulou",
    "Stamoulis",
    "Arvaniti",
    "Sotiropoulou",
    "Stamatis",
    "Ziogas",
    "Mouratidis",
    "Lazarou",
    "Moustakas",
    "Diamantopoulou",
    "Vasilaki",
    "Dimos",
    "Papoutsis",
    "Fragkou",
    "Chalkias",
    "Zafeiropoulos",
    "Manolis",
    "Kyriakopoulou",
    "Georgakopoulou",
    "Lekkas",
    "Andreadis",
    "Chronopoulos",
    "Papantoniou",
    "Polyzos",
    "Savvidou",
    "Petridou",
    "Liapis",
    "Papandreou",
    "Emmanouil",
    "Marinos",
    "Bouras",
    "Emmanouilidis",
    "Angeli",
    "Goulas",
    "Tzimas",
    "Zacharopoulos",
    "Nikolakopoulos",
    "Kalantzis",
    "Kollia",
    "Nikas",
    "Drakopoulos",
    "Michos",
    "Vlachopoulos",
    "Tsekouras",
    "Daskalaki",
    "Charalampous",
    "Apostolakis",
    "Kalaitzis",
    "Kalogirou",
    "Nakos",
    "Aslanidis",
    "Fotopoulou",
    "Stamatakis",
    "Koulouris",
    "Liakos",
    "Konstantopoulou",
    "Dimitrakopoulou",
    "Chatzopoulou",
    "Kosmas",
    "Eleftheriadou",
    "Grivas",
    "Oikonomidis",
    "Grigoriadou",
    "Karakostas",
    "Psarras",
    "Angelou",
    "Tsolakis",
    "Moschos",
    "Giannakis",
    "Giannakos",
    "Chatzidimitriou",
    "Iosifidis",
    "Papamichail",
    "Kanellopoulou",
    "Dimas",
    "Zografos",
    "Kalogeropoulou",
    "Chatzinikolaou",
    "Voulgari",
    "Markakis",
    "Karagiannidis",
    "Stamos",
    "Zafeiris",
    "Zachos",
    "Gkika",
    "Chatziioannou",
    "Symeonidou",
    "Stefanidou",
    "Triantafyllidou",
    "Kritikou",
    "Stavridis",
});

const NameFormats greekNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames greekPeopleNames{
    {greekMaleFirstNames, greekLastNames, {}, {}}, {greekFemaleFirstNames, greekLastNames, {}, {}}, greekNameFormats};

// Hungary

const auto hungarianMaleFirstNames = std::to_array<std::string_view>({
    "Bence",    "Máté",       "Dominik", "Levente",  "Noel",      "Dániel",    "Zalán",    "Marcell",   "Olivér",
    "Ádám",     "Milán",      "Dávid",   "Botond",   "Benett",    "Áron",      "Zsombor",  "Balázs",    "Márk",
    "Péter",    "Kristóf",    "Zétény",  "Nimród",   "László",    "Benedek",   "Tamás",    "Gergő",     "Patrik",
    "Zente",    "Bálint",     "András",  "Zoltán",   "Attila",    "Barnabás",  "Márton",   "Hunor",     "Kornél",
    "Martin",   "Ákos",       "Gábor",   "Alex",     "István",    "Vince",     "Ábel",     "Krisztián", "Zsolt",
    "János",    "Nolen",      "Erik",    "Bendegúz", "Sándor",    "József",    "Nándor",   "Richárd",   "Benjámin",
    "Soma",     "Ármin",      "Csaba",   "Kevin",    "Róbert",    "Ferenc",    "Mátyás",   "Norbert",   "Roland",
    "Vencel",   "Viktor",     "Csongor", "Szabolcs", "Mihály",    "Boldizsár", "Benjamin", "Simon",     "Gergely",
    "Bertalan", "Tibor",      "Zénó",    "Adrián",   "Miklós",    "Nikolasz",  "Vilmos",   "Kende",     "Imre",
    "Gellért",  "Krisztofer", "Donát",   "Vendel",   "Bende",     "Brájen",    "Dorián",   "Denisz",    "Sámuel",
    "Noé",      "Csanád",     "Lajos",   "Kolos",    "Alexander", "Laurent",   "Mirkó",    "György",    "Nátán",
    "Iván",
});

const auto hungarianFemaleFirstNames = std::to_array<std::string_view>({
    "Hanna",   "Anna",     "Zoé",      "Léna",    "Luca",    "Emma",    "Zsófia",    "Boglárka", "Lili",     "Mira",
    "Lilien",  "Nóra",     "Laura",    "Lara",    "Maja",    "Fanni",   "Jázmin",    "Izabella", "Olívia",   "Sára",
    "Gréta",   "Alíz",     "Adél",     "Lilla",   "Liza",    "Flóra",   "Dorka",     "Zselyke",  "Dóra",     "Csenge",
    "Janka",   "Liliána",  "Viktória", "Szofia",  "Réka",    "Panna",   "Petra",     "Dorina",   "Natasa",   "Blanka",
    "Eszter",  "Noémi",    "Emília",   "Bianka",  "Kamilla", "Szófia",  "Vivien",    "Rebeka",   "Elizabet", "Linett",
    "Szofi",   "Júlia",    "Szonja",   "Abigél",  "Mia",     "Bella",   "Lia",       "Natália",  "Tamara",   "Lotti",
    "Róza",    "Amira",    "Eliza",    "Panka",   "Milla",   "Zorka",   "Hanga",     "Nara",     "Norina",   "Fruzsina",
    "Mirella", "Dorottya", "Boróka",   "Emili",   "Regina",  "Johanna", "Alexandra", "Diána",    "Elena",    "Kinga",
    "Hédi",    "Borbála",  "Emese",    "Bíborka", "Barbara", "Letícia", "Lujza",     "Zita",     "Kincső",   "Nina",
    "Vanda",   "Veronika", "Zara",     "Lana",    "Lora",    "Kiara",   "Lívia",     "Rozina",   "Zejnep",   "Virág",
});

const auto hungarianLastNames = std::to_array<std::string_view>({
    "Nagy",     "Kovács",  "Tóth",    "Szabó",   "Horváth", "Varga",    "Kiss",    "Molnár", "Németh", "Farkas",
    "Balogh",   "Papp",    "Takács",  "Juhász",  "Lakatos", "Mészáros", "Oláh",    "Simon",  "Rácz",   "Fekete",
    "Szilágyi", "Török",   "Fehér",   "Balázs",  "Gál",     "Kis",      "Szűcs",   "Kocsis", "Orsós",  "Pintér",
    "Fodor",    "Szalai",  "Sipos",   "Magyar",  "Lukács",  "Gulyás",   "Biró",    "Király", "László", "Katona",
    "Jakab",    "Bogdán",  "Balog",   "Sándor",  "Boros",   "Fazekas",  "Kelemen", "Váradi", "Antal",  "Somogyi",
    "Orosz",    "Fülöp",   "Veres",   "Vincze",  "Hegedűs", "Budai",    "Deák",    "Pap",    "Bálint", "Pál",
    "Illés",    "Vass",    "Szőke",   "Vörös",   "Bognár",  "Fábián",   "Lengyel", "Bodnár", "Szücs",  "Hajdu",
    "Halász",   "Jónás",   "Máté",    "Székely", "Kozma",   "Gáspár",   "Pásztor", "Bakos",  "Dudás",  "Major",
    "Orbán",    "Hegedüs", "Virág",   "Barna",   "Novák",   "Soós",     "Tamás",   "Nemes",  "Pataki", "Balla",
    "Faragó",   "Kerekes", "Borbély", "Barta",   "Péter",   "Szekeres", "Csonka",  "Mezei",  "Márton", "Sárközi",
});

const std::array<std::string_view, 2> hungarianPrefixes{"Dr.", "Prof."};

const NameFormats hungarianNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames hungarianPeopleNames{{hungarianMaleFirstNames, hungarianLastNames, hungarianPrefixes, {}},
                                       {hungarianFemaleFirstNames, hungarianLastNames, hungarianPrefixes, {}},
                                       hungarianNameFormats};

// Iceland

const auto icelandicMaleFirstNames = std::to_array<std::string_view>({
    "Gunnar",   "Einar",   "Ragnar",   "Karl",      "Aron",        "Benedikt",  "Jon",      "Jakob",     "Steinar",
    "Ari",      "Viktor",  "Axel",     "Alexander", "Anton",       "Hannes",    "Hermann",  "Emil",      "Jens",
    "Hans",     "Geir",    "Daniel",   "David",     "Magnus",      "Adam",      "Stefan",   "Robert",    "John",
    "Piotr",    "Elmar",   "Georg",    "Albert",    "Johann",      "Krzysztof", "Erling",   "Tomasz",    "Marcin",
    "Jose",     "Carlos",  "Michael",  "Mariusz",   "Alex",        "Andrzej",   "Bjorn",    "Mikael",    "Richard",
    "Jan",      "Omar",    "Dariusz",  "Victor",    "Martin",      "Marek",     "Kingsley", "Erik",      "Oskar",
    "Tomas",    "Pawel",   "Johannes", "Jonas",     "Mark",        "Umar",      "Paulo",    "Rafal",     "Sebastian",
    "Ricardo",  "Frank",   "Joao",     "Usman",     "James",       "Thomas",    "Innocent", "Michal",    "Sam",
    "Miroslaw", "William", "Paul",     "Eric",      "Henry",       "Jaroslaw",  "Fernando", "Alfred",    "Matthias",
    "Andrew",   "Luis",    "Harald",   "Jason",     "Peter",       "Nils",      "Maciej",   "Edward",    "Kamil",
    "Andres",   "Roman",   "Oliver",   "Adolf",     "Slawomir",    "Artur",     "Ifeanyi",  "Bruno",     "Julius",
    "Charles",  "Antonio", "Jacek",    "Jorge",     "Elias",       "Wojciech",  "Patrick",  "George",    "Edvard",
    "Arthur",   "Lukasz",  "Marco",    "Simon",     "Christopher", "Benjamin",  "Clement",  "Arkadiusz", "Petrea",
    "Dario",    "Chinedu", "Dawid",    "Nuno",      "Damian",      "Mohamed",   "Ivan",     "Nicolas",   "Mateusz",
    "Godfrey",  "Carl",    "Zbigniew", "Philippe",  "Hector",      "Uche",      "Konrad",   "Gabriel",   "Mario",
    "Norbert",  "Manuel",  "Ismail",   "Stephen",   "Olivier",     "Felix",     "Fred",     "Kristofer", "Kevin",
    "Samuel",   "Hugo",    "Patryk",   "Ashok",     "Adrian",      "Julio",     "Bent",     "Ikechukwu", "Cesar",
    "Jerzy",    "Fritz",   "Haruna",   "Jonathan",
});

const auto icelandicFemaleFirstNames = std::to_array<std::string_view>({
    "Anna",      "Helga",    "Inga",     "Eva",       "Linda",     "Hanna",      "Edda",      "Erna",     "Sandra",
    "Sara",      "Karen",    "Rakel",    "Elsa",      "Helena",    "Andrea",     "Marta",     "Elva",     "Thelma",
    "Agnes",     "Alda",     "Olga",     "Una",       "Sonja",     "Rebekka",    "Gudrun",    "Ellen",    "Ester",
    "Alexandra", "Selma",    "Alma",     "Stella",    "Ana",       "Kristin",    "Birgitta",  "Telma",    "Dagmar",
    "Klara",     "Margret",  "Monika",   "Magdalena", "Agnieszka", "Lena",       "Katarzyna", "Solveig",  "Emma",
    "Esther",    "Tanja",    "Barbara",  "Mary",      "Anita",     "Julia",      "Vera",      "Elin",     "Petra",
    "Rut",       "Sigrun",   "Perla",    "Helen",     "Beata",     "Malgorzata", "Heba",      "Katrin",   "Ella",
    "Elma",      "Hilda",    "Laura",    "Asta",      "Elisabet",  "Johanna",    "Juan",      "Elzbieta", "Iris",
    "Kamilla",   "Joanna",   "Ewa",      "Lea",       "Adda",      "Amanda",     "Inger",     "Martha",   "Kim",
    "Greta",     "Natalia",  "Dorota",   "Nina",      "Tara",      "Jana",       "Sarah",     "Rosa",     "Ruth",
    "Kari",      "Tunde",    "Rui",      "Teresa",    "Karolina",  "Anne",       "Berta",     "Maren",    "Elizabeth",
    "Dora",      "Jolanta",  "Ann",      "Irena",     "Renata",    "Marzena",    "Marie",     "Iwona",    "Precious",
    "Justyna",   "Patricia", "Cristina", "Rita",      "Sylwia",    "Jennifer",   "Jenny",     "Claudia",  "Diana",
    "Ingrid",    "Miriam",   "Astrid",   "Stefania",  "Emilia",    "Hilma",      "Irma",      "Edyta",    "Alice",
    "Elena",     "Gift",     "Dana",     "Ida",       "Vanessa",   "Fanny",      "Josephine", "Paulina",  "Dennis",
    "Alina",     "Bozena",   "Angela",   "Nancy",     "Lara",      "Jessica",
});

const auto icelandicLastNames = std::to_array<std::string_view>({
    "Jónsdóttir",      "Jónsson",         "Sigurðardóttir",  "Guðmundsdóttir",   "Guðmundsson",     "Sigurðsson",
    "Gunnarsdóttir",   "Gunnarsson",      "ólafsson",        "ólafsdóttir",      "Magnúsdóttir",    "Magnússon",
    "Einarsson",       "Einarsdóttir",    "Kristjánsdóttir", "Kristjánsson",     "Björnsdóttir",    "Stefánsson",
    "Jóhannsson",      "Jóhannsdóttir",   "Björnsson",       "Stefánsdóttir",    "árnadóttir",      "Bjarnason",
    "Bjarnadóttir",    "árnason",         "Halldórsson",     "Helgason",         "Halldórsdóttir",  "Helgadóttir",
    "Sveinsson",       "Ragnarsdóttir",   "Kristinsson",     "Pétursdóttir",     "Pálsson",         "Guðjónsdóttir",
    "Kristinsdóttir",  "Pálsdóttir",      "Karlsson",        "Guðjónsson",       "Pétursson",       "þorsteinsson",
    "þorsteinsdóttir", "Sveinsdóttir",    "Ragnarsson",      "óskarsdóttir",     "Karlsdóttir",     "Haraldsson",
    "Jóhannesson",     "Jóhannesdóttir",  "Sigurjónsdóttir", "Birgisson",        "Birgisdóttir",    "Hauksson",
    "Harðardóttir",    "þórðardóttir",    "ásgeirsdóttir",   "Haraldsdóttir",    "Jónasdóttir",     "Hauksdóttir",
    "Harðarson",       "Jónasson",        "Sigurjónsson",    "Kjartansdóttir",   "Kjartansson",     "ágústsson",
    "þórðarson",       "Arnarson",        "Friðriksson",     "Baldursdóttir",    "Baldursson",      "ágústsdóttir",
    "Sverrisson",      "Hilmarsson",      "ásgeirsson",      "Friðriksdóttir",   "Guðnadóttir",     "Ingólfsson",
    "Guðnason",        "Ingólfsdóttir",   "Arnardóttir",     "Benediktsdóttir",  "Hilmarsdóttir",   "Sverrisdóttir",
    "Valdimarsdóttir", "þórarinsdóttir",  "Reynisson",       "Benediktsson",     "Björgvinsdóttir", "Gunnlaugsson",
    "Jonsson",         "Gunnlaugsdóttir", "Reynisdóttir",    "Aðalsteinsdóttir", "þórisdóttir",     "Skúladóttir",
    "Hafsteinsdóttir", "ómarsdóttir",     "Sævarsdóttir",    "Sigurðarson",      "Valdimarsson",    "þórisson",
    "Rúnarsdóttir",    "Snorrason",       "Vilhjálmsdóttir", "þórarinsson",      "Hermannsson",     "Svavarsdóttir",
    "Tryggvason",      "Sævarsson",       "Garðarsdóttir",   "þorvaldsdóttir",   "Rúnarsson",       "Bragason",
    "þorvaldsson",     "Hannesdóttir",    "Vilhjálmsson",    "Björgvinsson",     "Bragadóttir",     "Egilsson",
    "Ingvarsdóttir",   "Gylfason",        "Skúlason",        "Hjartardóttir",    "Hjartarson",      "Ingvarsson",
    "Hannesson",       "Guðlaugsdóttir",  "Lárusdóttir",     "ómarsson",         "Garðarsson",      "Gylfadóttir",
    "Guðlaugsson",     "Grétarsson",      "Snorradóttir",    "Aðalsteinsson",    "Lárusson",        "Tryggvadóttir",
    "Tómasson",        "Jakobsdóttir",
});

const NameFormats icelandic_personNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames icelandicPeopleNames{{icelandicMaleFirstNames, icelandicLastNames, {}, {}},
                                       {icelandicFemaleFirstNames, icelandicLastNames, {}, {}},
                                       icelandic_personNameFormats};

// India

const auto indianMaleFirstNames = std::to_array<std::string_view>({
    "Aadi",      "Aadil",      "Aahan",     "Aarav",       "Aarit",     "Aarush",   "Aaryan",    "Aaryav",
    "Aayush",    "Abeer",      "Abhay",     "Abhinav",     "Abhishek",  "Abhiyay",  "Adarsh",    "Adhvait",
    "Aditya",    "Advait",     "Advay",     "Ahaan",       "Akash",     "Akhil",    "Akhilesh",  "Akshant",
    "Akshat",    "Akshay",     "Akshaya",   "Alok",        "Aman",      "Amar",     "Amardeep",  "Amit",
    "Amitabh",   "Anand",      "Aniket",    "Anirudh",     "Anish",     "Ankit",    "Ankur",     "Ansh",
    "Anshul",    "Anshuman",   "Anupam",    "Anurag",      "Arin",      "Arjun",    "Armaan",    "Arnav",
    "Arpit",     "Arun",       "Arush",     "Arvind",      "Arya",      "Aryaman",  "Aryan",     "Aryana",
    "Aryaveer",  "Ashutosh",   "Atharv",    "Atul",        "Avinash",   "Avir",     "Ayush",     "Baldev",
    "Bhavesh",   "Bhavya",     "Bhushan",   "Bhuvan",      "Bimal",     "Birju",    "Brij",      "Brijesh",
    "Brijmohan", "Chirag",     "Darsh",     "Darshan",     "Deep",      "Deepak",   "Deepesh",   "Dev",
    "Devang",    "Devansh",    "Devendra",  "Devesh",      "Dhairik",   "Dhairya",  "Dhaval",    "Dheeraj",
    "Dhiraj",    "Dhriti",     "Dhruv",     "Dhruva",      "Dhruvansh", "Dinesh",   "Dipak",     "Dipesh",
    "Divyansh",  "Eshan",      "Gagan",     "Ganesh",      "Gaurav",    "Gautam",   "Girish",    "Gopal",
    "Govind",    "Harish",     "Harsh",     "Harshad",     "Harshal",   "Hemant",   "Himanshu",  "Hitesh",
    "Hridaan",   "Hriday",     "Hrithik",   "Hrithvik",    "Hrushi",    "Iqbal",    "Ishaan",    "Ishan",
    "Ishant",    "Ishwar",     "Jagat",     "Jagdish",     "Jai",       "Jatin",    "Jatish",    "Jay",
    "Jayant",    "Jitender",   "Jitendra",  "Kabir",       "Kabira",    "Kabiraj",  "Kailash",   "Kairav",
    "Kamal",     "Kamlesh",    "Kapil",     "Karan",       "Karna",     "Karthik",  "Karun",     "Kashyap",
    "Kavish",    "Keshav",     "Ketan",     "Kian",        "Kiran",     "Kishan",   "Kishore",   "Krish",
    "Krishna",   "Krishnaiah", "Kunal",     "Kushal",      "Lakshay",   "Lakshit",  "Lalit",     "Laxman",
    "Lokesh",    "Manav",      "Manik",     "Maninder",    "Manish",    "Manoj",    "Mayank",    "Mayur",
    "Mohan",     "Mohit",      "Mukesh",    "Mukul",       "Mukund",    "Nakul",    "Nandan",    "Nandlal",
    "Narayan",   "Naveen",     "Neeraj",    "Nihal",       "Nikhil",    "Nilesh",   "Nimish",    "Nirav",
    "Nishant",   "Nishith",    "Nitesh",    "Nitin",       "Nitish",    "Om",       "Omkar",     "Pankaj",
    "Pariket",   "Parikshit",  "Parth",     "Partha",      "Parthiv",   "Parv",     "Parveen",   "Pradeep",
    "Pramod",    "Pranav",     "Pranay",    "Pranit",      "Pranjal",   "Pratham",  "Pratyush",  "Pravin",
    "Prayag",    "Prithvi",    "Raahat",    "Raghav",      "Rahul",     "Rajat",    "Rajendra",  "Rajesh",
    "Rajiv",     "Rajkishor",  "Rajkumar",  "Rajnish",     "Rajveer",   "Rakshan",  "Raman",     "Ramesh",
    "Ranbir",    "Ranjan",     "Ranveer",   "Rapul",       "Ravi",      "Revansh",  "Reyansh",   "Rian",
    "Ridhaan",   "Rishabh",    "Rishi",     "Rishit",      "Ritesh",    "Ritish",   "Ritvik",    "Rohan",
    "Rohant",    "Rohit",      "Rudransh",  "Rupesh",      "Rushabh",   "Rushil",   "Saarthak",  "Sachin",
    "Sagar",     "Sahil",      "Sahitya",   "Sameer",      "Sameet",    "Sanjay",   "Sanjeev",   "Sanket",
    "Sarthak",   "Sarvesh",    "Satish",    "Shantanu",    "Shardul",   "Shashank", "Shaurya",   "Shivam",
    "Shivansh",  "Shlok",      "Shourya",   "Shresht",     "Shubham",   "Shyam",    "Siddharth", "Subhash",
    "Sudhir",    "Sumeet",     "Sumit",     "Sunil",       "Sunny",     "Suraj",    "Surender",  "Suresh",
    "Surya",     "Sushant",    "Suvan",     "Tanay",       "Tanish",    "Tanmay",   "Tanush",    "Tanvik",
    "Taran",     "Tarun",      "Tavish",    "Tejas",       "Tejus",     "Tushar",   "Uday",      "Umesh",
    "Utkarsh",   "Vaibhav",    "Vansh",     "Vanshaj",     "Varun",     "Ved",      "Vedant",    "Veer",
    "Veerendra", "Vibhav",     "Vicky",     "Vidit",       "Vidyut",    "Vihaan",   "Vihan",     "Vijay",
    "Vikas",     "Vikram",     "Vikrant",   "Vimal",       "Vinay",     "Vinayak",  "Vineet",    "Vinod",
    "Vipul",     "Virat",      "Virendra",  "Viresh",      "Vishal",    "Vishesh",  "Vishnu",    "Vishwas",
    "Vivaan",    "Vivan",      "Vivek",     "Viyan",       "Yaman",     "Yash",     "Yashas",    "Yashith",
    "Yashraj",   "Yashwant",   "Yashwanta", "Yashwardhan", "Yogesh",    "Yug",      "Yuva",      "Yuvan",
    "Yuvansh",   "Yuvraj",     "Yuvraja",   "Zaheer",      "Zahir",     "Zain",     "Zayyan",
});

const auto indianFemaleFirstNames = std::to_array<std::string_view>({
    "Aadrika",  "Aadya",      "Aakriti",  "Aanya",     "Aanyaya",   "Aaradhya",  "Aarohi",     "Aashi",     "Aashika",
    "Adhithi",  "Aditi",      "Aisha",    "Akshara",   "Alia",      "Amaira",    "Amara",      "Ananya",    "Anika",
    "Anisha",   "Anushka",    "Anvi",     "Anwaya",    "Aria",      "Arunima",   "Avani",      "Avishi",    "Avni",
    "Banita",   "Bhavana",    "Bhavisha", "Bhavna",    "Bhavya",    "Bhavyata",  "Bhumi",      "Bhumika",   "Bhuvitha",
    "Chakori",  "Chandika",   "Chandna",  "Chandni",   "Chandrika", "Charmika",  "Charu",      "Charul",    "Charvi",
    "Chhavi",   "Chitra",     "Deepika",  "Devanshi",  "Devika",    "Dhanya",    "Dhriti",     "Disha",     "Divisha",
    "Divishi",  "Divya",      "Diya",     "Drishti",   "Dyuthi",    "Eeshaal",   "EeshaaniEk", "Esha",      "Eshitha",
    "Falguni",  "Falini",     "Faria",    "Farida",    "Freesha",   "Fulki",     "Gargee",     "Gargi",     "Garima",
    "Gauri",    "Giribala",   "Gitali",   "Gitanjali", "Grishma",   "Harini",    "Haritha",    "Harshini",  "Harshita",
    "Himani",   "Hina",       "Hiral",    "Hrishita",  "Inaya",     "Ira",       "Isha",       "Ishana",    "Ishani",
    "Ishika",   "Ishiqa",     "Ishira",   "Ishita",    "Ishitha",   "Ishmita",   "Janvi",      "Janya",     "Jara",
    "Jaswita",  "Jia",        "Jivana",   "Jivika",    "Jivisha",   "Jiya",      "Jyotsna",    "Kanya",     "Kashvi",
    "Kashvii",  "Kavya",      "Khushi",   "Kia",       "Kiara",     "Kiran",     "Kirti",      "Kritika",   "Kshirja",
    "Lasya",    "Lavanya",    "Lavisha",  "Lavitha",   "Leah",      "Lisha",     "Manasvi",    "Manika",    "Manisha",
    "Mannat",   "Manvi",      "Manvita",  "Meenal",    "Meera",     "Megha",     "Mira",       "Myra",      "Mysha",
    "Naina",    "Nandita",    "Natasha",  "Neha",      "Nidhi",     "Nidhita",   "Niharika",   "Nishi",     "Nitya",
    "Nityari",  "Nityasha",   "Nityashi", "Nyra",      "Oishi",     "Ojal",      "Ojaswi",     "Oshin",     "Oviya",
    "Pankhudi", "Pankhuri",   "Pari",     "Parinidhi", "Parul",     "Pihu",      "Praniti",    "Pranjal",   "Prisha",
    "Prishita", "Priyanka",   "Qirat",    "Queenie",   "Quinara",   "Quisha",    "Rashi",      "Rasika",    "Rhea",
    "Riddhi",   "Rishi",      "Ritara",   "Rithika",   "Riya",      "Roshni",    "Rupali",     "Saanvi",    "Saesha",
    "Saisha",   "Sanika",     "Sanvi",    "Sanya",     "Sara",      "Saraswati", "Sarita",     "Savita",    "Shanaya",
    "Simran",   "Sita",       "Siya",     "Sneha",     "Sonam",     "Tamanna",   "Tanishi",    "Tanvi",     "Tishira",
    "Tithira",  "Trisha",     "Ujjwala",  "Urisha",    "Urvashi",   "Urvi",      "Utkarsha",   "Vaishnavi", "Vanshika",
    "Vanya",    "Vasundhara", "Vidhi",    "Vidulika",  "Vinita",    "Wansika",   "Wisha",      "Wynona",    "Xandra",
    "Xara",     "Yashi",      "Yashika",  "Yashvi",    "Yukta",     "Yukthi",    "Zaina",      "Zaira",     "Zara",
    "Zarika",   "Zarna",      "Zoya",
});

const auto indianLastNames = std::to_array<std::string_view>({
    "Acharya",      "Agarwal",      "Aggarwal",     "Ahluwalia",   "Ahuja",     "Amin",        "Anand",
    "Arora",        "Arvind",       "Babu",         "Bajaj",       "Bajwa",     "Banerjee",    "Bansal",
    "Batra",        "Bhagat",       "Bhardwaj",     "Bhargava",    "Bhasin",    "Bhat",        "Bhatia",
    "Bhatnagar",    "Bhatt",        "Bhattacharya", "Biswas",      "Bose",      "Chabra",      "Chadha",
    "Chakrabarti",  "Chakrabarty",  "Chakraborty",  "Chakravarti", "Chandra",   "Chatterjee",  "Chaturvedi",
    "Chauhan",      "Chawla",       "Chopra",       "Choudhary",   "Choudhury", "Das",         "Deshmukh",
    "Deshpande",    "Dewan",        "Dey",          "Dhawan",      "Dhillon",   "Dixit",       "Dube",
    "Dutt",         "Dutta",        "Easwaran",     "Edwin",       "Ekbote",    "Eswaran",     "Fadnavis",
    "Fernandes",    "Gandhi",       "Ghosh",        "Ghoshal",     "Gill",      "Gokhale",     "Goswami",
    "Gupta",        "Harikrishnan", "Hegde",        "Inamdar",     "Iyengar",   "Iyer",        "Jaiswal",
    "Jha",          "Joglekar",     "Joshi",        "Juneja",      "Kadam",     "Kakkar",      "Kamble",
    "Kapoor",       "Kashyap",      "Kaur",         "Khan",        "Khanna",    "Khera",       "Khosla",
    "Khurana",      "Kohli",        "Kulkarni",     "Kumar",       "Lal",       "Lalwani",     "Luthra",
    "Mahajan",      "Majumdar",     "Malhotra",     "Malik",       "Mandal",    "Mathur",      "Mazumdar",
    "Mehra",        "Mehrotra",     "Mehta",        "Menon",       "Mishra",    "Mittal",      "Mukherjee",
    "Mukhopadhyay", "Mundra",       "Nag",          "Nagpal",      "Nair",      "Nambiar",     "Nanda",
    "Narang",       "Narula",       "Nayar",        "Negi",        "Oberoi",    "Ojha",        "Oza",
    "Padmanabhan",  "Pandey",       "Pandit",       "Patil",       "Pawar",     "Pillai",      "Pradhan",
    "Prakash",      "Puri",         "Qazi",         "Qureshi",     "Raghavan",  "Raghunathan", "Rai",
    "Raj",          "Rajput",       "Rana",         "Rao",         "Rastogi",   "Rathore",     "Reddy",
    "Sabharwal",    "Sardar",       "Sarin",        "Saxena",      "Sen",       "Sengupta",    "Seth",
    "Sethi",        "Shah",         "Sharma",       "Shukla",      "Singh",     "Singhal",     "Sinha",
    "Soni",         "Sood",         "Suri",         "Talreja",     "Talwar",    "Thakkar",     "Thakur",
    "Tiwari",       "Trivedi",      "Tyagi",        "Upadhyay",    "Upadhyaya", "Uppal",       "Vaid",
    "Vaidya",       "Varghese",     "Varma",        "Verma",       "Vij",       "Vora",        "Vyas",
    "Wadhwa",       "Wagh",         "Waghmare",     "Wagle",       "Walia",     "Xavier",      "Yadav",
    "Zakaria",      "Zaveri",
});

const NameFormats indianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames indianPeopleNames{{indianMaleFirstNames, indianLastNames, {}, {}},
                                    {indianFemaleFirstNames, indianLastNames, {}, {}},
                                    indianNameFormats};

// Ireland

const auto irishMaleFirstNames = std::to_array<std::string_view>({
    "John",     "Michael",     "Patrick",   "Thomas",    "James",     "David",     "Joseph",     "Paul",
    "Sean",     "William",     "Martin",    "Peter",     "Gerard",    "Brian",     "Kevin",      "Brendan",
    "Liam",     "Anthony",     "Richard",   "Robert",    "Frank",     "Daniel",    "Stephen",    "Edward",
    "Mark",     "Alan",        "Tony",      "Timothy",   "Andrew",    "Declan",    "Philip",     "Conor",
    "Vincent",  "Christopher", "Niall",     "Kieran",    "Gerry",     "Bernard",   "Aidan",      "Raymond",
    "Barry",    "George",      "Colm",      "Derek",     "Eugene",    "Matthew",   "Kenneth",    "Eamon",
    "Desmond",  "Hugh",        "Charles",   "Oliver",    "Shane",     "Ciaran",    "Ian",        "Micheal",
    "Damien",   "Chris",       "Gary",      "Keith",     "Ronan",     "Adrian",    "Maurice",    "Neil",
    "Colin",    "Nicholas",    "Jeremiah",  "Owen",      "Eddie",     "Simon",     "Henry",      "Jerry",
    "Darren",   "Terry",       "Jack",      "Tommy",     "Gerald",    "Terence",   "Billy",      "Jimmy",
    "Jason",    "Rory",        "Larry",     "Jonathan",  "Harry",     "Leo",       "Graham",     "Trevor",
    "Eric",     "Donald",      "Arthur",    "Gavin",     "Dominic",   "Karl",      "Phillip",    "Steven",
    "Edmond",   "Nigel",       "Walter",    "Austin",    "Albert",    "Damian",    "Gabriel",    "Cyril",
    "Tomas",    "Greg",        "Luke",      "Roger",     "Bill",      "Joe",       "Adam",       "Charlie",
    "Gordon",   "Victor",      "Ivan",      "Justin",    "Alexander", "Bryan",     "Norman",     "Garry",
    "Stuart",   "Roy",         "Louis",     "Alex",      "Gareth",    "Ronald",    "Cornelius",  "Leonard",
    "Ross",     "Wayne",       "Jerome",    "Marcin",    "Jim",       "Edmund",    "Gregory",    "Piotr",
    "Fredrick", "Jamie",       "Samuel",    "Cian",      "Tomasz",    "Carl",      "Ben",        "Sam",
    "Pawel",    "Johnny",      "Julian",    "Ken",       "Marcus",    "Nick",      "Aaron",      "Clive",
    "Benjamin", "Marc",        "Ali",       "Marek",     "Fran",      "Lukasz",    "Hubert",     "Shaun",
    "Douglas",  "Ray",         "Malcolm",   "Pascal",    "Krzysztof", "Ryan",      "Geoffrey",   "Michal",
    "Mervyn",   "Ronnie",      "Jan",       "Nicky",     "Craig",     "Bob",       "Bobby",      "Dean",
    "Lawrence", "Rafal",       "Ernest",    "Edwin",     "Steve",     "Christian", "Robbie",     "Mathew",
    "Glen",     "Scott",       "Aiden",     "Tim",       "Jose",      "Muhammad",  "Killian",    "Alfred",
    "Glenn",    "Grzegorz",    "Stanley",   "Sebastian", "Jeremy",    "Sylvester", "Antonio",    "Mariusz",
    "Ivor",     "Frankie",     "Frederick", "Jeff",      "Connor",    "Harold",    "Mohammed",   "Neville",
    "Dominick", "Felix",       "Marius",    "Ralph",     "Maciej",    "Benny",     "Bart",       "Wesley",
    "Jacek",    "Herbert",     "Ron",       "Andrzej",   "Allan",     "Ahmed",     "Wojciech",   "Teddy",
    "Warren",   "Dick",        "Stefan",    "Artur",     "Roland",    "Mario",     "Bruce",      "Jon",
    "Clement",  "Stewart",     "Jude",      "Marco",     "Emmanuel",  "Russell",   "Allen",      "Andre",
    "Mohammad", "Howard",      "Joey",      "Alec",      "Dariusz",   "Max",       "Christoph",  "Basil",
    "Evan",     "Igor",        "Nicolas",   "Carlos",    "Rodney",    "Iain",      "Stan",       "Jaroslaw",
    "Reginald", "Matt",        "Roman",     "Dylan",     "Augustine", "Leon",      "Przemyslaw", "Arkadiusz",
    "Stephan",  "Darius",      "Hans",      "Mohamed",   "Vladimir",  "Freddie",   "Kenny",      "Graeme",
    "Luis",     "Hugo",        "Bruno",     "Norbert",   "Nathan",    "Clifford",  "Syed",       "Pierre",
    "Duncan",   "Roderick",    "Kamil",     "Alfie",     "Patrice",   "Jay",       "Jakub",      "Abdul",
    "Raphael",  "Oleg",        "Radoslaw",  "Derrick",   "Conrad",    "Lukas",     "Bartosz",    "Bert",
    "Roberto",  "Anton",       "Manuel",    "Ioan",      "Fabio",     "Jeffrey",   "Percy",      "Alistair",
    "Mateusz",  "Miguel",      "Andrei",    "Ricky",     "Pavel",     "Pedro",     "Guy",        "Luca",
    "Gilbert",  "Jacob",       "Kelvin",    "Benedict",  "Morris",    "Antony",    "Sergio",     "Alessandro",
    "Zbigniew", "Slawomir",    "Omar",      "Ricardo",   "Jordan",    "Guillaume", "Arnold",     "Christophe",
    "Joshua",   "Markus",      "Dominik",   "Javier",    "Marcel",    "Johnathan", "Cedric",     "Rick",
    "Lucas",    "Angelo",      "Frederic",  "Lionel",    "Olivier",   "Angus",     "Andrius",    "Cristian",
    "Erik",     "Francisco",   "Daryl",     "Eduardo",   "Wolfgang",  "Jorge",
});

const auto irishFemaleFirstNames = std::to_array<std::string_view>({
    "Mary",      "Margaret",  "Ann",        "Kathleen", "Anne",     "Catherine",  "Elizabeth",  "Patricia",
    "Noel",      "Eileen",    "Marie",      "Denis",    "Bridget",  "Helen",      "Teresa",     "Sarah",
    "Joan",      "Geraldine", "Maureen",    "Caroline", "Deirdre",  "Susan",      "Michelle",   "Bernadette",
    "Fiona",     "Carmel",    "Angela",     "Maria",    "Anna",     "Karen",      "Nora",       "Sheila",
    "Claire",    "Sinead",    "Pauline",    "Sharon",   "Siobhan",  "Lisa",       "Francis",    "Linda",
    "Sandra",    "Josephine", "Maura",      "Louise",   "Elaine",   "Martina",    "Frances",    "Rose",
    "Niamh",     "Lorraine",  "Laura",      "Jennifer", "Rita",     "Barbara",    "Christina",  "Valerie",
    "Noreen",    "Carol",     "Paula",      "Yvonne",   "Ellen",    "Christine",  "Marian",     "Emma",
    "Theresa",   "Philomena", "Jane",       "Annie",    "Nuala",    "Denise",     "Nicola",     "Una",
    "Alice",     "Ciara",     "Julie",      "Christy",  "Gillian",  "Rachel",     "Laurence",   "Amanda",
    "Clare",     "Brigid",    "Joanne",     "Aoife",    "Suzanne",  "Ruth",       "Monica",     "Evelyn",
    "Annette",   "Veronica",  "Aisling",    "Aine",     "Colette",  "Rosemary",   "Brenda",     "Jean",
    "Olive",     "Irene",     "Kate",       "Agnes",    "Audrey",   "Maeve",      "Jacqueline", "Edel",
    "Marion",    "Betty",     "Tara",       "Miriam",   "Kay",      "Pamela",     "Alison",     "Julia",
    "Grace",     "Dolores",   "Helena",     "Deborah",  "Phyllis",  "Aileen",     "Olivia",     "Esther",
    "Tracey",    "Nancy",     "Imelda",     "Vera",     "Eleanor",  "Cathy",      "Andrea",     "Rebecca",
    "Tracy",     "Therese",   "Anita",      "Janet",    "Tina",     "Maire",      "Lucy",       "Stephanie",
    "Peggy",     "Celine",    "Danny",      "Donna",    "Jacinta",  "Jenny",      "Dorothy",    "Robin",
    "Lorna",     "Emily",     "Sally",      "Liz",      "Hannah",   "Samantha",   "Amy",        "Florence",
    "Joanna",    "Sylvia",    "June",       "Ursula",   "Shirley",  "Antoinette", "Gemma",      "Martha",
    "Debbie",    "Regina",    "Katherine",  "May",      "Eva",      "Dennis",     "Hazel",      "Dara",
    "Katie",     "Doreen",    "Georgina",   "Moira",    "Natasha",  "Victoria",   "Wendy",      "Kitty",
    "Adrienne",  "Sara",      "Hilary",     "Johanna",  "Tanya",    "Avril",      "Leslie",     "Kelly",
    "Kim",       "Jessica",   "Joyce",      "Connie",   "Winifred", "Judith",     "Lynn",       "Cora",
    "Diane",     "Vivienne",  "Lynda",      "Marcella", "Natalie",  "Olga",       "Monika",     "Charlotte",
    "Kathy",     "Stella",    "Agnieszka",  "Madeline", "Heather",  "Vanessa",    "Josie",      "Muriel",
    "Annemarie", "Bernadett", "Marguerite", "Melissa",  "Norma",    "Marta",      "Diana",      "Judy",
    "Marina",    "Melanie",   "Mona",       "Dawn",     "Lena",     "Morgan",     "Lily",       "Celia",
    "Sonia",     "Magdalena", "Nellie",     "Beatrice", "Sadie",    "Joy",        "Lee",        "Sabrina",
    "Delia",     "Janice",    "Vivian",     "Gina",     "Mai",      "Jill",       "Noelle",     "Margret",
    "Hilda",     "Nicole",    "Loretta",    "Kathryn",  "Naomi",    "Leanne",     "Gwen",       "Elena",
    "Ashley",    "Marjorie",  "Cecilia",    "Violet",   "Natalia",  "Lesley",     "Clara",      "Gertrude",
    "Sue",       "Freda",     "Sheena",     "Michele",  "Norah",    "Rachael",    "Rosemarie",  "Vicky",
    "Lucia",     "Leona",     "Anastasia",  "Danielle", "Iris",     "Mel",        "Katarzyna",  "Lillian",
    "Gladys",    "Alexandra", "Malgorzata", "Sophie",   "Marianne", "Beata",      "Myra",       "Jeanette",
    "Cathleen",  "Gabrielle", "Inga",       "Karol",    "Carole",   "Ethel",      "Ana",        "Cathrine",
    "Ena",       "Karina",    "Ewa",        "Carolyn",  "Nina",     "Sonya",      "Zoe",        "Alma",
    "Edith",     "Ingrid",    "Gail",       "Claudia",  "Edwina",   "Kerry",      "Gloria",     "Katrina",
    "Elsie",     "Madeleine", "Rosie",      "Dan",      "Shelia",   "Margo",      "Cristina",   "Kristina",
    "Susanne",   "Isobel",    "Belinda",    "Debra",    "Iwona",    "Lynne",      "Doris",      "Erika",
    "Charlene",  "Lauren",    "Lilian",     "Simone",   "Molly",    "Nadine",     "Rena",       "Adele",
    "Renata",    "Janette",   "Lilly",      "Ina",      "Leah",     "Irina",      "Svetlana",   "Virginia",
    "Assumpta",  "Silvia",    "Sylwia",     "Ella",     "Cheryl",   "Karolina",   "Maggie",     "Stacey",
    "Rasa",      "Agata",     "Dorota",     "Arlene",   "Maud",     "Cynthia",
});

const auto irishLastNames = std::to_array<std::string_view>({
    "Murphy",      "Kelly",       "Byrne",      "Walsh",      "Ryan",       "Doyle",      "Lynch",      "Mccarthy",
    "Brennan",     "Dunne",       "Murray",     "Burke",      "Smith",      "Daly",       "Nolan",      "Kennedy",
    "Fitzgerald",  "Gallagher",   "Flynn",      "Collins",    "Power",      "Quinn",      "Farrell",    "Carroll",
    "Kavanagh",    "Connolly",    "Healy",      "Clarke",     "Duffy",      "Doherty",    "Brady",      "Whelan",
    "Kenny",       "Fitzpatrick", "Moore",      "Keane",      "Moran",      "Reilly",     "Mcgrath",    "Barry",
    "Foley",       "Hayes",       "Maher",      "Buckley",    "Martin",     "Casey",      "Roche",      "Hughes",
    "Smyth",       "Browne",      "Butler",     "Mcmahon",    "Hogan",      "Griffin",    "Egan",       "White",
    "Sweeney",     "Cullen",      "Hickey",     "Ward",       "Cunningham", "Maguire",    "Lyons",      "Mcdonnell",
    "Delaney",     "Cronin",      "Sheehan",    "Keogh",      "Mccormack",  "Higgins",    "Curran",     "Cahill",
    "Mcloughlin",  "Boyle",       "Crowley",    "Mckenna",    "Sheridan",   "Molloy",     "Barrett",    "Flanagan",
    "King",        "Moloney",     "Phelan",     "Dowling",    "Mooney",     "Mcnamara",   "Corcoran",   "Costello",
    "Hennessy",    "Malone",      "Lawlor",     "Conway",     "Fox",        "Kearney",    "Carey",      "Dempsey",
    "Duggan",      "Donnelly",    "Mccabe",     "Mcdermott",  "Dolan",      "Gleeson",    "Cleary",     "Joyce",
    "Mcdonagh",    "Kelleher",    "Mcdonald",   "Jones",      "Redmond",    "Mchugh",     "Conroy",     "Coleman",
    "Reynolds",    "Coughlan",    "Cummins",    "Kinsella",   "Hurley",     "Dillon",     "Fleming",    "Lee",
    "Keating",     "Clancy",      "Tobin",      "Breen",      "Campbell",   "Leahy",      "Sullivan",   "Morrissey",
    "Leonard",     "Rooney",      "Greene",     "Mahon",      "Finn",       "Coffey",     "Cassidy",    "Forde",
    "Heffernan",   "Long",        "Bourke",     "Cooney",     "Madden",     "Hynes",      "Mcevoy",     "Woods",
    "Williams",    "Finnegan",    "Kirwan",     "Geraghty",   "Morris",     "Wilson",     "Doran",      "Larkin",
    "Meehan",      "Hegarty",     "Galvin",     "Harrington", "Mulligan",   "Allen",      "Farrelly",   "Fahy",
    "Kiely",       "Jordan",      "Mcguinness", "Conlon",     "Kane",       "Lennon",     "Callaghan",  "Mccann",
    "Coyle",       "Boland",      "Twomey",     "Morgan",     "Monaghan",   "Treacy",     "Gorman",     "Dalton",
    "Quigley",     "Reid",        "Mcgovern",   "Mitchell",   "Noonan",     "Bradley",    "Keenan",     "Brown",
    "Russell",     "Mulcahy",     "Fagan",      "Dooley",     "Wall",       "Davis",      "Mclaughlin", "Mullen",
    "Horgan",      "Burns",       "Dwyer",      "Carr",       "Tierney",    "Keegan",     "Purcell",    "Cotter",
    "Naughton",    "Kearns",      "Scanlon",    "Lane",       "Fogarty",    "Horan",      "Thompson",   "Murtagh",
    "Harte",       "Fallon",      "Flood",      "Wallace",    "Hannon",     "Grant",      "Hanley",     "Scully",
    "Kehoe",       "Johnston",    "Slattery",   "Ahern",      "Glynn",      "Taylor",     "Connor",     "Corrigan",
    "Scott",       "Kiernan",     "Curtin",     "Rogers",     "Gannon",     "Bolger",     "Shanahan",   "Regan",
    "Flaherty",    "Donohoe",     "Nugent",     "Behan",      "Mannion",    "Gibbons",    "Donovan",    "Quirke",
    "Bergin",      "Clifford",    "Downey",     "Graham",     "Hyland",     "Quinlan",    "Henry",      "Mcmanus",
    "Prendergast", "Gaffney",     "Robinson",   "Moriarty",   "Carolan",    "Gavin",      "Condon",     "Mcgowan",
    "Coyne",       "Moco",        "Young",      "Bennett",    "Mcgee",      "Mcnally",    "Mullins",    "Brophy",
    "Howard",      "Matthews",    "Hayden",     "Fitzsimons", "Obrien",     "Gill",       "Feeney",     "Reidy",
    "Brosnan",     "Donoghue",    "Darcy",      "Mcsweeney",  "Jennings",   "Johnson",    "Walshe",     "Cox",
    "Cooke",       "Moynihan",    "Stewart",    "Owens",      "Geoghegan",  "Sheehy",     "Ennis",      "Lawless",
    "Crowe",       "Shannon",     "Gray",       "Boylan",     "Mangan",     "Rice",       "Holland",    "Furlong",
    "Jackson",     "Connell",     "Vaughan",    "Oconnor",    "Sexton",     "Maloney",    "Whyte",      "Devine",
    "Broderick",   "Anderson",    "Hunt",       "English",    "Halpin",     "Blake",      "Dunphy",     "Mcguire",
    "Harris",      "Walker",      "Devlin",     "Mcbride",    "O'brien",    "O'connor",   "O'sullivan", "O'neill",
    "O'reilly",    "O'connell",   "O'donnell",  "O'shea",     "O'leary",    "O'keeffe",   "O'donovan",  "O'mahony",
    "O'callaghan", "O'donoghue",  "O'rourke",   "O'toole",    "O'dwyer",    "O'driscoll", "O'grady",    "O'regan",
    "O'malley",    "O'riordan",   "O'gorman",   "O'halloran", "O'hara",     "O'meara",    "O'loughlin", "O'flynn",
    "O'hanlon",    "O'dowd",      "O'flaherty", "O'byrne",    "O'carroll",  "O'farrell",
});

const auto irishSuffixes = std::to_array<std::string_view>({
    "Jr.",
    "Sr.",
    "I",
    "II",
    "III",
    "IV",
    "V",
    "MD",
    "DDS",
    "PhD",
    "DVM",
});

const auto irishMalePrefixes = std::to_array<std::string_view>({"Mr.", "Dr."});

const auto irishFemalePrefixes = std::to_array<std::string_view>({"Mrs.", "Ms.", "Miss", "Dr."});

const NameFormats irishNameFormats{{{"{firstName} {lastName}", 49},
                                    {"{prefix} {firstName} {lastName}", 7},
                                    {"{firstName} {lastName} {suffix}", 7},
                                    {"{prefix} {firstName} {lastName} {suffix}", 1}}};

const PeopleNames irishPeopleNames{{irishMaleFirstNames, irishLastNames, irishMalePrefixes, irishSuffixes},
                                   {irishFemaleFirstNames, irishLastNames, irishFemalePrefixes, irishSuffixes},
                                   irishNameFormats};

// Israel

const auto israelisMaleFirstNames = std::to_array<std::string_view>({
    "דוד",    "לביא",   "אריאל",  "יוסף",  "נועם",   "אורי",    "רפאל",    "ארי",    "משה",    "יהודה",  "אברהם",
    "איתן",   "דניאל",  "ישראל",  "איתי",  "שמואל",  "אליה",    "יהונתן",  "יצחק",   "שמעון",  "יעקב",   "מיכאל",
    "עומר",   "יונתן",  "אהרון",  "חיים",  "אימרי",  "אדם",     "שלמה",    "איתמר",  "עידו",   "יאיר",   "ינאי",
    "מאיר",   "ישי",    "מרדכי",  "הראל",  "בניה",   "ניתאי",   "בנימין",  "אליהו",  "מנחם",   "ליאם",   "בארי",
    "אלון",   "עילאי",  "עמית",   "בן",    "נתן",    "אור",     "נתנאל",   "ריף",    "אביב",   "גיא",    "נהוראי",
    "מאור",   "ריי",    "רום",    "ליאו",  "נוה",    "יובל",    "הלל",     "עומרי",  "שחר",    "אלרואי", "רוי",
    "נריה",   "נדב",    "דור",    "צבי",   "מלאכי",  "רועי",    "עידן",    "ראם",    "אביתר",  "נבו",    "תום",
    "אלחנן",  "אוריאל", "מעיין",  "אריה",  "שלום",   "גבריאל",  "אליאב",   "עוז",    "אליעזר", "שילה",   "ארבל",
    "ירדן",   "לני",    "יואב",   "נחמן",  "גפן",    "ליאור",   "יהושע",   "אביאל",  "דן",     "לוי",    "אמיתי",
    "יהב",    "עמנואל", "זיו",    "אייל",  "אלעזר",  "מתן",     "ידידיה",  "פנחס",   "יהלי",   "אופק",   "אופיר",
    "אבישי",  "בר",     "דביר",   "יחיאל", "עברי",   "אוריה",   "שקד",     "לירוי",  "שי",     "רני",    "ים",
    "נפתלי",  "יגל",    "פלג",    "אסף",   "ינון",   "אדיר",    "רון",     "שליו",   "אושר",   "כפיר",   "אשר",
    "ברוך",   "רותם",   "שניאור", "אפרים", "גל",     "דין",     "טום",     "רואי",   "תומר",   "אוראל",  "נטע",
    "אלמוג",  "עתי",    "ראובן",  "יותם",  "שמחה",   "זוהר",    "טומי",    "שון",    "רז",     "עופרי",  "יחזקאל",
    "יהל",    "ירין",   "יואל",   "צור",   "סהר",    "ליאב",    "דב",      "עקיבא",  "טל",     "זאב",    "גלעד",
    "ברי",    "גבע",    "איליי",  "אלישע", "הדר",    "אילון",   "ניב",     "אדר",    "קורן",   "עילי",   "ליעד",
    "הילל",   "אליאור", "אחיה",   "סיני",  "אושרי",  "קאי",     "אמרי",    "אלימלך", "בועז",   "יוחנן",  "כרמל",
    "אלרועי", "יאן",    "שאול",   "אסיף",  "שלו",    "ליאל",    "ברק",     "עדן",    "ארז",    "עדיאל",  "דרור",
    "ניל",    "ישעיה",  "עופר",   "יפתח",  "נחום",   "אראל",    "כרמי",    "טוהר",   "פאר",    "מרום",   "דורי",
    "ארד",    "לאו",    "רעי",    "אלעד",  "עמיחי",  "מאורי",   "נגב",     "גור",    "אביחי",  "ליאון",  "עובדיה",
    "עדי",    "אורן",   "ריו",    "און",   "שוהם",   "אלרוי",   "טוביה",   "בניהו",  "עזרא",   "גיל",    "אלאור",
    "נח",     "דולב",   "אפק",    "אביעד", "אלנתן",  "יאר",     "אילן",    "דילן",   "סאן",    "נאור",   "ניצן",
    "מבשר",   "אליעד",  "קדם",    "תבור",  "לאון",   "אליאל",   "אלקנה",   "ריין",   "נורי",   "יונה",   "אלכסנדר",
    "מתניה",  "מייקל",  "איתיאל", "אלרום", "בצלאל",  "יוחאי",   "יקותיאל", "מיאל",   "סול",    "דורון",  "אבינועם",
    "אוהד",   "חגי",    "יששכר",  "שיר",   "ענבר",   "אביגדור", "יוגב",    "ניסים",  "שגיא",   "ניר",    "נרי",
    "מתנאל",  "אייר",   "מנור",   "לידור", "אביאור", "ישעיהו",  "עזריאל",  "עמוס",   "תבל",    "חנוך",   "להב",
    "צורי",   "נחמיה",  "שמשון",  "כרם",   "אלי",    "הוד",     "יער",     "אפיק",   "שגב",    "אלישיב", "אמיר",
});

const auto israelisFemaleFirstNames = std::to_array<std::string_view>({
    "תמר",   "מאיה",   "אביגיל", "נועה",   "שרה",    "איילה",  "אדל",    "יעל",    "שירה",    "אסתר",   "חנה",
    "אלה",   "מרים",   "רחל",    "רבקה",   "ליה",    "חיה",    "טליה",   "רוני",   "רומי",    "מיכל",   "נויה",
    "אריאל", "רות",    "ליבי",   "עלמה",   "הלל",    "אמה",    "גפן",    "גאיה",   "נעמי",    "הודיה",  "עומר",
    "נגה",   "אגם",    "יובל",   "שי-לי",  "אפרת",   "לאה",    "מלכה",   "אורי",   "אוריה",   "יהלי",   "תהילה",
    "אמילי", "יהודית", "תהל",    "ארבל",   "טוהר",   "ליאן",   "הדסה",   "אבישג",  "הילה",    "אלישבע", "אלמה",
    "נעמה",  "עדי",    "שקד",    "עופרי",  "שי",     "מיקה",   "הדס",    "דבורה",  "מעיין",   "לני",    "נטע",
    "נועם",  "הדר",    "הללי",   "מיאל",   "אליה",   "ליב",    "נחמה",   "אור",    "ברכה",    "אופיר",  "אביב",
    "ריף",   "צפורה",  "שיר",    "שחר",    "עמנואל", "חוה",    "ליאל",   "מוריה",  "מיכאלה",  "דניאל",  "ירדן",
    "שושנה", "זוהר",   "אביטל",  "עדן",    "מאי",    "ניב",    "אלי",    "שוהם",   "אלין",    "עמית",   "אביה",
    "גלי",   "בר",     "אן",     "שיה",    "בתיה",   "יולי",   "פיגא",   "הילי",   "רני",     "אנאל",   "שייה",
    "גילי",  "שני",    "יסכה",   "איימי",  "זיו",    "בת שבע", "כרמל",   "סול",    "דריה",    "נאיה",   "אודל",
    "יערה",  "שולמית", "שילת",   "דניאלה", "אמונה",  "ענהאל",  "אמיליה", "ליבא",   "מילה",    "רונה",   "אורין",
    "ארייה", "שיינא",  "טובה",   "רותם",   "גיטל",   "גל",     "יוכבד",  "מאור",   "אוריאן",  "תבל",    "טל",
    "שיראל", "שפרה",   "יעלה",   "פנינה",  "אלכס",   "ליאור",  "משי",    "אליאן",  "יהב",     "איילת",  "רננה",
    "צופיה", "עמליה",  "אורה",   "תאיר",   "מיילי",  "איב",    "לורן",   "לירי",   "זואי",    "אמי",    "ים",
    "יהל",   "סופיה",  "קרני",   "אליענה", "רון",    "בת",     "אלונה",  "בלומה",  "דני",     "לייה",   "אדוה",
    "ניצן",  "לינוי",  "רז",     "רנה",    "הינדא",  "עדינה",  "ענבר",   "לין",    "דור",     "אייר",   "אחינועם",
    "פלג",   "אודיה",  "אריאה",  "עדיה",   "עטרה",   "דינה",   "נלי",    "גילה",   "ארין",    "אמור",   "צביה",
    "כרמי",  "הלני",   "נוי",    "אנה",    "ליאה",   "גוני",   "שיינדל", "לוטם",   "פערל",    "ליאם",   "דורון",
    "גולדה", "כרם",    "פריאל",  "ליהי",   "תפארת",  "כליל",   "אודליה", "נינה",   "אלומה",   "יוטא",   "בארי",
    "רינה",  "תמרה",   "אופק",   "דרור",   "חן",     "לילי",   "אהובה",  "רעות",   "אמלי",    "רייזל",  "תגל",
    "עידן",  "מישל",   "אווה",   "מנור",   "מילי",   "נסיה",   "דוריה",  "בילא",   "גיל",     "סתיו",   "קים",
    "אלמוג", "אדר",    "חושן",   "ספיר",   "אילת",   "סהר",    "יפה",    "אלינור", "מיי",     "שלי",    "תום",
    "מיתר",  "נילי",   "היילי",  "יסמין",  "ירין",   "ריין",   "לירון",  "איטה",   "ליהיא",   "מרגלית", "מייה",
    "גיטי",  "אושר",   "תומר",   "אופל",   "איידל",  "חגית",   "מנוחה",  "ניקול",  "סימה",    "דר",     "נאוה",
    "תהלה",  "מאורי",  "אסנת",   "טויבא",  "רוחמה",  "שירי",   "הניה",   "פריידא", "קדם",     "בל",     "אליאנה",
    "קמה",   "רום",    "פאר",    "לי",     "מטר",    "סימא",   "עליזה",  "ריי",    "אוליביה", "אשירה",  "הענא",
});

const auto israelisLastNames = std::to_array<std::string_view>({
    "כהן",      "לוי",      "מזרחי",    "פרץ",       "ביטון",        "דהן",    "אברהם",    "פרידמן",   "אזולאי",
    "מלכה",     "כץ",       "דוד",      "אוחיון",    "חדד",          "עמר",    "גבאי",     "יוסף",     "בן דוד",
    "אדרי",     "קליין",    "חן",       "טל",        "לוין",         "שפירא",  "חזן",      "מחאמיד",   "זועבי",
    "לוין",     "מנשה",     "אוחנה",    "סועאד",     "גולן",         "סגל",    "אשכנזי",   "יצחק",     "בר",
    "יעקב",     "מור",      "שלום",     "אליהו",     "דיין",         "סויסה",  "לביא",     "אלבז",     "שמש",
    "אטיאס",    "בכר",      "נחום",     "שטרן",      "שרעבי",        "אלון",   "רוזנברג",  "ממן",      "בן חמו",
    "מימון",    "שחר",      "וקנין",    "ששון",      "עובדיה",       "אסולין", "שורץ",     "מאיר",     "וייס",
    "גרינברג",  "חסן",      "אמסלם",    "גולדשטיין", "ברוך",         "טאהא",   "עזרא",     "יפרח",     "פלדמן",
    "רובין",    "הראל",     "סלאמה",    "סבג",       "קדוש",         "מנסור",  "עמאש",     "חיים",     "שחאדה",
    "רז",       "קורן",     "גולדברג",  "צור",       "חסון",         "אברמוב", "שרון",     "רבינוביץ", "סופר",
    "חלבי",     "ישראל",    "דדון",     "יצחקי",     "שוורץ",        "ישראלי", "דניאל",    "נעים",     "אהרוני",
    "לב",       "עיסא",     "רחמים",    "הרוש",      "נאסר",         "שושן",   "ברק",      "יחזקאל",   "דוידוב",
    "יאסין",    "מילר",     "אורן",     "עודה",      "קפלן",         "סלימאן", "בנימין",   "נגר",      "חיון",
    "גרוס",     "רוזן",     "גל",       "הרשקוביץ",  "סולומון",      "אסרף",   "מלול",     "אבו כף",   "סלע",
    "שטרית",    "אבראהים",  "ברקוביץ",  "יהודה",     "פלד",          "צרפתי",  "הייב",     "בן שושן",  "זידאן",
    "זוהר",     "שלו",      "עלי",      "זיו",       "עמאר",         "גאנם",   "גוטמן",    "אהרון",    "גנאים",
    "מויאל",    "מרעי",     "אבו ליל",  "אלפסי",     "אליאס",        "אדלר",   "פרנקל",    "גאבר",     "פלג",
    "פישר",     "ברגר",     "ראובן",    "מסרי",      "שכטר",         "צדוק",   "ויצמן",    "הלוי",     "אביטן",
    "אהרון",    "לוגסי",    "נסים",     "רוזנפלד",   "מועלם",        "שוויקי", "ליפשיץ",   "דרעי",     "עטיה",
    "תורגמן",   "עואד",     "שוקרון",   "שקד",       "דגן",          "שמעוני", "דיאב",     "רוט",      "ליברמן",
    "יעקובוב",  "ליבוביץ",  "סעיד",     "מנחם",      "בן חיים",      "חורי",   "פולק",     "ברכה",     "עאסלה",
    "פוקס",     "חדאד",     "גולדמן",   "שמואלי",    "שגב",          "אברהמי", "מוסא",     "תורג'מן",  "פרי",
    "הררי",     "אילוז",    "סרחאן",    "מנשה",      "אוזן",         "מרדכי",  "קוגן",     "יונס",     "צברי",
    "עומר",     "קרן",      "שדה",      "ח'לאילה",   "שאול",         "אביטל",  "שני",      "סלומון",   "קאסם",
    "אלימלך",   "שטיין",    "עבדאללה",  "שיטרית",    "דרור",         "מלמד",   "יונה",     "כחלון",    "חביב",
    "טויטו",    "חביבאללה", "סבן",      "חמו",       "פנחס",         "בן עמי", "סעדי",     "בוזגלו",   "דנינו",
    "פז",       "חמדאן",    "בן סימון", "רוזנטל",    "בן משה",       "סעד",    "גדיר",     "יוסופוב",  "בוסקילה",
    "בדארנה",   "ארביב",    "אמארה",    "דביר",      "אבו אל היג'א", "אביב",   "עליאן",    "שלמה",     "פורת",
    "פריד",     "גוטליב",   "חיימוב",   "קאופמן",    "רביבו",        "אשר",    "אבו סאלח", "מרקוביץ",  "אביטבול",
    "אברמוביץ", "יעקבי",    "שניידר",   "אבו רמילה", "אלקיים",       "רגב",    "נחמני",    "זהבי",     "פינטו",
});

const NameFormats israelisNameFormats{{"{lastName} {firstName}", 1}};

const PeopleNames israeliPeopleNames{{israelisMaleFirstNames, israelisLastNames, {}, {}},
                                     {israelisFemaleFirstNames, israelisLastNames, {}, {}},
                                     israelisNameFormats};

// Italy

const auto italianMaleFirstNames = std::to_array<std::string_view>({
    "Abaco",      "Abbondanzio",  "Abbondio",     "Abdone",     "Abelardo",     "Abele",       "Abenzio",
    "Abibo",      "Abramio",      "Abramo",       "Acacio",     "Acario",       "Accursio",    "Achille",
    "Acilio",     "Aciscolo",     "Acrisio",      "Adalardo",   "Adalberto",    "Adalfredo",   "Adalgiso",
    "Adalrico",   "Adamo",        "Addo",         "Adelardo",   "Adelberto",    "Adelchi",     "Adelfo",
    "Adelgardo",  "Adelmo",       "Adeodato",     "Adolfo",     "Adone",        "Adriano",     "Adrione",
    "Afro",       "Agabio",       "Agamennone",   "Agapito",    "Agazio",       "Agenore",     "Agesilao",
    "Agostino",   "Agrippa",      "Aiace",        "Aidano",     "Aimone",       "Aladino",     "Alamanno",
    "Alano",      "Alarico",      "Albano",       "Alberico",   "Alberto",      "Albino",      "Alboino",
    "Albrico",    "Alceo",        "Alceste",      "Alcibiade",  "Alcide",       "Alcino",      "Aldo",
    "Aldobrando", "Aleandro",     "Aleardo",      "Aleramo",    "Alessandro",   "Alessio",     "Alfio",
    "Alfonso",    "Alfredo",      "Algiso",       "Alighiero",  "Almerigo",     "Almiro",      "Aloisio",
    "Alvaro",     "Alviero",      "Alvise",       "Amabile",    "Amadeo",       "Amando",      "Amanzio",
    "Amaranto",   "Amato",        "Amatore",      "Amauri",     "Ambrogio",     "Ambrosiano",  "Amedeo",
    "Amelio",     "Amerigo",      "Amico",        "Amilcare",   "Amintore",     "Amleto",      "Amone",
    "Amore",      "Amos",         "Ampelio",      "Anacleto",   "Andrea",       "Angelo",      "Aniceto",
    "Aniello",    "Annibale",     "Ansaldo",      "Anselmo",    "Ansovino",     "Antelmo",     "Antero",
    "Antimo",     "Antino",       "Antioco",      "Antonello",  "Antonio",      "Apollinare",  "Apollo",
    "Apuleio",    "Aquilino",     "Araldo",       "Aratone",    "Arcadio",      "Archimede",   "Archippo",
    "Arcibaldo",  "Ardito",       "Arduino",      "Aresio",     "Argimiro",     "Argo",        "Arialdo",
    "Ariberto",   "Ariele",       "Ariosto",      "Aris",       "Aristarco",    "Aristeo",     "Aristide",
    "Aristione",  "Aristo",       "Aristofane",   "Aristotele", "Armando",      "Arminio",     "Arnaldo",
    "Aronne",     "Arrigo",       "Arturo",       "Ascanio",    "Asdrubale",    "Asimodeo",    "Assunto",
    "Asterio",    "Astianatte",   "Ataleo",       "Atanasio",   "Athos",        "Attila",      "Attilano",
    "Attilio",    "Auberto",      "Audace",       "Augusto",    "Aureliano",    "Aurelio",     "Auro",
    "Ausilio",    "Averardo",     "Azeglio",      "Azelio",     "Bacco",        "Baldassarre", "Balderico",
    "Baldo",      "Baldomero",    "Baldovino",    "Barbarigo",  "Bardo",        "Bardomiano",  "Barnaba",
    "Barsaba",    "Barsimeo",     "Bartolo",      "Bartolomeo", "Basileo",      "Basilio",     "Bassiano",
    "Bastiano",   "Battista",     "Beato",        "Bellino",    "Beltramo",     "Benedetto",   "Beniamino",
    "Benigno",    "Benito",       "Benvenuto",    "Berardo",    "Berengario",   "Bernardo",    "Beronico",
    "Bertoldo",   "Bertolfo",     "Biagio",       "Bibiano",    "Bindo",        "Bino",        "Birino",
    "Bonagiunta", "Bonaldo",      "Bonaventura",  "Bonavita",   "Bonifacio",    "Bonito",      "Boris",
    "Bortolo",    "Brancaleone",  "Brando",       "Bruno",      "Bruto",        "Caino",       "Caio",
    "Calanico",   "Calcedonio",   "Callisto",     "Calogero",   "Camillo",      "Candido",     "Cantidio",
    "Canziano",   "Carlo",        "Carmelo",      "Carmine",    "Caronte",      "Carponio",    "Casimiro",
    "Cassiano",   "Cassio",       "Casto",        "Cataldo",    "Catullo",      "Cecco",       "Cecilio",
    "Celso",      "Cesare",       "Cesario",      "Cherubino",  "Chiaffredo",   "Cino",        "Cipriano",
    "Cirano",     "Ciriaco",      "Cirillo",      "Cirino",     "Ciro",         "Clarenzio",   "Claudio",
    "Cleandro",   "Clemente",     "Cleonico",     "Climaco",    "Clinio",       "Clodomiro",   "Clodoveo",
    "Colmanno",   "Colmazio",     "Colombano",    "Colombo",    "Concetto",     "Concordio",   "Corbiniano",
    "Coreno",     "Coriolano",    "Cornelio",     "Coronato",   "Corrado",      "Cosimo",      "Cosma",
    "Costante",   "Costantino",   "Costanzo",     "Cremenzio",  "Crescente",    "Crescenzio",  "Crespignano",
    "Crispino",   "Cristaldo",    "Cristiano",    "Cristoforo", "Crocefisso",   "Cuniberto",   "Cupido",
    "Daciano",    "Dacio",        "Dagoberto",    "Dalmazio",   "Damaso",       "Damiano",     "Damocle",
    "Daniele",    "Danilo",       "Danio",        "Dante",      "Dario",        "Davide",      "Davino",
    "Decimo",     "Delfino",      "Demetrio",     "Democrito",  "Demostene",    "Deodato",     "Desiderato",
    "Desiderio",  "Didimo",       "Diego",        "Dino",       "Diocleziano",  "Diodoro",     "Diogene",
    "Diomede",    "Dione",        "Dionigi",      "Dionisio",   "Divo",         "Dodato",      "Domenico",
    "Domezio",    "Domiziano",    "Donatello",    "Donato",     "Doriano",      "Doroteo",     "Duccio",
    "Duilio",     "Durante",      "Eberardo",     "Ecclesio",   "Edgardo",      "Edilberto",   "Edmondo",
    "Edoardo",    "Efisio",       "Efrem",        "Egeo",       "Egidio",       "Eginardo",    "Egisto",
    "Eleuterio",  "Elia",         "Eliano",       "Elifio",     "Eligio",       "Elio",        "Eliodoro",
    "Eliseo",     "Elita",        "Elmo",         "Elogio",     "Elpidio",      "Elvezio",     "Elvino",
    "Emanuele",   "Emidio",       "Emiliano",     "Emilio",     "Emmerico",     "Empirio",     "Endrigo",
    "Enea",       "Enecone",      "Ennio",        "Enrico",     "Enzo",         "Eraclide",    "Eraldo",
    "Erardo",     "Erasmo",       "Erberto",      "Ercolano",   "Ercole",       "Erenia",      "Eriberto",
    "Erico",      "Ermanno",      "Ermenegildo",  "Ermes",      "Ermete",       "Ermilo",      "Erminio",
    "Ernesto",    "Eros",         "Esaù",         "Esuperio",   "Eterie",       "Ettore",      "Euclide",
    "Eufebio",    "Eufemio",      "Eufronio",     "Eugenio",    "Eusebio",      "Euseo",       "Eustorgio",
    "Eustosio",   "Eutalio",      "Evaldo",       "Evandro",    "Evaristo",     "Evasio",      "Everardo",
    "Evidio",     "Evodio",       "Evremondo",    "Ezechiele",  "Ezio",         "Fabiano",     "Fabio",
    "Fabrizio",   "Famiano",      "Fausto",       "Fazio",      "Fedele",       "Federico",    "Fedro",
    "Felice",     "Feliciano",    "Ferdinando",   "Fermiano",   "Fermo",        "Fernando",    "Ferruccio",
    "Festo",      "Fidenziano",   "Fidenzio",     "Filiberto",  "Filippo",      "Filomeno",    "Fiorenziano",
    "Fiorenzo",   "Flaviano",     "Flavio",       "Fleano",     "Floriano",     "Folco",       "Fortunato",
    "Fosco",      "Francesco",    "Franco",       "Frido",      "Frontiniano",  "Fulberto",    "Fulgenzio",
    "Fulvio",     "Furio",        "Furseo",       "Fuscolo",    "Gabino",       "Gabriele",    "Gaetano",
    "Gaglioffo",  "Gaio",         "Galdino",      "Galeazzo",   "Galileo",      "Gallicano",   "Gandolfo",
    "Garimberto", "Gaspare",      "Gastone",      "Gaudenzio",  "Gaudino",      "Gautiero",    "Gavino",
    "Gedeone",    "Geminiano",    "Generoso",     "Genesio",    "Gennaro",      "Gentile",     "Genziano",
    "Gerardo",    "Gerasimo",     "Geremia",      "Gerino",     "Germano",      "Gerolamo",    "Geronimo",
    "Geronzio",   "Gervasio",     "Gesualdo",     "Gherardo",   "Giacinto",     "Giacobbe",    "Giacomo",
    "Giadero",    "Giambattista", "Gianbattista", "Giancarlo",  "Giandomenico", "Gianfranco",  "Gianluca",
    "Gianluigi",  "Gianmarco",    "Gianmaria",    "Gianmario",  "Gianni",       "Gianpaolo",   "Gianpiero",
    "Gianpietro", "Gianuario",    "Giasone",      "Gilberto",   "Gildo",        "Gillo",       "Gineto",
    "Gino",       "Gioacchino",   "Giobbe",       "Gioberto",   "Giocondo",     "Gioele",      "Giona",
    "Gionata",    "Giordano",     "Giorgio",      "Giosuè",     "Giosuele",     "Giotto",      "Giovanni",
    "Giove",      "Gioventino",   "Giovenzio",    "Girardo",    "Girolamo",     "Giuda",       "Giuliano",
    "Giulio",     "Giuseppe",     "Giustiniano",  "Giusto",     "Glauco",       "Goffredo",    "Golia",
    "Gomberto",   "Gondulfo",     "Gonerio",      "Gonzaga",    "Gordiano",     "Gosto",       "Gottardo",
    "Graciliano", "Grato",        "Graziano",     "Gregorio",   "Grimaldo",     "Gualberto",   "Gualtiero",
    "Guelfo",     "Guerrino",     "Guglielmo",    "Guiberto",   "Guido",        "Guiscardo",   "Gumesindo",
    "Gustavo",    "Iacopo",       "Iacopone",     "Iago",       "Icaro",        "Icilio",      "Ido",
    "Iginio",     "Igino",        "Ignazio",      "Igor",       "Ilario",       "Ildebrando",  "Ildefonso",
    "Illidio",    "Illuminato",   "Immacolato",   "Indro",      "Innocente",    "Innocenzo",   "Iorio",
    "Ippocrate",  "Ippolito",     "Ireneo",       "Isacco",     "Isaia",        "Ischirione",  "Isidoro",
    "Ismaele",    "Italo",        "Ivan",         "Ivano",      "Ivanoe",
});

const auto italianFemaleFirstNames = std::to_array<std::string_view>({
    "Abbondanza", "Acilia",      "Ada",         "Adalberta",  "Adalgisa",   "Addolorata", "Adelaide",  "Adelasia",
    "Adele",      "Adelina",     "Adina",       "Adria",      "Adriana",    "Agape",      "Agata",     "Agnese",
    "Agostina",   "Aida",        "Alba",        "Alberta",    "Albina",     "Alcina",     "Alda",      "Alessandra",
    "Alessia",    "Alfonsa",     "Alfreda",     "Alice",      "Alida",      "Alina",      "Allegra",   "Alma",
    "Altea",      "Amalia",      "Amanda",      "Amata",      "Ambra",      "Amelia",     "Amina",     "Anastasia",
    "Anatolia",   "Ancilla",     "Andromeda",   "Angela",     "Angelica",   "Anita",      "Anna",      "Annabella",
    "Annagrazia", "Annamaria",   "Annunziata",  "Antea",      "Antigone",   "Antonella",  "Antonia",   "Apollina",
    "Apollonia",  "Appia",       "Arabella",    "Argelia",    "Arianna",    "Armida",     "Artemisa",  "Asella",
    "Asia",       "Assunta",     "Astrid",      "Atanasia",   "Aurelia",    "Aurora",     "Ausilia",   "Ausiliatrice",
    "Ave",        "Aza",         "Azelia",      "Azzurra",    "Babila",     "Bambina",    "Barbara",   "Bartolomea",
    "Basilia",    "Bassilla",    "Batilda",     "Beata",      "Beatrice",   "Belina",     "Benedetta", "Beniamina",
    "Benigna",    "Benvenuta",   "Berenice",    "Bernadetta", "Betta",      "Bianca",     "Bibiana",   "Bice",
    "Brigida",    "Brigitta",    "Bruna",       "Brunilde",   "Calogera",   "Calpurnia",  "Camelia",   "Camilla",
    "Candida",    "Capitolina",  "Carina",      "Carla",      "Carlotta",   "Carmela",    "Carmen",    "Carola",
    "Carolina",   "Casilda",     "Casimira",    "Cassandra",  "Cassiopea",  "Catena",     "Caterina",  "Cecilia",
    "Celeste",    "Celinia",     "Chiara",      "Cinzia",     "Cirilla",    "Clara",      "Claudia",   "Clelia",
    "Clemenzia",  "Cleo",        "Cleofe",      "Cleopatra",  "Cloe",       "Clorinda",   "Cointa",    "Colomba",
    "Concetta",   "Consolata",   "Cora",        "Cordelia",   "Corinna",    "Cornelia",   "Cosima",    "Costanza",
    "Crescenzia", "Cristiana",   "Cristina",    "Crocefissa", "Cronida",    "Cunegonda",  "Cuzia",     "Dafne",
    "Dalida",     "Dalila",      "Damiana",     "Daniela",    "Daria",      "Deanna",     "Debora",    "Degna",
    "Delfina",    "Delia",       "Delinda",     "Delizia",    "Demetria",   "Deodata",    "Desdemona", "Desiderata",
    "Devota",     "Diamante",    "Diana",       "Dianora",    "Diletta",    "Dina",       "Diodata",   "Dionisia",
    "Doda",       "Dolores",     "Domenica",    "Donata",     "Donatella",  "Donna",      "Dora",      "Dorotea",
    "Druina",     "Dulina",      "Ebe",         "Edda",       "Edelberga",  "Editta",     "Edvige",    "Egizia",
    "Egle",       "Elaide",      "Elda",        "Elena",      "Eleonora",   "Elettra",    "Eliana",    "Elide",
    "Elimena",    "Elisa",       "Elisabetta",  "Elisea",     "Ella",       "Eloisa",     "Elsa",      "Elvia",
    "Elvira",     "Emanuela",    "Emilia",      "Emiliana",   "Emma",       "Enimia",     "Enrica",    "Eracla",
    "Ermelinda",  "Ermenegarda", "Ermenegilda", "Erminia",    "Ernesta",    "Ersilia",    "Esmeralda", "Estella",
    "Ester",      "Esterina",    "Eufemia",     "Eufrasia",   "Eugenia",    "Eulalia",    "Euridice",  "Eusebia",
    "Eutalia",    "Eva",         "Evangelina",  "Evelina",    "Fabiana",    "Fabiola",    "Fatima",    "Fausta",
    "Federica",   "Fedora",      "Felicia",     "Felicita",   "Fernanda",   "Fiammetta",  "Filippa",   "Filomena",
    "Fiordaliso", "Fiore",       "Fiorella",    "Fiorenza",   "Flaminia",   "Flavia",     "Flaviana",  "Flora",
    "Floriana",   "Floridia",    "Florina",     "Foca",       "Fortunata",  "Fosca",      "Franca",    "Francesca",
    "Fulvia",     "Gabriella",   "Gaia",        "Galatea",    "Gaudenzia",  "Gelsomina",  "Geltrude",  "Gemma",
    "Generosa",   "Genesia",     "Genoveffa",   "Germana",    "Gertrude",   "Ghita",      "Giacinta",  "Giada",
    "Gigliola",   "Gilda",       "Giliola",     "Ginevra",    "Gioacchina", "Gioconda",   "Gioia",     "Giorgia",
    "Giovanna",   "Gisella",     "Giuditta",    "Giulia",     "Giuliana",   "Giulitta",   "Giuseppa",  "Giuseppina",
    "Giusta",     "Glenda",      "Gloria",      "Godeberta",  "Godiva",     "Grazia",     "Graziana",  "Graziella",
    "Greta",      "Griselda",    "Guenda",      "Guendalina", "Gundelinda", "Ianira",     "Ida",       "Idea",
    "Iginia",     "Ilaria",      "Ilda",        "Ildegarda",  "Ildegonda",  "Ileana",     "Ilenia",    "Ilia",
    "Ilva",       "Imelda",      "Immacolata",  "Incoronata", "Ines",       "Innocenza",  "Iolanda",   "Iole",
    "Iona",       "Ione",        "Ionne",       "Irene",      "Iride",      "Iris",       "Irma",      "Irmina",
    "Isa",        "Isabella",    "Iside",       "Isidora",    "Isotta",     "Italia",     "Ivetta",    "Lara",
    "Laura",      "Lavinia",     "Lea",         "Leda",       "Lelia",      "Lena",       "Leonia",    "Leonilda",
    "Leontina",   "Letizia",     "Lia",         "Liana",      "Liberata",   "Liboria",    "Licia",     "Lidania",
    "Lidia",      "Liliana",     "Linda",       "Lisa",       "Livia",      "Liviana",    "Lodovica",  "Loredana",
    "Lorella",    "Lorena",      "Lorenza",     "Loretta",    "Loriana",    "Luana",      "Luce",      "Lucia",
    "Luciana",    "Lucilla",     "Lucrezia",    "Ludovica",   "Luigia",     "Luisa",      "Luminosa",  "Luna",
    "Macaria",    "Maddalena",   "Mafalda",     "Magda",      "Maida",      "Manuela",    "Mara",      "Marana",
    "Marcella",   "Mareta",      "Margherita",  "Maria",      "Marianna",   "Marica",     "Mariella",  "Marilena",
    "Marina",     "Marinella",   "Marinetta",   "Marisa",     "Marita",     "Marta",      "Martina",   "Maruta",
    "Marzia",     "Massima",     "Matilde",     "Maura",      "Melania",    "Melissa",    "Melitina",  "Menodora",
    "Mercede",    "Messalina",   "Mia",         "Michela",    "Milena",     "Mimma",      "Mina",      "Minerva",
    "Minervina",  "Miranda",     "Mirella",     "Miriam",     "Mirta",      "Moira",      "Monica",    "Morena",
    "Morgana",    "Nadia",       "Natalia",     "Natalina",   "Neiva",      "Nerea",      "Nicla",     "Nicoletta",
    "Nilde",      "Nina",        "Ninfa",       "Nives",      "Noemi",      "Norina",     "Norma",     "Novella",
    "Nuccia",     "Nunziata",    "Odetta",      "Odilia",     "Ofelia",     "Olga",       "Olimpia",   "Olinda",
    "Olivia",     "Oliviera",    "Ombretta",    "Ondina",     "Onesta",     "Onorata",    "Onorina",   "Orchidea",
    "Oriana",     "Orietta",     "Ornella",     "Orsola",     "Orsolina",   "Ortensia",   "Osanna",    "Otilia",
    "Ottilia",    "Palladia",    "Palmazio",    "Palmira",    "Pamela",     "Paola",      "Patrizia",  "Pelagia",
    "Penelope",   "Perla",       "Petronilla",  "Pia",        "Piera",      "Placida",    "Polissena", "Porzia",
    "Prisca",     "Priscilla",   "Proserpina",  "Prospera",   "Prudenzia",  "Quartilla",  "Quieta",    "Quiteria",
    "Rachele",    "Raffaella",   "Rainelda",    "Rebecca",    "Regina",     "Renata",     "Riccarda",  "Rina",
    "Rita",       "Roberta",     "Romana",      "Romilda",    "Romina",     "Romola",     "Rosa",      "Rosalia",
    "Rosalinda",  "Rosamunda",   "Rosanna",     "Rosita",     "Rosmunda",   "Rossana",    "Rossella",  "Rufina",
    "Saba",       "Sabina",      "Sabrina",     "Samanta",    "Samona",     "Sandra",     "Santina",   "Sara",
    "Savina",     "Scolastica",  "Sebastiana",  "Seconda",    "Secondina",  "Sefora",     "Selene",    "Selvaggia",
    "Semiramide", "Serafina",    "Serena",      "Severa",     "Sibilla",    "Sidonia",    "Silvana",   "Silvia",
    "Simona",     "Simonetta",   "Siria",       "Smeralda",   "Soave",      "Sofia",      "Sofronia",  "Solange",
    "Sonia",      "Speranza",    "Stefania",    "Stella",     "Susanna",    "Sveva",      "Tabita",    "Tamara",
    "Tarquinia",  "Tarsilla",    "Taziana",     "Tea",        "Tecla",      "Telica",     "Teodata",   "Teodolinda",
    "Teodora",    "Teresa",      "Teudosia",    "Tina",       "Tiziana",    "Tosca",      "Trasea",    "Tullia",
    "Ugolina",    "Ulfa",        "Uliva",       "Unna",       "Vala",       "Valentina",  "Valeria",   "Valeriana",
    "Vanda",      "Vanessa",     "Vanna",       "Venera",     "Veneranda",  "Venere",     "Venusta",   "Vera",
    "Verdiana",   "Verena",      "Veriana",     "Veridiana",  "Veronica",   "Viliana",    "Vilma",     "Vincenza",
    "Viola",      "Violante",    "Virginia",    "Vissia",     "Vittoria",   "Viviana",    "Wanda",     "Zabina",
    "Zaira",      "Zama",        "Zanita",      "Zarina",     "Zelinda",    "Zenobia",    "Zita",      "Zoe",
    "Zosima",
});

const auto italianLastNames = std::to_array<std::string_view>({
    "Abate",       "Abbate",      "Abbondanza",  "Abbrescia",    "Accardi",     "Accardo",    "Accurso",
    "Aceto",       "Acquadro",    "Acquaviva",   "Acquistapace", "Adami",       "Adamo",      "Addari",
    "Addis",       "Adragna",     "Affinito",    "Agnello",      "Agostinelli", "Agostini",   "Agresta",
    "Aiello",      "Alaimo",      "Albanese",    "Albano",       "Alberti",     "Alcamo",     "Alessandrini",
    "Alessi",      "Alfano",      "Alfieri",     "Alivernini",   "Allegretti",  "Aloi",       "Altieri",
    "Amabile",     "Amadori",     "Amato",       "Ambrosini",    "Ambrosino",   "Ambrosio",   "Amendola",
    "Amerio",      "Amici",       "Amico",       "Ammoscato",    "Amoroso",     "Amoruso",    "Ancona",
    "Andreoli",    "Andrisani",   "Anelli",      "Angeletti",    "Angeli",      "Angelini",   "Angeloni",
    "Angelucci",   "Annunziata",  "Anselmi",     "Anselmo",      "Antenucci",   "Antezza",    "Antonacci",
    "Antonelli",   "Antonini",    "Antonucci",   "Anzalone",     "Apicella",    "Aprile",     "Aquino",
    "Aramini",     "Arcangeli",   "Arena",       "Argenio",      "Argento",     "Argiolas",   "Artioli",
    "Asaro",       "Ascione",     "Astolfi",     "Atzeni",       "Atzori",      "Avallone",   "Avola",
    "Azara",       "Babini",      "Bacci",       "Baggio",       "Baiocco",     "Balboni",    "Baldacci",
    "Baldassarre", "Baldi",       "Baldini",     "Baldo",        "Balducci",    "Balestra",   "Balistreri",
    "Ballarin",    "Balsamo",     "Balzano",     "Bandini",      "Bani",        "Baraldi",    "Barbagallo",
    "Barbarossa",  "Barbarulo",   "Barbato",     "Barbera",      "Barberi",     "Barberis",   "Barbero",
    "Barbieri",    "Barbiero",    "Barile",      "Barillà",      "Barletta",    "Barone",     "Baroni",
    "Barra",       "Barreca",     "Barresi",     "Barretta",     "Barsotti",    "Bartoli",    "Bartolini",
    "Bartolomei",  "Bartolucci",  "Basile",      "Bassani",      "Bassi",       "Basso",      "Battaglia",
    "Battista",    "Battisti",    "Battistini",  "Bedini",       "Belli",       "Bellini",    "Bellomo",
    "Bellotti",    "Bellucci",    "Belmonte",    "Belotti",      "Beltrame",    "Beltrami",   "Belvisi",
    "Benatti",     "Benedetti",   "Benedetto",   "Benetti",      "Benigni",     "Benini",     "Bennardo",
    "Bentivoglio", "Benvenuti",   "Benvenuto",   "Berardi",      "Beretta",     "Bergamasco", "Bergamini",
    "Bernabei",    "Bernardi",    "Bernardini",  "Bernasconi",   "Bertaccini",  "Bertani",    "Bertelli",
    "Berti",       "Bertini",     "Berto",       "Bertolini",    "Bertolussi",  "Beso",       "Betti",
    "Bevilacqua",  "Biagetti",    "Biagi",       "Biagini",      "Bianc",       "Biancheri",  "Bianchetti",
    "Bianchi",     "Bianchini",   "Bianco",      "Biasci",       "Biccari",     "Biggi",      "Bindi",
    "Bini",        "Bionaz",      "Biondi",      "Bisconti",     "Bisio",       "Bizzarri",   "Blanc",
    "Blasi",       "Bocchi",      "Boccia",      "Bodini",       "Boffa",       "Boi",        "Bologna",
    "Bonanni",     "Bonanno",     "Bonato",      "Bonazzi",      "Bonelli",     "Bonetti",    "Bongiorno",
    "Bongiovanni", "Boni",        "Bonifazi",    "Bonini",       "Bono",        "Bonomi",     "Bonomo",
    "Bordoni",     "Borelli",     "Borghi",      "Borgia",       "Borrelli",    "Bortolin",   "Bortoluzzi",
    "Bortot",      "Boscaino",    "Boschetti",   "Boschi",       "Bosco",       "Boscolo",    "Bosi",
    "Botta",       "Bove",        "Bovolenta",   "Bozzi",        "Bozzo",       "Braia",      "Brambilla",
    "Branca",      "Brandi",      "Bresciani",   "Bressan",      "Briano",      "Brigandì",   "Brignone",
    "Brizzi",      "Brogi",       "Brumat",      "Brunelli",     "Brunetti",    "Bruni",      "Bruno",
    "Bruschi",     "Bruzzone",    "Buccheri",    "Bucci",        "Bulgarelli",  "Buongiorno", "Buonomo",
    "Burgio",      "Butera",      "Buzzi",       "Cabras",       "Caccamo",     "Caccavo",    "Cacciapuoti",
    "Cacciatore",  "Cadoni",      "Caforio",     "Caggiano",     "Caiazza",     "Caiazzo",    "Calabrese",
    "Calabria",    "Calabrò",     "Calafiore",   "Calandro",     "Calì",        "Califano",   "Callegari",
    "Calligaris",  "Calò",        "Camilli",     "Cammarata",    "Cammisa",     "Campagna",   "Campana",
    "Campanella",  "Campanile",   "Campanini",   "Campisi",      "Campo",       "Campoli",    "Campus",
    "Canale",      "Cancelliere", "Cancellieri", "Candela",      "Candido",     "Canepa",     "Cangiano",
    "Canino",      "Cannas",      "Cannata",     "Cannella",     "Cannone",     "Canova",     "Cantagallo",
    "Cantoni",     "Canu",        "Capannolo",   "Capasso",      "Capecchi",    "Capelli",    "Capitani",
    "Capizzi",     "Capoccia",    "Capocotta",   "Capogna",      "Capone",      "Caporaso",   "Cappai",
    "Cappelletti", "Cappelli",    "Cappello",    "Cappiello",    "Capponi",     "Capraro",    "Capriotti",
    "Capuano",     "Caputo",      "Cara",        "Caracciolo",   "Caradonna",   "Carbon",     "Carbone",
    "Carboni",     "Cardella",    "Cardia",      "Cardinale",    "Cardinali",   "Cardini",    "Cardone",
    "Carè",        "Careddu",     "Carella",     "Caretti",      "Caricari",    "Carini",     "Carletti",
    "Carli",       "Carlini",     "Carlino",     "Carloni",      "Carlotti",    "Carlucci",   "Carminati",
    "Carnevale",   "Carnovale",   "Carollo",     "Carosi",       "Carrara",     "Carraro",    "Carrieri",
    "Carriero",    "Carrozza",    "Carrozzo",    "Carrus",       "Carta",       "Caruso",     "Carvelli",
    "Casadei",     "Casadio",     "Casagrande",  "Casale",       "Casali",      "Casano",     "Casavecchia",
    "Cascio",      "Cascone",     "Casella",     "Caselli",      "Caserta",     "Casini",     "Casolaro",
    "Cassano",     "Castagna",    "Castaldi",    "Castaldo",     "Castellana",  "Castellani", "Castellano",
    "Castelli",    "Castiello",   "Castiglioni", "Casu",         "Casula",      "Catalano",   "Cataldi",
    "Cataldo",     "Catania",     "Catanzaro",   "Catarsi",      "Catellani",   "Caterino",   "Cattaneo",
    "Cattani",     "Cavaliere",   "Cavalieri",   "Cavallari",    "Cavallaro",   "Cavalli",    "Cavallini",
    "Cavallo",     "Cavriani",    "Ceccarelli",  "Ceccarini",    "Cecchetti",   "Cecchi",     "Cecchini",
    "Cecere",      "Celano",      "Celentano",   "Celeste",      "Celi",        "Celli",      "Cellini",
    "Cenni",       "Centofanti",  "Cerise",      "Cerrani",      "Cerrato",     "Cerri",      "Cerullo",
    "Cerutti",     "Cesaretti",   "Cherchi",     "Cherubini",    "Chessa",      "Chessari",   "Chiacchio",
    "Chianese",    "Chiaravalle", "Chiavacci",   "Chilà",        "Chimenti",    "Chiodi",     "Chirico",
    "Ciaccio",     "Ciacio",      "Ciampa",      "Cianci",       "Ciani",       "Ciavarella", "Cicala",
    "Cicatiello",  "Ciccarelli",  "Cicchetti",   "Ciccone",      "Cifarelli",   "Cilli",      "Cimmino",
    "Cinelli",     "Cingolani",   "Cini",        "Cioffi",       "Cioni",       "Ciotola",    "Ciotti",
    "Cipolla",     "Cipollaro",   "Cipolletta",  "Cipriani",     "Cipriano",    "Cirelli",    "Cirillo",
    "Citro",       "Cittadino",   "Ciuffetelli", "Ciuffreda",    "Ciulla",      "Clemente",   "Coccia",
    "Cocco",       "Cola",        "Colangelo",   "Colantuono",   "Colella",     "Coletta",    "Colombo",
    "Colonna",     "Colucci",     "Comito",      "Como",         "Concas",      "Conforti",   "Congiu",
    "Coniglio",    "Consiglio",   "Cont",        "Conte",        "Conti",       "Contini",    "Contu",
    "Coppola",     "Corazza",     "Cordioli",    "Coretti",      "Corona",      "Corradi",    "Corradini",
    "Corrado",     "Corrao",      "Correale",    "Corridori",    "Corsi",       "Corsini",    "Corso",
    "Cortese",     "Cortesi",     "Corti",       "Cortinovis",   "Cosentino",   "Coslovich",  "Cossu",
    "Costa",       "Costantin",   "Costantini",  "Costantino",   "Costanza",    "Costanzo",   "Cottone",
    "Coviello",    "Cozzani",     "Cozzi",       "Cozzolino",    "Crea",        "Cremona",    "Cremonesi",
    "Crepaldi",    "Crescenzi",   "Crespi",      "Crevatin",     "Crimi",       "Crippa",     "Crisafulli",
    "Criscenti",   "Crisci",      "Criscione",   "Crispino",     "Cristiano",   "Cristofaro", "Critelli",
    "Croce",       "Cruciani",    "Cruciata",    "Cucchiara",    "Cucciniello", "Cuomo",      "Curci",
    "Curcio",      "Curreli",     "Fucci",       "Fumagalli",    "Fumarola",    "Fuoco",      "Furlan",
    "Furno",       "Fusaro",      "Fusco",       "Gabriele",     "Gabrielli",   "Gaddini",    "Gagliano",
    "Gagliardi",   "Galante",     "Galasso",     "Galati",       "Galeazzi",    "Galli",      "Gallo",
    "Galluzzo",    "Gamba",       "Gamberini",   "Gambino",      "Gamper",      "Gandolfo",   "Gangemi",
    "Garau",       "Garavaglia",  "Gargiulo",    "Garifo",       "Garofalo",    "Gaspari",    "Gasparini",
    "Gasser",      "Gatta",       "Gatti",       "Gatto",        "Gaudiano",
});

const auto italianPrefixes = std::to_array<std::string_view>({"Sig.", "Dott.", "Dr.", "Ing."});

const NameFormats italianNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames italianPeopleNames{{italianMaleFirstNames, italianLastNames, italianPrefixes, {}},
                                     {italianFemaleFirstNames, italianLastNames, italianPrefixes, {}},
                                     italianNameFormats};

// Japan

const auto japaneseMaleFirstNames = std::to_array<std::string_view>({
    "正一", "正二", "正三", "清",   "辰雄", "三郎", "昭二", "昭三", "茂",   "勇",   "明",   "弘",   "勝",   "稔",
    "博",   "隆",   "誠",   "浩",   "健一", "大輔", "達也", "翔太", "拓也", "健太", "大輝", "翔",   "駿",   "蓮",
    "大翔", "陸",   "悠真", "蒼",   "昭",   "実",   "進",   "和夫", "修",   "浩一", "哲也", "剛",   "大介", "海斗",
    "拓海", "颯太", "悠斗", "陽翔", "湊",   "新",   "樹",   "正雄", "勝利", "豊",   "和彦", "学",   "直樹", "健太郎",
    "大和", "陽向", "瑛太", "陽太", "正",   "一郎", "勲",   "徹",   "浩二", "亮",   "大樹", "翼",   "義雄", "昭一",
    "功",   "武",   "淳",   "和也", "大地", "匠",   "悠人", "朝陽", "律",   "武雄", "英樹", "秀樹", "祐介", "雄太",
    "翔平", "大貴", "太陽", "颯",   "陽斗", "颯真", "蒼空", "優斗", "正治", "昇",   "竜也", "聡",   "貴大", "直人",
    "輝",   "優太", "悠太", "陸斗", "太一", "悠",   "浩之", "健",   "博之", "亮太", "一輝", "空",   "陽大", "正夫",
    "秀雄", "幸雄", "勉",   "崇",   "洋平", "拓哉", "涼太", "大雅", "海翔", "龍生", "結翔", "悠翔", "葵",   "正男",
    "辰男", "一男", "優",   "雄大", "康平", "智也", "竜",   "悠希", "歩夢",
});

const auto japaneseFemaleFirstNames = std::to_array<std::string_view>({
    "千代子", "静子", "文子",   "キヨ",   "千代",   "ハル",   "キミ",   "久子", "ヨシ",   "清子", "正子",   "貞子",
    "八重子", "きよ", "フミ",   "ハナ",   "和子",   "愛子",   "きみ",   "トミ", "はる",   "マサ", "美代子", "光子",
    "幸子",   "照子", "芳子",   "節子",   "昭子",   "信子",   "敏子",   "典子", "君子",   "弘子", "洋子",   "美智子",
    "京子",   "悦子", "栄子",   "孝子",   "良子",   "恵子",   "勝子",   "紀子", "恵美子", "啓子", "久美子", "由美子",
    "裕子",   "順子", "智子",   "明美",   "愛",     "恵",     "麻衣",   "麻美", "香織",   "美穂", "彩",     "沙織",
    "愛美",   "絵美", "舞",     "恵美",   "陽子",   "成美",   "あゆみ", "美香", "理恵",   "友美", "智美",   "明日香",
    "由佳",   "瞳",   "麻衣子", "彩香",   "真由美", "純子",   "直美",   "優子", "裕美",   "由美", "直子",   "めぐみ",
    "美紀",   "浩子", "ゆかり", "さくら", "美咲",   "陽菜",   "葵",     "七海", "美羽",   "結衣", "美月",   "凛",
    "美優",   "未来", "優奈",   "萌",     "優花",   "美桜",   "莉子",   "菜月", "結愛",   "結菜", "花音",   "心優",
    "真央",   "玲奈", "ひなた", "彩乃",   "彩花",   "優衣",   "琴音",   "百花", "遥",     "里奈", "茜",     "千尋",
    "楓",     "亜美", "佳奈",   "真由",   "桃子",   "菜々子", "千夏",   "菜摘", "奈々",   "早紀", "杏",     "陽葵",
    "芽依",   "結月", "咲良",   "凜",     "紬",     "心愛",   "花",     "心春", "愛莉",   "愛菜", "桜",     "心結",
    "美結",
});

const auto japaneseLastNames = std::to_array<std::string_view>({
    "佐藤", "鈴木", "高橋",   "田中", "渡辺", "伊藤", "山本", "中村", "小林", "加藤",
    "吉田", "山田", "佐々木", "山口", "斎藤", "松本", "井上", "木村", "林",   "清水",
});

const NameFormats japaneseNameFormats{{"{lastName} {firstName}", 1}};
const PeopleNames japanesePeopleNames{{japaneseMaleFirstNames, japaneseLastNames, {}, {}},
                                      {japaneseFemaleFirstNames, japaneseLastNames, {}, {}},
                                      japaneseNameFormats};

// Kazakhstan

const auto kazakhMaleFirstNames = std::to_array<std::string_view>({
    "Абай",        "Абдолла",    "Абдулла",    "Абдуллаһ",  "Абдінасір",  "Абылай",    "Азамат",      "Азиз",
    "Айбек",       "Айберген",   "Айболат",    "Айдос",     "Айнабек",    "Айсұлтан",  "Айтқали",     "Ақжан",
    "Ақжол",       "Аққу",       "Алдаберген", "Алдияр",    "Алпан",      "Алпысбай",  "Алтай",       "Алтынбай",
    "Алтынбек",    "Амангелді",  "Амандос",    "Аманжол",   "Амантай",    "Аңсар",     "Арыстан",     "Арыстанбек",
    "Асанбай",     "Асанғали",   "Асқар",      "Асылбек",   "Ахмет",      "Аюхан",     "Әбдуәли",     "Әбдіғазиз",
    "Әбдіғани",    "Әбдіғаппар", "Әбдіжаппар", "Әбдіжәміл", "Әбдіқадыр",  "Әбділда",   "Әбділхамит",  "Әбдімәлік",
    "Әбдіразақ",   "Әбдірахман", "Әбдірәсіл",  "Әбдішүкір", "Әбиірбек",   "Әбубәкір",  "Әбуғали",     "Әбілғазы",
    "Әбілқайыр",   "Әділ",       "Әділбек",    "Әділхан",   "Әкежан",     "Әлжан",     "Әлжаппар",    "Әли",
    "Әлімжан",     "Әлімқұл",    "Әліхан",     "Әмірбек",   "Әнес",       "Әсет",      "Әубәкір",     "Әуелхан",
    "Бағдат",      "Базарбай",   "Байысхан",   "Бақтияр",   "Бақытжан",   "Балтабек",  "Бауыржан",    "Бәкір",
    "Бейбіт",      "Бейсен",     "Бекболат",   "Бекзат",    "Бердібек",   "Бердіқұл",  "Болат",       "Боранбай",
    "Бөріхан",     "Бүркітбай",  "Біржан",     "Ғабдулла",  "Ғабдұлла",   "Ғабит",     "Ғазиз",       "Ғазизхан",
    "Ғалымжан",    "Ғани",       "Ғаппар",     "Ғұсман",    "Дамир",      "Даниял",    "Дастан",      "Дәлелхан",
    "Дәнеш",       "Дәулет",     "Дәуіт",      "Дос",       "Досай",      "Дүйсен",    "Дүйсенғазы",  "Дінмұхамед",
    "Дінмұхаммед", "Еділ",       "Елнұр",      "Ербол",     "Ерболат",    "Ербосын",   "Ережеп",      "Ерекен",
    "Еркебұлан",   "Еркін",      "Ерлан",      "Ермек",     "Ермұхамет",  "Ерік",      "Есбосын",     "Есенгелді",
    "Ескендір",    "Жабайхан",   "Жақия",      "Жақсыбек",  "Жақып",      "Жақыпбек",  "Жамалбек",    "Жанбек",
    "Жанболат",    "Жандос",     "Жанқуат",    "Жанпейіс",  "Жаңабай",    "Жаңабек",   "Жарасбай",    "Жарасқан",
    "Жармұқамет",  "Жармұхамед", "Жармұхамет", "Жауынбай",  "Жеменей",    "Жеңіс",     "Жолбарыс",    "Жолжақсы",
    "Жомарт",      "Жомартбай",  "Жұмабай",    "Жұмағали",  "Жұмақан",    "Жұмамұрат", "Жұманазар",   "Жұмекен",
    "Жүніс",       "Жүсіп",      "Жүсіпбек",   "Жылқыбек",  "Зәкария",    "Зәкәрия",   "Зұлпықар",    "Ибрахим",
    "Ибраһим",     "Иса",        "Ислам",      "Исламбек",  "Исхақ",      "Кәрім",     "Кенжеғұл",    "Кеңгірбай",
    "Керімбай",    "Күнесхан",   "Қабдолла",   "Қабдула",   "Қабдыкәрім", "Қабыл",     "Қажымқан",    "Қайролла",
    "Қалихан",     "Қалқаман",   "Қанат",      "Қаныш",     "Қаржаубай",  "Қасым",     "Қасымбек",    "Қатшыбек",
    "Қияқбай",     "Қойшыбек",   "Қуан",       "Қуаныш",    "Қуат",       "Құбыш",     "Құдайберген", "Құдыс",
});

const auto kazakhFemaleFirstNames = std::to_array<std::string_view>({
    "Ағила",   "Ажар",    "Ажаргүл",  "Аида",      "Айгүл",     "Айдай",     "Айжан",    "Айжүрек",  "Айнұр",
    "Айша",    "Ақбота",  "Ақгүл",    "Ақерке",    "Ақжан",     "Ақкүміс",   "Аққу",     "Алмагүл",  "Алтынай",
    "Анаргүл", "Ардақ",   "Аруай",    "Аружан",    "Аяжан",     "Аяужан",    "Әдила",    "Әдина",    "Әлима",
    "Әлия",    "Әсел",    "Әсемай",   "Әсемгүл",   "Базаргүл",  "Бақаргүл",  "Батима",   "Бибігүл",  "Ботакөз",
    "Гауһар",  "Гүлажар", "Гүлай",    "Гүлбақыт",  "Гүлбаршын", "Гүлболсын", "Гүлдана",  "Гүлжан",   "Гүлжауһар",
    "Гүлмира", "Гүлнар",  "Гүлнұр",   "Гүлпариза", "Гүлфайрус", "Гүлфариза", "Гүлшара",  "Гүлшат",   "Ғайни",
    "Ғалия",   "Дариға",  "Дәлила",   "Дина",      "Динара",    "Ділдә",     "Елерке",   "Жазира",   "Жамал",
    "Жанар",   "Жаннұр",  "Жансая",   "Жәмила",    "Жолжақсы",  "Жұлдыз",    "Жібек",    "Зейнекүл", "Зияда",
    "Зылиха",  "Күнсұлу", "Қадиша",   "Қанағат",   "Қарақат",   "Қасиет",    "Ләззат",   "Ләтипа",   "Лейла",
    "Мақпал",  "Марияш",  "Мәриям",   "Нәзерке",   "Нұрбақыт",  "Нұрзада",   "Нұрсауле", "Нұрсұлу",  "Перизат",
    "Райана",  "Райхан",  "Раяна",    "Рәбиға",    "Риза",      "Роза",      "Сабина",   "Сайрагүл", "Салтанат",
    "Самал",   "Сара",    "Сәнімгүл", "Сәуле",     "Таңшолпан", "Ұлбала",    "Ұлбике",   "Ұлболсын", "Ұлдана",
    "Ұлжан",   "Ұлмекен", "Фатима",   "Хадиша",    "Халида",    "Шарапат",   "Шахзада",  "Шаһизада", "Шәмшия",
    "Шәрбану", "Шолпан",  "Ырысты",   "Іңкәр",
});

const auto kazakhMaleLastNames = std::to_array<std::string_view>({
    "Асылмұратов", "Әбдірахманов", "Әлімқұлов",  "Етекбаев", "Жүнісов",    "Жүсіп",    "Иманқұлов",
    "Исламқұлов",  "Қуатбаев",     "Құлов",      "Мәженов",  "Назарбаев",  "Рүстемов", "Сәтбаев",
    "Сұлтанбеков", "Сүгірбаев",    "Сүлейменов", "Тоқаев",   "Торайғыров", "Тұяқбаев", "Ысмайылов",
});

const auto kazakhFemaleLastNames = std::to_array<std::string_view>({
    "Әбдірахманова",
    "Әлімқұлова",
    "Етекбаева",
    "Жүнісова",
    "Иманқұлова",
    "Исламқұлова",
    "Қуатбаева",
    "Құлова",
    "Мәженова",
    "Назарбаева",
    "Рүстемова",
    "Сұлтанбекова",
    "Сүгірбаева",
    "Сүлейменова",
    "Тоқаева",
    "Тұяқбаева",
    "Ысмайылова",
    "Асылмұратова",
});

const NameFormats kazakh_personNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames kazakhPeopleNames{{kazakhMaleFirstNames, kazakhMaleLastNames, {}, {}},
                                    {kazakhFemaleFirstNames, kazakhFemaleLastNames, {}, {}},
                                    kazakh_personNameFormats};

// Korea

const auto koreanMaleFirstNames = std::to_array<std::string_view>({
    "서연", "민서", "서현", "지우", "서윤", "지민", "수빈", "하은", "예은", "윤서", "민준", "지후", "지훈",
    "준서", "현우", "예준", "건우", "현준", "민재", "우진", "은주", "서준", "도윤", "시우", "주원", "하준",
    "지호", "준우", "도현", "선우", "서진", "연우", "유준", "정우", "승우", "승현", "시윤", "준혁", "은우",
    "지환", "승민", "유찬", "윤우", "민성", "준영", "시후", "진우", "지원", "수현", "재윤", "시현", "동현",
    "수호", "태윤", "민규", "재원", "한결", "민우", "재민", "은찬", "윤호", "시원", "이준", "민찬", "지안",
    "시온", "성민", "준호", "승준", "성현", "이안", "현서", "재현", "하율", "지한", "우빈", "태민", "지성",
    "예성", "민호", "태현", "지율", "민혁", "서우", "성준", "은호", "규민", "정민", "윤성", "윤재", "하람",
    "하진", "민석", "준수", "은성", "태양", "예찬", "준희", "도훈", "하민",
});

const auto koreanLastNames = std::to_array<std::string_view>({
    "김", "이", "박", "최", "정", "강", "조", "윤", "장", "임", "오", "한", "신", "서", "권", "황", "안", "송", "류",
    "홍", "유", "전", "양", "고", "문", "손", "배", "백", "허", "노", "심", "하", "주", "구", "곽", "성", "차", "우",
    "진", "민", "나", "지", "엄", "변", "채", "원", "방", "천", "공", "현", "함", "여", "염", "석", "추", "도", "소",
    "설", "선", "마", "길", "연", "위", "표", "명", "기", "반", "라", "왕", "금", "옥", "육", "인", "맹", "제", "모",
    "남", "탁", "국", "어", "경", "은", "편", "용", "예", "봉", "사", "부", "가", "복", "태", "목", "형", "피", "두",
    "감", "호", "음", "빈", "동", "온", "시", "범", "좌", "팽", "승", "간", "상", "갈", "단", "견", "당",
});

const NameFormats koreanNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames koreanPeopleNames{{koreanMaleFirstNames, koreanLastNames, {}, {}},
                                    {koreanMaleFirstNames, koreanLastNames, {}, {}},
                                    koreanNameFormats};

// Latvia

const auto latvianMaleFirstNames = std::to_array<std::string_view>({
    "Ainārs",    "Alfrēds",    "Aloīzs",    "Aļģirds",  "Aļģis",   "Andris",   "Andžejs",   "Andžs",      "Anrijs",
    "Anšlavs",   "Arkādijs",   "Armīns",    "Artūrs",   "Arvīds",  "Ādams",    "Ādolfs",    "Ārijs",      "Āris",
    "Boļeslavs", "Broņislavs", "Dāgs",      "Dāvids",   "Dāvis",   "Edgars",   "Edvīns",    "Eduards",    "Edžus",
    "Egīls",     "Einārs",     "Eižens",    "Elmārs",   "Emīls",   "Ermīns",   "Ervīns",    "Ēriks",      "Ēvalds",
    "Gunārs",    "Ģederts",    "Ģirts",     "Ilgmārs",  "Ilmārs",  "Inārs",    "Indriķis",  "Ingmārs",    "Jānis",
    "Jāzeps",    "Jevgeņijs",  "Jēkabs",    "Jurģis",   "Jūlijs",  "Kārlis",   "Klāvs",     "Konrāds",    "Konstantīns",
    "Kristiāns", "Krišjānis",  "Krišs",     "Leonīds",  "Marģers", "Matīss",   "Mārcis",    "Māris",      "Mārtiņš",
    "Miķelis",   "Monvīds",    "Niklāvs",   "Ojārs",    "Oļegs",   "Oļģerts",  "Pāvils",    "Pēteris",    "Regnārs",
    "Renārs",    "Ričards",    "Romāns",    "Rūdis",    "Rūdolfs", "Rūsiņš",   "Tālis",     "Tālivaldis", "Tālrīts",
    "Uģis",      "Valdemārs",  "Valentīns", "Valērijs", "Vilmārs", "Vitālijs", "Voldemārs", "Zigfrīds",   "Zigmārs",
    "Žanis",
});

const auto latvianFemaleFirstNames = std::to_array<std::string_view>({
    "Aīda",      "Akvelīna",   "Albertīne", "Alvīne",    "Amālija",    "Astrīda",   "Aurēlija",   "Ārija",   "Baņuta",
    "Bārbala",   "Beāte",      "Dagmāra",   "Dainuvīte", "Dārta",      "Diāna",     "Dzirkstīte", "Dzīle",   "Edīte",
    "Elfrīda",   "Elīna",      "Elīza",     "Elmīra",    "Elvīra",     "Emīlija",   "Ernestīne",  "Evelīna", "Ērika",
    "Frīda",     "Gabriela",   "Gudrīte",   "Ģertrūde",  "Helēna",     "Hermīne",   "Ināra",      "Ingrīda", "Ingūna",
    "Irēna",     "Īrisa",      "Janīna",    "Jasmīna",   "Jautrīte",   "Judīte",    "Justīne",    "Jūlija",  "Jūsma",
    "Karīna",    "Katrīna",    "Klāra",     "Kristiāna", "Kristīne",   "Lavīze",    "Lāsma",      "Leonīda", "Leontīne",
    "Liāna",     "Liliāna",    "Lība",      "Līga",      "Līksma",     "Līna",      "Līva",       "Līvija",  "Līze",
    "Luīze",     "Lūcija",     "Malvīne",   "Māra",      "Mārīte",     "Mētra",     "Modrīte",    "Mudīte",  "Olīvija",
    "Otīlija",   "Patrīcija",  "Paulīne",   "Pārsla",    "Regīna",     "Renāte",    "Rudīte",     "Rūta",    "Sabīne",
    "Sarmīte",   "Saulcerīte", "Skaidrīte", "Spīdola",   "Tamāra",     "Terēze",    "Tīna",       "Trīne",   "Undīne",
    "Valentīna", "Valērija",   "Vēsma",     "Vijolīte",  "Vilhelmīne", "Vizbulīte", "Zeltīte",    "Ziedīte", "Zigfrīda",
    "Zigrīda",   "Zīle",       "Žanete",    "Žanna",     "Ženija",     "Žubīte",
});

const auto latvianMaleLastNames = std::to_array<std::string_view>({
    "Aigars",     "Alksnis",     "Andersons",   "Aspers",      "Auziņš",     "Ābele",       "Āboliņš",    "Āboltiņš",
    "Balodis",    "Beļavskis",   "Bergmanis",   "Bērziņš",     "Bērzīts",    "Birziņš",     "Birznieks",  "Bīriņš",
    "Blaus",      "Blekte",      "Breņķis",     "Briedis",     "Brunkevičs", "Builis",      "Bušs",       "Caune",
    "Cālītis",    "Cīrulis",     "Čudars",      "Danielsons",  "Dīriķis",    "Dombrovskis", "Dreimanis",  "Dzenis",
    "Eglītis",    "Endziņš",     "Fogelis",     "Freimanis",   "Gailis",     "Gailītis",    "Gibala",     "Graudiņš",
    "Gribusts",   "Grīnbergs",   "Grūbe",       "Jansons",     "Jēkabsons",  "Jukums",      "Kalējs",     "Kalniņš",
    "Karlsons",   "Kauliņš",     "Kārkliņš",    "Kļaviņš",     "Kohs",       "Koks",        "Krancis",    "Krastiņš",
    "Kraulis",    "Krauze",      "Krēsliņš",    "Krieviņš",    "Krievs",     "Kronbergs",   "Krūmiņš",    "Kulmanis",
    "Laiviņš",    "Landmanis",   "Lapiņš",      "Lapsiņš",     "Lasmanis",   "Laubergs",    "Lācis",      "Legzdiņš",
    "Lielmanis",  "Liepiņš",     "Līcis",       "Līdaks",      "Ločs",       "Lodiņš",      "Lūsis",      "Matisons",
    "Mednis",     "Meiers",      "Mekšs",       "Melderis",    "Melngailis", "Mihailovs",   "Miķelsons",  "Niedra",
    "Osis",       "Ostrovskis",  "Ozers",       "Ozoliņš",     "Ozols",      "Palters",     "Pavlovskis", "Pencis",
    "Petrovskis", "Pērkons",     "Pētersons",   "Podziņš",     "Polis",      "Pretkalniņš", "Priedītis",  "Puķītis",
    "Putniņš",    "Riekstiņš",   "Romanovskis", "Rozenbahs",   "Rozītis",    "Rubenis",     "Rudzītis",   "Runcis",
    "Salmiņš",    "Seržants",    "Sietnieks",   "Siliņš",      "Sīlis",      "Skudra",      "Smuģis",     "Sproģis",
    "Sprūde",     "Strautiņš",   "Strautmanis", "Strazdiņš",   "Širovs",     "Šīrants",     "Šmits",      "Šteins",
    "Tālbergs",   "Teicāns",     "Tiltiņš",     "Tīrelis",     "Tučs",       "Ūdris",       "Valters",    "Vasils",
    "Vecumnieks", "Veinbergs",   "Veiss",       "Vilks",       "Vilsons",    "Viļums",      "Vinters",    "Vītiņš",
    "Vītoliņš",   "Vītols",      "Vītums",      "Vollis",      "Zariņš",     "Zālītis",     "Zeidmanis",  "Zeltiņš",
    "Ziemelis",   "Zībers",      "Zunda",       "Zvaigzne",    "Zvejnieks",  "Zvinelis",    "Mucenieks",  "Dzirkalis",
    "Arājs",      "Sējējs",      "Podnieks",    "Ratnieks",    "Deģis",      "Bondars",     "Sondors",    "Namiķis",
    "Jumiķis",    "Muižnieks",   "Bajārs",      "Ķēniņš",      "Ķeizars",    "Barons",      "Vaivads",    "Zviedrs",
    "Vācietis",   "Lejiņš",      "Vanags",      "Celms",       "Liepa",      "Kalns",       "Celmiņš",    "Lejnieks",
    "Bisenieks",  "Kurzemnieks", "Saulietis",   "Pakalnietis", "Pridāns",    "Grodums",     "Kalnājs",    "Porgants",
    "Sukuts",     "Meškūns",     "Rožkalns",    "Mazjānis",    "Latkovskis", "Jankovskis",  "Šileiko",    "Budreiko",
    "Ivanovs",    "Morozovs",    "Logins",      "Gasjūns",     "Pauļuks",    "Sosārs",
});

const auto latvianFemaleLastNames = std::to_array<std::string_view>({
    "Aigare",     "Alksne",      "Andersone",  "Aspere",     "Auziņa",     "Ābele",       "Āboliņa",   "Āboltiņa",
    "Balode",     "Beļavske",    "Bergmane",   "Bērziņa",    "Bērzīte",    "Birziņa",     "Birzniece", "Bīriņa",
    "Blaua",      "Blekte",      "Breņķe",     "Briede",     "Brunkeviča", "Buile",       "Buša",      "Cauna",
    "Cālīte",     "Cīrule",      "Čudara",     "Danielsone", "Dīriķe",     "Dombrovska",  "Dreimane",  "Dzene",
    "Eglīte",     "Endziņa",     "Fogele",     "Freimane",   "Gaile",      "Gailīte",     "Gibala",    "Graudiņa",
    "Gribuste",   "Grīnberga",   "Grūba",      "Jansone",    "Jēkabsone",  "Jukuma",      "Kalēja",    "Kalniņa",
    "Karlsone",   "Kauliņa",     "Kārkliņa",   "Kļaviņa",    "Koha",       "Koka",        "Krance",    "Krastiņa",
    "Kraule",     "Krauze",      "Krēsliņa",   "Krieviņa",   "Krieva",     "Kronberga",   "Krūmiņa",   "Kulmane",
    "Laiviņa",    "Landmane",    "Lapiņa",     "Lapsiņa",    "Lasmane",    "Lauberga",    "Lāce",      "Legzdiņa",
    "Lielmane",   "Liepiņa",     "Līce",       "Līdaka",     "Loča",       "Lodiņa",      "Lūse",      "Matisone",
    "Medne",      "Meiere",      "Mekša",      "Meldere",    "Melngaile",  "Mihailova",   "Miķelsone", "Niedra",
    "Ose",        "Ostrovska",   "Ozere",      "Ozoliņa",    "Ozola",      "Paltere",     "Pavlovska", "Pence",
    "Petrovska",  "Pērkona",     "Pētersone",  "Podziņa",    "Pole",       "Pretkalniņa", "Priedīte",  "Puķīte",
    "Putniņa",    "Riekstiņa",   "Romanovska", "Rozenbaha",  "Rozīte",     "Rubene",      "Rudzīte",   "Runce",
    "Salmiņa",    "Seržante",    "Sietniece",  "Siliņa",     "Sīle",       "Skudra",      "Smuģe",     "Sproģe",
    "Sprūde",     "Strautiņa",   "Strautmane", "Strazdiņa",  "Širova",     "Šīrante",     "Šmite",     "Šteina",
    "Tālberga",   "Teicāne",     "Tiltiņa",    "Tīrele",     "Tuča",       "Ūdre",        "Valtere",   "Vasile",
    "Vecumniece", "Veinberga",   "Veisa",      "Vilka",      "Vilsone",    "Viļuma",      "Vintere",   "Vītiņa",
    "Vītoliņa",   "Vītola",      "Vītuma",     "Volle",      "Zariņa",     "Zālīte",      "Zeidmane",  "Zeltiņa",
    "Ziemele",    "Zībere",      "Zunda",      "Zvaigzne",   "Zvejniece",  "Zvinele",     "Muceniece", "Dzirkale",
    "Arāja",      "Sējēja",      "Podniece",   "Ratniece",   "Deģe",       "Bondare",     "Sondore",   "Namiķe",
    "Jumiķe",     "Muižniece",   "Bajāre",     "Ķēniņa",     "Ķeizare",    "Barone",      "Vaivade",   "Zviedre",
    "Vāciete",    "Lejiņa",      "Vanaga",     "Celma",      "Liepa",      "Kalna",       "Celmiņa",   "Lejniece",
    "Biseniece",  "Kurzemniece", "Sauliete",   "Pakalniete", "Pridāne",    "Groduma",     "Kalnāja",   "Porgante",
    "Sukute",     "Meškūna",     "Rožkalne",   "Mazjāne",    "Latkovska",  "Jankovska",   "Šileiko",   "Budreiko",
    "Ivanova",    "Morozova",    "Logina",     "Gasjūne",    "Pauļuka",    "Sosāre",      "Meškūne",
});

const auto latvianSuffixes = std::to_array<std::string_view>({"k-dze", "kundze"});

const auto latvianPrefixes = std::to_array<std::string_view>({"Prof.", "Dr.", "Biedrs"});

const NameFormats latvianNameFormats{
    {{"{firstName} {lastName}", 8}, {"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames latvianPeopleNames{
    {latvianMaleFirstNames, latvianMaleLastNames, latvianPrefixes, latvianSuffixes},
    {latvianFemaleFirstNames, latvianFemaleLastNames, latvianPrefixes, latvianSuffixes},
    latvianNameFormats};

// Lebanon

const auto lebaneseMaleFirstNames = std::to_array<std::string_view>({
    "Mohamed",  "Ali",      "Ahmed",     "Hassan",    "Hussein",     "Abdel",       "George",   "Elias",   "Yousef",
    "Joseph",   "Mahmoud",  "Ibrahim",   "Khaled",    "Antoine",     "Fadi",        "Mostafa",  "Michel",  "Jan",
    "Rabie",    "Samir",    "Waleed",    "Abbas",     "Omar",        "Nabil",       "Khalil",   "Bilal",   "Emad",
    "Ghassan",  "Jamal",    "Ziad",      "Samih",     "Bassam",      "Sami",        "Khader",   "Salim",   "Tarek",
    "Fouad",    "Samer",    "Shady",     "Qasim",     "Marwan",      "Ramy",        "Saeed",    "Abdo",    "Kamal",
    "Jamil",    "Essam",    "Adnan",     "Tawfiq",    "Adel",        "Raymond",     "Salam",    "Hossam",  "Nadeem",
    "Waseem",   "Hisham",   "Mousa",     "Asaad",     "Mahy",        "Naji",        "Amin",     "Mounir",  "Nazih",
    "Mazen",    "Suleiman", "Habib",     "Haytham",   "Ayman",       "Yahya",       "Milad",    "Maher",   "Riad",
    "Emile",    "Hani",     "Mahdi",     "Osama",     "Wael",        "Hadi",        "Kamel",    "Amer",    "Robert",
    "Kamil",    "Sayed",    "Karim",     "Saad",      "Nader",       "Talal",       "Rashid",   "Simon",   "Daniel",
    "Fayez",    "Anton",    "Nasser",    "Zuhair",    "Mark",        "Haidar",      "Bashir",   "Pascal",  "Farid",
    "Firas",    "Salah",    "Jack",      "Malik",     "Fares",       "Ghazi",       "Ismail",   "Charles", "Hamza",
    "Johnny",   "Saleh",    "Rony",      "Christian", "Fawzi",       "Suhail",      "Morris",   "Nizar",   "Akram",
    "Rafiq",    "Mohsen",   "Raed",      "Andre",     "Roy",         "Anthony",     "Majed",    "Faisal",  "Yvon",
    "Zakaria",  "Anwar",    "Zaher",     "Naeem",     "Richard",     "Adal",        "Joel",     "Shafiq",  "Foad",
    "Yasser",   "Mansour",  "Afif",      "Anis",      "Jaafar",      "Mario",       "Radwan",   "Atef",    "Patrick",
    "Edmond",   "Jalal",    "Marcel",    "Sherif",    "Badr",        "Lewis",       "Hilal",    "Salem",   "Roland",
    "Zain",     "William",  "Yacoub",    "Henry",     "Farouk",      "Antonio",     "Fahd",     "Osman",   "Victor",
    "David",    "Ralph",    "Rafi",      "Carlos",    "Hatem",       "Jawad",       "Halim",    "Michael", "Aziz",
    "Salman",   "Alain",    "Bernard",   "Amir",      "Karam",       "Badi",        "Rabah",    "Magi",    "John",
    "Antonius", "Taha",     "Anas",      "Randy",     "Gilbert",     "Adham",       "Sultan",   "Ammar",   "Rashad",
    "Amar",     "Dia",      "Taleb",     "Alexander", "Peter",       "Makram",      "Younis",   "Roly",    "Iqbal",
    "Andriy",   "Iyad",     "Wahib",     "Ashraf",    "Sidi",        "Lawrence",    "Hamad",    "Louay",   "Jesse",
    "Yassin",   "Hamid",    "Jamie",     "Julian",    "Lucien",      "Carl",        "Chris",    "Zaki",    "Remy",
    "Alberto",  "Tamer",    "Ayoub",     "Nazim",     "Rasheed",     "Majid",       "Sadiq",    "Stephan", "Merlin",
    "Abdullah", "Raouf",    "Ricardo",   "Noman",     "Murad",       "Christopher", "Angel",    "Serge",   "Massoud",
    "Hany",     "Roberto",  "Asad",      "Mesbah",    "Marco",       "Eduardo",     "Mike",     "Fadel",   "Sheikh",
    "Atta",     "Andrew",   "Roderick",  "Frederick", "Ishaq",       "Safi",        "Edgar",    "Najih",   "Cesar",
    "Adam",     "Mamoun",   "Domenech",  "Gerard",    "Dian",        "Lotfi",       "Wahid",    "Keita",   "Zaid",
    "Taher",    "Nizam",    "Abraham",   "Lucian",    "Romeo",       "Shahid",      "Martin",   "Aram",    "Ronald",
    "Amjad",    "Saif",     "Fahim",     "Ramadan",   "Elio",        "Hamed",       "Emilio",   "Safwan",  "Laurent",
    "Bert",     "Carlo",    "Leon",      "James",     "Manuel",      "Jaber",       "Ayad",     "Stephen", "Adeel",
    "Samuel",   "Marin",    "Mazhar",    "Cyril",     "Aqeel",       "Christof",    "Kristian", "Giorgio", "Kalin",
    "Mikhail",  "Fathi",    "Farhan",    "Badri",     "Arman",       "Radi",        "Demetri",  "Guy",     "Nasir",
    "Saleem",   "Luqman",   "Yousif",    "Sam",       "Amr",         "Daoud",       "Ernest",   "Mokhtar", "Jose",
    "Raoul",    "Alex",     "Jano",      "Sabri",     "Albert",      "Thomas",      "Benjamin", "Fabian",  "Saber",
    "Sergio",   "Steve",    "Justin",    "Germain",   "Alfons",      "Hector",      "Shah",     "Mathieu", "Rubin",
    "Karmel",   "Clemens",  "Oliver",    "Erik",      "Bakr",        "Mikael",      "Shamil",   "Nael",    "Nassar",
    "Jonathan", "Akbar",    "Jalil",     "Mumtaz",    "Haj",         "Armin",       "Fatih",    "Taj",     "Mohamed",
    "Ali",      "Ahmed",    "Hassan",    "Hussein",   "Abdel",       "George",      "Elias",    "Yousef",  "Joseph",
    "Mahmoud",  "Ibrahim",  "Khaled",    "Antoine",   "Fadi",        "Mostafa",     "Michel",   "Jan",     "Rabie",
    "Samir",    "Waleed",   "Abbas",     "Omar",      "Nabil",       "Khalil",      "Bilal",    "Emad",    "Ghassan",
    "Jamal",    "Ziad",     "Samih",     "Bassam",    "Sami",        "Khader",      "Salim",    "Tarek",   "Fouad",
    "Samer",    "Shady",    "Qasim",     "Marwan",    "Ramy",        "Saeed",       "Abdo",     "Kamal",   "Jamil",
    "Essam",    "Adnan",    "Tawfiq",    "Adel",      "Raymond",     "Salam",       "Hossam",   "Nadeem",  "Waseem",
    "Hisham",   "Mousa",    "Asaad",     "Mahy",      "Naji",        "Amin",        "Mounir",   "Nazih",   "Mazen",
    "Suleiman", "Habib",    "Haytham",   "Ayman",     "Yahya",       "Milad",       "Maher",    "Riad",    "Emile",
    "Hani",     "Mahdi",    "Osama",     "Wael",      "Hadi",        "Kamel",       "Amer",     "Robert",  "Kamil",
    "Sayed",    "Karim",    "Saad",      "Nader",     "Talal",       "Rashid",      "Simon",    "Daniel",  "Fayez",
    "Anton",    "Nasser",   "Zuhair",    "Mark",      "Haidar",      "Bashir",      "Pascal",   "Farid",   "Firas",
    "Salah",    "Jack",     "Malik",     "Fares",     "Ghazi",       "Ismail",      "Charles",  "Hamza",   "Johnny",
    "Saleh",    "Rony",     "Christian", "Fawzi",     "Suhail",      "Morris",      "Nizar",    "Akram",   "Rafiq",
    "Mohsen",   "Raed",     "Andre",     "Roy",       "Anthony",     "Majed",       "Faisal",   "Yvon",    "Zakaria",
    "Anwar",    "Zaher",    "Naeem",     "Richard",   "Adal",        "Joel",        "Shafiq",   "Foad",    "Yasser",
    "Mansour",  "Afif",     "Anis",      "Jaafar",    "Mario",       "Radwan",      "Atef",     "Patrick", "Edmond",
    "Jalal",    "Marcel",   "Sherif",    "Badr",      "Lewis",       "Hilal",       "Salem",    "Roland",  "Zain",
    "William",  "Yacoub",   "Henry",     "Farouk",    "Antonio",     "Fahd",        "Osman",    "Victor",  "David",
    "Ralph",    "Rafi",     "Carlos",    "Hatem",     "Jawad",       "Halim",       "Michael",  "Aziz",    "Salman",
    "Alain",    "Bernard",  "Amir",      "Karam",     "Badi",        "Rabah",       "Magi",     "John",    "Antonius",
    "Taha",     "Anas",     "Randy",     "Gilbert",   "Adham",       "Sultan",      "Ammar",    "Rashad",  "Amar",
    "Dia",      "Taleb",    "Alexander", "Peter",     "Makram",      "Younis",      "Roly",     "Iqbal",   "Andriy",
    "Iyad",     "Wahib",    "Ashraf",    "Sidi",      "Lawrence",    "Hamad",       "Louay",    "Jesse",   "Yassin",
    "Hamid",    "Jamie",    "Julian",    "Lucien",    "Carl",        "Chris",       "Zaki",     "Remy",    "Alberto",
    "Tamer",    "Ayoub",    "Nazim",     "Rasheed",   "Majid",       "Sadiq",       "Stephan",  "Merlin",  "Abdullah",
    "Raouf",    "Ricardo",  "Noman",     "Murad",     "Christopher", "Angel",       "Serge",    "Massoud", "Hany",
    "Roberto",  "Asad",     "Mesbah",    "Marco",     "Eduardo",     "Mike",        "Fadel",    "Sheikh",  "Atta",
    "Andrew",   "Roderick", "Frederick", "Ishaq",     "Safi",        "Edgar",       "Najih",    "Cesar",   "Adam",
    "Mamoun",   "Domenech", "Gerard",    "Dian",      "Lotfi",       "Wahid",       "Keita",    "Zaid",    "Taher",
    "Nizam",    "Abraham",  "Lucian",    "Romeo",     "Shahid",      "Martin",      "Aram",     "Ronald",  "Amjad",
    "Saif",     "Fahim",    "Ramadan",   "Elio",      "Hamed",       "Emilio",      "Safwan",   "Laurent", "Bert",
    "Carlo",    "Leon",     "James",     "Manuel",    "Jaber",       "Ayad",        "Stephen",  "Adeel",   "Samuel",
    "Marin",    "Mazhar",   "Cyril",     "Aqeel",     "Christof",    "Kristian",    "Giorgio",  "Kalin",   "Mikhail",
    "Fathi",    "Farhan",   "Badri",     "Arman",     "Radi",        "Demetri",     "Guy",      "Nasir",   "Saleem",
    "Luqman",   "Yousif",   "Sam",       "Amr",       "Daoud",       "Ernest",      "Mokhtar",  "Jose",    "Raoul",
    "Alex",     "Jano",     "Sabri",     "Albert",    "Thomas",      "Benjamin",    "Fabian",   "Saber",   "Sergio",
    "Steve",    "Justin",   "Germain",   "Alfons",    "Hector",      "Shah",        "Mathieu",  "Rubin",   "Karmel",
    "Clemens",  "Oliver",   "Erik",      "Bakr",      "Mikael",      "Shamil",      "Nael",     "Nassar",  "Jonathan",
    "Akbar",    "Jalil",    "Mumtaz",    "Haj",       "Armin",       "Fatih",       "Taj",
});

const auto lebaneseFemaleFirstNames = std::to_array<std::string_view>({
    "Fatima",     "Zainab",    "Mary",      "Mariam",     "Laila",      "Mona",      "Elly",      "Amal",
    "Nada",       "Hoda",      "Samira",    "Rita",       "Khadija",    "Rima",      "Nadia",     "Souad",
    "Iman",       "Lina",      "Rania",     "Sarah",      "Rana",       "Wafa",      "Hanan",     "Ghada",
    "Hala",       "Zahra",     "Samar",     "Siham",      "Wissam",     "Nawal",     "Jihad",     "Fadia",
    "Nasreen",    "Nour",      "Jamila",    "Manal",      "Aida",       "Hana",      "Sana",      "Mirna",
    "Salwa",      "Heba",      "Elham",     "Maria",      "Hayam",      "Nihad",     "Susan",     "Maya",
    "Dalal",      "Randa",     "Hanna",     "Sahar",      "Najwa",      "Faten",     "Mai",       "Alaa",
    "Dany",       "Sabah",     "Sawsan",    "Ibtisam",    "Noha",       "Josephine", "Nicola",    "Georgette",
    "Antoinette", "Samia",     "Hayat",     "Diana",      "Salma",      "Amna",      "Najah",     "Hind",
    "Farah",      "Rasha",     "Sonia",     "Jacqueline", "Rose",       "Magda",     "Lilian",    "Nadine",
    "Alia",       "Najat",     "Safa",      "Anam",       "Therese",    "Linda",     "Janet",     "Elissa",
    "Karol",      "Rajaa",     "Yasmin",    "Rayan",      "Maha",       "Marwa",     "Donia",     "Lara",
    "Nancy",      "Claude",    "Elian",     "Maysaa",     "Halima",     "Rene",      "Latifa",    "Afaf",
    "Basma",      "Najla",     "Ayesha",    "Nathalie",   "Jocelyn",    "Katia",     "Batoul",    "Reda",
    "Ahlam",      "Norma",     "Christine", "Caroline",   "Malak",      "Nidal",     "Elin",      "Karen",
    "Marilyn",    "Yola",      "Janan",     "Shadia",     "Rabab",      "Carla",     "Nicole",    "Stefani",
    "Fadwa",      "Samah",     "Nahla",     "Lamia",      "Sandra",     "Nora",      "Jihan",     "Jessica",
    "Tania",      "Hina",      "Ihsan",     "Badia",      "Kawthar",    "Hikmat",    "Dalia",     "Fattoum",
    "Rehab",      "Nelly",     "Dina",      "Jannat",     "Kholoud",    "Nazira",    "Nariman",   "Mira",
    "Elia",       "Violet",    "Catherine", "Elaine",     "Bahia",      "Pauline",   "Bushra",    "Ala",
    "Aya",        "Joyce",     "Aisha",     "Asma",       "Sally",      "Grace",     "Asmahan",   "Julia",
    "Joanna",     "Fayrouz",   "Dima",      "Kristel",    "Suhaila",    "Sabine",    "Juliet",    "Dana",
    "Emily",      "Olga",      "Carmen",    "Nina",       "Bernadette", "Nahed",     "Awatif",    "Nasim",
    "Pamela",     "Ana",       "Madeleine", "Dania",      "Ikram",      "Marian",    "Claire",    "Silva",
    "Deniz",      "Lydia",     "Margret",   "Lubna",      "Duaa",       "Sandy",     "Vera",      "Zina",
    "Lea",        "Mariana",   "Christina", "Raja",       "Yara",       "Abla",      "Laura",     "Hilda",
    "Hajar",      "Amira",     "Anaya",     "Fahima",     "Lamis",      "Ria",       "Cynthia",   "Rachel",
    "Asmaa",      "Manar",     "Claudine",  "Marina",     "Naima",      "Rahma",     "Wahiba",    "Tamara",
    "Joan",       "Dori",      "Walaa",     "Rosa",       "Melia",      "Ramona",    "Georgina",  "Saba",
    "Asia",       "Janine",    "Nahia",     "Lana",       "Elizabeth",  "Amanda",    "Khadra",    "Claudia",
    "Bahija",     "Anahid",    "Giselle",   "Ayat",       "Jana",       "Lucy",      "Jennifer",  "Selin",
    "Rani",       "Nayla",     "Inas",      "Zeina",      "Myriam",     "Tatiana",   "Hassiba",   "Najma",
    "Silvana",    "Alexandra", "Barbara",   "Francis",    "Juliana",    "Liza",      "Annie",     "Ann",
    "Adama",      "Lorraine",  "Shaheen",   "Hadia",      "Yousra",     "Ela",       "Elina",     "Mari",
    "Melissa",    "Arlette",   "Samara",    "Ara",        "Jenna",      "Joy",       "Claudette", "Hasna",
    "Khawla",     "Reina",     "Vanessa",   "Clara",      "Monique",    "Wesal",     "Fawzia",    "Rebecca",
    "Nabila",     "Ramia",     "Suzy",      "Charlotte",  "Mounira",    "Esther",    "Helen",     "Anita",
    "Christel",   "Doris",     "Nermin",    "Kamelia",    "Wiam",       "Nasima",    "Silvia",    "Amina",
    "Dalida",     "Louisa",    "Zakia",     "Katy",       "Noel",       "Andrea",    "Razan",     "Shahnaz",
    "Natasha",    "Rosy",      "Donna",     "Isabelle",   "Natalya",    "Maram",     "Nasra",     "Amani",
    "Amelia",     "Margaret",  "Tina",      "Tamar",      "Ava",        "Milada",    "Gloria",    "Azza",
    "Angela",     "Beatrice",  "Amalia",    "Elsa",       "Nahida",     "Bayan",     "Valery",    "Rabia",
    "Rokaya",     "Hawa",      "Eliza",     "Theresa",    "Eliana",     "Salima",    "Monica",    "Sima",
    "Nargis",     "Brigitte",  "Kamila",    "Samiha",     "Daniella",   "Dolly",
});

const auto lebaneseLastNames = std::to_array<std::string_view>({
    "El din",      "Allah",     "El khoury", "Khalil",     "Saad",        "El hajj",     "Ali",        "Ibrahim",
    "El masry",    "Hassan",    "Haidar",    "Hamoud",     "Saleh",       "Ahmed",       "Suleiman",   "Mousa",
    "Younis",      "Yousef",    "Karam",     "Eisaa",      "Mansour",     "Farhat",      "Aboud",      "Haddad",
    "Harb",        "Deeb",      "Hussein",   "Ayoub",      "Eid",         "Saliba",      "Hanna",      "Shaheen",
    "Dahir",       "Matar",     "Murad",     "El haddad",  "Awad",        "Ismail",      "Jaber",      "Hamdan",
    "Assaf",       "Yassin",    "Hashem",    "Abbas",      "Mohamed",     "Obaid",       "Diab",       "Aoun",
    "Zaiter",      "Qasim",     "Al ashqar", "Rizk",       "Azar",        "Nader",       "Salama",     "Darwish",
    "El hassan",   "El sayed",  "Jouda",     "Khalifa",    "Farah",       "Hamza",       "Osman",      "Saeed",
    "Raad",        "Nasser",    "Taleb",     "Fares",      "Jaafar",      "Nasr",        "Mari",       "Saab",
    "Zidan",       "Ghanim",    "Hegazy",    "Nemeh",      "Al khatib",   "Kanaan",      "Nassar",     "El halaby",
    "Salloum",     "Al zain",   "Shehadeh",  "Zgheib",     "Khader",      "Daou",        "Habib",      "Mostafa",
    "Dagher",      "Al hayek",  "El shami",  "Abdo",       "Salim",       "Khoury",      "Yazbek",     "Anton",
    "Samaan",      "Saqr",      "Shams",     "El helou",   "Maalouf",     "Hatoum",      "Wahby",      "Najm",
    "Al asmar",    "Fayad",     "Wahba",     "Rahal",      "Rashid",      "Asaad",       "Hamada",     "Tannous",
    "Al moussawi", "Fawaz",     "Hamieh",    "Sfeir",      "Nasif",       "Fakih",       "Jabbour",    "Barakat",
    "Samah",       "Shamoun",   "Al-qazi",   "Elias",      "Shalhoub",    "Fadel",       "Mahmoud",    "Attieh",
    "Al husseini", "Gerges",    "Moawad",    "Awadah",     "Al hujairi",  "Al rifai",    "Akl",        "Shaaban",
    "Assi",        "Faraj",     "Al mekdad", "Al awar",    "Aqeqi",       "Dawood",      "Muhanna",    "Yahya",
    "Zaid",        "Ghosn",     "Sarkis",    "Choucair",   "Melhem",      "Ramadan",     "Salman",     "Shehab",
    "Yamin",       "Al sheikh", "Taha",      "Juma",       "Shukr",       "Madi",        "Saba",       "Yacoub",
    "Beydoun",     "Boutros",   "Al-hakim",  "Sorour",     "Amhaz",       "Khaled",      "Ammar",      "Al khaliq",
    "Itani",       "Matta",     "El gamal",  "El fagaly",  "Bassil",      "Hassoun",     "Al bitar",   "Al sayegh",
    "Mazloum",     "Basharah",  "Al ahmad",  "Shadid",     "Mortada",     "Kassab",      "Al najjar",  "Al bustani",
    "Zain",        "Stephan",   "Maroun",    "Yaghi",      "Tameh",       "Sabra",       "Bitar",      "Tarbih",
    "Salem",       "Maatouq",   "Mashik",    "Hamad",      "El ayoubi",   "Al hussein",  "Tabet",      "Atwi",
    "Shibli",      "Mahdi",     "Kfoury",    "Shaia",      "El mawla",    "Touma",       "Makhoul",    "Shoman",
    "Khalaf",      "Mubarak",   "El shaar",  "Naeem",      "Francis",     "Alama",       "Reda",       "Trad",
    "Sawaya",      "Shoaib",    "Odeh",      "Ayyash",     "Zaher",       "Safi",        "Omar",       "Ghandour",
    "El raey",     "King",      "Lahoud",    "Badr",       "Sassin",      "El assaad",   "Nakhla",     "Zaarour",
    "Marwa",       "Badran",    "Sherif",    "Ziadeh",     "Azzam",       "Al mohammed", "Jamal",      "Al-kurdi",
    "Al rayes",    "El gohary", "Makki",     "Hammadi",    "Fahd",        "Mezher",      "Suwaidan",   "El arab",
    "El sahely",   "Habashi",   "Zakaria",   "Talis",      "Gharib",      "Sawan",       "Hilal",      "Ghannam",
    "Al zubi",     "Bazzi",     "Rahma",     "Munther",    "Massoud",     "Makarem",     "El shamaly", "Bolas",
    "Dargham",     "Wakim",     "Saif",      "Mohsen",     "El maghraby", "Karim",       "Al sabbagh", "Al rahman",
    "Al qadri",    "Al sbai",   "Hobeika",   "Jawad",      "Alloush",     "Al-qadi",     "Gabr",       "Rabah",
    "Ishaq",       "Ghazal",    "Aqeel",     "El ghareeb", "Al-karim",    "Khair",       "Al homsi",   "Noman",
    "Nahra",       "Karaki",    "Iskandar",  "Sweid",      "Waked",       "El taweel",   "Abdullah",   "Sobh",
    "Raslan",      "Amer",      "Nicola",    "Baro",       "Hayek",       "Al ghoul",    "Khater",     "Shaker",
    "Kamel",       "Noun",      "Najim",     "Mahfouz",    "Fakher",      "El badawy",   "Massad",     "Khatar",
    "Amin",        "Mitry",     "Safa",      "Al-khader",  "Najjar",      "Salami",      "Sarhan",     "Allam",
    "Hatem",       "Orabi",     "Fakhoury",  "Jabbara",    "Al mostafa",  "Antar",       "Sabbagh",    "Muslim",
    "Aldbs",       "Emad",      "Radwan",    "Al atrash",  "Sayegh",      "Makhlouf",    "Mikhael",    "Rustom",
    "Hajjar",      "Jibril",    "Salhab",    "Latif",      "El banna",    "Qablan",      "Dia",        "Qadour",
    "Salibi",      "Badawi",    "Al hamwi",  "Al saleh",   "Baalbaki",    "Heikal",
});

const NameFormats lebaneseNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames lebanesePeopleNames{{lebaneseMaleFirstNames, lebaneseLastNames, {}, {}},
                                      {lebaneseFemaleFirstNames, lebaneseLastNames, {}, {}},
                                      lebaneseNameFormats};

// Lithuania

const auto lithuanianMaleFirstNames = std::to_array<std::string_view>({
    "Jonas",      "Vytautas",   "Antanas",   "Juozas",     "Petras",    "Algirdas",    "Stasys",     "Saulius",
    "Rimantas",   "Algimantas", "Mindaugas", "Darius",     "Tomas",     "Andrius",     "Gintaras",   "Romualdas",
    "Gintautas",  "Arvydas",    "Gediminas", "Egidijus",   "Alvydas",   "Sigitas",     "Valdas",     "Vidmantas",
    "Audrius",    "Rolandas",   "Marius",    "Viktoras",   "Vladas",    "Povilas",     "Albinas",    "Algis",
    "Virginijus", "Aleksandr",  "Vaclovas",  "Romas",      "Robertas",  "Aleksandras", "Remigijus",  "Giedrius",
    "Dainius",    "Donatas",    "Vladimir",  "Linas",      "Kazys",     "Edmundas",    "Nerijus",    "Vaidas",
    "Raimondas",  "Albertas",   "Rimas",     "Eugenijus",  "Mantas",    "Vidas",       "Leonas",     "Edvardas",
    "Ivan",       "Tadas",      "Viktor",    "Raimundas",  "Julius",    "Paulius",     "Evaldas",    "Justinas",
    "Vilius",     "Valentinas", "Sergej",    "Stanislav",  "Henrikas",  "Rimvydas",    "Vitalijus",  "Nikolaj",
    "Aurimas",    "Martynas",   "Andrej",    "Virgilijus", "Jan",       "Vaidotas",    "Alfredas",   "Karolis",
    "Vladimiras", "Michail",    "Aidas",     "Edgaras",    "Ignas",     "Jevgenij",    "Dalius",     "Igor",
    "Pavel",      "Arturas",    "Oleg",      "Aivaras",    "Vladislav", "Rytis",       "Valerijus",  "Jurijus",
    "Edvinas",    "Justas",     "Modestas",  "Lukas",      "Eduardas",  "Deividas",    "Kostas",     "Laimonas",
    "Gytis",      "Irmantas",   "Laurynas",  "Dmitrij",    "Piotr",     "Aleksej",     "Aurelijus",  "Anton",
    "Leonid",     "Rokas",      "Renatas",   "Sergejus",   "Ernestas",  "Artur",       "Valentin",   "Arnoldas",
    "Andrejus",   "Arunas",     "Olegas",    "Edvard",     "Eimantas",  "Igoris",      "Boris",      "Kestutis",
    "Dovydas",    "Tautvydas",  "Miroslav",  "Vaclav",     "Romuald",   "Roman",       "Eduard",     "Konstantin",
    "Ramunas",    "Erikas",     "Arnas",     "Franc",      "Vadim",     "Robert",      "Sarunas",    "Matas",
    "Jaroslav",   "Henrik",     "Ruslan",    "Dominykas",  "Maksim",    "Zilvinas",    "Zenon",      "Valdemar",
    "Edgar",      "Daniel",     "Mark",      "Fiodor",     "Leon",      "Leonard",     "Ruslanas",   "Stefan",
    "Alex",       "Ricardas",   "Edmund",    "Alexander",  "Albert",    "Marek",       "Ilja",       "Erik",
    "Andrey",     "Feliks",     "David",     "Deivis",     "Sergey",    "Ernest",      "Airidas",    "Adam",
    "Anatoly",    "John",       "Stepan",    "Vladyslav",  "Lech",      "Slavomir",    "Lev",        "Vlad",
    "Bogdan",     "Antoni",     "Andrei",    "Michael",    "Henryk",    "Artiom",      "Zydrunas",   "Ale",
    "Edvin",      "Max",        "Dmitry",    "Tom",        "Rafal",     "Raimond",     "Martin",     "Darek",
    "Evgeny",     "Andrzej",    "Arnold",    "German",     "Hendrik",   "Sergei",      "Victor",     "Armandas",
    "Edward",     "Aleks",      "Iosif",     "Maxim",      "Alfred",    "Michal",      "Andrew",     "Kiril",
    "Svajunas",   "Bernard",    "Nedas",     "Marat",      "Petr",      "Artem",       "Evgeniy",    "Nikolai",
    "Dmitriy",    "Gleb",       "Pawel",     "Joseph",     "Richard",   "Edgard",      "Aleksey",    "Alik",
    "Paul",       "Adolf",      "Thomas",    "Albin",      "Patrik",    "Ronald",      "Nikolay",    "Ricard",
    "Tomasz",     "Aleksander", "Mikhail",   "Carlos",     "Jaroslaw",  "Samuil",      "Kevin",      "Kirill",
    "Vitaly",     "Deivid",     "Volodymyr", "Abram",      "Roland",    "Danil",       "Sasa",       "Ilya",
    "Chris",      "Roberto",    "Vincent",   "Ben",        "Vitaliy",   "Jacek",       "Haris",      "Frank",
    "Marko",      "Filip",      "Rostislav", "Rudolf",     "Vsevolod",  "Norbert",     "Dominik",    "Oleksandr",
    "Just",       "Karl",       "Timur",     "Fabio",      "Luka",      "Edwin",       "Farid",      "Aron",
    "Adrian",     "Jay",        "Ali",       "Gabriel",    "Dawid",     "Jamal",       "Alfons",     "Gonzalo",
    "Tim",        "Patryk",     "Nick",      "Benedikt",   "Mario",     "Sebastian",   "Di",         "Steven",
    "Vyacheslav", "Markus",     "Elvis",     "Julian",     "George",    "Radek",       "Yaroslav",   "Waldemar",
    "Omer",       "Georg",      "Lukasz",    "Ervin",      "Daniil",    "Simon",       "Andre",      "Murat",
    "Jack",       "Oskar",      "Radoslav",  "Rustam",     "Nikolas",   "Anatoliy",    "Emile",      "Gennady",
    "Damian",     "Natan",      "Francesco", "Alexey",     "Alpha",     "Juris",       "Viacheslav", "Mustafa",
    "Tommy",      "James",      "Yury",      "Ousmane",    "Zygmunt",   "Ignat",       "Yuriy",      "Moussa",
    "Jonathan",   "Justin",     "Raimund",   "Anthony",    "Benjamin",  "Klaus",       "Miron",      "Valeriy",
    "Antonio",    "Nicolas",    "Stefano",   "Christian",  "Bruno",     "Aleksandar",
});

const auto lithuanianFemaleFirstNames = std::to_array<std::string_view>({
    "Ona",        "Irena",      "Janina",     "Aldona",     "Regina",    "Elena",     "Marija",    "Daiva",
    "Vida",       "Zita",       "Stanislava", "Rasa",       "Dalia",     "Virginija", "Lina",      "Kristina",
    "Rita",       "Vanda",      "Laima",      "Jolanta",    "Valentina", "Rima",      "Galina",    "Valerija",
    "Violeta",    "Asta",       "Vilma",      "Liudmila",   "Veronika",  "Tatjana",   "Albina",    "Ana",
    "Jelena",     "Jurgita",    "Inga",       "Olga",       "Svetlana",  "Loreta",    "Emilija",   "Natalija",
    "Edita",      "Irina",      "Aleksandra", "Nina",       "Julija",    "Renata",    "Viktorija", "Teresa",
    "Diana",      "Vitalija",   "Ilona",      "Monika",     "Vilija",    "Alma",      "Joana",     "Ingrida",
    "Vaida",      "Milda",      "Sigita",     "Vladislava", "Ieva",      "Lidija",    "Jolita",    "Alina",
    "Tamara",     "Neringa",    "Zinaida",    "Danuta",     "Roma",      "Laura",     "Sandra",    "Vera",
    "Larisa",     "Jovita",     "Vita",       "Kazimiera",  "Aurelija",  "Reda",      "Irma",      "Erika",
    "Ina",        "Eleonora",   "Justina",    "Sofija",     "Gitana",    "Raimonda",  "Margarita", "Romualda",
    "Marina",     "Zina",       "Raisa",      "Ala",        "Sonata",    "Oksana",    "Natalja",   "Leonora",
    "Leonarda",   "Jekaterina", "Karolina",   "Simona",     "Zuzana",    "Evelina",   "Liubov",    "Donata",
    "Judita",     "Egle",       "Dana",       "Greta",      "Ligita",    "Antonina",  "Adolfina",  "Daina",
    "Aniceta",    "Helena",     "Anna",       "Paulina",    "Agne",      "Brigita",   "Aida",      "Filomena",
    "Ruta",       "Halina",     "Vaiva",      "Inesa",      "Marjan",    "Silva",     "Indre",     "Silvija",
    "Barbora",    "Gintare",    "Leonidas",   "Raminta",    "Elzbieta",  "Palmira",   "Dovile",    "Giedre",
    "Lijana",     "Jurate",     "Lolita",     "Sabina",     "Odeta",     "Aiste",     "Miroslava", "Karina",
    "Magdalena",  "Ausra",      "Marijana",   "Ernesta",    "Beata",     "Teodora",   "Denis",     "Ineta",
    "Marta",      "Malvina",    "Aliona",     "Gerda",      "Gabriele",  "Alfreda",   "Eva",       "Lena",
    "Vlada",      "Gabija",     "Liuba",      "Angelina",   "Roberta",   "Ema",       "Alfonsa",   "Zivile",
    "Nida",       "Barbara",    "Dina",       "Ramune",     "Lida",      "Ivona",     "Bernadeta", "Rolanda",
    "Gema",       "Nijole",     "Toma",       "Migle",      "Ugne",      "Polina",    "Alevtina",  "Iveta",
    "Leonida",    "Birute",     "Deimante",   "Ksenija",    "Gita",      "Ada",       "Leontina",  "Nikita",
    "Roza",       "Audrone",    "Varvara",    "Taisa",      "Alvina",    "Faina",     "Zenona",    "Agata",
    "Gina",       "Matilda",    "Dima",       "Nelia",      "Albertina", "Virgina",   "Rosita",    "Anfisa",
    "Kamile",     "Alisa",      "Dijana",     "Tereza",     "Palma",     "Honorata",  "Martyna",   "Neli",
    "Aneta",      "Julia",      "Indra",      "Grazina",    "Modesta",   "Romana",    "Augustina", "Danute",
    "Iraida",     "Alla",       "Lyda",       "Ela",        "Tania",     "Ilma",      "Maja",      "Dominika",
    "Elina",      "Ida",        "Justyna",    "Danguole",   "Katerina",  "Severina",  "Kira",      "Izolda",
    "Sasha",      "Lilia",      "Inna",       "Mira",       "Zana",      "Jana",      "Klara",     "Natalia",
    "Irmina",     "Ekaterina",  "Liza",       "Dorota",     "Arina",     "Anzelika",  "Maria",     "Viktoriya",
    "Darja",      "Santa",      "Laimute",    "Ira",        "Tatiana",   "Vasilisa",  "Bozena",    "Julita",
    "Olesia",     "Liana",      "Nora",       "Milana",     "Nadezda",   "Benigna",   "Alena",     "Aura",
    "Tatyana",    "Victoria",   "Nona",       "Gabriela",   "Mariana",   "Viera",     "Katia",     "Berta",
    "Agripina",   "Nelli",      "Ninel",      "Anita",      "Hana",      "Jolanda",   "Alis",      "Luiza",
    "Katya",      "Raimunda",   "Zenaida",    "Juliana",    "Anastasia", "Valia",     "Samanta",   "Erna",
    "Linda",      "Janis",      "Sania",      "Ewelina",    "Olya",      "Yulia",     "Marian",    "Elvira",
    "Milena",     "Nika",       "Christina",  "Leonila",    "Beatrice",  "Lora",      "Faustina",  "Ania",
    "Bela",       "Snezana",    "Alberta",    "Natali",     "Elia",      "Nela",      "Ramona",    "Johana",
    "Krystyna",   "Aina",       "Fedora",     "Angelika",   "Anelia",    "Rufina",    "Neonila",   "Zenia",
    "Anastasiya", "Bogumila",   "Emma",       "Jaroslava",  "Marja",     "Dasha",     "Mila",      "Ramute",
    "Sonia",      "Aksana",     "Ella",       "Elizaveta",  "Kamila",    "Nastia",    "Rimma",     "Valera",
    "Asia",       "Hilda",      "Adolfa",     "Angele",     "Elmira",    "Natalya",
});

const auto lithuanianLastNames = std::to_array<std::string_view>({
    "Petrauskas",   "Jankauskas",   "Kazlauskas",   "Stankevičius", "Vasiliauskas", "Butkus",       "Urbonas",
    "Kavaliauskas", "žukauskas",    "Ramanauskas",  "Paulauskas",   "Navickas",     "Rimkus",       "šimkus",
    "Pocius",       "Savickas",     "žilinskas",    "Sakalauskas",  "Adomaitis",    "Petraitis",    "Mikalauskas",
    "Kaminskas",    "Baranauskas",  "Kairys",       "žemaitis",     "Bagdonas",     "Ivanauskas",   "Ivanova",
    "Mockus",       "Venckus",      "Rutkauskas",   "Jonaitis",     "Vaitkus",      "Norkus",       "Vyšniauskas",
    "Juška",        "Dambrauskas",  "Bernotas",     "Malinauskas",  "Budrys",       "Radzevičius",  "Vitkauskas",
    "Markevičius",  "Kubilius",     "Grigas",       "Jurevičius",   "Lukošius",     "Stankus",      "Mažeika",
    "Sabaliauskas", "Sadauskas",    "Mackevič",     "Gricius",      "Jonas",        "Stonkus",      "Lapinskas",
    "Juškevičius",  "Povilaitis",   "Rakauskas",    "Banys",        "Barkauskas",   "Bružas",       "Bartkus",
    "šidlauskas",   "Mickus",       "šukys",        "Martinkus",    "Ivanov",       "Jonušas",      "Balsys",
    "Urbonavičius", "Barauskas",    "Vitkus",       "Vaičiulis",    "Petkus",       "Giedraitis",   "Jokubauskas",
    "Tomaševič",    "Butkevičius",  "Baltrušaitis", "čepulis",      "Vytautas",     "Stonys",       "Andrijauskas",
    "Miškinis",     "Urbanovič",    "Miliauskas",   "Alekna",       "Noreika",      "Astrauskas",   "Juknevičius",
    "Matulevičius", "Klimas",       "Buivydas",     "Bukauskas",    "Valaitis",     "Grigaitis",    "Brazauskas",
    "Gedvilas",     "Vasiljeva",    "Remeika",      "Gudaitis",     "Petronis",     "Sokolova",     "Karalius",
    "Ramonas",      "Laurinaitis",  "Popova",       "Milašius",     "Novikova",     "Kuznecova",    "Petrulis",
    "Navikas",      "Moroz",        "Katinas",      "Janulis",      "Burba",        "Grigonis",     "Molis",
    "Lebedeva",     "Jasaitis",     "Smirnova",     "Tamulis",      "Sutkus",       "Rudys",        "Murauskas",
    "Lazauskas",    "Laukaitis",    "Orlova",       "Jakaitis",     "Gudas",        "Aleksa",       "Jankus",
    "Kozlova",      "Urba",         "Antanas",      "Pavlova",      "Masiulis",     "Stanaitis",    "Andrejeva",
    "Janulevičius", "Degutis",      "Milius",       "Stravinskas",  "Baronas",      "Karvelis",     "Naujokas",
    "Petrova",      "Grabauskas",   "Timofejeva",   "Kalvaitis",    "Dargis",       "Solovjova",    "Grigorjeva",
    "Morozova",     "Kisel",        "Aleksejeva",   "Tomas",        "Pumputis",     "Briedis",      "Palaima",
    "Gaidys",       "Petrov",       "Daukša",       "Koval",        "Savickis",     "Stepanova",    "Dapkus",
    "Anužis",       "Sokolov",      "Janina",       "Danielius",    "Jegorova",     "Dulko",        "Andrulis",
    "Brazaitis",    "Raila",        "Rybakova",     "Gylys",        "Matulaitis",   "Juozas",       "Ona",
    "Sereika",      "Kovalenko",    "Rastenis",     "Staniulis",    "Galinis",      "Grybauskas",   "Marozas",
    "Miloš",        "Matulis",      "Stukas",       "Gedgaudas",    "Smirnov",      "Statkus",      "Stankevicius",
    "Volodko",      "Popov",        "Jocas",        "Makarova",     "Komar",        "Jasas",        "Jakovleva",
    "Drozd",        "Kolesnikova",  "Sergejeva",    "Masaitis",     "Verseckas",    "Linkus",       "Simonaitis",
    "Gaidelis",     "Bondarenko",   "Nikitina",     "Vasiljev",     "Kazak",        "Maslauskas",   "Valys",
    "Tumas",        "Solovjov",     "Marcinkus",    "Valiulis",     "Rudis",        "Volkova",      "Lukas",
    "Miau",         "Marija",       "Fedotova",     "Petrik",       "Balciunas",    "Guseva",       "Valeika",
    "Vaitkevicius", "Gintautas",    "Adamonis",     "Stasys",       "Petras",       "Babič",        "Kozlovska",
    "Suckel",       "Simkus",       "Zukauskas",    "Senkus",       "Romanova",     "Frolova",      "Tautkus",
    "Algirdas",     "Mazeika",      "Gustaitis",    "Barzda",       "Tarasova",     "Kuzma",        "Belova",
    "Paliulis",     "Rimas",        "Visockis",     "Karosas",      "Novikov",      "Pavilonis",    "Sabonis",
    "Stankaitis",   "Bakas",        "Butrim",       "Matonis",      "Orlov",        "Kuzmina",      "Bogdanova",
    "Kasputis",     "Kaupas",       "Zubkova",      "Bogdanov",     "Paulius",      "Gavrilova",    "Borisova",
    "Banelis",      "Medvedeva",    "Antonova",     "Valenta",      "Jankovska",    "Lebed",        "Bakutis",
    "Irena",        "Maksimova",    "Adamovic",     "Mikuta",       "Zamara",       "Petrauskaite", "Sidorova",
    "Volk",         "Mikulska",     "Alionis",      "Filipovič",    "Fiodorov",     "Aleksandrov",  "Vilutis",
    "Miller",       "Urban",        "Novik",        "Songin",       "Pavlov",       "Matvejeva",    "Gerasimova",
    "Rud",          "Musteikis",    "Jankauskaite", "Dmitrijeva",   "Baranova",     "Kulik",        "Jagminas",
    "Radžius",      "Aleksandrova", "Zaiceva",      "Kirilova",     "Gustas",       "Latvys",       "Kuras",
    "Radvilas",     "Jefimova",     "Bogdanovic",   "Stropus",      "Semeniuk",     "Danilova",     "Kutka",
    "Dobilas",      "Tatjana",      "Popovas",      "Kim",          "Kalinina",     "Saulis",       "Serva",
    "Saveljeva",    "Silko",        "Rukas",        "Kraujalis",    "Kod",          "Karpova",      "Li",
    "German",       "Savicka",      "Titova",       "Markova",      "Romanov",      "Zubkov",       "Nazarova",
    "Kropa",        "Nera",         "Drozdova",     "Kozlov",       "Milaknis",     "Jan",          "Kalenda",
});

const NameFormats lithuanianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames lithuanianPeopleNames{{lithuanianMaleFirstNames, lithuanianLastNames, {}, {}},
                                        {lithuanianFemaleFirstNames, lithuanianLastNames, {}, {}},
                                        lithuanianNameFormats};

// Macedonia

const auto macedonianMaleFirstNames = std::to_array<std::string_view>({
    "Александар", "Алексо",     "Ангел",     "Ангеле",      "Андон",    "Андреј",     "Андреја",   "Анте",
    "Антим",      "Антон",      "Антониј",   "Апостол",     "Арсениј",  "Асен",       "Атанас",    "Атанасиј",
    "Ацо",        "Благе",      "Благој",    "Благоја",     "Блаже",    "Богатин",    "Богдан",    "Богоја",
    "Богослав",   "Божидар",    "Божин",     "Божо",        "Бојан",    "Борис",      "Борислав",  "Борјан",
    "Борко",      "Боро",       "Бошко",     "Бранко",      "Вангел",   "Ване",       "Ванче",     "Ванчо",
    "Васил",      "Василиј",    "Василко",   "Велимир",     "Величко",  "Велјан",     "Вељо",      "Венко",
    "Веселин",    "Видое",      "Виктор",    "Владимир",    "Владо",    "Властимир",  "Воислав",   "Војдан",
    "Војкан",     "Волче",      "Гаврил",    "Гане",        "Георги",   "Герасим",    "Глигор",    "Гого",
    "Горан",      "Горјан",     "Гоце",      "Григор",      "Гроздан",  "Давид",      "Дамјан",    "Данаил",
    "Даниел",     "Данчо",      "Дарко",     "Дејан",       "Делјан",   "Делчо",      "Диме",      "Димитар",
    "Димитрие",   "Димитриј",   "Димитрија", "Димко",       "Димо",     "Димче",      "Добри",     "Дојчин",
    "Доротеј",    "Драган",     "Драги",     "Драгиша",     "Драгољуб", "Драгомир",   "Драгослав", "Душан",
    "Душко",      "Ѓоко",       "Ѓорѓи",     "Ѓоре",        "Ѓорче",    "Ѓурчин",     "Евгениј",   "Евтим",
    "Емануел",    "Емил",       "Еразмо",    "Ефтим",       "Живко",    "Зафир",      "Захариј",   "Здравко",
    "Златан",     "Злате",      "Златко",    "Зоран",       "Ѕвездан",  "Ѕвонко",     "Иван",      "Ивко",
    "Иво",        "Игнат",      "Игор",      "Илија",       "Исак",     "Јаков",      "Јане",      "Јоаким",
    "Јован",      "Јованче",    "Јовица",    "Јово",        "Јозо",     "Јонче",      "Јордан",    "Јосиф",
    "Кирил",      "Киро",       "Климент",   "Козма",       "Коле",     "Константин", "Костадин",  "Коце",
    "Кочо",       "Крсте",      "Крсто",     "Круме",       "Ксенте",   "Кузман",     "Лазар",     "Лазо",
    "Леонид",     "Лука",       "Љубе",      "Љубен",       "Љубиша",   "Љубомир",    "Љупчо",     "Македон",
    "Максим",     "Марин",      "Марјан",    "Марко",       "Мате",     "Матеја",     "Мето",      "Методи",
    "Методиј",    "Методија",   "Милан",     "Миле",        "Милош",    "Милчо",      "Минчо",     "Миодраг",
    "Мирко",      "Мирослав",   "Мирче",     "Митре",       "Михаил",   "Михајло",    "Михо",      "Мичо",
    "Младен",     "Момо",       "Момчило",   "Нанче",       "Наум",     "Небојша",    "Неделко",   "Ненад",
    "Неофит",     "Нестор",     "Нешо",      "Никита",      "Никифор",  "Никодим",    "Никола",    "Огнен",
    "Оливер",     "Орде",       "Павле",     "Пантелејмон", "Панче",    "Партениј",   "Паскал",    "Пенчо",
    "Перо",       "Петар",      "Петко",     "Петре",       "Пеце",     "Пимен",      "Прокопиј",  "Прохор",
    "Раде",       "Радован",    "Рајко",     "Ранко",       "Ризо",     "Ристе",      "Ристо",     "Ромил",
    "Саве",       "Саво",       "Самоил",    "Самуил",      "Санде",    "Сашко",      "Силјан",    "Симон",
    "Славе",      "Славко",     "Славчо",    "Слободан",    "Сократ",   "Сотир",      "Софе",      "Софрониј",
    "Спасе",      "Спиро",      "Србо",      "Сребре",      "Стале",    "Стево",      "Стефан",    "Стоилко",
    "Стојан",     "Стојмен",    "Стојче",    "Страте",      "Страхил",  "Танас",      "Ташко",     "Ташо",
    "Темелко",    "Теодор",     "Теодосиј",  "Теофан",      "Теофил",   "Тихо",       "Тихомир",   "Тоде",
    "Тодор",      "Томе",       "Томислав",  "Трајан",      "Трајко",   "Трајче",     "Трифун",    "Тришо",
    "Филип",      "Харалампие", "Христијан", "Христо",      "Цветан",   "Цветко",     "Цеко",      "Часлав",
    "Чедо",       "Чедомир",    "Шишман",
});

const auto macedonianFemaleFirstNames = std::to_array<std::string_view>({
    "Ава",       "Александра", "Ана",       "Анастасија", "Анѓа",       "Ангела",      "Ангелина",  "Анче",
    "Атанасија", "Билјана",    "Бисера",    "Бистра",     "Блага",      "Благица",     "Благојка",  "Благородна",
    "Благуна",   "Блаженка",   "Богдана",   "Богиња",     "Божана",     "Бојана",      "Борјана",   "Борка",
    "Босилка",   "Бранка",     "Валентина", "Вања",       "Варвара",    "Василка",     "Василија",  "Васка",
    "Велика",    "Вера",       "Вероника",  "Весела",     "Весна",      "Виолета",     "Војданка",  "Воскресија",
    "Габриела",  "Галаба",     "Галена",    "Голубинка",  "Гордана",    "Горица",      "Горјана",   "Грозда",
    "Дамјана",   "Даница",     "Данка",     "Даринка",    "Дафина",     "Деница",      "Десанка",   "Деспина",
    "Дивна",     "Добрила",    "Добринка",  "Домника",    "Донка",      "Драга",       "Драгана",   "Душанка",
    "Душка",     "Ѓурѓа",      "Ѓурѓица",   "Ева",        "Евгенија",   "Евдокија",    "Екатерина", "Елена",
    "Елеонора",  "Елисавета",  "Елица",     "Елка",       "Есма",       "Ефимија",     "Ефросина",  "Живка",
    "Загорка",   "Зафирка",    "Захаринка", "Здравка",    "Злата",      "Зора",        "Зорица",    "Зорка",
    "Ѕвезда",    "Ива",        "Ивана",     "Иванка",     "Илинденка",  "Илинка",      "Ирина",     "Искра",
    "Јаворка",   "Јагода",     "Јадранка",  "Јана",       "Јанка",      "Јасна",       "Јована",    "Јовка",
    "Јордана",   "Јорданка",   "Јулијана",  "Калина",     "Каранфилка", "Ката",        "Катерина",  "Катина",
    "Катица",    "Катја",      "Кераца",    "Кита",       "Климентина", "Константина", "Косара",    "Костадинка",
    "Крстана",   "Лена",       "Ленче",     "Лепа",       "Лидија",     "Лика",        "Лилјана",   "Лина",
    "Љуба",      "Љубинка",    "Љубица",    "Љупка",      "Магдалена",  "Македонка",   "Макрина",   "Мара",
    "Марија",    "Марика",     "Марина",    "Марта",      "Матрона",    "Меглена",     "Меланија",  "Мена",
    "Мери",      "Милева",     "Милена",    "Милица",     "Милка",      "Мирјана",     "Мирка",     "Нада",
    "Надежда",   "Наумка",     "Невенка",   "Неда",       "Неделка",    "Никодина",    "Николина",  "Олга",
    "Оливера",   "Павлина",    "Павлинка",  "Параскева",  "Пауна",      "Пелагија",    "Петкана",   "Петра",
    "Петранка",  "Писана",     "Радмила",   "Рајна",      "Ракита",     "Рафаила",     "Рахела",    "Рахилка",
    "Ристана",   "Родна",      "Роса",      "Роска",      "Ружа",       "Румена",      "Русалина",  "Русанка",
    "Сава",      "Сандра",     "Сара",      "Сашка",      "Светлана",   "Серафима",    "Симјанка",  "Симона",
    "Славенка",  "Славица",    "Славјанка", "Славка",     "Слободанка", "Снежана",     "Сончица",   "Софија",
    "Спаса",     "Спасена",    "Споменка",  "Сребра",     "Стамена",    "Стефанија",   "Стефанка",  "Стојанка",
    "Стојна",    "Стојмена",   "Сузана",    "Тамара",     "Таска",      "Татјана",     "Темјана",   "Теодора",
    "Теодосија", "Теофилка",   "Тодорка",   "Трајанка",   "Трена",      "Трендафилка", "Убавка",    "Фана",
    "Фанка",     "Филимена",   "Флоринка",  "Фросина",    "Христина",   "Цара",        "Цвета",     "Цветанка",
});

const auto macedonianMaleLastNames = std::to_array<std::string_view>({
    "Абаџиев",       "Аврамов",      "Аврамовски",   "Аврамчев",        "Ајановски",    "Алексиев",
    "Алексов",       "Алексовски",   "Ангеловски",   "Андов",           "Андонов",      "Андоновски",
    "Андреевски",    "Антонов",      "Анчевски",     "Апостолов",       "Апостолски",   "Арсов",
    "Арсовски",      "Атанасов",     "Атанасовски",  "Атков",           "Ацев",         "Ацевски",
    "Аџимитревски",  "Бабамовски",   "Бабунски",     "Бавтировски",     "Бајалски",     "Бакевски",
    "Балабанов",     "Басаровски",   "Бачовски",     "Беличанец",       "Белковски",    "Беловски",
    "Белогаски",     "Белчев",       "Бендевски",    "Билбиловски",     "Билјановски",  "Бислимовски",
    "Битовски",      "Блажевски",    "Близнаков",    "Богданов",        "Богдановски",  "Богдански",
    "Богоев",        "Богоески",     "Божиков",      "Божинов",         "Божиновски",   "Бојановски",
    "Бојаџи",        "Бојаџиев",     "Боцевски",     "Бошевски",        "Бошков",       "Бошковски",
    "Брезоски",      "Бубуловски",   "Бужаровски",   "Бузев",           "Бујуклиев",    "Бунев",
    "Варсаковски",   "Василевски",   "Василески",    "Васков",          "Величковски",  "Велјанов",
    "Велков",        "Висулчев",     "Владимирски",  "Владов",          "Влахов",       "Влахчев",
    "Вражалски",     "Гаврилски",    "Гајдов",       "Галевски",        "Гапо",         "Гаревски",
    "Генов",         "Георгиев",     "Георгиевски",  "Герасимовски",    "Гермов",       "Геров",
    "Гечевски",      "Гешоски",      "Гигов",        "Гиевски",         "Главаш",       "Глигоров",
    "Глигоровски",   "Голабовски",   "Грабул",       "Грабулоски",      "Грозданов",    "Грошев",
    "Грујовски",     "Групче",       "Гугувчевски",  "Гучев",           "Давидовски",   "Давков",
    "Давчев",        "Давчевски",    "Дамески",      "Дамовски",        "Дворчанец",    "Дејанов",
    "Демниев",       "Деспотовски",  "Дилев",        "Димитриев",       "Димитров",     "Димковски",
    "Димов",         "Димовски",     "Динев",        "Диневски",        "Дирјан",       "Добревски",
    "Доброхотов",    "Додевски",     "Дојчинов",     "Доневски",        "Дракул",       "Дренков",
    "Дрнков",        "Дуганов",      "Дуковски",     "Ѓаконовски",      "Ѓелев",        "Ѓорѓевски",
    "Ѓорчев",        "Ѓузелев",      "Ѓурлуков",     "Ѓурчинов",        "Евремов",      "Едиповски",
    "Ежов",          "Ефтимов",      "Жеков",        "Железаров",       "Жерновски",    "Забазноски",
    "Зафировски",    "Здравев",      "Здравковски",  "Зографски",       "Иванов",       "Ивановски",
    "Иваноски",      "Игновски",     "Игњатовски",   "Икономов",        "Илиев",        "Илиевски",
    "Иљоски",        "Исаковски",    "Ицев",         "Јакимовски",      "Јаконов",      "Јаневски",
    "Јанчевски",     "Јачев",        "Јованов",      "Јовановски",      "Јовчевски",    "Јорданов",
    "Јордановски",   "Јосифовски",   "Јосифоски",    "Јуруков",         "Јурукоски",    "Калајџиски",
    "Калчевски",     "Камберски",    "Кантарџиев",   "Караѓозов",       "Карадак",      "Караман",
    "Каранфиловски", "Карев",        "Каров",        "Каровски",        "Картов",       "Катарџиев",
    "Кепески",       "Керамитчиев",  "Керамичиев",   "Кирјазовски",     "Киселинов",    "Китаноски",
    "Кичевски",      "Кљусев",       "Кодра",        "Колишевски",      "Колозов",      "Кондов",
    "Кондовски",     "Конески",      "Константинов", "Константиновски", "Кордалов",     "Корубин",
    "Костадинов",    "Костовски",    "Котевски",     "Котески",         "Коцаре",       "Коцо",
    "Кочовски",      "Коџоман",      "Крле",         "Крстев",          "Крстевски",    "Крстески",
    "Крцковски",     "Кузмановски",  "Кулишев",      "Куновски",        "Куноски",      "Кушевски",
    "Ладински",      "Лазаревски",   "Лазаров",      "Лазески",         "Лазовски",     "Лакински",
    "Лактински",     "Лапе",         "Лахтов",       "Левенски",        "Лековски",     "Леов",
    "Лечевски",      "Липов",        "Личеноски",    "Лозановски",      "Луловски",     "Љутков",
    "Мазев",         "Мајсторов",    "Македонски",   "Малевски",        "Малески",      "Маловски",
    "Манаковски",    "Мангов",       "Манговски",    "Манев",           "Маневски",     "Манивилов",
    "Манчев",        "Манчевски",    "Маринов",      "Марковски",       "Мартиновски",  "Мартиноски",
    "Масин",         "Матевски",     "Матковски",    "Матовски",        "Маџиров",      "Миладинов",
    "Милосавлев",    "Милошевски",   "Милчин",       "Миљовски",        "Минчев",       "Миовски",
    "Миронски",      "Мисирков",     "Митевски",     "Митревски",       "Митриќески",   "Михајлов",
    "Михајловски",   "Мицев",        "Мицевски",     "Мицковски",       "Мицов",        "Мишковски",
    "Младенов",      "Младеновски",  "Мојсовски",    "Мокров",          "Момировски",   "Мошин",
    "Мукаетов",      "Муратовски",   "Мусалевски",   "Мустачки",        "Најденов",     "Најдовски",
    "Најчески",      "Наков",        "Наневски",     "Настев",          "Наумов",       "Наумовски",
    "Наумчески",     "Начев",        "Неделковски",  "Нестор",          "Несторовски",  "Нетков",
    "Нешковски",     "Николески",    "Николов",      "Николовски",      "Никушев",      "Нинов",
    "Оровчанец",     "Ортаков",      "Османли",      "Павловски",       "Павлоски",     "Пајтонџиев",
    "Паковски",      "Паликрушев",   "Панговски",    "Пандески",        "Пандилов",     "Пандовски",
    "Панков",        "Панов",        "Пановски",     "Панчевски",       "Папрадишки",   "Пејов",
    "Пендовски",     "Пенушлиски",   "Пепељуговски", "Перински",        "Перчинковски", "Петковски",
    "Петров",        "Петровски",    "Петрушев",     "Петрушевски",     "Пиперковски",  "Планински",
    "Подгорец",      "Полежиноски",  "Полјански",    "Попов",           "Поповски",     "Попоски",
    "Преспански",    "Прилепчански", "Прличко",      "Псалтиров",       "Пуцков",       "Равановски",
    "Радически",     "Раковски",     "Рацин",        "Ризов",           "Ристов",       "Ристовски",
    "Ристоски",      "Руменов",      "Русјаков",     "Русоманов",       "Салџиев",      "Санев",
    "Сариевски",     "Светиев",      "Серафимов",    "Серафимовски",    "Сидовски",     "Силјановски",
    "Симов",         "Симовски",     "Симоновски",   "Скаловски",       "Славенски",    "Славески",
    "Сланев",        "Смаќоски",     "Смилевски",    "Смокварски",      "Соколов",      "Солев",
    "Солунски",      "Софрониевски", "Спасеновски",  "Спасов",          "Спасовски",    "Спиров",
    "Спировски",     "Ставрев",      "Ставрески",    "Стаматоски",      "Станков",      "Станковски",
    "Старделов",     "Стеванов",     "Стевковски",   "Стефанов",        "Стефановски",  "Стојанов",
    "Стојановски",   "Стојаноски",   "Стојков",      "Стојковски",      "Стојменски",   "Стојчев",
    "Такев",         "Таковски",     "Талевски",     "Талески",         "Темелкоски",   "Теодосиевски",
    "Терзиев",       "Тодоров",      "Тодоровски",   "Тозија",          "Толевски",     "Томовски",
    "Тофоски",       "Точко",        "Тошев",        "Тошевски",        "Трајковски",   "Тренески",
    "Тренчевски",    "Трифуновски",  "Тројачанец",   "Трпеновски",      "Тушевски",     "Ќамилов",
    "Ќорнаков",      "Ќортошев",     "Ќурчиев",      "Угриновски",      "Узунов",       "Узуновски",
    "Урдин",         "Филиповски",   "Фирфов",       "Фотев",           "Хаџиманов",    "Хаџиниколов",
    "Христов",       "Христовски",   "Цанев",        "Цветановски",     "Цеков",        "Цековски",
    "Ценев",         "Црвенковски",  "Црвенов",      "Чакар",           "Чакмаков",     "Чаловски",
    "Чапаровски",    "Чаушев",       "Чашуле",       "Чевревски",       "Чејковски",    "Чемерски",
    "Чемков",        "Чипан",        "Чокревски",    "Чундев",          "Чупона",       "Чучков",
    "Чушкаров",      "Џамбаз",       "Џамбазов",     "Џаџев",           "Џеков",        "Џинлески",
    "Џорлев",        "Џувалековски", "Џунов",        "Шаревски",        "Шијак",        "Шијаков",
    "Шијаковски",    "Ширилов",      "Шишков",       "Шопов",           "Шоптрајанов",  "Штерјоски",
    "Шулинчевски",
});

const auto macedonianFemaleLastNames = std::to_array<std::string_view>({
    "Абаџиева",     "Абова",        "Аврамова",      "Аврамовска",      "Аврамчева",    "Ајановска",    "Алексиева",
    "Алексова",     "Алексовска",   "Ангеловска",    "Андова",          "Андонова",     "Андоновска",   "Андреевска",
    "Антонова",     "Анчевска",     "Апостолова",    "Апостолска",      "Арсова",       "Арсовска",     "Атанасова",
    "Атанасовска",  "Аткова",       "Ацева",         "Ацевска",         "Аџимитревска", "Бабамовска",   "Бабунска",
    "Бавтировска",  "Бајалска",     "Бакевска",      "Балабанова",      "Басаровска",   "Бачовска",     "Беличанец",
    "Белковска",    "Беловска",     "Белогаска",     "Белчева",         "Бендевска",    "Билбиловска",  "Билјановска",
    "Бислимовска",  "Битовска",     "Блажевска",     "Близнакова",      "Богданова",    "Богдановска",  "Богданска",
    "Богоева",      "Богоеска",     "Божикова",      "Божинова",        "Божиновска",   "Бојановска",   "Бојаџи",
    "Бојаџиева",    "Боцевска",     "Бошевска",      "Бошкова",         "Бошковска",    "Брезоска",     "Бубуловска",
    "Бужаровска",   "Бузева",       "Бујуклиева",    "Бунева",          "Варсаковска",  "Василевска",   "Василеска",
    "Васкова",      "Величковска",  "Велјанова",     "Велкова",         "Висулчева",    "Владимирска",  "Владова",
    "Влахова",      "Влахчева",     "Вражалска",     "Гаврилска",       "Гајдова",      "Галевска",     "Гапо",
    "Гаревска",     "Генова",       "Георгиева",     "Георгиевска",     "Герасимовска", "Гермова",      "Герова",
    "Гечевска",     "Гешоска",      "Гигова",        "Гиевска",         "Главаш",       "Глигорова",    "Глигоровска",
    "Голабовска",   "Грабул",       "Грабулоска",    "Грозданова",      "Грошева",      "Грујовска",    "Групче",
    "Гугувчевска",  "Гучева",       "Давидовска",    "Давкова",         "Давчева",      "Давчевска",    "Дамеска",
    "Дамовска",     "Дворчанец",    "Дејанова",      "Демниева",        "Деспотовска",  "Дилева",       "Димитриева",
    "Димитрова",    "Димковска",    "Димова",        "Димовска",        "Динева",       "Диневска",     "Добревска",
    "Додевска",     "Дојчинова",    "Доневска",      "Дракул",          "Дренкова",     "Дрнкова",      "Дуганова",
    "Дуковска",     "Ѓаконовска",   "Ѓелева",        "Ѓорѓевска",       "Ѓорчева",      "Ѓузелева",     "Ѓурлукова",
    "Ѓурчинова",    "Евремова",     "Едиповска",     "Ежова",           "Ефтимова",     "Жекова",       "Железарова",
    "Жерновска",    "Забазноска",   "Зафировска",    "Здравева",        "Здравковска",  "Зографска",    "Иванова",
    "Ивановска",    "Иваноска",     "Игновска",      "Игњатовска",      "Икономова",    "Илиева",       "Илиевска",
    "Иљоска",       "Исаковска",    "Ицева",         "Јакимовска",      "Јаконова",     "Јаневска",     "Јанчевска",
    "Јачева",       "Јованова",     "Јовановска",    "Јовчевска",       "Јорданова",    "Јордановска",  "Јосифовска",
    "Јосифоска",    "Јурукова",     "Јурукоска",     "Калајџиска",      "Калчевска",    "Камберска",    "Кантарџиева",
    "Караѓозова",   "Карадак",      "Караман",       "Каранфиловска",   "Караџа",       "Карева",       "Карова",
    "Каровска",     "Картова",      "Катарџиева",    "Кепеска",         "Керамитчиева", "Керамичиева",  "Кирјазовска",
    "Киселинова",   "Китаноска",    "Кичевска",      "Кљусева",         "Колишевска",   "Колозова",     "Кондова",
    "Кондовска",    "Конеска",      "Константинова", "Константиновска", "Кордалова",    "Коробар",      "Корубин",
    "Костадинова",  "Костовска",    "Котевска",      "Котеска",         "Коцаре",       "Коцо",         "Кочовска",
    "Коџоман",      "Крле",         "Крстева",       "Крстевска",       "Крстеска",     "Крцковска",    "Кузмановска",
    "Кулишева",     "Куновска",     "Куноска",       "Кутурец",         "Кушевска",     "Ладинска",     "Лазаревска",
    "Лазарова",     "Лазеска",      "Лазовска",      "Лакинска",        "Лактинска",    "Лапе",         "Лахтова",
    "Левенска",     "Лековска",     "Леова",         "Лечевска",        "Липова",       "Личеноска",    "Лозановска",
    "Луловска",     "Љуткова",      "Мазева",        "Мајсторова",      "Македонска",   "Малевска",     "Малеска",
    "Маловска",     "Манаковска",   "Мангова",       "Манговска",       "Манева",       "Маневска",     "Манивилова",
    "Манчева",      "Манчевска",    "Маринова",      "Марковска",       "Мартиновска",  "Мартиноска",   "Масин",
    "Матевска",     "Матковска",    "Матовска",      "Маџирова",        "Миладинова",   "Милосавлева",  "Милошевска",
    "Милчин",       "Миљовска",     "Минчева",       "Миовска",         "Миронска",     "Мисиркова",    "Митевска",
    "Митревска",    "Митриќеска",   "Михајлова",     "Михајловска",     "Мицева",       "Мицевска",     "Мицковска",
    "Мицова",       "Мишковска",    "Младенова",     "Младеновска",     "Мојсовска",    "Мокрова",      "Момировска",
    "Мошин",        "Мукаетова",    "Муратовска",    "Мусалевска",      "Мустачки",     "Најденова",    "Најдовска",
    "Најческа",     "Накова",       "Наневска",      "Настева",         "Наумова",      "Наумовска",    "Наумческа",
    "Начева",       "Неделковска",  "Нестор",        "Несторовска",     "Неткова",      "Нешковска",    "Николеска",
    "Николова",     "Николовска",   "Никушева",      "Нинова",          "Оровчанец",    "Ортакова",     "Османли",
    "Павловска",    "Павлоска",     "Пајтонџиева",   "Паковска",        "Паликрушева",  "Панговска",    "Пандеска",
    "Пандилова",    "Пандовска",    "Панкова",       "Панова",          "Пановска",     "Панчевска",    "Папрадишки",
    "Пејова",       "Пендовска",    "Пенушлиска",    "Пепељуговска",    "Перинска",     "Перчинковска", "Петковска",
    "Петрова",      "Петровска",    "Петрушева",     "Петрушевска",     "Пиперковска",  "Планинска",    "Подгорец",
    "Полежиноска",  "Полјанска",    "Попова",        "Поповска",        "Попоска",      "Преспанска",   "Прилепчанска",
    "Прличко",      "Псалтирова",   "Пуцкова",       "Равановска",      "Радическа",    "Раковска",     "Рацин",
    "Реџепова",     "Ризова",       "Ристова",       "Ристовска",       "Ристоска",     "Руменова",     "Русјакова",
    "Русоманова",   "Салџиева",     "Санева",        "Сариевска",       "Светиева",     "Серафимова",   "Серафимовска",
    "Сидовска",     "Силјановска",  "Симова",        "Симовска",        "Симоновска",   "Скаловска",    "Славенска",
    "Славеска",     "Сланева",      "Смаќоска",      "Смилевска",       "Смокварска",   "Соколова",     "Солева",
    "Солунска",     "Софрониевска", "Спасеновска",   "Спасова",         "Спасовска",    "Спирова",      "Спировска",
    "Ставрева",     "Ставреска",    "Стаматоска",    "Станкова",        "Станковска",   "Старделова",   "Стеванова",
    "Стевковска",   "Стефанова",    "Стефановска",   "Стојанова",       "Стојановска",  "Стојаноска",   "Стојкова",
    "Стојковска",   "Стојменска",   "Стојчева",      "Такева",          "Таковска",     "Талевска",     "Талеска",
    "Темелкоска",   "Теодосиевска", "Терзиева",      "Тодорова",        "Тодоровска",   "Тозија",       "Толевска",
    "Томовска",     "Тофоска",      "Точко",         "Тошева",          "Тошевска",     "Трајковска",   "Тренеска",
    "Тренчевска",   "Трифуновска",  "Тројачанец",    "Трпеновска",      "Тушевска",     "Ќамилова",     "Ќорнакова",
    "Ќортошева",    "Ќурчиева",     "Угриновска",    "Узунова",         "Узуновска",    "Урдин",        "Филиповска",
    "Фирфова",      "Фотева",       "Хаџиманова",    "Хаџиниколова",    "Христова",     "Христовска",   "Цанева",
    "Цветановска",  "Цекова",       "Цековска",      "Ценева",          "Црвенковска",  "Црвенова",     "Чакар",
    "Чакмакова",    "Чаловска",     "Чапаровска",    "Чаушева",         "Чашуле",       "Чевревска",    "Чејковска",
    "Чемерска",     "Чемкова",      "Чипан",         "Чокревска",       "Чундева",      "Чупона",       "Чучкова",
    "Чушкарова",    "Џамбаз",       "Џамбазова",     "Џаџева",          "Џекова",       "Џинлеска",     "Џорлева",
    "Џувалековска", "Џунова",       "Шаревска",      "Шијак",           "Шијакова",     "Шијаковска",   "Ширилова",
    "Шишкова",      "Шопова",       "Шоптрајанова",  "Штерјоска",       "Шулинчевска",
});

const auto macedonianMalePrefixes = std::to_array<std::string_view>({"г-дин", "д-р", "м-р"});

const auto macedonianFemalePrefixes = std::to_array<std::string_view>({"г-ѓа", "г-ца", "д-р", "м-р"});

const NameFormats macedonianNameFormats{{{"{firstName} {lastName}", 10}, {"{prefix} {firstName} {lastName}", 1}}};

const PeopleNames macedonianPeopleNames{
    {macedonianMaleFirstNames, macedonianMaleLastNames, macedonianMalePrefixes, {}},
    {macedonianFemaleFirstNames, macedonianFemaleLastNames, macedonianFemalePrefixes, {}},
    macedonianNameFormats};

// Maldives

const auto maldiviansMaleFirstNames = std::to_array<std::string_view>({
    "އާދަމް",
    "އިބްރާހީމް",
    "އިލްޔާސް",
    "އިސްމާޢީލް",
    "ޢީސާ",
    "އަބޫބަކްރު",
    "އަލީ",
    "އުސާމާ",
    "ދާއޫދު",
    "ޢަޠާ",
    "ޤުދާސް",
    "މުޙައްމަދު",
    "މޫސާ",
    "ޔޫސުފް",
});

const auto maldiviansFemaleFirstNames = std::to_array<std::string_view>({
    "ރަމްލާ",     "ހިންދު", "ޙަފްޞާ", "ޚަދީޖާ",  "ޒައިނަބު",  "ޞަފިއްޔާ", "ޢާއިޝާ", "މައިމޫނާ", "ޖުވައިރިއްޔާ", "ސައުދާ",  "މާރިޔާ",  "ރުޤައްޔާ", "ފާޠިމާ",
    "އުއްމުކުލްޘޫމޮ", "ޙައްވާ", "ސާރާ",  "ހާޖަރު",  "މަރްޔަމޮ",  "ޒުލައިޚާ", "އާސިޔާ", "ބަލްޤީސް", "އާމިނަތު",   "އަސްމާއު", "އުމާމާ",  "ބަރްކާ",  "ޖަމީލާ",
    "ޙަސްނާއު",    "ޙަލީމާ", "ޚަވްލާ", "ޚައިރާ",  "ރުމައިޞާއު", "ރުފައިދާ", "ރަޤީޤާ", "ސަޢާދު",  "ސަލްމާ",    "ސުލައިމް", "ސުމައްޔާ", "ޝަހީދާ",  "ޝިފާ",
    "ޢާތިކާ",     "ޤައިލާ", "ލަޔާލީ", "ކުލްޘޫމް", "މުސްލިމާ",  "ނަސީބާ",  "ނަފީސާ", "ނަވާރު",  "ހާނީ",     "ހުނައިދާ",
});

const auto maldiviansMaleLastNames = std::to_array<std::string_view>({
    "އަކްޙަލް", "އަޘްހަރު", "ބުރްހާން", "ބާޤީ",   "ބުހްލޫލް", "ބަޝޫޝް",  "ތަރަފް",  "ތައްމާމް",  "ތުކްލާން",  "ޘަވިއްޔު", "ޘާޣިމް",  "ޘަރިއްޔު", "ޖާބިރު",  "ޖާސިމް",
    "ޖަޛަލް",  "ޖުސުމް",  "ޖުންދު",  "ޙާބިސް",  "ޙާފިދު",  "ޙަބީބު",  "ޙަޘީޘް",  "ޙުސްނީ",   "ޚާބިސް",   "ޚާފިޤާނި", "ޚިތާމް",  "ޚައިރީ",  "ދާރިބު",  "ދައްރާކު",
    "ދިރްފިސް", "ދީވާން",  "ޛަހަބު",  "ޛުއްނޫރު", "ޛަކީ",   "ރާބިޙް",  "ރައުބު",  "ރަޙޫމް",   "ރޫދު",    "ޒިބްރިޖު", "ޒިރުޔާބު", "ޒޯފިޝާން", "ސާޤީ",   "ސިބުޠު",
    "ސަރުމަދު", "ސާޖީ",   "ޝާމިސް",  "ޝަބާބު",  "ޝުބޫލް",  "ޝަފްޝާފް", "ޝަލޫލް",  "ޞަބޫރު",   "ޞަދޫޙު",   "ޞަފިއްޔު", "ޞުރޫޙު",  "ޠާޙީ",   "ޠާލިބު",  "ޠިބާބު",
    "ޡަފަރު",  "ޡިލާލް",  "ޡުހޫރު",  "ޢާބިދީން", "ޢަބްޤަރީ", "ޢިއްޒަތު", "ޢައްމާރު", "ޢަންބަރު",  "ޢަންދަލީބު", "ޣާދިފު",  "ޣައްރާފް", "ޣައްސާން", "ޣުމޫމް",  "ޣަންދޫރު",
    "ޣަޔްދަން", "ފާއިޤު",  "ފަތޫޙް",  "ފަހުމާން", "ފައިލަޤު", "ޤާބޫސް",  "ޤިންދީލް", "ޤިންޠާރް",  "ޤަވާމް",   "ކަލަޖް",  "ކައުޘަރު", "ކައިސޫމް", "ލުބަދު",  "ލަބިޤު",
    "ލަސަން",  "ލުޠުފީ",  "ލުޤުމާން", "މައުރަބު", "މައުމަލް", "މައާބު",  "މަބްޞަރު", "މުތަނާސިޤް", "މުދުރިކަތު", "ނުޖުދު",  "ނުސައިބު", "ނުޢައިމް", "ނައްބާރު", "ނުހައިލް",
    "ވާބިޞް",  "ވަދީސް",  "ވާފިރު",  "ވައްޟާޙް", "ވަހްބު",  "ހާދީ",   "ހިދާޔަތް", "ހުޛުލޫލް",  "ހައްފާފް",  "ހިންދިސް", "ޔާޤޫތު",  "ޔަލްމަޢި", "ޔޫޝަޢު",  "ޔަގްޡާން",
});

const auto maldiviansFemaleLastNames = std::to_array<std::string_view>({
    "އިބާ",    "އަޘްމާރު",  "އަޘީލާ",   "އިރާދާ",   "އިރުތިޖާޢު", "އަރުވާ",     "އުމުނިއްޔާ", "އަންބާ",   "އަންޖަލް",  "އުމަރާ",   "ބަދުރިއްޔާ", "ބަރުޒާ",
    "ބަޝޫޝޤް",  "ބަތްނާ",   "ބަޝާޝާ",   "ބާޛިލާ",   "ބަތޫލް",   "ބަރުޢުމް",    "ބަޠްޙާއު",  "ބަހިއްޔު",  "ތާއްމާ",   "ތަބައްރުޢު", "ތަޙައްޝުމު", "ތަޛްހީބު",
    "ތަޒައްޔުން", "ތަޣްރީދު",  "ތިމްރާޙަތު", "ޘަބްރާ",   "ޘަރާ",    "ޘުރައްޔާ",    "ޘަވާ",    "ޘާނީ",    "ޖާސިމާ",   "ޖިދާ",    "ޖަސާރާ",   "ޖަލްވާ",
    "ޖުމްލާނާ",  "ޖުނައިނާ",  "ޖައިދާނާ",  "ޖުމާއު",   "ޙާރިސާ",   "ޙިބާއު",     "ޙަބީބާ",   "ޙަދްވާ",   "ޙަޝްމާ",   "ޙަޞްނާ",   "ޙަމްރާއު",  "ޙިންނާ",
    "ޙަނާން",   "ޚާލިޞާ",   "ޚަދްރާ",   "ޚުލަސާ",   "ޚަވްޟާ",   "ޚަވްލީ",     "ޚަވްދު",   "ދާއިޙާ",   "ދާޖިނާ",   "ދައުމާއު",  "ދުއްރުން",  "ދަރްޤާ",
    "ދަފީނާ",   "ދަލޫފް",   "ދާނިޔާ",   "ޛާބިރާ",   "ޛަބާރާ",   "ޛަރާ",      "ޛިކްރާ",   "ޛަމާމަތު",  "ރުއުޔާ",   "ރާބިޙާ",   "ރާސިޙާ",   "ރާފިޢާ",
    "ރާމާ",    "ރަބާބާ",   "ރަޖާ",    "ރަޙޫލް",   "ރައްސާމާ",  "ރުޝްދިއްޔާ",   "ރަފާހިއްޔާ", "ރުފައިދާ",  "ރައުޙިއްޔާ", "ރަވްނަޤް",  "ޒާޚިރާ",   "ޒާހް",
    "ޒަބަރުޖަދް", "ޒަޖާ",    "ޒުރްޤުމް",  "ޒަރްނަޤާ",  "ޒުމުއްރުދާ", "ޒާޚިޒުންބުޤާރާ", "ޒުންބުޤާ",  "ޒައިޒަފޫން", "ޒިދުނީ",   "ރާޖިޔާ",   "ރަޖާޙާ",   "ރިދާފާ",
    "ރިޢާ",    "ރަކީނާ",   "ރުމައިޒާ",  "ރަހީނާ",   "ޒާރާ",    "ޒުބައިދާ",    "ޒަރްކަޝާ",  "ޒަޢުފަރާނާ", "ސުއުދާ",   "ސާބިޤާ",   "ސާމިޤާ",   "ސުބުޙާތު",
    "ސަޙަރު",   "ސަޢުދިއްޔާ", "ސަލްސަބީލް", "ސުމައިޙާ",  "ސުންދުސް",  "ސަނަމް",     "ޝާއިޤާ",   "ޝާރިޔާ",   "ޝަބާބު",   "ޝަޖިއްޔާ",  "ޝަރްސާއު",  "ޝަރްޤިއްޔާ",
    "ޝުޢާޢު",   "ޝަފްޝާފް",  "ޝަލަބިއްޔާ", "ޝަމަރުދަލް", "ޝަމްސާ",   "ޝިންޝިނާ",    "ޞާއިބާ",   "ޞާފީނާޒް",  "ޞަބިއްޔާ",  "ޞުބައިޙާ",  "ޞިއްދީޤާ",  "ޞިންވާން",
    "ޟާޙިޔާ",   "ޟާމިނަތު",  "ޟުޙާ",    "ޟިފާފް",   "ޟިޔާ",    "ތާއިޢާ",     "ޠާހިރާ",   "ޠަރޫބު",   "ޠުލްސާ",   "ޠުމޫޙާ",   "ޠައްޔިބާ",  "ޡާފިރާ",
    "ޡަމްޔާ",   "ޡުއްލާ",   "ޡަހީރާ",   "ޢާއިދާ",   "ޢާޖިލާ",   "ޢާޤިބާ",     "ޢާލިޔާ",   "ޢަބީރު",   "ޢުތުބާ",   "ޢަދުލިއްޔާ", "ޢަޛްރާ",   "ޢަރޫޝާ",
    "ޢައްޒާ",   "ޢަޠާޔާ",   "ޢަފާފާ",   "ޢަލާނިޔާ",  "ޢަންނާބު",  "ޢަންބަރާ",    "ޢަންދަލީބު", "ޢަލާނާ",   "ޣާދާ",    "ޣާނީ",    "ޣާޔާ",    "ޣިނާ",
    "ޣީދު",    "ޣީރާ",    "ފާއިޒާ",   "ފާރީހާން",  "ފަތުޙިއްޔާ", "ފަރާއިދު",    "ފިރްޔާލް",  "ފުރައިސާ",  "ފިޠްރިއްޔާ", "ފުނޫން",   "ފުހައިދާ",  "ފައްވާޙާ",
    "ފައުޤިއްޔާ", "ފައިނާން",  "ޤާނިތު",   "ޤަބްސާ",   "ޤުޘަމް",   "ޤުދުވާ",     "ޤިޠާފް",   "ޤުމްރިއްޔާ", "ޤަމީން",   "ކާއިނާ",   "ކާސިބާ",   "ކާޙިލާ",
    "ކާފިޔާ",   "ކަބީރާ",   "ކަރާއިމް",  "ކުރައިމާން", "ކިފާޔާ",   "ކުލްޘޫމް",    "ކިނާނާ",   "ކަންޒު",   "ލުއުލުއު",  "ލުބާބާ",   "ލުއްޖާ",   "ލަސަން",
    "ލުބާ",    "ލިޔާނާ",   "މުއުމިނާ",  "މުއުނިސާ",  "މާހާ",    "މާހީ",      "މާސާ",    "މާޝީ",    "މާވީ",    "މަދާ",    "މަރްހޫބާ",  "މުސްކާ",
    "މަލަދު",   "މުނާ",    "މީފާ",    "ނާއިފާ",   "ނާލާ",    "ނާހީ",      "ނާވީ",    "ނުބްހާ",   "ނަޖާ",    "ނަޖްމާ",   "ނުޚްބާ",   "ނަރްޖިސް",
    "ނޫރުލްހުދާ", "ވާޙާ",    "ވިޖްދާން",  "ވުސްޢާ",   "ވަޝީކު",   "ވަޞްފާ",     "ވަހިއްޔާ",  "ހާދިއާ",   "ހާނީ",    "ހިބާ",    "ހުދާ",    "ހަނާ",
    "ހިންދާމް",  "ހޫނާ",    "ހައިނަމާ",  "ޔާސިރާ",   "ޔާޤޫތު",   "ޔަޤްޡާ",     "ޔަމްނާ",   "ޔަޝްމް",
});

const auto maldiviansMalePrefixes = std::to_array<std::string_view>({"އަމީރު", "އަމީރާ", "ބަނޑޭރި", "ބޮޑު"});

const auto maldiviansFemalePrefixes = std::to_array<std::string_view>({"އަމީރު", "އަމީރާ", "ބަނޑޭރި", "ބޮޑު"});

const NameFormats maldiviansNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames maldiviansPeopleNames{
    {maldiviansMaleFirstNames, maldiviansMaleLastNames, maldiviansMalePrefixes, {}},
    {maldiviansFemaleFirstNames, maldiviansFemaleLastNames, maldiviansFemalePrefixes, {}},
    maldiviansNameFormats};

// Malta

const auto malteseMaleFirstNames = std::to_array<std::string_view>({
    "Joseph",  "John",    "Mark",     "Mario",   "David",    "Paul",      "Michael", "Anthony",  "Joe",
    "Charles", "Matthew", "Robert",   "Daniel",  "Chris",    "George",    "Andrew",  "Peter",    "Kevin",
    "Adrian",  "Stephen", "Jonathan", "James",   "Martin",   "Simon",     "Ray",     "Steven",   "Alfred",
    "Victor",  "Raymond", "Karl",     "Brian",   "Alex",     "Ivan",      "Patrick", "Tony",     "Christopher",
    "Keith",   "Edward",  "Ian",      "Jason",   "Pierre",   "Christian", "Kenneth", "Philip",   "Alan",
    "Ryan",    "Luke",    "Albert",   "Louis",   "Antoine",  "Lawrence",  "Stefan",  "Roderick", "Vincent",
    "Malcolm", "Richard", "Josef",    "Darren",  "Emanuel",  "Nicholas",  "Manuel",  "Marco",    "Alexander",
    "Frank",   "Kurt",    "Andre",    "Anton",   "Gordon",   "Thomas",    "Sandro",  "Etienne",  "Charlie",
    "Reuben",  "Aaron",   "Lino",     "Justin",  "William",  "Julian",    "Bernard", "Ronald",   "Eric",
    "Sean",    "Neville", "Silvio",   "Godwin",  "Emmanuel", "Johann",    "Neil",    "Franco",   "Clayton",
    "Edwin",   "Oliver",  "Aldo",     "Gilbert", "Jeffrey",  "Duncan",    "Jeremy",  "Glenn",    "Henry",
    "Clive",
});

const auto malteseFemaleFirstNames = std::to_array<std::string_view>({
    "Maria",     "Anna",       "Mary",     "Carmen",    "Rita",     "Noel",       "Claire",   "Sarah",    "Stephanie",
    "Daniela",   "Christine",  "Michelle", "Josephine", "Carmel",   "Jean",       "Sandra",   "Ruth",     "Joanne",
    "Elaine",    "Josette",    "Sharon",   "Charmaine", "Charlene", "Caroline",   "Mariella", "Roberta",  "Miriam",
    "Rachel",    "Francesca",  "Amanda",   "Joanna",    "Andrea",   "Marie",      "Alison",   "Pauline",  "Elizabeth",
    "Marisa",    "Jessica",    "Lara",     "Denise",    "Doris",    "Rebecca",    "Louise",   "Karen",    "Francis",
    "Catherine", "Graziella",  "Lorraine", "Vanessa",   "Doreen",   "Diane",      "Patricia", "Rose",     "Jane",
    "Monica",    "Victoria",   "Margaret", "Romina",    "Simone",   "Alexia",     "Melanie",  "Fiona",    "Lisa",
    "Audrey",    "Antoinette", "Nadia",    "Emma",      "Anne",     "Jennifer",   "Angela",   "Nathalie", "Marlene",
    "Grace",     "Tania",      "Natasha",  "Laura",     "Claudia",  "Jacqueline", "Dennis",   "Nicole",   "Joan",
    "Ann",       "Alexandra",  "Isabelle", "Antonella", "Helen",    "Janice",     "Joyce",    "Nadine",   "Natalie",
    "Sue",       "Marica",     "Tiziana",  "Christina", "Marion",   "Therese",    "Elena",    "Abigail",  "Cynthia",
    "Susan",
});

const auto malteseLastNames = std::to_array<std::string_view>({
    "Borg",     "Vella",     "Camilleri",  "Farrugia",  "Zammit",      "Galea",     "Micallef", "Grech",    "Attard",
    "Cassar",   "Spiteri",   "Azzopardi",  "Mifsud",    "Caruana",     "Muscat",    "Agius",    "Pace",     "Fenech",
    "Schembri", "Abela",     "Sammut",     "Gatt",      "Gauci",       "Debono",    "Bugeja",   "Vassallo", "Bonnici",
    "Aquilina", "Portelli",  "Calleja",    "Scicluna",  "Cutajar",     "Falzon",    "Grima",    "Mallia",   "Ellul",
    "Saliba",   "Bonello",   "Cachia",     "Mizzi",     "Xuereb",      "Buttigieg", "Scerri",   "Busuttil", "Tabone",
    "Cauchi",   "Sciberras", "Bartolo",    "Mercieca",  "Baldacchino", "Buhagiar",  "Briffa",   "Formosa",  "Brincat",
    "Zahra",    "Bezzina",   "Sultana",    "Mangion",   "Chetcuti",    "Dimech",    "Darmanin", "Pisani",   "Psaila",
    "Cilia",    "Cini",      "Debattista", "Tanti",     "Sant",        "Mamo",      "Zerafa",   "Ciantar",  "Magro",
    "Said",     "Chircop",   "Zarb",       "Barbara",   "Dalli",       "Curmi",     "Frendo",   "Tonna",    "Cuschieri",
    "Abdilla",  "Deguara",   "Magri",      "Hili",      "Cremona",     "Cardona",   "Xerri",    "Balzan",   "Degiorgio",
    "Grixti",   "Bajada",    "Catania",    "Pullicino", "Cordina",     "Callus",    "Meli",     "Ebejer",   "Anastasi",
    "Theuma",
});

const NameFormats malteseNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames maltesePeopleNames{{malteseMaleFirstNames, malteseLastNames, {}, {}},
                                     {malteseFemaleFirstNames, malteseLastNames, {}, {}},
                                     malteseNameFormats};

// Mexico

const auto mexicanMaleFirstNames = std::to_array<std::string_view>({
    "Aarón",         "Abraham",
    "Adán",          "Agustín",
    "Alan",          "Alberto",
    "Alejandro",     "Alexander",
    "Alexis",        "Alfonso",
    "Alfredo",       "Andrés",
    "Ángel Daniel",  "Ángel Gabriel",
    "Antonio",       "Armando",
    "Arturo",        "Axel",
    "Benito",        "Benjamín",
    "Bernardo",      "Brandon",
    "Brayan",        "Carlos",
    "César",         "Claudio",
    "Clemente",      "Cristian",
    "Cristobal",     "Damián",
    "Daniel",        "David",
    "Diego",         "Eduardo",
    "Elías",         "Emiliano",
    "Emilio",        "Emmanuel",
    "Enrique",       "Erick",
    "Ernesto",       "Esteban",
    "Federico",      "Felipe",
    "Fernando",      "Fernando Javier",
    "Francisco",     "Francisco Javier",
    "Gabriel",       "Gael",
    "Gerardo",       "Germán",
    "Gilberto",      "Gonzalo",
    "Gregorio",      "Guillermo",
    "Gustavo",       "Hernán",
    "Homero",        "Horacio",
    "Hugo",          "Ignacio",
    "Iker",          "Isaac",
    "Isaias",        "Israel",
    "Ivan",          "Jacobo",
    "Jaime",         "Javier",
    "Jerónimo",      "Jesús",
    "Joaquín",       "Jorge",
    "Jorge Luis",    "José",
    "José Antonio",  "Jose Daniel",
    "José Eduardo",  "José Emilio",
    "José Luis",     "José María",
    "José Miguel",   "Juan",
    "Juan Carlos",   "Juan Manuel",
    "Juan Pablo",    "Julio",
    "Julio César",   "Kevin",
    "Leonardo",      "Lorenzo",
    "Lucas",         "Luis",
    "Luis Ángel",    "Luis Fernando",
    "Luis Gabino",   "Luis Miguel",
    "Manuel",        "Marco Antonio",
    "Marcos",        "Mariano",
    "Mario",         "Martín",
    "Mateo",         "Matías",
    "Mauricio",      "Maximiliano",
    "Miguel",        "Miguel Ángel",
    "Nicolás",       "Octavio",
    "Óscar",         "Pablo",
    "Patricio",      "Pedro",
    "Rafael",        "Ramiro",
    "Ramón",         "Raúl",
    "Ricardo",       "Roberto",
    "Rodrigo",       "Rubén",
    "Salvador",      "Samuel",
    "Sancho",        "Santiago",
    "Saúl",          "Sebastian",
    "Sergio",        "Tadeo",
    "Teodoro",       "Timoteo",
    "Tomás",         "Uriel",
    "Vicente",       "Víctor",
    "Victor Manuel",
});

const auto mexicanFemaleFirstNames = std::to_array<std::string_view>({
    "Abigail",
    "Abril",
    "Adela",
    "Adriana",
    "Alejandra",
    "Alexa",
    "Alicia",
    "Alondra Romina",
    "Amalia",
    "Ana Luisa",
    "Ana María",
    "Ana Sofía",
    "Ana Victoria",
    "Ana",
    "Andrea",
    "Ángela",
    "Anita",
    "Antonia",
    "Araceli",
    "Ariadna",
    "Barbara",
    "Beatriz",
    "Berta",
    "Blanca",
    "Camila",
    "Caridad",
    "Carla",
    "Carlota",
    "Carmen",
    "Carolina",
    "Catalina",
    "Cecilia",
    "Clara",
    "Claudia",
    "Concepción",
    "Conchita",
    "Cristina",
    "Daniela",
    "Débora",
    "Diana",
    "Dolores",
    "Dorotea",
    "Dulce María",
    "Elena",
    "Elisa",
    "Elizabeth",
    "Eloisa",
    "Elsa",
    "Elvira",
    "Emilia",
    "Emily",
    "Esmeralda",
    "Esperanza",
    "Estefanía",
    "Estela",
    "Ester",
    "Eva",
    "Evelyn",
    "Fatima",
    "Florencia",
    "Francisca",
    "Gabriela",
    "Gloria",
    "Graciela",
    "Guadalupe",
    "Guillermina",
    "Inés",
    "Irene",
    "Isabel",
    "Isabela",
    "Ivanna",
    "Jazmin",
    "Jennifer",
    "Jimena",
    "Josefina",
    "Juana",
    "Julia",
    "Julieta",
    "Kimberly",
    "Laura",
    "Leonor",
    "Leticia",
    "Lilia",
    "Liliana",
    "Lizbeth",
    "Lola",
    "Lorena",
    "Lourdes",
    "Lucia",
    "Luisa",
    "Luz",
    "Magdalena",
    "Manuela",
    "Marcela",
    "Margarita",
    "María Cristina",
    "María de Jesús",
    "María de los Ángeles",
    "María del Carmen",
    "María Elena",
    "María Eugenia",
    "María Fernanda",
    "María Guadalupe",
    "María José",
    "María Luisa",
    "María Soledad",
    "María Teresa",
    "María",
    "Mariana",
    "Maricarmen",
    "Marilu",
    "Marisol",
    "Marta",
    "Mayte",
    "Melany",
    "Melissa",
    "Mercedes",
    "Micaela",
    "Miranda",
    "Mónica",
    "Monserrat",
    "Naomi",
    "Natalia",
    "Nicole",
    "Norma",
    "Olivia",
    "Paola",
    "Patricia",
    "Paulina",
    "Pilar",
    "Ramona",
    "Raquel",
    "Rebeca",
    "Regina",
    "Reina",
    "Renata",
    "Rocio",
    "Rosa María",
    "Rosa",
    "Rosalia",
    "Rosario",
    "Sara",
    "Silvia",
    "Sofia",
    "Soledad",
    "Sonia",
    "Susana",
    "Teresa",
    "Valentina",
    "Valeria",
    "Vanessa",
    "Verónica",
    "Victoria",
    "Virginia",
    "Ximena Guadalupe",
    "Ximena",
    "Xochitl",
    "Yamileth",
    "Yaretzi",
    "Yolanda",
    "Zoe",
});

const auto mexicanLastNames = std::to_array<std::string_view>({
    "Abeyta",
    "Abrego",
    "Abreu",
    "Acevedo",
    "Acosta",
    "Acuña",
    "Adame",
    "Adorno",
    "Agosto",
    "Aguayo",
    "Águilar",
    "Aguilera",
    "Aguirre",
    "Alanis",
    "Alaniz",
    "Alarcón",
    "Alba",
    "Alcala",
    "Alcántar",
    "Alcaraz",
    "Alejandro",
    "Alemán",
    "Alfaro",
    "Alicea",
    "Almanza",
    "Almaraz",
    "Almonte",
    "Alonso",
    "Alonzo",
    "Altamirano",
    "Alva",
    "Alvarado",
    "Alvarez",
    "Amador",
    "Amaya",
    "Anaya",
    "Anguiano",
    "Angulo",
    "Aparicio",
    "Apodaca",
    "Aponte",
    "Aragón",
    "Aranda",
    "Araña",
    "Arce",
    "Archuleta",
    "Arellano",
    "Arenas",
    "Arevalo",
    "Arguello",
    "Arias",
    "Armas",
    "Armendáriz",
    "Armenta",
    "Armijo",
    "Arredondo",
    "Arreola",
    "Arriaga",
    "Arroyo",
    "Arteaga",
    "Atencio",
    "Ávalos",
    "Ávila",
    "Avilés",
    "Ayala",
    "Baca",
    "Badillo",
    "Báez",
    "Baeza",
    "Bahena",
    "Balderas",
    "Ballesteros",
    "Banda",
    "Bañuelos",
    "Barajas",
    "Barela",
    "Barragán",
    "Barraza",
    "Barrera",
    "Barreto",
    "Barrientos",
    "Barrios",
    "Batista",
    "Becerra",
    "Beltrán",
    "Benavides",
    "Benavídez",
    "Benítez",
    "Bermúdez",
    "Bernal",
    "Berríos",
    "Bétancourt",
    "Blanco",
    "Bonilla",
    "Borrego",
    "Botello",
    "Bravo",
    "Briones",
    "Briseño",
    "Brito",
    "Bueno",
    "Burgos",
    "Bustamante",
    "Bustos",
    "Caballero",
    "Cabán",
    "Cabrera",
    "Cadena",
    "Caldera",
    "Calderón",
    "Calvillo",
    "Camacho",
    "Camarillo",
    "Campos",
    "Canales",
    "Candelaria",
    "Cano",
    "Cantú",
    "Caraballo",
    "Carbajal",
    "Cardenas",
    "Cardona",
    "Carmona",
    "Carranza",
    "Carrasco",
    "Carrasquillo",
    "Carreón",
    "Carrera",
    "Carrero",
    "Carrillo",
    "Carrion",
    "Carvajal",
    "Casanova",
    "Casares",
    "Casárez",
    "Casas",
    "Casillas",
    "Castañeda",
    "Castellanos",
    "Castillo",
    "Castro",
    "Cavazos",
    "Cazares",
    "Ceballos",
    "Cedillo",
    "Ceja",
    "Centeno",
    "Cepeda",
    "Cerda",
    "Cervantes",
    "Cervántez",
    "Chacón",
    "Chapa",
    "Chavarría",
    "Chávez",
    "Cintrón",
    "Cisneros",
    "Collado",
    "Collazo",
    "Colón",
    "Colunga",
    "Concepción",
    "Contreras",
    "Cordero",
    "Córdova",
    "Cornejo",
    "Corona",
    "Coronado",
    "Corral",
    "Corrales",
    "Correa",
    "Cortés",
    "Cortez",
    "Cotto",
    "Covarrubias",
    "Crespo",
    "Cruz",
    "Cuellar",
    "Curiel",
    "Dávila",
    "de Anda",
    "de Jesús",
    "Delacrúz",
    "Delafuente",
    "Delagarza",
    "Delao",
    "Delapaz",
    "Delarosa",
    "Delatorre",
    "Deleón",
    "Delgadillo",
    "Delgado",
    "Delrío",
    "Delvalle",
    "Díaz",
    "Domínguez",
    "Domínquez",
    "Duarte",
    "Dueñas",
    "Duran",
    "Echevarría",
    "Elizondo",
    "Enríquez",
    "Escalante",
    "Escamilla",
    "Escobar",
    "Escobedo",
    "Esparza",
    "Espinal",
    "Espino",
    "Espinosa",
    "Espinoza",
    "Esquibel",
    "Esquivel",
    "Estévez",
    "Estrada",
    "Fajardo",
    "Farías",
    "Feliciano",
    "Fernández",
    "Ferrer",
    "Fierro",
    "Figueroa",
    "Flores",
    "Flórez",
    "Fonseca",
    "Franco",
    "Frías",
    "Fuentes",
    "Gaitán",
    "Galarza",
    "Galindo",
    "Gallardo",
    "Gallegos",
    "Galván",
    "Gálvez",
    "Gamboa",
    "Gamez",
    "Gaona",
    "Garay",
    "García",
    "Garibay",
    "Garica",
    "Garrido",
    "Garza",
    "Gastélum",
    "Gaytán",
    "Gil",
    "Girón",
    "Godínez",
    "Godoy",
    "Gollum",
    "Gómez",
    "Gonzales",
    "González",
    "Gracia",
    "Granado",
    "Granados",
    "Griego",
    "Grijalva",
    "Guajardo",
    "Guardado",
    "Guerra",
    "Guerrero",
    "Guevara",
    "Guillen",
    "Gurule",
    "Gutiérrez",
    "Guzmán",
    "Haro",
    "Henríquez",
    "Heredia",
    "Hernádez",
    "Hernandes",
    "Hernández",
    "Herrera",
    "Hidalgo",
    "Hinojosa",
    "Holguín",
    "Huerta",
    "Huixtlacatl",
    "Hurtado",
    "Ibarra",
    "Iglesias",
    "Irizarry",
    "Jaime",
    "Jaimes",
    "Jáquez",
    "Jaramillo",
    "Jasso",
    "Jiménez",
    "Jimínez",
    "Juárez",
    "Jurado",
    "Kadar rodriguez",
    "Kamal",
    "Kamat",
    "Kanaria",
    "Kanea",
    "Kanimal",
    "Kano",
    "Kanzaki",
    "Kaplan",
    "Kara",
    "Karam",
    "Karan",
    "Kardache soto",
    "Karem",
    "Karen",
    "Khalid",
    "Kindelan",
    "Koenig",
    "Korta",
    "Korta hernandez",
    "Kortajarena",
    "Kranz sans",
    "Krasnova",
    "Krauel natera",
    "Kuzmina",
    "Kyra",
    "Laboy",
    "Lara",
    "Laureano",
    "Leal",
    "Lebrón",
    "Ledesma",
    "Leiva",
    "Lemus",
    "León",
    "Lerma",
    "Leyva",
    "Limón",
    "Linares",
    "Lira",
    "Llamas",
    "Loera",
    "Lomeli",
    "Longoria",
    "López",
    "Lovato",
    "Loya",
    "Lozada",
    "Lozano",
    "Lucero",
    "Lucio",
    "Luevano",
    "Lugo",
    "Luna",
    "Macías",
    "Madera",
    "Madrid",
    "Madrigal",
    "Maestas",
    "Magaña",
    "Malave",
    "Maldonado",
    "Manzanares",
    "Mares",
    "Marín",
    "Márquez",
    "Marrero",
    "Marroquín",
    "Martínez",
    "Mascareñas",
    "Mata",
    "Mateo",
    "Matías",
    "Matos",
    "Maya",
    "Mayorga",
    "Medina",
    "Medrano",
    "Mejía",
    "Meléndez",
    "Melgar",
    "Mena",
    "Menchaca",
    "Méndez",
    "Mendoza",
    "Menéndez",
    "Meraz",
    "Mercado",
    "Merino",
    "Mesa",
    "Meza",
    "Miramontes",
    "Miranda",
    "Mireles",
    "Mojica",
    "Molina",
    "Mondragón",
    "Monroy",
    "Montalvo",
    "Montañez",
    "Montaño",
    "Montemayor",
    "Montenegro",
    "Montero",
    "Montes",
    "Montez",
    "Montoya",
    "Mora",
    "Morales",
    "Moreno",
    "Mota",
    "Moya",
    "Munguía",
    "Muñiz",
    "Muñoz",
    "Murillo",
    "Muro",
    "Nájera",
    "Naranjo",
    "Narváez",
    "Nava",
    "Navarrete",
    "Navarro",
    "Nazario",
    "Negrete",
    "Negrón",
    "Nevárez",
    "Nieto",
    "Nieves",
    "Niño",
    "Noriega",
    "Núñez",
    "Ñañez",
    "Ocampo",
    "Ocasio",
    "Ochoa",
    "Ojeda",
    "Olivares",
    "Olivárez",
    "Olivas",
    "Olivera",
    "Olivo",
    "Olmos",
    "Olvera",
    "Ontiveros",
    "Oquendo",
    "Ordóñez",
    "Orellana",
    "Ornelas",
    "Orosco",
    "Orozco",
    "Orta",
    "Ortega",
    "Ortiz",
    "Osorio",
    "Otero",
    "Ozuna",
    "Pabón",
    "Pacheco",
    "Padilla",
    "Padrón",
    "Páez",
    "Pagan",
    "Palacios",
    "Palomino",
    "Palomo",
    "Pantoja",
    "Paredes",
    "Parra",
    "Partida",
    "Patiño",
    "Paz",
    "Pedraza",
    "Pedroza",
    "Pelayo",
    "Peña",
    "Perales",
    "Peralta",
    "Perea",
    "Peres",
    "Pérez",
    "Pichardo",
    "Pineda",
    "Piña",
    "Pizarro",
    "Polanco",
    "Ponce",
    "Porras",
    "Portillo",
    "Posada",
    "Prado",
    "Preciado",
    "Prieto",
    "Puente",
    "Puga",
    "Pulido",
    "Quesada",
    "Quevedo",
    "Quezada",
    "Quinta",
    "Quintairos",
    "Quintana",
    "Quintanilla",
    "Quintero",
    "Quintero cruz",
    "Quintero de la cruz",
    "Quiñones",
    "Quiñónez",
    "Quiros",
    "Quiroz",
    "Rael",
    "Ramírez",
    "Ramón",
    "Ramos",
    "Rangel",
    "Rascón",
    "Raya",
    "Razo",
    "Regalado",
    "Rendón",
    "Rentería",
    "Reséndez",
    "Reyes",
    "Reyna",
    "Reynoso",
    "Rico",
    "Rincón",
    "Riojas",
    "Ríos",
    "Rivas",
    "Rivera",
    "Rivero",
    "Robledo",
    "Robles",
    "Rocha",
    "Rodarte",
    "Rodrígez",
    "Rodríguez",
    "Rodríquez",
    "Rojas",
    "Rojo",
    "Roldán",
    "Rolón",
    "Romero",
    "Romo",
    "Roque",
    "Rosado",
    "Rosales",
    "Rosario",
    "Rosas",
    "Roybal",
    "Rubio",
    "Ruelas",
    "Ruiz",
    "Saavedra",
    "Sáenz",
    "Saiz",
    "Salas",
    "Salazar",
    "Salcedo",
    "Salcido",
    "Saldaña",
    "Saldivar",
    "Salgado",
    "Salinas",
    "Samaniego",
    "Sanabria",
    "Sanches",
    "Sánchez",
    "Sandoval",
    "Santacruz",
    "Santana",
    "Santiago",
    "Santillán",
    "Sarabia",
    "Sauceda",
    "Saucedo",
    "Sedillo",
    "Segovia",
    "Segura",
    "Sepúlveda",
    "Serna",
    "Serrano",
    "Serrato",
    "Sevilla",
    "Sierra",
    "Sisneros",
    "Solano",
    "Solís",
    "Soliz",
    "Solorio",
    "Solorzano",
    "Soria",
    "Sosa",
    "Sotelo",
    "Soto",
    "Suárez",
    "Tafoya",
    "Tamayo",
    "Tamez",
    "Tapia",
    "Tejada",
    "Tejeda",
    "Téllez",
    "Tello",
    "Terán",
    "Terrazas",
    "Tijerina",
    "Tirado",
    "Toledo",
    "Toro",
    "Torres",
    "Tórrez",
    "Tovar",
    "Trejo",
    "Treviño",
    "Trujillo",
    "Ulibarri",
    "Ulloa",
    "Urbina",
    "Ureña",
    "Urías",
    "Uribe",
    "Urrutia",
    "Vaca",
    "Valadez",
    "Valdés",
    "Valdez",
    "Valdivia",
    "Valencia",
    "Valentín",
    "Valenzuela",
    "Valladares",
    "Valle",
    "Vallejo",
    "Valles",
    "Valverde",
    "Vanegas",
    "Varela",
    "Vargas",
    "Vásquez",
    "Vázquez",
    "Vega",
    "Vela",
    "Velasco",
    "Velásquez",
    "Velázquez",
    "Vélez",
    "Véliz",
    "Venegas",
    "Vera",
    "Verdugo",
    "Verduzco",
    "Vergara",
    "Viera",
    "Vigil",
    "Villa",
    "Villagómez",
    "Villalobos",
    "Villalpando",
    "Villanueva",
    "Villareal",
    "Villarreal",
    "Villaseñor",
    "Villegas",
    "Xacon",
    "Xairo Belmonte",
    "Xana",
    "Xenia",
    "Xiana",
    "Xicoy",
    "Yago",
    "Yami",
    "Yanes",
    "Yáñez",
    "Ybarra",
    "Yebra",
    "Yunta",
    "Zabaleta",
    "Zamarreno",
    "Zamarripa",
    "Zambrana",
    "Zambrano",
    "Zamora",
    "Zamudio",
    "Zapata",
    "Zaragoza",
    "Zarate",
    "Zavala",
    "Zayas",
    "Zelaya",
    "Zepeda",
    "Zúñiga",
});

const auto mexicanSuffixes = std::to_array<std::string_view>({
    "Jr.",
    "Sr.",
    "I",
    "II",
    "III",
    "IV",
    "V",
    "MD",
    "DDS",
    "PhD",
    "DVM",
    "Ing.",
    "Lic.",
    "Dr.",
    "Mtro.",
});

const auto mexicanMalePrefixes = std::to_array<std::string_view>({"Sr."});

const auto mexicanFemalePrefixes = std::to_array<std::string_view>({"Sra.", "Sta."});

const NameFormats mexicanNameFormats{
    {{"{firstName} {lastName}", 8}, {"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames mexicanPeopleNames{
    {mexicanMaleFirstNames, mexicanLastNames, mexicanMalePrefixes, mexicanSuffixes},
    {mexicanFemaleFirstNames, mexicanLastNames, mexicanFemalePrefixes, mexicanSuffixes},
    mexicanNameFormats};

// Moldova

const auto moldovanMaleFirstNames = std::to_array<std::string_view>({
    "Ion",        "Vasile",    "Mihail",     "Ivan",      "Gheorghe",   "Vladimir",   "Victor",     "Nicolae",
    "Andrei",     "Petru",     "Alexandru",  "Serghei",   "Valeriu",    "Constantin", "Nicolai",    "Sergiu",
    "Igor",       "Anatolie",  "Dumitru",    "Grigore",   "Iurie",      "Oleg",       "Pavel",      "Tudor",
    "Alexei",     "Alexandr",  "Fiodor",     "Veaceslav", "Vasili",     "Vitalie",    "Valentin",   "Boris",
    "Piotr",      "Sergey",    "Aleksandr",  "Dmitriy",   "Leonid",     "Stefan",     "Dmitri",     "Andrey",
    "Vadim",      "Ilie",      "Stepan",     "Anatoli",   "Simion",     "Ghenadie",   "Roman",      "Viorel",
    "Ruslan",     "Alexander", "Nikolay",    "Mikhail",   "Stanislav",  "Eugeniu",    "Maxim",      "Viktor",
    "Eduard",     "Anton",     "Mihai",      "Iuri",      "Grigori",    "Anatolii",   "Valerii",    "Maksim",
    "Marin",      "Vladislav", "Evgeniy",    "Radu",      "Iurii",      "Vitali",     "Teodor",     "Iacob",
    "Adrian",     "Aurel",     "Artur",      "Yuriy",     "Vitaliy",    "Petr",       "Valeriy",    "Timofei",
    "Dmitry",     "Andrian",   "Feodor",     "Vitalii",   "Filip",      "Marcel",     "Aleksey",    "Dorin",
    "Konstantin", "Artyom",    "Anatoliy",   "Mircea",    "Evgeny",     "Valerian",   "Ilya",       "Gavril",
    "Eugen",      "Daniel",    "Iulian",     "Arcadie",   "Vyacheslav", "Octavian",   "Danil",      "Alexey",
    "Gennadiy",   "Vasily",    "Cristian",   "Vitaly",    "Cristi",     "David",      "Egor",       "Liviu",
    "Serafim",    "Iosif",     "Nicu",       "Yury",      "Ioan",       "Zaharia",    "Georgiy",    "Svyatoslav",
    "Sergei",     "Dinu",      "Emilian",    "Bogdan",    "Leon",       "Artem",      "Anatoly",    "Kirill",
    "Nichita",    "Spiridon",  "Sasa",       "Nicanor",   "Sandu",      "Dorel",      "Artiom",     "Corneliu",
    "Dionisie",   "Rodion",    "Catalin",    "Sorin",     "Luca",       "Ignat",      "Emil",       "Fedor",
    "Ilarion",    "Georgy",    "Daniil",     "Nikolai",   "Filipp",     "Nelu",       "Stelian",    "Cornel",
    "Demian",     "Albert",    "Trifon",     "Kristian",  "Isidor",     "Marius",     "Dionis",     "Iustin",
    "Grigory",    "Yaroslav",  "Damian",     "Zahari",    "Gicu",       "Vsevolod",   "Gleb",       "Lev",
    "Romeo",      "Filimon",   "Haralambie", "Iaroslav",  "Timur",      "Calin",      "Gherasim",   "Mark",
    "Ionel",      "Hristofor", "Dragos",     "Gabriel",   "Emanoil",    "Avram",      "Petrea",     "Timofey",
    "Arcadi",     "Efrem",     "Silviu",     "Aleks",     "Jan",        "Cristofor",  "Lazar",      "Miron",
    "George",     "Matei",     "Semen",      "Arkadiy",   "Costel",     "Robert",     "Sevastian",  "Florin",
    "Nistor",     "Toader",    "Elisei",     "Ian",       "Renat",      "Alin",       "Lucian",     "Vladimer",
    "Rostislav",  "Onisim",    "Marc",       "Dorian",    "Taras",      "Alex",       "John",       "Vlad",
    "Adam",       "Ciprian",   "Timotei",    "Nick",      "Virgiliu",   "Rustam",     "Marcu",      "Isai",
    "Miroslav",   "Kris",      "Dinis",      "Traian",    "Felix",      "Nazar",      "Kiril",      "Titu",
    "Alik",       "Ionut",     "Edgar",      "Orest",     "Ioachim",    "Laurentiu",  "Dimitri",    "German",
    "Martin",     "Alecu",     "Erik",       "Iu",        "Cezar",      "Yurii",      "Ianos",      "Ovidiu",
    "Augustin",   "Aurelian",  "Vergil",     "Angel",     "Cristin",    "Samoil",     "Isac",       "Marat",
    "Doru",       "Edward",    "Eric",       "Gelu",      "Arseniy",    "Iordan",     "Arseni",     "Matvey",
    "Peter",      "Silvian",   "Ali",        "Emanuil",   "Andrii",     "Arsen",      "Dimitrie",   "Eugene",
    "Edvard",     "Sergio",    "Leonard",    "Sebastian", "Ostap",      "Anastas",    "Anghel",     "Casian",
    "Aleksei",    "Philip",    "Virgil",     "Svetoslav", "Richard",    "Petro",      "Alecsandru", "Silvestru",
    "Alen",       "Dragomir",  "Cristea",    "Denys",     "Paul",       "Niko",       "Remus",      "Neculai",
    "Sam",        "Samson",    "Eldar",      "Ghiorghi",  "Andre",      "Armen",      "Chris",      "Arnold",
    "Nicolaie",   "Simon",     "Johny",      "Kalin",     "Rafael",     "Rudolf",     "Yanik",      "Murat",
    "Nestor",     "Samir",     "Gheorge",    "Achim",     "Zhan",       "Georgel",    "Sergej",     "Samuil",
    "Ramin",      "Adolf",     "Dimitry",    "Viktar",    "Dani",       "Nikolas",    "Georgi",     "Said",
    "Aron",       "Rinat",     "Tom",        "Alan",      "Alexandar",  "Sergiy",     "Gari",       "Marco",
    "Teo",        "Steven",    "Aleksander", "Serge",     "Damir",      "Dimitriy",   "Vasil",      "Gregory",
    "Raul",       "Ernest",    "Vadym",      "Jack",      "Mustafa",    "Akim",
});

const auto moldovanFemaleFirstNames = std::to_array<std::string_view>({
    "Maria",      "Elena",     "Valentina",  "Tatiana",   "Olga",      "Natalia",    "Nina",        "Ana",
    "Svetlana",   "Vera",      "Galina",     "Lidia",     "Ecaterina", "Tamara",     "Irina",       "Nadejda",
    "Anna",       "Zinaida",   "Larisa",     "Alexandra", "Raisa",     "Ludmila",    "Eugenia",     "Liudmila",
    "Anastasia",  "Angela",    "Lilia",      "Iulia",     "Marina",    "Parascovia", "Aliona",      "Tatyana",
    "Natalya",    "Mariana",   "Ala",        "Liuba",     "Viorica",   "Victoria",   "Alina",       "Veronica",
    "Elizaveta",  "Diana",     "Silvia",     "Sofia",     "Ekaterina", "Denis",      "Cristina",    "Emilia",
    "Rodica",     "Oxana",     "Kristina",   "Valeri",    "Ioana",     "Evdochia",   "Eudochia",    "Anastasiya",
    "Vasilisa",   "Varvara",   "Claudia",    "Alla",      "Mariya",    "Lucia",      "Agafia",      "Domnica",
    "Antonina",   "Liubov",    "Lyudmila",   "Liliana",   "Ilia",      "Ina",        "Aurelia",     "Daria",
    "Inna",       "Olesea",    "Fedora",     "Nadezhda",  "Daniela",   "Axenia",     "Aurica",      "Mihaela",
    "Elisaveta",  "Inga",      "Valeria",    "Viktoriya", "Anya",      "Lilian",     "Margarita",   "Dina",
    "Alyona",     "Margareta", "Stela",      "Eleonora",  "Violeta",   "Darya",      "Nelea",       "Doina",
    "Nikita",     "Yuliya",    "Maia",       "Sasha",     "Yana",      "Olesya",     "Valeriya",    "Polina",
    "Dan",        "Viktoria",  "Aleksandra", "Kseniya",   "Zoia",      "Dorina",     "Taisia",      "Sava",
    "Adriana",    "Carolina",  "Alena",      "Lyubov",    "Yulia",     "Melania",    "Veronika",    "Corina",
    "Oksana",     "Marian",    "Elvira",     "Dana",      "Marianna",  "Karina",     "Teodora",     "Adela",
    "Vladislava", "Snejana",   "Xenia",      "Iuliana",   "Profira",   "Gabriela",   "Evgeniya",    "Iraida",
    "Alisa",      "Arina",     "Nicoleta",   "Ivanna",    "Rita",      "Iana",       "Mina",        "Marcela",
    "Catalina",   "Zhenya",    "Livia",      "Georgeta",  "Agripina",  "Uliana",     "Anzhela",     "Lina",
    "Anghelina",  "Jana",      "Andriana",   "Anisia",    "Albina",    "Karolina",   "Pavlina",     "Eva",
    "Stella",     "Aglaia",    "Adelina",    "Nika",      "Evgenia",   "Paulina",    "Lidiya",      "Laura",
    "Minodora",   "Dora",      "Felicia",    "Ivana",     "Nelli",     "Nataliya",   "Ksenia",      "Valery",
    "Evelina",    "Iustina",   "Natasa",     "Cornelia",  "Nikoleta",  "Angelina",   "Lora",        "Luminita",
    "Neli",       "Mila",      "Liliya",     "Ada",       "Renata",    "Rimma",      "Zina",        "Clara",
    "Sabina",     "Neonila",   "Vasilina",   "Alevtina",  "Mayya",     "Zhanna",     "Ionela",      "Tania",
    "Andreea",    "Anisoara",  "Snezhana",   "Lia",       "Vica",      "Ruslana",    "Roza",        "Sanda",
    "Taisa",      "Lucretia",  "Ilona",      "Hristina",  "Ira",       "Sanya",      "Toma",        "Leonora",
    "Gheorghina", "Lilya",     "Tina",       "Lida",      "Rima",      "Marta",      "Janna",       "Olimpia",
    "Nona",       "Violetta",  "Faina",      "Sorina",    "Sofiya",    "Danila",     "Alexandrina", "Tecla",
    "Greta",      "Dumitrita", "Virginia",   "Yan",       "Madalina",  "Nadia",      "Ariadna",     "Doinita",
    "Maryana",    "Nataly",    "Yulian",     "Rada",      "Anzhelika", "Gabi",       "Didina",      "Kira",
    "Teodosia",   "Angelica",  "Valerie",    "Leonida",   "Susana",    "Emma",       "Violina",     "Aneta",
    "Loredana",   "Mary",      "Regina",     "Luiza",     "Vania",     "Antonia",    "Lucica",      "Zoya",
    "Romina",     "Inesa",     "Florea",     "Elina",     "Cleopatra", "Inessa",     "Lolita",      "Yuliana",
    "Lenuta",     "Verginia",  "Anelia",     "Tudora",    "Sofya",     "Nelya",      "Lika",        "Sashka",
    "Nelly",      "Miroslava", "Silva",      "Katalina",  "Milena",    "Vitalina",   "Olivia",      "Niculina",
    "Marinela",   "Aurora",    "Zamfira",    "Mirela",    "Julia",     "Olesia",     "Magdalena",   "Rosina",
    "Paula",      "Magdalina", "Silviya",    "Steluta",   "Daniella",  "Stefana",    "Steliana",    "Maricica",
    "Reveca",     "Patricia",  "Asya",       "Katalin",   "Elmira",    "Lada",       "Florica",     "Malvina",
    "Constanta",  "Ulyana",    "Lili",       "Crina",     "Lana",      "Ninel",      "Olguta",      "Maya",
    "Darina",     "Reghina",   "Gaby",       "Floarea",   "Alexa",     "Anfisa",     "Stanislava",  "Ela",
    "Tamila",     "Ema",       "Mira",       "Vitalia",   "Estera",    "Iona",       "Camelia",     "Inka",
    "Vanda",      "Otilia",    "Sabrina",    "Sania",     "Janeta",    "Alvina",     "Erica",       "Rozalia",
    "Erika",      "Zlata",     "Angelika",   "Gabriella", "Antonela",  "Simona",
});

const auto moldovanLastNames = std::to_array<std::string_view>({
    "Rusu",       "Ceban",     "Ciobanu",   "Lungu",      "Cebotari",   "Munteanu",  "Popa",      "Gutu",
    "Rotari",     "Balan",     "Sirbu",     "Cojocari",   "Grosu",      "Melnic",    "Rotaru",    "Cojocaru",
    "Rosca",      "Turcan",    "Moraru",    "Morari",     "Cazacu",     "Cretu",     "Muntean",   "Botnari",
    "Ursu",       "Bivol",     "Russu",     "Ivanov",     "Popovici",   "Lupu",      "Mocanu",    "Chiriac",
    "Plamadeala", "Luca",      "Negru",     "Guzun",      "Turcanu",    "Miron",     "Popescu",   "Railean",
    "Arnaut",     "Florea",    "Cebanu",    "Popov",      "Plesca",     "Prodan",    "Andronic",  "Sandu",
    "Ivanova",    "Rusnac",    "Buga",      "Burlacu",    "Musteata",   "Malai",     "Borta",     "Croitor",
    "Bejenari",   "Cusnir",    "Pascari",   "Grecu",      "Golban",     "Oprea",     "Cernei",    "Pinzari",
    "Gheorghita", "Caraman",   "Bostan",    "Istrati",    "Spinu",      "Frunza",    "Cazac",     "Darii",
    "Lisnic",     "Raileanu",  "Bors",      "Josan",      "Topal",      "Croitoru",  "Birca",     "Colesnic",
    "Botnaru",    "Brinza",    "Gherman",   "Petrov",     "Postica",    "Bulat",     "Dragan",    "Chicu",
    "Cucu",       "Mardari",   "Carp",      "Postolachi", "Covali",     "Uzun",      "Platon",    "Gaina",
    "Radu",       "Antoci",    "Caraus",    "Albu",       "Codreanu",   "Tcaci",     "Gonta",     "Bejan",
    "Cara",       "Baciu",     "Ursachi",   "Bodrug",     "Ungureanu",  "Lupascu",   "Macovei",   "Ciubotaru",
    "Braga",      "Mihailov",  "Rata",      "Condrea",    "Pascal",     "Creciun",   "Craciun",   "Terzi",
    "Stratan",    "Moisei",    "Ciumac",    "Diaconu",    "Gangan",     "Cucos",     "Mereuta",   "Nastas",
    "Tataru",     "Capatina",  "Leahu",     "Goncear",    "Focsa",      "Bordian",   "Marin",     "Botezatu",
    "Crudu",      "Adam",      "Graur",     "Novac",      "Bitca",      "Esanu",     "Marian",    "Furtuna",
    "Apostol",    "Popova",    "Catana",    "Robu",       "Frunze",     "Secrieru",  "Bargan",    "Matei",
    "Andries",    "Mirza",     "Boboc",     "Stratulat",  "Paladi",     "Anghel",    "Cibotari",  "Trifan",
    "Butnaru",    "Mamaliga",  "Busuioc",   "Scripnic",   "Racu",       "Placinta",  "Jalba",     "Nistor",
    "Hincu",      "Bogdan",    "Cristea",   "David",      "Popusoi",    "Marcu",     "Gisca",     "Tofan",
    "Sava",       "Roman",     "Lazari",    "Erhan",      "Luchian",    "Balaban",   "Costin",    "Dodon",
    "Cotorobai",  "Bujor",     "Dabija",    "Vieru",      "Olaru",      "Macari",    "Mereacre",  "Dontu",
    "Lesnic",     "Prisacari", "Burca",     "Mazur",      "Moldovan",   "Olari",     "Gorea",     "Ababii",
    "Lazar",      "Casian",    "Gavriliuc", "Cecan",      "Duca",       "Lupasco",   "Lavric",    "Burlac",
    "Chirilov",   "Banari",    "Pislari",   "Oleinic",    "Grigoras",   "Ciorba",    "Gavrilita", "Manoli",
    "Prepelita",  "Anton",     "Sirghi",    "Burduja",    "Damian",     "Covalenco", "Cirlan",    "Cojuhari",
    "Bagrin",     "Ignat",     "Jardan",    "Stirbu",     "Moroz",      "Toma",      "Pintea",    "Eni",
    "Mocan",      "Arabadji",  "Perju",     "Gutan",      "Babin",      "Taran",     "Svet",      "Cheban",
    "Arama",      "Enachi",    "Dascal",    "Neagu",      "Cioban",     "Chistol",   "Lefter",    "Schiopu",
    "Groza",      "Petrova",   "Chihai",    "Baltag",     "Cebotaru",   "Donica",    "Bodiu",     "Buruiana",
    "Druta",      "Cozma",     "Grigorita", "Boico",      "Danilov",    "Cantir",    "Lozan",     "Caldare",
    "Negara",     "Palii",     "Bejenaru",  "Melnik",     "Gilca",      "Creanga",   "Nicolaev",  "Damaschin",
    "Calancea",   "Pogor",     "Gutul",     "Lupan",      "Colta",      "Gogu",      "Revenco",   "Pasat",
    "Buza",       "Stavila",   "Scutari",   "Moldovanu",  "Bologan",    "Savin",     "Efros",     "Puscas",
    "Paduret",    "Axenti",    "Butuc",     "Isac",       "Chirita",    "Danu",      "Costiuc",   "Catan",
    "Bolocan",    "Cuciuc",    "Sevcenco",  "Iovu",       "Bulgac",     "Belous",    "Scutelnic", "Leu",
    "Ciolac",     "Eremia",    "Negura",    "Romanciuc",  "Besliu",     "Chiosa",    "Demian",    "Iurcu",
    "Ciornii",    "Filip",     "Pruteanu",  "Gaidarji",   "Covalciuc",  "Proca",     "Movila",    "Gritco",
    "Pinzaru",    "Bulgaru",   "Bunescu",   "Baran",      "Mindru",     "Buzu",      "Pascaru",   "Savciuc",
    "Lesan",      "Scurtu",    "Harea",     "Grama",      "Bondarenco", "Buzdugan",  "Vlas",      "Gincu",
    "Cujba",      "Sevciuc",   "Loghin",    "Morozan",    "Avram",      "Sula",      "Virlan",    "Manole",
    "Arseni",     "Ionita",    "Chiperi",   "Pasa",       "Babii",      "Rotar",
});

const NameFormats moldovanNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames moldovanPeopleNames{{moldovanMaleFirstNames, moldovanLastNames, {}, {}},
                                      {moldovanFemaleFirstNames, moldovanLastNames, {}, {}},
                                      moldovanNameFormats};

// Monaco

const auto monacanMaleFirstNames = std::to_array<std::string_view>({
    "Philippe",    "David",    "Robert",   "Eric",       "Patrick",   "Christian",  "Michel",   "Pierre",  "Marc",
    "Olivier",     "Nicolas",  "Alain",    "Michael",    "John",      "Christophe", "Daniel",   "Paul",    "Laurent",
    "Thierry",     "Marco",    "Thomas",   "Peter",      "Alexandre", "Jacques",    "Georges",  "Paolo",   "Richard",
    "Julien",      "Franck",   "Joseph",   "Francesco",  "Mario",     "Bernard",    "Roberto",  "Pascal",  "Gilles",
    "Bruno",       "Stefano",  "Anthony",  "Massimo",    "Luca",      "Giovanni",   "Fabrice",  "James",   "Charles",
    "Vincent",     "William",  "Giuseppe", "Alessandro", "Carlo",     "Guillaume",  "Roger",    "Giorgio", "Alex",
    "Frederic",    "Alberto",  "Fabio",    "Emmanuel",   "Alexander", "Jan",        "Enrico",   "Antonio", "Antoine",
    "Simon",       "Didier",   "Roland",   "Martin",     "Benjamin",  "Louis",      "Serge",    "Raymond", "Mohamed",
    "Mark",        "Lionel",   "Guy",      "Patrice",    "Gregory",   "Maurice",    "Arnaud",   "Stephen", "Henri",
    "Federico",    "Maurizio", "Xavier",   "Andrew",     "Gilbert",   "Luigi",      "Jonathan", "Yves",    "Fabien",
    "Christopher", "Marcel",   "Igor",     "Albert",     "Franco",    "Sebastien",  "Riccardo", "George",  "Sylvain",
    "Kevin",       "Manuel",   "Chris",    "Pietro",     "Romain",    "Frank",      "Max",      "Mathieu", "Ian",
    "Cyril",       "Matteo",   "Lorenzo",  "Giancarlo",  "Tony",      "Vladimir",   "Jeremy",   "Yann",    "Filippo",
    "Colin",       "Nicholas", "Jerome",   "Davide",     "Steven",    "Damien",     "Dario",    "Gabriel", "Ali",
    "Gianluca",    "Philip",   "Gerard",   "Sergey",     "Arthur",    "Maxime",     "Ivan",     "Karl",    "Edouard",
    "Alan",        "Luc",      "Hans",     "Fred",       "Mauro",     "Angelo",     "Karim",    "Carlos",  "Matthieu",
    "Benoit",      "Hugo",     "Fabrizio", "Carl",       "Renato",    "Ahmed",      "Emanuele", "Claudio", "Edmond",
    "Edward",      "Joe",      "Jose",     "Cedric",     "Sergio",    "Fernando",   "Walter",   "Ludovic", "Florian",
    "Salvatore",   "Erik",     "Yoann",    "Luciano",    "Guido",     "Tom",        "Brian",    "Georg",   "Raphael",
    "Henry",       "Gary",     "Nabil",    "Luis",       "Pieter",    "Yvan",       "Giacomo",  "Ronald",  "Andre",
    "Oliver",      "Joel",     "Donald",   "Andrey",     "Bertrand",  "Marcus",     "Matthew",  "Gianni",  "Nick",
    "Stephan",     "Victor",   "Jorge",    "Adam",       "Neil",      "Adrien",     "Mamadou",  "Flavio",  "Gian",
    "Norbert",     "Stefan",   "Sandro",
});

const auto monacanFemaleFirstNames = std::to_array<std::string_view>({
    "Jean",       "Andrea",     "Maria",      "Catherine",  "Isabelle",  "Nathalie",   "Anne",      "Christine",
    "Sophie",     "Marie",      "Anna",       "Claude",     "Sylvie",    "Caroline",   "Elena",     "Jacqueline",
    "Nicole",     "Laura",      "Alexandra",  "Patricia",   "Elisabeth", "Monique",    "Brigitte",  "Dominique",
    "Christiane", "Sandrine",   "Mireille",   "Laurence",   "Virginie",  "Barbara",    "Sandra",    "Vanessa",
    "Florence",   "Cristina",   "Olga",       "Paola",      "Annie",     "Corinne",    "Claire",    "Audrey",
    "Marina",     "Carole",     "Michele",    "Sonia",      "Elodie",    "Karine",     "Simone",    "Francesca",
    "Suzanne",    "Colette",    "Charlotte",  "Muriel",     "Julie",     "Louise",     "Fabienne",  "Francis",
    "Sarah",      "Josiane",    "Nadia",      "Patrizia",   "Victoria",  "Evelyne",    "Stephane",  "Irina",
    "Karen",      "Chantal",    "Sabrina",    "Danielle",   "Monica",    "Magali",     "Sabine",    "Michelle",
    "Mary",       "Angela",     "Roberta",    "Carla",      "Yvette",    "Liliane",    "Daniela",   "Gabriele",
    "Jeanne",     "Samantha",   "Denise",     "Delphine",   "Josette",   "Rita",       "Eliane",    "Jeannine",
    "Alexis",     "Denis",      "Olivia",     "Pascale",    "Valerie",   "Tatiana",    "Sara",      "Eva",
    "Emmanuelle", "Nadine",     "Laetitia",   "Linda",      "Nicola",    "Jessica",    "Janine",    "Claudine",
    "Arlette",    "Diane",      "Laure",      "Marguerite", "Emma",      "Johanna",    "Maryse",    "Jennifer",
    "Yvonne",     "Diana",      "Natalia",    "Daniele",    "Carine",    "Alessandra", "Elizabeth", "Marine",
    "Lisa",       "Marion",     "Manuela",    "Solange",    "Nancy",     "Helen",      "Rachel",    "Alice",
    "Bettina",    "Emilie",     "Annick",     "Antonella",  "Huguette",  "Francine",   "Alexia",    "Chiara",
    "Eleonora",   "Valentina",  "Karin",      "Myriam",     "Laurie",    "Odette",     "Stephanie", "Ingrid",
    "Grace",      "Pauline",    "Ana",        "Virginia",   "Silvia",    "Veronique",  "Camille",   "Celine",
    "Teresa",     "Micheline",  "Christina",  "Cinzia",     "Jocelyne",  "Anastasia",  "Lucienne",  "Jane",
    "Alina",      "Marisa",     "Yannick",    "Christelle", "Andreas",   "Lydia",      "Rose",      "Juliette",
    "Bianca",     "Bernadette", "Svetlana",   "Robin",      "Nelly",     "Susan",      "Sylviane",  "Judith",
    "Aline",      "Claudette",  "Marianne",   "Pierrette",  "Elisa",     "Sylvia",     "Giovanna",  "Viviane",
    "Julia",      "Clara",      "Amandine",   "Nataliya",   "Stefania",  "Katia",      "Sacha",     "Luisa",
    "Paule",      "Madeleine",  "Marta",      "Nina",       "Claudia",   "Caterina",   "Edith",     "Veronica",
    "Larisa",     "Carmen",     "Marjorie",   "Elvira",     "Coralie",   "Silvana",    "Estelle",   "Ruth",
    "Michela",    "Carol",      "Camilla",    "Yulia",      "Jenny",     "Paulette",   "Anita",     "Carolina",
    "Eugenia",    "Joelle",     "Murielle",   "Cynthia",    "Janet",     "Lara",       "Fiona",     "Lucia",
    "Ursula",     "Stella",     "Wendy",      "Odile",      "Isabella",  "Renata",     "Lina",      "Juan",
    "Monika",     "Tiziana",    "Magdalena",  "Natacha",    "Adriana",   "Emilienne",  "Kelly",     "Ann",
    "Kristina",   "Margarita",  "Nicoletta",  "Melanie",    "Piera",     "Cindy",      "Elise",     "Marianna",
    "Carmela",    "Rosanna",    "Federica",   "Amanda",     "Melissa",   "Cecile",     "Armelle",   "Lesley",
    "Graziella",  "Ekaterina",  "Gabriella",  "Lucie",      "Anny",      "Raymonde",   "Yolande",   "Antoinette",
    "Ginette",    "Annette",    "Elisabetta", "Ariane",     "Marcelle",  "Leila",      "Eveline",   "Ashley",
    "Beatrice",   "Mathilde",   "Aude",       "Lydie",      "Inna",      "Iryna",      "Morgan",    "Vivian",
    "Emilia",     "Adrienne",   "Willy",      "Karina",     "Ida",       "Candice",    "Simonetta", "Serenella",
    "Marielle",   "Dorota",     "Eleni",      "Aurore",     "Erika",     "Marthe",     "Franca",    "Lorraine",
    "Fanny",      "Gilberte",   "Rossana",    "Isabel",     "Jasmine",   "Giulia",     "Santa",     "Jeannette",
    "Paula",      "Dennis",     "Mirella",    "Maddalena",  "Lucette",   "Emanuela",   "Flora",     "Maryline",
    "France",     "Elsa",       "Lorenza",    "Rosa",       "Deborah",   "Maya",       "Jesus",     "Irene",
    "Georgette",  "Amina",      "Justine",    "Rosella",    "Ewa",       "Henriette",  "Simona",    "Irma",
    "Agathe",     "Lucile",     "Danny",      "Christel",   "Veronika",  "Sally",      "Rosette",   "Margaret",
    "Dany",       "Antonia",    "Anouk",      "Giorgia",    "Joan",      "Khadija",    "Liliana",   "Serena",
    "Donatella",  "Galina",     "Annemarie",  "Astrid",     "Philippa",  "Lise",
});

const auto monacanLastNames = std::to_array<std::string_view>({
    "Rossi",
    "Lorenzi",
    "Pastor",
    "Crovetto",
    "Cellario",
    "Martin",
    "Smith",
    "Garcia",
    "Gastaud",
    "Diallo",
    "Bernard",
    "Giordano",
    "Cohen",
    "Viale",
    "Lopez",
    "Fissore",
    "Verrando",
    "Williams",
    "Biancheri",
    "Palmero",
    "Allavena",
    "Martini",
    "Roux",
    "Carpinelli",
    "Taylor",
    "Pastorelli",
    "Rinaldi",
    "Testa",
    "Ferrari",
    "Bernardi",
    "Marsan",
    "Martinez",
    "Gomez",
    "Vatrican",
    "Bruno",
    "Michel",
    "Pizzio",
    "Hall",
    "Blanc",
    "Grimaldi",
    "Perez",
    "Johnson",
    "Câmara",
    "Medecin",
    "Thomas",
    "Lavagna",
    "Scott",
    "Costa",
    "Ferrero",
    "Merlino",
    "Antognelli",
    "Agliardi",
    "Marquet",
    "Berti",
    "Vial",
    "Lewis",
    "Franco",
    "Garelli",
    "Boisson",
    "Ramirez",
    "Bertrand",
    "Prat",
    "Rodriguez",
    "Rey",
    "Garino",
    "Durand",
    "Battaglia",
    "Marzocco",
    "Ballestra",
    "Klein",
    "Legrand",
    "Bertola",
    "Benedetti",
    "Boisbouvier",
    "Pierre",
    "Brown",
    "Brezzo",
    "Sangiorgio",
    "Barry",
    "Morel",
    "Levy",
    "Fabre",
    "Fautrier",
    "Wright",
    "Tomatis",
    "Notari",
    "Giusti",
    "Barral",
    "Doria",
    "Li",
    "Gastaldi",
    "Morris",
    "Longo",
    "Gibelli",
    "Gallo",
    "Cooper",
    "Marchetti",
    "Hernandez",
    "Jensen",
    "Toure",
    "Green",
    "Raimondo",
    "Orsini",
    "Ricciardi",
    "Pallanca",
    "Ross",
    "Bianchi",
    "Gaglio",
    "Pasquier",
    "Jones",
    "Marcel",
    "Albertini",
    "Guglielmi",
    "Nardi",
    "Alberti",
    "Squarciafichi",
    "Vidal",
    "Calcagno",
    "Ribeiro",
    "Casanova",
    "Bellone",
    "Mendes",
    "Baldoni",
    "Blanchy",
    "Kim",
    "Davies",
    "Sbarrato",
    "Daniel",
    "Angelini",
    "Simon",
    "Zunino",
    "Lambert",
    "Bianco",
    "Pascal",
    "Miller",
    "Imbert",
    "Bonnet",
    "Solamito",
    "Richelmi",
    "Orengo",
    "Romano",
    "Muller",
    "Serra",
    "Mathieu",
    "Hansen",
    "Campana",
    "Leclercq",
    "Blanchi",
    "Sassi",
    "Michelis",
    "Aubert",
    "Evans",
    "Lombard",
    "Joly",
    "Edwards",
    "Mazza",
    "Reynaud",
    "Sosso",
    "Christensen",
    "Fontaine",
    "Lanteri",
    "Thompson",
    "Graziani",
    "Ballerio",
    "Seggiaro",
    "Harrison",
    "Moore",
    "Bah",
    "Da silva",
    "Adams",
    "Noaro",
    "Balde",
    "Siri",
    "Davidson",
    "Benoit",
    "King",
    "Mercier",
    "Wilson",
    "Petrini",
    "Augier",
    "Sylla",
    "Singh",
    "Tubino",
    "Faivre",
    "Albisetti",
    "Griffin",
    "Bonafede",
    "Lajoux",
    "Gabriel",
    "Merlo",
    "De millo-terrazzani",
    "De sevelinges",
    "Berrin",
    "Wagner",
    "Gomes",
    "Buzzi",
    "Van klaveren",
    "Meunier",
    "Palmer",
    "Dessi",
    "Perrin",
    "Vincent",
    "Guillaume",
    "Ferrua",
    "Valentini",
    "Olivié",
    "Larini",
    "Barilaro",
    "White",
    "Bosio",
    "Piano",
    "Robin",
    "Marin",
    "Clark",
    "Durando",
    "Stevenson",
    "Antonini",
    "Julien",
    "Jackson",
    "Barelli",
    "Ravera",
    "Anderson",
    "Wood",
    "Robini",
    "Rizzi",
    "Monte-carlo",
    "Perris",
    "Limone",
    "Revelli",
    "Petit",
    "Lacroix",
    "Rosticher",
    "Lefebvre",
    "Courtin",
    "Cottalorda",
    "Risso",
    "Vitale",
    "Agostini",
    "Santini",
    "Ragazzoni",
    "Moller",
    "Boeri",
    "Giovannini",
    "Pons",
    "Kelly",
    "Ginocchio",
    "Valle",
    "Realini",
    "Cassini",
    "Picco",
    "Manfredi",
    "Fiorucci",
    "Nucciarelli",
    "Scotto",
    "Russo",
    "Berger",
    "Dias",
    "Poyet",
    "Sanchez",
    "Manasse",
    "Keita",
    "Laurent",
    "Garoscio",
    "Gautier",
    "Ellis",
    "Ivaldi",
    "Traore",
    "Dalmasso",
    "Parker",
    "Mille",
    "Boero",
    "Nelson",
    "Vitali",
    "Brousse",
    "Cardone",
    "Verdino",
    "Reynier",
    "Blair",
    "Marchisio",
    "Jahlan",
    "Lee",
    "Grassi",
    "Bertoni",
    "Raynaud",
    "Bertani",
    "Gamba",
    "Orlando",
    "Orecchia",
    "Moreno",
    "Hill",
    "Rose",
    "Olivier",
    "Lechner",
    "Ouedraogo",
    "Mondino",
    "Moscati",
    "Oliveira",
    "Robertson",
    "Gonzalez",
    "Weill",
    "Wenden",
    "Sorensen",
    "Murray",
    "Brunner",
    "Falciani",
    "Bourgery",
    "Riva",
    "Passeron",
    "Robert",
    "Versace",
    "Renaud",
    "Payne",
    "Marquês",
    "Allen",
    "Varley",
    "Franzini",
    "Stillitano",
    "Giraldi",
    "Pinto",
    "Croesi",
    "Cruz",
    "Peeters",
    "Castellini",
    "Corradi",
    "Dessaigne",
    "Olivieri",
});

const NameFormats monacanNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames monacanPeopleNames{{monacanMaleFirstNames, monacanLastNames, {}, {}},
                                     {monacanFemaleFirstNames, monacanLastNames, {}, {}},
                                     monacanNameFormats};

// Norway

const auto norwegianMaleFirstNames = std::to_array<std::string_view>({
    "Markus", "Mathias",  "Kristian",   "Jonas",  "Andreas", "Alexander", "Martin",  "Sander",  "Daniel",  "Magnus",
    "Henrik", "Tobias",   "Kristoffer", "Emil",   "Adrian",  "Sebastian", "Marius",  "Elias",   "Fredrik", "Thomas",
    "Sondre", "Benjamin", "Jakob",      "Oliver", "Lucas",   "Oskar",     "Nikolai", "Filip",   "Mats",    "William",
    "Erik",   "Simen",    "Ole",        "Eirik",  "Isak",    "Kasper",    "Noah",    "Lars",    "Joakim",  "Johannes",
    "Håkon",  "Sindre",   "Jørgen",     "Herman", "Anders",  "Jonathan",  "Even",    "Theodor", "Mikkel",  "Aksel",
});

const auto norwegianFemaleFirstNames = std::to_array<std::string_view>({
    "Emma",     "Sara",     "Thea",    "Ida",      "Julie",   "Nora",    "Emilie",  "Ingrid", "Hanna",    "Maria",
    "Sofie",    "Anna",     "Malin",   "Amalie",   "Vilde",   "Frida",   "Andrea",  "Tuva",   "Victoria", "Mia",
    "Karoline", "Mathilde", "Martine", "Linnea",   "Marte",   "Hedda",   "Marie",   "Helene", "Silje",    "Leah",
    "Maja",     "Elise",    "Oda",     "Kristine", "Aurora",  "Kaja",    "Camilla", "Mari",   "Maren",    "Mina",
    "Selma",    "Jenny",    "Celine",  "Eline",    "Sunniva", "Natalie", "Tiril",   "Synne",  "Sandra",   "Madeleine",
});

const auto norwegianLastNames = std::to_array<std::string_view>({
    "Johansen",     "Hansen",        "Andersen",  "Kristiansen",   "Larsen",    "Olsen",      "Solberg",
    "Andresen",     "Pedersen",      "Nilsen",    "Berg",          "Halvorsen", "Karlsen",    "Svendsen",
    "Jensen",       "Haugen",        "Martinsen", "Eriksen",       "Sørensen",  "Johnsen",    "Myhrer",
    "Johannessen",  "Nielsen",       "Hagen",     "Pettersen",     "Bakke",     "Skuterud",   "Løken",
    "Gundersen",    "Strand",        "Jørgensen", "Kvarme",        "Røed",      "Sæther",     "Stensrud",
    "Moe",          "Kristoffersen", "Jakobsen",  "Holm",          "Aas",       "Lie",        "Moen",
    "Andreassen",   "Vedvik",        "Nguyen",    "Jacobsen",      "Torgersen", "Ruud",       "Krogh",
    "Christiansen", "Bjerke",        "Aalerud",   "Borge",         "Sørlie",    "Berge",      "Østli",
    "Ødegård",      "Torp",          "Henriksen", "Haukelidsæter", "Fjeld",     "Danielsen",  "Aasen",
    "Fredriksen",   "Dahl",          "Berntsen",  "Arnesen",       "Wold",      "Thoresen",   "Solheim",
    "Skoglund",     "Bakken",        "Amundsen",  "Solli",         "Smogeli",   "Kristensen", "Glosli",
    "Fossum",       "Evensen",       "Eide",      "Carlsen",       "Østby",     "Vegge",      "Tangen",
    "Smedsrud",     "Olstad",        "Lunde",     "Kleven",        "Huseby",    "Bjørnstad",  "Ryan",
    "Rasmussen",    "Nygård",        "Nordskaug", "Nordby",        "Mathisen",  "Hopland",    "Gran",
    "Finstad",      "Edvardsen",
});

const auto norwegianSuffixes = std::to_array<std::string_view>({"Jr.", "Sr.", "I", "II", "III", "IV", "V"});

const auto norwegianPrefixes = std::to_array<std::string_view>({"Dr.", "Prof."});

const NameFormats norwegianNameFormats{
    {{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames norwegianPeopleNames{
    {norwegianMaleFirstNames, norwegianLastNames, norwegianPrefixes, norwegianSuffixes},
    {norwegianFemaleFirstNames, norwegianLastNames, norwegianPrefixes, norwegianSuffixes},
    norwegianNameFormats};

// Palestine

const auto palestinianMaleFirstNames = std::to_array<std::string_view>({
    "Mohamed",   "Ahmed",      "Abdel",    "Mahmoud",  "Ibrahim", "Khaled",     "Ali",     "Yousef",  "Hassan",
    "Raed",      "Emad",       "Iyad",     "Omar",     "Jamal",   "Khalil",     "Ayman",   "Ramy",    "Samih",
    "Fayez",     "Ismail",     "Abdullah", "Mostafa",  "Mousa",   "Ashraf",     "Ziad",    "Hussein", "Osama",
    "Saeed",     "Samir",      "Maher",    "Sami",     "Hossam",  "Yasser",     "Hani",    "Fadi",    "Waleed",
    "Saleh",     "Suleiman",   "Nasser",   "Nabil",    "Bassam",  "Shady",      "Wael",    "Samer",   "Tarek",
    "Adel",      "Majed",      "Adnan",    "Essam",    "Salah",   "Bilal",      "Amjad",   "Kamal",   "Jamil",
    "Akram",     "Anwar",      "Kamel",    "Nader",    "Riad",    "Murad",      "Zuhair",  "Hisham",  "Yahya",
    "Fouad",     "Marwan",     "Ghassan",  "Mazen",    "Fathi",   "Amer",       "Hatem",   "Amin",    "Firas",
    "Ammar",     "Salem",      "Haytham",  "Mounir",   "Naeem",   "Khader",     "Salim",   "Hamza",   "Nael",
    "Tawfiq",    "Najih",      "Louay",    "Zaher",    "Atef",    "Sherif",     "Asaad",   "Fawzi",   "Anas",
    "Mahdi",     "Fares",      "Salam",    "Younis",   "Hamed",   "Tamer",      "Ramadan", "Jalal",   "Atta",
    "Jawad",     "Zakaria",    "Saad",     "Osman",    "Sufian",  "Talal",      "Farid",   "Bashir",  "Khamis",
    "Dia",       "Arafat",     "Naji",     "Suhail",   "Saber",   "Faisal",     "Rafiq",   "Rashid",  "Mamoun",
    "Nizar",     "Ghazi",      "Nazim",    "Mansour",  "Yacoub",  "Qasim",      "Salman",  "Malik",   "Taleb",
    "Adham",     "Zaki",       "Badr",     "Fahd",     "Taha",    "Rabie",      "Yassin",  "Waseem",  "Bakr",
    "Ishaq",     "Noman",      "Rashad",   "Hamada",   "Jaafar",  "Karam",      "Radwan",  "Amir",    "Ayoub",
    "Jaber",     "George",     "Azhar",    "Mesbah",   "Nazih",   "Hammad",     "Rabah",   "Aziz",    "Fatih",
    "Farouk",    "Hamad",      "Mohsen",   "Elias",    "Taher",   "Zaid",       "Makram",  "Sultan",  "Iftikhar",
    "Lotfi",     "Karim",      "Hilal",    "Amr",      "Radi",    "Rasheed",    "Sadiq",   "Wahid",   "Massoud",
    "Shafiq",    "Nizam",      "Saif",     "Abbas",    "Sabri",   "Anis",       "Mujahid", "Haidar",  "Sayed",
    "Iqbal",     "Mahy",       "Zain",     "Imtiaz",   "Afif",    "Raouf",      "Farhan",  "Nadi",    "Amar",
    "Nadeem",    "Nassar",     "Haroun",   "Idris",    "Hadi",    "Wahib",      "Badi",    "Anton",   "Adam",
    "Habib",     "Marzouk",    "Fahim",    "Johnny",   "Hamid",   "Rubin",      "Matar",   "Mabrouk", "Roly",
    "Ayad",      "Allan",      "Emile",    "William",  "Safwan",  "Michel",     "Abdo",    "Abou",    "Bahri",
    "Nasrallah", "Shoaib",     "Mubarak",  "Daoud",    "Hany",    "Abed",       "Safi",    "Asad",    "Antar",
    "Mikhail",   "Arif",       "Rehan",    "Raymond",  "Joseph",  "Bader",      "Sinan",   "Victor",  "Milad",
    "Aqeel",     "Said",       "Jack",     "Rabeh",    "Khalifa", "Ansar",      "Aba",     "Naser",   "Jalil",
    "Hanafi",    "Asif",       "Simon",    "Falak",    "Mumtaz",  "Rafi",       "Peter",   "Yvon",    "Nasir",
    "Morris",    "Abdelhakim", "Adeel",    "Luqman",   "Fikret",  "Abdelkader", "Edward",  "Gamal",   "Nayel",
    "Muzaffar",  "Irfan",      "Ihab",     "Muhammed", "Nazir",   "Hajjaj",     "Hakim",   "Sajid",   "Akbar",
    "Marcel",    "Hossein",    "John",     "Fadel",    "Majid",   "Zahir",      "Siraj",   "Madi",    "Mokhtar",
    "Ghali",     "Sheikh",     "Karmel",   "Robert",   "Angel",   "Seddik",     "Hama",    "Scott",   "Raman",
    "Soso",      "Aram",       "Jamie",    "Kamil",    "Mike",    "Daniel",     "Yousif",  "Alam",    "Mohammed",
    "Mazhar",    "Talha",
});

const auto palestinianFemaleFirstNames = std::to_array<std::string_view>({
    "Fatima",  "Mariam",   "Iman",       "Amal",      "Hanan",     "Mona",    "Manal",     "Aisha",      "Wafa",
    "Alaa",    "Nasreen",  "Sana",       "Samira",    "Asmaa",     "Jamila",  "Heba",      "Halima",     "Ibtisam",
    "Souad",   "Jihad",    "Maha",       "Hoda",      "Hana",      "Laila",   "Siham",     "Basma",      "Najah",
    "Nidal",   "Ghada",    "Nour",       "Samar",     "Rana",      "Nadia",   "Nawal",     "Nada",       "Zainab",
    "Sahar",   "Samah",    "Sabah",      "Hayam",     "Fathia",    "Rania",   "Khadija",   "Faten",      "Ahlam",
    "Hala",    "Sarah",    "Aida",       "Safa",      "Samia",     "Lina",    "Kholoud",   "Rasha",      "Wissam",
    "Anam",    "Dalal",    "Elham",      "Aziza",     "Najwa",     "Sawsan",  "Magda",     "Khadra",     "Susan",
    "Noha",    "Ela",      "Rima",       "Yasmin",    "Salwa",     "Maysaa",  "Ramia",     "Afaf",       "Nihad",
    "Randa",   "Fadwa",    "Khawla",     "Rajaa",     "Islam",     "Ayesha",  "Mai",       "Nahed",      "Inas",
    "Nahla",   "Lubna",    "Rehab",      "Duaa",      "Saada",     "Asmahan", "Manar",     "Latifa",     "Hind",
    "Alia",    "Nabila",   "Dina",       "Hayat",     "Shadia",    "Raida",   "Nariman",   "Kawthar",    "Abla",
    "Zahra",   "Reda",     "Marwa",      "Rahma",     "Jihan",     "Najla",   "Salama",    "Najat",      "Bahia",
    "Haya",    "Maazouza", "Fadia",      "Walaa",     "Ayat",      "Aya",     "Ikram",     "Bushra",     "Nasim",
    "Nasra",   "Arwa",     "Lamia",      "Salma",     "Maram",     "Hikmat",  "Shaimaa",   "Bayan",      "Ihsan",
    "Awatif",  "Farah",    "Diana",      "Nora",      "Linda",     "Farida",  "Fahima",    "Najma",      "Suhaila",
    "Donia",   "Amna",     "Nermin",     "Wesal",     "Fayrouz",   "Rabab",   "Sonia",     "Lana",       "Janan",
    "Dalia",   "Azza",     "Tamara",     "Nazira",    "Badia",     "Dima",    "Hadia",     "Mary",       "Afnan",
    "Nancy",   "Ria",      "Asia",       "Shahnaz",   "Dana",      "Tasnim",  "Lamis",     "Hanna",      "Raja",
    "Naima",   "Rani",     "Razan",      "Wiam",      "Malak",     "Sally",   "Anaya",     "Rose",       "Saba",
    "Camelia", "Mansoura", "Safia",      "Fattoum",   "Naama",     "Nargis",  "Bahija",    "Yousra",     "Amira",
    "Hilda",   "Hajar",    "Maria",      "Rama",      "Wahiba",    "Lara",    "Sima",      "Jacqueline", "Faiza",
    "Janet",   "Aman",     "Maysa",      "Kamelia",   "Georgette", "Nasima",  "Mari",      "Sumaya",     "Nicola",
    "Nadine",  "Marilyn",  "Fawzia",     "Yara",      "Amani",     "Elin",    "Batoul",    "Asma",       "Marzouka",
    "Fatna",   "Hassiba",  "Sajida",     "Karem",     "Rita",      "Helen",   "Amina",     "Ghalia",     "Nelly",
    "Reina",   "Rene",     "Mira",       "Samiha",    "Nahida",    "Diane",   "Madeleine", "Elissa",     "Sama",
    "Dahab",   "Salsabil", "Rokaya",     "Liana",     "Shaheen",   "Samara",  "Raya",      "Rahima",     "Norma",
    "Farhat",  "Elaine",   "Hina",       "Suzy",      "Sandy",     "Judy",    "Julia",     "Lydia",      "Violet",
    "Nayla",   "Saadia",   "Hasna",      "Lorraine",  "Sahra",     "Mimouna", "Jane",      "Karima",     "Therese",
    "Nawar",   "Badra",    "Margret",    "Rayan",     "Caroline",  "Marina",  "Yacine",    "Mirna",      "Lucy",
    "Dania",   "Jannat",   "Hilde",      "Christine", "Abrar",     "Elia",    "Ayada",     "Hawa",       "Sandra",
    "Najia",   "Angie",    "Soria",      "Liza",      "Katrina",   "Vera",    "Melia",     "Mahdia",     "Mounira",
    "Emily",   "Houria",   "Antoinette", "Mimi",      "Irene",     "Badria",  "Sabra",     "Tania",      "Olga",
    "Ala",     "Sina",     "Ana",        "Milada",    "Lilian",    "Zeina",   "Fatin",     "Omaima",     "Nana",
    "Zaida",   "Ramos",    "Carmen",     "Josephine", "Laura",     "Joan",    "Isha",      "Elina",      "Anahid",
    "Sinai",   "Nahid",    "Christina",  "Nina",      "Elian",     "An",      "Maida",     "Nafisa",     "Elizabeth",
    "Juliana", "Hamida",   "Jalila",     "Joanna",    "Nirmin",    "Francis", "Claire",    "Margo",      "Sabria",
    "Kitty",   "Rachida",  "Mariana",    "Natalia",   "Salima",    "Kamilia", "Doha",      "Karol",      "Catherine",
    "Camille", "Hakima",   "Nozha",      "Pauline",   "Andrea",    "Tatiana", "Arlette",   "Sare",       "Sakina",
    "Dany",    "Amelia",   "Yelena",     "Grace",     "Marian",    "Liane",   "Serena",    "Yana",
});

const auto palestinianLastNames = std::to_array<std::string_view>({
    "Awad",        "Al najjar",  "El masry",    "Odeh",        "Hamdan",       "Saleh",        "Al khatib",
    "Shaheen",     "Mansour",    "Mousa",       "Ahmed",       "Yassin",       "Hamad",        "Salama",
    "Eisaa",       "Dawood",     "Salah",       "Hussein",     "Hassan",       "Abdullah",     "Hammad",
    "Suleiman",    "Shehadeh",   "El shaer",    "Abed",        "Taha",         "Zaaroub",      "Ashour",
    "Nassar",      "Hamouda",    "Saad",        "Al astal",    "Jaradat",      "Ayad",         "Ali",
    "Khalil",      "Amr",        "Basharat",    "Ibrahim",     "Salem",        "Zaid",         "Jaber",
    "Alyan",       "Rabie",      "Gabr",        "Obaid",       "Mohamed",      "Omar",         "Musleh",
    "Daraghmeh",   "Khatib",     "Hegazy",      "Ghanim",      "Qasim",        "Bani odeh",    "Ismail",
    "Qadeeh",      "Abou ali",   "Abu bakr",    "Zidan",       "Hamed",        "Salman",       "Jouda",
    "Darwish",     "Dweikat",    "El sherif",   "Sobeeh",      "Najjar",       "Amer",         "Nasser",
    "Badr",        "El madhoun", "Al natsheh",  "Matar",       "Barakat",      "Nasr",         "Mostafa",
    "Al farra",    "Khader",     "El aga",      "Yousef",      "Radwan",       "Abu al-rab",   "Siam",
    "Mari",        "Hamid",      "Osman",       "Samara",      "Al sheikh",    "Abu odeh",     "Nofal",
    "Sobh",        "Adwan",      "Abu jazar",   "Al jaabari",  "Sabah",        "Saeed",        "Khalaf",
    "Safi",        "Ghannam",    "Sawafet",     "Eid",         "Jamal",        "Mahmoud",      "Zayed",
    "Younis",      "El taweel",  "Hassouna",    "Assaf",       "Nazzal",       "Jarrar",       "Abou mostafa",
    "Madi",        "Shaat",      "Abu taha",    "Hawtha",      "Shalalda",     "Salah el din", "Zaqout",
    "Salim",       "Abu daqah",  "Shurrab",     "Abdel aal",   "Abou omra",    "Mounaser",     "Jabareen",
    "Ramadan",     "Shamasneh",  "Ghaith",      "Rayan",       "Muslim",       "Abdo",         "Azzam",
    "Al-qadi",     "El attar",   "Al awadh",    "Attallah",    "Taqateq",      "Baraka",       "Qabha",
    "Afaneh",      "Assi",       "El gamal",    "Abu aram",    "Hamail",       "El kahlout",   "Abed rabbo",
    "Abu deh",     "Elhamham",   "Al akkad",    "Fakih",       "Kandil",       "Fayad",        "Badwan",
    "Abdel hadi",  "Helis",      "Ayyash",      "Diab",        "Al ashqar",    "Jibrin",       "Abouwarda",
    "El hajj",     "Massoud",    "Al kafarneh", "Abu shrkh",   "Abu shamaleh", "El amour",     "Rajabi",
    "Badawi",      "Masry",      "Haroub",      "Abu samra",   "Aboaid",       "Qeshtah",      "Zahran",
    "Gamhor",      "Jabari",     "Khalifa",     "Kanaan",      "Natsheh",      "El louh",      "Saqr",
    "Abu obaid",   "Zkarneh",    "Shaaban",     "Muhanna",     "Melhem",       "El halaby",    "Mohsen",
    "Abou zeid",   "Hanani",     "Yahya",       "Jadallah",    "Abu awad",     "Akl",          "Rabayeh",
    "Abou hassan", "Hamada",     "Maslamah",    "Abdeen",      "Abu labdah",   "Jahalin",      "Abu hilal",
    "Bakr",        "Alawneh",    "Omran",       "Hajjaj",      "Erekat",       "Bashir",       "Abou omar",
    "Salahat",     "Abou farah", "Juma",        "Zaher",       "Kamil",        "Adais",        "Abou hammad",
    "Halaik",      "Khattab",    "El haddad",   "Abdelkader",  "Al ghoul",     "Said",         "Badran",
    "Asaad",       "Tmazeh",     "Mekdad",      "Qawasmeh",    "Al hindi",     "Asfour",       "Abu jarad",
    "Ayesh",       "Nasrallah",  "Debabseh",    "Rajoub",      "Ghonaim",      "El helou",     "Kallab",
    "Shabir",      "Choucair",   "Abou taema",  "Zioud",       "Ziadeh",       "Abdul rahman", "Shraim",
    "Al nahhal",   "Arafat",     "Dhereh",      "Ragab",       "Farouk",       "Abu arra",     "Abusnina",
    "Nabhan",      "El ghalban", "Al qawasmeh", "Yacoub",      "Harb",         "Allan",        "Ghoban",
    "Al bayouk",   "Faroun",     "Barbakh",     "Al sultan",   "Sarsour",      "Al atrash",    "Fares",
    "Maarouf",     "Khadour",    "Hammadah",    "Dheir",       "Wahdan",       "Khamaisah",    "Hassanein",
    "Al khalidi",  "Habib",      "Al shawwa",   "Sarhan",      "Faraj",        "Labd",         "El sakka",
    "Rehan",       "Ammar",      "Dalloul",     "Al buhaisi",  "Awadallah",    "Hamidat",      "El banna",
    "Al-bazour",   "Sweety",     "Abu musa",    "Al maslamah", "Samour",       "El zaaneen",   "Kaabneh",
    "Aslim",       "Zeidat",     "Shalaby",     "Abu shaaban", "Kouar",        "Faraj allah",  "Mahamid",
    "El sayed",    "Abu zahra",  "Al shurafa",  "Ghazal",
});

const NameFormats palestinianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames palestinianPeopleNames{{palestinianMaleFirstNames, palestinianLastNames, {}, {}},
                                         {palestinianFemaleFirstNames, palestinianLastNames, {}, {}},
                                         palestinianNameFormats};

// Persian

const auto persianMaleFirstNames = std::to_array<std::string_view>({
    "آرمان",  "بهروز",   "کیان",  "فریدون",  "سپهر",    "میلاد", "آرشام",    "پوریا",   "بهراد",   "سیاوش", "احسان",
    "شایان",  "شهریار",  "آرتین", "امیدوار", "پارسا",   "کاوه",  "پیام",     "آرین",    "مانی",    "نیما",  "فرهاد",
    "کسری",   "کیوان",   "رهام",  "فرشاد",   "آرمین",   "فرزاد", "ساماندار", "امینوار", "امیرعلی", "رضوان", "حسینیان",
    "ناصر",   "محمدرضا", "صادق",  "علی",     "عباس",    "حمید",  "جواد",     "محمود",   "علیرضا",  "رامبد", "پیمان",
    "مهدی",   "سجاد",    "محمد",  "یوسف",    "محمدعلی", "حسین",  "رحمان",    "رضا",     "کاظم",    "مهران", "نادر",
    "مهدیار", "پرهام",   "پیروز", "میثم",    "احمد",    "مصطفی",
});

const auto persianFemaleFirstNames = std::to_array<std::string_view>({
    "آتنا",  "پریسا",   "نازنین", "شیرین", "الناز", "سارا",  "زهرا",     "ریحانه", "بهار",  "آیدا",   "نیکا",
    "مهسا",  "نگین",    "یاسمن",  "آرزو",  "لاله",  "روژین", "ترانه",    "پرنیا",  "مریم",  "پگاه",   "سلما",
    "ملیسا", "نسترن",   "مینا",   "پرشین", "مهتا",  "لیلا",  "مهرانگیز", "زینب",   "آزیتا", "پرستو",  "رها",
    "شهناز", "سوگند",   "فرانکا", "لیانا", "سمیرا", "فاطمه", "سهراب",    "هلنا",   "دلارا", "پارمیس", "آوا",
    "پانیذ", "آناهیتا", "سحر",    "ناهید", "آتوسا", "بهناز", "راحله",    "شبنم",   "فریبا", "نرگس",   "رها",
    "سهیلا", "شیدا",    "محیا",   "زهره",  "زیبا",  "رقیه",  "مهتاب",    "الهه",   "ریما",  "مهدیس",
});

const auto persianLastNames = std::to_array<std::string_view>({
    "خانی",
    "رحمانی",
    "احمدی",
    "زاده",
    "نژاد",
    "صفری",
    "جعفری",
    "محمدی",
    "اکبری",
    "میرزایی",
    "فرهادی",
    "رستمی",
    "نوری",
    "جلالی",
    "حسینی",
    "رسولی",
    "موسوی",
    "یوسفی",
    "مقدم",
    "پورمحمد",
    "بهرامی",
    "قربانی",
    "فیاض",
    "فرجی",
    "بخشی",
    "بهشتی",
    "سلطانی",
    "قاسمی",
    "جوادی",
    "جهانگیری",
    "نصیری",
    "عبداللهی",
    "علیزاده",
    "مجد",
    "مظاهری",
    "هاشمی",
    "پارسا",
    "پیروز",
    "حقیقت",
    "راد",
    "زمانی",
    "سلمانی",
    "شیرازی",
    "قنبری",
    "طاهری",
    "مهدوی",
    "نعمتی",
    "وکیلی",
    "کاظمی",
    "رئیسی",
    "میری",
    "سلیمانی",
    "مومنی",
    "جوکار",
    "بنیادی",
    "حیدری",
    "افشار",
    "شمس",
    "کشاورز",
    "رشیدی",
    "صادقی",
    "حق‌پرست",
    "بیگی",
    "حسن‌زاده",
    "قریشی",
    "بابایی",
    "نعیمی",
    "موحد",
    "خضری",
    "داوری",
    "نیکو",
    "پورجعفر",
    "طباطبایی",
    "گلزار",
    "صباحی",
    "خزایی",
    "رفیعی",
    "حقی",
    "هاشمیان",
    "سجادی",
    "توکلی",
    "یزدانی",
    "شهیدی",
    "پورفرج",
    "آقاجانی",
    "پارسایی",
    "نادری",
    "آهنگری",
    "اکتایی",
    "زند",
    "تقی‌زاده",
    "جوانمرد",
    "محمدیان",
    "خلج",
    "زندی",
    "سلیمی",
    "سلطانپور",
    "کمالی",
    "باقری",
    "فرج‌اللهی",
    "زنجانی",
    "مهدی‌زاده",
    "فرجامی",
    "قنبرزاده",
    "رهنما",
    "خسروجاه",
    "تهرانی",
    "اسدی",
    "معتمدی",
    "حسینی‌نژاد",
    "نظری",
});

const auto persianMalePrefixes = std::to_array<std::string_view>({"آقا", "دکتر", "استاد"});

const auto persianFemalePrefixes = std::to_array<std::string_view>({"خانم", "دکتر", "استاد"});

const NameFormats persianNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

// Convert UTF16 to UTF8 for Persian characters then use the data.
const PeopleNames persianPeopleNames{{persianMaleFirstNames, persianLastNames, persianMalePrefixes, {}},
                                     {persianFemaleFirstNames, persianLastNames, persianFemalePrefixes, {}},
                                     persianNameFormats};

// Poland

const auto polishMaleFirstNames = std::to_array<std::string_view>({
    "Aaron",      "Abraham",     "Adam",        "Adrian",     "Atanazy",    "Agaton",      "Alan",       "Albert",
    "Aleksander", "Aleksy",      "Alfred",      "Alwar",      "Ambroży",    "Anatol",      "Andrzej",    "Antoni",
    "Apollinary", "Apollo",      "Arkady",      "Arkadiusz",  "Archibald",  "Arystarch",   "Arnold",     "Arseniusz",
    "Artur",      "August",      "Baldwin",     "Bazyli",     "Benedykt",   "Beniamin",    "Bernard",    "Bertrand",
    "Bertram",    "Borys",       "Brajan",      "Bruno",      "Cezary",     "Cecyliusz",   "Karol",      "Krystian",
    "Krzysztof",  "Klarencjusz", "Klaudiusz",   "Klemens",    "Konrad",     "Konstanty",   "Konstantyn", "Kornel",
    "Korneliusz", "Korneli",     "Cyryl",       "Cyrus",      "Damian",     "Daniel",      "Dariusz",    "Dawid",
    "Dionizy",    "Demetriusz",  "Dominik",     "Donald",     "Dorian",     "Edgar",       "Edmund",     "Edward",
    "Edwin",      "Efrem",       "Efraim",      "Eliasz",     "Eleazar",    "Emil",        "Emanuel",    "Erast",
    "Ernest",     "Eugeniusz",   "Eustracjusz", "Fabian",     "Feliks",     "Florian",     "Franciszek", "Fryderyk",
    "Gabriel",    "Gedeon",      "Galfryd",     "Jerzy",      "Gerald",     "Gerazym",     "Gilbert",    "Gonsalwy",
    "Grzegorz",   "Gwido",       "Harald",      "Henryk",     "Herbert",    "Herman",      "Hilary",     "Horacy",
    "Hubert",     "Hugo",        "Ignacy",      "Igor",       "Hilarion",   "Innocenty",   "Hipolit",    "Ireneusz",
    "Erwin",      "Izaak",       "Izajasz",     "Izydor",     "Jakub",      "Jeremi",      "Jeremiasz",  "Hieronim",
    "Joachim",    "Jan",         "Janusz",      "Jonatan",    "Józef",      "Jozue",       "Julian",     "Juliusz",
    "Justyn",     "Kalistrat",   "Kazimierz",   "Wawrzyniec", "Laurenty",   "Laurencjusz", "Łazarz",     "Leon",
    "Leonard",    "Leonid",      "Ludwik",      "Łukasz",     "Lucjan",     "Magnus",      "Makary",     "Marceli",
    "Marek",      "Marcin",      "Mateusz",     "Maurycy",    "Maksym",     "Maksymilian", "Michał",     "Miron",
    "Modest",     "Mojżesz",     "Natan",       "Natanael",   "Nazariusz",  "Nazary",      "Nestor",     "Mikołaj",
    "Nikodem",    "Olaf",        "Oleg",        "Oliwier",    "Onufry",     "Orestes",     "Oskar",      "Ansgary",
    "Osmund",     "Pankracy",    "Pantaleon",   "Patryk",     "Patrycjusz", "Patrycy",     "Paweł",      "Piotr",
    "Filemon",    "Filip",       "Platon",      "Polikarp",   "Porfiry",    "Porfiriusz",  "Prokles",    "Prokul",
    "Prokop",     "Kwintyn",     "Randolf",     "Rafał",      "Rajmund",    "Reginald",    "Rajnold",    "Ryszard",
    "Robert",     "Roderyk",     "Roger",       "Roland",     "Roman",      "Romeo",       "Rudolf",     "Samson",
    "Samuel",     "Salwator",    "Sebastian",   "Serafin",    "Sergiusz",   "Seweryn",     "Zygmunt",    "Sylwester",
    "Szymon",     "Salomon",     "Spirydion",   "Stanisław",  "Szczepan",   "Stefan",      "Terencjusz", "Teodor",
    "Tomasz",     "Tymoteusz",   "Tobiasz",     "Walenty",    "Walentyn",   "Walerian",    "Walery",     "Wiktor",
    "Wincenty",   "Witalis",     "Włodzimierz", "Władysław",  "Błażej",     "Walter",      "Walgierz",   "Wacław",
    "Wilfryd",    "Wilhelm",     "Ksawery",     "Ksenofont",  "Zachariasz", "Zachary",
});

const auto polishFemaleFirstNames = std::to_array<std::string_view>({
    "Ada",        "Adelajda",   "Agata",     "Agnieszka", "Agrypina",   "Aida",      "Aleksandra", "Alicja",
    "Alina",      "Amanda",     "Anastazja", "Angela",    "Andżelika",  "Angelina",  "Anna",       "Hanna",
    "Antonina",   "Ariadna",    "Aurora",    "Barbara",   "Beatrycze",  "Berta",     "Brygida",    "Kamila",
    "Karolina",   "Kornelia",   "Katarzyna", "Cecylia",   "Chloe",      "Krystyna",  "Klara",      "Klaudia",
    "Klementyna", "Konstancja", "Koralia",   "Daria",     "Diana",      "Dina",      "Dorota",     "Edyta",
    "Eleonora",   "Eliza",      "Elżbieta",  "Izabela",   "Elwira",     "Emilia",    "Estera",     "Eudoksja",
    "Eudokia",    "Eugenia",    "Ewa",       "Ewelina",   "Ferdynanda", "Florencja", "Franciszka", "Gabriela",
    "Gertruda",   "Gloria",     "Gracja",    "Jadwiga",   "Helena",     "Henryka",   "Nadzieja",   "Ida",
    "Ilona",      "Irena",      "Irma",      "Izolda",    "Jakubina",   "Joanna",    "Janina",     "Żaneta",
    "Ginewra",    "Józefina",   "Judyta",    "Julia",     "Julita",     "Justyna",   "Kira",       "Cyra",
    "Kleopatra",  "Larysa",     "Laura",     "Laurencja", "Laurentyna", "Lea",       "Leila",      "Liliana",
    "Lilianna",   "Lilia",      "Lilla",     "Liza",      "Ludwika",    "Luiza",     "Łucja",      "Lucja",
    "Lidia",      "Amabela",    "Magdalena", "Malwina",   "Małgorzata", "Greta",     "Marianna",   "Maryna",
    "Marta",      "Martyna",    "Maria",     "Matylda",   "Maja",       "Melania",   "Michalina",  "Monika",
    "Noemi",      "Natalia",    "Nikola",    "Nina",      "Olga",       "Olimpia",   "Oliwia",     "Ofelia",
    "Patrycja",   "Paula",      "Pelagia",   "Penelopa",  "Filipa",     "Paulina",   "Rachela",    "Rebeka",
    "Regina",     "Renata",     "Rozalia",   "Róża",      "Roksana",    "Rufina",    "Ruta",       "Sabina",
    "Sara",       "Serafina",   "Sybilla",   "Sylwia",    "Zofia",      "Stella",    "Stefania",   "Zuzanna",
    "Tamara",     "Tacjana",    "Tekla",     "Teodora",   "Teresa",     "Walentyna", "Waleria",    "Wanesa",
    "Wiara",      "Weronika",   "Wiktoria",  "Wirginia",  "Bibiana",    "Bibianna",  "Wanda",      "Wilhelmina",
    "Ksawera",    "Ksenia",     "Zoe",
});

const auto polishLastNames = std::to_array<std::string_view>({
    "Adamczak",     "Adamczyk",      "Adamek",
    "Adamiak",      "Adamiec",       "Adamowicz",
    "Adamski",      "Adamus",        "Aleksandrowicz",
    "Andrzejczak",  "Andrzejewski",  "Antczak",
    "Augustyn",     "Augustyniak",   "Bagiński",
    "Balcerzak",    "Banach",        "Banasiak",
    "Banasik",      "Banaś",         "Baran",
    "Baranowski",   "Barański",      "Bartczak",
    "Bartkowiak",   "Bartnik",       "Bartosik",
    "Bednarczyk",   "Bednarek",      "Bednarski",
    "Bednarz",      "Białas",        "Białek",
    "Białkowski",   "Biedrzyński",   "Bielak",
    "Bielawski",    "Bielecki",      "Bielski",
    "Bieniek",      "Biernacki",     "Biernat",
    "Bieńkowski",   "Bilski",        "Bober",
    "Bochenek",     "Bogucki",       "Bogusz",
    "Borek",        "Borkowski",     "Borowiec",
    "Borowski",     "Bożek",         "Broda",
    "Brzeziński",   "Brzozowski",    "Buczek",
    "Buczkowski",   "Buczyński",     "Budziński",
    "Budzyński",    "Bujak",         "Bukowski",
    "Burzyński",    "Bąk",           "Bąkowski",
    "Błaszczak",    "Błaszczyk",     "Cebula",
    "Chmiel",       "Chmielewski",   "Chmura",
    "Chojnacki",    "Chojnowski",    "Cholewa",
    "Chrzanowski",  "Chudzik",       "Cichocki",
    "Cichoń",       "Cichy",         "Ciesielski",
    "Cieśla",       "Cieślak",       "Cieślik",
    "Ciszewski",    "Cybulski",      "Cygan",
    "Czaja",        "Czajka",        "Czajkowski",
    "Czapla",       "Czarnecki",     "Czech",
    "Czechowski",   "Czekaj",        "Czerniak",
    "Czerwiński",   "Czyż",          "Czyżewski",
    "Dec",          "Dobosz",        "Dobrowolski",
    "Dobrzyński",   "Domagała",      "Domański",
    "Dominiak",     "Drabik",        "Drozd",
    "Drozdowski",   "Drzewiecki",    "Dróżdż",
    "Dubiel",       "Duda",          "Dudek",
    "Dudziak",      "Dudzik",        "Dudziński",
    "Duszyński",    "Dziedzic",      "Dziuba",
    "Dąbek",        "Dąbkowski",     "Dąbrowski",
    "Dębowski",     "Dębski",        "Długosz",
    "Falkowski",    "Fijałkowski",   "Filipek",
    "Filipiak",     "Filipowicz",    "Flak",
    "Flis",         "Florczak",      "Florek",
    "Frankowski",   "Frąckowiak",    "Frączek",
    "Frątczak",     "Furman",        "Gadomski",
    "Gajda",        "Gajewski",      "Gaweł",
    "Gawlik",       "Gawron",        "Gawroński",
    "Gałka",        "Gałązka",       "Gil",
    "Godlewski",    "Golec",         "Gołąb",
    "Gołębiewski",  "Gołębiowski",   "Grabowski",
    "Graczyk",      "Grochowski",    "Grudzień",
    "Gruszczyński", "Gruszka",       "Grzegorczyk",
    "Grzelak",      "Grzesiak",      "Grzesik",
    "Grześkowiak",  "Grzyb",         "Grzybowski",
    "Grzywacz",     "Gutowski",      "Guzik",
    "Gwoździk",     "Gwóźdź",        "Góra",
    "Góral",        "Górecki",       "Górka",
    "Górniak",      "Górny",         "Górski",
    "Gąsior",       "Gąsiorowski",   "Głogowski",
    "Głowacki",     "Głąb",          "Hajduk",
    "Herman",       "Iwanowski",     "Iwański",
    "Izdebski",     "Jabłoński",     "Jackowski",
    "Jagielski",    "Jagiełło",      "Jagodziński",
    "Jakubiak",     "Jakubowski",    "Janas",
    "Janiak",       "Janicki",       "Janik",
    "Janiszewski",  "Jankowiak",     "Jankowski",
    "Janowski",     "Janus",         "Janusz",
    "Januszewski",  "Jaros",         "Jarosz",
    "Jarząbek",     "Jasiński",      "Jastrzębski",
    "Jaworski",     "Jaśkiewicz",    "Jezierski",
    "Jurek",        "Jurkiewicz",    "Jurkowski",
    "Juszczak",     "Jóźwiak",       "Jóźwik",
    "Jędrzejczak",  "Jędrzejczyk",   "Jędrzejewski",
    "Kacprzak",     "Kaczmarczyk",   "Kaczmarek",
    "Kaczmarski",   "Kaczor",        "Kaczorowski",
    "Kaczyński",    "Kaleta",        "Kalinowski",
    "Kalisz",       "Kamiński",      "Kania",
    "Kaniewski",    "Kapusta",       "Karaś",
    "Karczewski",   "Karpiński",     "Karwowski",
    "Kasperek",     "Kasprzak",      "Kasprzyk",
    "Kaszuba",      "Kawa",          "Kawecki",
    "Kałuża",       "Kaźmierczak",   "Kiełbasa",
    "Kisiel",       "Kita",          "Klimczak",
    "Klimek",       "Kmiecik",       "Kmieć",
    "Knapik",       "Kobus",         "Kogut",
    "Kolasa",       "Komorowski",    "Konieczna",
    "Konieczny",    "Konopka",       "Kopczyński",
    "Koper",        "Kopeć",         "Korzeniowski",
    "Kos",          "Kosiński",      "Kosowski",
    "Kostecki",     "Kostrzewa",     "Kot",
    "Kotowski",     "Kowal",         "Kowalczuk",
    "Kowalczyk",    "Kowalewski",    "Kowalik",
    "Kowalski",     "Koza",          "Kozak",
    "Kozieł",       "Kozioł",        "Kozłowski",
    "Kołakowski",   "Kołodziej",     "Kołodziejczyk",
    "Kołodziejski", "Krajewski",     "Krakowiak",
    "Krawczyk",     "Krawiec",       "Kruk",
    "Krukowski",    "Krupa",         "Krupiński",
    "Kruszewski",   "Krysiak",       "Krzemiński",
    "Krzyżanowski", "Król",          "Królikowski",
    "Książek",      "Kubacki",       "Kubiak",
    "Kubica",       "Kubicki",       "Kubik",
    "Kuc",          "Kucharczyk",    "Kucharski",
    "Kuchta",       "Kuciński",      "Kuczyński",
    "Kujawa",       "Kujawski",      "Kula",
    "Kulesza",      "Kulig",         "Kulik",
    "Kuliński",     "Kurek",         "Kurowski",
    "Kuś",          "Kwaśniewski",   "Kwiatkowski",
    "Kwiecień",     "Kwieciński",    "Kędzierski",
    "Kędziora",     "Kępa",          "Kłos",
    "Kłosowski",    "Lach",          "Laskowski",
    "Lasota",       "Lech",          "Lenart",
    "Lesiak",       "Leszczyński",   "Lewandowski",
    "Lewicki",      "Leśniak",       "Leśniewski",
    "Lipiński",     "Lipka",         "Lipski",
    "Lis",          "Lisiecki",      "Lisowski",
    "Maciejewski",  "Maciąg",        "Mackiewicz",
    "Madej",        "Maj",           "Majcher",
    "Majchrzak",    "Majewski",      "Majka",
    "Makowski",     "Malec",         "Malicki",
    "Malinowski",   "Maliszewski",   "Marchewka",
    "Marciniak",    "Marcinkowski",  "Marczak",
    "Marek",        "Markiewicz",    "Markowski",
    "Marszałek",    "Marzec",        "Masłowski",
    "Matusiak",     "Matuszak",      "Matuszewski",
    "Matysiak",     "Mazur",         "Mazurek",
    "Mazurkiewicz", "Maćkowiak",     "Małecki",
    "Małek",        "Maślanka",      "Michalak",
    "Michalczyk",   "Michalik",      "Michalski",
    "Michałek",     "Michałowski",   "Mielczarek",
    "Mierzejewski", "Mika",          "Miklaś",
    "Mikołajczak",  "Mikołajczyk",   "Mikulski",
    "Milczarek",    "Milewski",      "Miller",
    "Misiak",       "Misztal",       "Miśkiewicz",
    "Modzelewski",  "Molenda",       "Morawski",
    "Motyka",       "Mroczek",       "Mroczkowski",
    "Mrozek",       "Mróz",          "Mucha",
    "Murawski",     "Musiał",        "Muszyński",
    "Młynarczyk",   "Napierała",     "Nawrocki",
    "Nawrot",       "Niedziela",     "Niedzielski",
    "Niedźwiecki",  "Niemczyk",      "Niemiec",
    "Niewiadomski", "Noga",          "Nowacki",
    "Nowaczyk",     "Nowak",         "Nowakowski",
    "Nowicki",      "Nowiński",      "Olczak",
    "Olejniczak",   "Olejnik",       "Olszewski",
    "Orzechowski",  "Orłowski",      "Osiński",
    "Ossowski",     "Ostrowski",     "Owczarek",
    "Paczkowski",   "Pająk",         "Pakuła",
    "Palacz",       "Paluch",        "Panek",
    "Partyka",      "Pasternak",     "Paszkowski",
    "Pawelec",      "Pawlak",        "Pawlicki",
    "Pawlik",       "Pawlikowski",   "Pawłowski",
    "Pałka",        "Piasecki",      "Piechota",
    "Pieczek",      "Piekarski",     "Pietras",
    "Pietruszka",   "Pietrzak",      "Pietrzyk",
    "Pilarski",     "Pilch",         "Piotrowicz",
    "Piotrowski",   "Piwowarczyk",   "Piórkowski",
    "Piątek",       "Piątkowski",    "Piłat",
    "Pluta",        "Podgórski",     "Polak",
    "Popławski",    "Porębski",      "Prokop",
    "Prus",         "Przybylski",    "Przybysz",
    "Przybył",      "Przybyła",      "Ptak",
    "Puchalski",    "Pytel",         "Płonka",
    "Raczyński",    "Radecki",       "Radomski",
    "Rak",          "Rakowski",      "Ratajczak",
    "Robak",        "Rogala",        "Rogalski",
    "Rogowski",     "Rojek",         "Romanowski",
    "Rosa",         "Rosiak",        "Rosiński",
    "Ruciński",     "Rudnicki",      "Rudziński",
    "Rudzki",       "Rusin",         "Rutkowski",
    "Rybak",        "Rybarczyk",     "Rybicki",
    "Rzepka",       "Różański",      "Różycki",
    "Sadowski",     "Sawicki",       "Serafin",
    "Siedlecki",    "Sienkiewicz",   "Sieradzki",
    "Sikora",       "Sikorski",      "Sitek",
    "Siwek",        "Skalski",       "Skiba",
    "Skibiński",    "Skoczylas",     "Skowron",
    "Skowronek",    "Skowroński",    "Skrzypczak",
    "Skrzypek",     "Skóra",         "Smoliński",
    "Sobczak",      "Sobczyk",       "Sobieraj",
    "Sobolewski",   "Socha",         "Sochacki",
    "Sokołowski",   "Sokół",         "Sosnowski",
    "Sowa",         "Sowiński",      "Sołtys",
    "Sołtysiak",    "Sroka",         "Stachowiak",
    "Stachowicz",   "Stachura",      "Stachurski",
    "Stanek",       "Staniszewski",  "Stanisławski",
    "Stankiewicz",  "Stasiak",       "Staszewski",
    "Stawicki",     "Stec",          "Stefaniak",
    "Stefański",    "Stelmach",      "Stolarczyk",
    "Stolarski",    "Strzelczyk",    "Strzelecki",
    "Stępień",      "Stępniak",      "Surma",
    "Suski",        "Szafrański",    "Szatkowski",
    "Szczepaniak",  "Szczepanik",    "Szczepański",
    "Szczerba",     "Szcześniak",    "Szczygieł",
    "Szczęsna",     "Szczęsny",      "Szeląg",
    "Szewczyk",     "Szostak",       "Szulc",
    "Szwarc",       "Szwed",         "Szydłowski",
    "Szymański",    "Szymczak",      "Szymczyk",
    "Szymkowiak",   "Szyszka",       "Sławiński",
    "Słowik",       "Słowiński",     "Tarnowski",
    "Tkaczyk",      "Tokarski",      "Tomala",
    "Tomaszewski",  "Tomczak",       "Tomczyk",
    "Tomys",        "Tracz",         "Trojanowski",
    "Trzciński",    "Trzeciak",      "Turek",
    "Twardowski",   "Urban",         "Urbanek",
    "Urbaniak",     "Urbanowicz",    "Urbańczyk",
    "Urbański",     "Walczak",       "Walkowiak",
    "Warchoł",      "Wasiak",        "Wasilewski",
    "Wawrzyniak",   "Wesołowski",    "Wieczorek",
    "Wierzbicki",   "Wilczek",       "Wilczyński",
    "Wilk",         "Winiarski",     "Witczak",
    "Witek",        "Witkowski",     "Wiącek",
    "Więcek",       "Więckowski",    "Wiśniewski",
    "Wnuk",         "Wojciechowski", "Wojtas",
    "Wojtasik",     "Wojtczak",      "Wojtkowiak",
    "Wolak",        "Woliński",      "Wolny",
    "Wolski",       "Woś",           "Woźniak",
    "Wrona",        "Wroński",       "Wróbel",
    "Wróblewski",   "Wypych",        "Wysocki",
    "Wyszyński",    "Wójcicki",      "Wójcik",
    "Wójtowicz",    "Wąsik",         "Wąsowski",
    "Węgrzyn",      "Włodarczyk",    "Włodarski",
    "Zaborowski",   "Zabłocki",      "Zagórski",
    "Zając",        "Zajączkowski",  "Zakrzewski",
    "Zalewski",     "Zaremba",       "Zarzycki",
    "Zaręba",       "Zawada",        "Zawadzki",
    "Zdunek",       "Zieliński",     "Zielonka",
    "Ziółkowski",   "Zięba",         "Ziętek",
    "Zwoliński",    "Zych",          "Zygmunt",
    "Łapiński",     "Łuczak",        "Łukasiewicz",
    "Łukasik",      "Łukaszewski",   "Śliwa",
    "Śliwiński",    "Ślusarczyk",    "Świderski",
    "Świerczyński", "Świątek",       "Żak",
    "Żebrowski",    "Żmuda",         "Żuk",
    "Żukowski",     "Żurawski",      "Żurek",
    "Żyła",
});

const auto polishMalePrefixes = std::to_array<std::string_view>({"Pan"});

const auto polishFemalePrefixes = std::to_array<std::string_view>({"Pani"});

const NameFormats polishNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames polishPeopleNames{{polishMaleFirstNames, polishLastNames, polishMalePrefixes, {}},
                                    {polishFemaleFirstNames, polishLastNames, polishFemalePrefixes, {}},
                                    polishNameFormats};

// Portugal

const auto portugueseMaleFirstNames = std::to_array<std::string_view>({
    "Afonso",  "Alexandre", "André",     "Ângelo",   "António",   "Artur",    "Benjamim", "Bernardo",  "Bruno",
    "Carlos",  "César",     "Cristiano", "Daniel",   "David",     "Dinis",    "Diogo",    "Duarte",    "Edgar",
    "Eduardo", "Elias",     "Emanuel",   "Fábio",    "Feliciano", "Fernando", "Filipe",   "Francisco", "Frederico",
    "Gabriel", "Gaspar",    "Gil",       "Gonçalo",  "Guilherme", "Gustavo",  "Hélio",    "Henrique",  "Hugo",
    "Igor",    "Ígor",      "Isac",      "Ivan",     "Ivo",       "Jaime",    "João",     "Joaquim",   "Jorge",
    "José",    "Josué",     "Júlio",     "Leandro",  "Leonardo",  "Lourenço", "Lucas",    "Luís",      "Manel",
    "Manuel",  "Marcelo",   "Marco",     "Marcos",   "Mário",     "Martim",   "Mateus",   "Matias",    "Mauro",
    "Micael",  "Miguel",    "Moisés",    "Norberto", "Nuno",      "Paulo",    "Pedro",    "Rafael",    "Raul",
    "Renato",  "Ricardo",   "Roberto",   "Rodrigo",  "Romeu",     "Rúben",    "Rui",      "Salvador",  "Samuel",
    "Sandro",  "Santiago",  "Sebastião", "Sérgio",   "Simão",     "Tiago",    "Tomás",    "Tomé",      "Valentim",
    "Valter",  "Vasco",     "Vicente",   "Vítor",    "Xavier",
});

const auto portugueseFemaleFirstNames = std::to_array<std::string_view>({
    "Adriana",   "Alexandra", "Alice",    "Amélia",  "Ana",     "Ariana",    "Áurea",   "Aurora",   "Bárbara",
    "Beatriz",   "Benedita",  "Bruna",    "Caetana", "Camila",  "Carla",     "Carlota", "Carminho", "Carmo",
    "Carolina",  "Catarina",  "Cecília",  "Célia",   "Clara",   "Constança", "Daniela", "Débora",   "Diana",
    "Eduarda",   "Elisa",     "Ema",      "Emília",  "Érica",   "Eva",       "Fabiana", "Filipa",   "Flor",
    "Francisca", "Frederica", "Gabriela", "Helena",  "Inês",    "Irina",     "Íris",    "Isabel",   "Jéssica",
    "Joana",     "Júlia",     "Juliana",  "Julieta", "Lara",    "Laura",     "Leonor",  "Letícia",  "Lia",
    "Lorena",    "Luana",     "Luena",    "Luísa",   "Luna",    "Madalena",  "Mafalda", "Mara",     "Márcia",
    "Margarida", "Maria",     "Mariana",  "Marta",   "Matilde", "Melissa",   "Mia",     "Miriam",   "Natália",
    "Nicole",    "Núria",     "Ofélia",   "Olívia",  "Paula",   "Pilar",     "Rafaela", "Raquel",   "Rita",
    "Rosa",      "Safira",    "Sara",     "Sílvia",  "Sofia",   "Soraia",    "Tatiana", "Teresa",   "Valentina",
    "Vânia",     "Vera",      "Vitória",
});

const auto portugueseLastNames = std::to_array<std::string_view>({
    "Abreu",   "Albuquerque", "Almeida", "Alves",     "Amaral",     "Amorim",    "Andrade",   "Anjos",     "Antunes",
    "Araújo",  "Assunção",    "Azevedo", "Baptista",  "Barbosa",    "Barros",    "Batista",   "Borges",    "Braga",
    "Branco",  "Brito",       "Campos",  "Cardoso",   "Carneiro",   "Carvalho",  "Castro",    "Coelho",    "Correia",
    "Costa",   "Cruz",        "Cunha",   "Domingues", "Esteves",    "Faria",     "Fernandes", "Ferreira",  "Figueiredo",
    "Fonseca", "Freitas",     "Garcia",  "Gaspar",    "Gomes",      "Gonçalves", "Guerreiro", "Henriques", "Jesus",
    "Leal",    "Leite",       "Lima",    "Lopes",     "Loureiro",   "Lourenço",  "Macedo",    "Machado",   "Magalhães",
    "Maia",    "Marques",     "Martins", "Matias",    "Matos",      "Melo",      "Mendes",    "Miranda",   "Monteiro",
    "Morais",  "Moreira",     "Mota",    "Moura",     "Nascimento", "Neto",      "Neves",     "Nobre",     "Nogueira",
    "Nunes",   "Oliveira",    "Pacheco", "Paiva",     "Pereira",    "Pinheiro",  "Pinho",     "Pinto",     "Pires",
    "Ramos",   "Raposo",      "Reis",    "Ribeiro",   "Rocha",      "Rodrigues", "Santos",    "Saraiva",   "Silva",
    "Simões",  "Soares",      "Sousa",   "Sá",        "Tavares",    "Teixeira",  "Torres",    "Valente",   "Vaz",
    "Vicente", "Vieira",
});

const auto portugueseMalePrefixes = std::to_array<std::string_view>({"Sr.", "Dr.", "Prof.", "Eng."});

const auto portugueseFemalePrefixes = std::to_array<std::string_view>({"Sra.", "Dra.", "Prof.", "Eng."});

const NameFormats portugueseNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames portuguesePeopleNames{{portugueseMaleFirstNames, portugueseLastNames, portugueseMalePrefixes, {}},
                                        {portugueseFemaleFirstNames, portugueseLastNames, portugueseFemalePrefixes, {}},
                                        portugueseNameFormats};

// Romania

const auto romanianMaleFirstNames = std::to_array<std::string_view>({
    "Ion",      "Mihai",     "Andrei",   "Alexandru",  "George",    "Cristian",  "Florin",     "Răzvan",   "Lucian",
    "Adrian",   "Vlad",      "Gheorghe", "Constantin", "Daniel",    "Bogdan",    "Eduard",     "Valentin", "Gabriel",
    "Dumitru",  "Radu",      "Ștefan",   "Marin",      "Marian",    "Victor",    "Raul",       "Valeriu",  "Cosmin",
    "Marius",   "Mihail",    "Claudiu",  "Ciprian",    "Ionel",     "Cristinel", "Cătălin",    "Tudor",    "Cornel",
    "Corneliu", "Romeo",     "Romeu",    "Vasile",     "Petre",     "Alex",      "Liviu",      "Ionuț",    "Nicolae",
    "Nicoale",  "Octavian",  "Dan",      "Emil",       "Eugen",     "Horia",     "Gheorghiță", "Ghiță",    "Horea",
    "Matei",    "Sebastian", "Șerban",   "Viorel",     "Laurențiu", "Laurentiu", "Alin",       "Gavril",   "Ovidiu",
    "Sorin",    "Tiberiu",
});

const auto romanianFemaleFirstNames = std::to_array<std::string_view>({
    "Maria",    "Ioana",     "Elena",   "Ana",      "Cristina", "Andreea",   "Georgiana", "Alexandra", "Laura",
    "Diana",    "Raluca",    "Mihaela", "Larisa",   "Monica",   "Alina",     "Simona",    "Gabriela",  "Roxana",
    "Denisa",   "Camelia",   "Adriana", "Nicoleta", "Mariana",  "Carmen",    "Loredana",  "Irina",     "Liliana",
    "Eva",      "Elisabeta", "Aurelia", "Cătălina", "Silvia",   "Ana-Maria", "Doina",     "Florina",   "Magdalena",
    "Stefania", "Valentina", "Iuliana", "Ruxandra", "Georgeta", "Marilena",  "Lidia",     "Anda",      "Marinela",
    "Mirela",   "Flavia",    "Claudia", "Viorica",  "Sorina",   "Ramona",    "Sorana",    "Floriana",  "Livia",
    "Teodora",  "Magda",     "Marina",  "Dana",     "Lavinia",  "Anca",      "Delia",     "Daria",     "Oana",
});

const auto romanianLastNames = std::to_array<std::string_view>({
    "Popescu",  "Ionescu",   "Popa",     "Dumitru",   "Georgescu",    "Stan",       "Stoica",    "Constantin",
    "Barbu",    "Dobre",     "Nistor",   "Mihai",     "Cristea",      "Oprea",      "Dinu",      "Voicu",
    "Munteanu", "Neagu",     "Badea",    "Diaconu",   "Costache",     "Diaconescu", "Călinescu", "Luca",
    "Nicolae",  "Marinescu", "Petrescu", "Bădescu",   "Tudor",        "Cătănescu",  "Florescu",  "Gheorghiu",
    "Ioniță",   "Dobrescu",  "Bucur",    "Cojocaru",  "Preda",        "Mihăilescu", "Soare",     "Sava",
    "Albu",     "Moldovan",  "Zaharia",  "Balan",     "Vasilescu",    "Andrei",     "Stefan",    "Șerban",
    "Toma",     "Bogdan",    "Fodor",    "Sandu",     "Alexandrescu", "Sorin",      "Niculae",   "Oancea",
    "Mocanu",   "Radu",      "Gavril",   "Rusu",      "Filip",        "Ștefanescu", "Ene",       "Păun",
    "Moise",    "Dima",      "Covaci",   "Simion",    "Rădulescu",    "Cristea",    "Pascu",     "Hanganu",
    "Iacob",    "Coman",     "Lupu",     "Pavel",     "Ivan",         "Vlaicu",     "Anghel",    "Dumitrache",
    "Frățilă",  "Dincă",     "Brănescu", "Alexandru", "Căruntu",      "Iancu",      "Pintilie",  "Tudose",
    "Șchiopu",  "Petrică",   "Burcea",   "Manole",    "Mănescu",      "Panaite",    "Trandafir", "Mărgărit",
    "Dorin",    "Petrache",  "Bădăluță", "Mihu",      "Chiriac",
});

const auto romanianSuffixes = std::to_array<std::string_view>({"Jr.", "Sr."});

const auto romanianMalePrefixes = std::to_array<std::string_view>({"Dl", "Dra"});

const auto romanian_female_refixes = std::to_array<std::string_view>({"Dna", "Dra"});

const NameFormats romanianNameFormats{{{"{firstName} {lastName}", 49},
                                       {"{lastName} {firstName}", 7},
                                       {"{prefix} {firstName} {lastName}", 7},
                                       {"{firstName} {lastName} {suffix}", 7},
                                       {"{prefix} {firstName} {lastName} {suffix}", 1}}};

const PeopleNames romanianPeopleNames{
    {romanianMaleFirstNames, romanianLastNames, romanianMalePrefixes, romanianSuffixes},
    {romanianFemaleFirstNames, romanianLastNames, romanian_female_refixes, romanianSuffixes},
    romanianNameFormats};

// Russia

const auto russianMaleFirstNames = std::to_array<std::string_view>({
    "Авдей",      "Авксентий",  "Агапит",    "Агафон",    "Акакий",    "Акиндин",     "Александр", "Алексей",
    "Альберт",    "Анатолий",   "Андрей",    "Аникий",    "Аникита",   "Антон",       "Антонин",   "Анфим",
    "Аристарх",   "Аркадий",    "Арсений",   "Артём",     "Артемий",   "Артур",       "Архипп",    "Афанасий",
    "Богдан",     "Борис",      "Вавила",    "Вадим",     "Валентин",  "Валерий",     "Валерьян",  "Варлам",
    "Варсонофий", "Варфоломей", "Василий",   "Венедикт",  "Вениамин",  "Викентий",    "Виктор",    "Виссарион",
    "Виталий",    "Владимир",   "Владислав", "Владлен",   "Влас",      "Всеволод",    "Вячеслав",  "Гавриил",
    "Галактион",  "Геласий",    "Геннадий",  "Георгий",   "Герасим",   "Герман",      "Германн",   "Глеб",
    "Гордей",     "Григорий",   "Данакт",    "Даниил",    "Демид",     "Демьян",      "Денис",     "Дмитрий",
    "Добрыня",    "Донат",      "Дорофей",   "Евгений",   "Евграф",    "Евдоким",     "Евсей",     "Евстафий",
    "Егор",       "Емельян",    "Еремей",    "Ермолай",   "Ерофей",    "Ефим",        "Ефрем",     "Ждан",
    "Зиновий",    "Иакинф",     "Иван",      "Игнатий",   "Игорь",     "Изот",        "Илья",      "Иннокентий",
    "Ираклий",    "Ириней",     "Исаак",     "Исидор",    "Иуда",      "Иулиан",      "Капитон",   "Ким",
    "Кир",        "Кирилл",     "Климент",   "Кондрат",   "Конон",     "Константин",  "Корнилий",  "Кузьма",
    "Куприян",    "Лаврентий",  "Лев",       "Леонид",    "Леонтий",   "Логгин",      "Лука",      "Лукий",
    "Лукьян",     "Магистриан", "Макар",     "Максим",    "Мамонт",    "Марк",        "Мартын",    "Матвей",
    "Мелентий",   "Мина",       "Мирослав",  "Митрофан",  "Михаил",    "Мстислав",    "Назар",     "Нестор",
    "Никандр",    "Никанор",    "Никита",    "Никифор",   "Никодим",   "Николай",     "Никон",     "Олег",
    "Онисим",     "Онуфрий",    "Павел",     "Паисий",    "Панкратий", "Пантелеймон", "Парфений",  "Пафнутий",
    "Пахомий",    "Пётр",       "Платон",    "Поликарп",  "Порфирий",  "Потап",       "Пров",      "Прокопий",
    "Протасий",   "Прохор",     "Разумник",  "Родион",    "Роман",     "Ростислав",   "Руслан",    "Савва",
    "Савелий",    "Самуил",     "Святополк", "Святослав", "Севастьян", "Семён",       "Серафим",   "Сергей",
    "Сила",       "Сильвестр",  "Созон",     "Софрон",    "Спиридон",  "Станислав",   "Степан",    "Тарас",
    "Тимофей",    "Тимур",      "Тит",       "Тихон",     "Трифон",    "Трофим",      "Урбан",     "Фаддей",
    "Фёдор",      "Федосей",    "Федот",     "Феликс",    "Феоктист",  "Филат",       "Филимон",   "Филипп",
    "Фирс",       "Фока",       "Фома",      "Фотий",     "Фрол",      "Харитон",     "Хрисанф",   "Христофор",
});

const auto russianFemaleFirstNames = std::to_array<std::string_view>({
    "Агафья",    "Аглая",     "Агния",     "Агриппина", "Аза",        "Акулина",   "Алевтина",  "Александра",
    "Алина",     "Алиса",     "Алла",      "Анастасия", "Ангелина",   "Анжела",    "Анжелика",  "Анна",
    "Антонина",  "Анфиса",    "Валентина", "Валерия",   "Варвара",    "Василиса",  "Вера",      "Вероника",
    "Виктория",  "Владимира", "Галина",    "Глафира",   "Гликерия",   "Дана",      "Дарья",     "Евгения",
    "Евдокия",   "Евлалия",   "Евлампия",  "Евпраксия", "Евфросиния", "Екатерина", "Елена",     "Елизавета",
    "Епистима",  "Ермиония",  "Жанна",     "Зинаида",   "Злата",      "Зоя",       "Инга",      "Инесса",
    "Инна",      "Иоанна",    "Ираида",    "Ирина",     "Капитолина", "Карина",    "Каролина",  "Кира",
    "Клавдия",   "Ксения",    "Лада",      "Лариса",    "Лидия",      "Лилия",     "Любовь",    "Людмила",
    "Маргарита", "Марина",    "Мария",     "Марфа",     "Матрёна",    "Милана",    "Милица",    "Мирослава",
    "Надежда",   "Наталья",   "Нина",      "Нонна",     "Оксана",     "Октябрина", "Олимпиада", "Ольга",
    "Павлина",   "Пелагея",   "Пинна",     "Полина",    "Прасковья",  "Рада",      "Раиса",     "Регина",
    "Римма",     "Рогнеда",   "Светлана",  "Серафима",  "Снежана",    "София",     "Сусанна",   "Таисия",
    "Тамара",    "Татьяна",   "Улита",     "Ульяна",    "Урсула",     "Фаина",     "Феврония",  "Фёкла",
    "Феодора",   "Целестина", "Элеонора",  "Юлия",      "Яна",        "Ярослава",
});

const auto russianMaleLastNames = std::to_array<std::string_view>({
    "Авдеев",     "Агапов",      "Агафонов",   "Агеев",       "Акимов",     "Аксёнов",      "Александров",
    "Алексеев",   "Алёхин",      "Алешин",     "Алёшин",      "Ананьев",    "Андреев",      "Андрианов",
    "Аникин",     "Анисимов",    "Анохин",     "Антипов",     "Антонов",    "Артамонов",    "Артёмов",
    "Архипов",    "Астафьев",    "Астахов",    "Афанасьев",   "Бабушкин",   "Баженов",      "Балашов",
    "Баранов",    "Барсуков",    "Басов",      "Безруков",    "Беликов",    "Белкин",       "Белов",
    "Белоусов",   "Беляев",      "Беляков",    "Березин",     "Берия",      "Беспалов",     "Бессонов",
    "Бирюков",    "Блинов",      "Блохин",     "Бобров",      "Богданов",   "Богомолов",    "Болдырев",
    "Большаков",  "Бондарев",    "Борисов",    "Бородин",     "Бочаров",    "Булатов",      "Булгаков",
    "Буров",      "Быков",       "Бычков",     "Вавилов",     "Вагин",      "Васильев",     "Вдовин",
    "Верещагин",  "Вешняков",    "Виноградов", "Винокуров",   "Вишневский", "Владимиров",   "Власов",
    "Волков",     "Волошин",     "Воробьёв",   "Воронин",     "Воронков",   "Воронов",      "Воронцов",
    "Второв",     "Высоцкий",    "Гаврилов",   "Гайдуков",    "Гакабов",    "Галкин",       "Герасимов",
    "Гладков",    "Глебов",      "Глухов",     "Глушков",     "Гноев",      "Голиков",      "Голованов",
    "Головин",    "Голубев",     "Гончаров",   "Горбань",     "Горбачав",   "Горбачёв",     "Горбунов",
    "Гордеев",    "Горелов",     "Горлов",     "Горохов",     "Горшков",    "Горюнов",      "Горячев",
    "Грачёв",     "Греков",      "Грибков",    "Грибов",      "Григорьев",  "Гришин",       "Громов",
    "Губанов",    "Гуляев",      "Гуров",      "Гусев",       "Гущин",      "Давыдов",      "Данилов",
    "Дашков",     "Дегтярев",    "Дегтярёв",   "Дементьев",   "Демидов",    "Дёмин",        "Демьянов",
    "Денисов",    "Дмитриев",    "Добрынин",   "Долгов",      "Дорофеев",   "Дорохов",      "Дроздов",
    "Дружинин",   "Дубинин",     "Дубов",      "Дубровин",    "Дьяков",     "Дьяконов",     "Евдокимов",
    "Евсеев",     "Егоров",      "Ежов",       "Елизаров",    "Елисеев",    "Ельцин",       "Емельянов",
    "Еремеев",    "Ерёмин",      "Ермаков",    "Ермилов",     "Ермолаев",   "Ермолов",      "Еромлаев",
    "Ерофеев",    "Ерохин",      "Ершов",      "Ефимов",      "Ефремов",    "Жаров",        "Жданов",
    "Жилин",      "Жириновский", "Жуков",      "Журавлёв",    "Завьялов",   "Заец",         "Зайцев",
    "Захаров",    "Зверев",      "Звягинцев",  "Зеленин",     "Зимин",      "Зиновьев",     "Злобин",
    "Золотарев",  "Золотарёв",   "Золотов",    "Зорин",       "Зотов",      "Зубков",       "Зубов",
    "Зуев",       "Зыков",       "Зюганов",    "Иванов",      "Ивашов",     "Игнатов",      "Игнатьев",
    "Измайлов",   "Ильин",       "Ильинский",  "Ильюхин",     "Исаев",      "Исаков",       "Казаков",
    "Казанцев",   "Калачев",     "Калачёв",    "Калашников",  "Калинин",    "Калмыков",     "Калугин",
    "Капустин",   "Карасев",     "Карасёв",    "Карпов",      "Карташов",   "Касаткин",     "Касьянов",
    "Киреев",     "Кириллов",    "Киселёв",    "Кислов",      "Климов",     "Клюев",        "Князев",
    "Ковалёв",    "Коваленко",   "Коваль",     "Кожевников",  "Козин",      "Козлов",       "Козловский",
    "Козырев",    "Колесников",  "Колесов",    "Колосов",     "Колпаков",   "Кольцов",      "Комаров",
    "Комиссаров", "Кондратов",   "Кондратьев", "Кондрашов",   "Коновалов",  "Кононов",      "Константинов",
    "Копылов",    "Корнев",      "Корнеев",    "Корнилов",    "Коровин",    "Королёв",      "Корольков",
    "Коротков",   "Корчагин",    "Коршунов",   "Косарев",     "Костин",     "Котов",        "Кочергин",
    "Кочетков",   "Кочетов",     "Кошелев",    "Кравцов",     "Краснов",    "Красоткин",    "Кролик",
    "Кроликов",   "Круглов",     "Крылов",     "Крюков",      "Крючков",    "Кудрявцев",    "Кудряшов",
    "Кузин",      "Кузнецов",    "Кузьмин",    "Кукушкин",    "Кулагин",    "Кулаков",      "Кулешов",
    "Куликов",    "Куприянов",   "Курочкин",   "Лаврентьев",  "Лавров",     "Лазарев",      "Лапин",
    "Лаптев",     "Лапшин",      "Ларин",      "Ларионов",    "Латышев",    "Лебедев",      "Левин",
    "Леонов",     "Леонтьев",    "Литвинов",   "Лобанов",     "Логинов",    "Лопатин",      "Лосев",
    "Лужков",     "Лукин",       "Лукьянов",   "Лыков",       "Львов",      "Любимов",      "Майоров",
    "Макаров",    "Макеев",      "Максимов",   "Малахов",     "Малинин",    "Малофеев",     "Малышев",
    "Мальцев",    "Маркелов",    "Маркин",     "Марков",      "Мартынов",   "Масленников",  "Маслов",
    "Матвеев",    "Матвиенко",   "Медведев",   "Медейко",     "Мельников",  "Меньшов",      "Меркулов",
    "Мешков",     "Мещеряков",   "Минаев",     "Минин",       "Миронов",    "Митрофанов",   "Михайлов",
    "Михеев",     "Мишустин",    "Моисеев",    "Молчанов",    "Моргунов",   "Морозов",      "Москвин",
    "Муравьёв",   "Муратов",     "Муромцев",   "Мухин",       "Мясников",   "Навальный",    "Назаров",
    "Наумов",     "Некрасов",    "Нестеров",   "Нефёдов",     "Нечаев",     "Никитин",      "Никифоров",
    "Николаев",   "Никольский",  "Никонов",    "Никулин",     "Новиков",    "Новодворский", "Носков",
    "Носов",      "Овсянников",  "Овчинников", "Одинцов",     "Озеров",     "Окулов",       "Олейников",
    "Орехов",     "Орлов",       "Осипов",     "Островский",  "Павлов",     "Павловский",   "Панин",
    "Панков",     "Панкратов",   "Панов",      "Пантелеев",   "Панфилов",   "Парамонов",    "Парфёнов",
    "Пастухов",   "Пахомов",     "Пекарев",    "Петров",      "Петровский", "Петухов",      "Пименов",
    "Пирогов",    "Платонов",    "Плотников",  "Поздняков",   "Покровский", "Поликарпов",   "Поляков",
    "Пономарев",  "Пономарёв",   "Попов",      "Порошин",     "Порывай",    "Постников",    "Потапов",
    "Похлёбкин",  "Прокофьев",   "Прохоров",   "Прошин",      "Пугачёв",    "Путин",        "Раков",
    "Резицкий",   "Рогов",       "Родин",      "Родионов",    "Рожков",     "Розанов",      "Романов",
    "Рублёв",     "Рубцов",      "Рудаков",    "Руднев",      "Румянцев",   "Русаков",      "Русанов",
    "Рыбаков",    "Рыбкин",      "Рыжиков",    "Рыжков",      "Рыжов",      "Рябинин",      "Рябов",
    "Савельев",   "Савин",       "Савицкий",   "Сазонов",     "Сальников",  "Самойлов",     "Самсонов",
    "Сафонов",    "Сахаров",     "Светличный", "Светлов",     "Свешников",  "Свиридов",     "Севастьянов",
    "Седов",      "Селезнёв",    "Селиванов",  "Семёнов",     "Сёмин",      "Сергеев",      "Серебряков",
    "Серов",      "Сидоров",     "Сизов",      "Симонов",     "Синицин",    "Синицын",      "Ситников",
    "Скворцов",   "Смирнов",     "Снегирёв",   "Соболев",     "Собянин",    "Соколов",      "Соловьёв",
    "Сомов",      "Сорокин",     "Сотников",   "Софронов",    "Спиридонов", "Стариков",     "Старостин",
    "Степанов",   "Столяров",    "Стрелков",   "Стрельников", "Строев",     "Субботин",     "Суворов",
    "Судаков",    "Сурганов",    "Сурков",     "Суслов",      "Суханов",    "Сухарев",      "Сухов",
    "Сычёв",      "Тарасов",     "Терентьев",  "Терехов",     "Тимофеев",   "Титов",        "Тихомиров",
    "Тихонов",    "Ткачёв",      "Токарев",    "Толкачёв",    "Торшин",     "Третьяков",    "Трифонов",
    "Троицкий",   "Трофим",      "Трофимов",   "Троцкий",     "Трошин",     "Туманов",      "Уваров",
    "Ульянов",    "Усов",        "Успенский",  "Устинов",     "Уткин",      "Ушаков",       "Фадеев",
    "Фёдоров",    "Федосеев",    "Федосов",    "Федотов",     "Фетисов",    "Филатов",      "Филимонов",
    "Филиппов",   "Фирсов",      "Фокин",      "Фомин",       "Фомичев",    "Фомичёв",      "Фролов",
    "Харитонов",  "Хомяков",     "Хохлов",     "Хромов",      "Хрущёв",     "Худяков",      "Царёв",
    "Цветков",    "Чеботарёв",   "Черепанов",  "Черкасов",    "Чернов",     "Черной",       "Чёрный",
    "Черных",     "Чернышев",    "Чернышёв",   "Черняев",     "Чесноков",   "Чижов",        "Чистяков",
    "Чумаков",    "Шаповалов",   "Шапошников", "Шаров",       "Швецов",     "Шевелёв",      "Шевцов",
    "Шестаков",   "Шилов",       "Широков",    "Ширяев",      "Шишкин",     "Шмелёв",       "Шубин",
    "Шувалов",    "Шульгин",     "Щеглов",     "Щербаков",    "Щукин",      "Юдин",         "Яковлев",
    "Яшин",
});

const auto russianFemaleLastNames = std::to_array<std::string_view>({
    "Авдеева",      "Агапова",       "Агафонова",   "Агеева",       "Акимова",     "Аксёнова",     "Александрова",
    "Алексеева",    "Алёхина",       "Алешина",     "Алёшина",      "Ананьева",    "Андреева",     "Андрианова",
    "Аникина",      "Анисимова",     "Анохина",     "Антипова",     "Антонова",    "Артамонова",   "Артёмова",
    "Архипова",     "Астафьева",     "Астахова",    "Афанасьева",   "Бабушкина",   "Баженова",     "Балашова",
    "Баранова",     "Барсукова",     "Басова",      "Безрукова",    "Беликова",    "Белкина",      "Белова",
    "Белоусова",    "Беляева",       "Белякова",    "Березина",     "Берия",       "Беспалова",    "Бессонова",
    "Бирюкова",     "Блинова",       "Блохина",     "Боброва",      "Богданова",   "Богомолова",   "Болдырева",
    "Большакова",   "Бондарева",     "Борисова",    "Бородина",     "Бочарова",    "Булатова",     "Булгакова",
    "Бурова",       "Быкова",        "Бычкова",     "Вавилова",     "Вагина",      "Васильева",    "Вдовина",
    "Верещагина",   "Вешнякова",     "Виноградова", "Винокурова",   "Вишневская",  "Владимирова",  "Власова",
    "Волкова",      "Волошина",      "Воробьёва",   "Воронина",     "Воронкова",   "Воронова",     "Воронцова",
    "Второва",      "Высоцкая",      "Гаврилова",   "Гайдукова",    "Гакабова",    "Галкина",      "Герасимова",
    "Гладкова",     "Глебова",       "Глухова",     "Глушкова",     "Гноева",      "Голикова",     "Голованова",
    "Головина",     "Голубева",      "Гончарова",   "Горбань",      "Горбачёва",   "Горбунова",    "Гордеева",
    "Горелова",     "Горлова",       "Горохова",    "Горшкова",     "Горюнова",    "Горячева",     "Грачёва",
    "Грекова",      "Грибкова",      "Грибова",     "Григорьева",   "Гришина",     "Громова",      "Губанова",
    "Гуляева",      "Гурова",        "Гусева",      "Гущина",       "Давыдова",    "Дадаева",      "Дадина",
    "Данилова",     "Дарвина",       "Дашкова",     "Дегтярева",    "Дегтярёва",   "Дедова",       "Дементьева",
    "Демидова",     "Дёмина",        "Демьянова",   "Денисова",     "Дмитриева",   "Добрынина",    "Долгова",
    "Дорофеева",    "Дорохова",      "Дроздова",    "Дружинина",    "Дубинина",    "Дубова",       "Дубровина",
    "Дьякова",      "Дьяконова",     "Евдокимова",  "Евсеева",      "Егорова",     "Ежова",        "Елизарова",
    "Елисеева",     "Ельцина",       "Емельянова",  "Еремеева",     "Ерёмина",     "Ермакова",     "Ермилова",
    "Ермолаева",    "Ермолова",      "Еромлаева",   "Ерофеева",     "Ершова",      "Ефимова",      "Ефремова",
    "Жарова",       "Жданова",       "Жилина",      "Жириновская",  "Жукова",      "Журавлёва",    "Завьялова",
    "Заец",         "Зайцева",       "Захарова",    "Зверева",      "Звягинцева",  "Зеленина",     "Зимина",
    "Зиновьева",    "Злобина",       "Золотарева",  "Золотарёва",   "Золотова",    "Зорина",       "Зотова",
    "Зубкова",      "Зубова",        "Зуева",       "Зыкова",       "Зюганова",    "Иванова",      "Ивашова",
    "Игнатова",     "Игнатьева",     "Измайлова",   "Ильина",       "Ильинская",   "Ильюхина",     "Исаева",
    "Исакова",      "Казакова",      "Казанцева",   "Калачева",     "Калачёва",    "Калашникова",  "Калинина",
    "Калмыкова",    "Калугина",      "Капустина",   "Карасева",     "Карасёва",    "Карпова",      "Карташова",
    "Касаткина",    "Касьянова",     "Киреева",     "Кириллова",    "Киселёва",    "Кислова",      "Климова",
    "Клюева",       "Князева",       "Ковалёва",    "Коваленко",    "Коваль",      "Кожевникова",  "Козина",
    "Козлова",      "Козловская",    "Козырева",    "Колесникова",  "Колесова",    "Колосова",     "Колпакова",
    "Кольцова",     "Комарова",      "Комиссарова", "Кондратова",   "Кондратьева", "Кондрашова",   "Коновалова",
    "Кононова",     "Константинова", "Копылова",    "Корнева",      "Корнеева",    "Корнилова",    "Коровина",
    "Королёва",     "Королькова",    "Короткова",   "Корчагина",    "Коршунова",   "Косарева",     "Костина",
    "Котова",       "Кочергина",     "Кочеткова",   "Кочетова",     "Кошелева",    "Кравцова",     "Краснова",
    "Красоткина",   "Круглова",      "Крылова",     "Крюкова",      "Крючкова",    "Кудрявцева",   "Кудряшова",
    "Кузина",       "Кузнецова",     "Кузьмина",    "Кукушкина",    "Кулагина",    "Кулакова",     "Кулешова",
    "Куликова",     "Куприянова",    "Курочкина",   "Лаврентьева",  "Лаврова",     "Лазарева",     "Лапина",
    "Лаптева",      "Лапшина",       "Ларина",      "Ларионова",    "Латышева",    "Лебедева",     "Левина",
    "Леонова",      "Леонтьева",     "Литвинова",   "Лобанова",     "Логинова",    "Лопатина",     "Лосева",
    "Лужкова",      "Лукина",        "Лукьянова",   "Лыкова",       "Львова",      "Любимова",     "Майорова",
    "Макарова",     "Макеева",       "Максимова",   "Малахова",     "Малинина",    "Малофеева",    "Малышева",
    "Мальцева",     "Маркелова",     "Маркина",     "Маркова",      "Мартынова",   "Масленникова", "Маслова",
    "Матвеева",     "Матвиенко",     "Медведева",   "Медейко",      "Мельникова",  "Меньшова",     "Меркулова",
    "Мешкова",      "Мещерякова",    "Минаева",     "Минина",       "Миронова",    "Митрофанова",  "Михайлова",
    "Михеева",      "Мишустина",     "Моисеева",    "Молчанова",    "Моргунова",   "Морозова",     "Москвина",
    "Муравьёва",    "Муратова",      "Муромцева",   "Мухина",       "Мясникова",   "Навальная",    "Назарова",
    "Наумова",      "Некрасова",     "Нестерова",   "Нефёдова",     "Нечаева",     "Никитина",     "Никифорова",
    "Николаева",    "Никольская",    "Никонова",    "Никулина",     "Новикова",    "Новодворская", "Носкова",
    "Носова",       "Овсянникова",   "Овчинникова", "Одинцова",     "Озерова",     "Окулова",      "Олейникова",
    "Орехова",      "Орлова",        "Осипова",     "Островская",   "Павлова",     "Павловская",   "Панина",
    "Панкова",      "Панкратова",    "Панова",      "Пантелеева",   "Панфилова",   "Парамонова",   "Парфёнова",
    "Пастухова",    "Пахомова",      "Пекарева",    "Петрова",      "Петровская",  "Петухова",     "Пименова",
    "Пирогова",     "Платонова",     "Плотникова",  "Позднякова",   "Покровская",  "Поликарпова",  "Полякова",
    "Пономарёва",   "Попова",        "Порошина",    "Порывай",      "Постникова",  "Потапова",     "Похлёбкина",
    "Прокофьева",   "Прохорова",     "Прошина",     "Пугачёва",     "Путина",      "Ракова",       "Рогова",
    "Родина",       "Родионова",     "Рожкова",     "Розанова",     "Романова",    "Рублёва",      "Рубцова",
    "Рудакова",     "Руднева",       "Румянцева",   "Русакова",     "Русанова",    "Рыбакова",     "Рыжикова",
    "Рыжкова",      "Рыжова",        "Рябинина",    "Рябова",       "Савельева",   "Савина",       "Савицкая",
    "Сазонова",     "Сальникова",    "Самойлова",   "Самсонова",    "Сафонова",    "Сахарова",     "Светличная",
    "Светлова",     "Свешникова",    "Свиридова",   "Севастьянова", "Седова",      "Селезнёва",    "Селиванова",
    "Семёнова",     "Сёмина",        "Сергеева",    "Серебрякова",  "Серова",      "Сидорова",     "Сизова",
    "Симонова",     "Синицына",      "Ситникова",   "Скворцова",    "Скрябина",    "Смирнова",     "Снегирёва",
    "Соболева",     "Собянина",      "Соколова",    "Соловьёва",    "Сомова",      "Сорокина",     "Сотникова",
    "Софронова",    "Спиридонова",   "Старикова",   "Старостина",   "Степанова",   "Столярова",    "Стрелкова",
    "Стрельникова", "Строева",       "Субботина",   "Суворова",     "Судакова",    "Суркова",      "Суслова",
    "Суханова",     "Сухарева",      "Сухова",      "Сычёва",       "Тарасова",    "Терентьева",   "Терехова",
    "Тимофеева",    "Титова",        "Тихомирова",  "Тихонова",     "Ткачёва",     "Токарева",     "Толкачёва",
    "Торшина",      "Третьякова",    "Трифонова",   "Троицкая",     "Трофимова",   "Троцкая",      "Трошина",
    "Туманова",     "Уварова",       "Ульянова",    "Усова",        "Успенская",   "Устинова",     "Уткина",
    "Ушакова",      "Фадеева",       "Фёдорова",    "Федосеева",    "Федосова",    "Федотова",     "Фетисова",
    "Филатова",     "Филимонова",    "Филиппова",   "Фирсова",      "Фокина",      "Фомина",       "Фомичева",
    "Фомичёва",     "Фролова",       "Харитонова",  "Хомякова",     "Хромова",     "Хрущёва",      "Худякова",
    "Царёва",       "Цветкова",      "Чеботарёва",  "Черепанова",   "Черкасова",   "Черная",       "Чёрная",
    "Чернова",      "Черных",        "Чернышева",   "Чернышёва",    "Черняева",    "Чеснокова",    "Чижова",
    "Чистякова",    "Чумакова",      "Шаповалова",  "Шапошникова",  "Шарова",      "Швецова",      "Шевелёва",
    "Шевцова",      "Шестакова",     "Шилова",      "Широкова",     "Ширяева",     "Шишкина",      "Шмелёва",
    "Шубина",       "Шувалова",      "Шульгина",    "Щеглова",      "Щербакова",   "Щукина",       "Юдина",
    "Яковлева",     "Яшина",
});

const NameFormats russianNameFormats{{{"{firstName} {lastName}", 1}, {"{lastName} {firstName}", 1}}};

const PeopleNames russianPeopleNames{{russianMaleFirstNames, russianMaleLastNames, {}, {}},
                                     {russianFemaleFirstNames, russianFemaleLastNames, {}, {}},
                                     russianNameFormats};

// Serbia

const auto serbianMaleFirstNames = std::to_array<std::string_view>({
    "Adam",      "Aleksa",    "Aleksandar", "Aleksej",   "Aljoša",    "Anastasije", "Andrej",   "Andrija",
    "Anej",      "Arsenije",  "Aćim",       "Berislav",  "Blagoje",   "Boban",      "Bogdan",   "Bogoljub",
    "Bojan",     "Boris",     "Borivoje",   "Borjan",    "Boško",     "Boža",       "Božidar",  "Branimir",
    "Branislav", "Branko",    "Budimir",    "Cvijan",    "Cvijetin",  "Dabiživ",    "Damir",    "Damjan",
    "Danilo",    "Danko",     "Darko",      "David",     "Davor",     "Dejan",      "Desimir",  "Dimitrije",
    "Dragan",    "Dragiša",   "Dragoje",    "Dragojlo",  "Dragoljub", "Dragoslav",  "Dragutin", "Draža",
    "Dražen",    "Dren",      "Dušan",      "Duško",     "Filip",     "Filotije",   "Gavrilo",  "Georgije",
    "Gerasim",   "Gligorije", "Gojko",      "Golub",     "Goran",     "Gordan",     "Grgur",    "Grigorije",
    "Hvalimir",  "Ignjat",    "Igor",       "Ilarion",   "Ilija",     "Ivan",       "Ivica",    "Ivo",
    "Jadranko",  "Jakov",     "Jezdimir",   "Jovan",     "Jugoslav",  "Kalinik",    "Kirilo",   "Konstantin",
    "Kosta",     "Kristijan", "Krsto",      "Kuzman",    "Lazar",     "Ljubiša",    "Ljubomir", "Luka",
    "Mališa",    "Marinko",   "Marjan",     "Marko",     "Mateja",    "Mihailo",    "Mihajlo",  "Mija",
    "Mijat",     "Mika",      "Miladin",    "Milan",     "Milanko",   "Mile",       "Mileta",   "Milisav",
    "Milić",     "Miljan",    "Milojica",   "Milomir",   "Milorad",   "Miloš",      "Milutin",  "Miodrag",
    "Miomir",    "Mirko",     "Miroslav",   "Mitar",     "Miša",      "Miško",      "Mladen",   "Momir",
    "Nebojša",   "Nedeljko",  "Nemanja",    "Nenad",     "Neven",     "Nikola",     "Nikša",    "Novak",
    "Obrad",     "Obren",     "Ognjen",     "Oliver",    "Ostoja",    "Paja",       "Pavle",    "Pera",
    "Perica",    "Pero",      "Petar",      "Petko",     "Predrag",   "Prvoslav",   "Radenko",  "Radivoje",
    "Radiša",    "Radmilo",   "Radomir",    "Radosav",   "Radoslav",  "Radovan",    "Radoš",    "Radul",
    "Rajko",     "Ranko",     "Ratimir",    "Raško",     "Relja",     "Rodoljub",   "Saša",     "Simo",
    "Siniša",    "Slavoljub", "Slobodan",   "Srboljub",  "Sredoje",   "Stanislav",  "Stanko",   "Stefan",
    "Stevan",    "Stojan",    "Strahinja",  "Svetislav", "Svetolik",  "Svetomir",   "Tadija",   "Tihomir",
    "Tomislav",  "Vanja",     "Vasil",      "Vasko",     "Velibor",   "Velimir",    "Vidak",    "Vladimir",
    "Vlastimir", "Vojin",     "Vujadin",    "Vuk",       "Vukan",     "Vukašin",    "Vukota",   "Zafir",
    "Zarije",    "Zoran",     "Zvonko",     "Časlav",    "Čedomir",   "Đorđe",      "Đura",     "Đurađ",
});

const auto serbianFemaleFirstNames = std::to_array<std::string_view>({
    "Adrijana", "Aleksandra", "Ana",      "Anabela",   "Anastasija", "Andrea",     "Andrijana", "Aneta",
    "Anica",    "Anita",      "Anja",     "Anka",      "Anđela",     "Ava",        "Biljana",   "Bisenija",
    "Blagica",  "Bobana",     "Bogdana",  "Bojana",    "Bora",       "Borka",      "Bosiljka",  "Božena",
    "Božica",   "Branka",     "Brankica", "Ceca",      "Dajana",     "Daliborka",  "Damjanka",  "Dana",
    "Danijela", "Danila",     "Danka",    "Darija",    "Darinka",    "Dejana",     "Desanka",   "Dijana",
    "Divna",    "Dobrila",    "Dorotea",  "Draga",     "Dragana",    "Draginja",   "Drina",     "Dubravka",
    "Dunja",    "Dušanka",    "Dušica",   "Duška",     "Elena",      "Eleonora",   "Ema",       "Emilija",
    "Evdokija", "Gabrijela",  "Gala",     "Galina",    "Gorana",     "Gordana",    "Hana",      "Ilinka",
    "Ina",      "Irina",      "Ivanka",   "Jadranka",  "Jana",       "Jasmina",    "Jasna",     "Jelena",
    "Jelica",   "Jelisaveta", "Jelka",    "Jevrosima", "Jovana",     "Jovanka",    "Julia",     "Julijana",
    "Katarina", "Klara",      "Kristina", "Ksenija",   "Lada",       "Lara",       "Leposava",  "Lidija",
    "Ljiljana", "Ljuba",      "Ljubica",  "Magdalena", "Maja",       "Malina",     "Marica",    "Marina",
    "Markika",  "Marta",      "Maša",     "Mašinka",   "Melenija",   "Mia",        "Milana",    "Milanka",
    "Milena",   "Milica",     "Miljana",  "Mina",      "Mira",       "Mirela",     "Mirjana",   "Mirka",
    "Mirna",    "Mitra",      "Mlađana",  "Momira",    "Nada",       "Nadežda",    "Nastja",    "Natalija",
    "Nataša",   "Nađa",       "Neda",     "Nela",      "Neva",       "Nevena",     "Nikoleta",  "Nikolina",
    "Nina",     "Njegomirka", "Ognjana",  "Olga",      "Olgica",     "Olivera",    "Peruna",    "Petra",
    "Petrija",  "Radina",     "Radmila",  "Radojka",   "Rakita",     "Renja",      "Ristana",   "Roksana",
    "Romana",   "Rumena",     "Ruža",     "Sanda",     "Sandra",     "Sanja",      "Sara",      "Selena",
    "Senka",    "Silvija",    "Simonida", "Slava",     "Slavica",    "Slavka",     "Slađana",   "Slobodanka",
    "Smiljana", "Snežana",    "Sofija",   "Sonja",     "Srna",       "Stanislava", "Staša",     "Stefana",
    "Stojanka", "Sunčica",    "Suzana",   "Svetlana",  "Tamara",     "Tanja",      "Tara",      "Tatjana",
    "Tea",      "Teodora",    "Teona",    "Una",       "Vasilisa",   "Vera",       "Verica",    "Veselinka",
    "Vesna",    "Vida",       "Višnja",   "Vladana",   "Vlatka",     "Vojislava",  "Vujica",    "Vukica",
    "Zavida",   "Zora",       "Zorana",   "Zorka",     "Zvezdana",   "Čarna",      "Đurica",    "Željka",
});

const auto serbianLastNames = std::to_array<std::string_view>({
    "Abadžija",     "Abadžić",      "Abazović",    "Adamović",      "Adašević",    "Aguridić",   "Ajvazović",
    "Akifović",     "Aksentijević", "Aksentić",    "Aksin",         "Aksić",       "Aksović",    "Akšam",
    "Aladić",       "Aladžić",      "Alaković",    "Alanović",      "Alargić",     "Alavanja",   "Alavantić",
    "Aličić",       "Aljančić",     "Alković",     "Almašac",       "Alparac",     "Aluga",      "Andesilić",
    "Andić",        "Andonov",      "Andonović",   "Andrejević",    "Andrejin",    "Andrejić",   "Antonić",
    "Antonović",    "Antović",      "Antunović",   "Antušević",     "Anučin",      "Anušić",     "Anđelić",
    "Anđelković",   "Aćimović",     "Aćin",        "Aćić",          "Ačanski",     "Ačić",       "Ađanski",
    "Ašanin",       "Ašađanin",     "Aškraba",     "Aškrabić",      "Ašonja",      "Ašćerić",    "Babac",
    "Babalj",       "Babarogić",    "Babin",       "Babić",         "Babović",     "Bajčetić",   "Bajčić",
    "Bajšanski",    "Bakal",        "Balabanović", "Balašević",     "Baletić",     "Balević",    "Balešević",
    "Banić",        "Banićević",    "Banjac",      "Banjak",        "Banjalić",    "Barišić",    "Barošević",
    "Baroševčić",   "Basara",       "Basarić",     "Bastać",        "Batnožić",    "Bekavac",    "Bekjarev",
    "Beko",         "Bekonja",      "Bekovac",     "Bekvalac",      "Bekčić",      "Belivuk",    "Belić",
    "Belojević",    "Belopoljac",   "Belović",     "Belovuković",   "Beloševac",   "Belošević",  "Belčević",
    "Benaković",    "Birtašević",   "Birčaković",  "Birčanin",      "Bivolarević", "Bizetić",    "Bizumić",
    "Bićanin",      "Bjeletić",     "Bjelica",     "Bjelinić",      "Bjelivuk",    "Bjelić",     "Bjeličić",
    "Bjelja",       "Bjelkić",      "Bjelobaba",   "Bjelobrk",      "Bjeloglav",   "Bjelogrlić", "Bjelojević",
    "Blaćanin",     "Blašković",    "Blažanović",  "Bocić",         "Bocokić",     "Bodiroga",   "Bogisavljević",
    "Bogoljubović", "Bogosavac",    "Bolta",       "Boltić",        "Bolunović",   "Bomeštar",   "Bonderović",
    "Bondžić",      "Borilović",    "Borinčević",  "Borisavljević", "Borkovački",  "Borković",   "Borčanin",
    "Bosanac",      "Bosančić",     "Bosiljkić",   "Bosiljković",   "Bosiljčić",   "Bozalo",     "Bozdok",
    "Bozdokov",     "Božanić",      "Božanović",   "Božićković",    "Božičić",     "Božičković", "Bratojević",
    "Bratonožić",   "Bratović",     "Brezo",       "Brezovac",      "Brešković",   "Brežančić",  "Brkanović",
    "Brkić",        "Brđanin",      "Brđović",     "Buač",          "Buača",       "Bubalo",     "Bucalo",
    "Bucalović",    "Buljin",       "Buljubašić",  "Buljugić",      "Bulovan",     "Bunuševac",  "Burani",
    "Burazer",      "Burazor",      "Burević",     "Burgić",        "Butarević",   "Cikota",     "Cilikan",
    "Cimbaljević",  "Cimeša",       "Cvetković",   "Cvetljanin",    "Cvetojević",  "Cvetuljski", "Cvijan",
    "Cvijanović",   "Cvijančević",  "Cvijetin",    "Cvijetinović",  "Cvijetić",    "Cvijić",     "Cvjetičanin",
    "Dabić",        "Dabižljević",  "Dabović",     "Dafunić",
});

const NameFormats serbianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames serbianPeopleNames{{serbianMaleFirstNames, serbianLastNames, {}, {}},
                                     {serbianFemaleFirstNames, serbianLastNames, {}, {}},
                                     serbianNameFormats};

// Slovakia

const auto slovakMaleFirstNames = std::to_array<std::string_view>({
    "Drahoslav",  "Severín",  "Alexej",    "Ernest",    "Rastislav",  "Radovan",   "Dobroslav", "Dalibor",
    "Vincent",    "Miloš",    "Timotej",   "Gejza",     "Bohuš",      "Alfonz",    "Gašpar",    "Emil",
    "Erik",       "Blažej",   "Zdenko",    "Dezider",   "Arpád",      "Valentín",  "Pravoslav", "Jaromír",
    "Roman",      "Matej",    "Frederik",  "Viktor",    "Alexander",  "Radomír",   "Albín",     "Bohumil",
    "Kazimír",    "Fridrich", "Radoslav",  "Tomáš",     "Alan",       "Branislav", "Bruno",     "Gregor",
    "Vlastimil",  "Boleslav", "Eduard",    "Jozef",     "Víťazoslav", "Blahoslav", "Beňadik",   "Adrián",
    "Gabriel",    "Marián",   "Emanuel",   "Miroslav",  "Benjamín",   "Hugo",      "Richard",   "Izidor",
    "Zoltán",     "Albert",   "Igor",      "Július",    "Aleš",       "Fedor",     "Rudolf",    "Valér",
    "Marcel",     "Ervín",    "Slavomír",  "Vojtech",   "Juraj",      "Marek",     "Jaroslav",  "Žigmund",
    "Florián",    "Roland",   "Pankrác",   "Servác",    "Bonifác",    "Svetozár",  "Bernard",   "Júlia",
    "Urban",      "Dušan",    "Viliam",    "Ferdinand", "Norbert",    "Róbert",    "Medard",    "Zlatko",
    "Anton",      "Vasil",    "Vít",       "Adolf",     "Vratislav",  "Alfréd",    "Alojz",     "Ján",
    "Tadeáš",     "Ladislav", "Peter",     "Pavol",     "Miloslav",   "Prokop",    "Cyril",     "Metod",
    "Patrik",     "Oliver",   "Ivan",      "Kamil",     "Henrich",    "Drahomír",  "Bohuslav",  "Iľja",
    "Daniel",     "Vladimír", "Jakub",     "Krištof",   "Ignác",      "Gustáv",    "Jerguš",    "Dominik",
    "Oskar",      "Vavrinec", "Ľubomír",   "Mojmír",    "Leonard",    "Tichomír",  "Filip",     "Bartolomej",
    "Ľudovít",    "Samuel",   "Augustín",  "Belo",      "Oleg",       "Bystrík",   "Ctibor",    "Ľudomil",
    "Konštantín", "Ľuboslav", "Matúš",     "Móric",     "Ľuboš",      "Ľubor",     "Vladislav", "Cyprián",
    "Václav",     "Michal",   "Jarolím",   "Arnold",    "Levoslav",   "František", "Dionýz",    "Maximilián",
    "Koloman",    "Boris",    "Lukáš",     "Kristián",  "Vendelín",   "Sergej",    "Aurel",     "Demeter",
    "Denis",      "Hubert",   "Karol",     "Imrich",    "René",       "Bohumír",   "Teodor",    "Tibor",
    "Maroš",      "Martin",   "Svätopluk", "Stanislav", "Leopold",    "Eugen",     "Félix",     "Klement",
    "Kornel",     "Milan",    "Vratko",    "Ondrej",    "Andrej",     "Edmund",    "Oldrich",   "Oto",
    "Mikuláš",    "Ambróz",   "Radúz",     "Bohdan",    "Adam",       "Štefan",    "Dávid",     "Silvester",
});

const auto slovakFemaleFirstNames = std::to_array<std::string_view>({
    "Alexandra", "Karina",     "Daniela",    "Andrea",     "Antónia",   "Bohuslava",  "Dáša",     "Malvína",
    "Kristína",  "Nataša",     "Bohdana",    "Drahomíra",  "Sára",      "Zora",       "Tamara",   "Ema",
    "Tatiana",   "Erika",      "Veronika",   "Agáta",      "Dorota",    "Vanda",      "Zoja",     "Gabriela",
    "Perla",     "Ida",        "Liana",      "Miloslava",  "Vlasta",    "Lívia",      "Eleonóra", "Etela",
    "Romana",    "Zlatica",    "Anežka",     "Bohumila",   "Františka", "Angela",     "Matilda",  "Svetlana",
    "Ľubica",    "Alena",      "Soňa",       "Vieroslava", "Zita",      "Miroslava",  "Irena",    "Milena",
    "Estera",    "Justína",    "Dana",       "Danica",     "Jela",      "Jaroslava",  "Jarmila",  "Lea",
    "Anastázia", "Galina",     "Lesana",     "Hermína",    "Monika",    "Ingrida",    "Viktória", "Blažena",
    "Žofia",     "Sofia",      "Gizela",     "Viola",      "Gertrúda",  "Zina",       "Júlia",    "Juliana",
    "Želmíra",   "Ela",        "Vanesa",     "Iveta",      "Vilma",     "Petronela",  "Žaneta",   "Xénia",
    "Karolína",  "Lenka",      "Laura",      "Stanislava", "Margaréta", "Dobroslava", "Blanka",   "Valéria",
    "Paulína",   "Sidónia",    "Adriána",    "Beáta",      "Petra",     "Melánia",    "Diana",    "Berta",
    "Patrícia",  "Lujza",      "Amália",     "Milota",     "Nina",      "Margita",    "Kamila",   "Dušana",
    "Magdaléna", "Oľga",       "Anna",       "Hana",       "Božena",    "Marta",      "Libuša",   "Božidara",
    "Dominika",  "Hortenzia",  "Jozefína",   "Štefánia",   "Ľubomíra",  "Zuzana",     "Darina",   "Marcela",
    "Milica",    "Elena",      "Helena",     "Lýdia",      "Anabela",   "Jana",       "Silvia",   "Nikola",
    "Ružena",    "Nora",       "Drahoslava", "Linda",      "Melinda",   "Rebeka",     "Rozália",  "Regína",
    "Alica",     "Marianna",   "Miriama",    "Martina",    "Mária",     "Jolana",     "Ľudomila", "Ľudmila",
    "Olympia",   "Eugénia",    "Ľuboslava",  "Zdenka",     "Edita",     "Michaela",   "Stela",    "Viera",
    "Natália",   "Eliška",     "Brigita",    "Valentína",  "Terézia",   "Vladimíra",  "Hedviga",  "Uršuľa",
    "Alojza",    "Kvetoslava", "Sabína",     "Dobromila",  "Klára",     "Simona",     "Aurélia",  "Denisa",
    "Renáta",    "Irma",       "Agnesa",     "Klaudia",    "Alžbeta",   "Elvíra",     "Cecília",  "Emília",
    "Katarína",  "Henrieta",   "Bibiána",    "Barbora",    "Marína",    "Izabela",    "Hilda",    "Otília",
    "Lucia",     "Branislava", "Bronislava", "Ivica",      "Albína",    "Kornélia",   "Sláva",    "Slávka",
    "Judita",    "Dagmara",    "Adela",      "Nadežda",    "Eva",       "Filoména",   "Ivana",    "Milada",
});

const auto slovakMaleLastNames = std::to_array<std::string_view>({
    "Antal",      "Babka",      "Bahna",        "Bahno",       "Baláž",     "Baran",       "Baranka",     "Bartovič",
    "Bartoš",     "Bača",       "Bernolák",     "Beňo",        "Bicek",     "Bielik",      "Blaho",       "Bondra",
    "Bosák",      "Boška",      "Brezina",      "Bukovský",    "Chalupka",  "Chudík",      "Cibula",      "Cibulka",
    "Cibuľa",     "Cyprich",    "Cíger",        "Danko",       "Daňko",     "Daňo",        "Debnár",      "Dej",
    "Dekýš",      "Doležal",    "Dočolomanský", "Droppa",      "Dubovský",  "Dudek",       "Dula",        "Dulla",
    "Dusík",      "Dvonč",      "Dzurjanin",    "Dávid",       "Fabian",    "Fabián",      "Fajnor",      "Farkašovský",
    "Fico",       "Filc",       "Filip",        "Finka",       "Ftorek",    "Gašpar",      "Gašparovič",  "Gocník",
    "Gregor",     "Greguš",     "Grznár",       "Hablák",      "Habšuda",   "Halda",       "Haluška",     "Halák",
    "Hanko",      "Hanzal",     "Haščák",       "Heretik",     "Hečko",     "Hlaváček",    "Hlinka",      "Holub",
    "Holuby",     "Hossa",      "Hoza",         "Hraško",      "Hric",      "Hrmo",        "Hrušovský",   "Huba",
    "Ihnačák",    "Janeček",    "Janoška",      "Jantošovič",  "Janík",     "Janček",      "Jedľovský",   "Jendek",
    "Jonata",     "Jurina",     "Jurkovič",     "Jurík",       "Jánošík",   "Kafenda",     "Kaliský",     "Karul",
    "Keníž",      "Klapka",     "Kmeť",         "Kolesár",     "Kollár",    "Kolnik",      "Kolník",      "Kolár",
    "Korec",      "Kostka",     "Kostrec",      "Kováč",       "Kováčik",   "Koza",        "Kočiš",       "Krajíček",
    "Krajči",     "Krajčo",     "Krajčovič",    "Krajčír",     "Králik",    "Krúpa",       "Kubík",       "Kyseľ",
    "Kállay",     "Labuda",     "Lepšík",       "Lipták",      "Lisický",   "Lubina",      "Lukáč",       "Lupták",
    "Líška",      "Madej",      "Majeský",      "Malachovský", "Malíšek",   "Mamojka",     "Marcinko",    "Marián",
    "Masaryk",    "Maslo",      "Matiaško",     "Medveď",      "Melcer",    "Mečiar",      "Michalík",    "Mihalik",
    "Mihál",      "Mihálik",    "Mikloško",     "Mikulík",     "Mikuš",     "Mikúš",       "Milota",      "Mináč",
    "Mišík",      "Mojžiš",     "Mokroš",       "Mora",        "Moravčík",  "Mydlo",       "Nemec",       "Nitra",
    "Novák",      "Obšut",      "Ondruš",       "Otčenáš",     "Pauko",     "Pavlikovský", "Pavúk",       "Pašek",
    "Paška",      "Paško",      "Pelikán",      "Petrovický",  "Petruška",  "Peško",       "Plch",        "Plekanec",
    "Podhradský", "Podkonický", "Poliak",       "Pupák",       "Rak",       "Repiský",     "Romančík",    "Rus",
    "Ružička",    "Rybníček",   "Rybár",        "Rybárik",     "Samson",    "Sedliak",     "Senko",       "Sklenka",
    "Skokan",     "Skutecký",   "Slašťan",      "Sloboda",     "Slobodník", "Slota",       "Slovák",      "Smrek",
    "Stodola",    "Straka",     "Strnisko",     "Svrbík",      "Sámel",     "Sýkora",      "Tatar",       "Tatarka",
    "Tatár",      "Tatárka",    "Thomka",       "Tomeček",     "Tomka",     "Tomko",       "Truben",      "Turčok",
    "Uram",       "Urblík",     "Vajcík",       "Vajda",       "Valach",    "Valachovič",  "Valent",      "Valuška",
    "Vanek",      "Vesel",      "Vicen",        "Višňovský",   "Vlach",     "Vojtek",      "Vydarený",    "Zajac",
    "Zima",       "Zimka",      "Záborský",     "Zúbrik",      "Čapkovič",  "Čaplovič",    "Čarnogurský", "Čierny",
    "Čobrda",     "Ďaďo",       "Ďurica",       "Ďuriš",       "Šidlo",     "Šimonovič",   "Škriniar",    "Škultéty",
    "Šmajda",     "Šoltés",     "Šoltýs",       "Štefan",      "Štefanka",  "Šulc",        "Šurka",       "Švehla",
    "Šťastný",
});

const auto slovak_femalesLastNames = std::to_array<std::string_view>({
    "Antalová",    "Babková",      "Bahnová",      "Balážová",      "Baranová",      "Baranková",     "Bartovičová",
    "Bartošová",   "Bačová",       "Bernoláková",  "Beňová",        "Biceková",      "Bieliková",     "Blahová",
    "Bondrová",    "Bosáková",     "Bošková",      "Brezinová",     "Bukovská",      "Chalupková",    "Chudíková",
    "Cibulová",    "Cibulková",    "Cyprichová",   "Cígerová",      "Danková",       "Daňková",       "Daňová",
    "Debnárová",   "Dejová",       "Dekýšová",     "Doležalová",    "Dočolomanská",  "Droppová",      "Dubovská",
    "Dudeková",    "Dulová",       "Dullová",      "Dusíková",      "Dvončová",      "Dzurjaninová",  "Dávidová",
    "Fabianová",   "Fabiánová",    "Fajnorová",    "Farkašovská",   "Ficová",        "Filcová",       "Filipová",
    "Finková",     "Ftoreková",    "Gašparová",    "Gašparovičová", "Gocníková",     "Gregorová",     "Gregušová",
    "Grznárová",   "Habláková",    "Habšudová",    "Haldová",       "Halušková",     "Haláková",      "Hanková",
    "Hanzalová",   "Haščáková",    "Heretiková",   "Hečková",       "Hlaváčeková",   "Hlinková",      "Holubová",
    "Holubyová",   "Hossová",      "Hozová",       "Hrašková",      "Hricová",       "Hrmová",        "Hrušovská",
    "Hubová",      "Ihnačáková",   "Janečeková",   "Janošková",     "Jantošovičová", "Janíková",      "Jančeková",
    "Jedľovská",   "Jendeková",    "Jonatová",     "Jurinová",      "Jurkovičová",   "Juríková",      "Jánošíková",
    "Kafendová",   "Kaliská",      "Karulová",     "Kenížová",      "Klapková",      "Kmeťová",       "Kolesárová",
    "Kollárová",   "Kolniková",    "Kolníková",    "Kolárová",      "Korecová",      "Kostkaová",     "Kostrecová",
    "Kováčová",    "Kováčiková",   "Kozová",       "Kočišová",      "Krajíčeková",   "Krajčová",      "Krajčovičová",
    "Krajčírová",  "Králiková",    "Krúpová",      "Kubíková",      "Kyseľová",      "Kállayová",     "Labudová",
    "Lepšíková",   "Liptáková",    "Lisická",      "Lubinová",      "Lukáčová",      "Luptáková",     "Líšková",
    "Madejová",    "Majeská",      "Malachovská",  "Malíšeková",    "Mamojková",     "Marcinková",    "Mariánová",
    "Masaryková",  "Maslová",      "Matiašková",   "Medveďová",     "Melcerová",     "Mečiarová",     "Michalíková",
    "Mihaliková",  "Mihálová",     "Miháliková",   "Miklošková",    "Mikulíková",    "Mikušová",      "Mikúšová",
    "Milotová",    "Mináčová",     "Mišíková",     "Mojžišová",     "Mokrošová",     "Morová",        "Moravčíková",
    "Mydlová",     "Nemcová",      "Nováková",     "Obšutová",      "Ondrušová",     "Otčenášová",    "Pauková",
    "Pavlikovská", "Pavúková",     "Pašeková",     "Pašková",       "Pelikánová",    "Petrovická",    "Petrušková",
    "Pešková",     "Plchová",      "Plekanecová",  "Podhradská",    "Podkonická",    "Poliaková",     "Pupáková",
    "Raková",      "Repiská",      "Romančíková",  "Rusová",        "Ružičková",     "Rybníčeková",   "Rybárová",
    "Rybáriková",  "Samsonová",    "Sedliaková",   "Senková",       "Sklenková",     "Skokanová",     "Skutecká",
    "Slašťanová",  "Slobodová",    "Slobodníková", "Slotová",       "Slováková",     "Smreková",      "Stodolová",
    "Straková",    "Strnisková",   "Svrbíková",    "Sámelová",      "Sýkorová",      "Tatarová",      "Tatarková",
    "Tatárová",    "Tatárkaová",   "Thomková",     "Tomečeková",    "Tomková",       "Trubenová",     "Turčoková",
    "Uramová",     "Urblíková",    "Vajcíková",    "Vajdová",       "Valachová",     "Valachovičová", "Valentová",
    "Valušková",   "Vaneková",     "Veselová",     "Vicenová",      "Višňovská",     "Vlachová",      "Vojteková",
    "Vydarená",    "Zajacová",     "Zimová",       "Zimková",       "Záborská",      "Zúbriková",     "Čapkovičová",
    "Čaplovičová", "Čarnogurská",  "Čierná",       "Čobrdová",      "Ďaďová",        "Ďuricová",      "Ďurišová",
    "Šidlová",     "Šimonovičová", "Škriniarová",  "Škultétyová",   "Šmajdová",      "Šoltésová",     "Šoltýsová",
    "Štefanová",   "Štefanková",   "Šulcová",      "Šurková",       "Švehlová",      "Šťastná",
});

const auto slovakSuffixes = std::to_array<std::string_view>({"Phd."});

const auto slovakPrefixes = std::to_array<std::string_view>({"Ing.", "Mgr.", "JUDr.", "MUDr."});

const NameFormats slovakNameFormats{
    {{"{firstName} {lastName}", 8}, {"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName} {suffix}", 1}}};

const PeopleNames slovakPeopleNames{{slovakMaleFirstNames, slovakMaleLastNames, slovakPrefixes, slovakSuffixes},
                                    {slovakFemaleFirstNames, slovak_femalesLastNames, slovakPrefixes, slovakSuffixes},
                                    slovakNameFormats};

// Slovenia

const auto slovenianMaleFirstNames = std::to_array<std::string_view>({
    "Franc",     "Marko",      "Ivan",       "Andrej",    "Anton",    "Luka",      "Peter",      "Matej",
    "Milan",     "Robert",     "Branko",     "Bojan",     "Gregor",   "Stanislav", "Martin",     "David",
    "Igor",      "Dejan",      "Jan",        "Boris",     "Simon",    "Darko",     "Aleksander", "Roman",
    "Jakob",     "Vladimir",   "Janko",      "Mirko",     "Alen",     "Zoran",     "Filip",      "Miroslav",
    "Goran",     "Stanko",     "Mihael",     "Tim",       "Mark",     "Leon",      "Viktor",     "Dragan",
    "Erik",      "Benjamin",   "Rudolf",     "Zdravko",   "Danilo",   "Zlatko",    "Pavel",      "Edvard",
    "Bogdan",    "Patrik",     "Emil",       "Damir",     "Dominik",  "Daniel",    "Niko",       "Albin",
    "Aleks",     "Aleksandar", "Ivica",      "Tomislav",  "Mario",    "Leopold",   "Mladen",     "Valentin",
    "Davor",     "Karel",      "Marcel",     "Vlado",     "Oskar",    "Nenad",     "Ivo",        "Tian",
    "Elvis",     "Ladislav",   "Valter",     "Edin",      "Ervin",    "Sergej",    "Karl",       "Teo",
    "Ernest",    "Petar",      "Nikolaj",    "Vito",      "Renato",   "Alex",      "Slobodan",   "Mirsad",
    "Samir",     "Radovan",    "Liam",       "Dalibor",   "Bruno",    "Miro",      "Amir",       "Rudi",
    "Kevin",     "Bernard",    "Hasan",      "Albert",    "Adolf",    "Adam",      "Ferdinand",  "Dean",
    "Milorad",   "Branislav",  "Rafael",     "Emir",      "Taj",      "Almir",     "Muhamed",    "Edi",
    "Adnan",     "Alan",       "Lukas",      "Jon",       "Feliks",   "Ibrahim",   "Predrag",    "Tibor",
    "Vili",      "Ismet",      "Aleksej",    "Herman",    "Armin",    "Konrad",    "Haris",      "Leo",
    "Teodor",    "Branimir",   "Mehmed",     "Dino",      "Tristan",  "Jovan",     "Stefan",     "Ian",
    "Oliver",    "Dario",      "Oto",        "Husein",    "Kristian", "Mustafa",   "Henrik",     "Sven",
    "Gabriel",   "Edvin",      "Osman",      "Fikret",    "Tarik",    "Egon",      "Marino",     "Asim",
    "Miodrag",   "Rasim",      "Borislav",   "Ramiz",     "Fadil",    "Dragomir",  "Max",        "Amar",
    "Tom",       "Omer",       "Karlo",      "Patrick",   "Aldin",    "Adrian",    "Radoslav",   "Ahmet",
    "Sebastian", "Adem",       "Lev",        "Enver",     "Manuel",   "Ales",      "Tijan",      "Marsel",
    "Kris",      "Radomir",    "Vladislav",  "Arne",      "Kenan",    "Alexander", "Oton",       "Maksim",
    "Ali",       "Emanuel",    "Samuel",     "Antonio",   "Luca",     "Emin",      "Tomas",      "Mateo",
    "Milko",     "Jordan",     "Dani",       "Nick",      "Salih",    "Alfred",    "Otmar",      "Geza",
    "Aldo",      "Arif",       "Roberto",    "Kemal",     "Lazar",    "Tihomir",   "Miladin",    "Fredi",
    "Marin",     "Severin",    "Lucas",      "Fuad",      "Lars",     "Leonard",   "Bogomil",    "Leonardo",
    "Angel",     "Halid",      "Deni",       "Ruben",     "Radko",    "Zlatan",    "Thomas",     "Dorian",
    "Roland",    "Nedim",      "Simo",       "Halil",     "Ramadan",  "Sadik",     "Irfan",      "Smail",
    "Christian", "Adel",       "Adil",       "Dusan",     "Romeo",    "Fran",      "Naser",      "Sandro",
    "Said",      "Arben",      "Ahmed",      "Justin",    "Neven",    "Sami",      "Hubert",     "Sasa",
    "Eduard",    "Neo",        "Arnel",      "Jonatan",   "Julian",   "Michael",   "Andi",       "Amer",
    "Marco",     "Armando",    "Zoltan",     "Selim",     "Artur",    "Jonas",     "Harun",      "Juraj",
    "Anis",      "Rolando",    "Aziz",       "Kai",       "Demir",    "Aron",      "Matias",     "Faruk",
    "Loris",     "Hugo",       "Nail",       "Omar",      "Ismail",   "Ammar",     "Elvin",      "Naim",
    "Nikolas",   "Evan",       "Taras",      "Zaim",      "Sergio",   "Kosta",     "Tobias",     "Djordje",
    "Aladin",    "Natan",      "Abdulah",    "Valentino", "Zahid",    "Simeon",    "Matteo",     "Alin",
    "Ale",       "Eldar",      "Mauro",      "Hamza",     "Benedikt", "Diego",     "Murat",      "Isak",
    "Michel",    "Silvano",    "Alessandro", "Svetozar",  "Adriano",  "Sani",      "Fabio",      "Sinan",
    "Paolo",     "Rupert",     "Jaroslav",   "Herbert",   "Claudio",  "Inti",      "Hamid",      "Ilir",
    "Sergey",    "Florian",    "Shaban",     "Nel",       "Todor",    "Helmut",    "Dimitar",    "Arpad",
    "Tahir",     "Konstantin", "Mate",       "Job",       "Luan",     "Maxim",     "Walter",     "Mitko",
    "Edmond",    "Oleg",       "Vital",      "Bara",      "Veselin",  "Andrey",    "Petko",      "Faik",
    "Alban",     "Munir",      "Stanimir",   "Kian",      "Dinko",    "Engelbert",
});

const auto slovenianFemaleFirstNames = std::to_array<std::string_view>({
    "Marija",     "Ana",       "Maja",     "Marjan",     "Irena",    "Nina",      "Barbara",   "Petra",
    "Katja",      "Anja",      "Eva",      "Sara",       "Sonja",    "Tanja",     "Tina",      "Tatjana",
    "Katarina",   "Alenka",    "Milena",   "Vesna",      "Martina",  "Majda",     "Nika",      "Ivana",
    "Helena",     "Anica",     "Nada",     "Darja",      "Dragica",  "Kristina",  "Simona",    "Danica",
    "Olga",       "Suzana",    "Zdenka",   "Lara",       "Lidija",   "Denis",     "Sabina",    "Ema",
    "Angela",     "Vida",      "Marta",    "Ivanka",     "Silva",    "Veronika",  "Karmen",    "Darinka",
    "Aleksandra", "Anita",     "Brigita",  "Lana",       "Jana",     "Kaja",      "Klara",     "Monika",
    "Stanislava", "Lea",       "Natalija", "Nevenka",    "Jasmina",  "Elizabeta", "Tamara",    "Renata",
    "Branka",     "Slavica",   "Julija",   "Hana",       "Bernarda", "Bojana",    "Vera",      "Erika",
    "Danijela",   "Romana",    "Pavla",    "Mira",       "Laura",    "Valentina", "Sandra",    "Mirjana",
    "Sanja",      "Valerija",  "Gal",      "Ines",       "Ksenija",  "Mihaela",   "Ida",       "Pia",
    "Karolina",   "Viktorija", "Marina",   "Mia",        "Melita",   "Vlasta",    "Ela",       "Magdalena",
    "Albina",     "Lan",       "Gordana",  "Ljubica",    "Nadja",    "Nikola",    "Marinka",   "Karin",
    "Tea",        "Matilda",   "Marica",   "Vita",       "Emilija",  "Larisa",    "Lina",      "Iva",
    "Milka",      "Milica",    "Marijana", "Jelena",     "Rebeka",   "Justina",   "Mirjam",    "Stanka",
    "Rene",       "Slavka",    "Irma",     "Jasmin",     "Zlatka",   "Iris",      "Iza",       "Meta",
    "Jolanda",    "Miroslava", "Blanka",   "Tara",       "Manja",    "Alma",      "Nino",      "Erna",
    "Anka",       "Alina",     "Elena",    "Neli",       "Hermina",  "Zora",      "Magda",     "Daniela",
    "Vilma",      "Diana",     "Ljiljana", "Dragana",    "Dijana",   "Ingrid",    "Biserka",   "Mara",
    "Edita",      "Sofija",    "Mila",     "Sanela",     "Zorica",   "Manuela",   "Emina",     "Margareta",
    "Biljana",    "Silvestra", "Zlata",    "Lili",       "Lia",      "Elvira",    "Judita",    "Svetlana",
    "Roza",       "Dunja",     "Silvana",  "Silvija",    "Kim",      "Zara",      "Cirila",    "Doris",
    "Gizela",     "Neva",      "Vanda",    "Berta",      "Nikolina", "Hilda",     "Vanesa",    "Pavlina",
    "Mirela",     "Tone",      "Leonida",  "Violeta",    "Dolores",  "Zorka",     "Nermin",    "Karol",
    "Radmila",    "Dora",      "Fatima",   "Azra",       "Melisa",   "Agata",     "Liza",      "Aida",
    "Anika",      "Ina",       "Gabriela", "Izabela",    "Minka",    "Milenka",   "Jasminka",  "Adela",
    "Regina",     "Edina",     "Sofia",    "Leopoldina", "Karla",    "Mina",      "Vladimira", "Lena",
    "Greta",      "Selma",     "Samanta",  "Kiara",      "Evelin",   "Frida",     "Anna",      "Andrea",
    "Anel",       "Almira",    "Ada",      "Teodora",    "Adriana",  "Ester",     "Lorena",    "Neda",
    "Miranda",    "Mai",       "Amel",     "Luna",       "Dominika", "Rozina",    "France",    "Amra",
    "Bogdana",    "Patricia",  "Amina",    "Elma",       "Stana",    "Alisa",     "Jovanka",   "Stella",
    "Stela",      "Rosana",    "Dajana",   "Gloria",     "Rada",     "Ariana",    "Tinka",     "Suad",
    "Melani",     "Ella",      "Nihad",    "Medina",     "Tiana",    "Evelina",   "Vanessa",   "Sabrina",
    "Irina",      "Amanda",    "Danila",   "Nia",        "Maria",    "Emma",      "Nena",      "Zdravka",
    "Julia",      "Mirza",     "Esma",     "Noel",       "Lian",     "Sarah",     "Fani",      "Samira",
    "Jozefa",     "Amira",     "Ivona",    "Bruna",      "Nikita",   "Verona",    "Nela",      "Lenka",
    "Eli",        "Indira",    "Natasa",   "Natalia",    "Sanda",    "Leila",     "Hanna",     "Tereza",
    "Marja",      "Olivera",   "Eda",      "Estera",     "Ava",      "Nana",      "Romina",    "Jovana",
    "Rafaela",    "Linda",     "Dan",      "Loredana",   "Meri",     "Adelina",   "Emanuela",  "Aurora",
    "Fadila",     "Dina",      "Sava",     "Karina",     "Elza",     "Lin",       "Lora",      "Fatime",
    "Melanie",    "Anastasia", "Dana",     "Mirka",      "Evita",    "Rosa",      "Angelina",  "Elda",
    "Marisa",     "Carmen",    "Manda",    "Dorica",     "Gaia",     "Claudia",   "Eman",      "Denisa",
    "Tatiana",    "Janka",     "Lijana",   "Dea",        "Olena",    "Erin",      "Noemi",     "Edisa",
    "Inge",       "Leonora",   "Bela",     "Verena",     "Trina",    "Arian",
});

const auto slovenianLastNames = std::to_array<std::string_view>({
    "Novak",     "Horvat",     "Krajnc",     "Kovač",     "Korošec",     "Mlakar",    "Kos",        "Vidmar",
    "Golob",     "žagar",      "Hočevar",    "Košir",     "Turk",        "Kralj",     "Zupan",      "Bizjak",
    "Hribar",    "Rozman",     "Zajc",       "Kotnik",    "Oblak",       "Petek",     "Kolar",      "Kastelic",
    "Zupanc",    "Koren",      "Babič",      "Kranjc",    "Medved",      "Majcen",    "Knez",       "Pušnik",
    "Kovačević", "Dolenc",     "Pirc",       "Petrović",  "Hrovat",      "Lah",       "Zadravec",   "Kuhar",
    "Lešnik",    "Bezjak",     "Kokalj",     "Jug",       "Zorko",       "Erjavec",   "Sever",      "Vidovič",
    "Jerman",    "Jereb",      "Bogataj",    "Primožič",  "Jovanović",   "Kosi",      "Hodžić",     "Lesjak",
    "Furlan",    "Lavrič",     "Perko",      "Logar",     "žižek",       "Marković",  "Debeljak",   "Jenko",
    "Krivec",    "Pavlin",     "Vodopivec",  "Zver",      "Rupnik",      "Jarc",      "Kramberger", "Toplak",
    "Breznik",   "Ilić",       "Hribernik",  "Jazbec",    "žibert",      "Marolt",    "Vidic",      "Eržen",
    "Pintar",    "Nikolić",    "Simonič",    "železnik",  "Demšar",      "Blažič",    "Jelen",      "Popović",
    "šinkovec",  "Fras",       "Dolinar",    "Ramšak",    "Hozjan",      "Leban",     "Cerar",      "Hren",
    "Javornik",  "Rus",        "Blatnik",    "Kobal",     "Gorjup",      "Kolenc",    "Podgoršek",  "Savić",
    "Rajh",      "Kokol",      "Koželj",     "Gajšek",    "Mohorič",     "Lazar",     "Nemec",      "Sušnik",
    "Gomboc",    "Mavrič",     "Mrak",       "šmid",      "Rožman",      "Cvetko",    "Pavlović",   "Bajc",
    "Likar",     "Vovk",       "Babić",      "Bevc",      "Tratnik",     "Povše",     "Zakrajšek",  "Begić",
    "Humar",     "Skok",       "Gorenc",     "Hadžić",    "Stopar",      "Rojc",      "Stojanović", "Založnik",
    "Leskovar",  "Sitar",      "Gorišek",    "štrukelj",  "šuligoj",     "Kranjec",   "Fekonja",    "škof",
    "Zorec",     "Romih",      "šuštar",     "Pahor",     "Bračko",      "Pevec",     "šturm",      "Volk",
    "Kukovec",   "Brglez",     "Simčič",     "Dolinšek",  "Kogovšek",    "Križman",   "Bregar",     "Arh",
    "Krašovec",  "Murko",      "Papež",      "Turnšek",   "Bukovec",     "Kosmač",    "Rutar",      "Tomažin",
    "Kramar",    "Starc",      "Vuković",    "Jamnik",    "Hafner",      "Vrhovnik",  "Planinc",    "Dizdarević",
    "Ferk",      "Vončina",    "Ravnikar",   "Bergant",   "Lebar",       "Lipovšek",  "Godec",      "Vuk",
    "Tomić",     "Lončar",     "Praprotnik", "Hojnik",    "Kumer",       "Halilović", "Grošelj",    "Urh",
    "Mlinar",    "Kristan",    "Stanković",  "Habjan",    "Resnik",      "Klemenc",   "Cimerman",   "Balažic",
    "Kalan",     "Cafuta",     "Jeraj",      "Gruden",    "Zalokar",     "Primc",     "Zajec",      "Fabjan",
    "Pirnat",    "Možina",     "Topolovec",  "Zorman",    "Trstenjak",   "Hudorovac", "Kraševec",   "Zalar",
    "Rojko",     "Lampret",    "Peternelj",  "Hvala",     "Gantar",      "Rijavec",   "Grm",        "Roškar",
    "Repnik",    "Kladnik",    "Planinšek",  "Delić",     "Sedej",       "Smrekar",   "Gobec",      "Frelih",
    "Simić",     "Rebernik",   "Lukić",      "šega",      "Ornik",       "Babnik",    "Leskovšek",  "Drnovšek",
    "Mastnak",   "Kušar",      "Hrastnik",   "Pajk",      "Vogrinec",    "Krapež",    "žnidar",     "Avsec",
    "šabić",     "Kocbek",     "Strnad",     "Jović",     "Rogelj",      "Juvan",     "Vesel",      "Knežević",
    "Kotar",     "šubic",      "Mohar",      "Križaj",    "Požar",       "Plohl",     "Mitrović",   "Kopač",
    "Rudolf",    "šket",       "Kržišnik",   "Beganović", "štefančič",   "Dobnik",    "Ristić",     "Kocjan",
    "Bajt",      "Polanec",    "Dežman",     "Lazić",     "Dedić",       "Gashi",     "Avdić",      "Gradišnik",
    "Plevnik",   "Drevenšek",  "Todorović",  "Gorše",     "Sluga",       "Sajovic",   "Smodiš",     "Ocvirk",
    "Bernik",    "Bertoncelj", "Kosec",      "Fajfar",    "Hriberšek",   "Poljanšek", "Jager",      "Slana",
    "Podobnik",  "Malovrh",    "Kmetec",     "Pipan",     "Jelenc",      "Ficko",     "škulj",      "Veber",
    "Peternel",  "Zidar",      "Stropnik",   "Marić",     "Podlesnik",   "Grilc",     "Vodušek",    "Vogrin",
    "Bašić",     "Kenda",      "Kern",       "Tušek",     "Bytyqi",      "Arnuš",     "Stare",      "Pišek",
    "Plut",      "Nagode",     "Hajdinjak",  "Zagorc",    "Kostanjevec", "Klinc",     "Mulec",      "Benko",
    "Laznik",    "žitnik",     "Janković",   "Mali",      "Oman",        "Fink",      "Ogrizek",    "Mihelčič",
    "Bajec",     "Pivk",       "Skubic",     "Kočar",     "Virant",      "Oražem",    "Boštjančič", "Grobelnik",
    "Markelj",   "Tomše",      "Bratuša",    "Saje",      "Makovec",     "Križan",
});

const NameFormats slovenianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames slovenianPeopleNames{{slovenianMaleFirstNames, slovenianLastNames, {}, {}},
                                       {slovenianFemaleFirstNames, slovenianLastNames, {}, {}},
                                       slovenianNameFormats};

// South Africa

const auto southAfricanMaleFirstNames = std::to_array<std::string_view>({
    "Johan",   "Robert", "Michael",   "William", "Willem",  "David",   "Richard",  "Thomas",   "Charl",   "Christopher",
    "Daniel",  "Dante",  "Paul",      "Mark",    "George",  "Kenneth", "Steven",   "Edward",   "Ronald",  "Anthony",
    "Albert",  "Kevin",  "Jaco",      "Jacobus", "Mathuys", "Frankie", "Stephen",  "Andre",    "Raymond", "Joshua",
    "Dennis",  "Pieter", "Henrie",    "Rigard",  "Riaan",   "Joe",     "Johannes", "Hannes",   "Gerald",  "Gerhard",
    "Willie",  "Roy",    "Adam",      "Harry",   "Wayne",   "Billy",   "Steve",    "Louis",    "Eugene",  "Russell",
    "Bobbie",  "Victor", "Martin",    "Ernest",  "Phillip", "Craig",   "Alan",     "Shawn",    "Chris",   "Earl",
    "Jimmy",   "Brian",  "Mike",      "Leonard", "Dale",    "Allen",   "Vincent",  "Francois", "Eddie",   "Alexander",
    "Bernard", "Markus", "Micheal",   "Theo",    "Oscar",   "Derek",   "Wesley",   "Derrick",  "Herman",  "Rick",
    "Ruben",   "Cecil",  "Roland",    "Harvey",  "Adriaan", "Karl",    "Erik",     "Neil",     "Ian",     "Iwan",
    "Julian",  "Nick",   "Shaun",     "Cameron", "Wilbur",  "Rudolph", "Rudy",     "Bennie",   "Lukas",   "Simon",
    "Rufus",   "Hugo",   "Conrad",    "Tommie",  "Jan",     "Jacques", "Morne",    "Vernon",   "Duanne",  "Theunis",
    "Theuns",  "Wessel", "Stephaans",
});

const auto southAfricanFemaleFirstNames = std::to_array<std::string_view>({
    "Susan",     "Monica",   "Linda",     "Elsa",      "Margaret",   "Lisa",     "Karen",      "Helen",     "Sandra",
    "Sara",      "Kimberly", "Angelique", "Melissa",   "Brenda",     "Anna",     "Annelie",    "Katryn",    "Amanda",
    "Stefanie",  "Marie",    "Janet",     "Rosemarie", "Nicoleen",   "Paula",    "Robin",      "Rita",      "Edna",
    "Carmen",    "Cindy",    "Edith",     "Ethel",     "Ellen",      "Elaine",   "Charlotte",  "Pauline",   "Juanita",
    "Anita",     "Rhonda",   "Hazel",     "Debbie",    "Clara",      "Lucille",  "Eleanor",    "Alicia",    "Michele",
    "Geraldine", "Erika",    "Bernice",   "Audrey",    "Yvonne",     "Annette",  "Renette",    "Ida",       "Melanie",
    "Jolanda",   "Vanessa",  "Alma",      "Sue-Marie", "Carla",      "Rosemary", "Wilma",      "Kristin",   "Natalie",
    "Charlene",  "Melinda",  "Maureen",   "Tanya",     "Marlene",    "Heidi",    "Lydia",      "Vickie",    "Nina",
    "Leona",     "Jenny",    "Sonia",     "Kristina",  "Katrina",    "Belinda",  "Natasha",    "Cecile",    "Angie",
    "Lynda",     "Amelia",   "Monique",   "Kayla",     "Yvette",     "Olivia",   "Antoinette", "Bridgette", "Karla",
    "Leticia",   "Krista",   "Robyn",     "Rosalie",   "Bernadette", "Krystal",  "Nadine",     "Estelle",   "Lynette",
    "Eloise",    "Jana",     "Kerry",     "Jenna",     "Tasha",      "Sonja",    "Elisa",      "Kristie",
});

const auto southAfricanLastNames = std::to_array<std::string_view>({
    "van de Merwe", "Schoeman",
    "Barnard",      "de Kock",
    "Meintjies",    "le Roux",
    "Koen",         "Morkel",
    "Viljoen",      "Smit",
    "Nel",          "Grobelaar",
    "Oppenheimer",  "Castelyn",
    "du Preez",     "Cronnje",
    "Donald",       "Pringle",
    "Snell",        "Burger",
    "van Heerden",  "van de Heefer",
    "Wessels",      "Eksteen",
    "Kirsten",      "de Villiers",
    "Olivier",      "Steyn",
    "Kallis",       "Ackerman",
    "Dippenaar",    "Strydon",
    "Boje",         "Pretorius",
    "Langeveldt",   "Botha",
    "Duminy",       "Fuller",
    "Philander",    "Daniels",
    "Fichardt",     "van Zyl",
    "de Bruyn",     "van der Bijl",
    "Dyer",         "van Buuren",
    "Boyes",        "Versfeld",
    "Bisset",       "Castens",
    "Louw",         "Powell",
    "Snedden",      "van Renen",
    "Myburg",       "Bredenkamp",
    "Mellett",      "Hertzog",
    "Theunissen",   "de Waal",
    "Cloete",       "Krige",
    "Melker",       "Loubser",
    "Stegmann",     "Joubert",
    "Luyt",         "Roos",
    "Lombard",      "van der Hoff",
    "Immelman",     "Els",
    "Delaney",      "Strauss",
    "Meyer",        "Pienaar",
    "du Plessis",   "van Rooyen",
    "Kruger",       "Mostert",
    "Scholtz",      "Aucamp",
    "Albertyn",     "Bosman",
    "Bester",       "Truter",
    "Prinsloo",     "van Niekerk",
    "Zimmerman",    "Venter",
    "van den Berg", "de Wet",
    "Marais",       "van Jaarsveld",
    "Jordaan",      "Malan",
    "Viviers",      "Myburgh",
    "Hoffman",      "Bekker",
    "Rossouw",      "Rens",
    "Lochner",      "Hanekom",
    "Schmidt",      "Wentzel",
    "Bezuidenhout", "Cilliers",
    "Naude",        "de Vos",
    "Goosen",       "Durand",
    "Potgieter",    "van Deventer",
    "de Klerk",     "Spies",
    "Snyman",       "Oosthuizen",
    "Bosch",        "Vogel",
    "Fourie",       "van Staden",
    "Wagenaar",     "Wolmerans",
    "Veldsman",     "Tromp",
    "Serfontein",   "Claasen",
    "Heunis",       "Visagie",
    "Ferreira",     "Erasmus",
    "Knoetze",      "Jansen van Rensburg",
    "Styger",       "Roberts",
    "Richter",      "Lotter",
    "Swart",        "Badenhorst",
    "Laubscher",    "Hattingh",
    "Visser",       "Brink",
    "Theron",       "Paulse",
    "Basson",       "van der Westhuyzen",
    "Gerber",       "Human",
    "Uys",          "Hougaard",
    "Steenkamp",    "Pieterse",
    "Brits",        "Jantjies",
    "Etzebeth",     "de Jager",
    "Boshoff",      "Kriel",
    "Vosloo",       "Carstens",
});

const NameFormats southAfricanNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames southAfricanPeopleNames{{southAfricanMaleFirstNames, southAfricanLastNames, {}, {}},
                                          {southAfricanFemaleFirstNames, southAfricanLastNames, {}, {}},
                                          southAfricanNameFormats};

// Spain

const auto spanishMaleFirstNames = std::to_array<std::string_view>({
    "Agapetus", "Aimon",      "Ángel",     "Beltran",      "Berto",     "Bronco",    "Carles",    "Cipriano",
    "Cisco",    "Cortez",     "Cruz",      "Cuba",         "Daniel",    "Dario",     "Desiderio", "Diego",
    "Dimos",    "Fanuco",     "Federico",  "Fraco",        "Francisco", "Frisco",    "Gervasio",  "Gig",
    "Gonzalo",  "Guido",      "Guillermo", "Hermenegildo", "Hernan",    "Hilario",   "Ignado",    "Isidro",
    "Iván",     "Jaguar",     "Jair",      "Javier",       "Jerrold",   "Jordi",     "Jorge",     "Josep",
    "Juan",     "Juan Ramón", "Kiki",      "Larenzo",      "Lisandro",  "Loredo",    "Lorenzo",   "Macario",
    "Malvolio", "Manuel",     "Marjun",    "Miguel",       "Montana",   "Montego",   "Montel",    "Montenegro",
    "Nasario",  "Nemesio",    "Neper",     "Neron ",       "Nicandro",  "Noe",       "Oihane",    "Oleos",
    "Olindo",   "Onofre",     "Ora",       "Oro",          "Ovidio",    "Pablo",     "Paco",      "Pancho",
    "Pedro",    "Pío",        "Ponce",     "Porfio",       "Quico",     "Quinto",    "Quiqui",    "Quito",
    "Reyes",    "Reynaldo",   "Ricardo",   "Rico",         "Rio",       "Rogelio",   "Rubén",     "Ryo",
    "Sagaz",    "Salvador",   "Santiago",  "Saturnin",     "Seferino",  "Selestino", "Senon",     "Sergi",
    "Sierra",   "Tajo",       "Urvano",    "Vegas",        "Venacio",   "Vicente",   "Vidal",     "Xabat",
    "Xalvador", "Xaver",      "Xaverius",  "Xavier",       "Xavion",    "Xeres",     "Ximen",     "Zelipe",
    "Zenon ",
});

const auto spanishFemaleFirstNames = std::to_array<std::string_view>({
    "Adalia",    "Aidia",    "Alva",       "Ana",       "Andrea",   "Anni",      "Aureliano", "Belinda",
    "Bettina",   "Carey",    "Carlotta",   "Coco",      "Damita",   "Delfina",   "Duenna",    "Dulcie",
    "Elvira",    "Enriqua",  "Esmerelda",  "Esperanza", "Fe",       "Fonda",     "Frida",     "Friera",
    "Gitana",    "Gotzone",  "Guadalupe",  "Hermosa",   "Ines",     "Isabel",    "Itzel",     "Jade",
    "Jardena",   "Jennifer", "Julitta",    "Kesare",    "Kiki",     "Lacienega", "Ladonna",   "Landrada",
    "Laura",     "Lela",     "Lenora",     "Leya",      "Liani",    "Linda",     "Lluvia",    "Lola",
    "Lolita",    "Lorena",   "Luisa",      "Lujuana",   "Lupita",   "Lux",       "Luz",       "Madeira ",
    "Madra",     "Maica",    "Maitane",    "Maitea",    "Majondra", "Manda",     "Manuela",   "María de los Ángeles",
    "Mariola",   "Mariposa", "Marisa",     "Marta",     "Matilde",  "Melosa",    "Mio",       "Mira",
    "Montana",   "Mora",     "Neiva",      "Nerita",    "Neva",     "Nieve",     "Nina",      "Novia",
    "Novyanna",  "Oihane",   "Ora",        "Osana",     "Paca",     "Paloma",    "Placida",   "Puma",
    "Querida",   "Queta",    "Quinceanos", "Raeka",     "Rafaela",  "Ramona",    "Rana",      "Ria",
    "Rigoberta", "Roana",    "Rosalind",   "Rosalynda", "Roser",    "Sally",     "Salma",     "Sancha",
    "Sancho",    "Sandia",   "Santana",    "Santina",   "Savana",   "Savanna",   "Seina ",    "Senalda",
    "Senona",    "Sierra",   "Sofiel",     "Solana",    "Soledad",  "Sonora",    "Taina",     "Tea",
    "Tejana",    "Terceiro", "Teresa",     "Texcean",   "Tia",      "Tierra",    "Tina",      "Tranquilia",
    "Trella",    "Tulia",    "Ursulina",   "Verdad",    "Veta",     "Vina",      "Xalbadora", "Xevera",
    "Xiao",      "Xiomara",  "Yadra",      "Yomaris",   "Zaneta",   "Zarela",    "Zarita",    "Zaviera",
    "Zequinha",  "Zuriaa",   "Zurine ",
});

const auto spanishLastNames = std::to_array<std::string_view>({"Abeyta",
                                                               "Abrego",
                                                               "Abreu",
                                                               "Acevedo",
                                                               "Acosta",
                                                               "Acuña",
                                                               "Adame",
                                                               "Adorno",
                                                               "Agosto",
                                                               "Aguayo",
                                                               "Aguilar",
                                                               "Aguilera",
                                                               "Aguirre",
                                                               "Alanis",
                                                               "Alaníz",
                                                               "Alarcón",
                                                               "Alba",
                                                               "Alcalá",
                                                               "Alcántar",
                                                               "Alcaraz",
                                                               "Alejandro",
                                                               "Alemán",
                                                               "Alfaro",
                                                               "Alicea",
                                                               "Almanza",
                                                               "Almaráz",
                                                               "Almonte",
                                                               "Alonso",
                                                               "Alonzo",
                                                               "Altamirano",
                                                               "Alva",
                                                               "Alvarado",
                                                               "Álvarez",
                                                               "Amador",
                                                               "Amaya",
                                                               "Anaya",
                                                               "Anguiano",
                                                               "Angulo",
                                                               "Aparicio",
                                                               "Apodaca",
                                                               "Aponte",
                                                               "Aragón",
                                                               "Araña",
                                                               "Aranda",
                                                               "Arce",
                                                               "Archuleta",
                                                               "Arellano",
                                                               "Arenas",
                                                               "Arevalo",
                                                               "Argüello",
                                                               "Arias",
                                                               "Armas",
                                                               "Armendáriz",
                                                               "Armenta",
                                                               "Armijo",
                                                               "Arredondo",
                                                               "Arreola",
                                                               "Arriaga",
                                                               "Arroyo",
                                                               "Arteaga",
                                                               "Atencio",
                                                               "Ávalos",
                                                               "Ávila",
                                                               "Avilés",
                                                               "Ayala",
                                                               "Baca",
                                                               "Badillo",
                                                               "Báez",
                                                               "Baeza",
                                                               "Bahena",
                                                               "Balderas",
                                                               "Ballesteros",
                                                               "Banda",
                                                               "Bañuelos",
                                                               "Barajas",
                                                               "Barela",
                                                               "Barragán",
                                                               "Barraza",
                                                               "Barrera",
                                                               "Barreto",
                                                               "Barrientos",
                                                               "Barrios",
                                                               "Batista",
                                                               "Becerra",
                                                               "Beltrán",
                                                               "Benavides",
                                                               "Benavídez",
                                                               "Benítez",
                                                               "Bermúdez",
                                                               "Bernal",
                                                               "Berríos",
                                                               "Betancourt",
                                                               "Blanco",
                                                               "Bonilla",
                                                               "Borrego",
                                                               "Botello",
                                                               "Bravo",
                                                               "Briones",
                                                               "Briseño",
                                                               "Brito",
                                                               "Bueno",
                                                               "Burgos",
                                                               "Bustamante",
                                                               "Bustos",
                                                               "Caballero",
                                                               "Cabán",
                                                               "Cabrera",
                                                               "Cadena",
                                                               "Caldera",
                                                               "Calderón",
                                                               "Calvillo",
                                                               "Camacho",
                                                               "Camarillo",
                                                               "Campos",
                                                               "Canales",
                                                               "Candelaria",
                                                               "Cano",
                                                               "Cantú",
                                                               "Caraballo",
                                                               "Carbajal",
                                                               "Cárdenas",
                                                               "Cardona",
                                                               "Carmona",
                                                               "Carranza",
                                                               "Carrasco",
                                                               "Carrasquillo",
                                                               "Carreón",
                                                               "Carrera",
                                                               "Carrero",
                                                               "Carrillo",
                                                               "Carrión",
                                                               "Carvajal",
                                                               "Casanova",
                                                               "Casares",
                                                               "Casarez",
                                                               "Casas",
                                                               "Casillas",
                                                               "Castañeda",
                                                               "Castellanos",
                                                               "Castillo",
                                                               "Castro",
                                                               "Cavazos",
                                                               "Cazares",
                                                               "Ceballos",
                                                               "Cedillo",
                                                               "Ceja",
                                                               "Centeno",
                                                               "Cepeda",
                                                               "Cerda",
                                                               "Cervantes",
                                                               "Cervántez",
                                                               "Chacón",
                                                               "Chapa",
                                                               "Chavarría",
                                                               "Chávez",
                                                               "Cintrón",
                                                               "Cisneros",
                                                               "Collado",
                                                               "Collazo",
                                                               "Colón",
                                                               "Colunga",
                                                               "Concepción",
                                                               "Contreras",
                                                               "Cordero",
                                                               "Córdova",
                                                               "Cornejo",
                                                               "Corona",
                                                               "Coronado",
                                                               "Corral",
                                                               "Corrales",
                                                               "Correa",
                                                               "Cortés",
                                                               "Cortéz",
                                                               "Cotto",
                                                               "Covarrubias",
                                                               "Crespo",
                                                               "Cruz",
                                                               "Cuellar",
                                                               "Curiel",
                                                               "Dávila",
                                                               "de Anda",
                                                               "de Jesús",
                                                               "Delacrúz",
                                                               "Delafuente",
                                                               "Delagarza",
                                                               "Delao",
                                                               "Delapaz",
                                                               "Delarosa",
                                                               "Delatorre",
                                                               "Deleón",
                                                               "Delgadillo",
                                                               "Delgado",
                                                               "Delrío",
                                                               "Delvalle",
                                                               "Díaz",
                                                               "Domínguez",
                                                               "Duarte",
                                                               "Dueñas",
                                                               "Durán",
                                                               "Echevarría",
                                                               "Elizondo",
                                                               "Enríquez",
                                                               "Escalante",
                                                               "Escamilla",
                                                               "Escobar",
                                                               "Escobedo",
                                                               "Esparza",
                                                               "Espinal",
                                                               "Espino",
                                                               "Espinosa",
                                                               "Espinosa de los Monteros",
                                                               "Espinoza",
                                                               "Esquibel",
                                                               "Esquivel",
                                                               "Estévez",
                                                               "Estrada",
                                                               "Fajardo",
                                                               "Farías",
                                                               "Feliciano",
                                                               "Fernández",
                                                               "Ferrer",
                                                               "Fierro",
                                                               "Figueroa",
                                                               "Flores",
                                                               "Flórez",
                                                               "Fonseca",
                                                               "Frías",
                                                               "Fuentes",
                                                               "Gaitán",
                                                               "Galarza",
                                                               "Galindo",
                                                               "Gallardo",
                                                               "Gallegos",
                                                               "Galván",
                                                               "Gálvez",
                                                               "Gamboa",
                                                               "Gámez",
                                                               "Gaona",
                                                               "Garay",
                                                               "García",
                                                               "Garibay",
                                                               "Garica",
                                                               "Garrido",
                                                               "Garza",
                                                               "Gastélum",
                                                               "Gaytán",
                                                               "Gil",
                                                               "Girón",
                                                               "Godínez",
                                                               "Godoy",
                                                               "Gómez",
                                                               "González",
                                                               "Gracia",
                                                               "Granado",
                                                               "Granados",
                                                               "Griego",
                                                               "Grijalva",
                                                               "Guajardo",
                                                               "Guardado",
                                                               "Guerra",
                                                               "Guerrero",
                                                               "Guevara",
                                                               "Guillén",
                                                               "Gurule",
                                                               "Gutiérrez",
                                                               "Guzmán",
                                                               "Haro",
                                                               "Henríquez",
                                                               "Heredia",
                                                               "Hernández",
                                                               "Herrera",
                                                               "Hidalgo",
                                                               "Hinojosa",
                                                               "Holguín",
                                                               "Huerta",
                                                               "Hurtado",
                                                               "Ibarra",
                                                               "Iglesias",
                                                               "Irizarry",
                                                               "Jaime",
                                                               "Jaimes",
                                                               "Jáquez",
                                                               "Jaramillo",
                                                               "Jasso",
                                                               "Jiménez",
                                                               "Juárez",
                                                               "Jurado",
                                                               "Laboy",
                                                               "Lara",
                                                               "Laureano",
                                                               "Leal",
                                                               "Lebrón",
                                                               "Ledesma",
                                                               "Leiva",
                                                               "Lemus",
                                                               "León",
                                                               "Lerma",
                                                               "Leyva",
                                                               "Limón",
                                                               "Linares",
                                                               "Lira",
                                                               "Llamas",
                                                               "Loera",
                                                               "Lomeli",
                                                               "Longoria",
                                                               "López",
                                                               "Lovato",
                                                               "Loya",
                                                               "Lozada",
                                                               "Lozano",
                                                               "Lucero",
                                                               "Lucio",
                                                               "Luevano",
                                                               "Lugo",
                                                               "Luna",
                                                               "Macías",
                                                               "Madera",
                                                               "Madrid",
                                                               "Madrigal",
                                                               "Maestas",
                                                               "Magaña",
                                                               "Malave",
                                                               "Maldonado",
                                                               "Manzanares",
                                                               "Mares",
                                                               "Marín",
                                                               "Márquez",
                                                               "Marrero",
                                                               "Marroquín",
                                                               "Martínez",
                                                               "Mascareñas",
                                                               "Mata",
                                                               "Mateo",
                                                               "Matías",
                                                               "Matos",
                                                               "Maya",
                                                               "Mayorga",
                                                               "Medina",
                                                               "Medrano",
                                                               "Mejía",
                                                               "Meléndez",
                                                               "Melgar",
                                                               "Mena",
                                                               "Menchaca",
                                                               "Méndez",
                                                               "Mendoza",
                                                               "Menéndez",
                                                               "Meraz",
                                                               "Mercado",
                                                               "Merino",
                                                               "Mesa",
                                                               "Meza",
                                                               "Miramontes",
                                                               "Miranda",
                                                               "Mireles",
                                                               "Mojica",
                                                               "Molina",
                                                               "Mondragón",
                                                               "Monroy",
                                                               "Montalvo",
                                                               "Montañez",
                                                               "Montaño",
                                                               "Montemayor",
                                                               "Montenegro",
                                                               "Montero",
                                                               "Montes",
                                                               "Montéz",
                                                               "Montoya",
                                                               "Mora",
                                                               "Morales",
                                                               "Moreno",
                                                               "Mota",
                                                               "Moya",
                                                               "Munguía",
                                                               "Muñiz",
                                                               "Muñoz",
                                                               "Murillo",
                                                               "Muro",
                                                               "Nájera",
                                                               "Naranjo",
                                                               "Narváez",
                                                               "Nava",
                                                               "Navarrete",
                                                               "Navarro",
                                                               "Nazario",
                                                               "Negrete",
                                                               "Negrón",
                                                               "Nevárez",
                                                               "Nieto",
                                                               "Nieves",
                                                               "Niño",
                                                               "Noriega",
                                                               "Núñez",
                                                               "Ocampo",
                                                               "Ocasio",
                                                               "Ochoa",
                                                               "Ojeda",
                                                               "Olivares",
                                                               "Olivárez",
                                                               "Olivas",
                                                               "Olivera",
                                                               "Olivo",
                                                               "Olmos",
                                                               "Olvera",
                                                               "Ontiveros",
                                                               "Oquendo",
                                                               "Ordóñez",
                                                               "Orellana",
                                                               "Ornelas",
                                                               "Orosco",
                                                               "Orozco",
                                                               "Orta",
                                                               "Ortega",
                                                               "Ortiz",
                                                               "Osorio",
                                                               "Otero",
                                                               "Ozuna",
                                                               "Pabón",
                                                               "Pacheco",
                                                               "Padilla",
                                                               "Padrón",
                                                               "Páez",
                                                               "Pagan",
                                                               "Palacios",
                                                               "Palomino",
                                                               "Palomo",
                                                               "Pantoja",
                                                               "Paredes",
                                                               "Parra",
                                                               "Partida",
                                                               "Patiño",
                                                               "Paz",
                                                               "Pedraza",
                                                               "Pedroza",
                                                               "Pelayo",
                                                               "Peña",
                                                               "Perales",
                                                               "Peralta",
                                                               "Perea",
                                                               "Pérez",
                                                               "Pichardo",
                                                               "Piña",
                                                               "Pineda",
                                                               "Pizarro",
                                                               "Polanco",
                                                               "Ponce",
                                                               "Porras",
                                                               "Portillo",
                                                               "Posada",
                                                               "Prado",
                                                               "Preciado",
                                                               "Prieto",
                                                               "Puente",
                                                               "Puga",
                                                               "Pulido",
                                                               "Quesada",
                                                               "Quezada",
                                                               "Quiñones",
                                                               "Quiñónez",
                                                               "Quintana",
                                                               "Quintanilla",
                                                               "Quintero",
                                                               "Quiróz",
                                                               "Rael",
                                                               "Ramírez",
                                                               "Ramos",
                                                               "Rangel",
                                                               "Rascón",
                                                               "Raya",
                                                               "Razo",
                                                               "Regalado",
                                                               "Rendón",
                                                               "Rentería",
                                                               "Reséndez",
                                                               "Reyes",
                                                               "Reyna",
                                                               "Reynoso",
                                                               "Rico",
                                                               "Rincón",
                                                               "Riojas",
                                                               "Ríos",
                                                               "Rivas",
                                                               "Rivera",
                                                               "Rivero",
                                                               "Robledo",
                                                               "Robles",
                                                               "Rocha",
                                                               "Rodarte",
                                                               "Rodríguez",
                                                               "Rojas",
                                                               "Rojo",
                                                               "Roldán",
                                                               "Rolón",
                                                               "Romero",
                                                               "Romo",
                                                               "Roque",
                                                               "Rosado",
                                                               "Rosales",
                                                               "Rosario",
                                                               "Rosas",
                                                               "Roybal",
                                                               "Rubio",
                                                               "Ruelas",
                                                               "Ruíz",
                                                               "Saavedra",
                                                               "Sáenz",
                                                               "Saiz",
                                                               "Salas",
                                                               "Salazar",
                                                               "Salcedo",
                                                               "Salcido",
                                                               "Saldaña",
                                                               "Saldivar",
                                                               "Salgado",
                                                               "Salinas",
                                                               "Samaniego",
                                                               "Sanabria",
                                                               "Sánchez",
                                                               "Sandoval",
                                                               "Santacruz",
                                                               "Santana",
                                                               "Santiago",
                                                               "Santillán",
                                                               "Sarabia",
                                                               "Sauceda",
                                                               "Saucedo",
                                                               "Sedillo",
                                                               "Segovia",
                                                               "Segura",
                                                               "Sepúlveda",
                                                               "Serna",
                                                               "Serrano",
                                                               "Serrato",
                                                               "Sevilla",
                                                               "Sierra",
                                                               "Sisneros",
                                                               "Solano",
                                                               "Solís",
                                                               "Soliz",
                                                               "Solorio",
                                                               "Solorzano",
                                                               "Soria",
                                                               "Sosa",
                                                               "Sotelo",
                                                               "Soto",
                                                               "Suárez",
                                                               "Tafoya",
                                                               "Tamayo",
                                                               "Tamez",
                                                               "Tapia",
                                                               "Tejada",
                                                               "Tejeda",
                                                               "Téllez",
                                                               "Tello",
                                                               "Terán",
                                                               "Terrazas",
                                                               "Tijerina",
                                                               "Tirado",
                                                               "Toledo",
                                                               "Toro",
                                                               "Torres",
                                                               "Tórrez",
                                                               "Tovar",
                                                               "Trejo",
                                                               "Treviño",
                                                               "Trujillo",
                                                               "Ulibarri",
                                                               "Ulloa",
                                                               "Urbina",
                                                               "Ureña",
                                                               "Urías",
                                                               "Uribe",
                                                               "Urrutia",
                                                               "Vaca",
                                                               "Valadez",
                                                               "Valdés",
                                                               "Valdez",
                                                               "Valdivia",
                                                               "Valencia",
                                                               "Valentín",
                                                               "Valenzuela",
                                                               "Valladares",
                                                               "Valle",
                                                               "Vallejo",
                                                               "Valles",
                                                               "Valverde",
                                                               "Vanegas",
                                                               "Varela",
                                                               "Vargas",
                                                               "Vázquez",
                                                               "Vega",
                                                               "Vela",
                                                               "Velasco",
                                                               "Velásquez",
                                                               "Velázquez",
                                                               "Vélez",
                                                               "Véliz",
                                                               "Venegas",
                                                               "Vera",
                                                               "Verdugo",
                                                               "Verduzco",
                                                               "Vergara",
                                                               "Viera",
                                                               "Vigil",
                                                               "Villa",
                                                               "Villagómez",
                                                               "Villalobos",
                                                               "Villalpando",
                                                               "Villanueva",
                                                               "Villarreal",
                                                               "Villaseñor",
                                                               "Villegas",
                                                               "Yáñez",
                                                               "Ybarra",
                                                               "Zambrano",
                                                               "Zamora",
                                                               "Zamudio",
                                                               "Zapata",
                                                               "Zaragoza",
                                                               "Zarate",
                                                               "Zavala",
                                                               "Zayas",
                                                               "Zelaya",
                                                               "Zepeda",
                                                               "Zúñiga"});

const auto spanishMalePrefixes = std::to_array<std::string_view>({"Sr."});

const auto spanishFemalePrefixes = std::to_array<std::string_view>({"Sra.", "Sta."});

const NameFormats spanishNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames spanishPeopleNames{{spanishMaleFirstNames, spanishLastNames, spanishMalePrefixes, {}},
                                     {spanishFemaleFirstNames, spanishLastNames, spanishFemalePrefixes, {}},
                                     spanishNameFormats};

// Sweden

const auto swedishMaleFirstNames = std::to_array<std::string_view>({
    "Lars",      "Mikael",   "Anders",  "Johan",   "Erik",      "Per",         "Peter",   "Karl",     "Thomas",
    "Jan",       "Daniel",   "Fredrik", "Andreas", "Hans",      "Stefan",      "Mohamed", "Mats",     "Marcus",
    "Mattias",   "Magnus",   "Jonas",   "Oskar",   "Alexander", "Niklas",      "Bengt",   "Martin",   "Bo",
    "Nils",      "Viktor",   "Patrik",  "Björn",   "David",     "Leif",        "Filip",   "Henrik",   "Joakim",
    "Emil",      "Christer", "Ulf",     "Sven",    "Simon",     "Christoffer", "Anton",   "Gustav",   "Robert",
    "Christian", "William",  "Tommy",   "Kjell",   "Lucas",     "Rickard",     "Håkan",   "Adam",     "Göran",
    "Jakob",     "Jonathan", "Rolf",    "Lennart", "Robin",     "Sebastian",   "Elias",   "Tobias",   "John",
    "Stig",      "Axel",     "Linus",   "Oliver",  "Kent",      "Hugo",        "Isak",    "Roger",    "Jesper",
    "Claes",     "Albin",    "Ludvig",  "Jörgen",  "Rasmus",    "Max",         "Ali",     "Jimmy",    "Kenneth",
    "Joel",      "Gunnar",   "Dennis",  "Josef",   "Johnny",    "Olle",        "Liam",    "Leo",      "Olof",
    "Pontus",    "Åke",      "Kurt",    "Kevin",   "Samuel",    "Edvin",       "Gabriel", "Torbjörn", "Arvid",
    "Felix",
});

const auto swedishFemaleFirstNames = std::to_array<std::string_view>({
    "Anna",      "Eva",       "Maria",       "Karin",     "Sara",    "Kristina",  "Lena",         "Emma",
    "Kerstin",   "Marie",     "Malin",       "Ingrid",    "Jenny",   "Hanna",     "Linda",        "Annika",
    "Susanne",   "Elin",      "Monica",      "Birgitta",  "Sofia",   "Johanna",   "Inger",        "Carina",
    "Elisabeth", "Julia",     "Ulla",        "Katarina",  "Linnéa",  "Emelie",    "Ida",          "Helena",
    "Camilla",   "Åsa",       "Anette",      "Sandra",    "Gunilla", "Anita",     "Maja",         "Marianne",
    "Margareta", "Amanda",    "Anneli",      "Therese",   "Ann",     "Josefin",   "Cecilia",      "Jessica",
    "Helen",     "Lisa",      "Matilda",     "Caroline",  "Alice",   "Frida",     "Ulrika",       "Elsa",
    "Barbro",    "Siv",       "Rebecca",     "Madeleine", "Klara",   "Ebba",      "Sofie",        "Isabelle",
    "Agneta",    "Gun",       "Wilma",       "Lina",      "Berit",   "Pia",       "Ella",         "Ellen",
    "Yvonne",    "Louise",    "Astrid",      "Moa",       "Olivia",  "Nathalie",  "Mona",         "Agnes",
    "Emilia",    "Alexandra", "Alva",        "Britt",     "Erika",   "Viktoria",  "Ann-Christin", "Felicia",
    "Ann-Marie", "Sonja",     "Britt-Marie", "Pernilla",  "Lovisa",  "Charlotte", "Linn",         "Gunnel",
    "Nina",      "Mikaela",   "Karolina",    "Lisbeth",
});

const auto swedishLastNames = std::to_array<std::string_view>({
    "Andersson",   "Johansson",  "Karlsson",   "Nilsson",    "Eriksson",  "Larsson",   "Olsson",    "Persson",
    "Svensson",    "Gustafsson", "Pettersson", "Jonsson",    "Jansson",   "Hansson",   "Bengtsson", "Jönsson",
    "Lindberg",    "Jakobsson",  "Magnusson",  "Olofsson",   "Lindström", "Lindqvist", "Lindgren",  "Berg",
    "Axelsson",    "Bergström",  "Lundberg",   "Lind",       "Lundgren",  "Lundqvist", "Mattsson",  "Berglund",
    "Fredriksson", "Sandberg",   "Henriksson", "Forsberg",   "Sjöberg",   "Ali",       "Wallin",    "Mohamed",
    "Engström",    "Eklund",     "Danielsson", "Lundin",     "Håkansson", "Björk",     "Bergman",   "Gunnarsson",
    "Holm",        "Wikström",   "Samuelsson", "Isaksson",   "Fransson",  "Bergqvist", "Nyström",   "Holmberg",
    "Arvidsson",   "Löfgren",    "Söderberg",  "Nyberg",     "Blomqvist", "Claesson",  "Nordström", "Ahmed",
    "Mårtensson",  "Lundström",  "Hassan",     "Viklund",    "Björklund", "Eliasson",  "Pålsson",   "Berggren",
    "Sandström",   "Lund",       "Nordin",     "Ström",      "Åberg",     "Falk",      "Ekström",   "Hermansson",
    "Holmgren",    "Hellström",  "Dahlberg",   "Hedlund",    "Sundberg",  "Sjögren",   "Ek",        "Blom",
    "Abrahamsson", "Martinsson", "Öberg",      "Andreasson", "Strömberg", "Månsson",   "Hansen",    "Åkesson",
    "Dahl",        "Lindholm",   "Norberg",    "Holmqvist",
});

const auto swedishPrefixes = std::to_array<std::string_view>({"Dr.", "Prof.", "PhD."});

const NameFormats swedishNameFormats{{{"{firstName} {lastName}", 10}, {"{prefix} {firstName} {lastName}", 1}}};

const PeopleNames swedishPeopleNames{{swedishMaleFirstNames, swedishLastNames, swedishPrefixes, {}},
                                     {swedishFemaleFirstNames, swedishLastNames, swedishPrefixes, {}},
                                     swedishNameFormats};

// Switzerland

const auto swissMaleFirstNames = std::to_array<std::string_view>({
    "Alfons",       "Alfred",      "Alois",     "André",        "Andreas",      "Angelo",      "Antoine",
    "Anton",        "Antonio",     "Armin",     "Arnold",       "Arthur",       "Beat",        "Bernard",
    "Bernhard",     "Bruno",       "Carlo",     "Charles",      "Christian",    "Christoph",   "Christophe",
    "Claude",       "Claudio",     "Daniel",    "Daniele",      "David",        "Denis",       "Didier",
    "Dieter",       "Dominik",     "Dominique", "Edgar",        "Eduard",       "Edwin",       "Emil",
    "Enrico",       "Eric",        "Erich",     "Ernst",        "Erwin",        "Eugen",       "Felix",
    "Ferdinand",    "Francesco",   "Francis",   "Franco",       "François",     "Frank",       "Franz",
    "Frédéric",     "Fredy",       "Fridolin",  "Friedrich",    "Fritz",        "Gabriel",     "Georg",
    "Georges",      "Gérald",      "Gérard",    "Gerhard",      "Gianni",       "Gilbert",     "Giorgio",
    "Giovanni",     "Giuseppe",    "Gottfried", "Guido",        "Guy",          "Hans",        "Hans-Peter",
    "Hans-Rudolf",  "Hans-Ulrich", "Hansjörg",  "Hanspeter",    "Hansruedi",    "Hansueli",    "Harry",
    "Heinrich",     "Heinz",       "Helmut",    "Henri",        "Herbert",      "Hermann",     "Hubert",
    "Hugo",         "Jacques",     "Jakob",     "Jan",          "Jean-Claude",  "Jean-Daniel", "Jean-François",
    "Jean-Jacques", "Jean-Louis",  "Jean-Luc",  "Jean-Marc",    "Jean-Marie",   "Jean-Paul",   "Jean-Pierre",
    "Johann",       "Johannes",    "John",      "Jörg",         "Josef",        "Joseph",      "Jürg",
    "Karl",         "Klaus",       "Konrad",    "Kurt",         "Laurent",      "Leo",         "Louis",
    "Luca",         "Luigi",       "Lukas",     "Manfred",      "Manuel",       "Marc",        "Marcel",
    "Marco",        "Mario",       "Markus",    "Martin",       "Massimo",      "Matthias",    "Maurice",
    "Max",          "Michael",     "Michel",    "Nicolas",      "Niklaus",      "Norbert",     "Olivier",
    "Oskar",        "Otto",        "Paolo",     "Pascal",       "Patrick",      "Paul",        "Peter",
    "Philipp",      "Philippe",    "Pierre",    "Pierre-Alain", "Pierre-André", "Pius",        "Rainer",
    "Raymond",      "Reinhard",    "Remo",      "Renato",       "Rene",         "René",        "Reto",
    "Richard",      "Rudolf",      "Ruedi",     "Samuel",       "Sandro",       "Serge",       "Silvio",
    "Simon",        "Stefan",      "Stephan",   "Stéphane",     "Theo",         "Theodor",     "Thomas",
    "Ueli",         "Ulrich",      "Urs",       "Victor",       "Viktor",       "Walter",      "Werner",
    "Willi",        "Willy",       "Wolfgang",  "Yves",
});

const auto swissFemaleFirstNames = std::to_array<std::string_view>({
    "Alice",     "Andrea",     "Angela",    "Anita",     "Anna",       "Anne",      "Anne-Marie", "Annemarie",
    "Astrid",    "Barbara",    "Beatrice",  "Beatrix",   "Bernadette", "Bettina",   "Brigitta",   "Brigitte",
    "Carmen",    "Caroline",   "Catherine", "Chantal",   "Charlotte",  "Christa",   "Christiane", "Christina",
    "Christine", "Claire",     "Claudia",   "Claudine",  "Corinne",    "Cornelia",  "Daniela",    "Danielle",
    "Denise",    "Dominique",  "Dora",      "Doris",     "Edith",      "Eliane",    "Elisabeth",  "Elsa",
    "Elsbeth",   "Erica",      "Erika",     "Esther",    "Eva",        "Eveline",   "Evelyne",    "Fabienne",
    "Florence",  "Françoise",  "Franziska", "Gabriela",  "Gabrielle",  "Gertrud",   "Gisela",     "Hanna",
    "Heidi",     "Helen",      "Helena",    "Helene",    "Hildegard",  "Ingrid",    "Irene",      "Iris",
    "Isabelle",  "Jacqueline", "Janine",    "Jean",      "Johanna",    "Jolanda",   "Josette",    "Josiane",
    "Judith",    "Julia",      "Karin",     "Katharina", "Laura",      "Laurence",  "Liliane",    "Liselotte",
    "Lydia",     "Madeleine",  "Maja",      "Manuela",   "Margrit",    "Margrith",  "Maria",      "Marianne",
    "Marion",    "Marlène",    "Marlies",   "Marlis",    "Martha",     "Martina",   "Martine",    "Maya",
    "Michele",   "Micheline",  "Monica",    "Monika",    "Monique",    "Myriam",    "Nadia",      "Nadja",
    "Nathalie",  "Nelly",      "Nicole",    "Paola",     "Patricia",   "Petra",     "Pia",        "Priska",
    "Regina",    "Regula",     "Renata",    "Renate",    "Ruth",       "Sabine",    "Sandra",     "Silvia",
    "Simone",    "Sonia",      "Sonja",     "Stéphanie", "Susanna",    "Susanne",   "Suzanne",    "Sylvia",
    "Sylvie",    "Therese",    "Toni",      "Ursula",    "Verena",     "Véronique", "Vreni",      "Yolande",
    "Yvette",    "Yvonne",
});

const auto swissLastNames = std::to_array<std::string_view>({
    "Ackermann",    "Aebi",       "Albrecht",   "Ammann",     "Amrein",      "Arnold",       "Bachmann",
    "Bader",        "Bär",        "Bättig",     "Bauer",      "Baumann",     "Baumgartner",  "Baur",
    "Beck",         "Benz",       "Berger",     "Bernasconi", "Betschart",   "Bianchi",      "Bieri",
    "Blaser",       "Blum",       "Bolliger",   "Bosshard",   "Braun",       "Brun",         "Brunner",
    "Bucher",       "Bühler",     "Bühlmann",   "Burri",      "Christen",    "Egger",        "Egli",
    "Eichenberger", "Erni",       "Ernst",      "Eugster",    "Fankhauser",  "Favre",        "Fehr",
    "Felber",       "Felder",     "Ferrari",    "Fischer",    "Flückiger",   "Forster",      "Frei",
    "Frey",         "Frick",      "Friedli",    "Fuchs",      "Furrer",      "Gasser",       "Geiger",
    "Gerber",       "Gfeller",    "Giger",      "Gloor",      "Graf",        "Grob",         "Gross",
    "Gut",          "Haas",       "Häfliger",   "Hafner",     "Hartmann",    "Hasler",       "Hauser",
    "Hermann",      "Herzog",     "Hess",       "Hirt",       "Hodel",       "Hofer",        "Hoffmann",
    "Hofmann",      "Hofstetter", "Hotz",       "Huber",      "Hug",         "Hunziker",     "Hürlimann",
    "Imhof",        "Isler",      "Iten",       "Jäggi",      "Jenni",       "Jost",         "Kägi",
    "Kaiser",       "Kälin",      "Käser",      "Kaufmann",   "Keller",      "Kern",         "Kessler",
    "Knecht",       "Koch",       "Kohler",     "Kuhn",       "Küng",        "Kunz",         "Lang",
    "Lanz",         "Lehmann",    "Leu",        "Leunberger", "Lüscher",     "Lustenberger", "Lüthi",
    "Lutz",         "Mäder",      "Maier",      "Marti",      "Martin",      "Maurer",       "Mayer",
    "Meier",        "Meili",      "Meister",    "Merz",       "Mettler",     "Meyer",        "Michel",
    "Moser",        "Müller",     "Näf",        "Ott",        "Peter",       "Pfister",      "Portmann",
    "Probst",       "Rey",        "Ritter",     "Roos",       "Roth",        "Rüegg",        "Schäfer",
    "Schaller",     "Schär",      "Schärer",    "Schaub",     "Scheidegger", "Schenk",       "Scherrer",
    "Schlatter",    "Schmid",     "Schmidt",    "Schneider",  "Schnyder",    "Schoch",       "Schuler",
    "Schumacher",   "Schürch",    "Schwab",     "Schwarz",    "Schweizer",   "Seiler",       "Senn",
    "Sidler",       "Siegrist",   "Sigrist",    "Spörri",     "Stadelmann",  "Stalder",      "Staub",
    "Stauffer",     "Steffen",    "Steiger",    "Steiner",    "Steinmann",   "Stettler",     "Stocker",
    "Stöckli",      "Stucki",     "Studer",     "Stutz",      "Suter",       "Sutter",       "Tanner",
    "Thommen",      "Tobler",     "Vogel",      "Vogt",       "Wagner",      "Walder",       "Walter",
    "Weber",        "Wegmann",    "Wehrli",     "Weibel",     "Wenger",      "Wettstein",    "Widmer",
    "Winkler",      "Wirth",      "Wirz",       "Wolf",       "Wüthrich",    "Wyss",         "Zbinden",
    "Zehnder",      "Ziegler",    "Zimmermann", "Zingg",      "Zollinger",   "Zürcher",
});

const auto swissMalePrefixes = std::to_array<std::string_view>({"Herr", "Dr.", "Prof. Dr."});

const auto swissFemalePrefixes = std::to_array<std::string_view>({"Frau", "Dr.", "Prof. Dr."});

const NameFormats swissNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 9}}};

const PeopleNames swissPeopleNames{{swissMaleFirstNames, swissLastNames, swissMalePrefixes, {}},
                                   {swissFemaleFirstNames, swissLastNames, swissFemalePrefixes, {}},
                                   swissNameFormats};

// Syria

const auto syrianMaleFirstNames = std::to_array<std::string_view>({
    "Mohamed",  "Ahmed",    "Abdel",    "Hassan",   "Ali",      "Saleh",      "Mahmoud",  "Abou",        "Ibrahim",
    "Hussein",  "Khaled",   "Fayez",    "Sheikh",   "Mostafa",  "Abdo",       "Anwar",    "Hilal",       "Yousef",
    "Ghazi",    "Omar",     "Hama",     "Kamel",    "Zuhair",   "Khalil",     "Suleiman", "Zaher",       "Ismail",
    "Adnan",    "Zain",     "Yasser",   "Ammar",    "Farhan",   "Nabil",      "Raed",     "Yahya",       "Mousa",
    "Samer",    "Sherif",   "Waleed",   "Jamal",    "Bassam",   "Rabie",      "Ayman",    "Khader",      "Faisal",
    "Emad",     "Hossam",   "Mohsen",   "Samih",    "Taha",     "Fadi",       "Abdullah", "Nader",       "Fahd",
    "Samir",    "Hamad",    "Osama",    "Mahdi",    "Ghassan",  "Firas",      "Anas",     "Fatih",       "Amer",
    "Ziad",     "Saeed",    "Zakaria",  "Ramadan",  "Yassin",   "Qasim",      "Bashir",   "Saad",        "Nasser",
    "Nazih",    "Marwan",   "Riad",     "Malik",    "Tarek",    "Naji",       "Hamid",    "Jamil",       "Ramy",
    "Sultan",   "Adel",     "Radwan",   "Asaad",    "Haytham",  "Badr",       "Akram",    "Mazen",       "Haidar",
    "Saif",     "Jaafar",   "Salah",    "Kamal",    "Iyad",     "Shady",      "Wael",     "Hamza",       "Bahri",
    "Osman",    "Majed",    "Fares",    "Rashid",   "Karam",    "Louay",      "Talal",    "Mounir",      "Salem",
    "Waseem",   "Essam",    "Nizar",    "Hamada",   "Maher",    "Fouad",      "Younis",   "Aziz",        "Amin",
    "Mehdi",    "George",   "Elias",    "Sami",     "Jan",      "Fadel",      "Salim",    "Hamed",       "Bilal",
    "Salam",    "Salman",   "Hisham",   "Mansour",  "Abbas",    "Haj",        "Shah",     "Hani",        "Ahmad",
    "Tawfiq",   "Fawzi",    "Jalal",    "Nadeem",   "Sayed",    "Farouk",     "Hatem",    "Mamoun",      "Abdelkader",
    "Azad",     "Suhail",   "Abu",      "Farid",    "Taleb",    "Sher",       "Naeem",    "Safwan",      "Hadi",
    "Milad",    "Karim",    "Jaber",    "Najih",    "Adal",     "Dia",        "Khamis",   "Zaki",        "Adham",
    "Habib",    "Hammad",   "Asher",    "Tayeb",    "Matar",    "Taher",      "Wahid",    "Haji",        "Amjad",
    "Mohammed", "Amar",     "Dimas",    "Tamer",    "Hammadi",  "Khalifa",    "Atta",     "Mohammad",    "Ben",
    "Atef",     "Faiz",     "Murad",    "Sadiq",    "Zaid",     "Noman",      "Mohamad",  "Ayoub",       "Amir",
    "Badi",     "Nazir",    "Baba",     "Michel",   "Idris",    "Rashad",     "Radi",     "Yacoub",      "Rafiq",
    "Fathi",    "Mubarak",  "Daniel",   "Rami",     "Massoud",  "Aqeel",      "Makki",    "Joseph",      "Bakr",
    "Muhammed", "Shafiq",   "Asif",     "Antar",    "Mikhail",  "Majid",      "Luqman",   "Mustafa",     "Daoud",
    "Nabeel",   "Ashraf",   "Anton",    "Raouf",    "Asad",     "Sam",        "Safi",     "Mazhar",      "Akka",
    "Amr",      "Wahib",    "Gamal",    "Arslan",   "Fahim",    "Antoine",    "Saber",    "Anis",        "Hasan",
    "Kamil",    "Farhad",   "Abdalla",  "Rehan",    "Makram",   "Afif",       "Sayah",    "Muhammad",    "Abdelghani",
    "Hany",     "Basel",    "Othman",   "Rabah",    "Lawrence", "Latif",      "Raymond",  "Irfan",       "Emile",
    "Hussam",   "Abed",     "Shadi",    "Foad",     "William",  "Abdelhakim", "Jawad",    "Siraj",       "Simon",
    "Mesbah",   "Nael",     "Lotfi",    "Samy",     "Abdi",     "Angel",      "Sidi",     "Bada",        "Sasa",
    "Imad",     "Scott",    "Nazar",    "Rai",      "Badri",    "Kader",      "Yaser",    "Morris",      "Ram",
    "Shoaib",   "Sufian",   "Arman",    "Joel",     "Mahfoud",  "Seydou",     "Nazim",    "Abdulrahman", "Issam",
    "Falak",    "Abdul",    "Ayad",     "Rafi",     "Dani",     "Mahy",       "Manea",    "Ishaq",       "Nabi",
    "Sabri",    "Haitham",  "Muzaffar", "Taj",      "Zahir",    "Johnny",     "Kasem",    "Mokhtar",     "Fatah",
    "Nassar",   "Jad",      "Mahmood",  "Rahal",    "Malek",    "Hanif",      "Fattah",   "Alex",        "Anass",
    "Shamil",   "Iftikhar", "Mhamad",   "Mujahid",  "Marcel",   "Iqbal",      "Zubair",   "Nayel",       "Marzouk",
    "Yvon",     "Rahim",    "Leon",     "Haroun",   "Mouloud",  "Talha",      "Halim",    "Youssef",     "Fikret",
    "Edmond",   "Tareq",    "Walid",    "Antonius", "Nizam",    "Aslan",      "John",     "Aday",        "Faraz",
    "Rasheed",  "Akash",    "Ibo",      "Wassim",   "Muad",     "Salih",      "Mikael",   "Husam",
});

const auto syrianFemaleFirstNames = std::to_array<std::string_view>({
    "Nour",    "Fatima",    "Dara",       "Mariam",    "Darya",     "Amara",   "Mari",      "Jihad",      "Hikmat",
    "Sabah",   "Hanan",     "Nihad",      "Khadija",   "Rana",      "Mona",    "Zainab",    "Alaa",       "Heba",
    "Hala",    "Rama",      "Hoda",       "Nasreen",   "Wissam",    "Sarah",   "Manal",     "Reda",       "Rasha",
    "Iman",    "Zahra",     "Nada",       "Jamila",    "Wafa",      "Najah",   "Samira",    "Aya",        "Maysaa",
    "Amal",    "Yara",      "Amina",      "Marwa",     "Laila",     "Halima",  "Fattoum",   "Ayesha",     "Aisha",
    "Lina",    "Samar",     "Joan",       "Bayan",     "Safa",      "Haya",    "Ela",       "Farah",      "Alia",
    "Hana",    "Faten",     "Walaa",      "Hind",      "Rania",     "Souad",   "Duaa",      "Dalal",      "Babila",
    "Sana",    "Yasmin",    "Ghada",      "Nawar",     "Siham",     "Nadia",   "Bushra",    "Fadia",      "Rima",
    "Maha",    "Ihsan",     "Nidal",      "Sahar",     "Georgette", "Najwa",   "Ibtisam",   "Amna",       "Batoul",
    "Wiam",    "Nawal",     "Maya",       "Dima",      "Manar",     "Dana",    "Razan",     "Sally",      "Nahed",
    "Salwa",   "Hayam",     "Diana",      "Asmaa",     "Zara",      "Sawsan",  "Rose",      "Susan",      "Dania",
    "Elian",   "Catherine", "Fadwa",      "Arwa",      "Kholoud",   "Salama",  "Badia",     "Fahima",     "Lana",
    "Fayrouz", "Mary",      "Inas",       "Kawthar",   "Beda",      "Safia",   "Randa",     "Nahla",      "Basma",
    "Hasna",   "Aziza",     "Samah",      "Anara",     "Sabine",    "Anam",    "Maria",     "Saba",       "Rehab",
    "Meliha",  "Lubna",     "Nora",       "Badra",     "Jade",      "Noha",    "Ahlam",     "Fatim",      "Karen",
    "Khadra",  "Rajaa",     "Asma",       "Hanna",     "Aida",      "Medea",   "Hayat",     "Elham",      "Azra",
    "Ala",     "Maram",     "Sumaya",     "Jihan",     "Zeina",     "Amira",   "Nadine",    "Naima",      "Hina",
    "Samia",   "Rahma",     "Sima",       "Ghazala",   "Ramia",     "Hajar",   "Rayan",     "Hajira",     "Ana",
    "Nasra",   "Nahida",    "Nasim",      "Afaf",      "Najma",     "Salma",   "Doreen",    "Shaheen",    "Lamis",
    "Sare",    "Raja",      "Elia",       "Hamida",    "Wesal",     "Magda",   "Fawzia",    "Houria",     "Mai",
    "Latifa",  "Rokaya",    "Saadia",     "Farida",    "Ayada",     "Lamia",   "Bahia",     "Linda",      "Samiha",
    "Shadia",  "Nariman",   "Nicola",     "Nayla",     "Faiza",     "Badria",  "Najat",     "Mirna",      "Mounira",
    "Judy",    "Suhaila",   "Hawa",       "Josephine", "Dale",      "Mara",    "Afra",      "Asmahan",    "Awatif",
    "Fayza",   "Medina",    "Afnan",      "Nicole",    "Hajra",     "Janet",   "Donia",     "Nozha",      "Najla",
    "Nazira",  "Elly",      "Antoinette", "Dany",      "Elaine",    "Ria",     "Yousra",    "Saada",      "Rahila",
    "Asia",    "Fathia",    "Hassiba",    "Mahdia",    "Nabila",    "Sakina",  "Khawla",    "Azza",       "Wahiba",
    "Madiha",  "Lara",      "Rabia",      "Rita",      "Dina",      "Rabab",   "Radia",     "Sonia",      "Sasha",
    "Elena",   "Malak",     "Bahija",     "Sabria",    "Mia",       "Omaima",  "Madeleine", "Karima",     "Marilyn",
    "Yola",    "Norma",     "Ghalia",     "Raida",     "Hiba",      "Dalia",   "Malika",    "Katia",      "Julia",
    "Nasima",  "Shahnaz",   "Sanaa",      "Salima",    "Abla",      "Sandra",  "Fatna",     "Jacqueline", "Banna",
    "Elissa",  "Lydia",     "Nahia",      "Dahab",     "Claude",    "Janan",   "Zahia",     "Therese",    "Rene",
    "Sara",    "Noor",      "Amia",       "Eveline",   "Christine", "Najia",   "Mimouna",   "Hadia",      "Anisa",
    "Yohanna", "Shahida",   "Hanifa",     "Violet",    "Juliet",    "Sarra",   "Melia",     "Ziara",      "Helen",
    "Ayat",    "Rani",      "Mariana",    "Lorraine",  "Anaya",     "Juliana", "Silva",     "Tamara",     "Na",
    "Nelly",   "Rahima",    "Lilian",     "Ikram",     "Hakima",    "Islam",   "Huda",      "Noura",      "Georgina",
    "Barbara", "Rachida",   "Samra",      "Helena",    "Amani",     "Nargis",  "Adila",     "Jana",       "Rawan",
    "Sama",    "Sultana",   "Tania",      "Liza",      "Lian",      "Aman",    "Olga",      "Soria",      "Amra",
    "Nafisa",  "Zahida",    "Najiha",     "Amelia",    "Caroline",  "Nirmin",  "Zina",      "Suzan",      "May",
    "Majida",  "Yana",      "Mouna",      "Jocelyn",   "Moumna",    "Eman",    "Rabha",     "Thania",
});

const auto syrianLastNames = std::to_array<std::string_view>({
    "Khaled",      "Mohamed",    "Al numan",  "Ahmed",      "Ali",        "Hussein",      "Hassan",    "Ibrahim",
    "Mahmoud",     "Samaan",     "Alzuhur",   "Saleh",      "Alththania", "Allah",        "Mostafa",   "Suleiman",
    "Yousef",      "Aldaman",    "Khalil",    "Khalaf",     "El din",     "Ismail",       "Eisaa",     "Hamoud",
    "Abdel",       "Saeed",      "Mousa",     "Omar",       "Jassim",     "Sheikh",       "Awad",      "Deeb",
    "Al-karim",    "Azza",       "Abdo",      "Qasim",      "Elkader",    "Osman",        "Abbas",     "Badr",
    "Al-razzaq",   "El hajj",    "Khader",    "Abdullah",   "Hamad",      "El abd",       "Yassin",    "Al jassim",
    "Aboud",       "Shamalieh",  "Al aziz",   "Haj",        "Ramadan",    "Hamid",        "Asaad",     "El arab",
    "Obaid",       "Jerash",     "Khalif",    "Nasser",     "Salim",      "Shams",        "Al hamoud", "Fares",
    "Rashid",      "Khair",      "Juma",      "Hamdan",     "Eid",        "Salem",        "Darwish",   "Salloum",
    "Adnan",       "Younis",     "Sobhy",     "Mansour",    "Jamil",      "Habib",        "Yujad",     "Nayef",
    "Shehadeh",    "Salman",     "El latif",  "Amin",       "Taha",       "Mari",         "Al ward",   "Salama",
    "Hilal",       "Alyousf",    "Hamada",    "Al mousa",   "Turki",      "Duyub",        "Jumah",     "Khadour",
    "Diab",        "Al awamid",  "Haji",      "Sharqia",    "Al omar",    "Yahya",        "Farhan",    "Mohsen",
    "Shaaban",     "Al khatib",  "Hamed",     "Fahd",       "Hamidi",     "Masry",        "Dawood",    "Saad",
    "Bashir",      "Kamel",      "Al-majeed", "Al sheikh",  "Matar",      "El helou",     "Haddad",    "El sharqeya",
    "Mahfoud",     "El hadi",    "Al ain",    "El wahab",   "Fayez",      "Haidar",       "Zidan",     "Hassoun",
    "Hamdou",      "Hamza",      "Al hamad",  "Omran",      "Tawfiq",     "Khalifa",      "Ayed",      "Sah",
    "Qasarneh",    "Al fattah",  "Al thina",  "Daoud",      "Islam",      "Hammadi",      "Fadel",     "Sherif",
    "Al rahim",    "Shaheen",    "Al haji",   "Arab",       "Barakat",    "Hamo",         "Ghanim",    "Maarouf",
    "Idris",       "El salam",   "Sameh",     "Al humaidi", "Aladol",     "Aziz",         "Muslim",    "Ragab",
    "Mukhlaf",     "Attieh",     "Fawaz",     "Al dahour",  "Alawi",      "Jamal",        "Jan",       "Artouz",
    "Adel",        "El ghany",   "Halab",     "Refaat",     "Oued",       "Fayad",        "Waleed",    "Kamal",
    "Assaf",       "Al aboud",   "Jiblah",    "Al aliawi",  "Aswad",      "Zakaria",      "Mehrez",    "Al mashrafah",
    "Saber",       "Al-khader",  "Faraj",     "Nawaf",      "Faisal",     "Khalouf",      "Sultan",    "Al othman",
    "Zuhair",      "Aref",       "Elias",     "Mariam",     "Mamdouh",    "Bedour",       "Radwan",    "Hanna",
    "Al-abdullah", "Ghazi",      "Asas",      "Sattouf",    "Zaky",       "El sayed",     "Hamdy",     "Alloush",
    "Adeeb",       "Al menem",   "Al-zayt",   "Raslan",     "Odeh",       "Asber",        "Yabous",    "Abyad",
    "Fawzi",       "Al ismail",  "Allawi",    "Mounir",     "Jandal",     "Abou",         "Taher",     "Nour",
    "Najm",        "Al najjar",  "Naguib",    "Tayseer",    "Saqr",       "Qabaliyah",    "Muhanna",   "Al-khalif",
    "Bakr",        "Ezz",        "Nasan",     "Hashem",     "Alakhas",    "Jaber",        "Nouri",     "Al-mansour",
    "Ayoub",       "Al turkman", "Hassou",    "El hamdou",  "Nader",      "Shahoud",      "Fouad",     "Abdul rahman",
    "Hammad",      "Taleb",      "Alo",       "Al jabbar",  "Samir",      "Qadour",       "Salah",     "Al hilal",
    "Fendi",       "Mihoub",     "Reda",      "Melhem",     "Nawos",      "Al tina",      "Bakri",     "Al jalil",
    "Dahir",       "Damascus",   "Bouzan",    "Fattoum",    "Bakour",     "Sue",          "Bilal",     "El hadid",
    "Al jamaa",    "Zaher",      "Amoun",     "Jadoua",     "Marwan",     "Jadaan",       "Qubailah",  "Gamalan",
    "Qalq",        "Daham",      "Hajj",      "Al dahir",   "Kaeika",     "Aqeel",        "Tahta",     "Karim",
    "Murad",       "El shamaly", "Kurdi",     "Aklah",      "Shawah",     "Abdel rahman", "Nasr",      "Rabie",
    "Rustom",      "Barhoum",    "Al hamada", "Assi",       "Al qasim",   "El halaby",    "Khoury",    "Al-kurdi",
    "Hama",        "Maala",      "Saud",      "Khatib",     "El fares",   "Abdul karim",  "Ela",       "Sharqi",
    "Akram",       "Farah",      "Rahil",     "Riad",       "Al hayy",    "Al assaf",     "Shehab",    "Hatem",
    "Sakour",      "Heloum",     "Rateb",     "Ghaleb",     "Jneid",      "Nazir",        "Nemeh",     "Al-basset",
    "Badawi",      "Halaboun",   "Moheisen",  "Hegazy",     "Al sbaa",    "Tamer",        "Nazzal",    "Wassouf",
    "Al-hakim",    "El said",    "Khamis",    "Kabir",      "Habash",     "Maahed",       "Madameh",   "Gerges",
    "Oly",         "Abdelkader", "Al awad",   "Al safa",    "Sayah",      "Trama",
});

const NameFormats syrianNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames syrianPeopleNames{{syrianMaleFirstNames, syrianLastNames, {}, {}},
                                    {syrianFemaleFirstNames, syrianLastNames, {}, {}},
                                    syrianNameFormats};

// Turkey

const auto turkishMaleFirstNames = std::to_array<std::string_view>({
    "Ahmet",  "Mehmet", "Mustafa", "Ali",    "Emre",   "Burak", "Can",   "Cem",    "Eren",   "Onur",  "Kaan",
    "Serkan", "Hakan",  "Yusuf",   "Okan",   "Barış",  "Tolga", "Kerem", "Deniz",  "Oğuz",   "Umut",  "Gökhan",
    "İsmail", "Metin",  "Orhan",   "İlker",  "Levent", "Selim", "Berk",  "Uğur",   "Murat",  "Tarık", "Cihan",
    "Fatih",  "Furkan", "Yiğit",   "Tarkan", "Volkan", "Mert",  "Alp",   "Taylan", "Burhan", "Ege",   "Koray",
});

const auto turkishFemaleFirstNames = std::to_array<std::string_view>({
    "Ayşe",  "Fatma", "Zeynep",  "Emine", "Merve", "Elif",  "Melis",  "Esra",  "İrem",  "Ebru",   "Selma",
    "Leyla", "Selin", "Gamze",   "Gizem", "Ceren", "Deniz", "Aslı",   "Nazlı", "Pınar", "İlayda", "Dilara",
    "Büşra", "Rabia", "Yasemin", "Gül",   "Nur",   "Melek", "Sude",   "Şeyma", "Beren", "Eylül",  "İpek",
    "Hande", "Cansu", "Özge",    "Hilal", "Bilge", "Serra", "İlknur", "Cemre", "Tuğba", "Yelda",
});

const auto turkishLastNames = std::to_array<std::string_view>({
    "Yılmaz",      "Demir",    "Kaya",     "Çelik",    "Yıldız",  "Türk",     "Güneş",    "Ergin",  "Arslan",
    "Doğan",       "Kurt",     "Şahin",    "Güler",    "Çetin",   "Aydın",    "Yalçın",   "Kaplan", "Aksoy",
    "Taş",         "Koç",      "Erdoğan",  "Yılmazer", "Bulut",   "Acar",     "Uçar",     "Kılıç",  "Karahan",
    "Tekin",       "Soydan",   "Çalışkan", "Öztürk",   "Yalın",   "Kara",     "Özkan",    "Akın",   "Aktaş",
    "Büyükcengiz", "Okan",     "Şimşek",   "Keskin",   "Kayabaş", "Kayabaşı", "Atalay",   "Ünal",   "Demirci",
    "Yaman",       "Gündoğdu", "Çevik",    "Durmaz",   "Aslan",   "Güzel",    "Kahraman",
});

const auto turkishMalePrefixes = std::to_array<std::string_view>({"Bay", "Dr.", "Prof. Dr."});

const auto turkishFemalePrefixes = std::to_array<std::string_view>({"Bayan", "Dr.", "Prof. Dr."});

const NameFormats turkish_personNameFormats{{{"{prefix} {firstName} {lastName}", 1}, {"{firstName} {lastName}", 8}}};

const PeopleNames turkishPeopleNames{{turkishMaleFirstNames, turkishLastNames, turkishMalePrefixes, {}},
                                     {turkishFemaleFirstNames, turkishLastNames, turkishFemalePrefixes, {}},
                                     turkish_personNameFormats};

// Ukraine

const auto ukrainianMaleFirstNames = std::to_array<std::string_view>({
    "Августин",  "Аврелій",     "Адам",       "Адріян",    "Азарій",    "Алевтин",  "Альберт",   "Анастас",
    "Анастасій", "Анатолій",    "Андрій",     "Антін",     "Антон",     "Антоній",  "Аркадій",   "Арсен",
    "Арсеній",   "Артем",       "Архип",      "Аскольд",   "Афанасій",  "Біломир",  "Білослав",  "Богдан",
    "Божемир",   "Божен",       "Болеслав",   "Боримир",   "Боримисл",  "Борис",    "Борислав",  "Братимир",
    "Братислав", "Братомил",    "Братослав",  "Брячислав", "Будимир",   "Буйтур",   "Буревіст",  "В’ячеслав",
    "Вадим",     "Валентин",    "Валерій",    "Василь",    "Велемир",   "Віктор",   "Віталій",   "Влад",
    "Владислав", "Володимир",   "Володислав", "Всевлад",   "Всеволод",  "Всеслав",  "Гаврило",   "Гарнослав",
    "Геннадій",  "Георгій",     "Герасим",    "Гліб",      "Гнат",      "Гордій",   "Горимир",   "Горислав",
    "Градимир",  "Григорій",    "Далемир",    "Данило",    "Дарій",     "Даромир",  "Денис",     "Дмитро",
    "Добромир",  "Добромисл",   "Доброслав",  "Євген",     "Єремій",    "Захар",    "Захарій",   "Зборислав",
    "Звенигор",  "Звенимир",    "Звенислав",  "Земислав",  "Зеновій",   "Зиновій",  "Злат",      "Златомир",
    "Зоремир",   "Зореслав",    "Зорян",      "Іван",      "Ігор",      "Ізяслав",  "Ілля",      "Кий",
    "Корнелій",  "Корнилій",    "Корнило",    "Корній",    "Костянтин", "Кузьма",   "Лаврентій", "Лаврін",
    "Лад",       "Ладислав",    "Ладо",       "Ладомир",   "Левко",     "Листвич",  "Лук’ян",    "Любодар",
    "Любозар",   "Любомир",     "Макар",      "Максим",    "Мар’ян",    "Маркіян",  "Марко",     "Матвій",
    "Мечислав",  "Микита",      "Микола",     "Мирон",     "Мирослав",  "Михайло",  "Мстислав",  "Мусій",
    "Назар",     "Назарій",     "Натан",      "Немир",     "Нестор",    "Олег",     "Олександр", "Олексій",
    "Олелько",   "Олесь",       "Омелян",     "Орест",     "Орхип",     "Остап",    "Охрім",     "Павло",
    "Панас",     "Пантелеймон", "Петро",      "Пилип",     "Подолян",   "Потап",    "Радим",     "Радимир",
    "Ратибор",   "Ратимир",     "Родіон",     "Родослав",  "Роксолан",  "Роман",    "Ростислав", "Руслан",
    "Святополк", "Святослав",   "Семибор",    "Сергій",    "Синьоок",   "Славолюб", "Славомир",  "Славута",
    "Сніжан",    "Сологуб",     "Станіслав",  "Степан",    "Стефаній",  "Стожар",   "Тарас",     "Тиміш",
    "Тимофій",   "Тихон",       "Тур",        "Устим",     "Хвалимир",  "Хорив",    "Чорнота",   "Щастислав",
    "Щек",       "Юліан",       "Юрій",       "Юхим",      "Ян",        "Ярема",    "Яровид",    "Яромил",
    "Яромир",    "Ярополк",     "Ярослав",
});

const auto ukrainianFemaleFirstNames = std::to_array<std::string_view>({
    "Аврелія",    "Аврора",     "Агапія",     "Агата",       "Агафія",     "Агнеса",   "Агнія",     "Агрипина",
    "Ада",        "Аделаїда",   "Аделіна",    "Адріана",     "Азалія",     "Алевтина", "Аліна",     "Алла",
    "Альбіна",    "Альвіна",    "Анастасія",  "Анатолія",    "Ангеліна",   "Анжела",   "Анна",      "Антонида",
    "Антоніна",   "Антонія",    "Анфіса",     "Аполлінарія", "Аполлонія",  "Аркадія",  "Артемія",   "Афанасія",
    "Білослава",  "Біляна",     "Благовіста", "Богдана",     "Богуслава",  "Божена",   "Болеслава", "Борислава",
    "Броніслава", "В’ячеслава", "Валентина",  "Валерія",     "Варвара",    "Василина", "Вікторія",  "Вілена",
    "Віленіна",   "Віліна",     "Віола",      "Віолетта",    "Віра",       "Віргінія", "Віта",      "Віталіна",
    "Влада",      "Владислава", "Власта",     "Всеслава",    "Галина",     "Ганна",    "Гелена",    "Далеслава",
    "Дана",       "Дарина",     "Дарислава",  "Діана",       "Діяна",      "Добринка", "Добромила", "Добромира",
    "Добромисла", "Доброслава", "Долеслава",  "Доляна",      "Жанна",      "Жозефіна", "Забава",    "Звенислава",
    "Зінаїда",    "Злата",      "Зореслава",  "Зорина",      "Зоряна",     "Зоя",      "Іванна",    "Ілона",
    "Інна",       "Іннеса",     "Ірина",      "Ірма",        "Калина",     "Каріна",   "Катерина",  "Квітка",
    "Квітослава", "Клавдія",    "Крентта",    "Ксенія",      "Купава",     "Лада",     "Лариса",    "Леся",
    "Ликера",     "Лідія",      "Лілія",      "Любава",      "Любислава",  "Любов",    "Любомила",  "Любомира",
    "Люборада",   "Любослава",  "Людмила",    "Людомила",    "Майя",       "Мальва",   "Мар’яна",   "Марина",
    "Марічка",    "Марія",      "Марта",      "Меланія",     "Мечислава",  "Милодара", "Милослава", "Мирослава",
    "Мілана",     "Мокрина",    "Мотря",      "Мстислава",   "Надія",      "Наталія",  "Неля",      "Немира",
    "Ніна",       "Огняна",     "Оксана",     "Олександра",  "Олена",      "Олеся",    "Ольга",     "Ореста",
    "Орина",      "Орислава",   "Орися",      "Оріяна",      "Павліна",    "Палажка",  "Пелагея",   "Пелагія",
    "Поліна",     "Поляна",     "Потішана",   "Радміла",     "Радослава",  "Раїна",    "Раїса",     "Роксолана",
    "Ромена",     "Ростислава", "Руслана",    "Світлана",    "Святослава", "Слава",    "Сміяна",    "Сніжана",
    "Соломія",    "Соня",       "Софія",      "Станислава",  "Сюзана",     "Таїсія",   "Тамара",    "Тетяна",
    "Устина",     "Фаїна",      "Февронія",   "Федора",      "Феодосія",   "Харитина", "Христина",  "Христя",
    "Юліанна",    "Юлія",       "Юстина",     "Юхима",       "Юхимія",     "Яна",      "Ярина",     "Ярослава",
});

const auto ukrainianMalesLastNames = std::to_array<std::string_view>({
    "Андрухович",  "Бабух",        "Балабан",        "Балабух",       "Балакун",       "Балицький",     "Бамбула",
    "Бандера",     "Барановський", "Бачей",          "Башук",         "Бердник",       "Білич",         "Бондаренко",
    "Борецький",   "Боровський",   "Борочко",        "Боярчук",       "Брицький",      "Бурмило",       "Бутько",
    "Василин",     "Василишин",    "Васильківський", "Вергун",        "Вередун",       "Верещук",       "Витребенько",
    "Вітряк",      "Волощук",      "Гайдук",         "Гайовий",       "Гайчук",        "Галаєнко",      "Галатей",
    "Галаціон",    "Гаман",        "Гамула",         "Ганич",         "Гарай",         "Гарун",         "Гладківський",
    "Гладух",      "Глинський",    "Гнатишин",       "Гойко",         "Головець",      "Горбач",        "Гордійчук",
    "Горовий",     "Городоцький",  "Гречко",         "Григоришин",    "Гриневецький",  "Гриневський",   "Гришко",
    "Громико",     "Данилишин",    "Данилко",        "Демків",        "Демчишин",      "Дзюб’як",       "Дзюба",
    "Дідух",       "Дмитришин",    "Дмитрук",        "Довгалевський", "Дурдинець",     "Євенко",        "Євпак",
    "Ємець",       "Єрмак",        "Забіла",         "Зварич",        "Зінкевич",      "Зленко",        "Іванишин",
    "Іванів",      "Іванців",      "Калач",          "Кандиба",       "Карпух",        "Каськів",       "Кивач",
    "Коваленко",   "Ковальський",  "Коломієць",      "Коман",         "Компанієць",    "Кононець",      "Кордун",
    "Корецький",   "Корнїйчук",    "Коров’як",       "Коцюбинський",  "Кулинич",       "Кульчицький",   "Лагойда",
    "Лазірко",     "Лановий",      "Латаний",        "Латанський",    "Лахман",        "Левадовський",  "Ликович",
    "Линдик",      "Ліхно",        "Лобачевський",   "Ломовий",       "Луговий",       "Луцький",       "Луцьків",
    "Лученко",     "Лучко",        "Лютий",          "Лящук",         "Магера",        "Мазайло",       "Мазило",
    "Мазун",       "Майборода",    "Майстренко",     "Маковецький",   "Малкович",      "Мамій",         "Маринич",
    "Марієвський", "Марків",       "Махно",          "Миклашевський", "Миклухо",       "Милославський", "Михайлюк",
    "Міняйло",     "Могилевський", "Москаль",        "Москалюк",      "Мотрієнко",     "Негода",        "Ногачевський",
    "Опенько",     "Осадко",       "Павленко",       "Павлишин",      "Павлів",        "Пагутяк",       "Паламарчук",
    "Палій",       "Паращук",      "Пасічник",       "Пендик",        "Петик",         "Петлюра",       "Петренко",
    "Петрин",      "Петришин",     "Петрів",         "Плаксій",       "Погиба",        "Поліщук",       "Пономарів",
    "Поривай",     "Поривайло",    "Потебенько",     "Потоцький",     "Пригода",       "Приймак",       "Притула",
    "Прядун",      "Розпутній",    "Романишин",      "Романів",       "Ромей",         "Роменець",      "Ромочко",
    "Савицький",   "Саєнко",       "Свидригайло",    "Семеночко",     "Семещук",       "Сердюк",        "Силецький",
    "Сідлецький",  "Сідляк",       "Сірко",          "Скиба",         "Скоропадський", "Слободян",      "Сосюра",
    "Сплюх",       "Спотикач",     "Стахів",         "Степанець",     "Стецьків",      "Стигайло",      "Сторожук",
    "Сторчак",     "Стоян",        "Сучак",          "Сушко",         "Тарасюк",       "Тиндарей",      "Ткаченко",
    "Третяк",      "Троян",        "Трублаєвський",  "Трясило",       "Трясун",        "Уманець",       "Унич",
    "Усич",        "Федоришин",    "Хитрово",        "Цимбалістий",   "Цушко",         "Червоній",      "Шамрило",
    "Шевченко",    "Шестак",       "Шиндарей",       "Шиян",          "Шкараба",       "Шудрик",        "Шумило",
    "Шупик",       "Шухевич",      "Щербак",         "Юрчишин",       "Юхно",          "Ющик",          "Ющук",
    "Яворівський", "Яловий",       "Ялюк",           "Янюк",          "Ярмак",         "Яцишин",        "Яцьків",
    "Ящук",
});

const auto ukrainianFemaleLastNames = std::to_array<std::string_view>({
    "Андрухович", "Бабух",         "Балабан",      "Балабуха",    "Балакун",     "Балицька",    "Бамбула",
    "Бандера",    "Барановська",   "Бачей",        "Башук",       "Бердник",     "Білич",       "Бондаренко",
    "Борецька",   "Боровська",     "Борочко",      "Боярчук",     "Брицька",     "Бурмило",     "Бутько",
    "Василишина", "Васильківська", "Вергун",       "Вередун",     "Верещук",     "Витребенько", "Вітряк",
    "Волощук",    "Гайдук",        "Гайова",       "Гайчук",      "Галаєнко",    "Галатей",     "Галаціон",
    "Гаман",      "Гамула",        "Ганич",        "Гарай",       "Гарун",       "Гладківська", "Гладух",
    "Глинська",   "Гнатишина",     "Гойко",        "Головець",    "Горбач",      "Гордійчук",   "Горова",
    "Городоцька", "Гречко",        "Григоришина",  "Гриневецька", "Гриневська",  "Гришко",      "Громико",
    "Данилишина", "Данилко",       "Демків",       "Демчишина",   "Дзюб’як",     "Дзюба",       "Дідух",
    "Дмитришина", "Дмитрук",       "Довгалевська", "Дурдинець",   "Євенко",      "Євпак",       "Ємець",
    "Єрмак",      "Забіла",        "Зварич",       "Зінкевич",    "Зленко",      "Іванишина",   "Калач",
    "Кандиба",    "Карпух",        "Кивач",        "Коваленко",   "Ковальська",  "Коломієць",   "Коман",
    "Компанієць", "Кононець",      "Кордун",       "Корецька",    "Корнїйчук",   "Коров’як",    "Коцюбинська",
    "Кулинич",    "Кульчицька",    "Лагойда",      "Лазірко",     "Ланова",      "Латан",       "Латанська",
    "Лахман",     "Левадовська",   "Ликович",      "Линдик",      "Ліхно",       "Лобачевська", "Ломова",
    "Лугова",     "Луцька",        "Луцьків",      "Лученко",     "Лучко",       "Люта",        "Лящук",
    "Магера",     "Мазайло",       "Мазило",       "Мазун",       "Майборода",   "Майстренко",  "Маковецька",
    "Малкович",   "Мамій",         "Маринич",      "Марієвська",  "Марків",      "Махно",       "Миклашевська",
    "Миклухо",    "Милославська",  "Михайлюк",     "Міняйло",     "Могилевська", "Москаль",     "Москалюк",
    "Мотрієнко",  "Негода",        "Ногачевська",  "Опенько",     "Осадко",      "Павленко",    "Павлишина",
    "Павлів",     "Пагутяк",       "Паламарчук",   "Палій",       "Паращук",     "Пасічник",    "Пендик",
    "Петик",      "Петлюра",       "Петренко",     "Петрина",     "Петришина",   "Петрів",      "Плаксій",
    "Погиба",     "Поліщук",       "Пономарів",    "Поривай",     "Поривайло",   "Потебенько",  "Потоцька",
    "Пригода",    "Приймак",       "Притула",      "Прядун",      "Розпутня",    "Романишина",  "Ромей",
    "Роменець",   "Ромочко",       "Савицька",     "Саєнко",      "Свидригайло", "Семеночко",   "Семещук",
    "Сердюк",     "Силецька",      "Сідлецька",    "Сідляк",      "Сірко",       "Скиба",       "Скоропадська",
    "Слободян",   "Сосюра",        "Сплюха",       "Спотикач",    "Степанець",   "Стигайло",    "Сторожук",
    "Сторчак",    "Стоян",         "Сучак",        "Сушко",       "Тарасюк",     "Тиндарей",    "Ткаченко",
    "Третяк",     "Троян",         "Трублаєвська", "Трясило",     "Трясун",      "Уманець",     "Унич",
    "Усич",       "Федоришина",    "Цушко",        "Червоній",    "Шамрило",     "Шевченко",    "Шестак",
    "Шиндарей",   "Шиян",          "Шкараба",      "Шудрик",      "Шумило",      "Шупик",       "Шухевич",
    "Щербак",     "Юрчишина",      "Юхно",         "Ющик",        "Ющук",        "Яворівська",  "Ялова",
    "Ялюк",       "Янюк",          "Ярмак",        "Яцишина",     "Яцьків",      "Ящук",
});

const auto ukrainianMalePrefixes = std::to_array<std::string_view>({"Пан"});

const auto ukrainianFemalePrefixes = std::to_array<std::string_view>({"Пані"});

const NameFormats ukrainianNameFormats{{
    {"{firstName} {lastName}", 1},
    {"{lastName} {firstName}", 1},
}};

const PeopleNames ukrainianPeopleNames{
    {ukrainianMaleFirstNames, ukrainianMalesLastNames, ukrainianMalePrefixes, {}},
    {ukrainianFemaleFirstNames, ukrainianFemaleLastNames, ukrainianFemalePrefixes, {}},
    ukrainianNameFormats};

// USA

const NameFormats usaNameFormats{{{"{firstName} {lastName}", 49},
                                  {"{firstName} {lastName}-{lastName}", 7},
                                  {"{prefix} {firstName} {lastName}", 7},
                                  {"{firstName} {lastName} {suffix}", 7},
                                  {"{prefix} {firstName} {lastName} {suffix}", 1}}};

const PeopleNames usaPeopleNames{{englishMaleFirstNames, englishLastNames, englishMalePrefixes, englishSuffixes},
                                 {englishFemaleFirstNames, englishLastNames, englishFemalePrefixes, englishSuffixes},
                                 usaNameFormats};

// Vietnam

const auto vietnameseMaleFirstNames = std::to_array<std::string_view>({
    "An Cơ",       "An Khang",    "Ân Lai",      "An Nam",      "An Nguyên",   "An Ninh",     "An Tâm",
    "Ân Thiện",    "An Tường",    "Anh Đức",     "Anh Dũng",    "Anh Duy",     "Anh Hoàng",   "Anh Khải",
    "Anh Khoa",    "Anh Khôi",    "Anh Minh",    "Anh Quân",    "Anh Quốc",    "Anh Sơn",     "Anh Tài",
    "Anh Thái",    "Anh Tú",      "Anh Tuấn",    "Anh Tùng",    "Anh Việt",    "Anh Vũ",      "Bá Cường",
    "Bá Kỳ",       "Bá Lộc",      "Bá Long",     "Bá Phước",    "Bá Thành",    "Bá Thiện",    "Bá Thịnh",
    "Bá Thúc",     "Bá Trúc",     "Bá Tùng",     "Bách Du",     "Bách Nhân",   "Bằng Sơn",    "Bảo An",
    "Bảo Bảo",     "Bảo Chấn",    "Bảo Định",    "Bảo Duy",     "Bảo Giang",   "Bảo Hiển",    "Bảo Hoa",
    "Bảo Hoàng",   "Bảo Huy",     "Bảo Huynh",   "Bảo Huỳnh",   "Bảo Khánh",   "Bảo Lâm",     "Bảo Long",
    "Bảo Pháp",    "Bảo Quốc",    "Bảo Sơn",     "Bảo Thạch",   "Bảo Thái",    "Bảo Tín",     "Bảo Toàn",
    "Bích Nhã",    "Bình An",     "Bình Dân",    "Bình Đạt",    "Bình Định",   "Bình Dương",  "Bình Hòa",
    "Bình Minh",   "Bình Nguyên", "Bình Quân",   "Bình Thuận",  "Bình Yên",    "Bửu Chưởng",  "Bửu Diệp",
    "Bữu Toại",    "Cảnh Tuấn",   "Cao Kỳ",      "Cao Minh",    "Cao Nghiệp",  "Cao Nguyên",  "Cao Nhân",
    "Cao Phong",   "Cao Sĩ",      "Cao Sơn",     "Cao Sỹ",      "Cao Thọ",     "Cao Tiến",    "Cát Tường",
    "Cát Uy",      "Chấn Hùng",   "Chấn Hưng",   "Chấn Phong",  "Chánh Việt",  "Chế Phương",  "Chí Anh",
    "Chí Bảo",     "Chí Công",    "Chí Dũng",    "Chí Giang",   "Chí Hiếu",    "Chí Khang",   "Chí Khiêm",
    "Chí Kiên",    "Chí Nam",     "Chí Sơn",     "Chí Thanh",   "Chí Thành",   "Chiến Thắng", "Chiêu Minh",
    "Chiêu Phong", "Chiêu Quân",  "Chính Tâm",   "Chính Thuận", "Chính Trực",  "Chuẩn Khoa",  "Chung Thủy",
    "Công Án",     "Công Ân",     "Công Bằng",   "Công Giang",  "Công Hải",    "Công Hào",    "Công Hậu",
    "Công Hiếu",   "Công Hoán",   "Công Lập",    "Công Lộc",    "Công Luận",   "Công Luật",   "Công Lý",
    "Công Phụng",  "Công Sinh",   "Công Sơn",    "Công Thành",  "Công Tráng",  "Công Tuấn",   "Cường Dũng",
    "Cương Nghị",  "Cương Quyết", "Cường Thịnh", "Đắc Cường",   "Đắc Di",      "Đắc Lộ",      "Đắc Lực",
    "Đắc Thái",    "Đắc Thành",   "Đắc Trọng",   "Đại Dương",   "Đại Hành",    "Đại Ngọc",    "Đại Thống",
    "Dân Hiệp",    "Dân Khánh",   "Đan Quế",     "Đan Tâm",     "Đăng An",     "Đăng Đạt",    "Đăng Khánh",
    "Đăng Khoa",   "Đăng Khương", "Đăng Minh",   "Đăng Quang",  "Danh Nhân",   "Danh Sơn",    "Danh Thành",
    "Danh Văn",    "Đạt Dũng",    "Đạt Hòa",     "Đình Chiểu",  "Đình Chương", "Đình Cường",  "Đình Diệu",
    "Đình Đôn",    "Đình Dương",  "Đình Hảo",    "Đình Hợp",    "Đình Kim",    "Đinh Lộc",    "Đình Lộc",
    "Đình Luận",   "Định Lực",    "Đình Nam",    "Đình Ngân",   "Đình Nguyên", "Đình Nhân",   "Đình Phú",
    "Đình Phúc",   "Đình Quảng",  "Đình Sang",   "Định Siêu",   "Đình Thắng",  "Đình Thiện",  "Đình Toàn",
    "Đình Trung",  "Đình Tuấn",   "Đoàn Tụ",     "Đồng Bằng",
});

const auto vietnameseFemaleFirstNames = std::to_array<std::string_view>({
    "An Bình",    "An Di",      "An Hạ",       "An Hằng",     "An Khê",     "An Nhiên",    "An Nhàn",     "Anh Chi",
    "Anh Hương",  "Anh Mai",    "Anh Phương",  "Anh Thi",     "Anh Thy",    "Anh Thơ",     "Anh Thư",     "Anh Thảo",
    "Anh Vũ",     "Anh Đào",    "Ban Mai",     "Bình Minh",   "Bình Yên",   "Bích Chiêu",  "Bích Châu",   "Bích Duyên",
    "Bích Hiền",  "Bích Huệ",   "Bích Hà",     "Bích Hạnh",   "Bích Hải",   "Bích Hảo",    "Bích Hậu",    "Bích Hằng",
    "Bích Hồng",  "Bích Hợp",   "Bích Lam",    "Bích Liên",   "Bích Loan",  "Bích Nga",    "Bích Ngà",    "Bích Ngân",
    "Bích Ngọc",  "Bích Như",   "Bích Phượng", "Bích Quyên",  "Bích Quân",  "Bích San",    "Bích Thoa",   "Bích Thu",
    "Bích Thảo",  "Bích Thủy",  "Bích Trang",  "Bích Trâm",   "Bích Ty",    "Bích Vân",    "Bích Điệp",   "Bích Đào",
    "Băng Băng",  "Băng Tâm",   "Bạch Cúc",    "Bạch Hoa",    "Bạch Kim",   "Bạch Liên",   "Bạch Loan",   "Bạch Mai",
    "Bạch Quỳnh", "Bạch Trà",   "Bạch Tuyết",  "Bạch Vân",    "Bạch Yến",   "Bảo Anh",     "Bảo Bình",    "Bảo Châu",
    "Bảo Huệ",    "Bảo Hà",     "Bảo Hân",     "Bảo Lan",     "Bảo Lễ",     "Bảo Ngọc",    "Bảo Phương",  "Bảo Quyên",
    "Bảo Quỳnh",  "Bảo Thoa",   "Bảo Thúy",    "Bảo Tiên",    "Bảo Trâm",   "Bảo Trân",    "Bảo Trúc",    "Bảo Uyên",
    "Bảo Vy",     "Bảo Vân",    "Bội Linh",    "Cam Thảo",    "Chi Lan",    "Chi Mai",     "Chiêu Dương", "Cát Cát",
    "Cát Linh",   "Cát Ly",     "Cát Tiên",    "Cát Tường",   "Cẩm Hiền",   "Cẩm Hường",   "Cẩm Hạnh",    "Cẩm Linh",
    "Cẩm Liên",   "Cẩm Ly",     "Cẩm Nhi",     "Cẩm Nhung",   "Cẩm Thúy",   "Cẩm Tú",      "Cẩm Vân",     "Cẩm Yến",
    "Di Nhiên",   "Diên Vỹ",    "Diễm Chi",    "Diễm Châu",   "Diễm Hương", "Diễm Hạnh",   "Diễm Hằng",   "Diễm Khuê",
    "Diễm Kiều",  "Diễm Liên",  "Diễm Lộc",    "Diễm My",     "Diễm Phúc",  "Diễm Phương", "Diễm Phước",  "Diễm Phượng",
    "Diễm Quyên", "Diễm Quỳnh", "Diễm Thúy",   "Diễm Thư",    "Diễm Thảo",  "Diễm Trang",  "Diễm Trinh",  "Diễm Uyên",
    "Diệp Anh",   "Diệp Vy",    "Diệu Anh",    "Diệu Hiền",   "Diệu Hoa",   "Diệu Huyền",  "Diệu Hương",  "Diệu Hạnh",
    "Diệu Hằng",  "Diệu Hồng",  "Diệu Lan",    "Diệu Linh",   "Diệu Loan",  "Diệu Nga",    "Diệu Ngà",    "Diệu Ngọc",
    "Diệu Nương", "Diệu Thiện", "Diệu Thúy",   "Diệu Vân",    "Diệu Ái",    "Duy Hạnh",    "Duy Mỹ",      "Duy Uyên",
    "Duyên Hồng", "Duyên My",   "Duyên Mỹ",    "Duyên Nương", "Dã Lan",     "Dã Lâm",      "Dã Thảo",     "Dạ Hương",
    "Dạ Lan",     "Dạ Nguyệt",  "Dạ Thi",      "Dạ Thảo",     "Dạ Yến",     "Gia Hân",     "Gia Khanh",   "Gia Linh",
    "Gia Nhi",    "Gia Quỳnh",  "Giang Thanh", "Giang Thiên", "Giao Hưởng", "Giao Kiều",   "Giao Linh",   "Giáng Ngọc",
    "Giáng Tiên", "Giáng Uyên", "Hiếu Giang",  "Hiếu Hạnh",   "Hiếu Khanh", "Hiếu Minh",   "Hiền Chung",  "Hiền Hòa",
    "Hiền Mai",   "Hiền Nhi",   "Hiền Nương",  "Hiền Thục",   "Hiểu Lam",   "Hiểu Vân",    "Hoa Liên",    "Hoa Lý",
});

const auto vietnameseLastNames = std::to_array<std::string_view>({
    "Phạm",  "Nguyễn", "Trần", "Lê",    "Lý",  "Hoàng", "Phan",   "Vũ",   "Tăng",
    "Đặng",  "Bùi",    "Đỗ",   "Hồ",    "Ngô", "Dương", "Đào",    "Đoàn", "Vương",
    "Trịnh", "Đinh",   "Lâm",  "Phùng", "Mai", "Tô",    "Trương", "Hà",   "Huỳnh",
});

const NameFormats vietnameseNameFormats{{"{firstName} {lastName}", 1}};

const PeopleNames vietnamesePeopleNames{{vietnameseMaleFirstNames, vietnameseLastNames, {}, {}},
                                        {vietnameseFemaleFirstNames, vietnameseLastNames, {}, {}},
                                        vietnameseNameFormats};
}
